
#include "OHSharedUtils.h"
#include "kSanPlugIn.h"
#include "kSanSimulationPrivate.h"
#include "LJ.h"
#include "shlbProcInfo.h"
#include "string.h"  // to get def for strlen

#define	   kInitErr  -108  // i.e. a memFullErr
//#define	   kDialogOK  0
//#define    kDialogCancelled 1
//#define	   kMoveToFront	(WindowPtr)-1L
#define    kLJParamsDialogResID  141 // this resource in the shared library resource fork
//#define    nilPointer 0L
//#define	    nilString  "\p"
//#define  kDefaultAlertID  400 // kSan has this resource
//#define  kBaseResID  129  
#define  kLJNeighborsRezID  129  
#define  kLJAtomTypesRezID  130  

#define kMaxDouble 1E99
#define kMinDouble -1E99

#define forceConversion 9636.8 // this is how many ( (g/mol) A / ps ps ) per (eV /A)


#define    kPlugInDataRezID 132 // please use IDs 132 - 139 if you need more than one
void addAndWriteResource(Handle theHandle, ResType rType, long rezID, Str255 rezName);

short  LJoneParticleForceCalcu	 ( kSanPotentialPlugIn *ppi, kSanSimulation *sim,  particle * basePart, long partArrIndex );
short LJoneParticleEnergyCalcu (  kSanPotentialPlugIn *ppi, kSanSimulation *sim,  particle * basePart,long partArrIndex,  double * retVal)  ; 
short  LJoneParticleVirialCalcu	 (  kSanPotentialPlugIn *ppi, kSanSimulation *sim, particle * basePart, long partArrIndex,  kSanMatrix  *result);
 
short initClassLJPotential (void)
{
	short err = noErr;
	kozoClass *newClass;
	kozoObject *co;

	newClass = addClass(&co,ClassLJ, ClassPotentialPlugIn, sizeof (LJPlugIn), "Lennard-Jones");
	if (newClass)
	{
		err =	setFuncs(newClass, 0L,  LJConstructor,
		nilPointer, LJGetData, LJSetData, nilPointer,
		 nilPointer, nilPointer, nilPointer, nilPointer);

		 err =  kozoClassAddRemove3ItemsToSave(newClass, true,   pAttractiveExponent,	pRepulsiveExponent,	pAttractivePrefactor );
		 err =  kozoClassAddRemove3ItemsToSave(newClass, true,  pRepulsivePrefactor,	pForceShift, pRNought );
 	}
	else
	{
		err = true;
	}
	return (err);
}
short  LJConstructor (KozoDispatchStack *ds, constructorMethodArgs *cma)
{
#pragma unused (cma)
	short err = noErr;
	LJPlugIn *LJ;
 	kSanPotentialPlugIn *api;
 	api = KDSGetPrivateData(ds, ClassPotentialPlugIn );
 	LJ = KDSGetPrivateData(ds, ClassLJ );
 	
 	api->potentialPlugPrivateData = (void *)LJ;
	api->onePartECalc = LJOneParticleEnergyCalc;
	api->onePartFCalc =  LJOneParticleForceCalc;
	api->onePartVirialCalc =  LJOneParticleVirialCalc;
    api->manyPartECalc =   LJManyParticleEnergyCalc;
	api->manyPartFCalc =    LJManyParticleForceCalc;

 	LJ->ppi = api;
 	LJ->AttractiveExponent = -6.0;
	LJ->RepulsiveExponent = -12.0;
	LJ->AttractivePrefactor = -1.0;
	LJ->RepulsivePrefactor = 1.0;
	LJ->ForceShift = 0.0;
	LJ->RNought = 2.0;
	LJ->InverseRNought = 1.0 / LJ->RNought;
	LJ->APrefactorTimesAExponent = LJ->AttractiveExponent * LJ->AttractivePrefactor;
	LJ->RPrefactorTimesRExponent = LJ->RepulsiveExponent * LJ->RepulsivePrefactor;

	KDSReturnError(ds,err);
}

short  LJGetData (KozoDispatchStack *ds, getMethodArgs *gma)
{
	short err = noErr;
	LJPlugIn *LJ;
 	LJ = KDSGetPrivateData(ds, ClassLJ );
	switch (gma->property)
	{
		case pAttractiveExponent:
		 	err = AECreateDesc (typeLongFloat,&LJ->AttractiveExponent, sizeof(double), gma->result);
			break;
		case pAttractivePrefactor:
			err = AECreateDesc (typeLongFloat,&LJ->AttractivePrefactor, sizeof(double), gma->result);
			break;
		case pRepulsivePrefactor:
			err = AECreateDesc (typeLongFloat,&LJ->RepulsivePrefactor, sizeof(double), gma->result);
			break;
		case pRepulsiveExponent:
			err = AECreateDesc (typeLongFloat,&LJ->RepulsiveExponent, sizeof(double), gma->result);
			break;
		case pForceShift:
			err = AECreateDesc (typeLongFloat,&LJ->ForceShift, sizeof(double), gma->result);
			break;
		case pRNought:
			err = AECreateDesc (typeLongFloat,&LJ->RNought, sizeof(double), gma->result);
			break;
		default:
			KDSReturnContinue(ds);
	} 
	KDSDoneDispatch(ds);
	KDSReturnError(ds, err);
}
short  LJSetData (KozoDispatchStack *ds, setMethodArgs *sma)
{
	short err = noErr;
	LJPlugIn *LJ;
 	LJ = KDSGetPrivateData(ds, ClassLJ );
	switch (sma->property)
	{
		case pAttractiveExponent:
		 	OHCoerceToDouble (sma->data, &LJ->AttractiveExponent);
			goto UpdateFactors;
		case pAttractivePrefactor:
			OHCoerceToDouble (sma->data, &LJ->AttractivePrefactor );
			goto UpdateFactors;
		case pRepulsivePrefactor:
			OHCoerceToDouble (sma->data, &LJ->RepulsivePrefactor );
			goto UpdateFactors;
		case pRepulsiveExponent:
			OHCoerceToDouble (sma->data, &LJ->RepulsiveExponent );
			goto UpdateFactors;
		case pRNought:
			OHCoerceToDouble (sma->data, &LJ->RNought );
			goto UpdateFactors;
		UpdateFactors:
			LJ->APrefactorTimesAExponent = LJ->AttractiveExponent * LJ->AttractivePrefactor;
			LJ->RPrefactorTimesRExponent = LJ->RepulsiveExponent * LJ->RepulsivePrefactor;
			LJ->InverseRNought = 1.0 / LJ->RNought;
		    break;
		case pForceShift:
			OHCoerceToDouble (sma->data, &LJ->ForceShift );
			break;
		default:
			KDSReturnContinue(ds);
	}
	KDSDoneDispatch(ds);
	KDSReturnError(ds, err);
}

short LJManyParticleEnergyCalc ( kSanPotentialPlugIn *ppi , kSanSimulation *sim,  long firstPart, long lastPart, double *retVal) 
{
	short err = noErr;
	short thisType;
	long partArrIndex;
	particle *basePart;
	double totalEnergy  = 0.0;
	double returned;
	
 	basePart = simLockParts(sim); 

	for (partArrIndex=(firstPart);partArrIndex<= (lastPart); ++partArrIndex) 
	{	
		thisType = BPNthType(basePart, partArrIndex);
		
		LJoneParticleEnergyCalcu (  ppi, sim, basePart, partArrIndex,  &returned) ;
		basePart->basePotential[partArrIndex] = returned;
		totalEnergy += returned;									
	}
	simReleaseParts(sim); 

	*retVal = (totalEnergy);
	return (err);
}
									
short  LJManyParticleForceCalc( kSanPotentialPlugIn *ppi , kSanSimulation *sim,  long firstPart, long lastPart) 
								// put forces in (*theParticleHandle)[i].force
								// return void
{
	short err = noErr;
	long partArrIndex;
 	particle *basePart;

	basePart = simLockParts(sim); 
	for (partArrIndex=(firstPart);partArrIndex<= (lastPart); ++partArrIndex) 
	{	
		err = LJoneParticleForceCalcu( ppi, sim, basePart, partArrIndex);
	}
	simReleaseParts(sim); 
	return (err);
}
							
short  LJOneParticleForceCalc( kSanPotentialPlugIn *ppi , kSanSimulation *sim, long partArrIndex)  
								// put force in basePart->baseForce[partArrIndex]
								// return err
{
	short err = noErr;
	particle *basePart;
 
	basePart = simLockParts(sim); 

	err = LJoneParticleForceCalcu(  ppi, sim, basePart,   partArrIndex);
	simReleaseParts(sim); 
	return (err);
}
short  LJOneParticleVirialCalc  ( kSanPotentialPlugIn *ppi , kSanSimulation *sim, long partArrIndex, kSanMatrix *rMat) 
								// put virial in vec
								// return err
{
	short err = noErr;
	particle *basePart;
 
	basePart = simLockParts(sim); 

	err = LJoneParticleVirialCalcu( ppi, sim, basePart, partArrIndex,  rMat);

	simReleaseParts(sim); 
	return (err);
}
short  LJOneParticleEnergyCalc ( kSanPotentialPlugIn *ppi , kSanSimulation *sim, long partArrIndex, double *retVal)
								// put force in basePart->baseForce[partArrIndex]
								// return err
{
	short err = noErr;
	double energy ;
	particle *basePart;
 
	basePart = simLockParts(sim); 

	LJoneParticleEnergyCalcu(  ppi, sim,  basePart, partArrIndex,  &energy);
    basePart->basePotential[partArrIndex]   = energy;
    *retVal = energy;
   
	simReleaseParts(sim); 
	return (err);
}
								
short  LJoneParticleVirialCalcu	 ( kSanPotentialPlugIn *ppi, kSanSimulation *sim, particle * basePart, long partArrIndex,  kSanMatrix *rMat)
								 
								 
{
	short err = noErr;
	LJPlugIn *LJ = ppi->potentialPlugPrivateData;
	long q;
	long NNNum;
//	NNeighborInfo *nni;
	double force;
	double sep;
	double normalizedSep;
	double rExp;
	double aExp;
	double screen;
	kSanVector  sepVec;
	kSanVector  localVec;
	kSanVector  fVec;
	kSanMatrix localMat; // declare floating pt variables last
	kSanMatrix mat; // declare floating pt variables last
	
 	zerokSanMatrixD (&mat);
	zerokSanVectorD (&fVec);
 	rExp = LJ->RepulsiveExponent - 1.0;
	aExp = LJ->AttractiveExponent - 1.0;
	
	if (basePart->basePi[partArrIndex].neighborScreeningDirty) LJ->ppi->fillScreeningArray ( ppi, sim, basePart, partArrIndex);
	NNNum = basePart->basePi[partArrIndex].actualNumNeighbors ;
 	for (q=0;q< NNNum;++q)
	{
		screen = NNIGetNthScreening (sim, basePart, partArrIndex, q );
		if (screen > 0.0)
		{
			//NNIGetNthVec(sim, basePart, partArrIndex, q, &sepVec);
			//sep = NNIGetNthSep(sim, basePart, partArrIndex, q) ;
			sep = NNIGetNthSepAndVec(sim, basePart, partArrIndex, q, &sepVec) ;
			
			
			normalizedSep = sep * LJ->InverseRNought;
 			force = LJ->RPrefactorTimesRExponent * pow(normalizedSep,rExp);
			force += LJ->APrefactorTimesAExponent * pow(normalizedSep,aExp);
 			force = force / sep;  // normalize by radius so we can multiply by components
            
            multkSanVectorD (&sepVec, force, &fVec);  
			//fVec.a = force * sepVec.a ;
			//fVec.b = force * sepVec.b ;	 		
			//fVec.c = force * sepVec.c ;
			
			if (screen >= 1.0)  // its equal to one
			{ // adding to fVec done below
			}	
			else if (screen > 0.0)
			{
				kSanVector  screeningDifferential;
				double energy;
					
	 			// here we have to account for the gradient in the 
				// screening factor as well as the force
					
				LJ->ppi->screeningDifferential ( ppi, sim,  basePart, partArrIndex,  q,  &screeningDifferential);
					// this Function calculates the screening differentials for us, i.e. dScreen/dx
					// now, because force = dE/dx and E = screening * Ec
						
					// now that we have dSdx, dSdy and dSdz, we need to calculate the energy 
					// note this is not the whole energy, just the energy of the interaction with 
					// the particle q
					
				energy = LJ->RepulsivePrefactor * pow(normalizedSep, LJ->RepulsiveExponent);
				energy += LJ->AttractivePrefactor * pow(normalizedSep, LJ->AttractiveExponent);
					
					// now we do the math dE/dx = (screening * dEc/dx) + (Ec * dScreen/dx)
								
				force *= NNIGetNthScreening(sim, basePart, partArrIndex, q); 
				// at this step mult by components to decompose the force into vector components
					// add the screening differential terms
					
				   //localVec.a = energy * screeningDifferential.a; //  + (Ec * dScreen/dx)
				   //localVec.b = energy * screeningDifferential.b; //  + (Ec * dScreen/dy)
				   //localVec.c = energy * screeningDifferential.c; //  + (Ec * dScreen/dz)
				multkSanVectorD (&screeningDifferential, energy, &localVec);
				
					// multiply by the distance -- magic sign cancelling
				addkSanVectorsD (&fVec, &localVec , &fVec );
				// fVec.a += localVec.a ;
				// fVec.b += localVec.b ;
				// fVec.c += localVec.c ;
			}
			// now turn the force into a stress tensor:
			multkSanVectorD (&fVec, sepVec.a, &localMat.A);
			multkSanVectorD (&fVec, sepVec.b, &localMat.B);
#ifndef __2DVectors__
			multkSanVectorD (&fVec, sepVec.c, &localMat.C);
#endif			
			addkSanMatricesD (&mat, &localMat, &mat);
		} // end screen array > 0
	}
	multkSanMatrixD(&mat, 0.5, rMat); // this accounts for counting opposing forces twice
	return (err);
}
						
short  LJoneParticleForceCalcu	 (kSanPotentialPlugIn *ppi, kSanSimulation *sim, particle * basePart, long partArrIndex)
								 // put force in part[i].force
								// return noErr
{
	short err = noErr;
	LJPlugIn *LJ = ppi->potentialPlugPrivateData;
	long q;
	long NNNum;
	NNeighborInfo *nni;
	kSanVector  sepVec;
	double force;
	double screen;
	double normalizedSep;
	double sep;
	double rExp;
	double aExp;
	kSanVector  fVec; // declare floating pt variables last
	
	zerokSanVectorD(&fVec);
 	rExp = LJ->RepulsiveExponent - 1.0;
	aExp = LJ->AttractiveExponent - 1.0;
	
	NNNum = basePart->basePi[partArrIndex].actualNumNeighbors ;
	nni = basePart->basePi[partArrIndex].firstNeighborInfo;
	
	if (basePart->basePi[partArrIndex].neighborScreeningDirty) LJ->ppi->fillScreeningArray (  ppi, sim, basePart, partArrIndex);
	for (q=0;q<NNNum;++q)
	{
		screen = NNIGetNthScreening (sim, basePart, partArrIndex, q);
		if (screen  > 0.0)
		{
 			NNIGetNthVec (sim, basePart, partArrIndex, q, &sepVec);
 			sep = NNIGetNthSep (sim, basePart, partArrIndex, q);
			normalizedSep =  sep * LJ->InverseRNought;
			force = LJ->RPrefactorTimesRExponent * pow(normalizedSep,rExp);
			force += LJ->APrefactorTimesAExponent * pow(normalizedSep,aExp);
 		 	force = force / sep;  // normalize by radius so we can multiply by components

			if (screen >= 1.0)  // its equal to one
			{
				multAddkSanVectorsD (&fVec, &sepVec, force,  &fVec);
				//fVec.a += force * sepVec.a;	 // at this step mult by components			
				//fVec.b += force * sepVec.b;				
				//fVec.c += force * sepVec.c;
			}	
			else 
			{
				kSanVector  screeningDifferential;
				double energy;
					
	 			// here we have to account for the gradient in the 
				// screening factor as well as the force
					
				LJ->ppi->screeningDifferential (  ppi, sim, basePart, partArrIndex, q, &screeningDifferential);
					// this Function calculates the screening differentials for us, i.e. dScreen/dx
					// now, because force = dE/dx and E = screening * Ec
						
					// now that we have dSdx, dSdy and dSdz, we need to calculate the energy 
					// note this is not the whole energy, just the energy of the interaction with 
					// the particle q
					
				energy = LJ->RepulsivePrefactor * pow(normalizedSep, LJ->RepulsiveExponent);
				energy += LJ->AttractivePrefactor * pow(normalizedSep, LJ->AttractiveExponent);
					
					// now we do the math dE/dx = (screening * dEc/dx) + (Ec * dScreen/dx)
								
				force *= screen; // force is already normalized by radius 
				// at this step mult by components to decompose the force into vector components
				multAddkSanVectorsD (&fVec, &sepVec, force,  &fVec);
				//fVec.a += force * sepVec.a;	// + (screening * dEc/dx)	
				//fVec.b += force * sepVec.b;	// + (screening * dEc/dy)		
				//fVec.c += force * sepVec.c;	// + (screening * dEc/dz)					
				
					// add the screening differential terms
				multAddkSanVectorsD (&fVec, &screeningDifferential, energy,  &fVec);
				//fVec.a += energy * screeningDifferential.a; //  + (Ec * dScreen/dx)
				//fVec.b += energy * screeningDifferential.b; //  + (Ec * dScreen/dy)
				//fVec.c += energy * screeningDifferential.c; //  + (Ec * dScreen/dz)
					// add the direct force terms when we come out of the else
			}
		}	 // end screen array > 0			
	}
	multkSanVectorD (&fVec, forceConversion * 0.5,  &fVec );
	//multkSanVectorD (&fVec, 0.5,  &fVec );
	//fVec.a *= forceConversion;
	//fVec.b *= forceConversion;
	//fVec.c *= forceConversion;
	basePart->baseForce[partArrIndex] = fVec;
	return (err);
}
								 	
short LJoneParticleEnergyCalcu ( kSanPotentialPlugIn *ppi, kSanSimulation *sim , particle * basePart,  long partArrIndex,  double * retVal)   
							 
// return energy of particle
{
	short err = noErr;
	LJPlugIn *LJ = ppi->potentialPlugPrivateData;
	long q;
	long NNNum;
 	double particleEnergy;
	double thisEnergy;
	double sep;
	double normalizedSep;
	double screen;
 	
	particleEnergy = 0.0;
	NNNum = basePart->basePi[partArrIndex].actualNumNeighbors  ;
	if (basePart->basePi[partArrIndex].neighborScreeningDirty) LJ->ppi->fillScreeningArray ( ppi, sim, basePart, partArrIndex);
	for (q=0;q<NNNum;++q)
	{
		screen = NNIGetNthScreening (sim, basePart, partArrIndex, q);
		if (screen > 0)
		{
	 		sep = NNIGetNthSep (sim, basePart, partArrIndex, q);
			normalizedSep =  sep * LJ->InverseRNought;
 			thisEnergy = LJ->RepulsivePrefactor * pow(normalizedSep, LJ->RepulsiveExponent);
			thisEnergy += LJ->AttractivePrefactor * pow(normalizedSep, LJ->AttractiveExponent);

			if (screen >= 1.0)
			{
				particleEnergy += thisEnergy ;	
			}
			else  
			{
 				particleEnergy += (thisEnergy * screen);	
 			}	
		}
	}
	*retVal = ( particleEnergy * 0.5);	 // factor of 0.5 for sharing the potential
	return (err);
}
								 	
