#include "VerletMethod.h"
#include "kSanSharedUtils.h"
#define kOneOverRootThree (0.5773502692)
#define kOneOverRootTwo (0.707106781187)
#include "kSanSimulationPrivate.h"
#include "kSanMethodPlugIn.h"
#define bigVelocityErr -101
#define smallVelocityErr -102

short  VerletMethodConstructor (KozoDispatchStack *ds, constructorMethodArgs *cma);
short   VerletMethodDestructor (KozoDispatchStack *ds, destructorMethodArgs* dma);
short  VerletGetData (KozoDispatchStack *ds, getMethodArgs *gma);
 short  VerletSetData (KozoDispatchStack *ds, setMethodArgs *sma);
short  VerletMethodDoMessage (KozoDispatchStack *ds, doMessageMethodArgs *dmma);
  short verletCalcHalfVelocities(kozoObject *verletObj, VerletMethod *verlet );
//short  VerletMethodSaveSig(kozoObject *verletObj,  VerletMethod *mpi);
//void   initPrevPositions(kozoObject *verletObj, VerletMethod *verlet);
//short  VerletReallocate(kozoObject *verletObj, VerletMethod *verlet );
short VerletChangedInfo (kozoObject *verletObj, VerletMethod *verlet,  whatWasChangedStruct *wwc);
short VerletIteration (  kSanMethodPlugIn *mpi,  kSanSimulation *sim) ;
short VerletCooling (  kSanMethodPlugIn *mpi,  kSanSimulation *sim, double *netTime, double *totKinEnergy);
 short 	verletEstimateVelocities(kozoObject *verletObj, VerletMethod *verlet,  particle *basePart  );

short 	verletChangePositions(kozoObject *mpiObj, VerletMethod *verlet );
float 	verletChangeGroupPositionsFree(kozoObject *mpiObj, VerletMethod *verlet, kSanGroup *gp );
float 	verletChangeGroupPositionsBlock(kozoObject *mpiObj, VerletMethod *verlet, kSanGroup *gp );
float 	verletChangeGroupPositionsMixed(kozoObject *mpiObj, VerletMethod *verlet, kSanGroup *gp );
float 	verletChangeGroupPositionsFixed(kozoObject *mpiObj, VerletMethod *verlet, kSanGroup *gp );

char verletSafeCalcNextPositions (kozoObject *mpiObj, VerletMethod *verlet,  particle *basePart, kSanGroup **baseGroup, long numGroups , char diffTimestep);


short initClassVerletMethod(void)
{
	short err = noErr;
	kozoClass *newClass;
	kozoObject *co;

	newClass = addClass(&co,ClassVerletMethod, ClassMethodPlugIn, sizeof (VerletMethod), "Verlet method");
	if (newClass)
	{
		err =	setFuncs(newClass, 0L,  VerletMethodConstructor,
		VerletMethodDestructor, VerletGetData, VerletSetData, VerletMethodDoMessage,
		 nilPointer, nilPointer, nilPointer, nilPointer);

		err =  kozoClassAddRemove2ItemsToSave(newClass, true,   pMaximumDisplacement,  pItersPerTimestep);
	}
	return (err);
}
short  VerletMethodConstructor (KozoDispatchStack *ds, constructorMethodArgs *cma)
{
#pragma unused (cma)
	short err = noErr;
	VerletMethod *verlet;
	kSanMethodPlugIn *mpi;
	kSanSimulation *sim = nilPointer;
 	mpi = KDSGetPrivateData(ds, ClassMethodPlugIn );
 	
	verlet  = (VerletMethod *) ds->thisPD;
	
 	VSListInitialize (&verlet->accels, sizeof(kSanVector ));
 	verlet->maxDelPos = 0.01;
	verlet->maxDelSafetyFactor = 0.5;
	verlet->verletItersPerSimIter = 1;
 	mpi->methodPlugPrivateData = (void *)verlet;
	mpi->doIteration = VerletIteration;
	mpi->doCooling = VerletCooling;
	
 // initialization of the value in verlet->sim takes place when kSetNewContainerMessage is called
 
 	KDSReturnError(ds,err);
}
 short   VerletMethodDestructor (KozoDispatchStack *ds, destructorMethodArgs* dma)
{
#pragma unused(dma)
	VerletMethod *verlet;
  	
	verlet  = (VerletMethod *) ds->thisPD;
  	KDSReturnContinue(ds);
}
short  VerletMethodDoMessage (KozoDispatchStack *ds, doMessageMethodArgs *dmma)
{
	short err = noErr;
	VerletMethod *verlet;
  	
	verlet  = (VerletMethod *) ds->thisPD;
	switch (dmma->message)
	{
		case kSetNewContainerMessage:
			if (dmma->data)
			{
			      verlet->sim = nilPointer;
			      kozoObjectGetObjectInHierarchy((kozoObject *)dmma->data, ClassSimulation, nilPointer , & verlet->sim);  
			}
			goto Default;
		case kThermalizeMessage:
		case kSetToZeroMessage:
			err = dispatchLoop(ds, dmma);
 			break;
		case kSanChangedInfoMessage :
 			break;
		Default :
		default :
			KDSReturnContinue(ds);
			break;
	}
	KDSDoneDispatch(ds);
	KDSReturnError(ds, err);
}

short  VerletGetData (KozoDispatchStack *ds, getMethodArgs *gma)
{
	short err = noErr;
	VerletMethod *verlet;
  	
	verlet  = (VerletMethod *) ds->thisPD;
	switch (gma->property)
	{
		case pItersPerTimestep:
		 	err = makeLongDesc ( verlet->verletItersPerSimIter, gma->result);
			break;
		case pMaximumDisplacement:
		 	err = makeDoubleDesc ( verlet->maxDelPos,  gma->result);
			break;
 		default:
			KDSReturnContinue(ds);
	} 
	KDSDoneDispatch(ds);
	KDSReturnError(ds, err);
}
short  VerletSetData (KozoDispatchStack *ds, setMethodArgs *sma)
{
	short err = noErr;
	VerletMethod *verlet;
  	
	verlet  = (VerletMethod *) ds->thisPD;
	switch (sma->property)
	{
		case pItersPerTimestep:
		 	OHCoerceToLong(sma->data,  &verlet->verletItersPerSimIter );
			break;
		case pMaximumDisplacement:
		 	OHCoerceToDouble (sma->data, &verlet->maxDelPos);
			break;
 		default:
			KDSReturnContinue(ds);
	}
 	KDSDoneDispatch(ds);
	KDSReturnError(ds, err);
}
 
float    estHypotenuse(float a, float b);
float    estHypotenuse(float a, float b)
{  
	float pointTwoA;  
	float pointFourA;  
	float pointSixA;  
	float hypot ;
	
	 b = fabs(b);
	 a = fabs(a);
 	
    if (b > a)
    {
    	float temp = a;
    	a = b;
    	b = temp;
    }
    // now we know a is greater than b
    pointTwoA = a * 0.2;
    pointFourA = pointTwoA + pointTwoA;
    if ( b < pointFourA)
    {
        if (b < pointTwoA)
        {
	        hypot = a +  0.099 * b  ;
	    }
	    else
	    {
	        hypot = 1.0198 * a +  0.28615 * ( b - pointTwoA) ;
	    }
	}
	else
	{
		pointSixA = pointTwoA + pointFourA;
        if (b < pointSixA)
        {
	         hypot = (1.07703 * a) + 0.4458 * (b - pointFourA) ;
	    }
	    else
	    {
	         hypot = (1.16619 * a) + 0.62005 * (b - pointSixA) ;
	    }
    }
    return(hypot);
}

 
float    estVectorLength(float a, float b, float c);
float    estVectorLength(float a, float b, float c)
{  
    float len;
    
    len = estHypotenuse(a,b);
    len = estHypotenuse (len, c);
			  // all this just to avoid calculating a square root
	return (len);
}
 
float    getMaxDel(VerletMethod *verlet,  particle *basePart, long numParts);
float    getMaxDel(VerletMethod *verlet,  particle *basePart, long numParts)
{  
    float maxVel = 0;
	long partArrIndex;
	for (partArrIndex = 0; partArrIndex<numParts; ++partArrIndex)
	{
		float thisMax;
#ifndef __2DVectors__
		thisMax = estVectorLength(basePart->baseVelocity[partArrIndex].a, basePart->baseVelocity[partArrIndex].b, basePart->baseVelocity[partArrIndex].c);
#else
		thisMax = estVectorLength(basePart->baseVelocity[partArrIndex].a, basePart->baseVelocity[partArrIndex].b, 0L);
#endif
		if (thisMax > maxVel) maxVel = thisMax;
	}
    maxVel *= verlet->verletDeltaT; // turns maxVel into maxDel
    return (maxVel);
}


  
  
 short VerletIteration ( kSanMethodPlugIn *mpi,  kSanSimulation *sim) // long	doVerletMDTimeStep (void)
{
	short err = noErr;
  	long numGroups;
 	VerletMethod *verlet ;
 	kozoObject *verletObj = mpi->mpiObj;
 	verlet = mpi->methodPlugPrivateData;
 	
 	verlet->verletItersDone = 0;
 	verlet->verletDeltaT = sim->deltaT /  ((float) verlet->verletItersPerSimIter);
 	
 	if (verlet->sim == sim)
 	{
	 	numGroups = simGetNumGroups(sim);
		verlet->baseGroup = simLockGroups(sim);
		verlet->baseType = simLockAtomTypes(sim);
	 	verlet->basePart = simLockParts(sim);
	 	verlet->numGroups  = simGetNumGroups(sim);
	 	
	 	while (verlet->verletItersDone < verlet->verletItersPerSimIter)
	 	{
 		 	
			// install the values previously calculated
			// and move the current positions to the previous positions
			err = verletChangePositions(verletObj, verlet); 

			// using the velocity verlet scheme, get the half step velocities
			err = verletCalcHalfVelocities(verletObj, verlet); 
		  	
			// having changed the positions, we need to calculate the force
		  	err = calculateForce(sim);
		  	
			// using the velocity verlet scheme, get the full step velocities -- same as half velocities, because the accels have been updated
			err = verletCalcHalfVelocities(verletObj, verlet ); 
		  	
 			verlet->verletItersDone += 1;
		} // end verletIters
		
		// we have an automatic sensor for whether the timestep can be increased
		if ((verlet->numSequentialNoTimestepChange > 50)  && (verlet->verletItersPerSimIter > 1 ))
		{
			verlet->verletItersPerSimIter  = verlet->verletItersPerSimIter  - 1;
 		} 

		simIncrementStateCount(sim);
		simReleaseGroups(sim);
		simReleaseAtomTypes(sim);
		simReleaseParts(sim);
	}
 	return (err);
}


  
short 	verletChangePositions(kozoObject *mpiObj, VerletMethod *verlet )
{
	short err = noErr;
	short changedDT;
 	long gpIndex;
 	kSanGroup *gp;
  	kSanSimulation *sim = verlet->sim;
 	 float maxDel ;
	 float nthMaxDel = 0.0;
  
 		maxDel = 0.0;
	 	for(gpIndex=0;  gpIndex< verlet->numGroups; ++gpIndex)
		{
			gp =  verlet->baseGroup[gpIndex];
 		 	
			if ((gp->XYZConstraints[0] == kGroupMovesFreely)  && (gp->XYZConstraints[1] == kGroupMovesFreely)
#ifndef __2DVectors__
				  && (gp->XYZConstraints[2] == kGroupMovesFreely)
#endif
														  ) // totally free
			{
				 nthMaxDel = verletChangeGroupPositionsFree(mpiObj, verlet, gp);
				 
			}
			else if  ((gp->XYZConstraints[0] == 0)  && (gp->XYZConstraints[1] == 0) 
#ifndef __2DVectors__
				 && (gp->XYZConstraints[2] == 0) 
#endif
				  								 ) // fixed 
			{
				 nthMaxDel = verletChangeGroupPositionsFixed (mpiObj, verlet, gp);
	 		}
			else if  ((gp->XYZConstraints[0] != kGroupMovesFreely)  &&
				(gp->XYZConstraints[1] != kGroupMovesFreely) 
#ifndef __2DVectors__
				 && (gp->XYZConstraints[2] != kGroupMovesFreely)
#endif
				 								                 ) // moves as a block in 1 2 or 3 axes
			{
				 nthMaxDel = verletChangeGroupPositionsBlock (mpiObj, verlet, gp);
			}
			else  // some other combination of fixed, block and free
			{
				 nthMaxDel = verletChangeGroupPositionsMixed (mpiObj, verlet, gp);
			}
			if (nthMaxDel > maxDel) maxDel = nthMaxDel;

 		}
	    // heres where we handle the case of too much change in position
		if (maxDel > verlet->maxDelPos)
		{
           
            if (verlet->verletDeltaT  > 0.0 )
            {  // elegant way to undo this step
            	 verlet->verletDeltaT = -  verlet->verletDeltaT ;
            	 verletChangePositions( mpiObj,  verlet );
            	 verlet->verletDeltaT = -  verlet->verletDeltaT ;
           }

			verlet->verletItersPerSimIter *=  2;
			verlet->verletItersDone *=  2;
			verlet->verletDeltaT *= 0.5;
			verlet->numSequentialNoTimestepChange = 0;
            changedDT = true;
            verletChangePositions( mpiObj,  verlet );
  		}
		else if (maxDel < (verlet->maxDelPos * verlet->maxDelSafetyFactor))
 		{
		     ++verlet->numSequentialNoTimestepChange;
		}
		else
		{
			verlet->numSequentialNoTimestepChange = 0;
		}
 
  	
	return (noErr);
}
float verletChangeGroupPositionsFree(kozoObject *verletObj, VerletMethod *verlet, kSanGroup *gp)
{
#pragma unused(verletObj)
 	short err = noErr;
	long partArrIndex;
	long i;
	long *partList = lockList (&gp->particles);  
	long numParts = OHListCount (&gp->particles);  
	particle *basePart = verlet->basePart;
	kSanVector maxDel;
	
	kSanVector *basePos = verlet->basePart->basePosition;
	kSanVector *baseVel = verlet->basePart->baseVelocity;
	kSanVector *baseForce = verlet->basePart->baseForce;
	double oneHalfDelTSq =  0.5 * verlet->verletDeltaT * verlet->verletDeltaT;
	
    kSanVector delPos;
    kSanVector nthAccel;
    
  	for(i=0;i<numParts;++i)
	{
		partArrIndex = partList[i];
	    multkSanVectorD(  &baseVel[partArrIndex], verlet->verletDeltaT, &delPos);
	    multkSanVectorD(  &baseForce[partArrIndex],   verlet->baseType[BPNthType(basePart, partArrIndex)]->specs.inverseMass,  &nthAccel);
		multAddkSanVectorsD( &delPos , &nthAccel , oneHalfDelTSq, &delPos);
 	    addkSanVectorsD (&basePos[partArrIndex], &delPos, &basePos[partArrIndex]);
 	    if (maxDel.a < delPos.a) maxDel.a = delPos.a;
 	    if (maxDel.b < delPos.b) maxDel.b = delPos.b;
#ifndef __2DVectors__
	    if (maxDel.c < delPos.c) maxDel.c = delPos.c;
#endif 	   
 	}	

	releaseList (&gp->particles);  
 	return (lengthOfkSanVectorD (&maxDel));
}
float verletChangeGroupPositionsBlock(kozoObject *verletObj, VerletMethod *verlet, kSanGroup *gp)
{
#pragma unused(verletObj)
 	short err = noErr;
	long partArrIndex;
	long i;
	long *partList = lockList (&gp->particles);  
	long numParts = OHListCount (&gp->particles);  
	particle *basePart = verlet->basePart;
	
	kSanVector *basePos = verlet->basePart->basePosition;
	kSanVector *baseVel = verlet->basePart->baseVelocity;
	kSanVector *baseForce = verlet->basePart->baseForce;
	double oneHalfDelTSq =  0.5 * verlet->verletDeltaT * verlet->verletDeltaT;
	double nthMass ;
	double totalMass = 0.0 ;
	
     kSanVector delPos;
    kSanVector accel;
    kSanVector inertia;
    kSanVector forceSum;
    
    zerokSanVectorD (&forceSum);
    zerokSanVectorD (&inertia);
    
  	for(i=0;i<numParts;++i)
	{
		partArrIndex = partList[i];
		nthMass =  verlet->baseType[BPNthType(basePart, partArrIndex)]->specs.inverseMass;
		totalMass += nthMass;
		multAddkSanVectorsD( &inertia ,  &baseVel[partArrIndex], nthMass,  &inertia);
		addkSanVectorsD (&forceSum, &baseForce[partArrIndex], &forceSum);  
 	}	

    multkSanVectorD( &inertia, verlet->verletDeltaT / totalMass ,  &delPos);
    multkSanVectorD(  &forceSum, 1.0 / totalMass ,  &accel);
	multAddkSanVectorsD( &delPos , &accel , oneHalfDelTSq, &delPos);
	
  	for(i=0;i<numParts;++i)
	{
		partArrIndex = partList[i];
 
 	    addkSanVectorsD (&basePos[partArrIndex], &delPos, &basePos[partArrIndex]);
 	}	
 
	releaseList (&gp->particles);  
 	return (lengthOfkSanVectorD (&delPos));
}


float   verletChangeGroupPositionsFixed(kozoObject *verletObj, VerletMethod *verlet, kSanGroup *gp)
{
// this func will return the maxDel
 	short err = noErr; 
 		return (0.0);
}


float   verletChangeGroupPositionsMixed(kozoObject *verletObj, VerletMethod *verlet, kSanGroup *gp)
{
// this func will return the maxDel
 	short err = noErr;
 	short axis;
	long partArrIndex;
	long i;
	long *partList = lockList (&gp->particles);  
	long numParts = OHListCount (&gp->particles);  
	
	kSanVector *basePos = verlet->basePart->basePosition;
	kSanVector *baseVel = verlet->basePart->baseVelocity;
	kSanVector *baseForce = verlet->basePart->baseForce;
	particle *basePart = verlet->basePart;
	kSanVector maxDel;
	double oneHalfDelTSq =  0.5 * verlet->verletDeltaT * verlet->verletDeltaT;
	double nthMaxDel;
//	double nthMass ;
	double totalMass = 0.0 ;
	
    double delPos;
    double accel;
    double inertia;
    double forceSum;
    
    forceSum = 0;
    inertia = 0 ;
	zerokSanVectorD(&maxDel);
	
	for (axis = 0; axis < NUMDIMENSIONS; ++axis)
	{
		nthMaxDel   = 0.0;
		if (gp->XYZConstraints[axis] == kGroupMovesFreely) 
		{
		  	for(i=0;i<numParts;++i)
			{
				partArrIndex = partList[i];
								
			    delPos =  ((double *)(&baseVel[partArrIndex]))[axis] *  verlet->verletDeltaT;
				accel = ((double *)(&baseForce[partArrIndex]))[axis] *  verlet->baseType[BPNthType(basePart, partArrIndex)]->specs.inverseMass;
				delPos += accel * oneHalfDelTSq;
				((double *)(&basePos[partArrIndex]))[axis] += delPos;
				
				if (nthMaxDel < delPos ) nthMaxDel  = delPos;
		 	}	
		 	
		}
		else if (gp->XYZConstraints[axis] == kGroupMovesAsABlock) 
		{
			double inertia = 0.0;
		 	double forceSum = 0.0;
 			double totalMass = 0.0;
			double thisMass = 0.0;
		  	
			for (i = 0; i< numParts; ++i)
			{
				partArrIndex = partList[i];
 				thisMass = verlet->baseType[BPNthType(basePart, partArrIndex)]->specs.mass;
				totalMass += thisMass;
				inertia += ((double *)(&baseVel[partArrIndex]))[axis]  * thisMass ;
				forceSum += ((double *)(&baseForce[partArrIndex]))[axis];
		 	}	
		 	delPos = inertia * verlet->verletDeltaT / totalMass;
		 	accel = forceSum/ totalMass;
		 	delPos += accel* oneHalfDelTSq;
            nthMaxDel = delPos;
  			for (i = 0; i< numParts; ++i)
			{
				partArrIndex = partList[i];
				
				((double *)(&basePos[partArrIndex]))[axis] += delPos; 
		 	}	
		}
		else // axis is fixed
		{
  		}
  		((double *)(&maxDel))[axis] = nthMaxDel;
	}
  
	releaseList (&gp->particles);  

 	return (lengthOfkSanVectorD (&maxDel));
}




 short verletCalcHalfVelocities(kozoObject *verletObj, VerletMethod *verlet )
{
 	long i;
	long numParts = simGetNumParts (verlet->sim);  
	
 	kSanVector *baseVel = verlet->basePart->baseVelocity;
	kSanVector *baseForce = verlet->basePart->baseForce;
	kSanParticle *basePi =  verlet->basePart->basePi;
	
 	double oneHalfDelT  =  0.5 * verlet->verletDeltaT ;
	kSanVector ithAccel;
       
  	for(i=0;i<numParts;++i)
	{
	    multkSanVectorD(  &baseForce[i], verlet->baseType[basePi[i].type]->specs.inverseMass, &ithAccel);
        multAddkSanVectorsD (&baseVel[i],  &ithAccel , oneHalfDelT, &baseVel[i]);
    }
 	return (noErr);
}




 
 short VerletCooling (  kSanMethodPlugIn *mpi,  kSanSimulation *sim, double *netTime, double *totKinEnergy)
{
	kSanGroup *gp;
	kSanGroup **baseGroup;
	long numParts;
	long i;
	long *list;
	short numGroups;
	short groupI;
	VerletMethod *verlet ;
 	kozoObject *verletObj = mpi->mpiObj;
 	particle *basePart;
	kSanAtomtype **types;
  	float largerOfTemps;
	float myCoolFac;
	float nthCoolFac;
	float frac;
	float eqTempToUse;
	float devTimeToUse;
    float thirdKT;
	
 	verlet = mpi->methodPlugPrivateData;
 	if (verlet->sim == sim)
 	{
 	 	
	    types = simLockAtomTypes(sim);
	 	numGroups = simGetNumGroups(sim);
		baseGroup = simLockGroups(sim);
		basePart = simLockParts(sim);
 	 	
	 	for (groupI = 0; groupI<numGroups; ++groupI)
		{
			gp = baseGroup[groupI];
			if (gp->doCooling)
			{
				numParts = OHListCount(&gp->particles);
				if (isnan(gp->actualTemp)) gp->actualTemp = 0;
				if (isnan(gp->eqTemp)) gp->eqTemp = 0;

				gp->recentTempDeviationCollTime += (*netTime);
				if ((4.0 * gp->coolingTimeConstant) > gp->recentTempDeviationCollTime)
				{
					 devTimeToUse = gp->recentTempDeviationCollTime;
				}
				 else
				 {
					 devTimeToUse = 4.0 * gp->coolingTimeConstant;
				 }
				 
				frac = ((*netTime) /   devTimeToUse );
				
				if (frac > 1.0) frac = 1.0;
				else if (frac < 0) frac = 0;
				
				if (gp->actualTemp < (0.1 * gp->eqTemp))
				{
				     gp->recentTempDeviation = 0;
				}
				
 			    gp->recentTempDeviation +=  (gp->actualTemp - gp->eqTemp) *  frac ;
  			    eqTempToUse = gp->eqTemp - gp->recentTempDeviation;

				list = lockList(&gp->particles);
				if (eqTempToUse > gp->actualTemp)  largerOfTemps = eqTempToUse;
				else   largerOfTemps = gp->actualTemp;
				if (largerOfTemps > 0.1)
				{
					float aveKE;
					aveKE = 
					myCoolFac = 1.0 +  (((eqTempToUse - gp->actualTemp ) * (*netTime) )/
						 (gp->coolingTimeConstant * largerOfTemps) ) ;

			          if (myCoolFac > 1.1) myCoolFac = 1.1;
			          else if (!(myCoolFac > 0.0)) myCoolFac = 0.0001;

					if (myCoolFac > 1.0)
					{
						for (i = 0; i < numParts; ++i)
						{
							long partArrIndex = list[i];
 						   
							multkSanVectorD (&basePart->baseVelocity[partArrIndex], myCoolFac, &basePart->baseVelocity[partArrIndex]);
 			 			}
					}
					else
					{
			//  sim->simTemperature = kinenergy * kTwoOverThreeK / (float)simGetNumParts(sim);
            //  #define kTwoOverThreeK 7729.4686  // units are per eV
						 #define  eVUnitsOneHalfKT  	5797.10145 // units are   per eV
						 #define  olofUnitsOneHalfKT  	0.601558759132 // units are  <null>   per  (g/mol) A / ps ps )
						 #define  olofUnitsOneThirdKT  	0.401039172755 // units are  <null>   per  (g/mol) A / ps ps )
                        // want to multiply this by (eV per (g/mol) A A / ps ps)   
                         // i.e. multiply by ( 1 / 9636.8 )
                         // oneHalf kT is 0.601558759132 in units of    <null>   per  (g/mol) A / ps ps )

  
					    myCoolFac = myCoolFac * myCoolFac; // apply to energies now!
					    thirdKT = olofUnitsOneThirdKT;
						for (i = 0; i < numParts; ++i)
						{
							long partArrIndex = list[i];
							double nthKE;
							float nthAboveThirdKT;
							float newKE;
							
							nthKE = dotkSanVectorsD (&basePart->baseVelocity[partArrIndex], &basePart->baseVelocity[partArrIndex] );
 							nthKE  *= ( types[BPNthType(basePart, partArrIndex)]->specs.mass );	
							
							// nthKE in units of (g/mol) A A / ps ps
							if (nthKE > thirdKT)
							{
							    nthAboveThirdKT = nthKE - thirdKT;
 							    newKE = thirdKT + nthAboveThirdKT * myCoolFac;
							    nthCoolFac = pow ((newKE / nthKE), 0.5);
 							   
							    multkSanVectorD ( &basePart->baseVelocity[partArrIndex], nthCoolFac, &basePart->baseVelocity[partArrIndex] );
 							}
			 			}
		 			}
				}
				releaseList (&gp->particles);
			}
		}
		*totKinEnergy = -1;

 		simReleaseGroups(sim);
		simReleaseParts(sim);
	    simReleaseAtomTypes(sim);
	}
 	return (noErr);
}



void checkVectorNAN(kSanVector * vec);
void checkVectorNAN(kSanVector * vec)
{
		if (vectorIsNan(vec))
		{
			OHSystemBeep();
			fixNANVector(vec);
		}
 }

 


