#include "kSanObjects.h"
#include "kSanSimulation.h"
#include "kSanSimulationPublic.h"
#include "kSanAEHandlers.h"
#include "kSanAECallbacks.h"
#include "kSanAEDefines.h"
short kSanDoEqualParticlesComparison ( AEDesc *firstObject,  AEDesc *secondObject, short *isItTrue);
short ParticleTokenContainsParticleToken ( AEDesc *firstObject,  AEDesc *secondObject, short *isItTrue);
short GroupTokenContainsParticleToken ( AEDesc *firstObject,  AEDesc *secondObject, short *isItTrue);
short WindowTokenContainsParticleToken ( AEDesc *firstObject,  AEDesc *secondObject, short *isItTrue);
short kSanDoContainsComparison ( AEDesc *firstObject,  AEDesc *secondObject, short *isItTrue);
short SelectionTokenContainsParticleToken ( AEDesc *firstObject,  AEDesc *secondObject, short *isItTrue);
short evaluateObject (AEDesc *token, double *value);

pascal OSErr kSanDisposeToken(AEDesc *theToken)
{
	short err = errAEEventNotHandled;
	if (theToken->descriptorType == ClassPlugIn)
	{
		err = disposePlugInTokenDesc(theToken);
	}
	else if (theToken->descriptorType == ClassParticle)
	{
		err = disposeParticleTokenDesc(theToken);
	}
	else if (theToken->descriptorType == ClassPlugInOwnedObject)
	{
		err = disposePlugInOwnedObjectTokenDesc(theToken);
	}
	else if (theToken->descriptorType == ClassProperty)
	{
		kSanPropertyToken myTok;
		if (theToken->dataHandle == nilPointer)
		{
			SysBeep(20);
		}
		else
		{
			myTok = (*(kSanPropertyToken *)(*theToken->dataHandle));
			if (myTok.objClass == ClassParticle)
			{
				err = disposeParticleToken((kSanParticleToken *)myTok.objPtr);
			}
			else if (myTok.objClass == ClassPlugIn)
			{
				err = disposePlugInToken((kSanPlugInToken *)myTok.objPtr);
			}
			else if (myTok.objClass == ClassPlugInOwnedObject)
			{
				err = disposePlugInOwnedObjectToken((kSanPlugInOwnedObjectToken *)myTok.objPtr);
			}
		}
	}
	return (err);
}

short kSanDoEqualParticlesComparison ( AEDesc *firstObject,  AEDesc *secondObject, short *isItTrue)
{
	short err = noErr;
	short resultOne;
	short resultTwo;
	kSanParticleToken *partTokenOne; 
	kSanParticleToken *partTokenTwo;
	partTokenOne = (*(kSanParticleToken **)(*firstObject->dataHandle));
	partTokenTwo = (*(kSanParticleToken **)(*secondObject->dataHandle));
	resultOne = listContainsList (&partTokenOne->partList, &partTokenTwo->partList);
	resultTwo = listContainsList ( &partTokenTwo->partList, &partTokenOne->partList);
	*isItTrue = (resultOne && resultTwo);
	return (err);
}

short ParticleTokenContainsParticleToken ( AEDesc *firstObject,  AEDesc *secondObject, short *isItTrue)
{
	short err = noErr;
	short result ;
	kSanParticleToken *partTokenOne; 
	kSanParticleToken *partTokenTwo;
	partTokenOne = (*(kSanParticleToken **)(*firstObject->dataHandle));
	partTokenTwo = (*(kSanParticleToken **)(*secondObject->dataHandle));
	result = listContainsList (&partTokenOne->partList, &partTokenTwo->partList);
	*isItTrue = result;
	return (err);
}

short GroupTokenContainsParticleToken ( AEDesc *firstObject,  AEDesc *secondObject, short *isItTrue)
{
	short err = noErr;
	short result ;
	kSanGroup *gp; 
	kSanParticleToken *partToken; 
	gp = (*(kSanGroup **)(*firstObject->dataHandle));
	partToken = (*(kSanParticleToken **)(*secondObject->dataHandle));
	if ((long) gp->obj.container != partToken->owningSim) result = false;
	else  result = listContainsList (&gp->particles, &partToken->partList);
	*isItTrue = result;
	return (err);
}

short WindowTokenContainsParticleToken ( AEDesc *firstObject,  AEDesc *secondObject, short *isItTrue)
{
	short err = noErr;
	short result ;
	AGWindow *ag; 
	kSanParticleToken *partToken; 
	ag = (*(AGWindow **)(*firstObject->dataHandle));
	partToken = (*(kSanParticleToken **)(*secondObject->dataHandle));
	if (ag->win.obj.oClass != ClassAGWindow) result = false;
	else if ((long) ag->win.obj.container != partToken->owningSim) result = false;
	else  result = listContainsList (&ag->partList, &partToken->partList);
	*isItTrue = result;
	return (err);
}

short SelectionTokenContainsParticleToken ( AEDesc *firstObject,  AEDesc *secondObject, short *isItTrue)
{
	short err = noErr;
	short result ;
	kSanSimulation *sim; 
	kSanParticleToken *partToken; 
	kSanPropertyToken propToken; 
	propToken = (*(kSanPropertyToken *)(*firstObject->dataHandle));
	sim = (kSanSimulation *) propToken.objPtr;
	partToken = (*(kSanParticleToken **)(*secondObject->dataHandle));
	if (propToken.propCode != pSelection) result = false;
	else if ((long) sim != partToken->owningSim) result = false;
	else  result = listContainsList (&sim->selections, &partToken->partList);
	*isItTrue = result;
	return (err);
}

short kSanDoContainsComparison ( AEDesc *firstObject,  AEDesc *secondObject, short *isItTrue)
{
	short err = noErr;
	switch (secondObject->descriptorType)
	{
		case ClassParticle:
			switch (firstObject->descriptorType)
			{
				case ClassParticle:
					ParticleTokenContainsParticleToken (firstObject, secondObject, isItTrue);
					break;
				case ClassWindow:
					WindowTokenContainsParticleToken (firstObject, secondObject, isItTrue);
					break;
				case ClassParticleGroup:
					GroupTokenContainsParticleToken (firstObject, secondObject, isItTrue);
					break;
				case ClassSelection:
					SelectionTokenContainsParticleToken (firstObject, secondObject, isItTrue);
					break;
			}
			break;
	}
	return (err);
}

short evaluateObject (AEDesc *token, double *value)
{
	short err = errAEEventNotHandled;
	AEDesc coercedDesc = {typeNull, nilPointer};
	
	if (token->descriptorType == cProperty)
	{
		AEDesc dataDesc = {typeNull, nilPointer};
		getTokenData (token, &dataDesc);
		err = kSanCoerceToDouble (&dataDesc, value);	
		AEDisposeDesc (&dataDesc);
	}
	else
	{
		err = kSanCoerceToDouble (token, value);	
	}
	AEDisposeDesc (&coercedDesc);
	return (err);
}

short kSanDoNumericComparison ( DescType comparisonOperator, AEDesc *firstObject,
					 AEDesc *secondObject, short *isItTrue)
{
	short err = noErr;
	double firstObjectValue;
	double secondObjectValue;
	err = evaluateObject(firstObject, &firstObjectValue);
	if (!err) err = evaluateObject(secondObject, &secondObjectValue);
	if (!err)
	{
		switch ( comparisonOperator )
		{
			case kAEGreaterThan:
				(*isItTrue) = (firstObjectValue > secondObjectValue);
				break;
			case kAEGreaterThanEquals:
				(*isItTrue) = (firstObjectValue >= secondObjectValue);
				break;
			case kAEEquals:
				(*isItTrue) = (firstObjectValue == secondObjectValue);
				break;
			case kAELessThan:
				(*isItTrue) = (firstObjectValue < secondObjectValue);
				break;
			case kAELessThanEquals:
				(*isItTrue) = (firstObjectValue <= secondObjectValue);
				break;
		}	
	}
	return (err);
}

short disposeParticleTokenDesc(AEDesc *theToken)
{
	short err;
	kSanParticleToken *myToken;
	myToken = (*(kSanParticleToken **)*theToken->dataHandle);
	err = disposeParticleToken (myToken);
	return (err);
}
short disposePlugInTokenDesc(AEDesc *theToken)
{
	short err;
	kSanPlugInToken *myToken;
	myToken = (*(kSanPlugInToken **)*theToken->dataHandle);
	err = disposePlugInToken (myToken);
	return (err);
}
short disposePlugInOwnedObjectTokenDesc(AEDesc *theToken)
{
	short err;
	kSanPlugInOwnedObjectToken *myToken;
	myToken = (*(kSanPlugInOwnedObjectToken **)*theToken->dataHandle);
	err = disposePlugInOwnedObjectToken (myToken);
	return (err);
}

short disposeParticleToken(kSanParticleToken *theToken)
{
	--theToken->refCount;
	if (theToken->refCount == 0)
	{
		disposeList(&theToken->partList);
		DisposePtr ((Ptr)theToken);
	}
	return (noErr);
}
short disposePlugInToken(kSanPlugInToken *theToken)
{
	--theToken->refCount;
	if (theToken->refCount == 0)
	{
		DisposePtr ((Ptr)theToken);
	}
	return (noErr);
}
short disposePlugInOwnedObjectToken(kSanPlugInOwnedObjectToken *theToken)
{
	--theToken->refCount;
	if (theToken->refCount == 0)
	{
		DisposePtr ((Ptr)theToken);
	}
	return (noErr);
}

kSanPlugInToken *copyPlugInToken(kSanPlugInToken *myToken)
{
	++myToken->refCount;
	return (myToken);
}
kSanPlugInOwnedObjectToken *copyPlugInOwnedObjectToken(kSanPlugInOwnedObjectToken *myToken)
{
	++myToken->refCount;
	return (myToken);
}

kSanParticleToken *copyParticleToken(kSanParticleToken *myToken)
{
	++myToken->refCount;
	return (myToken);
}

short initializePlugInToken(kSanPlugInToken **myToken, long sim, long plugPtr)
{
	short err;
	*myToken = (kSanPlugInToken  *) NewPtr(sizeof(kSanPlugInToken));
	err = MemError();
	if (!err)
	{
		(*myToken)->owningSim = (long) sim;
		(*myToken)->plug = (long) plugPtr;
		(*myToken)->refCount = 1;
	}
	return (err);
}
short initializePlugInOwnedObjectToken(kSanPlugInOwnedObjectToken **myToken, long sim, long plugPtr)
{
	short err;
	*myToken = (kSanPlugInOwnedObjectToken  *) NewPtr(sizeof(kSanPlugInOwnedObjectToken));
	err = MemError();
	if (!err)
	{
		(*myToken)->owningSim = (long) sim;
		(*myToken)->plug =  plugPtr;
		(*myToken)->objClass = 0L;
		(*myToken)->objPtr = 0L;
		(*myToken)->refCount = 1;
	}
	return (err);
}

short initializeParticleToken(kSanParticleToken **myToken)
{
	short err;
	*myToken = (kSanParticleToken *) NewPtr (sizeof(kSanParticleToken));
	err = MemError();
	if (!err)
	{
		(*myToken)->refCount = 1;
		err = initializeList(&(*myToken)->partList);
	}
	return (err);
}
