#include "kSanAGPane.h"
#include "MacOSUtils.h"


void AGPaneMaintainCursor(AGPane *ag);
void AGPaneSetMouseModeButtons(AGPane *ag);

short AGPaneAddParticleList (AGPane *ag,  OHList *newPartList);
short AGPaneAddToPartList(AGPane *ag, long partToAdd);
void AGPaneRemoveParticleList(AGPane *ag, OHList * list);




void initializeWindowOrderList(AGPane *ag )
{
	long *thisList;
	long i;
	long count;
	count = numItems(&ag->partList); 
	SetHandleSize(ag->orderList.listData, sizeof(long) * count);
	thisList = lockList(&ag->orderList); 
	for (i = 0; i < count; ++i)
	{
		thisList[i] = i;
	}
	releaseList(&ag->orderList); 
}

/************  DoShowBondsSwitch   ****************/
void doShowBondsSwitch	(AGPane *ag)
{
	setBondsVisible (ag, ((short)(!(ag->windowView.showBonds))));
	callDoMessage(OHOb(ag), kInvalSelfMessage, 0L);
}
#define      iStopMovie 8



void 	doSelectedRotate(AGPane *ag, char myKeyCode, short timesFive)
{
	float latAngle;	
	float longAngle;
	float spinAngle;
	newMatrixF fMat;
	newMatrixF cMat;
	getAngles(ag, myKeyCode,&latAngle, &longAngle, &spinAngle);	
	if (timesFive)
	{
		latAngle *= 5;
		longAngle *= 5;
		spinAngle *= 5;
	}
	identityMatrixF(&fMat);
	SpinMatrixInDegrees(&fMat, latAngle, longAngle, spinAngle);	
	getCramersOperatorsF(&fMat, &cMat);
	
	rotateParticles(ag->sim, &fMat, true) ; // selected only
}

void defaultView(viewInfo *dView)
{	
	dView->magnification =12;
	dView->perspective = 1000;
	dView->particleSizeFactor = 1.0;
	ResetMatrix (&dView->viewMatrix);
	dView->cellCenter.h = 100; 
	dView->cellCenter.v = 100;
	dView->showBonds = FALSE;
	dView->shadeMethod = kNoShade;
	dView->sizeMethod = kNoSize;
	dView->BGColour = getColour(myWhiteColour);
	dView->neutralShadeDepth = 0.0;
	dView->shadeRange = 20.0;
}

void	AGPaneDoMouseModeSwitch	(AGPane*ag)
{
	if (ag->mode == kSelectAndHighlight)
	{
		AGPaneSetMouseMode (ag, kMouseManipulation );
	}
	else if (ag->mode == kMouseManipulation)
	{
		AGPaneSetMouseMode ( ag, kSelectAndHighlight );
	}
}


void AGPaneSetMouseModeButtons(AGPane *ag)
{
	callDoMessage (OHOCtr(ag), kSetMouseModeButtonsMessage , 0L);
}

void AGPaneSetWindowCenter(AGPane *ag)
{
	short nowVal, maxVal, wHeight, wWidth;
	Rect worldRect;
	Point newCenter;

	OHGWorldPaneGetWorldRect(&ag->gwp, &worldRect);

	wHeight = worldRect.bottom - worldRect.top;
	wWidth = worldRect.right -worldRect.left;
	
	if (ag->gwp.pane.vScroll)
	{
		nowVal = GetControlValue(ag->gwp.pane.vScroll->control.cHandle);
		maxVal = GetControlMaximum(ag->gwp.pane.vScroll->control.cHandle);
		newCenter.v =  (( (maxVal *0.5)- nowVal )  * kMaxWindowHeight / maxVal)   ;
	}
	if (ag->gwp.pane.hScroll)
	{
		nowVal = GetControlValue(ag->gwp.pane.hScroll->control.cHandle);
		maxVal = GetControlMaximum(ag->gwp.pane.hScroll->control.cHandle);
		newCenter.h =    (( (maxVal *0.5) - nowVal) * kMaxWindowWidth / maxVal) ;
	}
	ag->windowView.cellCenter = newCenter;
}

short particleIsInWindow (AGPane *ag , long whichPart)
{
	long index;
	short retVal;
	index = whichItem(&ag->partList, whichPart);
	if (index == -1) retVal = false;
	else retVal = true;
	return (retVal);
}

grafData * lockGrafData  (AGPane *ag) 
{
	grafData *gd;
	if (ag->grafLockCount == 0) HLock(ag->grafDataHandle);
	gd = (grafData *) (*ag->grafDataHandle);
	++ag->grafLockCount;
	return (gd);
}
void releaseGrafData  (AGPane *ag) 
{
	--ag->grafLockCount;
	if (ag->grafLockCount == 0) HUnlock(ag->grafDataHandle);
}

short grafDataResize   (AGPane *ag, long numParts) 
{
	short err;
	long i;
	long oldNumParts;
	grafData * myPtr;
		
	oldNumParts = numItems(&ag->partList);
	SetHandleSize (ag->grafDataHandle, (long)sizeof(grafData)*(numParts+1L));
	err = MemError();
	if (!err) 
	{
		myPtr = lockGrafData(ag);
		for (i=oldNumParts;i< numParts ;++i)
		{
			defaultGrafData(myPtr);
		}		
		releaseGrafData(ag);
	}
	return (err);
}

void defaultGrafData (grafData * gPtr)
{
	gPtr->inWindow = true;
	gPtr->selected = true;
	gPtr->grapha = 1;
	gPtr->graphb = 1;
	gPtr->graphc = 1;
	gPtr->howManyForebonds = 0 ;
	gPtr->howManyBackbonds = 0 ;
	gPtr->circle.right = 1;
	gPtr->circle.left = 0;
	gPtr->circle.top = 1;
	gPtr->circle.bottom = 2;
	gPtr->colour = getColour(myRedColour);
}

short AGPaneAddParticleList (AGPane *ag,  OHList *newPartList)
{ // take a list and add all of its pieces which are not already in the window
	short err = noErr;
	err =   OHListEveryItemAsSecondArg( newPartList,  AGPaneAddToPartList, (long) ag);
	return (err);
}	
short AGPaneAddToPartList(AGPane *ag, long partToAdd)
{
	long indexInAGList;
	long partCount;	
	short err = noErr;
	
	partCount = numItems(&ag->partList);
	indexInAGList = whichItem (&ag->partList, partToAdd );
	if (indexInAGList == -1)
	{	 // not already in list, so add it
		err = addToList(&ag->orderList, partCount); 
		if (!err)
		{
			SetHandleSize(ag->grafDataHandle, ((long)sizeof(grafData)*(partCount + 2L)));
			err = MemError();
		}
		if (!err) err = addToList (&ag->partList, partToAdd);

		if (!err)
		{
			grafData *myPtr;
			myPtr = lockGrafData(ag);
			defaultGrafData(&myPtr[partCount]);
			myPtr[partCount].index = partToAdd;
			releaseGrafData(ag);
		}
	}
	return (err);
}
void AGPaneChangeParticleIndex(AGPane * ag, long oldIndex, long newIndex)
{
	long *thisList;
	long i;
	long partsInWindow;
	grafData * gPtr;
	thisList = lockList(&ag->partList); 
	gPtr = lockGrafData(ag);
	i = 0;
	partsInWindow = numItems(&ag->partList);
	while ( i < partsInWindow)
	{
		if (thisList[i] == oldIndex) // ??? could optimize
		{
			thisList[i] = newIndex;
		} 
		if (gPtr[i].index == oldIndex)
		{
			gPtr[i].index = newIndex;
		} 
		++i;
	}
	releaseList(&ag->partList);  
	releaseGrafData(ag);
}	
void AGPaneRemoveParticleList(AGPane *ag, OHList * list)
{
	 OHListEveryItemAsSecondArg( list,  AGPaneRemoveParticle, (long) ag);
}	
short AGPaneRemoveParticle(AGPane *ag, long whichPart)
{
	long *thisList;
	long i;
	long partsInWindow;
	long oldIndexInPartList;
	grafData * gPtr;

	partsInWindow = numItems(&ag->partList);
	oldIndexInPartList = whichItem(&ag->partList, whichPart);
	if (oldIndexInPartList != -1)
	{
		removeItem(&ag->orderList, oldIndexInPartList); 
				//  if whichPart is the partList[3] take 3 out of the orderList
		i = 0;
		thisList = lockList(&ag->orderList);
		while ( i < (partsInWindow - 1))
		{
			if (thisList[i] > oldIndexInPartList) thisList[i] = thisList[i] - 1;  
			++i;
				// decrement all items greater than 3
		}
		releaseList(&ag->orderList);
		// order list done
		
		removeItem(&ag->partList, whichPart); 
		// partList done

		gPtr = lockGrafData(ag);
		i = 0;	
		while ( (gPtr[i].index != whichPart) && (  i < (partsInWindow - 1)))
		{
			++i;
		}
		while ( i < (partsInWindow - 1)) // start from the old index
		{   // copy each item in the list above to take the place of the deleted part
			gPtr[i] =  gPtr[i + 1];
			++i;
		}
		releaseGrafData(ag);
		// grafData done
	}	
	return (noErr);
}


void getAngles(AGPane*ag,char myKeyCode,float * latAngle,float * longAngle,float * spinAngle)
{
	*spinAngle = 0.0;
	switch (myKeyCode)
	{		
		case '1':		
			*latAngle = fabs(ag->manip.latRotateAngle);
			*longAngle =	-1.0 * fabs(ag->manip.longRotateAngle);
			break;

		case '2':		
			*latAngle = 0.0;
			*longAngle =	-1.0 * fabs(ag->manip.latRotateAngle);
			break;

		case '3':		
			*latAngle = -1.0 *fabs( ag->manip.latRotateAngle);
			*longAngle =	-1.0 *fabs(ag->manip.longRotateAngle);
			break;

		case '4':
			*latAngle = fabs(ag->manip.latRotateAngle);
			*longAngle =	0.0;
			break;	

		case '6':		
			*latAngle =  -1.0 *fabs(ag->manip.latRotateAngle);
			*longAngle =	0.0;
			break;			
				
		case '7':
			*latAngle = fabs(ag->manip.latRotateAngle);
			*longAngle =	fabs(ag->manip.longRotateAngle);
			break;	
				
		case '8':		
			*latAngle = 0.0;
			*longAngle =	fabs(ag->manip.longRotateAngle);
			break;

		case '9':		
			*latAngle =  -1.0 *fabs(ag->manip.latRotateAngle);
			*longAngle =	fabs(ag->manip.longRotateAngle);
			break;	
					
		case '/':		// ccw rotation
			*latAngle =  0.0;
			*longAngle =	0.0;
			*spinAngle =	-1.0 *fabs(ag->manip.axialRotateAngle);
			break;	
					
		case '*':		// cw rotation
			*latAngle =  0.0;
			*longAngle =	0.0;
			*spinAngle =	fabs(ag->manip.axialRotateAngle);
			break;						
			
		default:
			*latAngle = 0.0;
			*longAngle = 0.0;
			*spinAngle = 0.0;
			break;
	}
}
