#include "kSanAGWindow.h"
#include "MacOSUtils.h"


void AGWindowMaintainCursor( AGWindow *ag);

short AGWindowAddParticleList ( AGWindow *ag,  OHList *newPartList);
short AGWindowAddToPartList( AGWindow *ag, long partToAdd);
void AGWindowRemoveParticleList( AGWindow *ag, OHList * list);
void 	AGWindowDoSelectedRotate( AGWindow *ag, char myKeyCode, short timesFive);
char AGWindowParticleIsInWindow ( AGWindow *ag , long partArrIndex);

void defaultGrafData (grafData * gPtr);


short AGWindowReinitializeGrafDataList( AGWindow *ag)
{
	 long oldCount;
	 long count;
	 long i;
	 long *partArray;
	 grafData * gPtr;
	 
	 oldCount = VSListCount (&ag->grafDataList);
	 count = AGWindowNumParts(ag);
	 
  	ag->grafDataDirty = false;  // avoid recursive call
	gPtr = AGWindowLockGrafData( ag);
    for (i = count; i < oldCount; ++i)
	{
		if (gPtr[i].bonds)
		{
		     OHSystemBeep();
			 VSListDispose (&(gPtr[i].bonds->bondList));
			 DisposePtr((Ptr) gPtr[i].bonds) ;
		}
	}
	AGWindowReleaseGrafData(  ag);
	 
	 
 	 VSListSetNumItems (&ag->grafDataList,  count) ;
	 gPtr = AGWindowLockGrafData( ag);
	  agCheckPartList(ag);
	 partArray = lockList(&ag->particleList);
	 
	 for (i = 0; i < count; ++i)
	 {
	 	gPtr[i].bonds = nilPointer;
	 	defaultGrafData(&gPtr[i]);
	 	gPtr[i].index = partArray[i];
	 }
	 AGWindowReleaseGrafData(  ag);
  	 

  	 releaseList(&ag->particleList);
 	return (noErr);
}	

short AGWindowInitializeWindowOrderList( AGWindow *ag )
{
	short err = noErr;
	long *thisList;
	long i;
	long count;
	count = AGWindowNumParts(ag); 
	err = OHListSetNumItems(&ag->orderList,  count);
	if (!err)
	{
		thisList = lockList(&ag->orderList); 
		for (i = 0; i < count; ++i)
		{
			thisList[i] = i;
		}
		releaseList(&ag->orderList); 
	    ag->orderListDirty = false;
	}
	return(err);
}

void 	AGWindowDoSelectedRotate( AGWindow *ag, char myKeyCode, short timesFive)
{
	float latAngle;	
	float longAngle;
	float spinAngle;
	kSanMatrix  fMat;
	kSanMatrix  cMat;
	AGWindowGetAngles( ag, myKeyCode,&latAngle, &longAngle, &spinAngle);	
	if (timesFive)
	{
		latAngle *= 5;
		longAngle *= 5;
		spinAngle *= 5;
	}
	identitykSanMatrixD(&fMat);
	SpinkSanMatrixInDegreesD(&fMat, latAngle, longAngle, spinAngle);	
	getCramerskSanOperatorsD(&fMat, &cMat);
	
	rotateParticles( ag->sim, &cMat, true) ; // selected only
}

void AGDefaultView(viewInfo *dView)
{	
	dView->magnification =12;
	dView->perspective = 1000;
	dView->particleSizeFactor = 1.0;
	identitykSanMatrixD (&dView->viewMatrix);
	dView->lastOffsetUsed.h = 0; 
	dView->lastOffsetUsed.v = 0;
	dView->showBonds = false;
	dView->shadeMethod = kNoShade;
	dView->sizeMethod = kNoSize;
 	dView->neutralShadeDepth = 0.0;
	dView->shadeRange = 20.0;
}


char AGWindowParticleIsInWindow ( AGWindow *ag , long partArrIndex)
{
	char ret = false;
	if ( (partArrIndex >= 0 ) && (partArrIndex < ag->partBits.howManyItems))
	{
		ret = getNthBit(&ag->partBits, partArrIndex);
	}
	return(ret);
 }


void defaultGrafData (grafData * gPtr)
{
	gPtr->inWindow = true;
	gPtr->selected = true;
	gPtr->grapha = 1;
	gPtr->graphb = 1;
#ifndef __2DVectors__ 
	gPtr->graphc = 1;
#endif	
	/*
	gPtr->howManyForebonds = 0 ;
	gPtr->howManyBackbonds = 0 ;
	*/
	gPtr->circle.right = 1;
	gPtr->circle.left = 0;
	gPtr->circle.top = 1;
	gPtr->circle.bottom = 2;
	gPtr->colour = getColour(myRedColour);
}

short AGWindowAddParticleList ( AGWindow *ag,  OHList *newPartList)
{ // take a list and add all of its pieces which are not already in the window
	short err = noErr;
	long newCount;
 	long i;
	long nthItem;
  	// set the handle sizes first to avoid memory thrashing for a long list
 	 
	 newCount = OHListCount(newPartList);
	 if (newCount > 0)
	 {
	 	 ag->partListDirty = true;
		 ag->orderListDirty = true;
		 ag->grafDataDirty = true;
		 
		 for ( i = 0 ; i < newCount; ++i)
		 {
		      nthItem = getNthItem(newPartList, i );
		      BitListAccommodate(&ag->partBits, nthItem );
		 	  setNthBitTrue(&ag->partBits, nthItem );
		 }
	 }
	return (err);
}	
short AGWindowAddAllParticles( AGWindow *ag )
{ // take a list and add all of its pieces which are not already in the window
 	long newCount;
 	long i;
   	// set the handle sizes first to avoid memory thrashing for a long list
 	 
	 newCount = simGetNumParts(ag->sim);
	 if (newCount > 0)
	 {
	 	 ag->partListDirty = true;
		 ag->orderListDirty = true;
		 ag->grafDataDirty = true;
		 
		 for ( i = 0 ; i < newCount; ++i)
		 {
 		      BitListAccommodate(&ag->partBits, i );
		 	  setNthBitTrue(&ag->partBits, i );
		 }
	 }
	return (noErr);
}	


short AGWindowAddToPartList( AGWindow *ag, long partToAdd)
{
 	short err = noErr;
	
 	if (!(AGWindowParticleIsInWindow(ag, partToAdd)))
	{	 // not already in list, so add it
	    BitListAccommodate (&ag->partBits,partToAdd);
	    setNthBitTrue(&ag->partBits,partToAdd);
	    
		if (!ag->partListDirty) 
		{
		    err = OHListAdd (&ag->particleList, partToAdd);
		}
		else
		{
		     ag->orderListDirty = true;
		}
		if (!ag->orderListDirty) 
		{
		    err = OHListAdd(&ag->orderList, OHListCount ( &ag->particleList) - 1); 
		}
		
    	ag->grafDataDirty = true;
  	}
	return (err);
}
void AGWindowChangeParticleIndex( AGWindow * ag, long oldIndex, long newIndex)
{
	char oldIndexWasInWindow;
	oldIndexWasInWindow = getNthBit(&ag->partBits, oldIndex);
	 BitListAccommodate (&ag->partBits,(newIndex > oldIndex) ? newIndex : oldIndex);
	setNthBit( &ag->partBits, newIndex , oldIndexWasInWindow);
	setNthBitFalse( &ag->partBits, oldIndex);
	ag->partListDirty = true;
	ag->orderListDirty = true;
	ag->grafDataDirty = true;
}	
void AGWindowRemoveParticleList( AGWindow *ag, OHList * list)
{
	 OHListEveryItemAsSecondArg( list,  AGWindowRemoveParticle,  ag);
}	
short AGWindowRemoveParticle( AGWindow *ag, long partArrIndex)
{
    BitListAccommodate(&ag->partBits, partArrIndex);
 	if (getNthBit(&ag->partBits, partArrIndex)) 
 	{
 	      setNthBitFalse(&ag->partBits, partArrIndex);
 	      ag->partListDirty = true;
 	      ag->orderListDirty = true;
	      ag->grafDataDirty = true;
	}
	return (noErr);
}


void AGWindowGetAngles( AGWindow *ag,char myKeyCode,float * latAngle,float * longAngle,float * spinAngle)
{
	*spinAngle = 0.0;
	switch (myKeyCode)
	{		
		case '1':		
			*latAngle = fabs(ag->manip.latRotateAngle);
			*longAngle =	-1.0 * fabs(ag->manip.longRotateAngle);
			break;

		case '2':		
			*latAngle = 0.0;
			*longAngle =	-1.0 * fabs(ag->manip.latRotateAngle);
			break;

		case '3':		
			*latAngle = -1.0 *fabs( ag->manip.latRotateAngle);
			*longAngle =	-1.0 *fabs(ag->manip.longRotateAngle);
			break;

		case '4':
			*latAngle = fabs(ag->manip.latRotateAngle);
			*longAngle =	0.0;
			break;	

		case '6':		
			*latAngle =  -1.0 *fabs(ag->manip.latRotateAngle);
			*longAngle =	0.0;
			break;			
				
		case '7':
			*latAngle = fabs(ag->manip.latRotateAngle);
			*longAngle =	fabs(ag->manip.longRotateAngle);
			break;	
				
		case '8':		
			*latAngle = 0.0;
			*longAngle =	fabs(ag->manip.longRotateAngle);
			break;

		case '9':		
			*latAngle =  -1.0 *fabs(ag->manip.latRotateAngle);
			*longAngle =	fabs(ag->manip.longRotateAngle);
			break;	
					
		case '/':		// ccw rotation
			*latAngle =  0.0;
			*longAngle =	0.0;
			*spinAngle =	-1.0 *fabs(ag->manip.axialRotateAngle);
			break;	
					
		case '*':		// cw rotation
			*latAngle =  0.0;
			*longAngle =	0.0;
			*spinAngle =	fabs(ag->manip.axialRotateAngle);
			break;						
			
		default:
			*latAngle = 0.0;
			*longAngle = 0.0;
			*spinAngle = 0.0;
			break;
	}
}
