#include "kSanAtomGraphicsWindow.h"
#include "shlbProcInfo.h"

void SetAllMouseModeButtons(AGWindow *ag);

long  getColourMethod(AGWindow *ag)
{
	return (ag->windowView.colourMethod);
}
long  getSizeMethod(AGWindow *ag)
{
	return (ag->windowView.sizeMethod);
}
long  getShadeMethod(AGWindow *ag)
{
	return (ag->windowView.shadeMethod);
}
long  getGraphicsMethod(AGWindow *ag, long descriptorType)
{	
	long methodID = 0L;
	switch (descriptorType)
	{
		case pColourMethod:
			methodID = getColourMethod(ag);
			break;
		case pSizeMethod:
			methodID = getSizeMethod(ag);
			break;
		case pShadeMethod:
			methodID = getShadeMethod(ag);
			break;
		default:
			break;
	}
	return (methodID);
}
short setColourMethodMenuChoice(AGWindow *ag,  short menuIndex)
{
	short err = noErr;
	short realIndex = menuIndex ;
	
	kSanColourPlugIn * cPlug;
	if (realIndex == numItems(&colourPlugIns)) realIndex = 0;
	{
		cPlug = (kSanColourPlugIn * ) getNthItem (&colourPlugIns, realIndex);
		err = setColourPlugIn(ag, cPlug);
	}
	return (err);
}
short setColourMethod (AGWindow *ag,  long enumeratedID)
{
	short err = noErr;
	kSanColourPlugIn * cPlug;
	cPlug = (kSanColourPlugIn * ) getPlugRef (enumeratedID);
	err = setColourPlugIn(ag, cPlug);
	return (err);
}
short setColourPlugIn(AGWindow *ag, kSanColourPlugIn * cPlug)
{
	short err = noErr;
	if (ag->cPlug == nilPointer) // during initialize cant call set func
	{
		ag->colourMethodIndex = whichItem(&colourPlugIns, (long)(cPlug));
		ag->cPlug = cPlug ;
	}
	else if  (cPlug != ag->cPlug) 
	{
		if  (cPlug != nilPointer)
		{
			if (ag->cPlug != nilPointer) AGColourPlugDoEvent(ag , kSanCleanupMessage, 0L, 0L);
			ag->cPlug = cPlug ;
			ag->colourMethodIndex = whichItem(&colourPlugIns, (long)(cPlug));
		}
		else
		{
			err = true;
			ag->cPlug = (kSanColourPlugIn *) getNthItem(&colourPlugIns, 0);
			ag->colourMethodIndex = 0;
		}
		if (ag->cPlug != nilPointer) AGColourPlugDoEvent(ag , kSanInitMessage, 0L, 0L);
	}
	return (err);
}
short setSizeMethod (AGWindow *ag, long enumeratedID)
{
	ag->windowView.sizeMethod = enumeratedID;
	switch (ag->windowView.sizeMethod)
	{
 		case kSizeByWindow:
 			ag->setMySize = setMySizeByWindow;
 			ag->setMyActualSize = setMyActualSizeByWindow;
 			break;
		case kNoSize:
			ag->setMySize = setNoSize;
			ag->setMyActualSize = setNoActualSize;
			break;
		default:
		case kSizeByAtomType:
			ag->setMySize = setMySizeByAtomType;
			ag->setMyActualSize = setMyActualSizeByAtomType;
			break;
	}
	return (noErr);
}
short setShadeMethod (AGWindow *ag, long enumeratedID)
{
	ag->windowView.shadeMethod = enumeratedID; // boolean
	switch (ag->windowView.shadeMethod)
	{
		case kShadeByDepth:
			ag->setForeColour = setForeColourWithShade;
			break;			
		case kNoShade:
		default:
			ag->setForeColour= setForeColourNoShade;
			break;
	}			
	return (noErr);
}
short setGraphicsMethod (AGWindow *ag, long descriptorType, long enumeratedID)
{
	short err = 0;
	switch (descriptorType)
	{
		case pColourMethod:
			setColourMethod(ag, enumeratedID);
			break;
		case pSizeMethod:
			setSizeMethod(ag, enumeratedID);
			break;
		case pShadeMethod:
			setShadeMethod(ag, enumeratedID);
			break;
		default:
			err = true;
			break;
	}
	return err;
}

viewInfo *agGetViewPtr  (kozoObject *agObj)
{
	AGWindow *ag;
	ag = (AGWindow *) agObj;
	return (&ag->windowView);
}

short  agSetView  (AGWindow *ag, viewInfo *theView)
{
	setMagnification(ag, theView->magnification);
	setPerspective(ag, theView->perspective);
	setParticleSize(ag, theView->particleSize);
	setBondsVisible(ag, theView->showBonds);
	setShadeMethod(ag, theView->shadeMethod);
	setColourMethod(ag, theView->colourMethod);
	setSizeMethod(ag, theView->sizeMethod);
	setViewMatrix(ag, &theView->viewMatrix);
	setBGColour(ag, &theView->BGColour);
	AGWinSetCenter(ag, theView->cellCenter.h,  theView->cellCenter.v);
	setShadeEven(ag, theView->neutralShadeDepth);
	setShadeRange(ag, theView->shadeRange);
	return (noErr);
}

short  AGWinSetData (KozoDispatchStack *ds, setMethodArgs *sma)
{
	short err = noErr;
	float val;
	OHCoerceToFloat (data, &val);
	err = WinSetData(&ag->win, property, data);
	if (err == kSanErrPropertyNotFound)
	{
		switch(property)
		{
			case pMagnification:
				setMagnification(ag, val);
				break;
			case pPerspective:
				setPerspective(ag, val);
				break;
			case pShadeRange:
				setShadeRange(ag, val);
				break;
			case pShadeEven:
				setShadeEven(ag, val);
				break;
			case pColourMethod:
			case pSizeMethod:
			case pShadeMethod:
				{
					long methodID;
					methodID = *(long *) *data->dataHandle;
					if (!err)  setGraphicsMethod(ag, property, methodID);
				}
				break;

			case pViewAngle:					
				{
					floatVector viewVec;
					doubleVector viewVecD;
					err = getVectorDesc((double *)&viewVecD, data);
					viewVec.a = viewVecD.a;
					viewVec.b = viewVecD.b;
					viewVec.c = viewVecD.c;
					if (!err)
					{
						SetMatrixInDegrees(viewVec.a, viewVec.b, viewVec.c,&(ag->windowView.viewMatrix));
					}
				}
				break;
			default:
			KDSReturnContinue();
		}
	}
	KDSDoneDispatch();
	KDSReturnError(ds, err);
}

void AGWinSetSize(AGWindow *ag, short newHeight, short newWidth)
{
	setWindowSize(&ag->win, newHeight, newWidth);
	calcScrollValues(ag);
}

void setDefaultAGTitle(AGWindow *ag, Handle ownerName)
{
	Str255 pascalName;
	long numWins;
	char * newname ;
	long size;
	numWins = ag->agID ;
	newname = (char *)pascalName;
	size = GetHandleSize(ownerName);
	if (size > 64) size = 64;
	memcpy(&newname[1], *ownerName, size);
	sprintf (&newname[size + 1 ], "  graphics %ld", numWins);
	newname[0] = strlen(&newname[1]);
	SetWTitle(AGPaneWindowPtr(ag), pascalName);
}		

void setBondsVisible	(AGWindow *ag, short viz)
{
	WindowPtr fWin;
	ag->windowView.showBonds = viz;
	fWin = FrontWindow();
	if (fWin == AGPaneWindowPtr(ag)) setShowBondsMenuItem (ag, viz);
}
void setShowBondsMenuItem (AGWindow *ag,short nowShowing)
{
	short theID;
	if (nowShowing)
	{
		theID = kHideBondsStringID + appData->intlRezOffset;
	}
	else
	{
		 theID = kShowBondsStringID + appData->intlRezOffset;
	}
	 setMenuToSTR(mAGWindow, iBonds, theID, "\p bond visability");
}

void calcScrollValues(AGWindow *ag)
{
	short nowVal, maxVal, wHeight, wWidth;
	Point theCenter;
	ControlHandle cHandle;
	
	wHeight = ag->win.worldRect.bottom - ag->win.worldRect.top;
	wWidth = ag->win.worldRect.right - ag->win.worldRect.left;
	theCenter = ag->windowView.cellCenter;
	
	cHandle = getVScrollHandle(AGPaneWindowPtr(ag));
	maxVal = GetControlMaximum(cHandle);
	nowVal= (maxVal -((theCenter.v  + (kMaxWindowHeight * 0.5) + (wHeight/2))  *  (maxVal / kMaxWindowHeight )) )  ;
	SetControlValue(cHandle, nowVal);
	
	cHandle = getHScrollHandle(AGPaneWindowPtr(ag));
	maxVal = GetControlMaximum(cHandle);
	nowVal= (maxVal -((theCenter.h  + (kMaxWindowHeight * 0.5) + (wHeight/2))  *  (maxVal / kMaxWindowHeight )) )  ;
	SetControlValue(cHandle, nowVal);
}

void AGWinSetMouseMode(AGWindow *ag, short whichMode)
{
	short theID;
	
	ag->mode = whichMode;
	switch (whichMode) 
	{
		case kSelectAndHighlight:
			theID =  kMouseManipulationStringID + appData->intlRezOffset;
			break;
		case kMouseManipulation:
			theID = kSelectAndHighlightStringID + appData->intlRezOffset;
			break;
	}
	SetAllMouseModeButtons(ag);
	 setMenuToSTR(mAGWindow, iMouseManipulation, theID , "\p mouse mode");
}
void SetAllMouseModeButtons(AGWindow *ag)
{
	short i = 0;
	short count;
	kSanButton * nthButt;
	
	ag->win.nowToggled = (!ag->win.nowToggled);
	count = numItems(&ag->win.buttons);
	while (i < count)  // check controls
	{
		nthButt = (kSanButton *)getNthItem(&ag->win.buttons, i);
		if (nthButt->type == kRollCellButton)
		{
			setButtonVisibility(nthButt, (ag->mode == kSelectAndHighlight));
			InvalWindowRect (&ag->win,  &nthButt->rect);
		}
		else if  (nthButt->type == kSelectFingerButton)
		{
			setButtonVisibility(nthButt, (ag->mode == kMouseManipulation));
			InvalWindowRect (&ag->win,  &nthButt->rect);
		}
		++i;
	}

}

void AGWinSetCenter(AGWindow *ag, short hVal, short vVal)
{
	 ag->windowView.cellCenter.h = hVal;
	 ag->windowView.cellCenter.v = vVal;
}
void   setViewMatrix(AGWindow *ag, newMatrixF *trix)
{
      ag->windowView.viewMatrix    = *trix;
}
void   setBGColour(AGWindow *ag, RGBColor * colour)
{
      ag->windowView.BGColour    = *colour;
}

void   setMagnification(AGWindow *ag, float val)
{
      ag->windowView.magnification    = val;
}
void   setParticleSize(AGWindow *ag, float val)
{
      ag->windowView.particleSize    = val;
}
void   setPerspective(AGWindow *ag, float val)
{
       ag->windowView.perspective    = val;
}
void   setShadeRange(AGWindow *ag, float val)
{
       ag->windowView.shadeRange    = val;
}
void   setShadeEven(AGWindow *ag, float val)
{
       ag->windowView.neutralShadeDepth    = val;
}

short AGWinGetData (KozoDispatchStack *ds, getMethodArgs *gma)
{
	short err;
	err = WinGetData( &ag->win, property, retDesc);
	if (err)
	switch (property)
	{
		case pColourMethod:
			err = AECreateDesc(typeLongInteger, &ag->windowView.colourMethod,sizeof(long), retDesc);
			break;
		case pSizeMethod:
			err = AECreateDesc(typeLongInteger, &ag->windowView.sizeMethod,sizeof(long), retDesc);
			break;
		case pShadeMethod:
			err = AECreateDesc(typeLongInteger, &ag->windowView.shadeMethod,sizeof(long), retDesc);
			break;
		default:
		        KDSReturnContinue();
	}
	KDSDoneDispatch();
	KDSReturnError(ds, err);
}

float   getMagnification(AGWindow *ag)
{
        float retVal;
        retVal =  ag->windowView.magnification    ;
        return (retVal);
}
float   getPerspective(AGWindow *ag)
{
        float retVal;
      retVal =  ag->windowView.perspective    ;
     return (retVal);
}
float   getShadeRange(AGWindow *ag)
{
        float retVal;
      retVal = ag->windowView.shadeRange    ;
     return (retVal);
}
float   getShadeEven(AGWindow *ag)
{
        float retVal;
      retVal =  ag->windowView.neutralShadeDepth    ;
     return (retVal);
}
