#include <fp.h>
#include "kSanCell.h"

 


void calcDimensions (kSanCell *thisCell)
{  // set the cellDim values used to be set cell dim
	kSanVector  crossProd;

// do b
#ifndef __2DVectors__
	crosskSanVectorsD (&thisCell->primitiveVectors.C, &thisCell->primitiveVectors.A, &crossProd);
 	normalizekSanVectorD(&crossProd, &crossProd);
	thisCell->cellDim.b = dotkSanVectorsD(&crossProd, &thisCell->primitiveVectors.B);
	thisCell->invCellDim.b = 1 / thisCell->cellDim.b;
	
// do a
 	crosskSanVectorsD (&thisCell->primitiveVectors.B, &thisCell->primitiveVectors.C, &crossProd);
 	normalizekSanVectorD(&crossProd, &crossProd);
	thisCell->cellDim.a = dotkSanVectorsD(&crossProd, &thisCell->primitiveVectors.A);
	thisCell->invCellDim.a = 1 / thisCell->cellDim.a;

// do c	
 	crosskSanVectorsD (&thisCell->primitiveVectors.A, &thisCell->primitiveVectors.B, &crossProd);
	normalizekSanVectorD(&crossProd, &crossProd);
	thisCell->cellDim.c = dotkSanVectorsD(&crossProd, &thisCell->primitiveVectors.C);	
	thisCell->invCellDim.c = 1 / thisCell->cellDim.c;

#else
 
 //do b
          getOrthogonal2dVectorsD(&thisCell->primitiveVectors.A, &crossProd);
           // this essentially rotates the vector 90 degrees
 	normalizekSanVectorD(&crossProd, &crossProd);
	thisCell->cellDim.b = dotkSanVectorsD(&crossProd, &thisCell->primitiveVectors.B);
	thisCell->invCellDim.b = 1 / thisCell->cellDim.b;
	
 //do a
        getOrthogonal2dVectorsD(&thisCell->primitiveVectors.B, &crossProd);
           // this essentially rotates the vector 90 degrees
        multkSanVectorD( &crossProd, -1.0,  &crossProd);
        // after rotating 90 degrees, this flips it, so the result is a -90 degree rotation
 	normalizekSanVectorD(&crossProd, &crossProd);
	thisCell->cellDim.a = dotkSanVectorsD(&crossProd, &thisCell->primitiveVectors.A);
	thisCell->invCellDim.a = 1 / thisCell->cellDim.a;
	
 #endif

	calcAreas(thisCell);
}

double getMinimumDimension (kSanCell *thisCell)
{  // given the primitive vectors, calc the rest
	double minim;
	minim = thisCell->cellDim.a;
	if (thisCell->cellDim.b < minim) minim = thisCell->cellDim.b;
#ifndef __2DVectors__
	if (thisCell->cellDim.c < minim) minim = thisCell->cellDim.c;
#endif
	return (minim);
}

void getAreas (kSanCell *thisCell, kSanVector * theAreas)
{  
	*theAreas = thisCell->areas;
}

void calcAreas (kSanCell *thisCell)
{  
#ifndef __2DVectors__
	thisCell->areas.a = thisCell->cellDim.b * thisCell->cellDim.c * sin(thisCell->alpha) * sin(thisCell->beta) * sin(thisCell->gamma);
	thisCell->areas.b = thisCell->cellDim.c * thisCell->cellDim.a * sin(thisCell->beta) * sin(thisCell->gamma);
	thisCell->areas.c = thisCell->cellDim.a * thisCell->cellDim.b * sin(thisCell->gamma); // simple!
#else
	thisCell->areas.a =  thisCell->cellDim.b * thisCell->cellDim.a * sin(thisCell->alpha) ;
	thisCell->areas.b = 0;
//	thisCell->areas.c = 0; // simple!
#endif
}


void calcCellPropertiesFromEdgesAndAngles (kSanCell *thisCell)
{
	double a, b, c, alpha, beta, gamma, volume;
	double cosAlpha, cosBeta, cosGamma;
	double sinAlpha, sinBeta, sinGamma;
	double recipVolume;
#ifdef __2DVectors__
#pragma unused ( recipVolume)
#pragma unused (volume )
#pragma unused (alpha )
#pragma unused (sinAlpha )
#pragma unused ( beta)
#pragma unused (cosAlpha )
#pragma unused (cosBeta )
#pragma unused (sinBeta )
#pragma unused (c )
#endif	
	a = thisCell->cellEdges.a;
	b = thisCell->cellEdges.b;
	gamma = thisCell->gamma;
	cosGamma = cos (gamma);
	sinGamma = sin (gamma);

 #ifndef __2DVectors__
	alpha = thisCell->alpha;
	sinAlpha = sin (alpha);
	c = thisCell->cellEdges.c;
	cosAlpha = cos (alpha);
	beta = thisCell->beta;
	cosBeta = cos (beta);
	sinBeta = sin (beta);
 
	volume = a * b * c *  sqrt( 1 + 2 * cosAlpha * cosBeta * cosGamma - 
	        (cosAlpha * cosAlpha + cosBeta * cosBeta + cosGamma * cosGamma) );
	recipVolume = 1 / volume;
	
	thisCell->reciprocalCellEdges.a = b * c * sinAlpha * recipVolume;
	thisCell->reciprocalCellEdges.b = a * c * sinBeta * recipVolume;
	thisCell->reciprocalCellEdges.c = b * a * sinGamma * recipVolume;
	// done recip edge length 
	
	// assume that A.a = a , and that A.b = 0 and A.c = 0
	// further assume that B.c = 0   
	// i.e that the primitive lattice vector A lies in the x direction,
	// and that the primitive lattice vector B lies in the x-y plane,
	thisCell->primitiveVectors.A.a = a;
	thisCell->primitiveVectors.A.b = 0;
	thisCell->primitiveVectors.A.c = 0;
	thisCell->primitiveVectors.B.c = 0;
	thisCell->primitiveVectors.B.b = b * sinGamma;
	thisCell->primitiveVectors.B.a = b * cosGamma;
    // knowing A and B, we can calculate Cstar with Cstar = A cross B / V
    {
	    kSanVector  AcrossB;
	    kSanVector  cramersK;
	    kSanMatrix  trix;
		crosskSanVectorsD (&thisCell->primitiveVectors.A, &thisCell->primitiveVectors.B, &AcrossB);
		multkSanVectorD(&AcrossB, recipVolume, &thisCell->reciprocalVectors.C);	
		// and we also know that volume = A cross B dot C
		// we use that for the third equation for a cramer's solution, the first two being
		// cosBeta = A dot C / ac
		// cosAlpha = B dot C / bc
	    trix.A = thisCell->primitiveVectors.A;
	    trix.B = thisCell->primitiveVectors.B;
	    trix.C = AcrossB;
	    cramersK.a = cosBeta * a * c;
	    cramersK.b = cosAlpha * b * c;
	    cramersK.c = volume;
		solvekSanCramersD(&cramersK, &trix, &thisCell->primitiveVectors.C);  // yes!!!!	
	}
	{  // do recip A
	    kSanVector  BcrossC;
		crosskSanVectorsD (&thisCell->primitiveVectors.B, &thisCell->primitiveVectors.C, &BcrossC);
		multkSanVectorD(&BcrossC, recipVolume, &thisCell->reciprocalVectors.A);		
         }
	{  // do recip B
	    kSanVector  CcrossA;
		crosskSanVectorsD (&thisCell->primitiveVectors.C, &thisCell->primitiveVectors.A, &CcrossA);
		multkSanVectorD(&CcrossA, recipVolume, &thisCell->reciprocalVectors.B);		
         } // done recip vectors
	
	
	thisCell->volume = volume;
	thisCell->reciprocalVolume = recipVolume;


#else
// 2d version
	{
		double area =   a * b * sinGamma;
		double recipArea = 1 / area;
		
		thisCell->reciprocalCellEdges.a = b *   recipArea;
		thisCell->reciprocalCellEdges.b = a  *   recipArea;
	 	// done recip edge length 
 		thisCell->primitiveVectors.A.a = a;
		thisCell->primitiveVectors.A.b = 0;
 		thisCell->primitiveVectors.B.b = b * sinGamma;
 		thisCell->primitiveVectors.B.a = b * cosGamma;
  		{  // do recip A
		    kSanVector  orthToB;
			getOrthogonalkSanVectorsD (&thisCell->primitiveVectors.B, &orthToB);
			normalizekSanVectorD (  &orthToB, &orthToB);
			multkSanVectorD(&orthToB, recipArea, &thisCell->reciprocalVectors.A);		
	         }
		{  // do recip B
		    kSanVector  orthToA;
			getOrthogonalkSanVectorsD (&thisCell->primitiveVectors.A, &orthToA);
			normalizekSanVectorD (  &orthToA,  &orthToA);
			multkSanVectorD(&orthToA, recipArea, &thisCell->reciprocalVectors.B);		
	         } // done recip vectors
		
		
		thisCell->volume = area;
		thisCell->reciprocalVolume = recipArea;
	}
#endif

	calcDimensions(thisCell);
}

 
void calcFromPrimitives (kSanCell *thisCell)
{  // given the primitive vectors, calc the rest
	double cosAlpha, cosBeta, cosGamma;
	
	thisCell->cellEdges.a = lengthOfkSanVectorD(&thisCell->primitiveVectors.A);
	thisCell->cellEdges.b = lengthOfkSanVectorD(&thisCell->primitiveVectors.B);

#ifndef __2DVectors__
	thisCell->cellEdges.c = lengthOfkSanVectorD(&thisCell->primitiveVectors.C);
 	thisCell->alpha = angleOfTwokSanVectorsD(&thisCell->primitiveVectors.B, &thisCell->primitiveVectors.C);
	thisCell->beta = angleOfTwokSanVectorsD(&thisCell->primitiveVectors.C, &thisCell->primitiveVectors.A);
#else
	thisCell->alpha = kPi * 0.5;
	thisCell->beta = kPi * 0.5;
#endif

	thisCell->gamma = angleOfTwokSanVectorsD(&thisCell->primitiveVectors.A, &thisCell->primitiveVectors.B);
	// done angles
 	cosAlpha = cos(thisCell->alpha);
	cosBeta = cos(thisCell->beta);
 	cosGamma = cos(thisCell->gamma);

#ifndef __2DVectors__
	thisCell->volume = thisCell->cellEdges.a * thisCell->cellEdges.b * thisCell->cellEdges.c *
			  sqrt( 1 + 2 * cosAlpha * cosBeta * cosGamma - 
	        (cosAlpha * cosAlpha + cosBeta * cosBeta + cosGamma * cosGamma) );
 #else
	thisCell->volume = thisCell->cellEdges.a * thisCell->cellEdges.b * sin(thisCell->gamma);
#endif
			  
	thisCell->reciprocalVolume = 1 / thisCell->volume;
	// done volumes

#ifndef __2DVectors__
	{  // do recip A
	    kSanVector  crossProd;
		crosskSanVectorsD (&thisCell->primitiveVectors.B, &thisCell->primitiveVectors.C, &crossProd);
		multkSanVectorD(&crossProd, thisCell->reciprocalVolume, &thisCell->reciprocalVectors.A);		
           // do recip B
 		crosskSanVectorsD (&thisCell->primitiveVectors.C, &thisCell->primitiveVectors.A, &crossProd);
		multkSanVectorD(&crossProd, thisCell->reciprocalVolume, &thisCell->reciprocalVectors.B);		
           // do recip C
 		crosskSanVectorsD (&thisCell->primitiveVectors.A, &thisCell->primitiveVectors.B, &crossProd);
		multkSanVectorD(&crossProd, thisCell->reciprocalVolume, &thisCell->reciprocalVectors.C);		
         } // done recip vectors
#else
	{  // do recip A
	    kSanVector  orth;
		getOrthogonalkSanVectorsD (&thisCell->primitiveVectors.B,  &orth);
		multkSanVectorD(&orth, thisCell->reciprocalVolume, &thisCell->reciprocalVectors.A);		
           // do recip B
		getOrthogonalkSanVectorsD (&thisCell->primitiveVectors.A,  &orth);
		multkSanVectorD(&orth, thisCell->reciprocalVolume, &thisCell->reciprocalVectors.B);		
           } // done recip vectors
#endif	


	thisCell->reciprocalCellEdges.a = lengthOfkSanVectorD(&thisCell->reciprocalVectors.A);
	thisCell->reciprocalCellEdges.b = lengthOfkSanVectorD(&thisCell->reciprocalVectors.B);
#ifndef __2DVectors__
	thisCell->reciprocalCellEdges.c = lengthOfkSanVectorD(&thisCell->reciprocalVectors.C);
#endif
	// done recip edge lengths
	calcDimensions(thisCell);
}
short isOrthorhombic(kSanCell *thisCell)
{
	short isOrtho = false;
	if ((fabs(thisCell->alpha - kPi * 0.5) < 0.001) &&
			(fabs(thisCell->beta - kPi * 0.5) < 0.001) &&
			(fabs(thisCell->gamma - kPi * 0.5) < 0.001) )
	{
	   isOrtho = true;
	}
	return (isOrtho);
}

void mVector (kSanCell *thisCell, kSanVector * ratio)
{  
	thisCell->primitiveVectors.A.a *= ratio->a;
	thisCell->primitiveVectors.B.a *= ratio->a;
	thisCell->primitiveVectors.A.b*= ratio->b; 
	thisCell->primitiveVectors.B.b*= ratio->b;
#ifndef __2DVectors__
	thisCell->primitiveVectors.C.a *= ratio->a;
	thisCell->primitiveVectors.C.b*= ratio->b;
	thisCell->primitiveVectors.A.c*= ratio->c; 
	thisCell->primitiveVectors.B.c*= ratio->c; 
	thisCell->primitiveVectors.C.c*= ratio->c;
#endif
}
void XYZtoUVW (kSanCell *thisCell,kSanVector * tricPos, kSanVector * uvwPos)
{  // should generally return u v and w between -0.5 and 0.5
	double u, v, w;
#ifdef __2DVectors__
#pragma unused ( w)
 #endif	
#ifndef __2DVectors__
	w = tricPos->c / thisCell->primitiveVectors.C.c ;
	v = ( tricPos->b - (w * thisCell->primitiveVectors.C.b) ) / (thisCell->primitiveVectors.B.b);
	u = ( tricPos->a - (w * thisCell->primitiveVectors.C.a + v * thisCell->primitiveVectors.B.a) ) / thisCell->primitiveVectors.A.a ;
	    uvwPos->c = w;
#else
	v = ( tricPos->b  ) / (thisCell->primitiveVectors.B.b);
	u = ( tricPos->a - ( v * thisCell->primitiveVectors.B.a) ) / thisCell->primitiveVectors.A.a ;
#endif
	    uvwPos->b = v;
	    uvwPos->a = u;
}
void UVWtoXYZ (kSanCell *thisCell,kSanVector * tricPos, kSanVector * uvwPos)
{
 	double u, v, w;
#ifdef __2DVectors__
#pragma unused ( w)
 #endif	
#ifndef __2DVectors__
   u = uvwPos->a * thisCell->primitiveVectors.A.a + 
    			uvwPos->b * thisCell->primitiveVectors.B.a + 
    			uvwPos->c * thisCell->primitiveVectors.C.a ; 
    v = uvwPos->b * thisCell->primitiveVectors.B.b + 
    			uvwPos->c * thisCell->primitiveVectors.C.b ; 
    w = uvwPos->c * thisCell->primitiveVectors.C.c ; 
     tricPos->c = w;
#else
    u = uvwPos->a * thisCell->primitiveVectors.A.a + 
    			uvwPos->b * thisCell->primitiveVectors.B.a  ; 
    v = uvwPos->b * thisCell->primitiveVectors.B.b  ; 
 #endif
    tricPos->b = v;
    tricPos->a = u;
}


void recoverRealPosition(kSanCell *thisCell,kSanVector * tricPos, kSanVector * adjPos)
{
	kSanVector  uvwPos;
	
	uvwPos.a = adjPos->a * thisCell->invCellDim.a;
	uvwPos.b = adjPos->b * thisCell->invCellDim.b;
#ifndef __2DVectors__
	uvwPos.c = adjPos->c * thisCell->invCellDim.c;	
#endif
	UVWtoXYZ (thisCell, tricPos, &uvwPos);
}
void getAdjustedPosition(kSanCell *thisCell,kSanVector * tricPos, kSanVector * adjPos)
{
	kSanVector  uvwPos;

	XYZtoUVW (thisCell, tricPos, &uvwPos);
	adjPos->a = uvwPos.a * thisCell->cellDim.a;
	adjPos->b = uvwPos.b * thisCell->cellDim.b;
#ifndef __2DVectors__
	adjPos->c = uvwPos.c * thisCell->cellDim.c;	
#endif
}

short  getAdjustedPositionWithWrap(kSanCell *thisCell,kSanVector * tricPos, kSanVector * adjPos)
{
	short nanFlag;
	kSanVector  uvwPos;

	XYZtoUVW (thisCell, tricPos, &uvwPos);
	
	if ((uvwPos.a <= 0.5)  && (uvwPos.a >= -0.5)) {}
	else if (uvwPos.a > 0.5) // do the wrap
	{
		uvwPos.a -= 1.0;
		subkSanVectorsD(tricPos, &thisCell->primitiveVectors.A, tricPos);
	}
	else if (uvwPos.a < -0.5)
	{
		uvwPos.a += 1.0;
		addkSanVectorsD(tricPos, &thisCell->primitiveVectors.A, tricPos);
	}
	else nanFlag = true;
	
	if ((uvwPos.b <= 0.5)  && (uvwPos.b >= -0.5)) {}
	else	if (uvwPos.b > 0.5) // do the y wrap
	{
		uvwPos.b -= 1.0;
		subkSanVectorsD(tricPos, &thisCell->primitiveVectors.B, tricPos);
	}
	else if (uvwPos.b < -0.5)
	{
		uvwPos.b += 1.0;
		addkSanVectorsD(tricPos, &thisCell->primitiveVectors.B, tricPos);
	}
	else nanFlag = true;
	
#ifndef __2DVectors__
	if ((uvwPos.c <= 0.5)  && (uvwPos.c >= -0.5)) {}
	else if (uvwPos.c > 0.5) // do the z wrap
	{
		uvwPos.c -= 1.0;
		subkSanVectorsD(tricPos, &thisCell->primitiveVectors.C, tricPos);
	}
	else if (uvwPos.c < -0.5)
	{
		uvwPos.c += 1.0;
		addkSanVectorsD(tricPos, &thisCell->primitiveVectors.C, tricPos);
	}
	else nanFlag = true;
#endif
	
	adjPos->a = uvwPos.a * thisCell->cellDim.a;
	adjPos->b = uvwPos.b * thisCell->cellDim.b;
#ifndef __2DVectors__
	adjPos->c = uvwPos.c * thisCell->cellDim.c;	
#endif
	
	return (nanFlag);
}
short vectorIsInsideCell(kSanCell *thisCell,kSanVector * vec) // returns whether point is inside or not
{
	// strategy:  use dist from plane as a criterion
	short isInside;
    double distFromCenterPlaneA;
    double distFromCenterPlaneB;
    double distFromCenterPlaneC;
    double XCut;
    double YCut;
    double ZCut;
#ifdef __2DVectors__
#pragma unused ( ZCut)
#pragma unused ( distFromCenterPlaneC)
 #endif	
    XCut = 0.5 / thisCell->reciprocalCellEdges.a;
    YCut = 0.5 / thisCell->reciprocalCellEdges.b;
#ifndef __2DVectors__
    ZCut = 0.5 / thisCell->reciprocalCellEdges.c;
#endif

    distFromCenterPlaneA = 
		distFromkSanPlaneThroughOriginD(vec, &thisCell->reciprocalVectors.A);
    if  ((distFromCenterPlaneA >= XCut) || (distFromCenterPlaneA <= (- XCut)))
	{
		isInside = false;
	}
	else
	{
	    distFromCenterPlaneB = 
			distFromkSanPlaneThroughOriginD(vec, &thisCell->reciprocalVectors.B);
		if ((distFromCenterPlaneB >= YCut) || (distFromCenterPlaneB <= (- YCut)))
		{
			isInside = false;
		}
		else
		{
		
#ifndef __2DVectors__
		    distFromCenterPlaneC = 
				distFromkSanPlaneThroughOriginD(vec, &thisCell->reciprocalVectors.C);
		    if ((distFromCenterPlaneC >= ZCut) || (distFromCenterPlaneC <= (- ZCut)))
			{
				isInside = false;
			}
			else
#endif

			{
				isInside = true;
			}
		}
	}
	return (isInside);	
}




