
#include "kSanCellResizePlugIn.h"
#include "kSanSimulationPrivate.h"

 short defaultCellResize (   kSanCellResizePlugIn *api,  kSanSimulation *sim);
short  CellResizeModuleGetData (KozoDispatchStack *ds, getMethodArgs *gma);

short initClassCellResizePlugIn (void)
{
	short err = noErr;
	kozoClass *newClass;
	kozoObject *co;

	newClass = addClass(&co,ClassCellResizePlugIn, ClasskSanPlugIn, sizeof (kSanCellResizePlugIn), "no resize");
	if (newClass)
	{
		err =	setFuncs(newClass, 0L,  CellResizePlugInConstructor,
		nilPointer, CellResizeModuleGetData, nilPointer, nilPointer,
		 nilHandleReportFunc, nilCountFunc, nilResolveFunc, nilCreateElementFunc);
	}
	else
	{
		err = true;
	}
	return (err);
}
short  CellResizeModuleGetData (KozoDispatchStack *ds, getMethodArgs *gma)
{
	short err = noErr;
	DescType theKind = ClassCellResizePlugIn;
	kSanCellResizePlugIn *api = (kSanCellResizePlugIn *)ds->thisPD;

  	switch (gma->property)
	{
		case pKind:
			err = AECreateDesc(typeType, &theKind, sizeof(DescType), gma->result);
			break;
 		default:
			KDSReturnContinue(ds);
	}
	KDSDoneDispatch(ds);
	KDSReturnError(ds, err);
}

short  CellResizePlugInConstructor (KozoDispatchStack *ds, constructorMethodArgs *cma)
{
	short err = noErr;
	kSanCellResizePlugIn *api = (kSanCellResizePlugIn *)ds->thisPD;
	kSanPlugIn *plug;
	plug = KDSGetPrivateData(ds, ClasskSanPlugIn  );
  	err = dispatchLoop(ds, cma);
	plug->plugPrivateData = api;
	plug->piType = ClassCellResizePlugIn;
	if (!api->doCellResize ) api->doCellResize = defaultCellResize;
 
	KDSReturnError(ds,err);
}
short defaultCellResize ( kSanCellResizePlugIn *api,  kSanSimulation *sim)
{
#pragma unused(sim)
#pragma unused(api)
	
	return (noErr);
}
short simCalcCellMechPressure ( kSanSimulation *sim, kSanVector  *stressVec)
{ // here we only calculate the strain terms, no shear terms
	short err = noErr;
	kSanVector  axisFracs;
		
	setkSanVectorD(&axisFracs, 1.0, 0.0, 0.0);
	simScaleAndCalcStress(sim, &axisFracs, &stressVec->a);
		
	setkSanVectorD(&axisFracs, 0.0, 1.0, 0.0);
	simScaleAndCalcStress(sim, &axisFracs, &stressVec->b);
		
#ifndef __2DVectors__
	setkSanVectorD(&axisFracs, 0.0, 0.0, 1.0);
	simScaleAndCalcStress(sim, &axisFracs, &stressVec->c);
#endif	
	return (err);
}
short simScaleAndCalcStress(kSanSimulation *sim, kSanVector  *dirVec, double *stress)
{  // xyzFractions is a unit vector pointing in the direction of the stress to calculate
  // return value in stress
#pragma unused(dirVec)
	short err = noErr;
	long count;
	long partArrIndex;
	particle *basePart;
	double *baseEs;
	double *longEs;
	double tenKOverN ;
	double totalForce;
	
	kSanVector  xyzFractions;
	kSanVector  oneOneOne;
	kSanVector  longScaleFactors;
	kSanVector  shortScaleFactors;
	kSanVector  scaleVec;	
	
	count = simGetNumParts(sim);
	tenKOverN = 10000.0 / ((double) count);
	normalizekSanVectorD (dirVec, &xyzFractions);
	setkSanVectorD (&oneOneOne, 1.0, 1.0, 1.0);
	setkSanVectorD (&longScaleFactors, 1.00005, 1.00005, 1.00005);
	setkSanVectorD (&shortScaleFactors, 0.99995, 0.99995, 0.99995);
	baseEs = (double *) NewPtr (sizeof(double) * count);
	longEs = (double *) NewPtr (sizeof(double) * count);
	multkSanVectorD (&xyzFractions, 0.00005 , &longScaleFactors);
	addkSanVectorsD ( &longScaleFactors, &oneOneOne, &longScaleFactors);
	multkSanVectorD (&xyzFractions, -0.00005 , &shortScaleFactors);
	addkSanVectorsD ( &shortScaleFactors, &oneOneOne, &shortScaleFactors);
	// lock the list containing data for each site
	// get the forces at no strain
	calculateEnergies (sim);
	
	basePart = simLockParts(sim);
	for (partArrIndex = 0; partArrIndex < count; ++partArrIndex)
	{
		baseEs[partArrIndex] =  basePart->basePotential[partArrIndex];
	}
	simReleaseParts(sim);

	scaleVec = longScaleFactors;
	// scale long
	err = scaleAxis(sim, pCellXYZLength,  &scaleVec);	
	calculateEnergies (sim);
	basePart = simLockParts(sim);
	for (partArrIndex = 0; partArrIndex < count; ++partArrIndex)
	{
		longEs[partArrIndex] =  basePart->basePotential[partArrIndex];
	}
	simReleaseParts(sim);
	divkSanVectorsD(&shortScaleFactors,  &longScaleFactors, &scaleVec);
	//  scaleVec.a = shortScaleFactors.a / longScaleFactors.a , etc.
 	err = scaleAxis(sim, pCellXYZLength, &scaleVec);
	calculateEnergies (sim);
	basePart = simLockParts(sim);
	totalForce = 0;
	for (partArrIndex = 0; partArrIndex < count; ++partArrIndex)
	{ // we have three points :  base, long and the current potential is short
		totalForce += longEs[partArrIndex] - basePart->basePotential[partArrIndex];
	}
	// by kSan convention, positive pressures are pushing the cell to get smaller
	// if we are in tension, these values shouls come out positive, as longEs should be higher than short Es
	
	simReleaseParts(sim);
	
	recipkSanVectorD(&shortScaleFactors, &scaleVec );
/*
	scaleVec.a = 1.0 / shortScaleFactors.a ;
	scaleVec.b = 1.0 / shortScaleFactors.b ;
 	scaleVec.c = 1.0 / shortScaleFactors.c ;
*/

 	err = scaleAxis(sim, pCellXYZLength,  &scaleVec);
	totalForce *= tenKOverN;
	*stress = totalForce;
	//done x
	
	// resore old potentials
	basePart = simLockParts(sim);
	for (partArrIndex = 0; partArrIndex < count; ++partArrIndex)
	{
		basePart->basePotential[partArrIndex] = baseEs[partArrIndex] ;
	}
	simReleaseParts(sim);

	DisposePtr ((Ptr)baseEs);
	DisposePtr ((Ptr)longEs);

	return (err);
}
short simCalcLinearStressNumerically(kSanSimulation *sim, kSanVector  *linStress)
{
	short err = noErr;
	kSanVector  cellMechPressure;
	kSanVector  kineticPressure;
	simCalcCellMechPressure(sim, &cellMechPressure);
	simCalcKineticPressure(sim, &kineticPressure);
	addkSanVectorsD(&cellMechPressure, &kineticPressure, linStress);
	return (err);
}
short simCalcLinearStressFromVirial ( kSanSimulation *sim, kSanVector  *linStress)
{
	short err = noErr;
	kSanVector  virialPressure;
	kSanVector  kineticPressure;
	simCalcVirialPressure(sim, &virialPressure);
	simCalcKineticPressure(sim, &kineticPressure);
	addkSanVectorsD(&virialPressure, &kineticPressure, linStress);
	return (err);
}
short simCalcVirialPressure (kSanSimulation *sim, kSanVector  *vPress)
{
	short err = noErr;
	kSanMatrix  totalVir;
	kSanMatrix vir;
	long i;
	zerokSanMatrixD(&totalVir);
	if (sim->potentialPlugIn->onePartVirialCalc != nilPointer)
	{
		for (i = 0; i < simGetNumParts(sim); ++i)
		{
	 	 	 err = sim->potentialPlugIn->onePartVirialCalc ( sim->potentialPlugIn, sim, i, &vir);
// virial at one particle should be returned in units of mass * A * A / (s * s) (i.e. an energy)
			addkSanMatricesD(&totalVir, &vir, &totalVir);
		}
		vPress->a = totalVir.A.a * sim->simCell.reciprocalVolume;
		vPress->b = totalVir.B.b * sim->simCell.reciprocalVolume;
#ifndef __2DVectors__
		vPress->c = totalVir.C.c * sim->simCell.reciprocalVolume;
#endif
	}
	else 
	{
		zerokSanVectorD(vPress);
		err = true;
	}
	return (err);
}
short simCalcKineticPressure (kSanSimulation *sim, kSanVector  *kPress)
{
 // according to the kSan convention, pressure is always positive when 
 // pressing from the outside to make the box smaller.
 // the kinetic component of the pressure is pushing the box to expand, 
 // so the values returned should always be negative
 
	short err = noErr;
 	long partArrIndex;
	particle *basePart;
 	kSanAtomtype **types;
	kSanVector  vec;
	double halfMass;
	zerokSanVectorD (&vec);
	basePart = simLockParts(sim);
	types = simLockAtomTypes(sim);
	for(partArrIndex=0;partArrIndex< simGetNumParts(sim);++partArrIndex)  // usually how many is the number of particles in the group
	{
 		// halfMass = (0.5) * types[BPNthType(basePart, partArrIndex)]->specs.mass ;
 		halfMass = (0.5) * types[BPNthType(basePart,partArrIndex)]->specs.mass ;
		vec.a -= basePart->baseVelocity[partArrIndex].a * basePart->baseVelocity[partArrIndex].a * halfMass;
		vec.b -= basePart->baseVelocity[partArrIndex].b * basePart->baseVelocity[partArrIndex].b * halfMass;
#ifndef __2DVectors__
		vec.c -= basePart->baseVelocity[partArrIndex].c * basePart->baseVelocity[partArrIndex].c * halfMass;
#endif
	}
	//sum of kinetic energies in (g/mol) A A / ps ps   
	
	// now div by cell size to get (g/mol) A  / (ps ps   A A)  force per square angstrom
	multkSanVectorD ( &vec, sim->simCell.reciprocalVolume, kPress);

//	kPress->a = vec.a *  sim->simCell.reciprocalVolume ;
//	kPress->b = vec.b *   sim->simCell.reciprocalVolume ;
//#ifndef __2DVectors__
//	kPress->c = vec.c * sim->simCell.reciprocalVolume ;
//#endif

	simReleaseParts(sim);
	simReleaseAtomTypes(sim);
	return (err);
}

short simCalcLinearStress ( kSanSimulation *sim, kSanVector  *linStress)
{
 // according to the kSan convention, pressure is always positive when 
 // pressing from the outside to make the box smaller.
 // thus, if the sample is in compression, these values should be negative
 // if the sample is in tension, these values should be positive
	short err = noErr;
	if (sim->potentialPlugIn->onePartVirialCalc != nilPointer) simCalcLinearStressFromVirial(sim, linStress);
	else simCalcLinearStressNumerically(sim, linStress);
	return (err);
}
short 	 simGetMomenta(kSanSimulation *sim, kSanVector  *vec)
{ // returns the momentum vector of the system in (g/mol) A  /  ps   
	long partArrIndex;
	short err = noErr;
 	particle * basePart;
	kSanAtomtype **types;
	kSanVector  *thisVel;
	kSanVector  localvec;
	kSanVector  offVec;
	double thisMass;
	
	zerokSanVectorD (&localvec);
	zerokSanVectorD (&offVec);
	basePart = simLockParts(sim);
	types = simLockAtomTypes(sim);
	for(partArrIndex=0;partArrIndex< simGetNumParts(sim);++partArrIndex)  // usually how many is the number of particles in the group
	{
		thisVel = &basePart->baseVelocity[partArrIndex];
		thisMass = types[BPNthType(basePart, partArrIndex)]->specs.mass;
		localvec.a += thisMass * fabs(thisVel->a);
		localvec.b +=  thisMass * fabs(thisVel->b);
#ifndef __2DVectors__
		localvec.c += thisMass *  fabs(thisVel->c);
#endif
		offVec.a += thisMass *  thisVel->a ;
		offVec.b +=  thisMass *  thisVel->b ;
#ifndef __2DVectors__
		offVec.c += thisMass *  thisVel->c ;
#endif
	}
	localvec.a -= offVec.a;
	localvec.b -= offVec.b;
#ifndef __2DVectors__
	localvec.c -= offVec.c;
#endif
//	energy *= (0.00005188444297); // this is 0.5 * ( 1 / 9636.8 )
			// converts from (g/mol) A A / ps ps  to  eV 
			// and multiplies by 1/2   as in 1/2 mv^2
	simReleaseParts(sim);
	simReleaseAtomTypes(sim);
	*vec =  localvec;
	return (err); // returns in eV
}
