
#include "kSanGroup.h"
#include "kSanSimulationPublic.h"
short kSanGroupConstructor (KozoDispatchStack *ds, constructorMethodArgs *cma);
short kSanGroupDestructor (KozoDispatchStack *ds, destructorMethodArgs* dma);

short kSanGroupGetData (KozoDispatchStack *ds, getMethodArgs *gma);
short  kSanGroupSetData (KozoDispatchStack *ds, setMethodArgs *sma);

short  kSanGroupCountContained(KozoDispatchStack *ds, countMethodArgs *cma);
short kSanGroupDoMessage (KozoDispatchStack *ds, doMessageMethodArgs *dmma);
short kSanGroupResolveContainedClass(KozoDispatchStack *ds, resolveMethodArgs *rma);
short kSanGroupHandleCreateElement(KozoDispatchStack *ds, crelMethodArgs *cma);
short	kSanGroupParticleTokenToParticleList(kozoObject *gpObj, kSanGroup *gp, OHToken *myToken, OHList *myList);
short  kSanGroupCountContainedObjects(KozoDispatchStack *ds, countMethodArgs *cma);

short 	initClassParticleGroup(void)
{
	short err = noErr;
	kozoClass *newClass;
	kozoObject *co;

	newClass = addClass(&co,ClassParticleGroup,  ClassKozoObject, sizeof (kSanGroup), "particle group");
	if (newClass)
	{
		err =	setFuncs(newClass, 0L,  kSanGroupConstructor,
		kSanGroupDestructor, kSanGroupGetData, kSanGroupSetData, nilPointer,
		 nilPointer, kSanGroupCountContainedObjects, kSanGroupResolveContainedClass, kSanGroupHandleCreateElement);
		err =  kozoClassAddRemove3ItemsToSave(newClass, true,   pThermalTimeConstant, pSavedEqTemperature,  pXConstraint); 
		err =  kozoClassAddRemove3ItemsToSave(newClass, true,   pYConstraint,  pZConstraint, pAppliedForce);
		err =  kozoClassAddRemove3ItemsToSave(newClass, true,   pColor,  pDoCooling,  pApplyAForce);
		err =  kozoClassAddRemove2ItemsToSave(newClass, true,   pRecentTempDevTime,  pRecentTempDev);
 	}
	return (err);
}
short kSanGroupDestructor (KozoDispatchStack *ds, destructorMethodArgs *cma)
{
 	kSanGroup *gp = (kSanGroup *) ds->thisPD;
 

	disposeList(&gp->particles);
	
	return (noErr);
}
short kSanGroupConstructor (KozoDispatchStack *ds, constructorMethodArgs *cma)
{
#pragma unused(cma)
	short err = noErr;
	long numGroups = 0;
	kozoClass *ohc;
	kSanGroup *gp = (kSanGroup *) ds->thisPD;
 
	initializeList(&gp->particles);
		
	gp->gpIndex = -1;
	gp->XYZConstraints[0] = kGroupMovesFreely;
	gp->XYZConstraints[1] = kGroupMovesFreely;
	gp->XYZConstraints[2] = kGroupMovesFreely;	
	gp->coolingTimeConstant = 0.3;
	gp->actualTemp = 0;
	gp->recentTempDeviation = 0;
	gp->doCooling = true;
	gp->applyAForce = false;
	gp->eqTemp = 0 ;
	zerokSanVectorD(&gp->forceApplied);
	ohc = getClassPtr(ClassParticleGroup);
	{
 		long uid = 0L;
		if (ohc) uid = ohc->instanceCount;

		gp->colour = newColour(groupColour + uid);
	}


 	err = dispatchLoop(ds, cma);
	
	dispatchCount (OHOCtr(ds->obj), ClassParticleGroup, &numGroups);
	gp->gpIndex = numGroups - 1;
	gp->recentTempDeviationCollTime = 0.0;
	KDSReturnError(ds,err);
}


/*
typedef struct crelMethodArgsType
{
	OHTokenDesc *insertionObjectToken; 
	long positionEnum;
	DescType whatClass;  
	AEDesc *initialDataDesc;
	AEDesc *newObjectDesc;
}
crelMethodArgs;
*/
short kSanGroupHandleCreateElement(KozoDispatchStack *ds, crelMethodArgs *cma)
{ // only works if for creating classes which are a subclass of kozoObject
  // objects which support contained object which are not a subclass of kozoObject need to override
  // especially for the case of kAEReplace!
	short err = noErr;
	kSanGroup *gp = (kSanGroup *) ds->thisPD;
	if (cma->whatClass != ClassParticle)
	{
		KDSReturnContinue(ds);
	}
	else 
	{
		OHTokenDesc simToken;
 		err = OHTokenDescConstructor(&simToken, OHOCtr(ds->obj));

	     err = AEPutKeyPtr (cma->initialDataDesc, ClassParticleGroup , typekozoObjectPtr, &ds->obj, sizeof(ds->obj));
		 if (!err) err = dispatchCreateElement (OHOCtr(ds->obj),  cma->whatClass, kAEEnd, 
					&simToken, cma->initialDataDesc,  cma->newObjectDesc) ;
		disposeOHTokenDesc(&simToken);
		KDSDoneDispatch(ds);
 	}
	KDSReturnError(ds, err);
}

short  kSanGroupCountContainedObjects(KozoDispatchStack *ds, countMethodArgs *cma)
{	
	short err = noErr;
	kSanGroup *gp = (kSanGroup *) ds->thisPD;
	if (cma->whatClass == ClassParticle) 
	{
		 *cma->result = OHListCount(&gp->particles);
		 KDSDoneDispatch(ds);
	}
	KDSReturnError(ds, err);
}

short kSanGroupResolveContainedClass(KozoDispatchStack *ds, resolveMethodArgs *rma)
{
	short err = noErr;
	OHList *thisClassList = nilPointer;
	kSanGroup *gp = (kSanGroup *) ds->thisPD;
	
	if (rma->classWanted != ClassParticle)
	{
		KDSReturnContinue(ds);
	}
	else 
	{
		OHToken startToken;
		OHTokenExtract (&startToken, rma->startToken);
		if (kozoObjectIsSubclass(startToken.theObject, ClassParticleGroup) && (startToken.identifier == pSelf))
		{
		// strategy make a temporary token with every particle in the group -- then call ParticleFromParticle
			AEDesc newStartTokenDesc = {typeNull, nilPointer};
			AEDesc *oldStartToken = rma->startToken;
			OHList *partArrIndexList ;
	 		OHToken newStartToken;
			
			allocateList (&partArrIndexList);
			copyListAddingX( &gp->particles, partArrIndexList, 1);
			OHTokenConstructor(&newStartToken, OHOCtr(ds->obj));
			AECreateDesc(typeOHListPtr, (Ptr)&partArrIndexList , 4L, &newStartToken.keydata);
			newStartToken.subObjectClass = ClassParticle;
			newStartToken.identifier = ClassContainedObject;
			AECreateDesc(ClassContainedObject, (Ptr)&newStartToken , sizeof(OHToken), &newStartTokenDesc);
			rma->startToken  = &newStartTokenDesc;
			
			// now rma->startTokenDesc is a token containing all the particles in the group 
			// err = ParticleFromParticle (ds, sim, rma);
			  dispatchResolveContainedClass(OHOCtr(ds->obj), rma);
			
			AEDisposeDesc ( &newStartTokenDesc);
			AEDisposeDesc ( &newStartToken.keydata);
			deallocateList (partArrIndexList);
			rma->startToken  = oldStartToken;
		}
		else
		{
			err = errAECoercionFail;
		}
	}

	KDSReturnError(ds, err);
}

short kSanGroupGetData (KozoDispatchStack *ds, getMethodArgs *gma)
{
	short err = noErr;
	kSanGroup *gp = (kSanGroup *) ds->thisPD;
	switch (gma->property)
	{
		case  pXAppliedForce :
			makeDoubleDesc(gp->forceApplied.a , gma->result);
			break;
		case  pYAppliedForce :
			makeDoubleDesc(gp->forceApplied.b , gma->result);
			break;
#ifndef __2DVectors__
		case  pZAppliedForce :
			makeDoubleDesc(gp->forceApplied.c , gma->result);
			break;
#endif
		case  pThermalTimeConstant:
			makeDoubleDesc(gp->coolingTimeConstant , gma->result);
			break;
		case  pRecentTempDevTime :
  			makeDoubleDesc(gp->recentTempDeviationCollTime , gma->result);
			break;
		case  pRecentTempDev :
 			makeDoubleDesc(gp->recentTempDeviation , gma->result);
			break;
		case  pSavedEqTemperature : // so that we dont mess up a saved simulation
		case  pEqTemperature :
			makeDoubleDesc(gp->eqTemp , gma->result);
			break;
		case  pTemperature :
			makeDoubleDesc(gp->actualTemp , gma->result);
			break;
		case  pXConstraint:
			makeShortDesc(gp->XYZConstraints[0] , gma->result);
			break;
		case  pYConstraint:
			makeShortDesc(gp->XYZConstraints[1] , gma->result);
			break;
		case  pZConstraint :
			makeShortDesc(gp->XYZConstraints[2] , gma->result);
			break;
 		case  pIndex :
			err =  makeShortDesc(gp->gpIndex + 1, gma->result);
			break;
		case  pAppliedForce :
			err =  makekSanVectorDescD(&gp->forceApplied, gma->result);
			break;
		case  pColor :
			err =  AECreateDesc(typeRGBColor, &gp->colour, sizeof(RGBColor), gma->result);
			break;
		case  pDoCooling :
			err =  makeBoolDesc (gp->doCooling, gma->result);
			break;
		case  pApplyAForce :
			err =  makeBoolDesc (gp->applyAForce, gma->result);
			break;
		default:
			KDSReturnContinue(ds);
	}
	KDSDoneDispatch(ds);
	KDSReturnError(ds, err);
}

short  kSanGroupSetData (KozoDispatchStack *ds, setMethodArgs *sma)
{
	short err = noErr;
    short aShort;
	kSanGroup *gp = (kSanGroup *) ds->thisPD;
	switch (sma->property)
	{
		case  pXAppliedForce :
		case  pYAppliedForce :
		case  pZAppliedForce :
		case  pAppliedForce :
			{
				kSanVector  myAppliedForce;
				if (sma->property == pAppliedForce) 
				{
					err = OHCoerceTokSanVectorD(sma->data, &myAppliedForce);
				}
				else
				{
					myAppliedForce = gp->forceApplied;
					
					switch (sma->property)
					{
						case  pXAppliedForce :
							err = OHCoerceToDouble(sma->data, &gp->forceApplied.a);
							break;
						case  pYAppliedForce :
							err = OHCoerceToDouble(sma->data, &gp->forceApplied.b);
							break;
#ifndef __2DVectors__
						case  pZAppliedForce :
							err = OHCoerceToDouble(sma->data, &gp->forceApplied.c);
							break;
#endif
					}
				}
				setAppliedForce (gp, &myAppliedForce);
			}
			break;
		case  pRecentTempDevTime :
			err = OHCoerceToDouble(sma->data, &gp->recentTempDeviationCollTime);
			break;
		case  pRecentTempDev :
			err = OHCoerceToDouble(sma->data, &gp->recentTempDeviation);
			break;
		case  pTemperature :
		case  pEqTemperature :
		case  pSavedEqTemperature : // so that we dont mess up a saved simulation
			err = OHCoerceToDouble(sma->data, &gp->eqTemp);
			if (sma->property != pSavedEqTemperature)  gp->recentTempDeviationCollTime = 0;
 			break;
 		case  pThermalTimeConstant:
			err = OHCoerceToDouble(sma->data, &gp->coolingTimeConstant);
			if (!(gp->coolingTimeConstant))  gp->doCooling = false;
			
 			break;
 		case  pXConstraint:
		case  pYConstraint:
		case  pZConstraint:
			err = OHCoerceToShort(sma->data, &aShort);
			setConstraints(gp, sma->property, aShort);
 			break;
		case  pColor :
 			 err = OHCoerceToRGB(sma->data, &gp->colour );
 			break;
		case  pDoCooling :
 			 err = OHCoerceToBool(sma->data, &gp->doCooling);
  			break;
		default:
			KDSReturnContinue(ds);
	}
	KDSDoneDispatch(ds);
	KDSReturnError(ds, err);
}

 short  kSanGroupRemoveIfEmpty(kozoObject *gpObj) 
{ //  check so that we are not removing the only group
	kSanGroup *gp = gpData (gpObj);
	if  (OHListCount(&gp->particles) == 0) 
	{
		long numGroups = 0;
 		dispatchCount(OHOCtr(gpObj), ClassParticleGroup, &numGroups);
		if (numGroups > 1)
		{
			  kozoDispose(gpObj);	
		}
	}	
	return (noErr);
}

void getTempWindowWidth(kSanGroup *gp,   double *width)
{
	double windowWidth = 0;
	long count = OHListCount(&gp->particles);
	if (count > 0)
	{
		windowWidth =  (gp->eqTemp / sqrt(count) );
	}
	*width = windowWidth;
}

 void	setConstraints(kSanGroup *gp,  long whichC, short theC)
{
	switch (whichC)
	{
		case pXConstraint:
			gp->XYZConstraints[0] = theC;
			break;
		case pYConstraint:
			gp->XYZConstraints[1] = theC;
			break;
		case pZConstraint:
			gp->XYZConstraints[2] = theC;
			break;
	}
}
 
 
 

void addExternalForces(kSanGroup *gp,  particle * basePart, kSanVector * areas)
{
	long j;
	long *gList;
	long count;
 	long partArrIndex;	
	count = OHListCount(&gp->particles);
	gList = lockList(&gp->particles);
	if (gp->applyAForce)
	{
		double invNumParts;
		kSanVector force;
		
  		invNumParts = 1.0 / ((float)(count));
  		multkSanVectorD (areas , invNumParts, &force);
  		prodkSanVectorsD (&force , &gp->forceApplied, &force);
  		
/*		forceA = gp->forceApplied.a * invNumParts * areas->a;
		forceB = gp->forceApplied.b * invNumParts * areas->b;
#ifndef __2DVectors__
		forceC = gp->forceApplied.c * invNumParts * areas->c;
#endif	*/	
		for (j = 0; j<count; ++j)  // external force
		{
		    partArrIndex = gList[j];
		    addkSanVectorsD (&basePart->baseForce[partArrIndex], &force, &basePart->baseForce[partArrIndex] );
 		}
	}
	releaseList(&gp->particles);
}

void  setAppliedForce(kSanGroup *gp,  kSanVector * theForce)
{
	gp->forceApplied = (*theForce);
	
	if (( gp->forceApplied.a == 0.0) &&
		( gp->forceApplied.b == 0.0) 
#ifndef __2DVectors__
		&& ( gp->forceApplied.c == 0.0)  
#endif	*/	
										)
	{
		gp->applyAForce = false;
	}
	else 
	{
		gp->applyAForce = true;
	}
}

