  
#include "OHAEDescUtils.h"
#include "kSanSimulationPrivate.h"
 
#define keyFractionalStrain 'Hzmv'
short simDoStrainEvent(kSanSimulation *sim , AEMessageData *md )
{
	short err = noErr;
	AEDesc	 data = {typeNull, NULL};
	long typeOfStrain = keyFractionalStrain;
	double theValue;
	DescType theProperty = 0L;
		
	if ( md->token->identifier == pSelf) theProperty = pCellXYZLength;
	else if ( md->token->identifier == cProperty) theProperty = md->token->property;
	
	
         err = AEGetParamDesc( md->event, keyAEData, typeLongFloat, &data );
	if (!err) typeOfStrain = keyAEData;
	else  
	{
		err = AEGetParamDesc( md->event, keyFractionalStrain, typeLongFloat, &data );
		if (!err)  typeOfStrain = keyFractionalStrain;
	}
	if (!err)  OHCoerceToDouble(&data, &theValue);
	
	switch (typeOfStrain)
	{
		case keyAEData:  // an absolute strain
			switch (theProperty)
			{
				case pCellXLength:									
					theValue = theValue / sim->simCell.cellEdges.a;	
					goto doScale;
				case pCellYLength:
					theValue = theValue / sim->simCell.cellEdges.b;	
					goto doScale;
#ifndef __2DVectors__
				case pCellZLength:
					theValue = theValue / sim->simCell.cellEdges.c;	
					goto doScale;
#endif
				doScale:	
					err = scaleAxis(sim, md->token->identifier, &theValue);	
					break;
				case pCellXYZLength:
					{
						kSanVector  vec;
						kSanVector  scalingVec;
						err = OHCoerceTokSanVectorD(&data, &vec);
						if (!err)
						{
							scalingVec.a = vec.a / sim->simCell.cellEdges.a;
							scalingVec.b = vec.b / sim->simCell.cellEdges.b;
#ifndef __2DVectors__
							scalingVec.c = vec.c / sim->simCell.cellEdges.c;
#endif
							err = scaleAxis(sim, pCellXYZLength, (double *)&scalingVec);	
						}
					}	
					break;
				default:	
					err= errAEEventNotHandled;
					break;
	    		}	
	    		
			break;
		case keyFractionalStrain:
			if (theProperty == pCellXYZLength) theProperty = pCellXYZUniform;
			err = scaleAxis(sim, theProperty, &theValue);
			break;		
	}
	simIncrementStateCount(sim);
	  AEDisposeDesc( &data );
	return (err);
}								

void getPrimitiveVectors(kSanSimulation *sim, kSanMatrix *trix)
{
 	*trix = sim->simCell.primitiveVectors;
}
void  reflectCellCallback ( kSanSimulation *sim,  particle *basePart, long i, void * dummy);
void  reflectCellCallback ( kSanSimulation *sim,  particle *basePart, long i, void * dummy)
{
#pragma unused(dummy)
	kSanVector  pVec;
	pVec = basePart->basePosition[i]; 
	pVec.a = - pVec.a;
	kSanParticleSetPosition(sim, i, &pVec);
	pVec = basePart->baseVelocity[i]; 
	pVec.a = - pVec.a;
	kSanParticleSetVelocity(sim, i,&pVec);
	pVec = basePart->baseForce[i]; 
	pVec.a = - pVec.a;
	kSanParticleSetForce(sim, i,&pVec);
}
void  reflectCell (kSanSimulation *sim)
{
	 processEveryParticle (sim, (void (*)( kSanSimulation *, particle *, long , void * )) reflectCellCallback,  0L);
	sim->simCell.primitiveVectors.B.a = - sim->simCell.primitiveVectors.B.a;
#ifndef __2DVectors__
	sim->simCell.primitiveVectors.C.a = - sim->simCell.primitiveVectors.C.a;
#endif
	calcFromPrimitives(&sim->simCell);	
}

short expandCell (kSanSimulation *sim, kSanLongVector * nA, kSanLongVector * nB, kSanLongVector *  nC)
{
#ifdef __2DVectors__
   #pragma unused(nC)
#endif
	char oldHandleState;
	char oldPHandleState;
	 
	long i;
	long vol;
	long err;
	long oldNumParts;
	long newNumParts;
	short rCount, sCount, tCount;
	
	short minr = 0;
	short maxr = 0;
	short mins = 0;
	short maxs = 0;
	short mint = 0;
	short maxt = 0;
	OHList  tempList;
	double rotAngle;
	kSanCell tempCell;
	kSanParticle *oldParts;
	kSanVector  *oldPositions;
 	Handle savedParticleHandle = nilPointer;
//	Handle savedForceHandle = nilPointer;
//	Handle savedVelocityHandle = nilPointer;
	Handle savedPositionHandle = nilPointer;
//	Handle savedPotentialHandle = nilPointer;
	
 	vol = getkSanCellVolumeL(nA, nB, nC);
 
	if (vol < 0) return (-1);
	oldNumParts = simGetNumParts(sim);
	newNumParts = oldNumParts * vol;

 	savedParticleHandle = sim->particlesLCH.handle;	
//	savedForceHandle = sim->forceLCH.handle;	
//	savedVelocityHandle = sim->velocityLCH.handle;	
	savedPositionHandle = sim->positionLCH.handle;	
//	savedPotentialHandle = sim->potentialLCH.handle;	
 	err = HandToHand (&savedParticleHandle); // copy the old data
//	err = HandToHand (&savedForceHandle); // copy the old data
//	err = HandToHand (&savedVelocityHandle); // copy the old data
	err = HandToHand (&savedPositionHandle); // copy the old data
//	err = HandToHand (&savedPotentialHandle); // copy the old data
	if (!err)
	{
		err = initializeList (&tempList);
	}
	if (!err)
	{
		err = resizeMemory(sim, newNumParts);
	}
	if (err) doAlert("\p couldn't allocate memory");
	else
	{
		particle * basePart;
		//particle * oldPart;
		kSanMatrix nP; // newPrimitives
		kSanMatrix oP; // old primitives
		
		for (i = 0 ; i<oldNumParts; ++i)
		{
			addToList (&tempList, i);		
		}
		kSanSimRemoveParticleArrayIndexList	(sim, &tempList, false); // old doClear
		
		basePart = simLockParts(sim);
		oldPositions = (kSanVector  *) HVetState(savedPositionHandle, &oldHandleState); // the copy
		oldParts = (kSanParticle *) HVetState(savedParticleHandle, &oldPHandleState); // the copy
			
		
		oP.A = sim->simCell.primitiveVectors.A;
		oP.B = sim->simCell.primitiveVectors.B;
		
#ifndef __2DVectors__
		oP.C = sim->simCell.primitiveVectors.C;
#endif		

	
#ifndef __2DVectors__
		nP.A.a = nA->a * oP.A.a +  nA->b * oP.B.a +  nA->c * oP.C.a ;
		nP.A.b = nA->a * oP.A.b +  nA->b * oP.B.b +  nA->c * oP.C.b ;
		nP.A.c = nA->a * oP.A.c +  nA->b * oP.B.c +  nA->c * oP.C.c ;

		nP.B.a = nB->a * oP.A.a +  nB->b * oP.B.a +  nB->c * oP.C.a ;
		nP.B.b = nB->a * oP.A.b +  nB->b * oP.B.b +  nB->c * oP.C.b ;
		nP.B.c = nB->a * oP.A.c +  nB->b * oP.B.c +  nB->c * oP.C.c ;

		nP.C.a = nC->a * oP.A.a +  nC->b * oP.B.a +  nC->c * oP.C.a ;
		nP.C.b = nC->a * oP.A.b +  nC->b * oP.B.b +  nC->c * oP.C.b ;
		nP.C.c = nC->a * oP.A.c +  nC->b * oP.B.c +  nC->c * oP.C.c ;
#else
		nP.A.a = nA->a * oP.A.a +  nA->b * oP.B.a  ;
		nP.A.b = nA->a * oP.A.b +  nA->b * oP.B.b  ;
 
		nP.B.a = nB->a * oP.A.a +  nB->b * oP.B.a   ;
		nP.B.b = nB->a * oP.A.b +  nB->b * oP.B.b  ;
#endif		    
		   sim->simCell.primitiveVectors = nP;
		   tempCell.primitiveVectors = nP;
		   calcFromPrimitives(&tempCell);
		   calcFromPrimitives(&sim->simCell);
			// now rotate the cell so its in the right space
		
		   // done expansion of primitive vectors
		   
		   // now loop through old Lattice cells with indices (r,s,t)
		   // , checking if  the lattice point (r+d, s+d, t+d) is
		   // inside the cell defined by nA, nB, nC
		   // note that we ignore the fact that nA, nB and nC may not be orthogonal
		   if (nA->a < 0) minr += nA->a;
		   else maxr += nA->a;
		   if (nA->b < 0) minr += nA->b;
		   else maxs += nA->b;
#ifndef __2DVectors__
		   if (nA->c < 0) minr += nA->c;
		   else maxt += nA->c;
#endif
		   if (nB->a < 0) minr += nB->a;
		   else maxr += nB->a;
		   if (nB->b < 0) minr += nB->b;
		   else maxs += nB->b;
#ifndef __2DVectors__
		   if (nB->c < 0) minr += nB->c;
		   else maxt += nB->c;
		   if (nC->a < 0) minr += nC->a;
		   else maxr += nC->a;
		   if (nC->b < 0) minr += nC->b;
		   else maxs += nC->b;
		   if (nC->c < 0) minr += nC->c;
		   else maxt += nC->c;
#endif
		   
		   maxr = maxr - minr;
		   maxs = maxs - mins;
		   maxt = maxt - mint;
		   minr = -maxr / 2 - 1;
		   mins = -maxs / 2 - 1;
		   mint = -maxt / 2 - 1;
		   maxr = maxr / 2 + 1;
		   maxs = maxs / 2 + 1;
		   maxt = maxt / 2 + 1;
		   
		   // now loop over these old lattice cells
		   
		   for (rCount = minr; rCount <= maxr; ++rCount)
		   {
			   for (sCount = mins; sCount <= maxs; ++sCount)
			   {
				   for (tCount = mint; tCount <= maxt; ++tCount)
				   {
			   	       kSanVector  offset;
			   	       kSanVector  newPos;
			   	       
#ifndef __2DVectors__
			   	       offset.a = rCount*oP.A.a +  sCount*oP.B.a + tCount*oP.C.a;
			   	       offset.b = rCount*oP.A.b +  sCount*oP.B.b + tCount*oP.C.b;
			   	       offset.c = rCount*oP.A.c +  sCount*oP.B.c + tCount*oP.C.c;
#else				   		// bug was here.  need to loop through and check point in cell 
				   		// for each particle
			   	       offset.a = rCount*oP.A.a +  sCount*oP.B.a  ;
			   	       offset.b = rCount*oP.A.b +  sCount*oP.B.b  ;
 #endif				   		
			   	       for (i = 0; i< oldNumParts; ++i)
			   	       {
			   	       	     addkSanVectorsD(&offset,  &oldPositions[i] , &newPos);
					   	    if (vectorIsInsideCell( &tempCell, &newPos))
					   	    {
					   	    	    if (!(pointOverlapsWithParticle(sim, &newPos, 0.1)))
					   	    	    {		
					   	    	    	long newIndex;		   	    	
					   	    		//particle newP;
					   	    		//newP = oldPart[i];
					   	    		//newP.position = newPos;
				   	       	                   newIndex = addParticleWithoutWrap(sim, &newPos, oldParts[i].type, oldParts[i].group, false);
					   	    	    }
				   	             }
				   	   }
				   }
			   }
		   }
#ifndef __2DVectors__
		   if (nP.A.b != 0)
		   {
		             rotAngle = atan(nP.A.c/nP.A.b);
			   rotateAllParticles(sim, kAroundXAxis ,rotAngle);	   
			   rotatekSanMatrixD(kAroundXAxis ,rotAngle, &nP);	
		   } else {}
#endif
		   // now get angle to rotate around z axis so that nP.A is parallel to 100   
		   if (nP.A.a != 0)
		   {
		      rotAngle =  atan(nP.A.b/nP.A.a);
			   rotateAllParticles(sim, kAroundZAxis ,rotAngle);	   
			   rotatekSanMatrixD(kAroundZAxis ,rotAngle, &nP);	   
		   } else {}
		   
		   if (nP.A.a < 0) // check direction
		   {
		      rotAngle =  kPi;
			   rotateAllParticles(sim, kAroundZAxis ,rotAngle);	   
			   rotatekSanMatrixD(kAroundZAxis ,rotAngle, &nP);	   
		   } else {}
		   
#ifndef __2DVectors__
		   // now get angle to rotate around x axis so that nP.B is in XY plane  
		   if (nP.B.b != 0)
		   {
		       rotAngle =  atan(nP.B.c/nP.B.b);
			   rotateAllParticles(sim, kAroundXAxis ,rotAngle);	   
			   rotatekSanMatrixD(kAroundXAxis ,rotAngle, &nP);	   
		   } else {}
		   
		   if (nP.B.b < 0) // check direction
		   {
		       rotAngle = kPi;
			   rotateAllParticles(sim, kAroundXAxis ,rotAngle);	   
			   rotatekSanMatrixD(kAroundXAxis ,rotAngle, &nP);	   
		   } else {}
#endif
		   
		sim->simCell.primitiveVectors = nP;
		calcFromPrimitives(&sim->simCell);
		setOrthorhombicTriclinicSwitch(sim);
		simReleaseParts(sim);
		HSetState(savedPositionHandle, oldHandleState);
 		HSetState(savedParticleHandle, oldPHandleState);
 	} // end HandToHand succeeded
 	
 	OHDisposeHandle(&savedPositionHandle);
 	OHDisposeHandle(&savedParticleHandle);
 	deleteOverlappingParticles(sim, 0.1, true); // true means call get neighbors
 	return (err);
}

