#include <Sound.h>
#include "kSanAGWindowPublic.h"
#include "kSanSimulationPrivate.h"
#include "OHSharedUtils.h"
#include "kozoObjectContents.h"
#include "OHApplicationPublic.h"

short	kSanSimClear	(kSanSimulation *sim)
{
	OHList selList;
	short err;
	
	initializeList (&selList);
	err = BitListToOHList (&sim->selectedBits, &selList);
	
	if (!err) kSanSimRemoveParticleArrayIndexList(sim, &selList, true);
	setSelectionsMenuItems(false); // nothing selected
	kozoObjectMessageForContents(simOBJ(sim), ClassWindow,  kInvalGraphicsMessage, 0L, 0L);
	disposeList(&selList);
	sim->flags.uniqueIDListDirty = true;
	return (err);
}
void	 changeIndexInSelections (kSanSimulation *sim, long newIndex, long oldIndex)
{
	if (getParticleSelected(sim, oldIndex))
	{
	    BitListAccommodate (&sim->selectedBits,(newIndex > oldIndex) ? newIndex : oldIndex);
		setNthBitFalse(&sim->selectedBits, oldIndex);
		setNthBitTrue(&sim->selectedBits,newIndex);
	}
}
void	 changeIndexInHighlights (kSanSimulation *sim, long newIndex, long oldIndex)
{
	if (getParticleHighlighted(sim, oldIndex))
	{
		removeFromHighlights(sim, oldIndex);
		addToHighlights(sim,newIndex);
	}
}
short	 kSanSimRemoveParticleToken	(kSanSimulation *sim, OHToken  *myToken)
{
	short err = noErr;
	short didinit = false;
 	OHList *useList = nilPointer;
	OHList myList;

	switch 	(myToken->keydata.descriptorType) //  -- could point to an OHList, or could specify range
	{
		case typeAbsoluteOrdinal :
		case typeRange :
		case typeLongInteger :
		{
			eilArgs eil;
			eil.maxVal = simGetNumParts(sim);
 			initializeList (&myList);
			didinit = true;
			eil.posList = &myList;
			eil.whatClass = ClassParticle;
			eil.obj = simOBJ(sim);
			err =  extractIndexList(&myToken->keydata, &eil);
			useList = &myList;
			goto removeEm;
 		case typeOHListPtr:
			useList = (*(OHList **)(*myToken->keydata.dataHandle));
			goto removeEm;
 		removeEm:
			if (!err) kSanSimRemoveParticleASIndexList(sim, useList, false);
		}
	}
	if (didinit) disposeList (&myList);
	return (err);
}
long	 kSanSimGetTokenIndex (kSanSimulation *sim, OHToken  *myToken)
{
	long theIndex = -1;

	switch  (myToken->keydata.descriptorType) //  -- could point to an OHList, or could specify range
	{
		case typeAbsoluteOrdinal :
		case typeRange :
		case typeLongInteger :
			{
				eilArgs eil;
				OHList myList;
				eil.maxVal = simGetNumParts(sim);
 				initializeList (&myList);
	 			eil.posList = &myList;
				eil.whatClass = ClassParticle;
				eil.obj = simOBJ(sim);
	  			extractIndexList(&myToken->keydata,  &eil);
				 theIndex = getListMinimum(&myList);
				disposeList (&myList);
			}
			break;
		case typeOHListPtr:
			theIndex = getListMinimum(*(OHList **)(*myToken->keydata.dataHandle));
			break;
	}

	return (theIndex);
}
void	 kSanSimRemoveParticleASIndexList	(kSanSimulation *sim, OHList * theList, short callGetNeighborFuncs)
{
	OHList arrayIndexList;
	initializeList (&arrayIndexList);
	copyListAddingX(theList, &arrayIndexList, -1);

	kSanSimRemoveParticleArrayIndexList(sim,&arrayIndexList,callGetNeighborFuncs);
	disposeList (&arrayIndexList);
}
short	 kSanSimRemoveParticleArrayIndexListCallback	(long partArrIndex, particle *basePart);
short	 kSanSimRemoveParticleArrayIndexListCallback	(long partArrIndex, particle *basePart)
{
	basePart->basePi[partArrIndex].clipboardFlag =  k3ParticleToDelete;
	return (noErr);
}
void	 kSanSimRemoveParticleArrayIndexList (kSanSimulation *sim, OHList * theList, short callGetNeighborFuncs)
{
	long partArrIndex;
	long numDeletedParts;
	long newCount;
	long currentIndex;
	particle *basePart;
	ThreadID thisThread;
	GetCurrentThread(&thisThread);
	if (sim->iterationThreadID !=  thisThread) waitForIterationToFinish(sim);
	
	numDeletedParts = OHListCount(theList);
	
	basePart = simLockParts(sim);
	 OHListEveryItemDoOneArg( theList,  kSanSimRemoveParticleArrayIndexListCallback,  basePart);

	partArrIndex=0;
	currentIndex = 0;
	newCount = simGetNumParts(sim) - numDeletedParts;
	while (partArrIndex < simGetNumParts(sim))
	{
		if (basePart->basePi[partArrIndex].clipboardFlag != k3ParticleToDelete) // copy the data
		{
			if  (partArrIndex != currentIndex)
			{
				//basePart[currentIndex] = basePart[partArrIndex];
				basePart->basePosition[currentIndex] = basePart->basePosition[partArrIndex];
				basePart->basePi[currentIndex] = basePart->basePi[partArrIndex];
				basePart->basePotential[currentIndex] = basePart->basePotential[partArrIndex];
				basePart->baseVelocity[currentIndex] = basePart->baseVelocity[partArrIndex];
				basePart->baseForce[currentIndex] = basePart->baseForce[partArrIndex];
				changeParticleIndexInAllWindows(sim, partArrIndex,currentIndex);
			}
			else {} // no need to copy
			++currentIndex;
		}	
		else 
		{
			// this is a particle to delete
			kSanNeighborsDestructor(sim, basePart, partArrIndex);
			kozoObjectMessageForContents(simOBJ(sim), ClassAGWindow,  kRemoveParticleMessage,  typeLongIntegerValue,(void *)partArrIndex);
 		}						 
		++partArrIndex;
	}
	sim->howManyParts = newCount;
	
	rebuildGroups(sim);	
	refindSelections(sim);	
	refindHighlights(sim);	
	kozoObjectMessageForContents(simOBJ(sim), ClassAGWindow,  kInitializeWindowOrderListMessage, 0L, 0L);
	sim->flags.uniqueIDListDirty = true;

	if (callGetNeighborFuncs)
	{
		long err;
		err = calcNeighbors(sim);		
	}	
	simReleaseParts(sim);
	 simIncrementStateCount(sim);
	simInformPluginsOfChange(sim, kChangedNumParts);
}
short  kSanSimPaste (kSanSimulation *sim)
{
         AEDesc scrapDesc = gNilDesc;
 	Ptr   scrapPtr;
	long positionInScrap = 0;
	long scrapSize;
	long numberPasted;
	long newNumberOfParticles;
	short err = noErr;
	
	waitForIterationToFinish(sim);
#ifndef __2DVectors__
	getFromAppClipboard ('kSan', &scrapDesc);
#else
	getFromAppClipboard ('kTwo', &scrapDesc);
#endif	


 	scrapSize = GetHandleSize (scrapDesc.dataHandle);
	if (scrapSize <= 0)
	{
		OHSystemBeep();  // err encountered
		err = true;
	}
	if (!err)
	{
		
		HLock (scrapDesc.dataHandle);
		scrapPtr = (*scrapDesc.dataHandle);
		numberPasted = countReturns(scrapDesc.dataHandle) - 1;
		newNumberOfParticles = simGetNumParts(sim) + numberPasted;
		err = resizeMemory(sim, newNumberOfParticles);
	 	if (!err)
		{
			positionInScrap = 0;
			goToNextLine(&positionInScrap, scrapPtr);
			err = (scrapSize <= positionInScrap);
		}
		if  (!err)
		{
			eraseSelections(sim);
			while (scrapSize > (positionInScrap + 4))
			{
				kSanVector  pos;
				long index;
				long offset;
				long newIndex; // a dummy
				index = readNextLong((scrapPtr + positionInScrap), &offset);
				positionInScrap += offset;
				pos.a = readNextDouble((scrapPtr + positionInScrap), &offset);
				positionInScrap += offset;
				pos.b = readNextDouble((scrapPtr + positionInScrap), &offset);
				positionInScrap += offset;
#ifndef __2DVectors__
				pos.c = readNextDouble((scrapPtr + positionInScrap), &offset);
				positionInScrap += offset;
#endif
				{
					kSanVector  vel;
					double mass;
					long atNum;
					char symbol[8];
					kozoObject *at;
					long typeIndex = 0;
					vel.a = readNextDouble((scrapPtr + positionInScrap), &offset);
					positionInScrap += offset;
					vel.b = readNextDouble((scrapPtr + positionInScrap), &offset);
					positionInScrap += offset;
#ifndef __2DVectors__
					vel.c = readNextDouble((scrapPtr + positionInScrap), &offset);
					positionInScrap += offset;
#endif
					readNextString(symbol, 8L,  (scrapPtr + positionInScrap), &offset);
					positionInScrap += offset;
					atNum = readNextLong((scrapPtr + positionInScrap), &offset);
					positionInScrap += offset;
					mass = readNextDouble((scrapPtr + positionInScrap), &offset);
					positionInScrap += offset;
					at = simGetTypeFromSymbolMassAndAN(sim, symbol, mass, atNum);
					{
						OHList *list;
						list = getListOfObjectsOfClassX (simOBJ(sim), ClassAtomType);
						if (list)
						{
							typeIndex = whichItem(list, (long) at);
							releaseListOfObjectsOfClassX(simOBJ(sim), ClassAtomType);
						}
						
					}
				         newIndex = addParticle(sim, &pos,  typeIndex, 0, (short)true);  // 0 adds to last group
					kSanParticleSetVelocity  (sim, newIndex, &vel);
				}
				 goToNextLine(&positionInScrap, scrapPtr);
				err = (scrapSize <= positionInScrap);
			}
			 simIncrementStateCount(sim);
		}
		HUnlock (scrapDesc.dataHandle);
 	}
 	AEDisposeDesc(&scrapDesc);
	simInformPluginsOfChange(sim, kChangedNeighbors + kChangedNumParts);
	kozoObjectMessageForContents(simOBJ(sim), ClassWindow,  kInvalGraphicsMessage, 0L, 0L);
	setSelectionsMenuItems(true); // something selected
	return (err);
}

void   zeroPartInfo (particle * basePart, long partArrIndex)
{
	zerokSanVectorD(&basePart->basePosition[partArrIndex]);
	zerokSanVectorD(&basePart->baseVelocity[partArrIndex]);
	zerokSanVectorD(&basePart->baseForce[partArrIndex]);
	basePart->basePi[partArrIndex].clipboardFlag = 0;  
	BPNthHighlighted(basePart, partArrIndex) = kNotHighlighted;
	BPNthType(basePart, partArrIndex) = 0;
	basePart->basePotential[partArrIndex] = 0;
}
short	goToNextLine	(long *pos, Ptr ptr)
{
	short err = noErr;
	while ((ptr[*pos] != '\r' ) &&(ptr[*pos] != '\n' )&&(ptr[*pos] != 0 ))
	{
		++*pos;
	}
	if  (ptr[*pos] == 0 ) err = true;
	else ++*pos;
	return (err);
}
long	countReturns	(Handle theHandle)
{
	long handSize;
	long i;
	char handState;
	Ptr handPtr;
	long numberOfReturns = 0;
	
	handSize = GetHandleSize(theHandle);
	handPtr = HVetState(theHandle, &handState);

	for (i = 0;i<handSize;++i)
	{
	   if ( (*(handPtr + i)) == '\r')  ++numberOfReturns;
	}
	
	HSetState(theHandle, handState);
	return (numberOfReturns);
}

short	kSanSimCut	(kSanSimulation *sim)
{
	short err;
	
	err = kSanSimCopy(sim);
	if (!err) err = kSanSimClear(sim);
	return (err);
}

short	kSanSimGetCopyText	(kSanSimulation *sim, AEDesc *result)
{
	short err = noErr;
	char temp[256];
	long positionInBuffer = 0;
  	long numParts;
 	long numSelected;
	particle * basePart;
	long partArrIndex;

	waitForIterationToFinish(sim);
	basePart = simLockParts(sim);
	numParts = simGetNumParts(sim);
	//selList = lockList(&sim->selections);
	numSelected = BitListNumTrue ( &sim->selectedBits );
    //	count = OHListCount (&sim->selections);
  
	if (numParts > 0)
	{
		kSanSelectedSiteCopyData  sscd;
		double KE;
		short serr = noErr;
  		sscd.desc.descriptorType = typeText;
  		sscd.desc.dataHandle = NewHandle(500 * (numSelected + 1));
 		if (sscd.desc.dataHandle == nilPointer) 
		{
		//	doAlert("\p low on memory. Unable to copy data to scrap");
			err = true;
		}	
		else
		{
			typespecs types;
			long hSize;
			SetHandleSize  (sscd.desc.dataHandle, 0);
 #ifndef __2DVectors__
			sprintf(temp,"label\txPosition\tyPosition\tzPosition\txVelocity\tyVelocity\tzVelocity\tSymbol\tAN\tmass\tpotential\tKE\t");
 #else
			sprintf(temp,"label\txPosition\tyPosition\txVelocity\tyVelocity\tSymbol\tAN\tmass\tpotential\tKE\t");
#endif
			HandleAppendC(sscd.desc.dataHandle, temp);
			sscd.message = selectedSiteMessageGetTitles;
			kozoObjectMessageForContents(simOBJ(sim), ClasskSanPlugIn,  kSanSelectedSiteCopyMessage, typeSelectedSiteCopyData, (&sscd));
			
// change the last tab to a return 
			 hSize = GetHandleSize (sscd.desc.dataHandle);
			(*sscd.desc.dataHandle)[hSize - 1] = '\r';


			for (partArrIndex = 0; partArrIndex < numParts; ++partArrIndex)
			{
			    if (getNthBit(&sim->selectedBits, partArrIndex))
			    {
 	 				simGetNthTypespecs(sim, &types,BPNthType(basePart, partArrIndex));
					KE = kineticEInEV(&basePart->baseVelocity[partArrIndex], types.mass);
					
	 #ifndef __2DVectors__
					sprintf(temp,"%d\t%.5g\t%.5g\t%.5g\t%.5g\t%.5g\t%.5g\t%s\t%d\t%.5g\t%.5g\t%.5g\t"
							,BPNthUniqueID(basePart, partArrIndex), 
							basePart->basePosition[partArrIndex].a,basePart->basePosition[partArrIndex].b,basePart->basePosition[partArrIndex].c,
							basePart->baseVelocity[partArrIndex].a,basePart->baseVelocity[partArrIndex].b,basePart->baseVelocity[partArrIndex].c,
							types.symbol,types.atNumber,types.mass, basePart->basePotential[partArrIndex],KE);
	#else
					sprintf(temp,"%d\t%.5g\t%.5g\t%.5g\t%.5g\t%s\t%d\t%.5g\t%.5g\t%.5g\t"
							,BPNthUniqueID(basePart, partArrIndex), 
							basePart->basePosition[partArrIndex].a,basePart->basePosition[partArrIndex].b,
							basePart->baseVelocity[partArrIndex].a,basePart->baseVelocity[partArrIndex].b,
							types.symbol,types.atNumber,types.mass, basePart->basePotential[partArrIndex],KE);

	#endif
					HandleAppendC(sscd.desc.dataHandle, temp);
					
					sscd.message = selectedSiteMessageCopySite;
					sscd.partArrIndex = partArrIndex;
					kozoObjectMessageForContents(simOBJ(sim), ClasskSanPlugIn,  kSanSelectedSiteCopyMessage,   typeSelectedSiteCopyData, (&sscd));
				// change the last tab to a return 
					hSize = GetHandleSize (sscd.desc.dataHandle);
					(*sscd.desc.dataHandle)[hSize - 1] = '\r';
				}

			}
			AEDuplicateDesc (&sscd.desc, result);
  			DisposeHandle(sscd.desc.dataHandle);
		}		
	}	
	simReleaseParts(sim);
	//releaseList(&sim->selections);
	
 	return (err);
}

/************  DoCopy   ****************/
short	 kSanSimCopy	(kSanSimulation *sim)
{
   short err = noErr;
	AEDesc copyText = gNilDesc;
  	err = kSanSimGetCopyText	( sim, &copyText);
  	if (!err)
  	{
  			 addToAppClipboard (&copyText);
#ifndef __2DVectors__
			copyText.descriptorType = 'kSan';
#else
			copyText.descriptorType = 'kTwo';
#endif
 			 addToAppClipboard (&copyText);
 	}
 	return (err);

}
