#include "kSanSimulationPrivate.h"
#include "kSanMenuDefines.h"
#include "kSanRezIDs.h"
short  simSetDoubleVal(kSanSimulation *sim,DescType theProp, AEDesc *dataDesc);
void	 simSetConstraint	(kSanSimulation *sim, long axis, const AEDesc *boolDesc);
void  simInstallCutoffParams(kSanSimulation *sim, double inner, double outer);
void	 simSetClockRunning	(kSanSimulation *sim, short going);
 short kSanSimGetParticleData(kSanSimulation *sim, getMethodArgs *gma);
    // kSanSimGetParticleData means get for purposes of saving to a file
short kSanSimSetParticleData(kSanSimulation *sim, setMethodArgs *gma);
short kSanSimGetCOD(KozoDispatchStack *ds, kSanSimulation *sim, getMethodArgs *gma);
short kSanSimSetCOD(KozoDispatchStack *ds, kSanSimulation *sim, setMethodArgs *sma);
short ParticleGetSelfReference(kSanSimulation *sim, OHToken *token, getMethodArgs *gma);
void	 simDoTimeSwitch	(kSanSimulation *sim)
{
	char wasGoing;
	wasGoing = evaluateBooleanProperty(simOBJ(sim), pClockRunning, true);
	packageAndCallSetBool(simOBJ(sim), pClockRunning, !wasGoing);
}
void	 handleSimError	(kSanSimulation *sim)
{
	short looper = 0;
	Str255 errStr;
	
	switch (sim->flags.error)
	{
		case  kSanErrPartNotInBox :
			CTo255("part not in box err", errStr);
			break;
		case  kSanErrUnallocatedBox :
			CTo255("unallocated box err", errStr);
			break;
		case  kSanErrGotAllNeighbors :
			CTo255("got all neighbors err", errStr);
			break;
		case  kSanErrBoxMemory :
			CTo255("box memory err", errStr);
			break;
	}
	sim->flags.error = noErr;
}

 short kSanSimSetParticleData(kSanSimulation *sim, setMethodArgs *sma)
{  
 	short err = noErr;
 	long partArrIndex;
 	particle *basePart;
	AEDesc thisDesc = {typeNull, nilPointer};
 	err = OHGetKeyPtr  (sma->data, pNumParts, typeLongInteger, &sim->howManyParts, 4L);
 	if (!err) err = resizeMemory(sim, simGetNumParts(sim));
 	if (!err)
 	{
 		basePart = simLockParts(sim);
		{
	 		VSList vecList;
	 		VSListInitialize(&vecList, sizeof(kSanVector ));
	 		// positions
	  	    err = AEGetKeyDesc (sma->data, pParticlePositions, typeWildCard, &thisDesc);
	  	    if (!err) err = OHCoerceToVSList(&thisDesc, &vecList);
	 		OHDisposeDesc(&thisDesc);
	  	    if (!err)
	  	    {
		 		for (partArrIndex = 0 ; partArrIndex < simGetNumParts(sim); ++partArrIndex)
		 		{
		 			VSListGetNthItem(&vecList, partArrIndex, &basePart->basePosition[partArrIndex]);
		 		}	 	    
	  	    }
			// velocities
	  	    err = AEGetKeyDesc (sma->data, pParticleVelocities, typeWildCard, &thisDesc);
	  	    if (!err) err = OHCoerceToVSList(&thisDesc, &vecList);
	 		OHDisposeDesc(&thisDesc);
	  	    if (!err)
	  	    {
		 		for (partArrIndex = 0 ; partArrIndex < simGetNumParts(sim); ++partArrIndex)
		 		{
		 			VSListGetNthItem(&vecList, partArrIndex, &basePart->baseVelocity[partArrIndex]);
		 		}	 	    
	  	    }
	 		VSListDispose(&vecList);
 		}
 		{
	 		charList cList;
	 		charListInitialize(&cList);
	 		// groups
	  	    err = AEGetKeyDesc (sma->data, pParticleGroups, typeWildCard, &thisDesc);
	  	    if (!err) err = OHCoerceToCharList(&thisDesc, &cList);
	 		OHDisposeDesc(&thisDesc);
	  	    if (!err)
	  	    {
		 		for (partArrIndex = 0 ; partArrIndex < simGetNumParts(sim); ++partArrIndex)
		 		{
		 			BPNthGroup(basePart, partArrIndex) = charListGetNthItem(&cList, partArrIndex);
		 		}	 	    
	  	    }
	 		// types
	  	    err = AEGetKeyDesc (sma->data, pParticleTypes, typeWildCard, &thisDesc);
	  	    if (!err) err = OHCoerceToCharList(&thisDesc, &cList);
	 		OHDisposeDesc(&thisDesc);
	  	    if (!err)
	  	    {
		 		for (partArrIndex = 0 ; partArrIndex < simGetNumParts(sim); ++partArrIndex)
		 		{
		 			BPNthType(basePart, partArrIndex) = charListGetNthItem(&cList, partArrIndex);
		 		}	 	    
	  	    }
 	 		charListDispose(&cList);
 		}
 		{
	 		OHList idList;
	 		initializeList(&idList);
	  	    err = AEGetKeyDesc (sma->data, pParticleUniqueIDs, typeWildCard, &thisDesc);
	  	    if (!err) err = OHCoerceToOHList(&thisDesc, &idList);
	 		OHDisposeDesc(&thisDesc);
	  	    if (!err)
	  	    {
		 		for (partArrIndex = 0 ; partArrIndex < simGetNumParts(sim); ++partArrIndex)
		 		{
		 			 BPNthUniqueID(basePart, partArrIndex) = getNthItem(&idList, partArrIndex);
		 			 BPNthSelected(basePart, partArrIndex) = false;
		 			 BPNthHighlighted(basePart, partArrIndex) = kNotHighlighted ;
		 		}	 	    
	  	    }
		// selections
	  	    err = AEGetKeyDesc (sma->data, pSelectedList, typeWildCard, &thisDesc);
	  	    if (!err) err = OHCoerceToOHList(&thisDesc, &idList);
	  	    setListSelected (sim, &idList, true);
	 		OHDisposeDesc(&thisDesc);
		// highlights
	  	    err = AEGetKeyDesc (sma->data, pHighlightList, typeWildCard, &thisDesc);
	  	    if (!err) err = OHCoerceToOHList(&thisDesc, &idList);
	  	    setListHighlighted (sim, &idList, true);
	 		OHDisposeDesc(&thisDesc);
	 		disposeList (&idList);
 		}
 		
 		// now we have to do all the other initializations that addParticle  would have done
		for (partArrIndex = 0 ; partArrIndex < simGetNumParts(sim); ++partArrIndex)
 		{
 			BPNthBoxIndex(basePart, partArrIndex) = 0;
 			basePart->basePi[partArrIndex].clipboardFlag = 0;
 			basePart->basePotential[partArrIndex] = 0;
  			err = kSanNeighborsConstructor(sim, basePart, partArrIndex);
			zerokSanVectorD(&basePart->baseForce[partArrIndex]);
		}	 	    
 		
 		 simReleaseParts(sim);
 		// linkArrayResize(sim, simGetNumParts(sim));
 		// other stuff is done on handling the kUnflattenFinishedMessage message
 	}
	return(err);
 }
 short kSanSimGetParticleData(kSanSimulation *sim, getMethodArgs *gma)
{  
 	short err = noErr;
 	long partArrIndex;
 	// get a record of basic data about each particle
 	// format in terms of property:  i.e. make a list of positions, velocities, types, groups, uniqueIDs 
 	// selections and highligths will be in the form of a list of indices which are selected / highlighted 
 	particle *basePart;
	AEDesc thisDesc = {typeNull, nilPointer};
	basePart = simLockParts(sim);
	OHMakeAERecord(gma->result);
 	AEPutKeyPtr  (gma->result, pNumParts, typeLongInteger, &sim->howManyParts, 4L);
 	{
 		VSList vecList;
 		VSListInitialize(&vecList, sizeof(kSanVector ));
 		VSListSetNumItems (&vecList, simGetNumParts(sim));
 		// positions
 		for (partArrIndex = 0 ; partArrIndex < simGetNumParts(sim); ++partArrIndex)
 		{
 			VSListSetNthItem(&vecList, partArrIndex, &basePart->basePosition[partArrIndex]);
 		}	
 		err = VSListToAEDesc(&vecList, &thisDesc);
 		if (!err) 
 		{
 			err = AEPutKeyDesc  (gma->result, pParticlePositions, &thisDesc);
 			OHDisposeDesc(&thisDesc);
 		}
 		// velocities
 		for (partArrIndex = 0 ; partArrIndex < simGetNumParts(sim); ++partArrIndex)
 		{
 			VSListSetNthItem(&vecList, partArrIndex, &basePart->baseVelocity[partArrIndex]);
 		}	
 		err = VSListToAEDesc(&vecList, &thisDesc);
 		if (!err) 
 		{
 			err = AEPutKeyDesc  (gma->result, pParticleVelocities, &thisDesc);
 			OHDisposeDesc(&thisDesc);
 		}
 		VSListDispose (&vecList);
 	}
 	{
 		charList cList;
 		charListInitialize(&cList);
 		charListSetNumItems (&cList, simGetNumParts(sim));
 		// group
 		for (partArrIndex = 0 ; partArrIndex < simGetNumParts(sim); ++partArrIndex)
 		{
 			charListSetNthItem(&cList, partArrIndex, BPNthGroup(basePart, partArrIndex));
 		}	
 		err = charListToAEDesc(&cList, &thisDesc);
 		if (!err) 
 		{
 			err = AEPutKeyDesc  (gma->result, pParticleGroups, &thisDesc);
 			OHDisposeDesc(&thisDesc);
 		}
		// type
 		for (partArrIndex = 0 ; partArrIndex < simGetNumParts(sim); ++partArrIndex)
 		{
 			charListSetNthItem(&cList, partArrIndex, BPNthType(basePart, partArrIndex));
 		}	
 		err = charListToAEDesc(&cList, &thisDesc);
 		if (!err) 
 		{
 			err = AEPutKeyDesc  (gma->result, pParticleTypes, &thisDesc);
 			OHDisposeDesc(&thisDesc);
 		}
   		charListDispose (&cList);
 	}
 	{
 		OHList idList;
 		initializeList(&idList);
 		OHListSetNumItems(&idList, simGetNumParts(sim));
 		// unique ID
 		for (partArrIndex = 0 ; partArrIndex < simGetNumParts(sim); ++partArrIndex)
 		{
 			setNthItem(&idList, partArrIndex, BPNthUniqueID(basePart, partArrIndex));
 		}	
 		err = OHListToAEDesc(&idList, &thisDesc);
 		if (!err) 
 		{
 			err = AEPutKeyDesc  (gma->result, pParticleUniqueIDs, &thisDesc);
 			OHDisposeDesc(&thisDesc);
 		}
 		// selectedList
  		OHListSetNumItems(&idList, 0);
		for (partArrIndex = 0 ; partArrIndex < simGetNumParts(sim); ++partArrIndex)
 		{
 			if (BPNthSelected(basePart, partArrIndex)) addToList(&idList, partArrIndex);
 		}	
 		err = OHListToAEDesc(&idList, &thisDesc);
 		if (!err) 
 		{
 			err = AEPutKeyDesc  (gma->result, pSelectedList, &thisDesc);
 			OHDisposeDesc(&thisDesc);
 		}
 		// highlightList
 		OHListSetNumItems(&idList, 0);
 		for (partArrIndex = 0 ; partArrIndex < simGetNumParts(sim); ++partArrIndex)
 		{
 			if (BPNthHighlighted(basePart, partArrIndex) != kNotHighlighted)  addToList(&idList, partArrIndex);
 		}	
 		err = OHListToAEDesc(&idList, &thisDesc);
 		if (!err) 
 		{
 			err = AEPutKeyDesc  (gma->result, pHighlightList, &thisDesc);
 			OHDisposeDesc(&thisDesc);
 		}
    	disposeList (&idList);
 	}
  	simReleaseParts(sim);
 	return(err);
}

short kSanSimSetCOD(KozoDispatchStack *ds, kSanSimulation *sim, setMethodArgs *sma)
{
	short err = noErr;
 	
 	switch (ds->coToken->subObjectClass)
	{
		case ClassParticle:
			err = standardSetCOD(ds, sim, sma, standardParticleSetData);
			goto Done;
		Done:
			KDSDoneDispatch(ds);
	}
  	KDSReturnError(ds, err);
 }

 short kSanSimGetCOD(KozoDispatchStack *ds, kSanSimulation *sim, getMethodArgs *gma)
{ // only works if for creating classes which are a subclass of kozoObject
  // objects which support contained object which are not a subclass of kozoObject need to override
  // especially for the case of kAEReplace!
  
 	short err = noErr;
 	if (ds->coToken->subObjectClass == ClassParticle)
	{
		if (ds->coToken->property == pSelf)
		{
			ParticleGetSelfReference( sim, ds->coToken, gma);
		}
		else
		{
			err = standardGetCOD(ds , sim, gma, standardParticleGetData);
		}
		KDSDoneDispatch(ds);
  	}
	else err = errAENoSuchObject;

	KDSReturnError(ds,err);
}

short  callParticleSetDataI0( kSanSimulation *sim, long partArrIndex, DescType property, AEDesc *data)
{  // partArrIndex strarts from 0  add 1 below to use the ParticleSetData function
#pragma unused(property)
	setMethodArgs sma;
	sma.property = property;
	sma.data = data;
	sma.alternateDataValue = nilPointer;
	return(standardParticleSetData(simOBJ(sim), (void *)sim,  partArrIndex + 1,  &sma));
}

short  callParticleGetDataI0( kSanSimulation *sim, long partArrIndex, DescType property, AEDesc *result, DescType desiredType)
{
	getMethodArgs  gma;
	gma.property = property;
	gma.result = result;
	gma.desiredType  = desiredType;
 	gma.alternateReturnValue = nilPointer;
	return(standardParticleGetData(simOBJ(sim), (void *)sim, partArrIndex + 1, &gma));
}

short  standardParticleGetData(kozoObject *simuObj, void *voidArg, long partArrIndex, getMethodArgs *gma)
{
#pragma unused(simuObj)
 	short err = noErr;
	particle *basePart;
	kSanSimulation *sim = (kSanSimulation *)voidArg;
	double theDouble;
 	DescType theType = typeNull;
	basePart = simLockParts (sim);
	partArrIndex -= 1;
 	switch (gma->property)
	{
		case pSelected:
			err = OHMakeBoolDesc (gma->result, BPNthSelected(basePart, partArrIndex));
 			break;
		case pReference: 
		case pSelf:  // return an object specifier
			err = standardCOObjectSpecifier(simOBJ(sim), ClassParticle, partArrIndex,  gma->result);
		    break;
		case pUniqueID:   
  			err = makeLongDesc(BPNthUniqueID(basePart, partArrIndex), gma->result);
			break;
		case pIndex:
 			err = makeLongDesc(partArrIndex + 1, gma->result);
		    break;
		case pLabel:
		//case 'itmn':
 			err = makeLongDesc(partArrIndex, gma->result);
		    break;
		case pXPosition:
			theDouble = basePart->basePosition[partArrIndex].a;
			goto typeLongFloatMark;
		case pYPosition:
			theDouble = basePart->basePosition[partArrIndex].b;
			goto typeLongFloatMark;

#ifndef __2DVectors__
 		case pZPosition:
			theDouble = basePart->basePosition[partArrIndex].c;
			goto typeLongFloatMark;
#endif
		case pXVelocity:
			theDouble = basePart->baseVelocity[partArrIndex].a;
			goto typeLongFloatMark;
		case pYVelocity:
			theDouble = basePart->baseVelocity[partArrIndex].b;
			goto typeLongFloatMark;

#ifndef __2DVectors__
		case pZVelocity:
			theDouble = basePart->baseVelocity[partArrIndex].c;
 			goto typeLongFloatMark;
#endif
		case pXForce:
			theDouble = basePart->baseForce[partArrIndex].a;
			goto typeLongFloatMark;
		case pYForce:
			theDouble = basePart->baseForce[partArrIndex].b;
			goto typeLongFloatMark;
#ifndef __2DVectors__
		case pZForce:
			theDouble = basePart->baseForce[partArrIndex].c;
			goto typeLongFloatMark;
#endif
		case pPotentialEnergy:
			theDouble = basePart->basePotential[partArrIndex];
			goto typeLongFloatMark;
 		case pKineticEnergy:
			{
				kSanAtomtype ** baseTypes;
				baseTypes = simLockAtomTypes (sim);
				theDouble = kineticEInEV(&basePart->baseVelocity[partArrIndex], baseTypes[BPNthType(basePart, partArrIndex)]->specs.mass);
				simReleaseAtomTypes (sim);
			}
		    goto typeLongFloatMark;
		typeLongFloatMark:
			err = makeDoubleDesc(theDouble, gma->result);
			break;
		case pTypeIndexI1:
			makeShortDesc(BPNthType(basePart, partArrIndex) + 1, gma->result);  // typeIndex starts at zero
			break;
		case pGroupIndex:
			makeShortDesc(BPNthGroup(basePart, partArrIndex), gma->result);
 			break;
		case pType:
			{
				OHList *atList  ;
				kozoObject *thisAT;
				atList = getListOfObjectsOfClassX (simOBJ(sim), ClassAtomType);
				if (atList)
				{
					thisAT = castTokozoObject( getNthItem (atList, BPNthType(basePart, partArrIndex)));
					err = kozoGetData (thisAT, pReference, gma->result);
					 releaseListOfObjectsOfClassX (simOBJ(sim), ClassAtomType);
				}
			}
			break;
		case pPosition:
			err = makekSanVectorDescD(&basePart->basePosition[partArrIndex], gma->result);
		    break;				
		case pVelocity:
			err = makekSanVectorDescD(&basePart->baseVelocity[partArrIndex], gma->result);
		    break;				
		case pForce:
			err = makekSanVectorDescD(&basePart->baseForce[partArrIndex], gma->result);
			break;
		default: 
		    {
		       kozoObject *propertyOwner = nilPointer;
		       getNamedItem (&sim->particlePropertyOwners, gma->property, (long *) &propertyOwner);
		       if (propertyOwner)
		       {
		           OHToken token;
		           token.theObject = propertyOwner;
		           token.identifier = ClassContainedObject;
		           token.subObjectClass = ClassParticle;
		           token.keyform = formAbsolutePosition;
		           partArrIndex += 1;
		           AECreateDesc (typeLongInteger, &partArrIndex, 4L, &token.keydata );
		           token.property = gma->property;
 				   err = kozoGetDataWithToken(propertyOwner, &token, pContainedObjectProperty, gma->desiredType, gma->result);
 		           AEDisposeDesc ( &token.keydata );
				   partArrIndex -= 1;
		       }
		       else
		       {
			        err = errAENoSuchObject ;
		       }
		    }
	}  
 	simReleaseParts(sim);
	return (err);
}
short  standardParticleSetData(kozoObject *simuObj,void *voidArg, long partArrIndex, setMethodArgs *sma)
{
#pragma unused(simuObj)
    // deprecate this function?
	short err;
 	particle *basePart;
	 kSanSimulation *sim = ( kSanSimulation *)voidArg;
	basePart = simLockParts(sim);
	partArrIndex -= 1;
	switch (sma->property)
	{
		case pProperties:
		    // gotcha!  recursive call to ParticleSetData, so we have to add 1 back to the index
		    err =  standardSetCOProperties(simOBJ(sim), sim,  partArrIndex + 1, sma, standardParticleSetData);
		 	break;
		 case pGroupIndex:
		 case ClassParticleGroup:
 		 	if ((sma->data->descriptorType == typeObjectSpecifier) || (sma->data->descriptorType == typekozoObjectPtr))
 		 	{
 			 	kozoObject * gpObj = nilPointer ;
 			 	long gpIndex = 0;
 			 	OHCoerceTokozoObject(sma->data, &gpObj);
 			 	if (gpObj) err = kozoGetDataPtr (gpObj, pIndex, typeLongInteger, &gpIndex, sizeof(long) );
		 	    if (!err)  err = kSanSimAddParticleToGroup(sim, gpIndex-1,  partArrIndex);
 		 	}
 		 	else
		 	{
			 	long groupIndex;
			 	err = OHCoerceToLong(sma->data, &groupIndex);
			 	if (!err) err = kSanSimAddParticleToGroup(sim, groupIndex-1,  partArrIndex);
			}
		case pSelected:
			{
				char select;  
				err = OHCoerceToBool (sma->data, &select);
				if (!err) setParticleSelected (sim, partArrIndex, select);
			}
			break;
		case pTypeIndexI1:
			{
			    short s;
				OHCoerceToShort(sma->data,  &s);
	 			BPNthType(basePart, partArrIndex)  = s - 1;   // typeIndex starts from 0 -- modify here for best UI
	  			if (BPNthType(basePart, partArrIndex) < 0 ) BPNthType(basePart, partArrIndex) = 0;
				if (BPNthType(basePart, partArrIndex)  >= simGetNumAtomtypes(sim) ) BPNthType(basePart, partArrIndex) = 0;
			}
		case pType:
			{
				if (sma->data->descriptorType == typeObjectSpecifier)
				{
					kozoObject *atObj = nilPointer;   
					kSanAtomtype *at = nilPointer;   
					err = OHCoerceTokozoObject (sma->data,  &atObj);
					if (atObj) kSanParticleSetTypeByObj (sim, partArrIndex, atObj);
					//if (atObj) at = atomtypeData(atObj);
					//if (at) kSanParticleSetTypeByName (sim, partArrIndex, at->specs.symbol);
				}
			}
		    break;
		case pXPosition:
		case pYPosition:
#ifndef __2DVectors__
		case pZPosition:
#endif
		case pPosition:
			{
				kSanVector  dv;
				dv = basePart->basePosition[partArrIndex];
				switch (sma->property)
				{
					case pXPosition:
						err = OHCoerceToDouble (sma->data, &dv.a );
						break;
					case pYPosition:
						err = OHCoerceToDouble (sma->data, &dv.b );
						break;
#ifndef __2DVectors__
					case pZPosition:
						err = OHCoerceToDouble (sma->data, &dv.c );
						break;
#endif
					case pPosition:
						err = OHCoerceTokSanVectorD (sma->data, &dv);
						break;
				}
				if (!err) kSanParticleSetPosition (sim, partArrIndex, &dv);
			}
			break;
		case pXVelocity:
		case pYVelocity:
#ifndef __2DVectors__
		case pZVelocity:
#endif		case pVelocity:
			{
				kSanVector  dv;
				dv = basePart->baseVelocity[partArrIndex];
				switch (sma->property)
				{
					case pXVelocity:
						err = OHCoerceToDouble (sma->data, &dv.a );
						break;
					case pYVelocity:
						err = OHCoerceToDouble (sma->data, &dv.b );
						break;
#ifndef __2DVectors__
					case pZVelocity:
						err = OHCoerceToDouble (sma->data, &dv.c );
						break;
#endif
					case pVelocity:
						err = OHCoerceTokSanVectorD (sma->data,  &dv);
						break;
				}
				if (!err) kSanParticleSetVelocity (sim, partArrIndex, &dv);
			}
			break;
		default:
			err = errAENotModifiable;
			break;
	} // end ClassParticle
	simReleaseParts(sim);
	return (err);
}

short  ParticleMakeObjSpecifier(kSanSimulation *sim, long partArrIndex, AEDesc *partDesc)
{    // the index used here starts counting at 1
	short err = noErr;
	long theID;
	AEDesc contDesc = gNilDesc;
	AEDesc singleIDDesc = gNilDesc;
	
	theID = kSanParticleGetUniqueID(sim, partArrIndex - 1);
	err = kozoGetData(simOBJ(sim), pReference, &contDesc);
	if (!err) err =AECreateDesc(typeLongInteger, (Ptr) &theID, 4L, &singleIDDesc);
	if (!err) err = CreateObjSpecifier(ClassParticle , &contDesc, formUniqueID, &singleIDDesc, false, partDesc);
	AEDisposeDesc (&singleIDDesc);
	AEDisposeDesc (&contDesc);
	return(err);
}

short  ParticlePropertyMakeObjSpecifier(kSanSimulation *sim, long partArrIndex, DescType property, AEDesc *propSpec)
{   // the particle index used here starts counting at 1
	short err = noErr;
 	AEDesc partDesc = {typeNull, nilPointer};
 	
 	err = ParticleMakeObjSpecifier(sim,  partArrIndex, &partDesc);
	if (!err) err = ObjSpecMakePropertySpec(&partDesc, property, propSpec);
		
	AEDisposeDesc( &partDesc );
	return (err);
}

short ParticleGetSelfReference(kSanSimulation *sim, OHToken *token, getMethodArgs *gma)
{
	short err = noErr;
	long theIndex;
 	long i = 0;
	long count;
	long theID;
	OHList *aList;
 	switch (token->keydata.descriptorType)
 	{
		case typeOHListPtr:
			aList = (*(OHList **) *token->keydata.dataHandle);
			count = OHListCount(aList);
			// special case for returning a list of particle IDs
			// make a list of ids, and then return "particle ID {3,4,5,7,10,12,13} of simulation 1"
			if (count != 1)
			{
				AEDesc contDesc = {typeNull, NULL};
			    AEDesc uniqueIDListDesc = {typeNull, nilPointer};
				err = kozoGetData(simOBJ(sim), pReference, &contDesc);
				err = AECreateList (nilPointer, 0L, false, &uniqueIDListDesc);
				if (!err)
				{
					for (i = 0; i < count; ++i)
					{
						theIndex = getNthItem(aList, i);
						theID = kSanParticleGetUniqueID(sim, theIndex - 1);
						AEPutPtr(&uniqueIDListDesc, 0, typeLongInteger, (Ptr) &theID, 4L );
					}
				}
				{
					if (!err) err = CreateObjSpecifier(ClassParticle , &contDesc, formUniqueID, &uniqueIDListDesc, false, gma->result);
				}
				AEDisposeDesc (&uniqueIDListDesc);
				AEDisposeDesc (&contDesc);
			}
			else // only 1 particle
			{
 				theIndex = getNthItem(aList, kFirstInList);
				err =  ParticleMakeObjSpecifier(sim, theIndex, gma->result);
			}
			break;
		case typeLongInteger:
 			err = OHCoerceToLong (&token->keydata, &theIndex);
			if (!err) err =  ParticleMakeObjSpecifier(sim, theIndex, gma->result);
			break;
	}
	return (err);
}
void simSetConstraint (kSanSimulation *sim, long axis, const AEDesc *boolDesc)
{
	char constrained;
	char unused;
#pragma unused(unused)
	short err;
	err = OHCoerceToChar( boolDesc, &constrained);
	if (!err) 
	{
		switch (axis)
		{
			case pXConstraint:
				sim->gConstrained.XAxis = constrained;
				break;
			case pYConstraint:
				sim->gConstrained.YAxis = constrained;
				break;
			case pZConstraint:
				sim->gConstrained.ZAxis = constrained;
				break;
		}
	}
}
void  simInstallCutoffParams(kSanSimulation *sim, double inner, double outer)
{
	double minCellDim;
	minCellDim =  getMinimumDimension (&sim->simCell);
	
	if (inner > outer) 
	{
		double tempD;
		tempD = inner;
		inner = outer;
		outer = tempD;
	}
	
	if ((outer + sim->neighborCutoffExtra) < (minCellDim * 0.33))
	{ // large cell, three or more boxes per axis
		sim->flags.pbcMode = kSanLargeModelPBC;
	}
	else if ((outer +  sim->neighborCutoffExtra) < (minCellDim * 0.5))
	{ // medium cell, two or more boxes per axis, 
	 	// but a pair of particles can only be neighbors once
		sim->flags.pbcMode = kSanMediumModelPBC;
	}
	else if ((outer +  sim->neighborCutoffExtra) < (minCellDim))
	{ // small cell, possibly only one box on an axis, 
			// a particle cannot be its own neighbor, but any pair of particles
			// many be neighbors multiple times
		sim->flags.pbcMode = kSanSmallModelPBC;
	}
	else 
	{ // because a particle can be its own neighbor, the calculation will not go
		sim->flags.pbcMode = kSanTooSmallModelPBC;
		if (sim->flags.clockRunning) packageAndCallSetBool(simOBJ(sim), pClockRunning, false);
		// (sim, );
		doAlert("\pcutoff value is too large for this cell size.  Cannot perform calculations");
	}
	sim->innerCutoff = inner;
	sim->outerCutoff = outer;
	sim->middleCutoff = 0.5 * (inner + outer);
 	
 	if (( outer - inner ) > 0 )
 	{
 	   sim->piOverRange = (kPi / ( outer - inner ));
 	}
 	else
 	{
 	 sim->piOverRange = 1.0;
 	}
 	
 	{
		short err;
		err = initBoxes(sim);   // because box size may have changed
		if (err)          // boxArrayHandle can not be locked
		{
			sim->flags.error = true;
			packageAndCallSetBool(simOBJ(sim), pClockRunning, false);
		//(sim, );
		}
	}
	 simInformPluginsOfChange(sim, kChangedCutoffs);
}

void	 simSetClockRunning	(kSanSimulation *sim, short going)
{
	short theRezID;
	short err;
	kozoObject *clockMenuItem;
	AEDesc strDesc = {typeChar, nilPointer};
	waitForIterationToFinish(sim);
	
	if (sim->flags.error)
	{
		handleSimError(sim);
	}
	if ((!going) || ((sim)->flags.error))
	{
		theRezID = kGoStringID ;
		sim->flags.clockRunning = false;
	}
	else 
	{
		theRezID = kStopStringID ;		
		sim->flags.clockRunning = true;
	}
	
	err = ExtractStringResource (&strDesc, theRezID);
	if (!err)
	{
		clockMenuItem = OHGetNamedReference(globalAppObject, iGoMenuItem);  
		kozoSetData(clockMenuItem, pName, &strDesc);
	}
	if (strDesc.dataHandle) DisposeHandle (strDesc.dataHandle);
 }
 short  simSetModuleDesc(kSanSimulation *sim, DescType theProp, const AEDesc *dataDesc)
{
	short err = noErr;
 	DescType classID;

	kozoObject *obj;
	err = OHCoerceTokozoObject (dataDesc, &obj);
	if (!err)  simInstallUniqueModule ( sim, theProp, obj);
 	else
	{	
		err = kSanResolveModule ( theProp, dataDesc, &classID);
		if (!err) simMakeAndInstallModule(sim, theProp, classID );
 	}
	return (err);
}

short  kSanSimSetData (KozoDispatchStack *ds, setMethodArgs *sma)
{
	short err = noErr;
	char aChar;
	kSanSimulation *sim = (kSanSimulation *) ds->thisPD;
 	switch (sma->property)
	{
		case pContainedObjectProperty:
			err = kSanSimSetCOD(ds, sim,  sma);
			KDSReturnError(ds, err);
		case pParticleData:  // when reading a file
			err = kSanSimSetParticleData(sim, sma);
			break;
		case pClockRunning:
			 OHCoerceToBool(sma->data, &aChar);
			simSetClockRunning	(sim, aChar);
			break;
		case pMenusDirty:
			 OHCoerceToBool(sma->data, &sim->flags.menusDirty);
			break;
		case pNeighborCutoffExtra:
			OHCoerceToDouble(sma->data, &sim->neighborCutoffExtra);
			goto installCutoffs;
		case pCutoffs:
			err = OHGetNthPtr(sma->data, 1, typeLongFloat, &sim->innerCutoff, sizeof(sim->innerCutoff));
			err = OHGetNthPtr(sma->data, 2, typeLongFloat, &sim->outerCutoff, sizeof(sim->outerCutoff));
 			goto installCutoffs;
		case pInnerCutoff:
			OHCoerceToDouble(sma->data, &sim->innerCutoff);
			if (sim->outerCutoff < sim->innerCutoff) sim->outerCutoff = sim->innerCutoff;
			goto installCutoffs;
		case pOuterCutoff:
			OHCoerceToDouble(sma->data, &sim->outerCutoff);
			if (sim->outerCutoff < sim->innerCutoff) sim->innerCutoff = sim->outerCutoff ;
			goto installCutoffs;
		installCutoffs:
			simInstallCutoffParams(sim, sim->innerCutoff, sim->outerCutoff);
			break;
		case pXConstraint:
		case pYConstraint:
		case pZConstraint:
			simSetConstraint(sim, sma->property, sma->data);
			break;
 		case pCellResizeMethod:
   		case pIterationMethod:
		case pPotentialMethod:
 			simSetModuleDesc(sim, sma->property, sma->data);
 			dispatchNilMessage(simOBJ(sim), kInvalWorldsMessage);
 			break;   
 		case pUniqueParticleCounter: // for file reading
			OHCoerceToType (sma->data, &sim->uniqueParticleCounter);
			break;   
		case pTime: // for file reading
			OHCoerceToDouble (sma->data, &sim->totalTimeElapsed);
			break;   
		case pIterations: // for file reading
			OHCoerceToType (sma->data, &sim->totalNumberOfIterations);
			break;   
		case pTimestep:
			OHCoerceToDouble (sma->data, &sim->deltaT);
			break;
 		case pXAppliedForce :
			OHCoerceToDouble (sma->data, &sim->simCell.appliedStrainStress.a);
			break;
		case pYAppliedForce :
			OHCoerceToDouble (sma->data, &sim->simCell.appliedStrainStress.b);
			break;
#ifndef __2DVectors__
		case pZAppliedForce :
			OHCoerceToDouble (sma->data, &sim->simCell.appliedStrainStress.c);
			break;
#endif
		case pAppliedForce :
			OHCoerceTokSanVectorD(sma->data, &sim->simCell.appliedStrainStress);
			break;
		case pCellXLength:
			OHCoerceToDouble (sma->data, &sim->simCell.cellEdges.a);
			goto edgesAndAngles;
		case pCellYLength:
			OHCoerceToDouble (sma->data, &sim->simCell.cellEdges.b);
			goto edgesAndAngles;
#ifndef __2DVectors__
		case pCellZLength:
			OHCoerceToDouble (sma->data, &sim->simCell.cellEdges.c);
			goto edgesAndAngles;
#endif
		case pCellXYZLength :
			OHCoerceTokSanVectorD(sma->data, &sim->simCell.cellEdges);
			goto edgesAndAngles;
		case pCellAlphaAngle:
			OHCoerceToDouble (sma->data, &sim->simCell.alpha);
			goto edgesAndAngles;
#ifndef __2DVectors__
		case pCellBetaAngle:
			OHCoerceToDouble (sma->data, &sim->simCell.beta);
			goto edgesAndAngles;
		case pCellGammaAngle:
			OHCoerceToDouble (sma->data, &sim->simCell.gamma);
 			goto edgesAndAngles;
		case pCellAngles :
			OHCoerceTokSanVectorD(sma->data, (kSanVector  *) &sim->simCell.alpha);
			goto edgesAndAngles;
#endif
		edgesAndAngles:
			calcCellPropertiesFromEdgesAndAngles(&sim->simCell);
			break;
		case pAutomaticUpdate :
 			OHCoerceToBool(sma->data, &sim->flags.automaticUpdate);
 			break;
 			
 		case pRedrawPeriod:
			OHCoerceToLong (sma->data, &sim->countThreshholds.redraw);
  			break;
 		case  pTempCorrPeriod:
			OHCoerceToLong (sma->data, &sim->countThreshholds.therm);
 			break;
 		case  pMemoryCleanupPeriod:
			OHCoerceToLong (sma->data, &sim->countThreshholds.memoryCleanup);
 			break;
 		case  pAutoSavePeriod:
			OHCoerceToLong (sma->data, &sim->countThreshholds.dataSave);
 			break;
 		case  pNeighborRecalcPeriod:
			OHCoerceToLong (sma->data, &sim->countThreshholds.neighbor);
 			break;
 		case  pVelocityCenteringPeriod:
			OHCoerceToLong (sma->data, &sim->countThreshholds.velocityCentering);
 			break;
 		case  pCellPressureCheckPeriod:
			OHCoerceToLong (sma->data, &sim->countThreshholds.cellPressure);
 			break;
		Default :
		default :
			KDSReturnContinue(ds);
			 break;
	}
	KDSDoneDispatch(ds);
	KDSReturnError(ds, err);
}

short  kSanSimGetData (KozoDispatchStack *ds, getMethodArgs *gma)
{
	short err = noErr;
 	double doubleVal = 0;
	kSanSimulation *sim = (kSanSimulation *) ds->thisPD;
	switch (gma->property)
	{
		case pContainedObjectProperty:
			err = kSanSimGetCOD(ds, sim, gma);
			KDSReturnError(ds, err);
		case pParticleData:  // for saving a file
			err = kSanSimGetParticleData(sim, gma);
			break;
		case pCellResizeMethod:
			 kozoGetDataWType (OHGetNamedReference (simOBJ(sim), ClassCellResizePlugIn), pSelf, gma->desiredType, gma->result);
			break;
		case pIterationMethod:
			 kozoGetDataWType (OHGetNamedReference (simOBJ(sim), ClassMethodPlugIn), pSelf, gma->desiredType, gma->result);
			break;
		case pPotentialMethod:
			 kozoGetDataWType (OHGetNamedReference (simOBJ(sim), ClassPotentialPlugIn), pSelf, gma->desiredType, gma->result);
			break;
		case pTimestep:
			doubleVal = sim->deltaT;
			goto caseTypeLongFloat;
		case pPotentialEnergy:
			doubleVal = sim->potEnergy;
			goto caseTypeLongFloat;
		case pKineticEnergy:
			doubleVal = sim->kinEnergy;
			goto caseTypeLongFloat;
		case pTotalEnergy:
			doubleVal =  sim->kinEnergy + sim->potEnergy;
			goto caseTypeLongFloat;
		case pUniqueParticleCounter:
			err = makeLongDesc(  sim->uniqueParticleCounter, gma->result );
 			break;
		case pTime:
			doubleVal = sim->totalTimeElapsed;
			goto caseTypeLongFloat;
		case pIterations:
			err = makeLongDesc(  sim->totalNumberOfIterations, gma->result );
 			break;
 
		case pCutoffs:
			OHMakeAEList (gma->result);
 			AEPutPtr(gma->result, 0, typeLongFloat, &sim->innerCutoff, sizeof(sim->innerCutoff));
			AEPutPtr(gma->result, 0, typeLongFloat, &sim->outerCutoff, sizeof(sim->outerCutoff));
  			break;
		case pInnerCutoff:
			doubleVal = sim->innerCutoff ;
			goto caseTypeLongFloat;
		case pOuterCutoff:
			doubleVal = sim->outerCutoff;
			goto caseTypeLongFloat;
			
		case pCellXLength:
			doubleVal = sim->simCell.cellEdges.a;
			goto caseTypeLongFloat;
		case pCellYLength:
			doubleVal = sim->simCell.cellEdges.b;
			goto caseTypeLongFloat;
#ifndef __2DVectors__
		case pCellZLength:
			doubleVal = sim->simCell.cellEdges.c;
			goto caseTypeLongFloat;
		case pCellAlphaAngle:
			doubleVal = sim->simCell.alpha;
			goto caseTypeLongFloat;
		case pCellBetaAngle:
			doubleVal = sim->simCell.beta;
			goto caseTypeLongFloat;
#endif
		case pCellGammaAngle:
			doubleVal = sim->simCell.gamma;
			goto caseTypeLongFloat;
			
 		case  pXAppliedForce :
			if (!sim->gConstrained.XAxis )   doubleVal = sim->simCell.appliedStrainStress.a;
			goto caseTypeLongFloat;
		case  pYAppliedForce :
			if (!sim->gConstrained.YAxis )   doubleVal = sim->simCell.appliedStrainStress.b;
			goto caseTypeLongFloat;
#ifndef __2DVectors__
		case  pZAppliedForce :
			if (!sim->gConstrained.ZAxis )  doubleVal = sim->simCell.appliedStrainStress.c;
			goto caseTypeLongFloat;
#endif
		case pCellAngles :
			makekSanVectorDescD((kSanVector  *) &sim->simCell.alpha, gma->result);
 			break;
		case pCellXYZLength :
			makekSanVectorDescD(&sim->simCell.cellEdges, gma->result);
 			break;
		case pAppliedForce :
			makekSanVectorDescD(&sim->simCell.appliedStrainStress, gma->result);
 			break;
 			
 			
 		case pAutomaticUpdate :
 			makeBoolDesc(sim->flags.automaticUpdate, gma->result);
			break;
		case pClockRunning:
			makeBoolDesc(sim->flags.clockRunning, gma->result);
			break;
 		caseTypeLongFloat:
			err = makeDoubleDesc( doubleVal, gma->result );
 			break;
 			
 		case pRedrawPeriod:
			err = makeLongDesc( sim->countThreshholds.redraw, gma->result );
 			break;
 		case  pTempCorrPeriod:
			err = makeLongDesc( sim->countThreshholds.therm, gma->result );
 			break;
 		case  pMemoryCleanupPeriod:
			err = makeLongDesc( sim->countThreshholds.memoryCleanup, gma->result );
 			break;
 		case  pAutoSavePeriod:
			err = makeLongDesc( sim->countThreshholds.dataSave, gma->result );
 			break;
 		case  pNeighborRecalcPeriod:
			err = makeLongDesc( sim->countThreshholds.neighbor, gma->result );
 			break;
 		case  pVelocityCenteringPeriod:
			err = makeLongDesc( sim->countThreshholds.velocityCentering, gma->result );
 			break;
 		case  pCellPressureCheckPeriod:
			err = makeLongDesc( sim->countThreshholds.cellPressure, gma->result );
 			break;
        case pCopytext:
             kSanSimGetCopyText	( sim, gma->result);
             break;

 		default :
			KDSReturnContinue(ds);
	}
	KDSDoneDispatch(ds);
	KDSReturnError(ds, err);
}
short  simGetNumGroups(kSanSimulation *sim )
{
 	long numGroups =  0;
 	dispatchCount (simOBJ(sim), ClassParticleGroup, &numGroups);
	return (numGroups);
}

short	 simPostNextYieldCallback	(kSanSimulation *sim, unsigned long theValue);
short	 simPostNextYieldCallback	(kSanSimulation *sim, unsigned long theValue)
{
	if ((theValue >= sim->totalNumberOfIterations) && (theValue < sim->flags.lowestYieldFlag))
	   sim->flags.lowestYieldFlag = theValue;
	return (noErr);
}

void	 simRaiseYieldFlag	(kSanSimulation *sim, unsigned long iter)
{
 	addToList (&sim->yieldFlags, iter);
	sim->flags.lowestYieldFlag = 0xFFFFFFFF;
	OHListEveryItemAsSecondArg( &(sim->yieldFlags),  simPostNextYieldCallback, sim);
} 

void	 simRemoveYieldFlag	(kSanSimulation *sim, unsigned long iter)
{
 	removeItem (&sim->yieldFlags, iter);
	sim->flags.lowestYieldFlag = 0xFFFFFFFF;
	OHListEveryItemAsSecondArg( &(sim->yieldFlags),  simPostNextYieldCallback, sim);
} 



void simGetCellPrimitives(kSanSimulation *sim, kSanMatrix  *primitiveVectors)
{
	*primitiveVectors = sim->simCell.primitiveVectors;
} 

DescType  simGetPlugInBeingUsed(kozoObject *simObj, DescType whatModule)
 {
 	DescType theType;
 	kozoObject *theObj = OHGetNamedReference(simObj, whatModule);
	if (theObj == nilPointer) theType = 0L;
	else theType = OHOCID(theObj);
 	return (theType);
}

long simIncrementStateCount(kSanSimulation *sim)
 {
 	++sim->stateCount;
 	return (sim->stateCount);
 }
double simSumPotentialEnergy(kSanSimulation *sim) 
{
	long  partArrIndex;
	particle *basePart;
	double energy = 0;
	basePart = simLockParts(sim);
	for (partArrIndex = 0; partArrIndex< simGetNumParts(sim); ++partArrIndex)
	{
	    energy +=  basePart->basePotential[partArrIndex];	
	}
	simReleaseParts(sim);
	return (energy);
} 
 void simSetPlugNotificationSuspended (kSanSimulation *sim, short suspend)
{
	if (suspend) ++sim->plugNotificationSuspended ;
	else 
	{
		--sim->plugNotificationSuspended ;
		if (sim->plugNotificationSuspended == false)
		    simInformPluginsOfChange(sim, 0); // zero means no new changes
	}
}
void simSetPlugNeighborUpdatesSuspended (kSanSimulation *sim, short suspend)
{
	if (suspend) ++sim->suspendNeighborUpdates;
	else  
	{
		--sim->suspendNeighborUpdates;
		if (sim->suspendNeighborUpdates == false)
		calcNeighbors(sim); // zero means no new changes
	}
}
