#include "OHApplicationPublic.h"
#include "OHBufferedView.h"
#include "kSanSimulationPrivate.h"
#include "Profiler.h"

#define kIterCycleTicks 10
#define kTwoThirds 0.6666666666666666667
#define kOneThird 0.333333333333333333
#define kGoldenRatio 0.61

#define kOne 1.00
#define kOneAndOneThousandth 1.0010000000
#define kPointNineNineNine 0.999000000
#define kHalfOfOneMillionth 0.00000050
#define kOneThousandth 0.0010
#define kOneHalf 0.50
#define kTwoOverThreeK 7729.4686  // units are per eV
#define kOneHalfK (kTwoOverThreeK * 0.75)   // units are per eV
#define keyIterationParameter  'kItr' // units are per eV

// k is 1.38 E-23 J/K or  
// 8.625 E-5  eV/K  or  using  9636.8 ( (g/mol) A / ps ps ) per (eV /A)
// 0.831174 (g/mol) A A / ps ps K
// so 2over3K is 0.802078
// oneHalf kT is 0.601558759132 in units of    <null>   per  (g/mol) A / ps ps )


void  simYieldToWaiters(kSanSimulation *sim);

short  simDoIterateEvent(kSanSimulation *sim, AEMessageData *md )
{

	short	err;
	short clockWasRunning;
	DescType actualType;
	long actualSize;
	long iters;
	unsigned long targetIters;
// 	BBEditDebugString("start do iterate event");
	clockWasRunning = sim->flags.clockRunning;
	err = AEGetParamPtr(md->event, keyIterationMultiple, typeLongInteger, &actualType,  &iters, 4L, &actualSize);	
	if ((err)  || (actualSize != 4L))
	{
		iters = 1;
		err = noErr;
	}
		targetIters = sim->totalNumberOfIterations + iters;
		simRaiseYieldFlag	(sim,  targetIters);
		packageAndCallSetBool(simOBJ(sim), pClockRunning, true);
 		while ( sim->totalNumberOfIterations < targetIters)
		{
	 	//	BBEditDebugString("iterate event  yield");
			yieldToThread(kYieldNow);
	 	//	BBEditDebugString("iterate event return");
		}
		simRemoveYieldFlag	(sim, targetIters);
		packageAndCallSetBool(simOBJ(sim), pClockRunning, clockWasRunning);

 	//BBEditDebugString("end do iterate event");

	return (err);
}
short  updateEnergy(kSanSimulation *sim) 
{
	long firstPart, lastPart;
	short err = noErr;

	if (sim->lastEnergyCalcStateCount < sim->stateCount) 
	{
		err = calcNeighbors(sim); // includes fill screening arrays
		if (!err)
		{	
			firstPart = 0;
			lastPart = simGetNumParts(sim) - 1;
			err = sim->potentialPlugIn->manyPartECalc ( sim->potentialPlugIn, sim, firstPart, lastPart,  &sim->potEnergy) ;  
	 		if (err) err = simHandlePlugInError(sim, sim->potentialPlugIn->ppiObj, err);
			else 
			{	
			         sim->lastEnergyCalcStateCount = sim->stateCount;
				sim->lastEnergyCalcTicks = TickCount();
			}
			 
//			dispatchNilMessage(simOBJ(sim), kInvalWorldMessage);
		}
	}
	return (err);
}

pascal void  * iterationThread   (kSanSimulation *sim)
{
	long err = noErr;

	while ( kozoObjectNowConstructing(simOBJ(sim)))
	{
		yieldToThread(true); 
	}
	while (!sim->flags.done)
	{	
		if (sim->flags.clockRunning)
		{
			long gnErr;
			++sim->counters.redraw;
			++sim->counters.neighbor;
			sim->flags.iterationInProgress = true;	
			
			if (sim->counters.redraw >= sim->countThreshholds.redraw) 
			{
 				 updateEnergy(sim);  
 			}
			else 	if (sim->counters.neighbor >= sim->countThreshholds.neighbor)
			{
 				gnErr = calcNeighbors(sim); /** includes bitfields and swapping sides ****/
 			}
			else 
			{
 				gnErr =lookForNeighbors(sim);  
 			}
			if (sim->counters.redraw >= sim->countThreshholds.redraw) 
			{
 				simRedraw(sim);
 			}

	
 			++sim->counters.cellPressure; 
		 	if (sim->counters.cellPressure >= sim->countThreshholds.cellPressure)
 			{
  				sim->resizePlugIn->doCellResize( sim->resizePlugIn,  sim);  // True means change the cell size 
  				sim->counters.cellPressure = 0;
  				gnErr = calcNeighbors(sim);  // includes bitfields and swapping sides 
 			}	
 			yieldToThread(false); 
 
 			++sim->totalNumberOfIterations;
			sim->totalTimeElapsed += sim->deltaT;
 			sim->methodPlugIn->doIteration ( sim->methodPlugIn,  sim) ;  
 
 			++sim->counters.therm;
			if (sim->counters.therm >= sim->countThreshholds.therm)
			{
				double netTime;
				netTime = sim->countThreshholds.therm * sim->deltaT;

				err =sim->methodPlugIn->doCooling ( sim->methodPlugIn,  sim,  &netTime, &sim->kinEnergy) ;  
				if (sim->kinEnergy < 0) sim->kinEnergy = groupKinECalc(sim);
				if (err) packageAndCallSetBool(simOBJ(sim), pClockRunning, false);
				sim->counters.therm = 0;

				
			}
			else 
			{
				sim->kinEnergy = groupKinECalc(sim);
			}
			++sim->counters.velocityCentering;
			if (sim->counters.velocityCentering >= sim->countThreshholds.velocityCentering)
			{
				if(sim->doVCentering) stabilize(sim);
				sim->counters.velocityCentering = 0;
			}

			sim->flags.iterationInProgress = false;
			if (sim->waitingThreads.howManyItems >0) 
			{
				simYieldToWaiters(sim); 
			}
			
			++sim->counters.dataSave;
			if (sim->counters.dataSave >= sim->countThreshholds.dataSave)
			{
				dispatchNilMessage(simOBJ(sim), kSimContinuingBackupMessage);   // dont ask for file spec
				sim->counters.dataSave = 0;
			}
			
			++sim->counters.memoryCleanup;	
			if (sim->counters.memoryCleanup >= sim->countThreshholds.memoryCleanup)
			{
				compressNeighborMemory(sim);
				sim->counters.memoryCleanup = 0;
			}
			yieldToThread(false); 
			kSanSimCallActivePlugIns(sim);
			while (sim->flags.lowestYieldFlag == sim->totalNumberOfIterations)
			{
				if (sim->flags.lowestYieldFlag == 0xFFFFFFFF) ++sim->totalNumberOfIterations;
				yieldToThread(true); 
			}
			++sim->stateCount;
		}
		else // clock is not running:  check lazies -- i.e. if we need to update energy calc
		{
			if ((sim->stateCount > sim->lastEnergyCalcStateCount) && (sim->flags.automaticUpdate) && ((sim->lastEnergyCalcTicks + 120) <  TickCount())) 
			{
 				dispatchNilMessage(simOBJ(sim),  kInvalWorldsMessage );
 			}
 			yieldToThread(true); 
 		}
	}
	return ((void *)kThreadHasReturned);
}
/*****  CalculateForce 
*		calculate separation and pass rsqu value to doForceCalc     
*		use return to change E, dvx, dvy and dvz values *
*   
*  	dvx dvy and dvz values are set to zero 
*	after changing the velocities in the 
*	ChangeVelocities() function
***/

// tersoff force calc returns (g/mol) Angstroms / ps ps 
//meam returns (g/mol) Angstroms / ps ps 
//LJ returns (g/mol) Angstroms / ps ps 
short	 calculateForce (kSanSimulation *sim)
{
	long lowPart, highPart;
	short progressErr = noErr;
	lowPart = 0;
	highPart = simGetNumParts(sim) -1;

	progressErr = sim->potentialPlugIn->manyPartFCalc (  sim->potentialPlugIn,  sim,  lowPart, highPart) ;
	if (progressErr) 
	{
		if (progressErr !=  errAEWaitCanceled)
		{		
		    simHandlePlugInError(sim,  sim->potentialPlugIn->ppiObj, progressErr);
		}
	}
	else
	{
		progressErr = simDoProgressFunction (sim);	
		addForceToGroups(sim);
	}
	return (progressErr);
}

void  addForceToGroups(kSanSimulation *sim)
{
	OHList *gpList;
	particle * basePart;
	kSanVector  areas;
	
	gpList = getListOfObjectsOfClassX (simOBJ(sim), ClassParticleGroup);
	if (gpList)
	{
		getAreas(&sim->simCell, &areas);
		basePart = simLockParts(sim);
		 OHListEveryItemDoTwoArgs( gpList,  addGroupForces,  basePart, &areas);	
		  releaseListOfObjectsOfClassX(simOBJ(sim), ClassParticleGroup);
	}
	simReleaseParts(sim);
}

/************  DoTemperatureChange  subtract E from hot system ****************/
/*** temperature is calculated from 2/3 <E> / N k    ****/
/*** May 25 changing meaning of simInfo->cooling factor to thermal decay time (fs)  ****/

double  groupKinECalc(kSanSimulation *sim)
{
	 long i;
 	long numGroups;
 	kSanGroup **gpList = simLockGroups(sim) ;
	double kinenergy = 0.0;
	double groupKinE = 0.0;
	double tempFactor;
  	
  	dispatchCount (simOBJ(sim), ClassParticleGroup, &numGroups);

#ifndef __2DVectors__
    tempFactor = 	kTwoOverThreeK;
#else
    tempFactor = 	kOneHalfK;
#endif 

	simLockAtomTypes(sim);
 	if (gpList)
	{
	     for ( i = 0 ; i < numGroups; ++i)
	     {
 			kSanGroup *gp = gpList[i];
 			long groupParts = OHListCount (&gp->particles);
			
			groupKinE = getKineticEnergyOfList(sim, &gp->particles);  
			kinenergy  +=  groupKinE;
			gp->actualTemp = groupKinE * tempFactor / ((double)groupParts);
		 }
		 
 		sim->simTemperature = kinenergy * tempFactor / (double)simGetNumParts(sim);
	}
	
	simReleaseGroups(sim) ;
	simReleaseAtomTypes(sim);
	
	return (kinenergy);
}
double 	 getKineticEnergyOfList(kSanSimulation *sim, OHList *theList)
{
	long i;
	long partArrIndex;
	long count;
	//particle * thispart;
	particle * basePart;
	kSanAtomtype **types;
	kSanVector  *thisVel;
	double energy;
	double value;
	
	basePart = simLockParts(sim);
	types = simLockAtomTypes(sim);
	energy = 0.0;
	count = OHListCount(theList);
	for(i=0;i< count;++i)  // usually how many is the number of particles in the group
	{
		partArrIndex = getNthItem (theList, i);
		thisVel = &basePart->baseVelocity[partArrIndex];
		
		value = (thisVel->a * thisVel->a);
		value += (thisVel->b * thisVel->b);
		
#ifndef __2DVectors__
		value += (thisVel->c * thisVel->c); // now this is V squ
#endif

		energy += (types[BPNthType(basePart, partArrIndex)]->specs.mass  * value);	
	}
	energy *= (0.00005188444297); // this is 0.5 * ( 1 / 9636.8 )
			// converts from (g/mol) A A / ps ps  to  eV 
			// and multiplies by 1/2   as in 1/2 mv^2
	simReleaseParts(sim);
	simReleaseAtomTypes(sim);
	return (energy); // returns in eV
}
double 	 kinECalc(kSanSimulation *sim)
{
	long partArrIndex;
	long count;
	//particle * thispart;
	kSanVector  *thisVel;
	particle * basePart;
	kSanAtomtype **types;
	double energy;
	double value;
	
	basePart = simLockParts(sim);
	types = simLockAtomTypes(sim);
	energy = 0.0;
	count = simGetNumParts(sim);
	for(partArrIndex=0;partArrIndex< count;++partArrIndex)  // usually how many is the number of particles in the group
	{
		thisVel = &basePart->baseVelocity[partArrIndex];
		value = (thisVel->a * thisVel->a);
		value += (thisVel->b * thisVel->b);
#ifndef __2DVectors__
		value += (thisVel->c * thisVel->c); // now this is V squ
#endif
		energy += (types[BPNthType(basePart, partArrIndex)]->specs.mass  * value);	
	}
	energy *= (0.00005188444297); // this is 0.5 * ( 1 / 9636.8 )
			// converts from (g/mol) A A / ps ps  to  eV 
			// and multiplies by 1/2   as in 1/2 mv^2
	simReleaseParts(sim);
	simReleaseAtomTypes(sim);
	return (energy); // returns in eV
}



// doCellPressure -- check the x, y, and z axes for relaxation if not fixed

void 	 stabilize (kSanSimulation *sim)  // added july 31 
{
	short partArrIndex;
	double xSum;
	double ySum;
	double zSum;
	double totalmass;
	double thismass;
	double factor;
	//particle * thispart;
	kSanVector  *thisVel;
	particle * basePart;
	kSanAtomtype **baseTypes;
	basePart = simLockParts(sim);
	baseTypes = simLockAtomTypes(sim);
	xSum = 0.0;
	ySum = 0.0;
	zSum = 0.0;
	totalmass = 0.0;
	for(partArrIndex=0;partArrIndex<simGetNumParts(sim);++partArrIndex)
	{
 		thisVel = &basePart->baseVelocity[partArrIndex];
		thismass = baseTypes[BPNthType(basePart, partArrIndex)]->specs.mass;
		xSum += (thisVel->a * thismass);
		ySum += (thisVel->b * thismass);
#ifndef __2DVectors__
		zSum += (thisVel->c * thismass);
#endif
		totalmass += thismass;
	}
	
	factor = -1.0 / totalmass;
	xSum = xSum * factor;
	ySum = ySum * factor;
#ifndef __2DVectors__
	zSum = zSum * factor;
#endif

	for(partArrIndex=0;partArrIndex<simGetNumParts(sim);++partArrIndex)
	{
 		thisVel = &basePart->baseVelocity[partArrIndex];
		thisVel->a += xSum;
		thisVel->b += ySum;
#ifndef __2DVectors__
		thisVel->c += zSum;
#endif
	}
	simReleaseParts (sim);
	simReleaseAtomTypes(sim);
}

void  simYieldToWaiters(kSanSimulation *sim)
{
	OHList aList;
	char bugStr[256];
	long i, count;
	long thisThread;
	initializeList (&aList);
	duplicateList (&sim->waitingThreads, &aList);
	count = OHListCount(&aList);
	GetCurrentThread ((unsigned long *)&thisThread);
 //	BBEditDebugString("iter thread yield to waiters");
	for (i = 0; i < count; ++i)
	{
		long nextThread;
		nextThread = getNthItem(&aList, i);
		sprintf(bugStr, "iter waiter yield from thread %ld to thread %ld", thisThread, nextThread);
 	//	BBEditDebugString(bugStr);
		YieldToThread((ThreadID)nextThread);
		sprintf(bugStr, "iter waiter return to thread %ld from thread %ld", thisThread, nextThread);
 	//	BBEditDebugString(bugStr);
	}
 //	BBEditDebugString("iter thread end yield to waiters");
 	disposeList (&aList);
}


void  waitForIterationToFinish(kSanSimulation *sim)
{
	ThreadID myThread;
	GetCurrentThread (&myThread);
	if (myThread != sim->iterationThreadID)
	{
		addToList(&sim->waitingThreads, (long)myThread);
		while (sim->flags.iterationInProgress)
		{
		      //   BBEditDebugString("wait for finish yield");
			yieldToThread(kYieldNow); 
 		      //   BBEditDebugString("wait for finish return");
		} 
		removeItem(&sim->waitingThreads, (long)myThread);
	}
}

void  abortIteration(kSanSimulation *sim, short stopTime)
{
	sim->flags.abortNow = true;
	while(sim->flags.iterationInProgress)
	{
 		//  BBEditDebugString("abort iter yield");
		YieldToThread(sim->iterationThreadID);
 		//  BBEditDebugString("abort iter return");
	}
	sim->totalNumberOfIterations = sim->totalNumberOfIterations - 1;
	if (stopTime) packageAndCallSetBool(simOBJ(sim), pClockRunning, false);
 } 

