#include <Sound.h>
#include "kSanSimulationPrivate.h"
#include "OHSharedUtils.h"
#include "OHAlerts.h"
short  subtractNeighborFromNeighborList(kSanSimulation *sim,   particle * basePart, long partArrIndex, long neighborIndex);
short  lookForOneParticlesNeighbors( kSanSimulation *sim, particle * basePart, long partArrIndex, box *  boxes);
void  verifyNotInNeighborList( kSanSimulation *sim, particle * basePart, long partArrIndex);
short simAssignNeighborPtrs( kSanSimulation *sim,  particle *basePart, long extraRoom);
short simReallocateNeighborPtr( kSanSimulation *sim,  particle *basePart, long extraRoom);

short calcNeighborsFromScratch(kSanSimulation *sim)
{ 
	short err = noErr;
	particle *basePart;
	long   count;
	long partArrIndex;
	count = sim->howManyParts;
 	sim->flags.neighborOverrun = true; // should be set to false in simAssignNeighborPtrs
	basePart = simLockParts(sim);
	for (partArrIndex = 0 ; partArrIndex < count; ++partArrIndex)
	{
		basePart->basePi[partArrIndex].neighborInfoDirty = true;
		basePart->basePi[partArrIndex].neighborScreeningDirty = true;
		basePart->basePi[partArrIndex].neighborSpaceAvailable = 0;
 		basePart->basePi[partArrIndex].extraNumNeighbors = 0;
 		basePart->basePi[partArrIndex].actualNumNeighbors = 0;
	}
	simReleaseParts(sim);
 	return(calcNeighbors(sim));
}
short resetNeighborPtrs(kSanSimulation *sim, long extraRoom)
{ 
	short err = noErr;
	particle *basePart;
	
 	basePart = simLockParts(sim);
	err =  simReallocateNeighborPtr(sim, basePart, extraRoom);
	simReleaseParts(sim);
 
	if (!err) err = lookForNeighbors(sim);
	return (err);
}
short simReallocateNeighborPtr(kSanSimulation *sim, particle *basePart, long extraRoom)
{
	short err = noErr;
	short memError = noErr;
	char done = false;
	long partArrIndex, count;
	long numTotal = 0;
  	long newSize;
 	Ptr currentPtr;
	Ptr oldPtr;
	count = sim->howManyParts;
	
	for (partArrIndex = 0 ; partArrIndex < count; ++partArrIndex)
	{
		basePart->basePi[partArrIndex].extraNumNeighbors += basePart->basePi[partArrIndex].actualNumNeighbors;
		basePart->basePi[partArrIndex].actualNumNeighbors = 0;
		numTotal += basePart->basePi[partArrIndex].extraNumNeighbors;
	}
 	sim->recentAverageNeighborSlots = (numTotal / sim->howManyParts) + 1 + extraRoom;
	numTotal =  ((((float)sim->howManyParts) * 1.05) + 16) * sim->recentAverageNeighborSlots ;
 	newSize = ((numTotal) * sizeof(NNeighborInfo))  ;
 	newSize = 128 + ((((long)newSize)/1024) + 1) * 1024;
 
	 oldPtr = sim->neighborsPtr;
	 sim->neighborsPtr = NewPtr(newSize);
	 if (sim->neighborsPtr) 
	 {
	 	DisposePtr(oldPtr);
	    memError = MemError();
	 }
	 else
	 {
	    sim->neighborsPtr = oldPtr;
	    memError = true;
	 }
	
	// used to be assign neighbor ptrs
	if ((sim->neighborsPtr) && (!memError))
	{
		
		while ((!memError) && (!done))
		{
			long remainingSpace;
		    currentPtr = sim->neighborsPtr;
			remainingSpace = newSize / sizeof(NNeighborInfo); 
			
			sim->neighborSpaceFrontSlack = 100;
			currentPtr +=  100 *  sizeof(NNeighborInfo);
			remainingSpace = remainingSpace - 100;
			
		 	for (partArrIndex = 0 ; partArrIndex < count; ++partArrIndex)
			{
				basePart->basePi[partArrIndex].neighborInfoDirty = true;
				basePart->basePi[partArrIndex].neighborScreeningDirty = true;
				basePart->basePi[partArrIndex].firstNeighborInfo = (NNeighborInfo *)currentPtr;
				basePart->basePi[partArrIndex].neighborSpaceAvailable = (basePart->basePi[partArrIndex].extraNumNeighbors + extraRoom);
				if (basePart->basePi[partArrIndex].neighborSpaceAvailable > 4096) 
				{
					basePart->basePi[partArrIndex].neighborSpaceAvailable = 4096;
					OHSystemBeep();
				}
		 		basePart->basePi[partArrIndex].extraNumNeighbors = 0;
		 		basePart->basePi[partArrIndex].actualNumNeighbors = 0;
		 		currentPtr += (basePart->basePi[partArrIndex].neighborSpaceAvailable) *  sizeof(NNeighborInfo);
		 		remainingSpace -= basePart->basePi[partArrIndex].neighborSpaceAvailable;
			} 
			
			if (count < 1)
   			{
 			    sim->neighborSpaceFrontSlack += remainingSpace;
			    done = true;
 			}
 			else if  (remainingSpace > 0)  
			{
				basePart->basePi[count - 1].neighborSpaceAvailable += remainingSpace;
			    sim->flags.neighborOverrun = false ;
			    done = true;
			}
			else 
			{
			   // for some reason the allocated neighbor ptr was too small
			     newSize = ((float)newSize) * 1.1;
  				 newSize = 128 + ((((long)newSize)/1024) + 1) * 1024;
  				 oldPtr = sim->neighborsPtr;
				 sim->neighborsPtr = NewPtr(newSize);
				 if (sim->neighborsPtr) 
				 {
				 	DisposePtr(oldPtr);
				    memError = MemError();
				 }
				 else
				 {
				    sim->neighborsPtr = oldPtr;
				    memError = true;
				 }
  			}
		}
	}
   	if (memError)
   	{
   		err = memError;
		OHSystemBeep();
	}
	return (err);
}
short kSanNeighborsConstructor( kSanSimulation *sim,  particle *basePart, long partArrIndex)
{
	short err = noErr;
	long newSpace = 0;
	basePart->basePi[partArrIndex].neighborInfoDirty = true;
	basePart->basePi[partArrIndex].neighborScreeningDirty = true;
	basePart->basePi[partArrIndex].extraNumNeighbors = 0;
	basePart->basePi[partArrIndex].actualNumNeighbors = 0;
	
	if (partArrIndex == 0) 
	{
		if (sim->neighborSpaceFrontSlack > 48) newSpace = 24;
		else if (sim->neighborSpaceFrontSlack > 0) newSpace = sim->neighborSpaceFrontSlack / 2;
    	sim->neighborSpaceFrontSlack = sim->neighborSpaceFrontSlack - newSpace;
        basePart->basePi[partArrIndex].neighborSpaceAvailable = newSpace; 
        basePart->basePi[partArrIndex].firstNeighborInfo = (NNeighborInfo *) (sim->neighborsPtr + (sim->neighborSpaceFrontSlack * sizeof(NNeighborInfo)));
 	    
	}
	else 
	{
	    long freeSpace;
	    freeSpace = basePart->basePi[partArrIndex-1].neighborSpaceAvailable - basePart->basePi[partArrIndex-1].actualNumNeighbors;
		if  (freeSpace > sim->recentAverageNeighborSlots ) newSpace = freeSpace  - sim->recentAverageNeighborSlots;
		else newSpace = freeSpace / 2;
		basePart->basePi[partArrIndex-1].neighborSpaceAvailable -=  newSpace;
	    basePart->basePi[partArrIndex].neighborSpaceAvailable = newSpace;
	    basePart->basePi[partArrIndex].firstNeighborInfo = &basePart->basePi[partArrIndex - 1].firstNeighborInfo[basePart->basePi[partArrIndex - 1].neighborSpaceAvailable];
		
	}

	return (err);
}
void kSanNeighborsDestructor( kSanSimulation *sim,  particle *basePart, long partArrIndex)
{
	if (partArrIndex == 0)
	{
 		sim->neighborSpaceFrontSlack += basePart->basePi[partArrIndex].neighborSpaceAvailable;
 	}
	else 
	{
		basePart->basePi[partArrIndex - 1].neighborSpaceAvailable += basePart->basePi[partArrIndex].neighborSpaceAvailable;
	}
	basePart->basePi[partArrIndex].extraNumNeighbors = 0;
	basePart->basePi[partArrIndex].actualNumNeighbors = 0;
	basePart->basePi[partArrIndex].neighborSpaceAvailable = 0;
}

short  simNumNeighbors(kSanSimulation *sim, particle *basePart, long partArrIndex)
{
#pragma unused(sim)
	return (basePart->basePi[partArrIndex].actualNumNeighbors);
}
short simRemoveNthNeighbor (kSanSimulation *sim, particle *basePart, long partArrIndex, long whichNeighbor)
{
#pragma unused(sim)
 	if ((basePart->basePi[partArrIndex].actualNumNeighbors > 1) && (basePart->basePi[partArrIndex].actualNumNeighbors > whichNeighbor)) 
	{
		NNeighborInfo *nni;
		nni = basePart->basePi[partArrIndex].firstNeighborInfo;
		basePart->basePi[partArrIndex].actualNumNeighbors = basePart->basePi[partArrIndex].actualNumNeighbors - 1;
		nni[whichNeighbor] = nni[basePart->basePi[partArrIndex].actualNumNeighbors];
	}
	else // why do we get here?
	{
		OHSystemBeep();
	}
	return (noErr);
}


short  findWhichNeighbor(kSanSimulation *sim, long partArrIndex, long neighborIndex, OHList *theNNDataIndexList)
{ // theNNDataIndex must point to an array of NUMNEIGHBORINGBOXES shorts, so that each instance of the the neighbor (i.e. in 
  // each of the neighboring cells) can be returned
	particle *basePart;
 	short howManyNeighbors;
	short i = 0;
	
	basePart = simLockParts(sim);
  	howManyNeighbors = numNNs( basePart, partArrIndex);
	while (i < howManyNeighbors) // this must start from zero and go up!!!!
	{
		if (NNIGetNthIndex(sim, basePart, partArrIndex, i) == neighborIndex) 
		{
			addToList(theNNDataIndexList, i);  // make a list of the index of the neighbor in  the neighbor info list
		}
		++i;
	}
	simReleaseParts(sim);
	return (OHListCount(theNNDataIndexList));
}

short  resetNeighborInfoForOnePart(kSanSimulation *sim, long partArrIndex)
{
	short err = noErr;
	short howManyNeighbors;
	long neighborIndex;
	particle * basePart;
   
 	if (sim->suspendNeighborUpdates == 0)
	{
		basePart = simLockParts(sim);
		//thisPart = &(basePart[partArrIndex]);
		//neighbors = &thisPart->neighbors;

		howManyNeighbors = numNNs(basePart, partArrIndex);
 		while (howManyNeighbors>0) 
		{
			howManyNeighbors = howManyNeighbors - 1;
			neighborIndex = NNIGetNthIndex(sim, basePart, partArrIndex, howManyNeighbors) ;
			subtractNeighborFromNeighborList(sim,  basePart,  neighborIndex,  partArrIndex);
			 // if a neighbor appears multiple times in the list, this function will get called
			 // multiple times for that neighborIndex
		}
		
		// verifyNotInNeighborList(sim, basePart, partArrIndex);

		if (!err) err = sim->setBoxForPart(sim, partArrIndex); // includes wrap!
		if (!err) 
		{
			box * boxes;
			boxes = simLockBoxes(sim);
			err = lookForOneParticlesNeighbors(sim,  basePart, partArrIndex,  boxes );
			simReleaseBoxes(sim);
		} 
		simReleaseParts(sim);
		
		if (err) 
		{
			err = calcNeighbors(sim);
		}
	}
    return (err);
}
void  verifyNotInNeighborList(kSanSimulation *sim, particle * basePart, long partArrIndex)
{
	long partArrIndexLoop;
	long neighborLoop;
	NNeighborInfo  * nni;
	for (partArrIndexLoop = 0; partArrIndexLoop < simGetNumParts(sim); ++partArrIndexLoop)
	{
		nni = NNIGetNNI(sim, basePart, partArrIndexLoop);  
		for (neighborLoop = 0; neighborLoop < basePart->basePi[partArrIndexLoop].actualNumNeighbors; ++neighborLoop)
		{  
			if (partArrIndex  ==  nniGetNthIndex(nni, neighborLoop)) OHSystemBeep();
		}	
	}
}
short  resetNeighborInfoForTwoParts(kSanSimulation *sim, long partA, long partB)
{
	short err = noErr;
	short localErr = noErr;
	long i;
	long howManyNeighborsA;
	long howManyNeighborsB;
	long neighborIndex;
	box * boxes;
	particle * basePart;
//	particle * A;
//	particle * B;
//	kSanNeighbors * aNeighbors;
//	kSanNeighbors * bNeighbors;
	NNeighborInfo  * nniA;
	NNeighborInfo  * nniB;
	
	
	if (sim->suspendNeighborUpdates == 0)
	{
		basePart = simLockParts(sim);
		//A = &(basePart[partA]);
		//B = &(basePart[partB]);
		//aNeighbors = &A->neighbors;
		//bNeighbors = &B->neighbors;

	// clean out partA's neighbor list
		howManyNeighborsA = numNNs(basePart, partA);
		nniA = basePart->basePi[partA].firstNeighborInfo;

		i = howManyNeighborsA;
		while (i>0)
		{
			--i;
			// neighborIndex = nniA[i].nIndex;
			 neighborIndex = nniGetNthIndex(nniA, i);
			err = subtractNeighborFromNeighborList(sim,  basePart,  neighborIndex,  partA);
			if (err) localErr = true;
			 // if a neighbor appears multiple times in the list, this function will get called
			 // multiple times for that neighborIndex
		}
		verifyNotInNeighborList(sim, basePart, partA);
	// this is the key advantage of the TwoParts algorithm -- set the boxes before looking for parts
		err = sim->setBoxForPart(sim, partA); // includes wrap!
		if (err) localErr = true;
		err = sim->setBoxForPart(sim, partB); // includes wrap!
		if (err) localErr = true;
		
		boxes = simLockBoxes(sim);
	// look for As neighbors	
		err = lookForOneParticlesNeighbors(sim,  basePart, partA,  boxes );
		if (err) localErr = true;
			
	// clean out partB's neighbor list
		howManyNeighborsB = numNNs(basePart, partB);
		nniB = basePart->basePi[partB].firstNeighborInfo;
		i = howManyNeighborsB;
		while (i>0)
		{
			--i;
			//neighborIndex = nniB[i].nIndex;
			 neighborIndex = nniGetNthIndex(nniB, i);
			 
			err = subtractNeighborFromNeighborList(sim,  basePart,  neighborIndex,  partB);
			if (err) localErr = true;
			 // if a neighbor appears multiple times in the list, this function will get called
			 // multiple times for that neighborIndex
		}
		verifyNotInNeighborList(sim, basePart, partB);
		
	// look for Bs neighbors	
		err = lookForOneParticlesNeighbors(sim,  basePart, partB,  boxes );
		if (err) localErr = true;	
		
		simReleaseBoxes(sim);
		simReleaseParts(sim);
		
		if (localErr) 
		{
			err = calcNeighbors(sim);
		}
	}
    return (err);
}
short  resetNeighborInfoForNParts(kSanSimulation *sim, OHList *list)
{
	short err = noErr;
	short localErr = noErr;
	long i;
	long n;
	long count;
	long howManyNeighbors;
	long neighborIndex;
	long partArrIndex;
	box * boxes;
	particle * basePart;
	NNeighborInfo  * nni;
	
	count = OHListCount(list);
	
	if (sim->suspendNeighborUpdates == 0)
	{
		basePart = simLockParts(sim);

		for (i = 0; i < count; ++i)
		{
			partArrIndex = getNthItem(list, i);
			howManyNeighbors = numNNs( basePart, partArrIndex );
			nni = basePart->basePi[partArrIndex].firstNeighborInfo;
			n = howManyNeighbors;
			while (n > 0)
			{
				--n;
				//neighborIndex = nni[n].nIndex;
				neighborIndex = nniGetNthIndex(nni, n);

				err = subtractNeighborFromNeighborList(sim,  basePart,  neighborIndex,  partArrIndex);
				if (err) localErr = true;
				 // if a neighbor appears multiple times in the list, this function will get called
				 // multiple times for that neighborIndex
			}
			basePart->basePi[partArrIndex].actualNumNeighbors = 0;
			// now part nthListItem has been removed from all its neighbors lists and 
			// has an empty neighbor list of its own;
		}
	// set all boxes
	
		for (i = 0; i < count; ++i)
		{
			partArrIndex = getNthItem(list, i);
			err = sim->setBoxForPart(sim, partArrIndex); // includes wrap!
		}
	
		boxes = simLockBoxes(sim);
		for (i = 0; i < count; ++i)
		{
			partArrIndex = getNthItem(list, i);
			err = sim->setBoxForPart(sim, partArrIndex); // includes wrap!
		}
		err = lookForParticleListsNeighbors(sim,  basePart, list,  boxes );
		
		simReleaseBoxes(sim);
		simReleaseParts(sim);
		
		if (localErr) 
		{
			err = calcNeighbors(sim);
		}
	}
    return (err);
}

short  subtractNeighborFromNeighborList(kSanSimulation *sim, particle * basePart, long partArrIndex, long neighborIndex)
{   
#pragma unused(sim)
	short err = noErr;
 	short found ;	
	long count;	
	long howManyNeighbors;
	NNeighborInfo  * nni;
	
	nni = basePart->basePi[partArrIndex].firstNeighborInfo;
	howManyNeighbors = numNNs( basePart, partArrIndex);
	count = 0;
	found = false;
	while ((count < howManyNeighbors) && (!found))
	{
		if (nniGetNthIndex(nni, count) == neighborIndex) found = true;
		else ++count;
	}
	if (found)
	{
		//simRemoveNthNeighbor (sim,  basePart, partArrIndex, count);
		// begin simRemoveNthNeighbor
 		if (count == basePart->basePi[partArrIndex].actualNumNeighbors - 1) // includes case of (neighbors->actualNumNeighbors == 1)
		{
			basePart->basePi[partArrIndex].actualNumNeighbors = basePart->basePi[partArrIndex].actualNumNeighbors - 1;
		}
		else if ((basePart->basePi[partArrIndex].actualNumNeighbors > 1) && (basePart->basePi[partArrIndex].actualNumNeighbors > count)) 
		{
			basePart->basePi[partArrIndex].actualNumNeighbors = basePart->basePi[partArrIndex].actualNumNeighbors - 1;
			basePart->basePi[partArrIndex].firstNeighborInfo[count] = basePart->basePi[partArrIndex].firstNeighborInfo[basePart->basePi[partArrIndex].actualNumNeighbors];
		}
		else // why do we get here?
		{
			OHSystemBeep();
		}
	 	basePart->basePi[partArrIndex].neighborScreeningDirty = true;	 	
		// end simRemoveNthNeighbor
	} // end if found
	else
	{  // its possible that a neighbor can't be found if we have overrun the memory available when calling addNNData
		if (!sim->flags.neighborOverrun ) OHSystemBeep();
	}
	
    return (err);
}

short   getAllSeparations(kSanSimulation *sim, long thisPartIndex, long nPartIndex, 
					kSanVector * mSepVecArray, double * mRadSquArray)
{
	short whichSep = 0;
	short localBoxIndex;
	long tBox;
	long nBox;
	char  boxRelations;
	char  boxPositions;
	char  translationFlags;
	char  unused;
#pragma unused(unused) 
	particle *basePart;
	box * thisBox;
	box * boxes;
	
	basePart = simLockParts(sim);
	boxes = simLockBoxes(sim);
	tBox = basePart->basePi[thisPartIndex].boxIndex;  // box of this part
	thisBox = &boxes[tBox];
 	nBox = basePart->basePi[nPartIndex].boxIndex;   // box of nPart
    boxPositions =  setBoxPositionFlags(sim, thisBox);

// loop over all NUMNEIGHBORINGBOXES neighboring boxes of thisPart, and see if it contains nPart
// if it does contain nPart, calculate the sep and sepVector, and store.  Incrementing howMany

	for (localBoxIndex = 0; localBoxIndex < NUMNEIGHBORINGBOXES; ++localBoxIndex)
	{
	     if (thisBox->neighboringBoxes[localBoxIndex] == nBox) // neighboring box index is the box which nPart is in
	     {
	          boxRelations = setBoxRelationFlags(localBoxIndex);  
	          translationFlags = setTranslationFlags(boxRelations, boxPositions);
      
	          mRadSquArray[whichSep] = sim->getSepSquared(sim, basePart, thisPartIndex, nPartIndex, translationFlags, &mSepVecArray[whichSep]);
	          ++whichSep;	          
	     }
	}
    simReleaseParts(sim);
    simReleaseBoxes(sim);
	return (whichSep);
}
short   getOneSepAndScreening(kSanSimulation *sim, particle * basePart, long oneIndex, long twoIndex , double * theSep, kSanVector * theSepVec, double * theScreening)
{ // calc for one pair of particles
	short howManySeps;
	short err = noErr;
	kSanVector  mSepVecArray[NUMNEIGHBORINGBOXES];
	double  mRadArray[NUMNEIGHBORINGBOXES];
	double  mScreenArray[NUMNEIGHBORINGBOXES];

//	mSepVecArray = (kSanVector *)NewPtr(sizeof(kSanVector ) * NUMNEIGHBORINGBOXES);
//	mRadArray = (double *)NewPtr(sizeof(double) * NUMNEIGHBORINGBOXES);
//	mScreenArray = (double *)NewPtr(sizeof(double) * NUMNEIGHBORINGBOXES);

	howManySeps = getAllSepsAndScreenings(sim, basePart, oneIndex, twoIndex, 
				mSepVecArray, mRadArray, mScreenArray);
	*theSep = mRadArray[0];	
	*theSepVec = mSepVecArray[0];	
	*theScreening = mScreenArray[0];	
	if (howManySeps == 0)
	{
		err = true;
	}
	else
	{
	    err = false;
	}
//	DisposePtr ((Ptr)mSepVecArray);
//	DisposePtr ((Ptr)mRadArray);
//	DisposePtr ((Ptr)mScreenArray);
         return (err);
}
short   getAllSepsAndScreenings(kSanSimulation *sim, particle *basePart, long thisPartIndex, long nPartIndex, 
					kSanVector * mSepVecArray, double * mRadArray, double * mScreenArray)
{
	short whichSep = 0;
	short localBoxIndex;
	long tBox;
	long nBox;
	char  boxRelations;
	char  boxPositions;
	char  translationFlags;
	char  unused;
#pragma unused(unused) 
	box *thisBox;
	box * boxes;
	
	boxes = simLockBoxes(sim);
	
	tBox = basePart->basePi[thisPartIndex].boxIndex;
	thisBox = &boxes[tBox];
 	nBox = basePart->basePi[nPartIndex].boxIndex;
   boxPositions = setBoxPositionFlags(sim, thisBox );
    
    mScreenArray[1] = 0;  // this will be changed if the pair were not in adjacent boxes

// loop over all NUMNEIGHBORINGBOXES neighboring boxes of thisPart, and see if it contains nPart
// if it does contain nPart, calculate the sep and sepVector, and store.  Incrementing howMany

	for (localBoxIndex = 0; localBoxIndex < NUMNEIGHBORINGBOXES; ++localBoxIndex)
	{
	     if (thisBox->neighboringBoxes[localBoxIndex] == nBox) // part is in this box
	     {
	     	   double sepSqu;
	           boxRelations = setBoxRelationFlags( localBoxIndex ) ; 
	           translationFlags = setTranslationFlags(boxRelations, boxPositions);
        
	            sepSqu = sim->getSepSquared(sim, basePart, thisPartIndex, nPartIndex, translationFlags, &mSepVecArray[whichSep]);
	            mRadArray[whichSep] = sqrt(sepSqu);
	            ++whichSep;
	     } else {}
	}
	if (whichSep != 0)
	{
	    short  i, j;
		OHList  theNNDataIndexList;
		
		initializeList(&theNNDataIndexList);
		
		findWhichNeighbor(sim, thisPartIndex, nPartIndex,  &theNNDataIndexList);
		// for each entry in mSepVecArray[whichSep], see if an entry exists in the NNData Table
		for (i=0; i<whichSep; ++i)
		{ // i is index of mRadArray, mSepVecArray and mScreenArray
			short foundIt = false;
			short howManyIndices;
			short j = 0;
			short theNNDataIndex;
			double tolerance = 0.01;
			kSanVector  vec;
			howManyIndices = OHListCount(&theNNDataIndexList);
			while (( j<howManyIndices ) && (!foundIt))
			{  // j is index in theNNDataIndex array
			    theNNDataIndex = getNthItem(&theNNDataIndexList, j);
			     NNIGetNthVec(sim, basePart, thisPartIndex, theNNDataIndex, &vec);
			    if (closekSanVectorsD(&vec, &mSepVecArray[i], &tolerance))
			    {
			         foundIt = true;
			         mScreenArray[i] = NNIGetNthScreening(sim, basePart, thisPartIndex, theNNDataIndex);
			    } else {}
			    ++j;
			}
			if (foundIt == false) mScreenArray[i] = 0;
			else {}
		}
		disposeList(&theNNDataIndexList);

		// now sort by separation
		for (i=0; i<whichSep; ++i)
		{
			for (j=i+1; j<whichSep; ++j)
			{
			    if (mRadArray[i] > mRadArray[j])
			    {
			        swapDoubles( &mRadArray[i], &mRadArray[j]);
			        swapDoubles( &mScreenArray[i], &mScreenArray[j]);
			        swapkSanVectorsD( &mSepVecArray[i], &mSepVecArray[j]);
			    }
			}
		}
	}
	else if ((thisPartIndex < simGetNumParts(sim)) && ( nPartIndex < simGetNumParts(sim)))  // provide one result
	{
	    subkSanVectorsD( &basePart->basePosition[thisPartIndex], &basePart->basePosition[nPartIndex], &mSepVecArray[0]);
	    mScreenArray[0] = 0;
	    mScreenArray[1] = -1; // signal that this was not in a neighboring box
	    mRadArray[0] = lengthOfkSanVectorD (&mSepVecArray[0]);
	    ++whichSep;
	}
	else // bad part info
	{
		zerokSanVectorD( & mSepVecArray[0] );
	    mScreenArray[0] = -1;
	    mRadArray[0] = 0;
	}
     simReleaseBoxes(sim);
	 return (whichSep);
}

short 	 wrapAroundAndBitfields (kSanSimulation *sim)
{   // wrap includes setting box info
	short err;

	err = sim->wrapAround(sim);  // this also updates box info
	if (err != 0) 
	{
		 if (err == wrapAroundHyperspaceErr)
		{
//			doAlert("\p Some particles have been deleted because they are too far outside the simulation cell.");
//			deleteHyperspaceParticles(sim);
			err = sim->wrapAround(sim);		
			if (err) sim->flags.error = wrapAroundHyperspaceErr;	
		}
	}	
	return (err);
}
short  zeroNeighborInfo (kSanSimulation *sim, long partArrIndex)
{
	short err = noErr;
	particle * basePart;
	
	basePart = simLockParts(sim);
	basePart->basePi[partArrIndex].actualNumNeighbors = 0;
	simReleaseParts(sim);
	return (err);
}
short  addParticleToBox (kSanSimulation *sim, particle *basePart,  long partArrIndex, long whichBox )
{ // this does not remove particle from its old box
	short err = noErr;
	box * boxes;
	
	boxes = simLockBoxes(sim);
	
	err = addPartToBox (sim, basePart,  partArrIndex,  whichBox, boxes);
		
	simReleaseBoxes(sim);
	return (err);
}
short  addPartToBox (kSanSimulation *sim, particle *basePart,  long partArrIndex, long whichBox, box * boxes)
{
	short err = noErr;
	box * thisBox;
	
	if ((whichBox > 0) && (whichBox <= sim->howManyBoxes))
	{ 
 		thisBox = &boxes[whichBox];
		BPNthBoxIndex(basePart, partArrIndex) = whichBox;	
		addToList (&thisBox->boxPartList, partArrIndex);
	}
	else   // box 0 keeps no particles in it, but any particle can think it belogs to box 0
	{   // particle is outside of cell -> bad particle
		// OHSystemBeep(); this is not an error, because the particle constructor's default is inf, inf, inf
		BPNthBoxIndex(basePart, partArrIndex) = 0;				
	}
	return (err);
}
short  lookForOneParticlesNeighbors(kSanSimulation *sim, particle * basePart, long partArrIndex, box * boxes)
{ // returns success
// this must be called only after the neighboring atom data has been cleared
	short err = noErr;
	short innerBoxLoop;
	short innerJackLoop;
	short innerJackLoopLimit;
	short innerBoxLoopLabel;
	long j;
	box * thisBox;
	box * thatBox;
	long * thatBoxIndexPtr;
	char  boxRelations;
	char  boxPositions;
	char  translationFlags;
	char  unused;
#pragma unused(unused) 

	double neighborCutoffSqu;
	double radSqu;
	kSanVector  sepVec;

	neighborCutoffSqu = ((sim->outerCutoff + sim->neighborCutoffExtra) 
	   * (sim->outerCutoff + sim->neighborCutoffExtra));  // need to change to real var

	thisBox = &boxes[BPNthBoxIndex(basePart, partArrIndex)];
   boxPositions = setBoxPositionFlags(sim, thisBox );
 	
	if (partArrIndex > simGetNumParts(sim))
	{
		OHSystemBeep();
		doAlert("\pindex out of range");		
	}
	
	basePart->basePi[partArrIndex].actualNumNeighbors = 0;

 	for (innerBoxLoop=0;innerBoxLoop<NUMNEIGHBORINGBOXES;++innerBoxLoop) 	// new 27 boxes nov 14 1996
	{					// loop over all NUMNEIGHBORINGBOXES boxes
		innerBoxLoopLabel = thisBox->neighboringBoxes[innerBoxLoop];
		thatBox = &boxes[innerBoxLoopLabel];
 		 boxRelations = setBoxRelationFlags ( innerBoxLoop);
	     translationFlags = setTranslationFlags( boxRelations,  boxPositions);

		innerJackLoopLimit = thatBox->boxPartList.howManyItems;
		thatBoxIndexPtr = lockList(&thatBox->boxPartList);
	 	for (innerJackLoop=0; innerJackLoop<innerJackLoopLimit; ++innerJackLoop) 	// loop over particles in box i		
		{					// loop over potential neighbors in box innerBoxLoop
			j = thatBoxIndexPtr[innerJackLoop];
			if (j!= partArrIndex)
			{
				radSqu = sim->getSepSquared(sim, basePart, partArrIndex, j, translationFlags, &sepVec);
				if (radSqu < neighborCutoffSqu)
				{
					if (addNNData(sim, basePart, partArrIndex, j, translationFlags, &radSqu, &sepVec)) err = true;
				}
			}
		}
		releaseList(&thatBox->boxPartList);
	}
		
	return (err);	
}
short  lookForParticleListsNeighbors(kSanSimulation *sim, particle * basePart, OHList *list, box * boxes)
{ 
 // be careful not to add found neighbors twice.  i.e. if a neighbor is found which is also part of the list, 
 // only add it if nIndex > thisIndex;
 
 // this should only be called if all the particles in the list have empty neighbor lists
 short err = noErr;
	short innerBoxLoop;
	short innerJackLoop;
	short innerJackLoopLimit;
	short innerBoxLoopLabel;
	long j;
	long i; 
	long count;
	long partArrIndex;
	box * thisBox;
	box * thatBox;
	long * thatBoxIndexPtr;
	char  boxRelations;
	char  boxPositions;
	char  translationFlags;
	char  unused;
#pragma unused(unused) 

	double neighborCutoffSqu;
	double radSqu;
	kSanVector  sepVec;

	neighborCutoffSqu = ((sim->outerCutoff + sim->neighborCutoffExtra) 
	   * (sim->outerCutoff + sim->neighborCutoffExtra));  // need to change to real var

	count = OHListCount(list);
	for (i = 0; i < count; ++i)
	{
		partArrIndex = getNthItem (list, i);
		
		basePart->basePi[partArrIndex].actualNumNeighbors = 0;
	}
	for (i = 0; i < count; ++i)
	{
		partArrIndex = getNthItem (list, i);
		
		thisBox = &boxes[BPNthBoxIndex(basePart, partArrIndex)];
         boxPositions =  setBoxPositionFlags(sim, thisBox);
		
	 	for (innerBoxLoop=0;innerBoxLoop<NUMNEIGHBORINGBOXES;++innerBoxLoop) 	// new 27 boxes nov 14 1996
		{					// loop over all 27 boxes
			innerBoxLoopLabel = thisBox->neighboringBoxes[innerBoxLoop];
			thatBox = &boxes[innerBoxLoopLabel];
 			boxRelations = setBoxRelationFlags (innerBoxLoop);
	        translationFlags = setTranslationFlags(boxRelations, boxPositions);

			innerJackLoopLimit = thatBox->boxPartList.howManyItems;
			thatBoxIndexPtr = lockList(&thatBox->boxPartList);
		 	for (innerJackLoop=0; innerJackLoop<innerJackLoopLimit; ++innerJackLoop) 	// loop over particles in box i		
			{					// loop over potential neighbors in box innerBoxLoop
			    
				j = thatBoxIndexPtr[innerJackLoop];
				if (j != partArrIndex)
				{
					radSqu = sim->getSepSquared(sim, basePart, partArrIndex, j, translationFlags, &sepVec);
					if (radSqu < neighborCutoffSqu)
					{
						if ((j > partArrIndex)  || (whichItem (list, j) == -1))
						{
								// add the data if the neighboring index is not in the list, or if 
								// it is in the list, check the j > partArrIndex criterion
							if (addNNData(sim,  basePart, partArrIndex, j, translationFlags, &radSqu, &sepVec)) err = true;
						}   
					}
				}
			}
			releaseList(&thatBox->boxPartList);
		}
	}	
		
	return (err);	
}
char  setBoxPositionFlags(kSanSimulation *sim, box *aBox)
{ 
	char boxPositionFlags = 0;

         // clear the bits we are replacing i.e. 1 - 6
	if (aBox->Xindex == (sim->boxDim.a - 1)) boxPositionFlags += kSanBaseBoxIsMaxX;
	if (aBox->Xindex == 0) boxPositionFlags += kSanBaseBoxIsMinX;

	if (aBox->Yindex == (sim->boxDim.b - 1)) boxPositionFlags += kSanBaseBoxIsMaxY;
	if (aBox->Yindex == 0) boxPositionFlags += kSanBaseBoxIsMinY;

#ifndef __2DVectors__

	if (aBox->Zindex == (sim->boxDim.c - 1)) boxPositionFlags += kSanBaseBoxIsMaxZ;
	if (aBox->Zindex == 0) boxPositionFlags += kSanBaseBoxIsMinZ;
#else 
    aBox->Zindex = 0;
#endif

	    // note that for a single box cell, the value will be 
	    // kSanBaseBoxIsMinZ + kSanBaseBoxIsMaxZ


	   return (boxPositionFlags);
}
char   setBoxRelationFlags( short easierIndex )
{  // index is a number between 0 and 26
	char boxRelationFlags = 0;
	char unused;
#pragma unused(unused)
	short xCord, yCord, zCord;

#ifdef __2DVectors__ 
#pragma unused (zCord)
#endif

#ifndef __2DVectors__ 
	xCord = (easierIndex / 9) ;
	easierIndex = easierIndex - xCord * 9;
	yCord = (easierIndex / 3) ;
	zCord = easierIndex - yCord * 3;
	boxRelationFlags =  (xCord * kSanNeighboringBoxAtSameX) +
    					 (yCord * kSanNeighboringBoxAtSameY) +
    					  (zCord * kSanNeighboringBoxAtSameZ) ; 
#else
	xCord = (easierIndex / 3) ;
	yCord = easierIndex - xCord * 3;
 	boxRelationFlags =  (xCord * kSanNeighboringBoxAtSameX) +
    					 (yCord * kSanNeighboringBoxAtSameY)  ; 
#endif

    return (boxRelationFlags);
    
}
char  setTranslationFlags(char boxRelations, char boxPositions)
{
	char translationFlags = 0;
	char relFlag;
     if (!(boxPositions & kSanMaskNotMiddleXBox))
    {
    }
	else 
	{
		relFlag = (boxRelations & kSanNeighboringBoxXMask);
		if (relFlag == kSanNeighboringBoxAtSameX)
		{
		}
		else if ((boxPositions & kSanBaseBoxIsMaxX) && (relFlag == kSanNeighboringBoxAtPlusX))
		{
			translationFlags +=  kSanTransAddXDimSign + kSanTransChangeXDim ;
  		}
		else if ((boxPositions & kSanBaseBoxIsMinX) && (relFlag == kSanNeighboringBoxAtMinusX))
		{
			translationFlags += kSanTransChangeXDim ;
		}
	}
    if (!(boxPositions & kSanMaskNotMiddleYBox))
    {
    }
	else 
	{
		relFlag = (boxRelations & kSanNeighboringBoxYMask);
		if (relFlag == kSanNeighboringBoxAtSameY)
		{
		}
		else if ((boxPositions & kSanBaseBoxIsMaxY) && (relFlag == kSanNeighboringBoxAtPlusY))
		{
			translationFlags += kSanTransAddYDimSign + kSanTransChangeYDim ;
		}
		else if ((boxPositions & kSanBaseBoxIsMinY) && (relFlag == kSanNeighboringBoxAtMinusY))
		{
			translationFlags += kSanTransChangeYDim ;
		}
	}
	
#ifndef __2DVectors__
    if (!(boxPositions & kSanMaskNotMiddleZBox))
    {
    }
	else 
	{
		relFlag = (boxRelations & kSanNeighboringBoxZMask);
		if (relFlag == kSanNeighboringBoxAtSameZ)
		{
		}
		else if ((boxPositions & kSanBaseBoxIsMaxZ) && (relFlag == kSanNeighboringBoxAtPlusZ))
		{
			translationFlags += kSanTransAddZDimSign + kSanTransChangeZDim ;
		}
		else if ((boxPositions & kSanBaseBoxIsMinZ) && (relFlag == kSanNeighboringBoxAtMinusZ))
		{
			translationFlags += kSanTransChangeZDim ;
		}
	}
#endif

	return (translationFlags);
}

char  setTranslationFlagsFromDels(char delX, char delY, char delZ)
{ // inputs are +1 , 0, or -1
#ifdef __2DVectors__ 
#pragma unused (delZ)
#endif
	char translationFlags = 0;
    if (delX)
 	{
 		translationFlags +=  kSanTransChangeXDim ;
 		if (delX > 0)
 		{
 		    translationFlags +=  kSanTransAddXDimSign  ;
 		}
 	}
    if (delY)
 	{
 		translationFlags +=  kSanTransChangeYDim ;
 		if (delY > 0)
 		{
 		    translationFlags +=  kSanTransAddYDimSign  ;
 		}
 	}
 	
#ifndef __2DVectors__
    if (delZ)
 	{
 		translationFlags +=  kSanTransChangeZDim ;
 		if (delZ > 0)
 		{
 		    translationFlags +=  kSanTransAddZDimSign  ;
 		}
 	}
 #endif
 
	return (translationFlags);
}

void  simGetGenericSeparations(kSanSimulation *sim, long partA, long partB, float *theSeps)
{ // get all 27 separations ( or 9 if in 2d )
 	particle * basePart;
	kSanVector  vecH;
	kSanVector  vecK;
	kSanVector  vecL;
	kSanVector  vecA;
	kSanVector  vecB;
	kSanVector  vecAPrime;
	kSanMatrix cellPrims;
	short h,k,l;
	short count = 0;
#ifdef __2DVectors__
#pragma unused ( l)
  #endif	
	
 	basePart = simLockParts(sim);
	vecA = basePart->basePosition[partA] ;
	vecB = basePart->basePosition[partB] ;
	simGetCellPrimitives(sim, &cellPrims);
	for (h = -1; h <=1; ++h)
	{
		multkSanVectorDFD (&cellPrims.A , h , &vecH);
		for (k = -1; k <=1; ++k)
		{
			multkSanVectorDFD (&cellPrims.B , k , &vecK);
#ifndef __2DVectors__
			for (l = -1; l <=1; ++l)
			{
				multkSanVectorDFD (&cellPrims.C , l , &vecL);
 				vecAPrime.c = vecH.c + vecK.c + vecL.c + vecA.c;
#else
				vecL.a = 0;
				vecL.b = 0;

#endif
 				vecAPrime.a = vecH.a + vecK.a + vecL.a + vecA.a;
				vecAPrime.b = vecH.b + vecK.b + vecL.b + vecA.b;
 				subkSanVectorsD(&vecAPrime, &vecB, &vecAPrime);
				theSeps[count]  = lengthOfkSanVectorD(&vecAPrime);
				++count;
#ifndef __2DVectors__
			}
#endif


		}	
	}
	simReleaseParts(sim);
}
short  lookForNeighbors(kSanSimulation *sim)
{ // returns success
	short err = noErr;
	short unused;
 	long partArrIndex;
 	
	long j;
	long outerBoxIndex;
	
	long outerJackLoop;
	long innerBoxLoopCount;
	
	particle * basePart;	
	box * boxes;

	long * outerBoxIndexPtr;
	long innerJackLoop;

	long outerJackLoopLimit;	
	char  boxRelations;
	char  boxPositions;
	char  translationFlags;
	char  unusedChar;
	
#pragma unused(unused) 
#pragma unused(unusedChar) 
	double neighborCutoffSqu;
	double radSqu;
	kSanVector  sepVec;

	basePart = simLockParts(sim);
	boxes = simLockBoxes(sim);

	neighborCutoffSqu = ((sim->outerCutoff + sim->neighborCutoffExtra) * (sim->outerCutoff + sim->neighborCutoffExtra));  // need to change to real var
	
 	for (partArrIndex=0;partArrIndex<simGetNumParts(sim);++partArrIndex) 	
 	{
 		basePart->basePi[partArrIndex].actualNumNeighbors = 0;
 		basePart->basePi[partArrIndex].extraNumNeighbors = 0;
	}
	
 	for (outerBoxIndex=0;outerBoxIndex<=sim->howManyBoxes + 1;++outerBoxIndex) 	
 		// loop over boxes	(box zero should have no particles in it
 	    // box	sim->howManyBoxes + 1 contains all particles in hyperspace.
 	{
  		boxPositions = setBoxPositionFlags (sim, &boxes[outerBoxIndex] );
 		outerBoxIndexPtr = lockList (&boxes[outerBoxIndex].boxPartList);
 		outerJackLoopLimit = boxes[outerBoxIndex].boxPartList.howManyItems;
	 	for (outerJackLoop=0; outerJackLoop<outerJackLoopLimit ; ++outerJackLoop) 
		{						// loop over particles in box outerBoxLoop		
			partArrIndex = outerBoxIndexPtr[outerJackLoop];
			if (partArrIndex>=simGetNumParts(sim))
			{
				OHSystemBeep();
				doAlert("\pindex out of range");		
			}
		// from 0 to 12
		 	for (innerBoxLoopCount=0;innerBoxLoopCount<MIDDLEBOXINDEX;++innerBoxLoopCount) 	// new for 27 boxes nov 14	
	 		{					// loop over neighboring boxes
				long * thatBoxIndexPtr;
				box * thatBox;
				long innerBoxLoopLabel;
				long innerJackLoopLimit;
				
				innerBoxLoopLabel = boxes[outerBoxIndex].neighboringBoxes[innerBoxLoopCount];
 				thatBox = &boxes[innerBoxLoopLabel];
				boxRelations = setBoxRelationFlags ( innerBoxLoopCount );
       		    translationFlags = setTranslationFlags(boxRelations, boxPositions);
				thatBoxIndexPtr = lockList (&thatBox->boxPartList);

				innerJackLoopLimit = thatBox->boxPartList.howManyItems;
			 	for (innerJackLoop=0;innerJackLoop<innerJackLoopLimit;++innerJackLoop) 	// loop over particles in box i		
	 			{					// loop over potential neighbors in box innerBoxLoop
				
					j = thatBoxIndexPtr[innerJackLoop];
					radSqu = sim->getSepSquared(sim, basePart, partArrIndex, j, translationFlags, &sepVec);
					if (radSqu < neighborCutoffSqu) 
					{
						if (addNNData(sim, basePart, partArrIndex, j, translationFlags, &radSqu, &sepVec)) err = true;
					}
				}
				releaseList (&thatBox->boxPartList);
 			}
		} // end first outerJackLoop loop
		
		
		// MIDDLEBOXINDEX only (this box)
		// set up things that stay constant for every item in outerJackLoop
	 //	innerBoxLoop = MIDDLEBOXINDEX;
	 	
	 /* debug
	 {
		long debugInnerBoxLoopLabel;
		debugInnerBoxLoopLabel = boxes[outerBoxIndex].neighboringBoxes[innerBoxLoop];
		if (debugInnerBoxLoopLabel != outerBoxIndex ) 
		{
			OHSystemBeep();
		}
	}
	*/   // end debug
	
		boxRelations =  setBoxRelationFlags (MIDDLEBOXINDEX); // MIDDLEBOXINDEX is the value for the same box
        translationFlags = setTranslationFlags(boxRelations, boxPositions);
		//innerJackLoopLimit = outerBox->boxPartList.howManyItems;
	 	for (outerJackLoop = 0 ; outerJackLoop<outerJackLoopLimit ; ++outerJackLoop) 
		{						// loop over particles in box outerBoxLoop	again	
			partArrIndex = outerBoxIndexPtr[outerJackLoop];
		 	for (innerJackLoop = outerJackLoop + 1;innerJackLoop<outerJackLoopLimit;++innerJackLoop) 	// note the changed indexing from (outerJackLoop + 1)	
 			{					// loop over potential neighbors in box innerBoxLoop
				j = outerBoxIndexPtr[innerJackLoop];
				//if (j > ithIndex) // extra line for box MIDDLEBOXINDEX -- not needed because we split the loops
				//{
				radSqu = sim->getSepSquared(sim, basePart, partArrIndex, j, translationFlags, &sepVec);
				if (radSqu < neighborCutoffSqu) 
				{
				      if (addNNData(sim, basePart, partArrIndex, j, translationFlags, &radSqu, &sepVec)) err = true;
				}
				//}
			}
		}
		releaseList (&boxes[outerBoxIndex].boxPartList);
	}	
	simReleaseParts(sim);
	simReleaseBoxes(sim);
	return (err);	// dont call handleTooManyNeighbors here!!!
}

short makeNeighborSpace(kSanSimulation *sim, particle *basePart, long partArrIndex);
short makeNeighborSpace(kSanSimulation *sim, particle *basePart, long partArrIndex)
{  
	short err = noErr;
	char done = false;
	char unused;
#pragma unused(unused)
	long topIndex;
	// strategy : first count up to the closest particle which has an extra spot available in the neighbor table
	// then count down moving one NNInfo struct  higher in the table for each particle
	long numParts = simGetNumParts(sim);

	topIndex = partArrIndex + 1;
	while ((topIndex < numParts) && (!done))
	{
		if (basePart->basePi[topIndex].actualNumNeighbors < basePart->basePi[topIndex].neighborSpaceAvailable)
		{
			done = true;
		}
		else
		{
			++ topIndex;
		}
	}
	if (done)
	{
		-- basePart->basePi[topIndex].neighborSpaceAvailable;
		while (topIndex > partArrIndex)
		{
 			// if there's at least one neighbor, move the zeroeth neighbor back
			if (basePart->basePi[topIndex].actualNumNeighbors > 0)
			{
			    basePart->basePi[topIndex].firstNeighborInfo[basePart->basePi[topIndex].actualNumNeighbors] = basePart->basePi[topIndex].firstNeighborInfo[0];
			}
			basePart->basePi[topIndex].firstNeighborInfo += 1;
			--topIndex;
		}
		++ basePart->basePi[partArrIndex].neighborSpaceAvailable;
	}
	else  // try going lower
	{
		long bottomIndex;
		bottomIndex = partArrIndex - 1;
		while ((bottomIndex >= 0) && (!done))
		{
			if (basePart->basePi[bottomIndex].actualNumNeighbors < basePart->basePi[bottomIndex].neighborSpaceAvailable)
			{
				done = true;
			}
			else
			{
				-- bottomIndex;
			}
		}
		
		if (done)
		{
			-- basePart->basePi[bottomIndex].neighborSpaceAvailable;
			++bottomIndex; // because we don't move anything in bottomIndex
		}
		else if (sim->neighborSpaceFrontSlack > 0)  // check the front reserve
		{
			-- sim->neighborSpaceFrontSlack;
			bottomIndex = 0;
			done = true;
		}
		
		if (done)
		{
			while (bottomIndex <= partArrIndex)
			{
 				basePart->basePi[bottomIndex].firstNeighborInfo -= 1;
				if (basePart->basePi[bottomIndex].actualNumNeighbors > 0)
				{
					basePart->basePi[bottomIndex].firstNeighborInfo[0] = basePart->basePi[bottomIndex].firstNeighborInfo[basePart->basePi[bottomIndex].actualNumNeighbors];
				}
				++bottomIndex;
			}
			++ basePart->basePi[partArrIndex].neighborSpaceAvailable;
		}
 		else
		{
		    err = true;
		}
	}
	return (err);	// dont call handleTooManyNeighbors here!!!
}

//#define packCondensedInfo(nni,index,nIndex,type,tFlags) (nni[index].condensedInfo = (((long)nIndex) + (((long)type) << 20) + (((long)tFlags) << 24)))
void packCondensedInfo(NNeighborInfo *nni,long index,long nIndex,short type,char tFlags) ;
void packCondensedInfo(NNeighborInfo *nni,long index,long nIndex,short type,char tFlags) 
{
	nni[index].thisIndex = nIndex; 
	nni[index].thisType = type; 
	nni[index].translationFlags = tFlags;
}

short addOneNNData(kSanSimulation *sim, particle *basePart, long partArrIndex, long neighborIndex, char translationFlags, double *rad, kSanVector * sepVec);
short addOneNNData(kSanSimulation *sim, particle *basePart, long partArrIndex, long neighborIndex, char translationFlags, double *rad, kSanVector * sepVec)
{  
#pragma unused(sepVec)
	short err = noErr;
	short unused;
#pragma unused(unused)
	long newIndex = -1;
	kSanParticle *thisPi;
	
	thisPi = &basePart->basePi[partArrIndex];
    thisPi->neighborScreeningDirty = true;
	if ( thisPi->actualNumNeighbors < thisPi->neighborSpaceAvailable)
	{
		newIndex = 	thisPi->actualNumNeighbors;
 	}
	else if ((partArrIndex >0) && (basePart->basePi[partArrIndex - 1].actualNumNeighbors < basePart->basePi[partArrIndex - 1].neighborSpaceAvailable))
	{
		thisPi->firstNeighborInfo -= 1;
		thisPi->neighborSpaceAvailable += 1;
		basePart->basePi[partArrIndex - 1].neighborSpaceAvailable -= 1;
		newIndex = 0;
	}
	else 
	{
		if (sim->flags.neighborOverrun) 
		{
			err = true;
		}
		else 
		{
	 		err = makeNeighborSpace(sim, basePart, partArrIndex);
			if (!err)  newIndex = thisPi->actualNumNeighbors;
		}

		if (err)
		{
			++basePart->basePi[partArrIndex].extraNumNeighbors;
			sim->flags.neighborOverrun = true ;
	 		basePart->basePi[partArrIndex].neighborInfoDirty = true;
	 		return (err);
		}
	}
	// return before here if error
	{
 		NNeighborInfo *nni;
		nni = thisPi->firstNeighborInfo;
		++thisPi->actualNumNeighbors;
		
		packCondensedInfo(nni, newIndex, neighborIndex, basePart->basePi[neighborIndex].type, translationFlags);

#ifdef NNIRemembersSep
		nni[newIndex].separation = *rad;
#endif

#ifdef NNIRemembersVec
		nni[newIndex].vec = *sepVec;
#endif

	}

	return (noErr);	// dont call handleTooManyNeighbors here!!!
}

#define toggleBit(field,mask)   (field = ((field & mask) ? (field & ( ~mask )) : ( field | mask) ))   
short addNNData(kSanSimulation *sim, particle *basePart, long partArrIndex, long neighborIndex, char translationFlags, double *radSqu, kSanVector * sepVec)
{  
	short err = noErr;
	short unused;
//#pragma unused(unused)
	double rad;

	rad = sqrt(*radSqu);
	if (partArrIndex == 1613)
	{
		unused = 3;
	}
	err = addOneNNData(sim, basePart,  partArrIndex,  neighborIndex, translationFlags, &rad, sepVec);
	sepVec->a = 0.0 - sepVec->a;
	sepVec->b = 0.0 - sepVec->b;

#ifndef __2DVectors__
	sepVec->c = 0.0 - sepVec->c;
#endif
	if (translationFlags)
	{
		if (translationFlags & kSanTransChangeXDim) toggleBit (translationFlags, kSanTransAddXDimSign);
		if (translationFlags & kSanTransChangeYDim) toggleBit (translationFlags, kSanTransAddYDimSign);
#ifndef __2DVectors__
		if (translationFlags & kSanTransChangeZDim) toggleBit (translationFlags, kSanTransAddZDimSign);
#endif

	}
 	err += addOneNNData(sim, basePart,  neighborIndex,  partArrIndex, translationFlags, &rad, sepVec);

	return (err);
} 
/************      calcNeighbors             ****************/
/** fills the neighbor handle with nearest neighbor data ***/
/** and set flags for location of particles               **/
/** we also test for particles outside the cell bounds   ***/
/** and swap them to the other side if they are           **/

short  calcNeighbors(kSanSimulation *sim) // ???? (true?)boxArrayHandle must be unlocked

{
	short err= noErr;
	err = wrapAroundAndBitfields(sim);  // this also updates box info	
	if (!err) 
	{
		if (sim->flags.neighborOverrun)  resetNeighborPtrs(sim, 8L);
		else
		{
			err = lookForNeighbors(sim);
			if (err) 
			{
				particle *basePart = simLockParts(sim);
				err = simReallocateNeighborPtr(sim, basePart, 8L);
				if (!err) err = lookForNeighbors(sim);
				simReleaseParts(sim);			
			}
		}
		if (!err) sim->counters.neighbor = 0;
	}
	if (err)
	{
		sim->flags.error = true;
//		doAlert("\p neighbor calculation error");
	}
	return (err);	
}					
