// newSimulation.c
#define kfccStructure 0
#define kbccStructure 1
#define kdcStructure 2
#define kscStructure 3
#define kEmptyCell 4
#include "kSanSimulationPrivate.h"
#include "OHFile.h"
#include "kSanDialog.h"


void  particleInit (kSanSimulation *sim,short xCells, short yCells, short zCells, 
							short structureType, double latCon);
void fillInfcc(kSanSimulation *sim, short xCells, short yCells, short zCells, double latCon);
void placeNewParticle(kSanSimulation *sim, float newX, float newY, float newZ);
void fillInDiamondCubic(kSanSimulation *sim, short xCells, short yCells, short zCells, double latCon);
void fillInbcc(kSanSimulation *sim,short xCells, short yCells, short zCells, double latCon);
void fillInsc(kSanSimulation *sim,short xCells, short yCells, short zCells, double latCon);
void setArtificialCellSize(kSanSimulation *sim);

short	DoNewDialog	(short *xPtr, short *yPtr, short *zPtr,
					 short *structureTypePtr, double *lcPtr)
{
	OHBasicDialog dialog;
	short		i;
	long 		tempStructure;
	Handle		applyItemHandle;
	ControlHandle structureTypeHandle[5];
	
	setupAppDialog (&dialog, kNewDialogResID);
	if (! dialog.done )
	{
		prepLongItem(dialog.dp, ixEditItem , 3);
		prepLongItem(dialog.dp, iyEditItem , 3);
		prepLongItem(dialog.dp, izEditItem , 3);
		prepDoubleItem(dialog.dp, ilcEditItem , 4.8);

		GetDialogItem (dialog.dp,ok,&dialog.itemType,&applyItemHandle,&dialog.itemRect);
		for (i=0 ; i<5 ; ++i)
		{
			Handle tempHandle;
			GetDialogItem (dialog.dp,(ifccRadio + i) ,&dialog.itemType,&tempHandle,&dialog.itemRect);
			structureTypeHandle[i] = (ControlHandle)tempHandle;
			SetControlValue(structureTypeHandle[i],(short)kUnselected);
		}

		tempStructure = kfccStructure;
		SetControlValue(structureTypeHandle[tempStructure],(short)kRadioSelected);
	}
	while ( ! dialog.done )
	{
		ModalDialog( nilPointer,&dialog.itemHit);
		switch (dialog.itemHit)
		{
			case ok:
			case cancel:
				dialog.done = true;
				break;		
			case ifccRadio:
			case ibccRadio:
			case iscRadio:
			case idcRadio:
			case iemptyRadio:
				SetControlValue(structureTypeHandle[tempStructure],kRadioUnselected);
				tempStructure = dialog.itemHit -10;
				SetControlValue(structureTypeHandle[tempStructure],kRadioSelected);
				break;
		}
	}
	if (dialog.itemHit != cancel)
	{
		*xPtr = getLongItem(dialog.dp, ixEditItem , 100, 1);
		*yPtr = getLongItem(dialog.dp, iyEditItem , 100, 1);
		*zPtr = getLongItem(dialog.dp, izEditItem , 100, 1);
		*lcPtr = getDoubleItem(dialog.dp, ilcEditItem , 1000.0, 0.00001);

		(*structureTypePtr)	= tempStructure;
	}
	OHBreakdownDialog(&dialog);
	return (dialog.itemHit);
}

short  makeSimStructure(kSanSimulation *sim, short xCells, short yCells, short zCells, 
							short structureType, double latCon)
{		
	Handle tempPadding = nilPointer;
	long err = noErr;
	
	tempPadding = NewHandle(10000L);
	if (tempPadding == nilPointer) 
	{
		doAlert ("\p not enough memory to initialize");
	}
	else
	{	
		particleInit (sim, xCells, yCells, zCells, structureType,latCon);
		OHDisposeHandle(&tempPadding);

		err = calcNeighbors(sim);
		if (!err) err = compressNeighborMemory(sim);
		if (err)  doAlert ("\punable to initialize cell");

		sim->counters.memoryCleanup = 0;
		simIncrementStateCount(sim);

	}
	return (err);
}
void   particleInit (kSanSimulation *sim, short xCells, short yCells, short zCells, 
							short structureType, double latCon)
{
	short partPerCell;
	short err;
	long newNumberOfParticles;
	sim->simCell.cellEdges.a = xCells * latCon;
	sim->simCell.cellEdges.b = yCells * latCon;
#ifndef __2DVectors__
	sim->simCell.cellEdges.c = zCells * latCon;
#endif
	sim->simCell.alpha = kPi * 0.5;
	sim->simCell.beta = kPi * 0.5;
	sim->simCell.gamma = kPi * 0.5;
	calcCellPropertiesFromEdgesAndAngles(&sim->simCell);
	err = calcNeighbors(sim);
	if ( structureType == kfccStructure) partPerCell = 4;
	else if ( structureType == kbccStructure) partPerCell = 2;
	else if ( structureType ==  kdcStructure) partPerCell = 8;
	else if ( structureType == kscStructure) partPerCell = 1;
	else if ( structureType == kEmptyCell) partPerCell = 0;
	else 		
		{
			doAlert ("\p Invalid structure type");
			partPerCell = 0;
		}
		
	newNumberOfParticles = (long)partPerCell * (long)xCells* (long)yCells* (long)zCells;
	if (newNumberOfParticles > 32500 )
	{
		 doAlert("\pcell size is too large");
		 newNumberOfParticles = 32500;
	}
	err = resizeMemory (sim, newNumberOfParticles);
	simSetPlugNotificationSuspended (sim, true);
	if (!err)
	{
		if      ( structureType == kfccStructure) fillInfcc(sim,  xCells, yCells, zCells, latCon);
		else if ( structureType == kbccStructure) fillInbcc(sim, xCells, yCells, zCells, latCon);
		else if ( structureType == kdcStructure) fillInDiamondCubic(sim,  xCells, yCells, zCells, latCon);
		else if ( structureType == kscStructure) fillInsc(sim,  xCells, yCells, zCells, latCon);
	}
	simSetPlugNotificationSuspended (sim, false);
	err = calcNeighbors(sim);
	{
		short gnErr;		
		gnErr = calcNeighbors(sim);
	}
}

/************  fill in fcc lattice  ****************/
void  fillInfcc(kSanSimulation *sim,  short xCells, short yCells, short zCells, double latCon)
{
	short i,x,y,z;
	double thisX, thisY, thisZ;
	double oneHalf;
#ifdef __2DVectors__
#pragma unused ( zCells)
#pragma unused ( z)
 #endif	
	oneHalf = (latCon / 2.0);
	i = 0;
	for (x = 0; x< xCells; ++x)
	{
		thisX = (x*latCon) - (sim->simCell.cellDim.a *0.5) ;
		for (y = 0; y< yCells; ++y)
		{
			thisY = (y*latCon) - (sim->simCell.cellDim.b *0.5) ;
#ifndef __2DVectors__
			for (z = 0; z< zCells; ++z)
			{
				thisZ = (z*latCon) -  (sim->simCell.cellDim.c *0.5) ;
#else 
            	 thisZ = 0;				
#endif
				placeNewParticle(sim,  thisX, thisY, thisZ);
				++i;
				placeNewParticle(sim,  thisX+ oneHalf, thisY+ oneHalf, thisZ);
				++i;
#ifndef __2DVectors__
				placeNewParticle(sim,  thisX+ oneHalf, thisY, thisZ+ oneHalf);
				++i;
				placeNewParticle(sim,  thisX, thisY+ oneHalf, thisZ+ oneHalf);
				++i;
			}
#endif
		}
	}
}
void  placeNewParticle(kSanSimulation *sim, float newX, float newY, float newZ)
{
	kSanVector  nPos;
#ifdef __2DVectors__
#pragma unused ( newZ)
  #endif	
	nPos.a = newX;
	nPos.b = newY;
	 
#ifndef __2DVectors__
	nPos.c = newZ;
#endif

	addParticle(sim, &nPos,0,0,false);
}
void  fillInDiamondCubic(kSanSimulation *sim,  short xCells, short yCells, short zCells, double latCon)
{
	short i,x,y,z;
	double thisX, thisY, thisZ;
	double oneHalf, oneQuarter;
#ifdef __2DVectors__
#pragma unused ( z)
#pragma unused ( zCells)
  #endif	
	oneHalf = (latCon / 2.0);
	oneQuarter = (latCon / 4.0);
	i = 0;
	for (x = 0; x< xCells; ++x)
	{
		thisX = (x*latCon) - (sim->simCell.cellDim.a *0.5) ;
		for (y = 0; y< yCells; ++y)
		{
			thisY = (y*latCon) - (sim->simCell.cellDim.b *0.5) ;
#ifndef __2DVectors__
			for (z = 0; z< zCells; ++z)
			{
				thisZ = (z*latCon) -  (sim->simCell.cellDim.c *0.5) ;
				placeNewParticle(sim,  thisX, thisY, thisZ);
				++i;
				placeNewParticle(sim,  thisX+ oneHalf, thisY+ oneHalf, thisZ);
				++i;
				placeNewParticle(sim,  thisX+ oneHalf, thisY, thisZ+ oneHalf);
				++i;
				placeNewParticle(sim,  thisX, thisY+ oneHalf, thisZ+ oneHalf);
				++i;
				thisX = thisX + oneQuarter;
				thisY = thisY + oneQuarter;
				thisZ = thisZ + oneQuarter;
				placeNewParticle(sim,  thisX, thisY, thisZ);
				++i;
				placeNewParticle(sim,  thisX+ oneHalf, thisY+ oneHalf, thisZ);
				++i;
				placeNewParticle(sim,  thisX+ oneHalf, thisY, thisZ+ oneHalf);
				++i;
				placeNewParticle(sim,  thisX, thisY+ oneHalf, thisZ+ oneHalf);
				++i;
				thisX = thisX - oneQuarter;
				thisY = thisY - oneQuarter;
				thisZ = thisZ - oneQuarter;
			}
#else
			placeNewParticle(sim,  thisX, thisY, thisZ);

#endif
		}
	}
}
void  fillInbcc(kSanSimulation *sim, short xCells, short yCells, short zCells, double latCon)
{
	short i,x,y,z;
	double thisX, thisY;
	double thisZ = 0;
	double oneHalf;
#ifdef __2DVectors__
#pragma unused ( z)
#pragma unused ( zCells)
  #endif	
	oneHalf = (latCon *0.5);
	i = 0;
	for (x = 0; x< xCells; ++x)
	{
		thisX = (x*latCon) - (sim->simCell.cellDim.a *0.5) ;
		for (y = 0; y< yCells; ++y)
		{
			thisY = (y*latCon) - (sim->simCell.cellDim.b *0.5) ;
#ifndef __2DVectors__
			for (z = 0; z< zCells; ++z)
			{
				thisZ = (z*latCon) -  (sim->simCell.cellDim.c *0.5) ;
 #endif
				placeNewParticle(sim,  thisX, thisY, thisZ);
				++i;
				placeNewParticle(sim,  thisX+ oneHalf, thisY+ oneHalf, thisZ+ oneHalf);
				++i;
#ifndef __2DVectors__
			}
 #endif
		}
	}
}
void  fillInsc(kSanSimulation *sim,  short xCells, short yCells, short zCells, double latCon)
{
	short i,x,y,z;
	double thisX, thisY, thisZ;
#ifdef __2DVectors__
#pragma unused ( z)
#pragma unused ( zCells)
  #endif	
	i = 0;
	for (x = 0; x< xCells; ++x)
	{
		thisX = (x*latCon) - (sim->simCell.cellDim.a *0.5) ;
		for (y = 0; y< yCells; ++y)
		{
			thisY = (y*latCon) - (sim->simCell.cellDim.b *0.5) ;
#ifndef __2DVectors__
  			for (z = 0; z< zCells; ++z)
			{
				thisZ = (z*latCon) -  (sim->simCell.cellDim.c *0.5) ;
#endif
				placeNewParticle(sim,  thisX, thisY, thisZ);
				++i;
#ifndef __2DVectors__
			}
 #endif
		}
	}
}

/*
void  setArtificialCellSize(kSanSimulation *sim)
{
	particle * basePart;
	long partArrIndex;
	kSanVector * thisPos;
	kSanVector maxDim;
	kSanVector minDim;
	
	basePart = simLockParts(sim);
	zerokSanVectorD (&maxDim);
	zerokSanVectorD (&minDim);
 	for (partArrIndex= 0;partArrIndex < simGetNumParts(sim); ++partArrIndex)
	{
		thisPos = &(basePart->basePosition[partArrIndex]);
		if (maxDim.a < thisPos->a) maxDim.a = thisPos->a;
		if (maxDim.b < thisPos->b) maxDim.b = thisPos->b;
		if (minDim.a > thisPos->a) minDim.a = thisPos->a;
		if (minDim.b > thisPos->b) minDim.b = thisPos->b;
#ifndef __2DVectors__
		if (maxDim.c < thisPos->c) maxDim.c = thisPos->c;
		if (minDim.c > thisPos->c) minDim.c = thisPos->c;
#endif
	}
	
	sim->simCell.cellEdges.a = (maxDim.a - minDim.a) + 3;
	sim->simCell.cellEdges.b = (maxDim.b - minDim.b) + 3;
	if (sim->simCell.cellEdges.b < 10.0) sim->simCell.cellEdges.b = 10.0;
	if (sim->simCell.cellEdges.c < 10.0) sim->simCell.cellEdges.c = 10.0;
#ifndef __2DVectors__
	sim->simCell.cellEdges.c = (maxDim.c - minDim.c) + 3;
	if (sim->simCell.cellEdges.a < 10.0) sim->simCell.cellEdges.a = 10.0;
#endif

	sim->simCell.alpha = 90;
	sim->simCell.beta = 90;
	sim->simCell.gamma = 90;
	setNewCellProperties(sim);
	
	sim->deltaT = 0.001;
	sim->totalTimeElapsed = 0.0;
	sim->totalNumberOfIterations = 0;
	simReleaseParts(sim);
}
*/

 void	kSanSimNewInitialStructureDialog	(kSanSimulation *sim)
{
	 short err = noErr;
	short dialogValue = kDialogOK;
	short xCells = 4;
	short yCells = 4;
	short zCells = 4;
	short structureType = 1;
	double latCon = 4.0;
	short shouldWeRotate = false;
	
	if (!err)
	{
		 dialogValue = DoNewDialog(&xCells, &yCells, &zCells, &structureType, &latCon);
		if (dialogValue == kDialogOK)
		{
			kozoObject *newWin;
			makeSimStructure(sim, xCells, yCells, zCells, structureType, latCon);
			spawnWindow(sim, nilPointer,  &newWin);
		}
		else
		{
			kozoDispose(simOBJ(sim));
		}
	}
}
/*
short loadOldSimulation(kSanSimulation *sim, OHFile *aFile)
{	
	short err = noErr;
	short dialogResult;
	FInfo fndrInfo;
	OHDocument *doc;
	doc = docData(simOBJ(sim));
	
	if(!err) 
	{	
		err = FSpGetFInfo(&aFile->fs,&fndrInfo);
	}
	if (!err)
	{
		if (fndrInfo.fdCreator != 'kSan' )
		{
			dialogResult = OpenAnywayDialog ();
			if (dialogResult != kDialogOK) err = true;
		}
	}
	if (!err) 
	{
		doc->file  = OHFileDuplicator (aFile);
		 err = loadNewInfo(sim);  
	}
	if (!err)
	{	
		wrapAroundAndBitfields(sim);  
		 simIncrementStateCount(sim);
	}
	return (err);
}
*/
