#include "kSanSimulationPrivate.h"
#include "kSanApplicationPublic.h"
#include "kSanPlugIn.h"

short  getPlugDataHandle(kSanSimulation *sim, kSanPlugIn *myPlug, Handle *piHandle);

short simHandlePlugInError(kSanSimulation *sim,  kozoObject *plugObj, short plugInError)
{
#pragma unused(sim)
#pragma unused(plugObj)
#pragma unused(plugInError)
 
	doAlert ("\pplug Error");
	return (noErr);
}

void  makeDefaultModule(kSanSimulation *sim, DescType moduleClass) 
{
	setModuleMenuChoice(sim, moduleClass, 1);  // not the default, but the first subclass 
}

short  setModuleMenuChoice(kSanSimulation *sim, DescType moduleClass, short theIndex)
{
	short err= noErr;
	OHList *subclasses;
	kozoClass *subclassChosen;
	
	subclasses = getSubclassList(moduleClass);
	if ((theIndex <= 0) || (theIndex > OHListCount(subclasses) )) theIndex = 1;

	subclassChosen = castTokozoClass( getNthItem( subclasses, theIndex - 1));
	err = simMakeAndInstallModule(sim, moduleClass , subclassChosen->classID);

	return (err);
}

DescType  simGetModuleClassID(kSanSimulation *sim, DescType whichModule);
DescType  simGetModuleClassID(kSanSimulation *sim, DescType whichModule)
{
	DescType theModuleID = 0L ;
	switch (whichModule)
	{
		case ClassPotentialPlugIn:
			theModuleID = simGetPotentialBeingUsed(sim );
			break;
		case ClassCellResizePlugIn:
			theModuleID =  simGetCellResizeBeingUsed(sim );
			break;
		case ClassMethodPlugIn:
			theModuleID =   simGetMethodBeingUsed(sim );
			break;
	}
	return (theModuleID);
}

kozoObject *  simGetModule(kSanSimulation *sim, DescType whichModule)
{
	return (OHGetNamedReference (simOBJ(sim), whichModule));
}
 short  simInstallUniqueModule(kSanSimulation *sim, DescType whichModule, kozoObject *newPlugIn)
{  // here, we've separated the plug-in ownership from the identity in the pointers
    // this is only for the unique modules!!!
	
	 OHStoreUniqueReference (newPlugIn, simOBJ(sim), whichModule);   // clone paired with unclone
	switch (whichModule)
	{
		case ClassPotentialPlugIn:
  			 if (newPlugIn) sim->potentialPlugIn  = ppiData(newPlugIn);
 			 else sim->potentialPlugIn  = nilPointer;
 			 goto Increment;
		case ClassCellResizePlugIn:
 			 if (newPlugIn)  sim->resizePlugIn  = rpiData(newPlugIn);
 			 else sim->resizePlugIn  = nilPointer;
 			 goto Increment;
		case ClassMethodPlugIn:
 			 if (newPlugIn) sim->methodPlugIn  = mpiData(newPlugIn);
			 else sim->methodPlugIn  = nilPointer;
 			 goto Increment;
		 Increment:
 			 simIncrementStateCount(sim);
			break;
		default:
 			 OHUnstoreReference (newPlugIn, simOBJ(sim), whichModule);  // this is here so that we undo the store that we just did!
			break;
 	}
 	return (noErr);
}
 
short  simMakeAndInstallModule(kSanSimulation *sim, DescType whichModule, DescType newModuleClassID)
{
	short err= noErr;
 	kozoObject *newPlugObj;
  	DescType currentModuleID;
 	currentModuleID = simGetModuleClassID(sim,  whichModule);
	if (currentModuleID != newModuleClassID)
	{
		if (classIDIsSubclass(newModuleClassID, whichModule))
		{
  			newPlugObj = kozoObjectGetNthItemOfClassX ( simOBJ(sim), newModuleClassID, 0L);
 	                   if (!newPlugObj) kozoNewNoData(&newPlugObj, newModuleClassID, simOBJ(sim));    
  
  			 if (newPlugObj)  simInstallUniqueModule(sim,  whichModule, newPlugObj);
 		}
	}
	packageAndCallSetBool(simOBJ(sim), pMenusDirty, true);
	return (err);
}
 
short  kSanSimCallActivePlugInsCallback(kozoObject *dcpObj, kSanSimulation *sim);
short  kSanSimCallActivePlugInsCallback(kozoObject *dcpObj, kSanSimulation *sim)
{
	kSanDataCollectionPlugIn *dcp;
  	dcp = dcpData (dcpObj);
	++dcp->counter;
	if  (dcp->counter >= dcp->threshhold)
	{
		dcp->counter  = 0;
		dcp->doCollection ( dcp,  sim) ;  
	}
	return (noErr);
}	
void  kSanSimCallActivePlugIns(kSanSimulation *sim)
{
	OHList  *collectingPlugIns;
	
	collectingPlugIns = getListOfObjectsOfClassX  (simOBJ(sim), ClassDataCollectionPlugIn);
	if (collectingPlugIns)   
	{
		OHListEveryItemDoOneArg( collectingPlugIns,  kSanSimCallActivePlugInsCallback, sim);
		releaseListOfObjectsOfClassX (simOBJ(sim), ClassDataCollectionPlugIn);
	}
}	
void  removeActivePlugIn(kSanSimulation *sim, DescType moduleClass)
{
	OHList *plugList;
	plugList  = getListOfObjectsOfClassX (simOBJ(sim), moduleClass);
	if (plugList)
	{
	 	while (OHListCount(plugList) > 0)
	 	{
	 		kozoObject * pi;
	 		pi = castTokozoObject( getNthItem(plugList, 0));
	 		kozoDispose(pi);
	  	}  
	  	 releaseListOfObjectsOfClassX(simOBJ(sim), moduleClass);
  	}
}

void  makeWhatWasChangedStruct (kSanSimulation *sim,  short whichChange, whatWasChangedStruct *weChanged)
{
	weChanged->other  = 0;
	weChanged->another = 0;
	weChanged->sim  = sim;
	if (whichChange & 0x0001) weChanged->atomTypes = 1;
	else weChanged->atomTypes = 0;
	if (whichChange & 0x0002) weChanged->cutoffs = 1;
	else weChanged->cutoffs = 0;
	if (whichChange & 0x0004) weChanged->neighbors = 1;
	else weChanged->neighbors = 0;
	if (whichChange & 0x0008) weChanged->numParts = 1;
	else weChanged->numParts = 0;
	if (whichChange & 0x0010) weChanged->cellSize = 1;
	else weChanged->cellSize = 0;
	if (whichChange & 0x0020) weChanged->groups = 1;
	else weChanged->groups = 0;
}
void  simInformMethodOfChangedPosition(kSanSimulation *sim, long partArrIndex)
{
	if (!(sim->plugNotificationSuspended))
	{
		kozoDoMessage (OHGetNamedReference (simOBJ(sim), ClassMethodPlugIn), kSanChangedPositionMessage, typeLongIntegerValue, (void *)partArrIndex);
	}
	else
	{
		sim->weChanged = (sim->weChanged | kChangedPartPositions);
	}
}
void  simInformPluginsOfChange(kSanSimulation *sim, short whichChange)
{
	sim->weChanged = (sim->weChanged | whichChange);
	if (!(sim->plugNotificationSuspended))
	{
		whatWasChangedStruct weChanged;
		makeWhatWasChangedStruct(sim, sim->weChanged, &weChanged);
		if (sim->weChanged & kChangedPartPositions) kozoDoMessage (OHGetNamedReference (simOBJ(sim), ClassMethodPlugIn), kSanChangedPositionMessage, typeLongIntegerValue, (void *)kSanAllParts);
		kozoObjectMessageForContents(simOBJ(sim), ClasskSanPlugIn,  kSanChangedInfoMessage,  typeWhatWasChangedStruct, (&weChanged));
		sim->weChanged = 0;
	}
}	

short	simNewDataCollectorChoice(  long item)
{
	short err = noErr;
	kozoClass * plugClass;
	kozoObject  *plug;
	OHList *dcPlugIns;
	kozoObject *sim;
	sim =  getFrontSim();
	if (sim)
	{	
		dcPlugIns = getSubclassList(ClassDataCollectionPlugIn);
		plugClass = castTokozoClass( getNthItem(dcPlugIns, item - 1));
		kozoNewNoData (&plug, plugClass->classID, sim );
	}
	else err = true;
	return (err);
}

void disposeCollectorMenu(void)
{
	MenuHandle theMenu = nilPointer;
	theMenu = GetMenu (mSimDataCollectors);
	if (theMenu) 
	{
		ReleaseResource ((Handle) theMenu);
	}
}	
short makeCollectorMenu(kSanSimulation *sim)
{
	short err = noErr;
	Str255 className;
	MenuHandle theMenu = nilPointer;
	OHList * dcList;
	kozoObject *nthOb;
	short i;
	short count;
	theMenu = GetMenu (mSimDataCollectors);
	dcList = getListOfObjectsOfClassX  (simOBJ(sim), ClassDataCollectionPlugIn);
	if (dcList)
	{
		count = OHListCount(dcList);
		if (theMenu)
		{
			for (i=0;i<count;++i)
			{
				nthOb = castTokozoObject(  getNthItem (dcList, i));
				OHStringGetP(nthOb->name, className);
				AppendMenu (theMenu, className);
			}
			InsertMenu(theMenu, -1);	
		}
		else err = true;
		releaseListOfObjectsOfClassX (simOBJ(sim), ClassDataCollectionPlugIn);
	}
	return (err);
	// the zeroth item is the default item, which does not get put into the list the user sees
}



