#include "kSanSimulationPrivate.h"
short simCalcLinearStress (kSanSimulation *sim, doubleVector *linStress);
short simCalcLinearStressFromVirial (kSanSimulation *sim, doubleVector *linStress);
short 	 simGetMomenta(kSanSimulation *sim, doubleVector *vec);
short simCalcLinearStressNumerically(kSanSimulation *sim, doubleVector *linStress);
short simCalcVirialPressure (kSanSimulation *sim, doubleVector *vPress);
short simCalcKineticPressure (kSanSimulation *sim, doubleVector *kPress);

void 	 doCellPressure (kSanSimulation *sim)  // added july 31 
{  // compressive stress is negative
	short err;
	double newVel;
	double delT;
	double damper;
	double delPos;
	doubleVector linStress;
	doubleVector appliedStress;
	
	delT = sim->totalTimeElapsed - sim->lastNoseCalc;
	sim->lastNoseCalc = sim->totalTimeElapsed ;
	if (delT > 100) delT = 100;
	damper = (1 / (1 + (delT / sim->noseTimeConst)));
	if ((!sim->gConstrained.XAxis) || (!sim->gConstrained.YAxis) || (!sim->gConstrained.ZAxis) )
			err = simCalcLinearStress (sim, &linStress);
			
	// sim->strainStress is in MPa
	// applied stress is now in units of ((g/mol) / (A * ps * ps))
	multVectorD(&sim->strainStress, kHellmanPressuresPerMPa, &appliedStress);
		
	// linear stress returned in units of eV per Angstrom cubed
	multVectorD(&linStress, kHellmanEnergiesPereV, &linStress);	
	// now, both stresses are in ((g/mol) / (A * ps * ps))
	
	if (!sim->gConstrained.XAxis)
	{
		newVel = (sim->noseVelocity.a * damper);
		newVel += ((linStress.a - appliedStress.a) * delT / sim->noseMass) ;
		// noseMass in ((g/mol) / (A * A))    result is vel
		delPos = newVel * delT;
		while (fabs(delPos) >  0.01 * sim->simCell.cellDim.a)
		{
			newVel *= 0.9;
			delPos = newVel * delT;
		}
		sim->noseVelocity.a = newVel;
        scaleAxis (sim,pCellXLength, 1+ (delPos / sim->simCell.cellDim.a));
	}
	if (!sim->gConstrained.YAxis)
	{
		newVel = (sim->noseVelocity.b * damper);
		newVel += ((linStress.b - appliedStress.b) * delT / sim->noseMass) ;
		// noseMass in mass per area    stress in force per area    result is vel
		delPos = newVel * delT;
		while (fabs(delPos) >  0.01 * sim->simCell.cellDim.b)
		{
			newVel *= 0.9;
			delPos = newVel * delT;
		}
		sim->noseVelocity.b = newVel;
        scaleAxis (sim,pCellYLength, 1+ (delPos / sim->simCell.cellDim.b));
	}
	if (!sim->gConstrained.ZAxis)
	{
		newVel = (sim->noseVelocity.c * damper);
		newVel += ((linStress.c - appliedStress.c) * delT / sim->noseMass) ;
		// noseMass in mass per area    stress in force per area    result is vel
		delPos = newVel * delT;
		while (fabs(delPos) > 0.01 * sim->simCell.cellDim.c)
		{
			newVel *= 0.9;
			delPos = newVel * delT;
		}
		sim->noseVelocity.c = newVel;
        scaleAxis (sim,pCellZLength, 1+ (delPos / sim->simCell.cellDim.c));
	}
//	if ((sim->gConstrained.Gamma) || (sim->gConstrained.Alpha) || (sim->gConstrained.Beta) )
//			err = simCalcShearStress (sim, &shearStress);
}
short simCalcLinearStressNumerically(kSanSimulation *sim, doubleVector *linStress)
{
	short err = noErr;
	return (err);
}
short simCalcLinearStressFromVirial (kSanSimulation *sim, doubleVector *linStress)
{
	short err = noErr;
	doubleVector virialPressure;
	doubleVector kineticPressure;
	simCalcVirialPressure(sim, &virialPressure);
	simCalcKineticPressure(sim, &kineticPressure);
	addVectorsD(&virialPressure, &kineticPressure, linStress);
	return (err);
}
short simCalcVirialPressure (kSanSimulation *sim, doubleVector *vPress)
{
	short err = noErr;
	doubleVector totalVir;
	doubleVector vir;
	long i;
	zeroVectorD(&totalVir);
	if (sim->potentialPlugIn->onePartVirialCalc != nilPointer)
	{
		for (i = 0; i < sim->howManyParts; ++i)
		{
	 	 	 err = sim->potentialPlugIn->onePartVirialCalc (sim->potentialPlugIn, OHOb(sim),  i, &vir);
// virial at one particle should be returned in units of mass * A * A / (s * s) (i.e. an energy)
			totalVir.a += vir.a;
			totalVir.b += vir.b;
			totalVir.c += vir.c;
		}
		vPress->a = totalVir.a * sim->simCell.reciprocalVolume;
		vPress->b = totalVir.b * sim->simCell.reciprocalVolume;
		vPress->c = totalVir.c * sim->simCell.reciprocalVolume;
	}
	else 
	{
		zeroVectorD(vPress);
		err = true;
	}
	return (err);
}
short simCalcKineticPressure (kSanSimulation *sim, doubleVector *kPress)
{
	short err = noErr;
	long i;
	particle *part;
	particle *thispart;
	kSanAtomtype **types;
	doubleVector vec;
	double halfMass;
	zeroVectorD (&vec);
	part = simLockParts((kozoObject *)sim);
	types = simLockAtomTypes((kozoObject *)sim);
	for(i=0;i< sim->howManyParts;++i)  // usually how many is the number of particles in the group
	{
		thispart = &part [i];
		halfMass = 0.5 * types[thispart->type]->specs.mass ;
		vec.a += thispart->velocity.a * thispart->velocity.a * halfMass;
		vec.b += thispart->velocity.b * thispart->velocity.b * halfMass;
		vec.c += thispart->velocity.c * thispart->velocity.c * halfMass;
	}
	//sum of kinetic energies in (g/mol) A A / ps ps   
	
	// now div by cell size to get (g/mol) A  / (ps ps   A A)  force per square angstrom
	kPress->a = vec.a * sim->simCell.reciprocalVolume;
	kPress->b = vec.b * sim->simCell.reciprocalVolume;
	kPress->c = vec.c * sim->simCell.reciprocalVolume;

	simReleaseParts((kozoObject *) sim);
	simReleaseAtomTypes((kozoObject *)sim);
	return (err);
}

short simCalcLinearStress (kSanSimulation *sim, doubleVector *linStress)
{
	short err = noErr;
	if (sim->potentialPlugIn->onePartVirialCalc != nilPointer) simCalcLinearStressFromVirial(sim, linStress);
	else simCalcLinearStressNumerically(sim, linStress);
	return (err);
}
short 	 simGetMomenta(kSanSimulation *sim, doubleVector *vec)
{ // returns the momentum vector of the system in (g/mol) A  /  ps   
	long i;
	short err = noErr;
	particle * thispart;
	particle * part;
	kSanAtomtype **types;
	doubleVector localvec;
	doubleVector offVec;
	double thisMass;
	
	zeroVectorD (&localvec);
	zeroVectorD (&offVec);
	part = simLockParts((kozoObject *)sim);
	types = simLockAtomTypes((kozoObject *)sim);
	for(i=0;i< sim->howManyParts;++i)  // usually how many is the number of particles in the group
	{
		thispart = &part [i];
		thisMass = types[thispart->type]->specs.mass;
		localvec.a += thisMass * fabs(thispart->velocity.a);
		localvec.b +=  thisMass * fabs(thispart->velocity.b);
		localvec.c += thisMass *  fabs(thispart->velocity.c);
		offVec.a += thisMass *  thispart->velocity.a ;
		offVec.b +=  thisMass *  thispart->velocity.b ;
		offVec.c += thisMass *  thispart->velocity.c ;
	}
	localvec.a -= offVec.a;
	localvec.b -= offVec.b;
	localvec.c -= offVec.c;
//	energy *= (0.00005188444297); // this is 0.5 * ( 1 / 9636.8 )
			// converts from (g/mol) A A / ps ps  to  eV 
			// and multiplies by 1/2   as in 1/2 mv^2
	simReleaseParts((kozoObject *) sim);
	simReleaseAtomTypes((kozoObject *)sim);
	*vec =  localvec;
	return (err); // returns in eV
}
