#include "kSanSimulationPrivate.h"
#include "OHSharedUtils.h"
void	 simRebuildUniqueIDList (kSanSimulation *sim );
long	 simSearchListForUniqueID (kSanSimulation *sim, long theID , long *indexInUniqueIDList);

short simUniqueIDListSetIndexForID(kSanSimulation *sim, long theID, long newIndex);
short simUniqueIDListSetIndexForID(kSanSimulation *sim, long theID, long newIndex)
{
	short err = noErr;
	long  oldValue;
	long indexInUniqueIDList;
	oldValue = simSearchListForUniqueID (sim,  theID , &indexInUniqueIDList);
	if (oldValue != -1) setNthNameNL(&(sim->uniqueIDList), indexInUniqueIDList, newIndex);
	else err = true;
	return (err);
}
short kSanParticleSwapUniqueIDs(kSanSimulation *sim, long partArrIndexA, long partArrIndexB)
{
	short err;
	long IDA;
	long IDB;
	IDA = kSanParticleGetUniqueID(sim,  partArrIndexA );
	IDB = kSanParticleGetUniqueID(sim,  partArrIndexB );
	if (sim->flags.uniqueIDListDirty)
	{
		kSanParticleSetUniqueID(sim,  partArrIndexA , IDB);
		kSanParticleSetUniqueID(sim,  partArrIndexB , IDA);
	}
	else // preserve cleanliness of uniqueIDList
	{
		(basePartPtr(sim))[partArrIndexA].uniqueID = IDB;
		(basePartPtr(sim))[partArrIndexB].uniqueID = IDA;
		
		err = simUniqueIDListSetIndexForID(sim, partArrIndexA, IDB);
		if (!err) err = simUniqueIDListSetIndexForID(sim, partArrIndexB, IDA);	
		if (err) sim->flags.uniqueIDListDirty = true;
	}
	return (noErr);
}

long	simParticleLabelFromUniqueID  (kSanSimulation *sim, long theID )
{
	long foundIndex ;
	foundIndex = simParticleLabelFromUniqueID (sim, theID ) - 1;
  	return (foundIndex);
}
long	simParticleIndexFromUniqueID  (kSanSimulation *sim, long theID )
{
	long foundLabel ;
	long listIndex ;
	foundLabel = simSearchListForUniqueID (sim,  theID , &listIndex);
	if (foundLabel == -1) 
	{
		sim->flags.uniqueIDListDirty = true;
		foundLabel = simSearchListForUniqueID (sim,  theID , &listIndex);
	}
	return (foundLabel);
}
long	 simSearchListForUniqueID (kSanSimulation *sim, long theID , long *indexInUniqueIDList)
{
	short thisGuessWasHigh ;
	unsigned long maximumJump;
	long highestPossibleVal;
	long lowestPossibleVal;
	long guess;
	long spread;
	long simNumParts ;
	long halfSpread;
	long thisID ;
	long foundIndex = 0;
	OHNamedList *uidList;
	
	lowestPossibleVal = 0;
	simNumParts = simGetNumParts(sim) ;
	uidList = &(sim->uniqueIDList);
	highestPossibleVal = MIN (simNumParts - 1 , theID) ;
	
	guess = theID * (((float)simNumParts) / ((float)(sim->uniqueParticleCounter + 1) ));
	if (guess > highestPossibleVal) guess = highestPossibleVal;
	else if (guess < 0) guess = 0;
	if (sim->flags.uniqueIDListDirty) simRebuildUniqueIDList (sim);

	while ((foundIndex == 0)  && (lowestPossibleVal <= highestPossibleVal))
	{
		thisID = getNthItemNL(uidList, guess);
		if (thisID != theID) 
		{
			if (thisID > theID)
			{  //guess too high
				highestPossibleVal = guess - 1;
				maximumJump = (thisID - theID) - 1; // maximumJump cannot be negative
				thisGuessWasHigh = true;
			}
			else 
			{ // guess too low
				lowestPossibleVal = guess + 1;
				maximumJump = (theID - thisID) - 1; // maximumJump cannot be negative
				thisGuessWasHigh = false;
			}
			spread = highestPossibleVal - lowestPossibleVal;
			halfSpread = (spread)/2; // halfSpread cannot be negative
			if (halfSpread < maximumJump) maximumJump = halfSpread; // maximumJump cannot be negative
			// maximumJump cannot be negative, but can be zero
			if (thisGuessWasHigh)
			{
				guess = highestPossibleVal - maximumJump;
			}
			else
			{
				guess = lowestPossibleVal + maximumJump;
			}
		}
		else
		{  // found it we're done
		   // indices, not labels are stored, so the first particle will return foundIndex = 1
			foundIndex = getNthNameNL(uidList, guess);
			if (foundIndex == -1) foundIndex = 0;
			else *indexInUniqueIDList = guess;
		}
	}
 	return (foundIndex);
}
void	 simRebuildUniqueIDList (kSanSimulation *sim)
{ // unique ID list associates a particle ID with its index (where label = index - 1)
	long partArrIndex;
	particle *basePart;
	NLExpand(&sim->uniqueIDList, simGetNumParts(sim));
	setNumItemsNL(&sim->uniqueIDList, 0);

	basePart = simLockParts(sim);
	for (partArrIndex = 0; partArrIndex<simGetNumParts(sim); ++partArrIndex)
	{
	       addToNL(&sim->uniqueIDList, BPNthUniqueID(basePart, partArrIndex), (partArrIndex + 1));  // +1 because we are storing indices, not labels
	}
	sortNLByData (&sim->uniqueIDList);
	simReleaseParts(sim);
	sim->flags.uniqueIDListDirty = false;
}
