#include "kSanSimulationPrivate.h"
#include "kSanDialog.h"
#include "OHSharedUtils.h"
#include "kSanSharedUtils.h"
#include "noseCellResizePlugIn.h"
short	noseCellStressDialog	(noseCellResizePlugIn *ncr );


short initClassNoseCellResize (void)
{
	short err = noErr;
	kozoClass *newClass;
	kozoObject *co;

	newClass = addClass(&co,ClassNoseCellResizePlugIn, ClassCellResizePlugIn, sizeof (noseCellResizePlugIn), "Nos resize");
	if (newClass)
	{
		err =	setFuncs(newClass, 0L,  noseCellResizerConstructor,
		nilPointer, noseCellResizerGetData, noseCellResizerSetData, noseCellResizerDoMessage,
		 nilPointer, nilPointer, nilPointer, nilPointer);
		 err =  kozoClassAddRemove2ItemsToSave(newClass, true,  pNoseDensity,  pNoseTimeConst);
	}
	else
	{
		err = true;
	}
	return (err);

}
short noseCellResizerConstructor (KozoDispatchStack *ds, constructorMethodArgs *cma)
{
#pragma unused (cma)
	short err = noErr;
 	noseCellResizePlugIn *ncr;
 	kSanCellResizePlugIn *crpi;
 	crpi = KDSGetPrivateData(ds, ClassCellResizePlugIn );
 	ncr = KDSGetPrivateData(ds, ClassNoseCellResizePlugIn );
 	
 	crpi->crPlugPrivateData =  ncr;
 	crpi->doCellResize = noseCellResize;
	ncr->noseTimeConst = 0.1; // the time constant
	ncr->noseMass = 100; // the mass per area of the cell walls for nose dynamics
	ncr->lastResizeTime = 0.0;
	zerokSanVectorD(&ncr->noseVelocity); // the velocity of the cell walls for nose dynamics
	KDSReturnError(ds,err);
}
short  noseCellResizerGetData (KozoDispatchStack *ds, getMethodArgs *gma)
{
	short err = noErr;
 	noseCellResizePlugIn *ncr;

 	ncr = KDSGetPrivateData(ds, ClassNoseCellResizePlugIn );
	switch (gma->property)
	{
		case pNoseDensity:
			err = AECreateDesc(typeLongFloat, &ncr->noseMass, sizeof(double), gma->result);
			break;
		case pNoseTimeConst:
			err = AECreateDesc(typeLongFloat, &ncr->noseTimeConst, sizeof(double), gma->result);
			break;
		default:
			KDSReturnContinue(ds);
	}
	KDSDoneDispatch(ds);
	KDSReturnError(ds, err);
}
short  noseCellResizerSetData (KozoDispatchStack *ds, setMethodArgs *sma)
{
	short err = noErr;
 	noseCellResizePlugIn *ncr;

 	ncr = KDSGetPrivateData(ds, ClassNoseCellResizePlugIn );
	switch (sma->property)
	{
		case pNoseDensity:
			err = OHCoerceToDouble(sma->data, &ncr->noseMass);
			break;
		case pNoseTimeConst:
			err = OHCoerceToDouble(sma->data, &ncr->noseTimeConst);
			break;
		default:
			KDSReturnContinue(ds);
	}
	KDSDoneDispatch(ds);
	KDSReturnError(ds, err);
}
short  noseCellResizerDoMessage (KozoDispatchStack *ds, doMessageMethodArgs *dmma)
{
	short err = noErr;
	
 	noseCellResizePlugIn *ncr;

 	ncr = KDSGetPrivateData(ds, ClassNoseCellResizePlugIn );
	switch (dmma->message)
	{
		case kDoDialogMessage:
			err = noseCellStressDialog	(ncr );
			break;
		default :
			KDSReturnContinue(ds);
	}
	KDSDoneDispatch(ds);
	KDSReturnError(ds, err);
}

short noseCellResize ( kSanCellResizePlugIn *crpi,  kSanSimulation *sim)
{
	short err;
	noseCellResizePlugIn *ncr;
 	double ratio;
	double newVel;
	double delT;
	double damper;
	double delPos;
	kSanVector  linStress;
	kSanVector  appliedStress;
	
 	ncr =  crpi->crPlugPrivateData;
	delT = sim->totalTimeElapsed - ncr->lastResizeTime;
	ncr->lastResizeTime = sim->totalTimeElapsed ;
	if (delT > 100) delT = 100;
	
	damper = (1.0 / (1.0 + (delT / ncr->noseTimeConst)));
	if ((!sim->gConstrained.XAxis) || (!sim->gConstrained.YAxis) || (!sim->gConstrained.ZAxis) )
			err = simCalcLinearStress (sim, &linStress);
			
	// sim->strainStress is in MPa
	// applied stress is now in units of ((g/mol) / (A * ps * ps))
	multkSanVectorD(&sim->simCell.appliedStrainStress, kHellmanPressuresPerMPa, &appliedStress);
		
	// linear stress returned in units of eV per Angstrom cubed
	multkSanVectorD(&linStress, (-1.0 * kHellmanEnergiesPereV), &linStress);	
	// now, both stresses are in ((g/mol) / (A * ps * ps))
	// however, compressive stress is negative, so sign must change when changing the wall velocity
	//  thus the -1 here

	if (!sim->gConstrained.XAxis)
	{
		newVel = (ncr->noseVelocity.a * damper);
		newVel += ((linStress.a - appliedStress.a) * delT / ncr->noseMass) ;
		// noseMass in ((g/mol) / (A * A))    result is vel
		delPos = newVel * delT;
		while (fabs(delPos) >  0.01 * sim->simCell.cellDim.a)
		{
			newVel *= 0.9;
			delPos = newVel * delT;
		}
		ncr->noseVelocity.a = newVel;
		ratio = 1 + (delPos / sim->simCell.cellDim.a);
        scaleAxis (sim,pCellXLength, &ratio);
	}
	if (!sim->gConstrained.YAxis)
	{
		newVel = (ncr->noseVelocity.b * damper);
		newVel += ((linStress.b - appliedStress.b) * delT / ncr->noseMass) ;
		// noseMass in mass per area    stress in force per area    result is vel
		delPos = newVel * delT;
		while (fabs(delPos) >  0.01 * sim->simCell.cellDim.b)
		{
			newVel *= 0.9;
			delPos = newVel * delT;
		}
		ncr->noseVelocity.b = newVel;
		ratio = 1+ (delPos / sim->simCell.cellDim.b);
        scaleAxis (sim,pCellYLength, &ratio);
	}
#ifndef __2DVectors__
	if (!sim->gConstrained.ZAxis)
	{
		newVel = (ncr->noseVelocity.c * damper);
		newVel += ((linStress.c - appliedStress.c) * delT / ncr->noseMass) ;
		// noseMass in mass per area    stress in force per area    result is vel
		delPos = newVel * delT;
		while (fabs(delPos) > 0.01 * sim->simCell.cellDim.c)
		{
			newVel *= 0.9;
			delPos = newVel * delT;
		}
		ncr->noseVelocity.c = newVel;
		ratio = 1+ (delPos / sim->simCell.cellDim.c);
        scaleAxis (sim,pCellZLength, &ratio);
	}
#endif
//	if ((sim->gConstrained.Gamma) || (sim->gConstrained.Alpha) || (sim->gConstrained.Beta) )
//			err = simCalcShearStress (sim, &shearStress);
	return (err);
}

short	noseCellStressDialog	(noseCellResizePlugIn *ncr)
{
 	OHBasicDialog dialog;
 	setupAppDialog	(&dialog, kNoseCellResizeDialogResID);
	if (! dialog.done )
	{
		prepDoubleItem(dialog.dp, iNoseDensityItem , ncr->noseMass);
		prepDoubleItem(dialog.dp, iNoseTimeConstItem , ncr->noseTimeConst);
	}
	while ( ! dialog.done )
	{
		ModalDialog( nilPointer,&dialog.itemHit);
		switch (dialog.itemHit)
		{
			case ok:
			case cancel:
				dialog.done = true;
				break;		
		}
	}
	if  ((dialog.itemHit != cancel) && (dialog.itemHit != -1))
	{
		 ncr->noseMass = getDoubleItem(dialog.dp,iNoseDensityItem ,kMaxDouble, 0.000001);
		 ncr->noseTimeConst = getDoubleItem(dialog.dp,iNoseTimeConstItem ,kMaxDouble, 0.000001);
	}
	OHBreakdownDialog(&dialog);
	return (dialog.itemHit);
}
