#include "VerletMethod.h"
#include "kSanSharedUtils.h"
#define kOneOverRootThree (0.5773502692)
#define kOneOverRootTwo (0.707106781187)
#include "kSanSimulationPrivate.h"
#include "kSanMethodPlugIn.h"
#define bigVelocityErr -101
#define smallVelocityErr -102

short  VerletMethodConstructor (KozoDispatchStack *ds, constructorMethodArgs *cma);
short   VerletMethodDestructor (KozoDispatchStack *ds, destructorMethodArgs* dma);
short  VerletGetData (KozoDispatchStack *ds, getMethodArgs *gma);
 short  VerletSetData (KozoDispatchStack *ds, setMethodArgs *sma);
short  VerletMethodDoMessage (KozoDispatchStack *ds, doMessageMethodArgs *dmma);
short  verletChangePositions(kozoObject *mpiObj, VerletMethod *verlet,  particle *basePart  );
 short verletCalcVelocities(kozoObject *verletObj, VerletMethod *verlet,  particle *basePart );
short  VerletMethodSaveSig(kozoObject *verletObj,  VerletMethod *mpi);
void   initPrevPositions(kozoObject *verletObj, VerletMethod *verlet);
short  VerletReallocate(kozoObject *verletObj, VerletMethod *verlet );
short VerletChangedInfo (kozoObject *verletObj, VerletMethod *verlet,  whatWasChangedStruct *wwc);
short VerletIteration (  kSanMethodPlugIn *mpi,  kSanSimulation *sim) ;
short VerletCooling (  kSanMethodPlugIn *mpi,  kSanSimulation *sim, double *netTime, double *totKinEnergy);
 short 	verletEstimateVelocities(kozoObject *verletObj, VerletMethod *verlet,  particle *basePart  );

    
float  freeGroupCalcNextPositionsDifferentTimestep(nextGpPositionsInfo *ngpi);
float  freeGroupCalcNextPositionsSameTimestep(nextGpPositionsInfo *ngpi);
float  fixedGroupCalcNextPositions (nextGpPositionsInfo *ngpi);
 float  mixedGroupCalcNextPositionsDifferentTimestep(nextGpPositionsInfo *ngpi);
float  mixedGroupCalcNextPositionsSameTimestep(nextGpPositionsInfo *ngpi);
float  blockGroupCalcNextPositionsDifferentTimestep(nextGpPositionsInfo *ngpi);
float  blockGroupCalcNextPositionsSameTimestep(nextGpPositionsInfo *ngpi);

char verletSafeCalcNextPositions (kozoObject *mpiObj, VerletMethod *verlet,  particle *basePart, kSanGroup **baseGroup, long numGroups , char diffTimestep);


short initClassVerletMethod(void)
{
	short err = noErr;
	kozoClass *newClass;
	kozoObject *co;

	newClass = addClass(&co,ClassVerletMethod, ClassMethodPlugIn, sizeof (VerletMethod), "Verlet method");
	if (newClass)
	{
		err =	setFuncs(newClass, 0L,  VerletMethodConstructor,
		VerletMethodDestructor, VerletGetData, VerletSetData, VerletMethodDoMessage,
		 nilPointer, nilPointer, nilPointer, nilPointer);

		err =  kozoClassAddRemove2ItemsToSave(newClass, true,   pMaximumDisplacement,  pItersPerTimestep);
	}
	return (err);
}
short  VerletMethodConstructor (KozoDispatchStack *ds, constructorMethodArgs *cma)
{
#pragma unused (cma)
	short err = noErr;
	VerletMethod *verlet;
	kSanMethodPlugIn *mpi;
	kSanSimulation *sim = nilPointer;
 	mpi = KDSGetPrivateData(ds, ClassMethodPlugIn );
 	
	verlet  = (VerletMethod *) ds->thisPD;
	
	VSListInitialize (&verlet->nextDelPositions, sizeof(kSanVector ));
	VSListInitialize (&verlet->prevDelPositions, sizeof(kSanVector ));
	VSListInitialize (&verlet->prevForces, sizeof(kSanVector ));
  	verlet->dirtyNextPositions = true;
  	verlet->dirtyPrevPositions = true;
  	verlet->dirtyPrevForces = true;
	verlet->prevTimestep = 0.0;
	verlet->nextTimestep = 0.0;
	verlet->maxDelPos = 0.01;
	verlet->maxDelSafetyFactor = 0.5;
	verlet->verletItersPerSimIter = 1;
 	mpi->methodPlugPrivateData = (void *)verlet;
	mpi->doIteration = VerletIteration;
	mpi->doCooling = VerletCooling;
	
 // initialization of the value in verlet->sim takes place when kSetNewContainerMessage is called
 
 	KDSReturnError(ds,err);
}

short  VerletGetData (KozoDispatchStack *ds, getMethodArgs *gma)
{
	short err = noErr;
	VerletMethod *verlet;
  	
	verlet  = (VerletMethod *) ds->thisPD;
	switch (gma->property)
	{
		case pItersPerTimestep:
		 	err = makeLongDesc ( verlet->verletItersPerSimIter, gma->result);
			break;
		case pMaximumDisplacement:
		 	err = makeDoubleDesc ( verlet->maxDelPos,  gma->result);
			break;
 		default:
			KDSReturnContinue(ds);
	} 
	KDSDoneDispatch(ds);
	KDSReturnError(ds, err);
}
short  VerletSetData (KozoDispatchStack *ds, setMethodArgs *sma)
{
	short err = noErr;
	VerletMethod *verlet;
  	
	verlet  = (VerletMethod *) ds->thisPD;
	switch (sma->property)
	{
		case pItersPerTimestep:
		 	OHCoerceToLong(sma->data,  &verlet->verletItersPerSimIter );
			break;
		case pMaximumDisplacement:
		 	OHCoerceToDouble (sma->data, &verlet->maxDelPos);
			break;
 		default:
			KDSReturnContinue(ds);
	}
 	KDSDoneDispatch(ds);
	KDSReturnError(ds, err);
}
 
float    estHypotenuse(float a, float b);
float    estHypotenuse(float a, float b)
{  
	float pointTwoA;  
	float pointFourA;  
	float pointSixA;  
	float hypot ;
	
	 b = fabs(b);
	 a = fabs(a);
 	
    if (b > a)
    {
    	float temp = a;
    	a = b;
    	b = temp;
    }
    // now we know a is greater than b
    pointTwoA = a * 0.2;
    pointFourA = pointTwoA + pointTwoA;
    if ( b < pointFourA)
    {
        if (b < pointTwoA)
        {
	        hypot = a +  0.099 * b  ;
	    }
	    else
	    {
	        hypot = 1.0198 * a +  0.28615 * ( b - pointTwoA) ;
	    }
	}
	else
	{
		pointSixA = pointTwoA + pointFourA;
        if (b < pointSixA)
        {
	         hypot = (1.07703 * a) + 0.4458 * (b - pointFourA) ;
	    }
	    else
	    {
	         hypot = (1.16619 * a) + 0.62005 * (b - pointSixA) ;
	    }
    }
    return(hypot);
}

 
float    estVectorLength(float a, float b, float c);
float    estVectorLength(float a, float b, float c)
{  
    float len;
    
    len = estHypotenuse(a,b);
    len = estHypotenuse (len, c);
			  // all this just to avoid calculating a square root
	return (len);
}
 
float    getMaxDel(VerletMethod *verlet,  particle *basePart, long numParts);
float    getMaxDel(VerletMethod *verlet,  particle *basePart, long numParts)
{  
    float maxVel = 0;
	long partArrIndex;
	for (partArrIndex = 0; partArrIndex<numParts; ++partArrIndex)
	{
		float thisMax;
#ifndef __2DVectors__
		thisMax = estVectorLength(basePart->baseVelocity[partArrIndex].a, basePart->baseVelocity[partArrIndex].b, basePart->baseVelocity[partArrIndex].c);
#else
		thisMax = estVectorLength(basePart->baseVelocity[partArrIndex].a, basePart->baseVelocity[partArrIndex].b, 0L);
#endif
		if (thisMax > maxVel) maxVel = thisMax;
	}
    maxVel *= verlet->verletDeltaT; // turns maxVel into maxDel
    return (maxVel);
}


  
char 	verletSafeCalcNextPositions(kozoObject *mpiObj, VerletMethod *verlet,  particle *basePart, kSanGroup **baseGroup, long numGroups , char diffTimestep)
{
	short err = noErr;
	char changedDT;
	char unused;
#pragma unused(unused)
	long gpIndex;
	kSanGroup *gp;
	kSanSimulation *sim = verlet->sim;
	nextGpPositionsInfo ngpi;
	float maxDel ;
	float nthMaxDel = 0.0;
	
	ngpi.verletObj = mpiObj;
	ngpi.verlet = verlet;
	ngpi.sim =  sim;
	ngpi.basePart  = basePart;

	ngpi.baseAT  =simLockAtomTypes(sim);
 	ngpi.prevDelPosArray = VSListLock(&verlet->prevDelPositions );
	ngpi.nextDelPosArray = VSListLock(&verlet->nextDelPositions );
	ngpi.prevForceArray = VSListLock(&verlet->prevForces );

 	do
	{
		maxDel = 0.0;
	 	for(gpIndex=0;  gpIndex<numGroups; ++gpIndex)
		{
			gp =  baseGroup[gpIndex];
	 	    ngpi.gp = gp;
		    ngpi.gpPartList = lockList(&gp->particles);
		 	ngpi.numPartsInGroup = OHListCount (&gp->particles);
		 	
			if ((gp->XYZConstraints[0] == kGroupMovesFreely)  &&
				(gp->XYZConstraints[1] == kGroupMovesFreely)
#ifndef __2DVectors__
				  && (gp->XYZConstraints[2] == kGroupMovesFreely)
#endif
														  ) // totally free
			{
				if (diffTimestep) nthMaxDel = freeGroupCalcNextPositionsDifferentTimestep(&ngpi);
				else nthMaxDel = freeGroupCalcNextPositionsSameTimestep(&ngpi);
			}
			else if  ((gp->XYZConstraints[0] == 0)  &&
				(gp->XYZConstraints[1] == 0) 
#ifndef __2DVectors__
				 && (gp->XYZConstraints[2] == 0) 
#endif
				  								 ) // fixed 
			{
				 nthMaxDel = fixedGroupCalcNextPositions (&ngpi);
	 		}
			else if  ((gp->XYZConstraints[0] != kGroupMovesFreely)  &&
				(gp->XYZConstraints[1] != kGroupMovesFreely) 
#ifndef __2DVectors__
				 && (gp->XYZConstraints[2] != kGroupMovesFreely)
#endif
				 								                 ) // moves as a block in 1 2 or 3 axes
			{
				if (diffTimestep) nthMaxDel = blockGroupCalcNextPositionsDifferentTimestep(&ngpi);
				else nthMaxDel = blockGroupCalcNextPositionsSameTimestep(&ngpi);
			}
			else  // some other combination of fixed, block and free
			{
				if (diffTimestep) nthMaxDel = mixedGroupCalcNextPositionsDifferentTimestep(&ngpi);
				else nthMaxDel = mixedGroupCalcNextPositionsSameTimestep(&ngpi);
			}
			if (nthMaxDel > maxDel) maxDel = nthMaxDel;

		    releaseList(&gp->particles);

		}
	    // heres where we handle the case of too much change in position
		if (maxDel > verlet->maxDelPos)
		{
			verlet->verletItersPerSimIter *=  2;
			verlet->verletItersDone *=  2;
			verlet->verletDeltaT *= 0.5;
 			diffTimestep = true;
			verlet->numSequentialNoTimestepChange = 0;
		}
		else if (maxDel < (verlet->maxDelPos * verlet->maxDelSafetyFactor))
 		{
		     ++verlet->numSequentialNoTimestepChange;
		}
		else
		{
			verlet->numSequentialNoTimestepChange = 0;
		}
	}
	while (maxDel > verlet->maxDelPos);

    verlet->nextTimestep = verlet->verletDeltaT;
    
	verlet->dirtyNextPositions = false;
 	simReleaseAtomTypes(sim);
	VSListRelease(&verlet->prevDelPositions );
	VSListRelease(&verlet->nextDelPositions );
	VSListRelease(&verlet->prevForces ); 
	
	
	return (changedDT);
}

float   freeGroupCalcNextPositionsDifferentTimestep(nextGpPositionsInfo *ngpi)
{
// this func will return the maxDel
 	short err = noErr;
 	long gpIndex;
 	particle *basePart = ngpi->basePart;
 	double dT = ngpi->verlet->verletDeltaT;
  	double dTSqu ;
  	double prevDelTSqu ;
	float maxDel = 0.0;
#ifndef __2DVectors__
	float maxDelOverRootThree = 0.0;
#else
 	float maxDelOverRootTwo = 0.0;
#endif    

	kSanVector  nthDel  ;
	double prevDT  ;
	double invDelT  ;
	double invPrevDelT  ;
	double delTDiff  ;
	double delTFrac  ;
	double delTSquPairSumOverTwo  ;
	double delTCubeDiffOverSix  ;

	double velWeight  ;
	double diffweight  ;

	prevDT =  ngpi->verlet->prevTimestep ;
 	velWeight = dT / ( prevDT + dT);
 	diffweight = 1.0 - velWeight;

	 dTSqu = dT * dT;
	 delTFrac = dT / prevDT ;
	 prevDelTSqu = prevDT * prevDT;
	 invDelT = 1.0 / ngpi->verlet->verletDeltaT ;
	 invPrevDelT = 1.0 / prevDT ;
	 delTDiff = ngpi->verlet->verletDeltaT - prevDT;
	 delTSquPairSumOverTwo = 0.5 * (dTSqu + prevDelTSqu) ;
	 delTCubeDiffOverSix = (1.0 / 6.0) * (dTSqu * dT - prevDelTSqu * prevDT) ;
 
//  the timestep was changed on us, so we have to include the odd terms in the taylor expansion --> they don't cancel out

  	for (gpIndex = 0 ; gpIndex < ngpi->numPartsInGroup; ++gpIndex)
	{
		long partArrIndex;
		double invMass ;
		
		double delTSquPairSumOverTwoMass ;
		double delTCubeDiffOverSixMass ;
	    partArrIndex = ngpi->gpPartList[gpIndex];

	// we have to use a full taylor expansion here
	     invMass = ngpi->baseAT[basePart->basePi[partArrIndex].type]->specs.inverseMass;
		 delTSquPairSumOverTwoMass = delTSquPairSumOverTwo * invMass;
		 delTCubeDiffOverSixMass = delTCubeDiffOverSix * invMass;

 	   // the two velocity components are weighted based on 
 	   // the fraction of the current dT compared to the previous dt
 	   // velweight is the weight applied to the velocity at this time
 	   // diffweight is the weight applied to the delPos
 	   
 	    multkSanVectorD ( &ngpi->prevDelPosArray[partArrIndex], diffweight * delTFrac , &nthDel );
		// nthDel.a =   ngpi->prevDelPosArray[partArrIndex].a * diffweight * delTFrac ;
		// nthDel.b =   ngpi->prevDelPosArray[partArrIndex].b * diffweight * delTFrac ;
		//  nthDel.c =   ngpi->prevDelPosArray[partArrIndex].c * diffweight * delTFrac ;

 	   // first order
 	    multAddkSanVectorsD (&nthDel, &basePart->baseVelocity[partArrIndex], velWeight * dT , &nthDel );
 		//nthDel.a +=   basePart->baseVelocity[partArrIndex].a * velWeight * dT  ;
		//nthDel.b +=   basePart->baseVelocity[partArrIndex].b * velWeight * dT  ;
		//nthDel.c +=   basePart->baseVelocity[partArrIndex].c * velWeight * dT  ;
		
	   // second order
 	    multAddkSanVectorsD (&nthDel, &basePart->baseForce[partArrIndex], delTSquPairSumOverTwoMass , &nthDel );
		//nthDel.a +=   basePart->baseForce[partArrIndex].a * delTSquPairSumOverTwoMass  ;
		//nthDel.b +=   basePart->baseForce[partArrIndex].b * delTSquPairSumOverTwoMass  ;
		//nthDel.c +=   basePart->baseForce[partArrIndex].c * delTSquPairSumOverTwoMass  ;
		
	   // third order
 
 		nthDel.a +=   (basePart->baseForce[partArrIndex].a - ngpi->prevForceArray[partArrIndex].a)  * invPrevDelT * delTCubeDiffOverSixMass  ;
		nthDel.b +=   (basePart->baseForce[partArrIndex].b - ngpi->prevForceArray[partArrIndex].b)  * invPrevDelT * delTCubeDiffOverSixMass  ;
#ifndef __2DVectors__
		nthDel.c +=   (basePart->baseForce[partArrIndex].c - ngpi->prevForceArray[partArrIndex].c)  * invPrevDelT * delTCubeDiffOverSixMass  ;
#endif
        ngpi->nextDelPosArray[partArrIndex] = nthDel;
		//ngpi->nextDelPosArray[partArrIndex].a =  nthDel.a    ;
		//ngpi->nextDelPosArray[partArrIndex].b =  nthDel.b    ;
		//ngpi->nextDelPosArray[partArrIndex].c =  nthDel.c    ;
		
#ifndef __2DVectors__
		if ((maxDelOverRootThree < fabs(nthDel.a)) || (maxDelOverRootThree < fabs(nthDel.b)) || (maxDelOverRootThree < fabs(nthDel.c)) )
#else
		if ((maxDelOverRootTwo < fabs(nthDel.a)) || (maxDelOverRootTwo < fabs(nthDel.b)) )
#endif
		{
		    float thisDel  ;
#ifndef __2DVectors__
		    thisDel = estVectorLength(nthDel.a, nthDel.b, nthDel.c);
#else
		    thisDel = estVectorLength(nthDel.a, nthDel.b, 0);
#endif
		    if (thisDel > maxDel)
			{
				maxDel = thisDel;
#ifndef __2DVectors__
				maxDelOverRootThree = maxDel * kOneOverRootThree;
#else
				maxDelOverRootTwo= maxDel * kOneOverRootTwo;
#endif
				
			}
		}
	}

 	return(maxDel);
}
float   freeGroupCalcNextPositionsSameTimestep(nextGpPositionsInfo *ngpi)
{
// this func will return the maxDel
 	short err = noErr;
 	long partArrIndex;
 	long gpIndex;
  	particle *basePart = ngpi->basePart;
	kSanSimulation *sim = ngpi->sim;
 	float maxDel = 0.0;
	float maxDelOverRootThree = 0.0;
 //  the timestep was not changed, so we the odd terms in the taylor expansion cancel out, and the expression is simple
    

	double delTSqu;
	kSanVector  nthDel  ;
	double delTSquOverMass;

 	delTSqu = ngpi->verlet->verletDeltaT * ngpi->verlet->verletDeltaT ;
 	for (gpIndex = 0; gpIndex< ngpi->numPartsInGroup; ++gpIndex)
	{
	    partArrIndex = ngpi->gpPartList[gpIndex];

		delTSquOverMass = delTSqu * ngpi->baseAT[basePart->basePi[partArrIndex].type]->specs.inverseMass;

		nthDel = ngpi->prevDelPosArray[partArrIndex];
		multAddkSanVectorsD (&nthDel,  &basePart->baseForce[partArrIndex], delTSquOverMass, &nthDel);
		
		//nthDel.a =     ngpi->prevDelPosArray[partArrIndex].a   +  basePart->baseForce[partArrIndex].a * delTSquOverMass ;
		//nthDel.b =     ngpi->prevDelPosArray[partArrIndex].b   +  basePart->baseForce[partArrIndex].b * delTSquOverMass ;
		//nthDel.c =     ngpi->prevDelPosArray[partArrIndex].c   +  basePart->baseForce[partArrIndex].c * delTSquOverMass ;

		ngpi->nextDelPosArray[partArrIndex] = nthDel;
		//ngpi->nextDelPosArray[partArrIndex].a =   nthDel.a    ;
		//ngpi->nextDelPosArray[partArrIndex].b =   nthDel.b    ;
		//ngpi->nextDelPosArray[partArrIndex].c =   nthDel.c    ;
		
#ifndef __2DVectors__
		if ((maxDelOverRootThree < fabs(nthDel.a)) || (maxDelOverRootThree < fabs(nthDel.b)) || (maxDelOverRootThree < fabs(nthDel.c)) )
#else
		if ((maxDelOverRootThree < fabs(nthDel.a)) || (maxDelOverRootThree < fabs(nthDel.b)) )
#endif
 		{
 		    float thisDel  ;
#ifndef __2DVectors__
		    thisDel = estVectorLength(nthDel.a, nthDel.b, nthDel.c);
#else
		    thisDel = estVectorLength(nthDel.a, nthDel.b, 0);
#endif
		    if (thisDel > maxDel)
			{
				maxDel = thisDel;
				maxDelOverRootThree = maxDel * kOneOverRootThree;
			}
		}
	}

 	return (maxDel);
}
float   blockGroupCalcNextPositionsSameTimestep(nextGpPositionsInfo *ngpi)
{
// this func will return the maxDel
 	short err = noErr;
 	long partArrIndex;
 	long gpIndex;
  	particle *basePart = ngpi->basePart;
	kSanSimulation *sim = ngpi->sim;
 	float maxDel = 0.0;
	float maxDelOverRootThree = 0.0;
 //  the timestep was not changed, so we the odd terms in the taylor expansion cancel out, and the expression is simple
    

 	double delTSqu;
	double thisMass;
	double totalMass;
 	kSanVector  delPos  ;
	kSanVector  inertia  ;
	kSanVector  forceSum  ;
	double delTSquOverMass;

	zerokSanVectorD (&inertia);
	zerokSanVectorD (&forceSum);
	zerokSanVectorD (&delPos);
	
 	delTSqu = ngpi->verlet->verletDeltaT * ngpi->verlet->verletDeltaT ;
 	for (gpIndex = 0; gpIndex< ngpi->numPartsInGroup; ++gpIndex)
	{
	    partArrIndex = ngpi->gpPartList[gpIndex];

 		thisMass =  ngpi->baseAT[basePart->basePi[partArrIndex].type]->specs.mass;
		totalMass += thisMass;

		 multAddkSanVectorsD (&inertia,  &ngpi->prevDelPosArray[partArrIndex], thisMass, &inertia);
		//inertia.a += ( ngpi->prevDelPosArray[partArrIndex].a ) * thisMass ;
		//inertia.b += ( ngpi->prevDelPosArray[partArrIndex].b ) * thisMass ;
		//inertia.c += ( ngpi->prevDelPosArray[partArrIndex].c ) * thisMass ;

		addkSanVectorsD (&forceSum, &basePart->baseForce[partArrIndex], &forceSum);  
		//forceSum.a += basePart->baseForce[partArrIndex].a;
		//forceSum.b += basePart->baseForce[partArrIndex].b;
		//forceSum.c += basePart->baseForce[partArrIndex].c;
 	}	
 	
 	multkSanVectorD (  &inertia , 1.0 /  totalMass, &inertia);
 	//inertia.a  = inertia.a / totalMass;
 	//inertia.b  = inertia.b / totalMass;
 	//inertia.c  = inertia.c / totalMass;
 
  	delTSquOverMass = ngpi->verlet->verletDeltaT * ngpi->verlet->verletDeltaT / totalMass;
	
// 	delX = inertiaX * ( verlet->verletDeltaT / ngpi->verlet->previousTimestep ) + forceX * delTSquOverMass;
//	delY = inertiaY * ( verlet->verletDeltaT / ngpi->verlet->previousTimestep ) + forceY * delTSquOverMass;
// 	delZ = inertiaZ * ( verlet->verletDeltaT / ngpi->verlet->previousTimestep ) + forceZ * delTSquOverMass;

	 multAddkSanVectorsD (&inertia,  &forceSum, delTSquOverMass, &delPos);
 	//delPos.a = inertia.a   + forceSum.a * delTSquOverMass;
	//delPos.b = inertia.b   + forceSum.b * delTSquOverMass;
 	//delPos.c = inertia.c   + forceSum.c * delTSquOverMass;

 	for (gpIndex = 0; gpIndex< ngpi->numPartsInGroup; ++gpIndex)
	{
	    partArrIndex = ngpi->gpPartList[gpIndex];
	    
		ngpi->nextDelPosArray[partArrIndex]  =   delPos    ;
		//ngpi->nextDelPosArray[partArrIndex].a =   delPos.a    ;
		//ngpi->nextDelPosArray[partArrIndex].b =   delPos.b    ;
		//ngpi->nextDelPosArray[partArrIndex].c =   delPos.c    ;
  	}	


#ifndef __2DVectors__
    maxDel = estVectorLength(delPos.a, delPos.b, delPos.c);
#else 
    maxDel = estVectorLength(delPos.a, delPos.b, 0.0);
#endif  

 	return (maxDel);
}
float   blockGroupCalcNextPositionsDifferentTimestep(nextGpPositionsInfo *ngpi)
{
// this func will return the maxDel
 	short err = noErr;
 	long partArrIndex;
 	long gpIndex;
  	particle *basePart = ngpi->basePart;
	kSanSimulation *sim = ngpi->sim;
 	float maxDel = 0.0;
	float maxDelOverRootThree = 0.0;
 //  the timestep was not changed, so we the odd terms in the taylor expansion cancel out, and the expression is simple
    
 	double dT =  ngpi->verlet->verletDeltaT;
 	double dTSqu;
	double prevDelTSqu;
	double invDelT;
	double delTDiff;
	kSanVector  delPos  ;
	kSanVector  prevForce  ;
	kSanVector  sumForce  ;
	kSanVector  inertia  ;
 	double delTSquPairSumOverTwoMass ;
	double delTCubeDiffOverSixMass ;

	double delTSquPairSumOverTwo  ;
	double delTCubeDiffOverSix  ;
	double thisMass  ;
	double totalMass  ;

	double velWeight  ;
	double diffweight  ;
	double invPrevDelT  ;
	double prevDT ;
	double delTFrac  ;
	prevDT = ngpi->verlet->prevTimestep ;
 	velWeight = dT / ( prevDT + dT);
 	diffweight = 1.0 - velWeight;


	 dTSqu = dT * dT;
	 invPrevDelT = 1.0 / prevDT;
	 prevDelTSqu = prevDT * prevDT;
	 invDelT = 1.0 / ngpi->verlet->verletDeltaT ;
	 delTDiff = ngpi->verlet->verletDeltaT - prevDT;
	 delTSquPairSumOverTwo = 0.5 * (dTSqu + prevDelTSqu) ;
	 delTCubeDiffOverSix = (1.0 / 6.0) * (dTSqu * dT - prevDelTSqu * prevDT) ;
	// we have to use a full taylor expansion here

	zerokSanVectorD(&prevForce );
	zerokSanVectorD(&delPos );
	zerokSanVectorD(&sumForce );
	zerokSanVectorD(&inertia );
 		
  	for (gpIndex = 0; gpIndex< ngpi->numPartsInGroup; ++gpIndex)
	{
	    partArrIndex = ngpi->gpPartList[gpIndex];

 		thisMass =  ngpi->baseAT[basePart->basePi[partArrIndex].type]->specs.mass;
		totalMass += thisMass;
		
		multAddkSanVectorsD(&inertia , &ngpi->prevDelPosArray[partArrIndex] , thisMass, &inertia );
		//inertia.a +=   ngpi->prevDelPosArray[partArrIndex].a   * thisMass ;
		//inertia.b +=   ngpi->prevDelPosArray[partArrIndex].b   * thisMass ;
		//inertia.c +=   ngpi->prevDelPosArray[partArrIndex].c   * thisMass ;
		
		addkSanVectorsD (&sumForce , &basePart->baseForce[partArrIndex] , &sumForce );
		//sumForce.a += basePart->baseForce[partArrIndex].a;
		//sumForce.b += basePart->baseForce[partArrIndex].b;
		//sumForce.c += basePart->baseForce[partArrIndex].c;
		
		
		addkSanVectorsD (&prevForce , &ngpi->prevForceArray[partArrIndex] , &prevForce );
		//prevForce.a += ngpi->prevForceArray[partArrIndex].a ;
		//prevForce.b += ngpi->prevForceArray[partArrIndex].b ;
		//prevForce.c += ngpi->prevForceArray[partArrIndex].c ;
 	}	
 	
 	multkSanVectorD (&inertia,  1.0 / totalMass , &inertia );
 	//inertia.a  = inertia.a / totalMass;
 	//inertia.b  = inertia.b / totalMass;
 	//inertia.c  = inertia.c / totalMass;
 
 	delTSquPairSumOverTwoMass = delTSquPairSumOverTwo / totalMass;
	delTCubeDiffOverSixMass = delTCubeDiffOverSix / totalMass;

// first order	
	delTFrac = ( ngpi->verlet->verletDeltaT / prevDT ) ;
 	multkSanVectorD (&inertia, delTFrac , &delPos );
  	//delPos.a = inertia.a * delTFrac ;
 	//delPos.b = inertia.b * delTFrac ;
  	//delPos.c = inertia.c * delTFrac ;
 
// second order	
	multAddkSanVectorsD(&delPos ,&sumForce , delTSquPairSumOverTwoMass, &delPos );
  	//delPos.a +=   sumForce.a * delTSquPairSumOverTwoMass;
 	//delPos.b +=   sumForce.b * delTSquPairSumOverTwoMass;
  	//delPos.c +=   sumForce.c * delTSquPairSumOverTwoMass;

// third order	
   	delPos.a +=   (sumForce.a - prevForce.a)  * invPrevDelT * delTCubeDiffOverSixMass;
 	delPos.b +=   (sumForce.b - prevForce.b)  * invPrevDelT * delTCubeDiffOverSixMass;
#ifndef __2DVectors__
  	delPos.c +=   (sumForce.c - prevForce.c)  * invPrevDelT * delTCubeDiffOverSixMass;
#endif
 
  	for (gpIndex = 0; gpIndex< ngpi->numPartsInGroup; ++gpIndex)
	{
	    partArrIndex = ngpi->gpPartList[gpIndex];
		ngpi->nextDelPosArray[partArrIndex]  =   delPos ;
   	}	

#ifndef __2DVectors__
    maxDel = estVectorLength(delPos.a, delPos.b, delPos.c);
#else 
    maxDel = estVectorLength(delPos.a, delPos.b, 0.0);
#endif  
  
 	return (maxDel);
}



float   mixedGroupCalcNextPositionsSameTimestep(nextGpPositionsInfo *ngpi)
{
// this func will return the maxDel
 	short err = noErr;
 	short axis;
 	long partArrIndex;
 	long gpIndex;
  	particle *basePart = ngpi->basePart;
	kSanSimulation *sim = ngpi->sim;
 	float maxDel = 0.0;
	float maxDelOverRootThree = 0.0;
	double dT ;
 	double dTSqu;
	double  nthDel  ;
	//double delTSquOverTwoMass;
	double delTSquOverMass;
	
 	dT = ngpi->verlet->verletDeltaT;
 	dTSqu = ngpi->verlet->verletDeltaT * ngpi->verlet->verletDeltaT ;

	for (axis = 0; axis < NUMDIMENSIONS; ++axis)
	{
		if (ngpi->gp->XYZConstraints[axis] == kGroupMovesFreely) 
		{
			for (gpIndex = 0; gpIndex< ngpi->numPartsInGroup; ++gpIndex)
			{
			    partArrIndex = ngpi->gpPartList[gpIndex];
		        delTSquOverMass = dTSqu * ngpi->baseAT[basePart->basePi[partArrIndex].type]->specs.inverseMass;

		        nthDel  =  ((double *)(&ngpi->prevDelPosArray[partArrIndex]))[axis]  + ((double *)(&basePart->baseForce[partArrIndex]))[axis]  * delTSquOverMass;

		        ((double *)(&ngpi->nextDelPosArray[partArrIndex]))[axis] =   nthDel  ;
  			}
		}
		else if (ngpi->gp->XYZConstraints[axis] == kGroupMovesAsABlock) 
		{
			double inertia = 0.0;
		 	double forceSum = 0.0;
 			double totalMass = 0.0;
			double thisMass = 0.0;
		  	
			for (gpIndex = 0; gpIndex< ngpi->numPartsInGroup; ++gpIndex)
			{
				partArrIndex = ngpi->gpPartList[gpIndex];
 				thisMass = ngpi->baseAT[basePart->basePi[partArrIndex].type]->specs.mass;
				totalMass += thisMass;
				inertia +=   ((double *)(&ngpi->prevDelPosArray[partArrIndex]))[axis]  * thisMass ;
				forceSum += ((double *)(&basePart->baseForce[partArrIndex]))[axis];
		 	}	
		 	{
 				double invTotalMass ;
		 		invTotalMass = 1.0 / totalMass;
		 		inertia  = inertia * invTotalMass;
		  		delTSquOverMass = dTSqu * invTotalMass;
		  	}
		 	nthDel = inertia + forceSum * delTSquOverMass;

			for (gpIndex = 0; gpIndex< ngpi->numPartsInGroup; ++gpIndex)
			{
				partArrIndex = ngpi->gpPartList[gpIndex];
				
				((double *)(&ngpi->nextDelPosArray[partArrIndex]))[axis] =   nthDel  ;
		 	}	
		}
		else // axis is fixed
		{
			for (gpIndex = 0; gpIndex< ngpi->numPartsInGroup; ++gpIndex)
			{
				partArrIndex = ngpi->gpPartList[gpIndex];
				((double *)(&ngpi->nextDelPosArray[partArrIndex]))[axis] = 0.0 ;
		    }
		   // ((double *)(&baseNextPos[ithIndex]))[axis] = ((double *)(&thisPart->position))[axis];
 		}
	}
 
 	for (gpIndex = 0; gpIndex< ngpi->numPartsInGroup; ++gpIndex)
	{
		kSanVector  nthDel;
	   partArrIndex = ngpi->gpPartList[gpIndex];

		nthDel  =   ngpi->nextDelPosArray[partArrIndex]   ;

#ifndef __2DVectors__
 		if ((maxDelOverRootThree < fabs(nthDel.a)) || (maxDelOverRootThree < fabs(nthDel.b)) || (maxDelOverRootThree < fabs(nthDel.c)) )
#else 
 		if ((maxDelOverRootThree < fabs(nthDel.a)) || (maxDelOverRootThree < fabs(nthDel.b))  )
#endif  
		{
		    float thisDel  ;
#ifndef __2DVectors__
    		thisDel = estVectorLength(nthDel.a, nthDel.b, nthDel.c);
#else 
   			thisDel = estVectorLength(nthDel.a, nthDel.b, 0.0);
#endif  

		    if (thisDel > maxDel)
			{
				maxDel = thisDel;
				maxDelOverRootThree = maxDel * kOneOverRootThree;
			}
		}
	}

 	return (maxDel);
}


float   mixedGroupCalcNextPositionsDifferentTimestep(nextGpPositionsInfo *ngpi)
{
// this is the ugliest of the functions
// we have to account for the fact that this group's particles have different constraints on each of the axes,
// and the timestep used for the previous calculation was not the one which we wilkl use for this calculation


 	short err = noErr;
 	short axis;
 	long partArrIndex;
 	long gpIndex;
  	particle *basePart = ngpi->basePart;
	kSanSimulation *sim = ngpi->sim;
 	float maxDel = 0.0;
	float maxDelOverRootThree = 0.0;
 	double dT ;
 	double dTSqu;
  	double prevDelTSqu ;
	double nthDel  ;
	double delTDiff  ;
	double delTFrac  ;
	double delTSquPairSumOverTwo  ;
	double delTCubeDiffOverSix  ;
 	double delTSquPairSumOverTwoMass ;
	double delTCubeDiffOverSixMass ;
	double invDelT ;
	
	double prevDT =   ngpi->verlet->prevTimestep ;
	double velWeight  ;
	double diffweight  ;

 	velWeight = dT / ( prevDT + dT);
 	diffweight = 1.0 - velWeight;
 	 dT = ngpi->verlet->verletDeltaT;
 	 delTFrac = dT /  prevDT;
 	 invDelT = 1.0 / dT;
 	 dTSqu = ngpi->verlet->verletDeltaT * ngpi->verlet->verletDeltaT ;
	 prevDelTSqu = prevDT * prevDT;
	 delTDiff = ngpi->verlet->verletDeltaT - prevDT;
	 delTSquPairSumOverTwo = 0.5 * (dTSqu + prevDelTSqu) ;
	 delTCubeDiffOverSix = (1.0 / 6.0) * (dTSqu * dT - prevDelTSqu * prevDT) ;

	for (axis = 0; axis < NUMDIMENSIONS; ++axis)
	{
		if (ngpi->gp->XYZConstraints[axis] == kGroupMovesFreely) 
		{
			for (gpIndex = 0; gpIndex< ngpi->numPartsInGroup; ++gpIndex)
			{
			    partArrIndex = ngpi->gpPartList[gpIndex];
		        delTSquPairSumOverTwoMass = delTSquPairSumOverTwo * ngpi->baseAT[basePart->basePi[partArrIndex].type]->specs.inverseMass;
				delTCubeDiffOverSixMass = delTCubeDiffOverSix *  ngpi->baseAT[basePart->basePi[partArrIndex].type]->specs.inverseMass;
				
			   // zeroeth order
		        nthDel  =    ((double *)(&ngpi->prevDelPosArray[partArrIndex]))[axis]  * diffweight * delTFrac;
		       
			   // first order
				nthDel  +=   ((double *)(&basePart->baseVelocity[partArrIndex]))[axis]  * velWeight * dT;

		       // second order
		        nthDel  +=    ((double *)(&basePart->baseForce[partArrIndex]))[axis]  * delTSquPairSumOverTwoMass;

			   // third order
				nthDel  +=   (((double *)(&basePart->baseForce[partArrIndex].a))[axis] - ((double *)(&ngpi->prevForceArray[partArrIndex]))[axis])  * invDelT * delTCubeDiffOverSixMass  ;

		        ((double *)(&ngpi->nextDelPosArray[partArrIndex]))[axis] =   nthDel  ;
  			}
		}
		else if (ngpi->gp->XYZConstraints[axis] == kGroupMovesAsABlock) 
		{
			double delInertia = 0.0;
			double velInertia = 0.0;
		 	double sumForce = 0.0;
		 	double prevForce = 0.0;
 			double totalMass = 0.0;
		    double thisMass = 0.0;
 		  	
			for (gpIndex = 0; gpIndex< ngpi->numPartsInGroup; ++gpIndex)
			{
				partArrIndex = ngpi->gpPartList[gpIndex];
 				thisMass = ngpi->baseAT[basePart->basePi[partArrIndex].type]->specs.mass;
				totalMass += thisMass;
				delInertia +=    ((double *)(&ngpi->prevDelPosArray[partArrIndex]))[axis]  * thisMass ;
				velInertia +=    ((double *)(&basePart->baseVelocity[partArrIndex]))[axis]  * thisMass ;
				sumForce += ((double *)(&basePart->baseForce[partArrIndex]))[axis];
				prevForce += ((double *)(&ngpi->prevForceArray[partArrIndex]))[axis];
		 	}	
		// -- we have to do the velweight / delweight thing here.	
		 	delInertia  = delInertia / totalMass;
		 	velInertia  = velInertia / totalMass;
		  //	delTSquOverMass = verlet->verletDeltaT * verlet->verletDeltaT / totalMass;
		  	 delTSquPairSumOverTwoMass = delTSquPairSumOverTwo / totalMass;
	         delTCubeDiffOverSixMass = delTCubeDiffOverSix / totalMass;

         // first order	
		 	nthDel = delInertia  * diffweight * delTFrac;
 
		 	nthDel += velInertia  * velWeight * dT;
 
        // second order	
		 	nthDel += sumForce  * delTSquPairSumOverTwoMass;
 
        // third order	
		 	nthDel += (sumForce  - prevForce )  * invDelT * delTCubeDiffOverSixMass;

			for (gpIndex = 0; gpIndex< ngpi->numPartsInGroup; ++gpIndex)
			{
				partArrIndex = ngpi->gpPartList[gpIndex];
				
				((double *)(&ngpi->nextDelPosArray[partArrIndex]))[axis] =   nthDel  ;
		 	}	
		}
		else // axis is fixed
		{
			for (gpIndex = 0; gpIndex< ngpi->numPartsInGroup; ++gpIndex)
			{
				partArrIndex = ngpi->gpPartList[gpIndex];
				((double *)(&ngpi->nextDelPosArray[partArrIndex]))[axis] = 0.0 ;
		    }
		   // ((double *)(&baseNextPos[ithIndex]))[axis] = ((double *)(&thisPart->position))[axis];
 		}
	}
 
 	for (gpIndex = 0; gpIndex< ngpi->numPartsInGroup; ++gpIndex)
	{
		kSanVector  nthDel;
	    partArrIndex = ngpi->gpPartList[gpIndex];

		nthDel  =   ngpi->nextDelPosArray[partArrIndex]   ;

#ifndef __2DVectors__
 		if ((maxDelOverRootThree < fabs(nthDel.a)) || (maxDelOverRootThree < fabs(nthDel.b)) || (maxDelOverRootThree < fabs(nthDel.c)) )
#else
 		if ((maxDelOverRootThree < fabs(nthDel.a)) || (maxDelOverRootThree < fabs(nthDel.b))  )
#endif
		{
		    float thisDel  ;
#ifndef __2DVectors__
    		thisDel = estVectorLength(nthDel.a, nthDel.b, nthDel.c);
#else 
   			thisDel = estVectorLength(nthDel.a, nthDel.b, 0.0);
#endif  
		    if (thisDel > maxDel)
			{
				maxDel = thisDel;
				maxDelOverRootThree = maxDel * kOneOverRootThree;
			}
		}
	}

 	return (maxDel);
}



float   fixedGroupCalcNextPositions(nextGpPositionsInfo *ngpi)
{
// this func will return the maxDel
 	long partArrIndex;
 	long gpIndex;
  	particle *basePart = ngpi->basePart;
 	float maxDel = 0.0;
 //  the timestep was not changed, so we the odd terms in the taylor expansion cancel out, and the expression is simple
    

 	for (gpIndex = 0; gpIndex< ngpi->numPartsInGroup; ++gpIndex)
	{
	    partArrIndex = ngpi->gpPartList[gpIndex];

		zerokSanVectorD (&ngpi->nextDelPosArray[partArrIndex])  ;
		
	}

 	return (maxDel);
}

 
void   initPrevPositions(kozoObject *verletObj, VerletMethod *verlet)
{ // we need to go back in time to calculate the previous positions.  then calculate the forces in this configuration.
  // we'll need to temporarily save current positions
  // we need to assume that the current velocity settings are correct
#pragma unused(verletObj)
 	short err = noErr;
	particle *basePart;
	long partArrIndex;
	long numParts;	
	kSanSimulation *sim = verlet->sim;
	kSanAtomtype **baseAT;
 	double delT ;
  	double delTSquOverTwo ;
 	double maxDel ;
 	 if (sim)
 	 {
	 	dispatchCount(simOBJ(sim), ClassParticle, &numParts);
		// first make sure the lists are all the right size
		
		err = VSListSetNumItems(&verlet->nextDelPositions, numParts);
		if (!err) err = VSListSetNumItems (&verlet->prevDelPositions, numParts);
		if (!err) err = VSListSetNumItems (&verlet->prevForces, numParts);
		if (!err)
		{
			kSanVector  *prevDelPosArray = VSListLock(&verlet->prevDelPositions );
 			kSanVector  *prevForceArray = VSListLock(&verlet->prevForces );
		 	basePart = simLockParts(sim);
			baseAT = simLockAtomTypes(sim);
	 		
			maxDel = getMaxDel (verlet, basePart,  numParts);
			if ( maxDel > verlet->maxDelPos)
			{
				verlet->verletItersPerSimIter = 1 + (long) (maxDel / verlet->maxDelPos);
 			}
			verlet->verletDeltaT = sim->deltaT / ((float)verlet->verletItersPerSimIter);
	        delT = verlet->verletDeltaT;
 	        verlet->prevTimestep = verlet->verletDeltaT;
	        
		    err = calculateForce(sim); 
			// save the current positions
			// set the positions to values from previous timestep
 			delTSquOverTwo =  delT * delT  * 0.5;
	   		for (partArrIndex = 0; partArrIndex<numParts; ++partArrIndex)
			{
				double delTSquOverTwoMass = delTSquOverTwo * baseAT[basePart->basePi[partArrIndex].type]->specs.inverseMass;

				multkSanVectorD (&basePart->baseVelocity[partArrIndex], - delT,  &prevDelPosArray[partArrIndex]);
				//prevDelPosArray[partArrIndex].a =   - basePart->baseVelocity[partArrIndex].a * delT;
				//prevDelPosArray[partArrIndex].b =   - basePart->baseVelocity[partArrIndex].b * delT;
				//prevDelPosArray[partArrIndex].c =   - basePart->baseVelocity[partArrIndex].c * delT;

				multAddkSanVectorsD (&prevDelPosArray[partArrIndex], &basePart->baseForce[partArrIndex], delTSquOverTwoMass,  &prevDelPosArray[partArrIndex]);
				//prevDelPosArray[partArrIndex].a += basePart->baseForce[partArrIndex].a * delTSquOverTwoMass;
				//prevDelPosArray[partArrIndex].a += basePart->baseForce[partArrIndex].b * delTSquOverTwoMass;
				//prevDelPosArray[partArrIndex].a += basePart->baseForce[partArrIndex].c * delTSquOverTwoMass;

				subkSanVectorsD (&basePart->basePosition[partArrIndex], &prevDelPosArray[partArrIndex] , &basePart->basePosition[partArrIndex] );
				//basePart->basePosition[partArrIndex].a -= prevDelPosArray[partArrIndex].a;
				//basePart->basePosition[partArrIndex].b -= prevDelPosArray[partArrIndex].b;
				//basePart->basePosition[partArrIndex].c -= prevDelPosArray[partArrIndex].c;

 			}

		    err = calculateForce(sim); 
		    
		    // copy forces and previous positions
		    // and reassign current positions
 	   		for (partArrIndex = 0; partArrIndex<numParts; ++partArrIndex)
			{
				
				prevForceArray[partArrIndex] = basePart->baseForce[partArrIndex];
				
				addkSanVectorsD (&basePart->basePosition[partArrIndex], &prevDelPosArray[partArrIndex] , &basePart->basePosition[partArrIndex] );

				//basePart->basePosition[partArrIndex].a += prevDelPosArray[partArrIndex].a;
				//basePart->basePosition[partArrIndex].b += prevDelPosArray[partArrIndex].b;
				//basePart->basePosition[partArrIndex].c += prevDelPosArray[partArrIndex].c;
	 		}
		    verlet->dirtyNextPositions = true;
		    verlet->dirtyPrevPositions = false;
		    verlet->dirtyPrevForces = false;

		
			simReleaseAtomTypes(sim);
			simReleaseParts (sim);
			VSListRelease(&verlet->prevDelPositions );
 			VSListRelease(&verlet->prevForces );
		}
	}
}
short   VerletMethodDestructor (KozoDispatchStack *ds, destructorMethodArgs* dma)
{
#pragma unused(dma)
	VerletMethod *verlet;
  	
	verlet  = (VerletMethod *) ds->thisPD;
 	VSListDispose (&verlet->nextDelPositions);
	VSListDispose (&verlet->prevDelPositions);
	VSListDispose (&verlet->prevForces);
	KDSReturnContinue(ds);
}
short  VerletMethodDoMessage (KozoDispatchStack *ds, doMessageMethodArgs *dmma)
{
	short err = noErr;
	VerletMethod *verlet;
  	
	verlet  = (VerletMethod *) ds->thisPD;
	switch (dmma->message)
	{
		case kSetNewContainerMessage:
			if (dmma->data)
			{
			      verlet->sim = nilPointer;
			      kozoObjectGetObjectInHierarchy((kozoObject *)dmma->data, ClassSimulation, nilPointer , & verlet->sim);  
			}
			goto Default;
		case kThermalizeMessage:
		case kSetToZeroMessage:
			err = dispatchLoop(ds, dmma);
		     verlet->dirtyNextPositions = true;
		     verlet->dirtyPrevForces = true;
		     verlet->dirtyPrevPositions = true;
			break;
		case kSanChangedInfoMessage :
		     verlet->dirtyNextPositions = true;
		     verlet->dirtyPrevForces = true;
		     verlet->dirtyPrevPositions = true;
			break;
		Default :
		default :
			KDSReturnContinue(ds);
			break;
	}
	KDSDoneDispatch(ds);
	KDSReturnError(ds, err);
}
 short VerletIteration ( kSanMethodPlugIn *mpi,  kSanSimulation *sim) // long	doVerletMDTimeStep (void)
{
	short err = noErr;
	char changedDT;
	char unused;
#pragma unused(unused)
  	long numGroups;
	particle *basePart;
	kSanGroup **baseGroup;
	VerletMethod *verlet ;
 	kozoObject *verletObj = mpi->mpiObj;
 	verlet = mpi->methodPlugPrivateData;
 	
 	verlet->verletItersDone = 0;
 	verlet->verletDeltaT = sim->deltaT /  ((float) verlet->verletItersPerSimIter);
 	
 	if (verlet->sim == sim)
 	{
	 	numGroups = simGetNumGroups(sim);
		baseGroup = simLockGroups(sim);
	 	basePart = simLockParts(sim);
	 	
	 	while (verlet->verletItersDone < verlet->verletItersPerSimIter)
	 	{
		 	if (verlet->dirtyPrevPositions) initPrevPositions(verletObj, verlet);
			
			if ((verlet->verletDeltaT != verlet->prevTimestep) || (verlet->dirtyNextPositions))
		 	{
		   		 changedDT = verletSafeCalcNextPositions (verletObj, verlet,   basePart, baseGroup, numGroups,  (verlet->verletDeltaT != verlet->prevTimestep));
		 	}
		 	
			// install the values previously calculated
			// and move the current positions to the previous positions
			err = verletChangePositions(verletObj, verlet,   basePart ); 
		  	
			// having changed the positions, we need to calculate the force
		  	err = calculateForce(sim);
		  	
			// go one step ahead and calculate the positions at the t+1 timestep
		   	changedDT = verletSafeCalcNextPositions (verletObj, verlet,  basePart, baseGroup, numGroups, false);
		  	
			// now we have positions at t-1 and t+1, so we use central difference to get the velocities
			// note vels  are really not part of the verlet algorithm, but we need to do this to satisfy kSan.
			//  we could also optimize this out in the future.
			
			err = verletCalcVelocities(verletObj, verlet,   basePart ); 
			verlet->verletItersDone += 1;
		} // end verletIters
		
		// we have an automatic sensor for whether the timestep can be increased
		if ((verlet->numSequentialNoTimestepChange > 50)  && (verlet->verletItersPerSimIter > 1 ))
		{
			verlet->verletItersPerSimIter  = verlet->verletItersPerSimIter  - 1;
 		} 

		simIncrementStateCount(sim);
		simReleaseGroups(sim);
		simReleaseParts(sim);
	}
 	return (err);
}
short VerletCooling (  kSanMethodPlugIn *mpi,  kSanSimulation *sim, double *netTime, double *totKinEnergy)
{
	kSanGroup *gp;
	kSanGroup **baseGroup;
	long numParts;
	long i;
	long *list;
	short numGroups;
	short groupI;
	VerletMethod *verlet ;
 	kozoObject *verletObj = mpi->mpiObj;
 	particle *basePart;
	kSanAtomtype **types;
	kSanVector  *nextDelPosArray;
	kSanVector  *prevDelPosArray;
 	float largerOfTemps;
	float myCoolFac;
	float nthCoolFac;
	float frac;
	float eqTempToUse;
	float devTimeToUse;
    float thirdKT;
	
 	verlet = mpi->methodPlugPrivateData;
 	if (verlet->sim == sim)
 	{
	 	if (verlet->dirtyPrevPositions) initPrevPositions(verletObj, verlet);

		
	 	
	    types = simLockAtomTypes(sim);
	 	numGroups = simGetNumGroups(sim);
		baseGroup = simLockGroups(sim);
		basePart = simLockParts(sim);
	 	prevDelPosArray =  (kSanVector  *) VSListLock (&verlet->prevDelPositions);
	 	nextDelPosArray =  (kSanVector  *) VSListLock (&verlet->nextDelPositions);
		
	 	
	 	for (groupI = 0; groupI<numGroups; ++groupI)
		{
			gp = baseGroup[groupI];
			if (gp->doCooling)
			{
				numParts = OHListCount(&gp->particles);
				if (isnan(gp->actualTemp)) gp->actualTemp = 0;
				if (isnan(gp->eqTemp)) gp->eqTemp = 0;

				gp->recentTempDeviationCollTime += (*netTime);
				if ((4.0 * gp->coolingTimeConstant) > gp->recentTempDeviationCollTime)
				{
					 devTimeToUse = gp->recentTempDeviationCollTime;
				}
				 else
				 {
					 devTimeToUse = 4.0 * gp->coolingTimeConstant;
				 }
				 
				frac = ((*netTime) /   devTimeToUse );
				
				if (frac > 1.0) frac = 1.0;
				else if (frac < 0) frac = 0;
				
				if (gp->actualTemp < (0.1 * gp->eqTemp))
				{
				     gp->recentTempDeviation = 0;
				}
				
 			    gp->recentTempDeviation +=  (gp->actualTemp - gp->eqTemp) *  frac ;
  			    eqTempToUse = gp->eqTemp - gp->recentTempDeviation;

				list = lockList(&gp->particles);
				if (eqTempToUse > gp->actualTemp)  largerOfTemps = eqTempToUse;
				else   largerOfTemps = gp->actualTemp;
				if (largerOfTemps > 0.1)
				{
					float aveKE;
					aveKE = 
					myCoolFac = 1.0 +  (((eqTempToUse - gp->actualTemp ) * (*netTime) )/
						 (gp->coolingTimeConstant * largerOfTemps) ) ;

			          if (myCoolFac > 1.1) myCoolFac = 1.1;
			          else if (!(myCoolFac > 0.0)) myCoolFac = 0.0001;

					if (myCoolFac > 1.0)
					{
						for (i = 0; i < numParts; ++i)
						{
							long partArrIndex = list[i];
							multkSanVectorD (&prevDelPosArray[partArrIndex], myCoolFac, &prevDelPosArray[partArrIndex]);
 						    //prevDelPosArray[partArrIndex].a *=  myCoolFac;
						    //prevDelPosArray[partArrIndex].b *=   myCoolFac;
						    //prevDelPosArray[partArrIndex].c *=   myCoolFac;

							multkSanVectorD (&nextDelPosArray[partArrIndex], myCoolFac, &nextDelPosArray[partArrIndex]);
						   // nextDelPosArray[partArrIndex].a *=   myCoolFac;
						   // nextDelPosArray[partArrIndex].b *=   myCoolFac;
						   // nextDelPosArray[partArrIndex].c *=   myCoolFac;
						   
							multkSanVectorD (&basePart->baseVelocity[partArrIndex], myCoolFac, &basePart->baseVelocity[partArrIndex]);
			 				//basePart->baseVelocity[partArrIndex].a *=  myCoolFac;
			 				//basePart->baseVelocity[partArrIndex].b *=  myCoolFac;
			 				//basePart->baseVelocity[partArrIndex].c *=  myCoolFac;
			 			}
					}
					else
					{
			//  sim->simTemperature = kinenergy * kTwoOverThreeK / (float)simGetNumParts(sim);
            //  #define kTwoOverThreeK 7729.4686  // units are per eV
						 #define  eVUnitsOneHalfKT  	5797.10145 // units are   per eV
						 #define  olofUnitsOneHalfKT  	0.601558759132 // units are  <null>   per  (g/mol) A / ps ps )
						 #define  olofUnitsOneThirdKT  	0.401039172755 // units are  <null>   per  (g/mol) A / ps ps )
                        // want to multiply this by (eV per (g/mol) A A / ps ps)   
                         // i.e. multiply by ( 1 / 9636.8 )
                         // oneHalf kT is 0.601558759132 in units of    <null>   per  (g/mol) A / ps ps )

  
					    myCoolFac = myCoolFac * myCoolFac; // apply to energies now!
					    thirdKT = olofUnitsOneThirdKT;
						for (i = 0; i < numParts; ++i)
						{
							long partArrIndex = list[i];
							double nthKE;
							float nthAboveThirdKT;
							float newKE;
							
							nthKE = dotkSanVectorsD (&basePart->baseVelocity[partArrIndex], &basePart->baseVelocity[partArrIndex] );
							//nthKE = (basePart->baseVelocity[partArrIndex].a * basePart->baseVelocity[partArrIndex].a);
							//nthKE += (basePart->baseVelocity[partArrIndex].b * basePart->baseVelocity[partArrIndex].b);
							//nthKE += (basePart->baseVelocity[partArrIndex].c * basePart->baseVelocity[partArrIndex].c);
							nthKE  *= ( types[basePart->basePi[partArrIndex].type]->specs.mass );	
							
							// nthKE in units of (g/mol) A A / ps ps
							if (nthKE > thirdKT)
							{
							    nthAboveThirdKT = nthKE - thirdKT;
 							    newKE = thirdKT + nthAboveThirdKT * myCoolFac;
							    nthCoolFac = pow ((newKE / nthKE), 0.5);
							    
							    multkSanVectorD ( & prevDelPosArray[partArrIndex], nthCoolFac, & prevDelPosArray[partArrIndex] );
							   // prevDelPosArray[partArrIndex].a *=   nthCoolFac;
							   // prevDelPosArray[partArrIndex].b *=   nthCoolFac;
							   // prevDelPosArray[partArrIndex].c *=   nthCoolFac;

							    multkSanVectorD ( & nextDelPosArray[partArrIndex], nthCoolFac, & nextDelPosArray[partArrIndex] );
							   //nextDelPosArray[partArrIndex].a *=   nthCoolFac;
							   // nextDelPosArray[partArrIndex].b *=   nthCoolFac;
							   // nextDelPosArray[partArrIndex].c *=   nthCoolFac;
							   
							    multkSanVectorD ( &basePart->baseVelocity[partArrIndex], nthCoolFac, &basePart->baseVelocity[partArrIndex] );
				 				//basePart->baseVelocity[partArrIndex].a *=  nthCoolFac;
				 				//basePart->baseVelocity[partArrIndex].b *=  nthCoolFac;
				 				//basePart->baseVelocity[partArrIndex].c *=  nthCoolFac;
							}
			 			}
		 			}
				}
				releaseList (&gp->particles);
			}
		}
		*totKinEnergy = -1;


	 	VSListRelease (&verlet->prevDelPositions);
		VSListRelease (&verlet->nextDelPositions);
		simReleaseGroups(sim);
		simReleaseParts(sim);
	    simReleaseAtomTypes(sim);
	}
 	return (noErr);
}



 short verletChangePositions(kozoObject *verletObj, VerletMethod *verlet,   particle *basePart )
{
#pragma unused(verletObj)
 	short err = noErr;
	long partArrIndex;
	long numParts = simGetNumParts(verlet->sim);
	kSanVector  *nextDelPosArray;
	kSanVector  *prevDelPosArray;
 	prevDelPosArray =  (kSanVector  *) VSListLock (&verlet->prevDelPositions);
 	nextDelPosArray =  (kSanVector  *) VSListLock (&verlet->nextDelPositions);
  	for(partArrIndex=0;partArrIndex<numParts;++partArrIndex)
	{
	    addkSanVectorsD (&basePart->basePosition[partArrIndex], &nextDelPosArray[partArrIndex], &basePart->basePosition[partArrIndex] );
		// basePart->basePosition[partArrIndex].a +=  nextDelPosArray[partArrIndex].a ;
		// basePart->basePosition[partArrIndex].b +=  nextDelPosArray[partArrIndex].b ;
		// basePart->basePosition[partArrIndex].c +=  nextDelPosArray[partArrIndex].c ;
		 prevDelPosArray[partArrIndex] = nextDelPosArray[partArrIndex]  ;
	}	
	verlet->prevTimestep = verlet->nextTimestep;
	VSListRelease (&verlet->prevDelPositions);
	VSListRelease (&verlet->nextDelPositions);
 	return (err);
}
 short verletCalcVelocities(kozoObject *verletObj, VerletMethod *verlet,   particle *basePart )
{
#pragma unused(verletObj)
 	short err = noErr;
	long partArrIndex;
	long numParts = simGetNumParts(verlet->sim);
	kSanVector  *nextDelPosArray;
	kSanVector  *prevDelPosArray;
	
	
 	prevDelPosArray =  (kSanVector  *) VSListLock (&verlet->prevDelPositions);
 	nextDelPosArray =  (kSanVector  *) VSListLock (&verlet->nextDelPositions);
 	if (verlet->verletDeltaT == verlet->prevTimestep)
 	{
		double oneOver2DT;
		oneOver2DT = 0.5 / verlet->verletDeltaT;
	  	for(partArrIndex=0;partArrIndex<numParts;++partArrIndex)
		{
			addkSanVectorsD (&nextDelPosArray[partArrIndex], &prevDelPosArray[partArrIndex], & basePart->baseVelocity[partArrIndex] );
			multkSanVectorD (&basePart->baseVelocity[partArrIndex], oneOver2DT, &basePart->baseVelocity[partArrIndex] );
			// basePart->baseVelocity[partArrIndex].a = oneOver2DT * (nextDelPosArray[partArrIndex].a + prevDelPosArray[partArrIndex].a) ;
			// basePart->baseVelocity[partArrIndex].b = oneOver2DT * (nextDelPosArray[partArrIndex].b + prevDelPosArray[partArrIndex].b) ;
			// basePart->baseVelocity[partArrIndex].c = oneOver2DT * (nextDelPosArray[partArrIndex].c + prevDelPosArray[partArrIndex].c) ;
		}	
	}
	else
	{
		double prevWeight;
		double nextWeight;
		double weightedOneOverPrevDT;
		double weightedOneOverNextDT;
		prevWeight = verlet->verletDeltaT /  (verlet->verletDeltaT + verlet->prevTimestep );
		nextWeight = 1.0 - prevWeight;
		weightedOneOverNextDT = nextWeight / verlet->verletDeltaT;
		weightedOneOverPrevDT = prevWeight / verlet->prevTimestep;
	  	for(partArrIndex=0;partArrIndex<numParts;++partArrIndex)
		{
			 basePart->baseVelocity[partArrIndex].a = weightedOneOverPrevDT * prevDelPosArray[partArrIndex].a   + weightedOneOverNextDT * nextDelPosArray[partArrIndex].a  ;
			 basePart->baseVelocity[partArrIndex].b = weightedOneOverPrevDT * prevDelPosArray[partArrIndex].b   + weightedOneOverNextDT * nextDelPosArray[partArrIndex].b  ;
#ifndef __2DVectors__
			 basePart->baseVelocity[partArrIndex].c = weightedOneOverPrevDT * prevDelPosArray[partArrIndex].c   + weightedOneOverNextDT * nextDelPosArray[partArrIndex].c  ;
#endif
 		}	
	}
	VSListRelease (&verlet->prevDelPositions);
	VSListRelease (&verlet->nextDelPositions);
 	return (err);
}
 

void checkVectorNAN(kSanVector * vec);
void checkVectorNAN(kSanVector * vec)
{
		if (vectorIsNan(vec))
		{
			OHSystemBeep();
			fixNANVector(vec);
		}
 }

 


