/*
 * Driver.c
 */

#ifdef STAND_ALONE
	#include <A4Stuff.h>
#else
	// all data must be far!
	#define EnterCodeResource()
	#define ExitCodeResource()
	#define SetCurrentA4() 	0L
	#define SetA4(x)		0L
	#define GetCurrentA4() 	0L
	#define UnloadA4Seg(x) 	0
#endif
#include "sysequ.h"
#include "PascalLib.h"
#include "Globals.h"
#include "Driver.h"

extern pascal UInt8 LMGetFSBusy (void) TWOWORDINLINE (0x1EB8, 0x0360);

static DrvQElPtr FindDrvQ (short drvNum)
{
	DrvQElPtr qep = (DrvQElPtr)(GetDrvQHdr()->qHead);
	while (qep) {
		if (qep->dQDrive == drvNum) return qep;
		qep = (DrvQElPtr)qep->qLink;
	}
	return nil;
}

static asm void blockClear (Ptr p, long n)
{
	fralloc
	move.l	p,a0
	move.l	n,d0
	lsr.l	#3,d0
	moveq	#0,d1
	bra		ss
ll:	move.l	d1,(a0)+
	move.l	d1,(a0)+
ss:	dbra	d0,ll
	frfree
	rts
}

QHdr	GlobeToDisposeOfQueue = {0, nil, nil};

void DisposeGlobals (DriveGlobalsPtr globe)
{
	if (globe->fRefNum) {
		if (LMGetFSBusy()) {
			Enqueue ((QElemPtr)&globe->dQAll.dQEl, &GlobeToDisposeOfQueue);
			return;
		}
		FSClose (globe->fRefNum);
	}
	DisposePtr ((Ptr)globe);
}

static void disposeDrive (DrvQEl *qep, QHdrPtr qhp)
{
	Dequeue ((QElemPtr)qep, qhp);
	DisposeGlobals (GetDrvGlobePtr (qep));
}

DriveGlobalsPtr GetDrvGlobePtr (DrvQElPtr qep)
{
	return (DriveGlobalsPtr) ((Ptr)qep - ((Ptr)&((DriveGlobalsPtr)0)->dQAll.dQEl - (Ptr)0));
}

static OSErr dOpen (IOParamPtr pb, DCtlPtr dce)
{
	EnterCodeResource ();
	dce->dCtlFlags |= dNeedLockMask | dStatEnableMask | dCtlEnableMask | dWritEnableMask | dReadEnableMask;
	dce->dCtlStorage = nil;
	ExitCodeResource ();
	return noErr;
}

static OSErr dClose (IOParamPtr pb, DCtlPtr dce)
{
	EnterCodeResource ();
	OSErr err = UnmountAndEjectMyVols ();
	UnlinkMyDrives ();
	ExitCodeResource ();
	return err;
}

static DCtlPtr currentDCE;
static IOParam *currentPB;

static OSErr dPrime (IOParamPtr pb, DCtlPtr dce)
{
	//DebugStr("\pdPrime ...");
	EnterCodeResource ();
	ulong numBytes, offset0, diskSize;
	OSErr err;
	short async = false;
	IOParam pb2;
	DrvQElPtr qep;

	qep = FindDrvQ (pb->ioVRefNum);
	if (!qep) {
		err = nsDrvErr;	// can't find my drive
		goto errExit;
	}
	
	numBytes = pb->ioReqCount;
	offset0 = dce->dCtlPosition;
	
	switch (pb->ioTrap & 0x00FF) {
		case aWrCmd:
			err = writErr;
			numBytes = 0;
		case aRdCmd:
			blockClear (pb->ioBuffer, numBytes);
			numBytes = 0;
			err = noErr;
			break;
		default:
			err = -1;
			numBytes = 0;
	}

	if (async && (err == noErr)) {
		err = 1;
		goto errExit;
	} else {
		dce->dCtlPosition += numBytes;
		pb->ioActCount = numBytes;
		pb->ioResult = err;
		goto errExit;
	}
errExit:
	//DebugStr("\pdPrime done");
	ExitCodeResource ();
	return err;
}

static short dControl (CntrlParamPtr pb, DCtlPtr dce)
{
	EnterCodeResource ();
	OSErr res;
	DriveGlobalsPtr globe;
	short csCode = pb->csCode;
	DrvQElPtr qep = FindDrvQ (pb->ioVRefNum);
	if (!qep) {
		if (csCode == accRun) {
			QHdrPtr	qhp = &GlobeToDisposeOfQueue;
			do {
				DrvQEl	*qep = (DrvQEl*)qhp->qHead;
				if (qep == nil) break;
				Dequeue ((QElemPtr)qep, qhp);
				DisposeGlobals (GetDrvGlobePtr (qep));
			} while (true);
			res = noErr;
		} else {
			res = nsDrvErr;
		}
	} else {
		//DebugStr("\pdControl ...");
		globe = GetDrvGlobePtr (qep);
		switch (csCode) {
			// codes & errors see DV 17
			case killCode:
				res = noErr;
				break;
			case 5:	// verify
				res = noErr;
				break;
			case 6:	// format
				res = noErr;
				break;
			case 7:	// eject
				disposeDrive (qep, GetDrvQHdr());
				res = noErr;
				break;
			case 23:	// Return Drive Info
				*(long*)&pb->csParam[0] = 0x00000410; // 0x10 = 'ram disk'
				res = noErr;
				break;
			case 21: // Return Physical Drive Icon
				*(Ptr*)&pb->csParam[0] = (Ptr)&globe->physIcon.data;
				res = noErr;
				break;
			case 22: // Return Media Icon
				*(Ptr*)&pb->csParam[0] = (Ptr)&globe->mediaIcon;
				res = noErr;
				break;
			case 24:
				*(unsigned long *)pb->csParam = 1;
				res = noErr;
				break;
			default:
				res = controlErr;
		}
		//DebugStr("\pdControl done");
	}
	ExitCodeResource ();
	return res;
}

static short dStatus (CntrlParamPtr pb, DCtlPtr dce)
{
	//DebugStr("\pdStatus ...");
	EnterCodeResource ();
	OSErr res;
	short csCode = pb->csCode;
	Ptr myDrvQ = (Ptr)FindDrvQ (pb->ioVRefNum);
	if (!myDrvQ) {
		res = nsDrvErr;	//can't find my drive
	} else switch (csCode) {
		// codes & errors s. DV 17
		case 6: // Return Format List
			res = statusErr;
			break;
		case 8: // Drive Status
			{
				BlockMoveData (myDrvQ, &pb->csParam[3], 12);
				pb->csParam[0] = 0x0000;	// current track
				pb->csParam[1] = 0x0008;
				pb->csParam[2] = 0x0100;
				pb->csParam[9] = 0x0000;
				pb->csParam[10] = 0;
				res = noErr;
				break;
			}
		default:
			res = statusErr;
	}
	ExitCodeResource ();
	//DebugStr("\pdStatus done");
	return res;
}

static asm void ioReturn () {
		move.w	struct(CntrlParam.ioTrap)(a0),d1
		btst	#noQueueBit,d1
		beq		queued
		
		tst.w	d0
		ble		immRtn
		clr.w	d0
immRtn:	move.w	d0,struct(CntrlParam.ioResult)(a0)
		rts

queued:	tst.w	d0
		ble		iodone
		clr.w	d0
		rts

iodone:	move.l	JIODone,-(a7)
		rts
}

static asm void myDriver ()
{
	dc.w	MyDriverMask	// flags
	dc.w	MyDriverDelay	// delay
	dc.w	0,0				// emask, emenu
	dc.w	-1				// openDriver
	dc.w	-1				// prime
	dc.w	-1				// control
	dc.w	-1				// status
	dc.w	-1				// close
	dc.b	MYDRIVERNAME,0
}

static asm void asmOpen () {
		movem.l	a0/a1,-(a7)
		jsr		dOpen
		movem.l	(a7)+,a0/a1
		rts
}

static asm void asmPrime () {
		movem.l	a0/a1,-(a7)
		jsr		dPrime
		movem.l	(a7)+,a0/a1
		jmp		ioReturn
}

static asm void asmControl () {
		movem.l	a0/a1,-(a7)
		jsr		dControl
		movem.l	(a7)+,a0/a1
		cmpi.w	#killCode,struct(CntrlParam.csCode)(a0)
		bne		ioRtn
		rts
ioRtn:	jmp		ioReturn
}

static asm void asmStatus () {
		movem.l	a0/a1,-(a7)
		jsr		dStatus
		movem.l	(a7)+,a0/a1
		jmp		ioReturn
}

static asm void asmClose () {
		movem.l	a0/a1,-(a7)
		jsr		dClose
		movem.l	(a7)+,a0/a1
		rts
}

static void setDriverProc (short offset, void(*proc)())
{
	Ptr	start;
	long diff;
	start = (Ptr)myDriver;
	diff = (Ptr)proc - start;
	*(short*)(&start[offset]) = (short)diff;
}

Ptr SetupDriverTable ()
{
	setDriverProc ( 8, (void(*)())asmOpen);
	setDriverProc (10, asmPrime);
	setDriverProc (12, asmControl);
	setDriverProc (14, asmStatus);
	setDriverProc (16, asmClose);

	return (Ptr)myDriver;
}

OSErr UnmountAndEjectMyVols ()
{
	//DebugStr("\pUnmountAndEjectMyVols ...");
	if (MyDriverRefNum) {
		short err, idx;
		HVolumeParam pb;
		Str255 name;
		DrvQElPtr dp;

		idx = 1;
		do {
			pb.ioVolIndex = idx;
			pb.ioCompletion = nil;
			pb.ioNamePtr = name;
			err = PBHGetVInfo ((HParamBlockRec*)&pb, false);
			if (!err) {
				dp = FindDrvQ (pb.ioVDrvInfo);
				if (dp->dQRefNum == MyDriverRefNum) {
					err = UnmountVol (nil, pb.ioVRefNum);
					if (err) {
						return err;
					}
					pb.ioVRefNum = pb.ioVDrvInfo;
					PBEject ((ParmBlkPtr)&pb);
					idx = 0;
				}
			}
			++idx;
		} while (!err);
	}
	return 0;
}
		
OSErr CloseMyDriver (Boolean deleteFiles)
{
	//DebugStr("\pCloseMyDriver ...");
	OSErr err = 0;
	if (MyDriverRefNum) {
		DCtlHandle	dCtl;
		dCtl = GetDCtlEntry (MyDriverRefNum);
		if ((**dCtl).dCtlFlags & (1<<5)) {
			err = UnmountAndEjectMyVols ();
			if (err) return err;

			UnlinkMyDrives ();
			err = CloseDriver (MyDriverRefNum);
		}
	}
	return err;
}


void UnlinkMyDrive (short drvNum)
{
	//DebugStr("\pUnlinkMyDrive ...");
	if (MyDriverRefNum) {
		QHdrPtr	qhp;
		DrvQEl	*qep;

		qhp = GetDrvQHdr();
		qep = (DrvQEl*)qhp->qHead;
		do {
			if (qep->dQRefNum == MyDriverRefNum && qep->dQDrive == drvNum) {
				disposeDrive (qep, qhp);
				return;
			}
			qep = (DrvQEl*)qep->qLink;
		} while (qep != nil);
	}
}

void UnlinkMyDrives ()
{
	//DebugStr("\pUnlinkMyDrives ...");
	if (MyDriverRefNum) {
		QHdrPtr	qhp;
		DrvQEl	*qep;

		qhp = GetDrvQHdr();
		qep = (DrvQEl*)qhp->qHead;
		do {
			if (qep->dQRefNum == MyDriverRefNum) {
				disposeDrive (qep, qhp);
				qep = (DrvQEl*)qhp->qHead;
				continue;
			}
			qep = (DrvQEl*)qep->qLink;
		} while (qep != nil);
	}
}

// EOF
