/*
 * DriverInstall.c
 */

#include <A4Stuff.h>
#include "sysequ.h"
#include "Globals.h"
#include "Driver.h"
#include "PascalLib.h"
#include "DeviceSupport.h"
#include "DriverInstall.h"

static DrvQElPtr FindDrvQ (short drvNum)
{
	DrvQElPtr qep = (DrvQElPtr)(GetDrvQHdr()->qHead);
	while (qep) {
		if (qep->dQDrive == drvNum) return qep;
		qep = (DrvQElPtr)qep->qLink;
	}
	return nil;
}

static short freeDriver ()
{
	DCtlHandle	dCtl;
	short		i;
	
	i = -10;
	do {
		dCtl = GetDCtlEntry (i);
	} while (dCtl && (~(--i) < *(short*)UnitNtryCnt));
	if (!dCtl) {
		return i;
	}
	return 0;
}

static short freeDrive ()
{
	QHdrPtr	qhp;
	DrvQEl	*qep;
	short	drvNum = 64;

	qhp = GetDrvQHdr();
	do {
		qep = (DrvQEl*)qhp->qHead;
		do {
			if (qep->dQDrive == drvNum) {
				++drvNum;
				break;
			}
			qep = (DrvQEl*)qep->qLink;
		} while (qep != nil);
	} while  (qep != nil);
	return drvNum;
}

static OSErr MyDriverInstall (Ptr drvrPtr, short drvrRefNum)
{
	OSErr		result = noErr;
	short		unitNum;
	DCtlHandle	*unitTable;
	DCtlHandle	dceHandle;
	
	unitNum = -drvrRefNum-1;
	unitTable = (DCtlHandle *)LMGetUTableBase ();

	ReserveMemSys (sizeof (DCtlEntry));
	
	dceHandle = (DCtlHandle)NewHandleSysClear (sizeof (DCtlEntry));
	if (dceHandle != nil) {
		(**dceHandle).dCtlDriver = drvrPtr;
		(**dceHandle).dCtlFlags = /*dRAMBasedMask |*/ MyDriverMask;
		(**dceHandle).dCtlRefNum = drvrRefNum;
		(**dceHandle).dCtlDelay = MyDriverDelay;
		unitTable[unitNum] = dceHandle;		
	} else {
		result = memFullErr;
	}
	
	return result;
}

OSErr InstallDriver ()
{
	short err;
	short refNum;

	if (OpenDriver (MYDRIVERNAME, &MyDriverRefNum) == noErr) {
		// already installed
		return 1;
	}

	Ptr p = SetupDriverTable ();
	MyDriverRefNum = freeDriver ();
	if (MyDriverRefNum == 0) {
		err = unitTblFullErr;
	} else {
		err = MyDriverInstall (p, MyDriverRefNum);
		if (!err) {
			if (OpenDriver (MYDRIVERNAME, &refNum) != noErr) DebugStr ("\pOpenDriver error!");
			if (refNum != MyDriverRefNum) DebugStr ("\pDriver refNum doesn't match!");
		}
	}
	return err;
}


OSErr DeinstallDriver (Boolean deleteFiles)
{
	if (MyDriverRefNum) {
		OSErr err = CloseMyDriver (deleteFiles);
		DrvrRemove (MyDriverRefNum);
	}
	MyDriverRefNum = 0;
	return noErr;
}

 
void MountAllDrives (void)
{
	QHdrPtr	qhp;
	DrvQEl	*qep;

	qhp = GetDrvQHdr();
	qep = (DrvQEl*)qhp->qHead;
	do {
		if (qep->dQRefNum == MyDriverRefNum) {
			VolumeParam pb;
			pb.ioVRefNum = qep->dQDrive;
			PBMountVol ((ParmBlkPtr)&pb);
		}
		qep = (DrvQEl*)qep->qLink;
	} while (qep != nil);
}

static OSErr addTheDrive (short &drvNum, DriveGlobalsPtr globe)
{
	DrvQElPtr	drvQElPtr;
	long		blk_size = globe->totalSize;

	drvNum = freeDrive ();
	if (drvNum < 0) {
		return nsDrvErr;
	}
	globe->dQAll.dQFlags = 0x00010000;
	//globe->dQAll.dQFlags = 0x00480000;			// "Eject" in Finder is disabled, but driver wants a call when PBEject is called
	drvQElPtr = (DrvQElPtr)&globe->dQAll.dQEl;
	drvQElPtr->qType = 1;		// qType of 1 means we do use dQDrvSz2
	drvQElPtr->dQFSID = 0;
	drvQElPtr->dQDrvSz  = LoWord (blk_size);
	drvQElPtr->dQDrvSz2 = HiWord (blk_size);
	AddDrive (MyDriverRefNum, drvNum, drvQElPtr);

	globe->drvNum = drvNum;
	globe->drvrRef = MyDriverRefNum;

	// Icon aus Rsrc holen
	Handle hdl = Get1NamedResource ('ICN#', "\pphys");
	if (hdl) {
		BlockMove (*hdl, &globe->physIcon.data, 256);
		BlockMove (*hdl, &globe->mediaIcon, 256);
		ReleaseResource (hdl);
	}
	
	if (globe->fRefNum) {
		FCBPBRec pb;
		Str32 fName;
		Str32 volName;
		pb.ioFCBIndx = 0;
		pb.ioVRefNum = 0;
		pb.ioRefNum = globe->fRefNum;
		pb.ioNamePtr = fName;
		PBGetFCBInfoSync (&pb);
		pstrcpy (globe->physIcon.locationStr, "\pFile  on ");
		pstrins (globe->physIcon.locationStr, volName, 10);
		pstrins (globe->physIcon.locationStr, fName, 6);
	} else {
		pstrcpy (globe->physIcon.locationStr, "\popen space");
	}

	return noErr;
}

OSErr InstallADrive (Boolean createNew, FSSpec *spec, DriveCreationDesc *creationParms, short &drvNum)
{
	DriveGlobalsPtr	globe;
	OSErr			err = 0;
	Boolean			virtualABlks = false;

	drvNum = 0;

	globe = (DriveGlobalsPtr) NewPtrSysClear (sizeof(DriveGlobals));
	if (!globe) {
		return memFullErr;
	}

	if (spec) {
		short refNum;
		if ((err = FSpOpenDF (spec, fsRdPerm, &refNum)) == noErr) {
			GetEOF (refNum, &globe->totalSize);
			FSClose (refNum);
		} else {
			DisposePtr ((Ptr)globe);
			return err;
		}
		if (!err) {
			// open the image file
			err = FSpOpenDF (spec, fsRdWrPerm, &globe->fRefNum);
			if (err) {
				globe->fRefNum = 0;
			} else {
				globe->vRefNum = spec->vRefNum;
			}
		}
	} else {
		globe->totalSize = creationParms->volumeSize;
	}

	if (!err) {
		THz saveZone = GetZone();
		SetZone (SystemZone());
		err = addTheDrive (drvNum, globe);
		SetZone (saveZone);
	}

	if (err) {
		if (globe->fRefNum) {
			FSClose (globe->fRefNum);
			globe->fRefNum = 0;
		}
		if (drvNum) {
			UnlinkMyDrive (drvNum);
		} else {
			DisposeGlobals (globe);
		}
		drvNum = 0;
	}
	return err;
}

void RemoveADrive (short &drvNum)
{
	if (drvNum) {
		UnlinkMyDrive (drvNum);
		drvNum = 0;
	}
}

DriveGlobalsPtr GetDriveGlobalsPtr (short drvNum)
{
	return GetDrvGlobePtr (FindDrvQ (drvNum));
}

#ifdef STAND_ALONE
long main (short funcNo, void *par)
{
	long res;
	EnterCodeResource ();
	switch (funcNo) {
	case 0:
		res = InstallDriver ();
		break;
	case 1:
		res = DeinstallDriver (*(Boolean*)par);
		break;
	case 2:
		InstallADrivePar *p = (InstallADrivePar *)par;
		res = InstallADrive (p-> createNew, p->spec, p->creationParms, p->drvNum);
		break;
	case 3:
		MountAllDrives ();
		res = 0;
		break;
	case 4:
		RemoveADrive (*(short*)par);
		res = 0;
		break;
	case 5:
		res = (long)GetDriveGlobalsPtr (*(short*)par);
		break;
	default:
		res = -1;
	}
	ExitCodeResource ();
	return res;
}
#endif

// EOF
