/*
 * ISO Dir Locator.h
 *
 * Directory Sector Locator Tables
 */

#pragma once

#include "ISO9660tt.h"
#include "ISO Reader.h"

#pragma push
#pragma cplusplus on


extern	Boolean LocBufOverflow;


typedef long BlkNum;

void InitLocBuffer (Ptr start, long size);

void ClearLocBuffer (short volID);
/*
 * Mu beim Start sowie beim jedem Unmount eines Volumes aufgerufen
 * werden, um den DirLoc-Cache zu lschen.
 * "volID=0" lscht ganzen Puffer.
 */


typedef	Byte	LocName[14];	// a Pascal string

typedef struct {				// call InitDirLocRec() to init the values
	BlkNum	currBlkNum;
	BlkNum	dirStartBlk;
	long	dirTotalBlks;
	long	currStartIdx;
	long	helper;				// helps to find the dirLoc in UpdateDirLocationBuffer
	BlkNum	prevBlk;
	LocName	lastNameInPrevBlk;	// needed in UpdateDirLocationBuffer to check the name ordering, don't touch from outside
} DirLocRec;

void InitDirLocRec (DirLocRec &loc);

Boolean LocateBlkNumByIndex (short volID, long dirID, long index, DirLocRec &loc);
Boolean LocateBlkNumByName (short volID, long dirID, StringPtr name, DirLocRec &loc);
/*
 * inputs:
 *   ByIndex: wenn index < 0, dann werden Werte des letzten bekannten Blocks geliefert.
 *   ByName: 'name' mu bereits mit FilterISOName vorbearbeitet worden sein.
 *
 * returns:
 *   false: dirID nicht nicht bekannt -> Aufrufer mu Block anders lokalisieren
 *   true:  dirID ist bekannt. Weitere returns:
 *     currBlkNum, currStartIdx: Zeigen auf Block und Index-Nr (0-based) des 1. Eintrags
 *       im Block (dieser Block und ggf. Folgeblcke mssen gelesen werden, bis der
 *       Eintrag gefunden ist).
 */


Boolean DirLocGetNoOfDirRecs (short volID, long dirID, long& count);
/*
 * count: no of entries excluding associated files, however, including self and parent entries.
 */

void UpdateDirLocationBuffer (short volID, long appleDirID, Byte *dirSector, long entriesInBlk, DirLocRec& loc);
/*
 * Must be called for each known dir sector in order to update the DirLoc buffer
 */

#pragma pop

// EOF
