/*
 * main appl of ISO 9660 / Joliet File System
 */

#include <string.h>
#include <stddef.h>

#include <FSM.h>
#include "PascalLib.h"
#include "FSM Client Msgs.h"
#include "FSM Main.h"

#include <SIOUX.h>
#include <stdio.h>


static Boolean AnyVolsMountedByFS (short fsID)
// checks if any volumes are mounted by the given File System
{
	HVolumeParam pb;
	short err, idx = 1;
	Str63 name;
	do {
		pb.ioVolIndex = idx++;
		pb.ioNamePtr = name;
		err = PBHGetVInfoSync ((HParamBlockRec*)&pb);
		if (err) break;
		if (pb.ioVFSID == fsID) {
			return true;
		}
	} while (true);
	return false;
}

static Boolean UnmountMyVols ()
// returns false when it's not safe to de-install the File System
// because there are still volumes mounted by it.
{
	short fsID = FSMClientGetFSID ();
	
	HVolumeParam pb;
	short err, idx;
	Str255 name;
	Boolean ok = true;

	idx = 1;
	do {
		pb.ioVolIndex = idx;
		pb.ioNamePtr = name;
		err = PBHGetVInfoSync ((HParamBlockRec*)&pb);
		if (err) break;
		if (pb.ioVFSID == fsID) {
			err = UnmountVol (nil, pb.ioVRefNum);
			if (!err) {
				--idx;
				Eject (nil, pb.ioVDrvInfo);
			} else {
				// unmount failed - probably some files are still open on the volume
				ok = false;
			}
		}
		++idx;
	} while (true);
	return ok;
}

static Boolean IsFSInstalled (UInt16 fsID)
{
	FSDRec	fsdRec;
	short	recSize = sizeof (FSDRec);
	OSErr err = GetFSInfo (fsmGetFSInfoByFSID, fsID, &recSize, &fsdRec);
	return (err == noErr);
}

static OSErr FSUninstall ()
{
	tSampleFSMessage msg;
	msg.msgSize = 16;
	msg.version = 1;
	msg.senderID = FSMClientGetFSID();
	msg.senderVer = 1;
	msg.message = kSampleFSMsgUninstall;
	OSErr err = InformFFS (FSMClientGetFSID(), &msg);
	return err;
}

static void error (short errCode, StringPtr msg)
{
	printf ("\nFatal error (%d): %#s", errCode, msg);
}


void main (void)
{
	OSErr	err;
	Str255	msg;

	SIOUXSettings.columns = 46;
	SIOUXSettings.rows = 9;
	SIOUXSettings.autocloseonquit = false;
	SIOUXSettings.asktosaveonclose = false;
	SIOUXSettings.toppixel = 50;
	SIOUXSettings.leftpixel = 30;
	printf ("FSM Client Sample\n");
	printf ("By Thomas Tempelmann, fsm@tempel.org\n\n");

	if (IsFSInstalled (FSMClientGetFSID())) {
		printf ("Removing old FSM Client ... ");
		err = FSUninstall ();
		if (err) {
			printf ("Failed! \n");
			return;
		} else {
			printf ("OK. \n");
		}
	}

	if (!FSMClientInstall (&err, msg, false)) {
		error (err, msg);
		return;
	}
}

// EOF
