/*
 * loader for FSM Client
 */

#include <SIOUX.h>
#include <string.h>
#include <stdio.h>
#include <FSM.h>
#include "PascalLib.h"
#include "FSM Stub.h"
#include "FSM Client Msgs.h"



/*
 * messaging with the FSM Client
 */

const OSType mySig = 'test';	// you may put your signature here (to help the FS to identify you)

static OSErr FSUninstall ()
{
	tSampleFSMessage msg;
	msg.msgSize = 16;
	msg.version = 1;
	msg.senderID = mySig;
	msg.senderVer = 1;
	msg.message = kSampleFSMsgUninstall;
	OSErr err = InformFFS (kSampleFSID, &msg);
	return err;
}

static OSErr FSAnnounceMount (short driveNum)
{
	tSampleFSMessage msg;
	msg.msgSize = 18;
	msg.version = 1;
	msg.senderID = mySig;
	msg.senderVer = 1;
	msg.message = kSampleFSMsgSetDrive;
	msg.driveNum = driveNum;
	OSErr err = InformFFS (kSampleFSID, &msg);
	return err;
}



static Boolean VolMountedByFS (UInt16 fsID)
// checks if any volumes are mounted by the given File System
{
	HVolumeParam pb;
	short err, idx = 1;
	Str63 name;
	do {
		pb.ioVolIndex = idx++;
		pb.ioNamePtr = name;
		err = PBHGetVInfoSync ((HParamBlockRec*)&pb);
		if (err) break;
		if (pb.ioVFSID == fsID) {
			return true;
		}
	} while (true);
	return false;
}

static Boolean IsFSInstalled (UInt16 fsID)
{
	FSDRec	fsdRec;
	short	recSize = sizeof (FSDRec);
	OSErr err = GetFSInfo (fsmGetFSInfoByFSID, fsID, &recSize, &fsdRec);
	return (err == noErr);
}

static void FSMClientInstall (FSMEntryProc doFS, short *errCode, Str255 errMsg)
{
	*errCode = doFS (1, (Ptr)errMsg);
}

static UInt16 FSMClientGetFSID (FSMEntryProc doFS)
{
	UInt16 fsID;
	if (doFS (2, (Ptr)&fsID) == noErr) {
		return fsID;
	}
	return 0;	// = invalid
}


static void error (short errCode, StringPtr msg)
{
	printf (" \nFatal error (%d) when installing FSM Client: %#s", errCode, msg);
}

int main (void)
{
	Str255 msg;
	OSErr err;
	FSMEntryProc codeEntry;

	SIOUXSettings.columns = 46;
	SIOUXSettings.rows = 9;
	SIOUXSettings.autocloseonquit = false;
	SIOUXSettings.asktosaveonclose = false;
	SIOUXSettings.toppixel = 50;
	SIOUXSettings.leftpixel = 30;
	
	printf ("FSM Client Loader.\n\n");
	
	// load the FSM Client code rsrc
	{
		// attn: rsrc must have "system heap" rsrc bit set!
		Handle h = Get1NamedResource ('CODE', "\pFSM Client");
		if (!h) {
			printf ("FSM Client CODE rsrc not found.");
			return -1;
		}
		DetachResource (h);
		HLock (h);
		codeEntry = (FSMEntryProc) *h;
	}

	if (kSampleFSID != FSMClientGetFSID(codeEntry)) {
		printf ("Oops - FSM Client has unexpected FS ID!");
		return -2;
	}
	
	// check if there's already a client installed
	if (IsFSInstalled (kSampleFSID)) {
		printf ("Removing old FSM Client ... ");
		err = FSUninstall ();
		if (err) {
			printf ("Failed! \n");
			return -3;
		} else {
			printf ("OK. \n");
		}
	}
		
	FSMClientInstall (codeEntry, &err, msg);
	if (err || msg[0]) {
		error (err, msg);
	} else {
		printf ("\nFS is ready.");
	}
}

// EOF
