/*
 * FSM Utils.c
 */

#include <string.h>
#include <Gestalt.h>

#include "FSM.h"
#include "HFS CI Procs.h"
#include "FSM Utils.h"


static	IconSet	gMediaIcon;

static void copyIcon (StringPtr rsrcName, OSType type, Handle *dest)
{
	Handle icn = Get1NamedResource (type, rsrcName);
	if (icn) {
		long l = GetHandleSize (icn);
		#if TRACING
			if (((unsigned long)l)>1024) DebugStr("\picon error");
		#endif
		*dest = NewHandleSys (l+4);
		if (*dest) {
			// 1st long in memblock is length of data, data starts at byte offset 4
			long **h = (long**) *dest;
			long *p = *h;
			*p = l;
			BlockMoveData (*icn, p+1, l);
		}
		ReleaseResource (icn);
	} else {
		ResError ();	// clear the error flag
		*dest = nil;
	}
}

static void copyIcons (StringPtr rsrcName, IconSet *i)
{
	copyIcon (rsrcName, 'ICN#', &i->large1BitIcon);
	copyIcon (rsrcName, 'icl4', &i->large4BitIcon);
	copyIcon (rsrcName, 'icl8', &i->large8BitIcon);
	copyIcon (rsrcName, 'ics#', &i->small1BitIcon);
	copyIcon (rsrcName, 'ics4', &i->small4BitIcon);
	copyIcon (rsrcName, 'ics8', &i->small8BitIcon);
}

OSErr LoadIcons ()
{
	copyIcons ("\pFS Icon", &gMediaIcon);
	return noErr;
}

static void disposeHdl (Handle &h)
{
	if (h != nil) {
		DisposeHandle (h);
		h = nil;
	}
}

static void disposeIcons (IconSet *i)
{
	disposeHdl (i->large1BitIcon);
	disposeHdl (i->large4BitIcon);
	disposeHdl (i->large8BitIcon);
	disposeHdl (i->small1BitIcon);
	disposeHdl (i->small4BitIcon);
	disposeHdl (i->small8BitIcon);
}

void UnloadIcons ()
{
	disposeIcons (&gMediaIcon);
}

static Boolean GetIcon (IconSet &i, DrvQElPtr /*dqe*/, short whichIcon, Ptr dest, long reqCount, long *actCount)
{
	{
		// this FS provides generic icons

		Handle h = nil;
		switch (whichIcon) {
			case kLargeIcon:		h = i.large1BitIcon;	break;
			case kLarge4BitIcon:	h = i.large4BitIcon;	break;
			case kLarge8BitIcon:	h = i.large8BitIcon;	break;
			case kSmallIcon:		h = i.small1BitIcon;	break;
			case kSmall4BitIcon:	h = i.small4BitIcon;	break;
			case kSmall8BitIcon:	h = i.small8BitIcon;	break;
		}
		
		long l;
		if (h) {
			// 1st long in memblock is length of data, data starts at byte offset 4
			l = *(long*)*h;
		} else {
			l = 0;
		}
		if (l) {
			*actCount = l;	// 26 Oct 99: moved up so that actCount always returns true Icon size
			if (l > reqCount) {
				Warning ("reqCount < iconSize");
				l = reqCount;
			}
			BlockMoveData (*h+4, dest, l);
		}
		return l > 0;
	}
	return false;
}

Boolean EnterVCB (short vRefNum, VCBPtr *vcbOut, OSErr *err)
{
	VCBPtr vcb;
	if ((*err = UTLocateVCBByRefNum (vRefNum, &vRefNum, &vcb)) != noErr) return false;
	if (vcb->vcbFSID != kFSID) {
		*err = rfNumErr;
		return false;
	}
	Entering (vcb);
	if (vcbOut) *vcbOut = vcb;
	return true;
}

void ExitVCB ()
{
	Exiting ();
}

OSErr DoGetIcon (FSMGetIconRec *icon)
{
	IconSet *theIcon;
	LogFStr (LKUser14, HERE, "Get icon on drv %d", icon->driveQElemPtr->dQDrive);
	theIcon = &gMediaIcon;
	if (GetIcon (*theIcon, icon->driveQElemPtr, icon->iconType, (Ptr)icon->iconBufferPtr, icon->requestSize, &icon->actualSize)) {
		// we provide the icon
		LogFStr (LKUser14, HERE, "Icon returned");
		return noErr;
	}
	// if we do not return an icon, the disk driver's icon will be used
	LogFStr (LKUser14, HERE, "Icon not avail");
	return afpItemNotFound;
}



short HasRequiredFSM (unsigned long *fsmVersion)
{
	long	response;
	short	result;
	
	result = 1;	// -> no FSM avail
	/* Make sure the File System Manager is installed */
	if (Gestalt (gestaltFSAttr, &response) == noErr) {
		if ((response & (1L << gestaltHasFileSystemManager)) != 0) {
			/* check the version of FSM. */
			result = 2;	// -> wrong version of FSM
			if (Gestalt (gestaltFSMVersion, &response) == noErr) {
				/* Make sure we have File System Manager 1.2 or later */
				if ((unsigned long)response >= 0x0120) {
					*fsmVersion = response;
					result = 0;
				}
			}
		}
	}
	return result;
}


OSErr GetCurrentDir (ParmBlkPtr pb, ItemID dirID, ItemID *currentDirectory, short *status)
{
	OSErr result;
	short moreMatches;
	short vRefNum;
	WDCBRecPtr wdCtrlBlockPtr;
	WDPBRec wdpb;
	VCBPtr vcb2;
	
	result = UTDetermineVol (pb, status, &moreMatches, &vRefNum, &vcb2);
	if (result == noErr) {
		assert (CurrVCB == vcb2);
		switch (*status) {

			/* Determined by full pathname */
			case dtmvFullPathame:
				/* Current directory is always the root directory */
				*currentDirectory = fsRtDirID;
				break;

			/* Determined by volume refNum or by drive number */
			case dtmvVRefNum:
			case dtmvDriveNum:
				/* Current directory is dirID if specified; */
				/* otherwise, it's the root directory */
				if (dirID != 0) {
					*currentDirectory = dirID;
				} else {
					*currentDirectory = fsRtDirID;
				}
				break;

			/* Determined by working directory refNum */
			case dtmvWDRefNum:
				/* Current directory is dirID if specified; */
				/* otherwise, it's wdDirID in the WDCB */
				if (dirID != 0) {
					*currentDirectory = dirID;
				} else {
					result = UTResolveWDCB (0, 0, pb->fileParam.ioVRefNum, &wdCtrlBlockPtr);
					if (result == noErr) *currentDirectory = wdCtrlBlockPtr->wdDirID;
				}
				break;

			/* Determined by default volume */
			case dtmvDefault:
				/* Current directory is dirID if specified; */
				/* otherwise, it's the default directory */
				if (dirID != 0) {
					*currentDirectory = dirID;
				} else {
					wdpb.ioNamePtr = NULL;
					/* the name not needed */
					result = UTGetDefaultVol (&wdpb);
					if (result == noErr) {
						*currentDirectory = wdpb.ioWDDirID;
					}
				}
				break;

			/* This should never happen, but... */
			default:
				result = paramErr;
				break;
		}
	}
	return result;
}

OSErr ParsePathName (ItemID currentDirectory, StringPtr namePtr, ObjectInfoRecPtr objectInfo)
{
	OSErr result;
	ParsePathRec parseRec;
	Str31 lastComponent;

	/* init parseRec to start parsing at beginning of namePtr */
	parseRec.namePtr = namePtr;
	parseRec.startOffset = 0;
	parseRec.componentLength = 0;
	parseRec.moreName = false;
	parseRec.foundDelimiter = false;

	/* Get the length of the volume name (if any) and use it to adjust
	 * the startOffset to point to the beginning of the partial pathname.
	 * UTParsePathname also ensures the pathname is not NULL or an empty
	 * string. */
	result = UTParsePathname (&parseRec.startOffset, parseRec.namePtr);

	if (result == noErr) {
		/* Check for leading delimiter of the partial pathname */
		result = UTGetPathComponentName (&parseRec);
		if (result == noErr) {
			if (parseRec.componentLength == 0 && parseRec.foundDelimiter) {
				/* Get past initial delimiter */
				++parseRec.startOffset;
			}
			/*
			 * At this point, startOffset is pointing at the first
			 * character of the first component name (if any).
			 */
			/* Parse until there is no more pathname to parse. */
			while ((result == noErr) && parseRec.moreName) {
				/* Search for the next delimiter from startOffset. */
				result = UTGetPathComponentName (&parseRec);
				if (result == noErr) {
					if (parseRec.componentLength == 0) {
						/* A delimiter was found immediately following another
						 * delimiter. Set current directory to parent of
						 * current directory. If current directory is
						 * fsRtDirID, then return bdNamErr (that handles the
						 * case of 'Root::' or 'Root:SubDir:::', etc.) */
						if (currentDirectory != fsRtDirID) {
							/* Call foreign file system specific code to get
							 * parent of current directory */
							ItemID parID;
							result = FFSGetParent (currentDirectory, parID);
							currentDirectory = parID;
						} else {
							/* Pathname tried to go up from root directory */
							result = bdNamErr;
						}
						/* startOffset = start of next component (if any). */
						++parseRec.startOffset;
					} else if (parseRec.moreName) {
						/* A component was found and it isn't the last
						 * component, so it must be a directory. Make
						 * lastComponent in current directory the new current
						 * directory. */
						lastComponent[0] = parseRec.componentLength;
						memcpy (&lastComponent[1], (Ptr)((long)parseRec.namePtr + parseRec.startOffset + 1), parseRec.componentLength);
						/* Call foreign file system specific code to get
						 * directory by name in currentDirectory */
						ItemID theID;
						result = FFSGetDir (lastComponent, currentDirectory, theID);
						currentDirectory = theID;
						/* startOffset = start of next component (if any). */
						parseRec.startOffset += parseRec.componentLength + 1;
					}
				}
			}
			if (result == noErr) {
				/* There is no more pathname to parse. */
				if (parseRec.componentLength == 0) {
					/* The pathname ended with '::' or the pathname was
					 * simply a volume name ('VolName:'), so current
					 * directory is the object. */
					objectInfo->isDir = true;
					objectInfo->fileName[0] = 0;
					objectInfo->dirID = currentDirectory;
				} else {
					/* The pathname ended with 'name:' or 'name', so name is
					 * the object. */
					objectInfo->dirID = currentDirectory;
					objectInfo->fileName[0] = parseRec.componentLength;
					objectInfo->isDir = false;
					memcpy (&(objectInfo->fileName[1]), (Ptr)((long)parseRec.namePtr + parseRec.startOffset + 1), parseRec.componentLength);
					/* this is in the UDF FS, but when mounting "BRUSA.img", Disk Copy reports a "-43" error:
					  (perhaps because in UDF FS this is only done when "isDir" is passed as a non-zero argument,
					  and which is zero in some cases, so that this code is not executed then)
					// we must check the actual dir entry in order to know if this is a dir or not
					// (e.g, GetCatInfo & CatMove can pass a dir name without ":" at end in the ioNamePtr).
					long theID;
					result = FFSGetDir (objectInfo->fileName, currentDirectory, theID);
					if (result == noErr) {
						objectInfo->isDir = true;
						objectInfo->fileName[0] = 0;
						objectInfo->dirID = theID;
					}
					*/
				}
			}
		}
	} else {
		if ((namePtr == NULL) || (namePtr[0] == 0)) {
			// pathname was NULL or a zero length string, so we found a
			// directory at the end of the string
			objectInfo->isDir = true;
			objectInfo->fileName[0] = 0;
			objectInfo->dirID = currentDirectory;
			result = noErr;
		} else {
			result = bdNamErr;
		}
	}
	return result;
}


Boolean IsVolumeEjected (VCBPtr theVCB)
// A volume is eject if its driver refnum is positive.
{
	return theVCB->vcbDRefNum > 0;
}

DrvFlagsPtr FlagsForDrv (DrvQElPtr drvQEl)
{
	return ((DrvFlagsPtr) drvQEl) - 1;
}

Boolean DriveIsEjectable (DrvQElPtr drvQEl)
{
	DrvFlagsPtr drvFlags;
	drvFlags = FlagsForDrv(drvQEl);
	return ( (drvFlags->dQFlagDiskInPlace < kDrvQElNonEjectableDiskPresent) || 
				(drvFlags->dQFlagDiskInPlace >= kDrvQElNonEjectableDiskPresentButDriverWantsCall) );
}

Boolean IsVolumeOffline (VCBPtr theVCB)
// A volume is offline if it isn't in any drive.
{
	return theVCB->vcbDrvNum == 0;
}


// EOF
