/*
 * HFS CI Procs.c
 */

#include <string.h>
#include "PascalLib.h"

#include "FSM.h"
#include "FSM Utils.h"
#include "FS Support.h"
#include "HFS CompInt.h"
#include "HFS CI Procs.h"

typedef struct {
	// extended VCB fields every FFS should have
	unsigned long	ffsCreator;			// our creator type (another good way to ID remounts)
	short			dbRefNum;			// refNum for Desktop Manager file if we implement our own
	Handle			finderHandle;
} MyVolInfoRec;

VCBPtr			CurrVCB = NULL;
MyVolInfoRec	*MyVolInfo = NULL;


#define MIN(a,b) (a<b?a:b)

static	long	FSVolGlobalsOffset = 0, MyVolGlobalsOffset = 0;
static	long	origZero = 0;

// global file reference numbers
#define	DT_objID		5	// Desktop database file reference number


/*
 * Setup functions (called when any of the FFS functions is entered/exited)
 */

void Entering (VCBPtr vcb)
{
	assert ((signed long)CurrVCB <= 0);
	if (vcb && Assert (vcb->vcbFSID == kFSID && ((MyVolInfoRec*)((Ptr)vcb + MyVolGlobalsOffset))->ffsCreator == kFSID)) {
		CurrVCB = vcb;
		MyVolInfo = (MyVolInfoRec*)((Ptr)vcb + MyVolGlobalsOffset);
		FSAtomicStart (vcb, (Ptr)vcb + FSVolGlobalsOffset);
	} else {
		// happens with Eject, MountVol
		CurrVCB = (VCB*)0xB5555533;	// some negative number, also causing a bus error when mistakenly deref'd
		MyVolInfo = (MyVolInfoRec*)0xB5555511;
		FSAtomicStart (NULL, NULL);
	}
}

void Exiting (void)
{
	FSAtomicStop ();
	CurrVCB = NULL;
	MyVolInfo = NULL;
}


/*
 * FSM support functions (called by FS Communications Function)
 */

OSErr DoLg2Phys (VCBPtr vcb, FCBRecPtr fcb, 
				short fileRefNum, unsigned long filePosition, unsigned long reqCount,
				unsigned long *volOffset, long *contiguousBytes)
{
	return ioErr;
/* +++ later!
	#pragma unused (vcb)

	OSErr	err = noErr;
	
	// Due to a bug in FSM <1.2.6, the fcb-ptr is not correct (instead of its address, the
	// content of its first long is passed). We have to get the FCB on our own here:
	if (fsmVersion < (UInt32) 0x0126) {
		UTResolveFCB (fileRefNum, &fcb);
	}
	
	LogFStr (LKUser1, nil, "Lg2Phys pos %lx req %lx", filePosition, reqCount);

	long firstExtLen = RoundUp (((long*)fcb->fcbExtRec)[1], 512);
	long extentStart = ((long*)fcb->fcbExtRec)[0];
	long extentOfs = filePosition;
	if (filePosition + reqCount > firstExtLen) {
		DirRecRef nxtObjRef = ((long*)fcb->fcbExtRec)[2];
		if (filePosition < firstExtLen) {
			*contiguousBytes = firstExtLen - filePosition;
		} else if (nxtObjRef == 0) {
			// no more extents
			err = eofErr;
		} else {
			// find the dirRec with the extent
			long rem = filePosition - firstExtLen;
			do {
				// find all the extents and sum up their length
				static DirEntryDesc entry;
				err = ISOGetDirRecByRef (nxtObjRef, entry, nxtObjRef);
				assert (!err);
				if (err) break;
				long extLen = RoundUp (GetL(entry.dirRec.dataLen_L), 512);
				if (rem < extLen) {
					// found it
					extentStart = GetL(entry.dirRec.firstLogBlk_L);
					extentOfs = rem;
					if (rem + reqCount > extLen) {
						*contiguousBytes = extLen - rem;
					} else {
						*contiguousBytes = reqCount;
					}
					break;
				}
				rem -= extLen;
				if (!(entry.dirRec.fileFlags & multiextentMask)) {
					// no more extents
					err = eofErr;
				}
			} while (!err);
		}
	} else {
		*contiguousBytes = reqCount;
	}
	*volOffset = (extentStart * ISO_LOGICAL_SECTOR_SIZE + extentOfs) >> 9;	// >>9 is required in contrary to the FSM 1.2 docs

	LogFStr (LKUser1, nil, " returns ofs %ld len %lx", *volOffset, *contiguousBytes);

	return err;
*/
}

OSErr DoIDDisk (Ptr diskBlock)
{
	#pragma unused (diskBlock)
	// no rejection here; that means that we'll always get a MountVol call
	return noErr;
}

OSErr DoIDVolMount (VolumeMountInfoHeader *volMountInfo)
{
	if (volMountInfo->media == kFSID) {
		return noErr;
	} else {
		return extFSErr;
	}
}

static openFork (Boolean highLevel, void *paramBlock, Boolean rsrcFork)
{
	OSErr			err;
	HFileInfo		*f = (HFileInfo*)paramBlock;
	DirEntryInfo	info;
	FSItemProperties	prop;

	err = UTCheckVolOffline (CurrVCB->vcbVRefNum);
	if (err) return err;

	err = GetDirRef (highLevel, paramBlock, false, info);
	if (err) return err;
	
	FSItemGetItemProperties (info.objRef, &prop);
	
	if (prop.isDir) {
		// it's a directory, not a file
		return fnfErr;
	}

	// check permissions - let's simply reject any write attempt here
	short perm = ((IOParam*)paramBlock)->ioPermssn;
	if (perm == fsWrPerm || perm == fsRdWrPerm || perm == fsRdWrShPerm) {
		return permErr;
	}
	
	long fileLen = rsrcFork ? prop.rsrcForkLogLen : prop.dataForkLogLen;
	
	// all error checks are done - now open the file, allocate the FCB
	FCBRecPtr fcb;
	err = UTAllocateFCB (&f->ioFRefNum, &fcb);
	if (!Assert (!err)) return err;

	memset (fcb, 0, sizeof (FCBRec));
	fcb->fcbFlNm = info.objID;
	fcb->fcbEOF = fileLen;
	fcb->fcbFlags = fcbOwnClumpMask | (rsrcFork?fcbResourceMask:0);
	fcb->fcbPLen = RoundUp (fcb->fcbEOF, 512);
	fcb->fcbVPtr = CurrVCB;
	fcb->fcbClmpSize = fcb->fcbPLen;
	fcb->fcbDirID = info.parID;
	FSItemFileOpen (info.objRef, rsrcFork, f->ioFRefNum, fcb, (FSOpenFileRecPtr)&fcb->fcbExtRec);
	FSItemGetItemName31 (info.objRef, fcb->fcbCName);

	fcb->fcbFType = prop.fileType;
	
	// this would optionally retrieve an appropriate type & creator from the file extension, using Internet Config:
	//	GetDefaultTypeAndCreator (fcb->fcbCName, &fcb->fcbFType, nil);

	return noErr;
}

OSErr DoOpen (Boolean highLevel, void *paramBlock)
{
	return openFork (highLevel, paramBlock, false);
}

OSErr DoOpenRF (Boolean highLevel, void *paramBlock)
{
	return openFork (highLevel, paramBlock, true);
}

OSErr DoClose (Boolean highLevel, void *paramBlock)
{
	#pragma unused (highLevel)
	return UTReleaseFCB (((HFileInfo*)paramBlock)->ioFRefNum);
}

OSErr DoRead (Boolean highLevel, void *paramBlock)
{
	#pragma unused (highLevel)
	IOParam	*pb = (IOParam*)paramBlock;
	OSErr	err;
	Boolean	eofError = false;

	pb->ioActCount = 0;

	err = UTCheckVolOffline (CurrVCB->vcbVRefNum);
	if (err) return err;
	
	FCBRecPtr fcb;
	err = UTResolveFCB (pb->ioRefNum, &fcb);
	if (!err) {
		long	reqCount = pb->ioReqCount;
		long	startPos;
		short	posMode = pb->ioPosMode & 0x000F;

		/*
		 * calculate start position and length
		 */
		if (posMode == fsFromStart) {
			startPos = 0;
		} else if (posMode == fsFromLEOF) {
			startPos = fcb->fcbEOF;
		} else if (posMode == fsFromMark) {
			startPos = fcb->fcbCrPs;
		} else if (posMode == fsAtMark) {
			startPos = fcb->fcbCrPs - pb->ioPosOffset;	// ignore ioPosOffset
		} else {
			return paramErr;
		}
		startPos += pb->ioPosOffset;
		if (startPos < 0) {
			return posErr;
		}
		if (startPos > fcb->fcbEOF) {
			return eofErr;
		}
		if (startPos+reqCount > fcb->fcbEOF) {
			eofError = true;
			reqCount = fcb->fcbEOF - startPos;
		}
		fcb->fcbCrPs = startPos;

		/*
		 * Read the data
		 */
		if (reqCount > 0) {
			UInt32 actCount = 0;
			err = FSItemFileRead ((FSOpenFileRecPtr)&fcb->fcbExtRec, startPos, reqCount, pb->ioBuffer, &actCount);
			pb->ioActCount = actCount;
		}

		pb->ioPosOffset = (fcb->fcbCrPs += pb->ioActCount);
	}
	if (!err && eofError) err = eofErr;
	return err;
}

OSErr DoGetEOF (Boolean highLevel, void *paramBlock)
{
	#pragma unused (highLevel)
	IOParam	*pb = (IOParam*)paramBlock;
	FCBRecPtr fcb;
	OSErr err = UTResolveFCB (pb->ioRefNum, &fcb);
	if (!err) {
		pb->ioMisc = (Ptr) (pb->ioPosOffset = fcb->fcbEOF);
	} else {
		pb->ioMisc = (Ptr) (pb->ioPosOffset = 0);
	}
	return err;
}

OSErr DoGetFPos (Boolean highLevel, void *paramBlock)
{
	#pragma unused (highLevel)
	IOParam	*pb = (IOParam*)paramBlock;
	FCBRecPtr fcb;
	OSErr err = UTResolveFCB (pb->ioRefNum, &fcb);
	if (!err) {
		pb->ioPosOffset = fcb->fcbCrPs;
	} else {
		pb->ioPosOffset = 0;
	}
	pb->ioReqCount = pb->ioActCount = pb->ioPosMode = 0;
	return err;
}

OSErr DoSetFPos (Boolean highLevel, void *paramBlock)
{
	#pragma unused (highLevel)
	IOParam	*pb = (IOParam*)paramBlock;
	FCBRecPtr fcb;
	OSErr err = UTResolveFCB (pb->ioRefNum, &fcb);
	if (!err) {
		long	pos;
		short	posMode = pb->ioPosMode & 0x000F;
		if (posMode == fsFromStart) {
			pos = 0;
		} else if (posMode == fsFromLEOF) {
			pos = fcb->fcbEOF;
		} else if (posMode == fsFromMark) {
			pos = fcb->fcbCrPs;
		} else if (posMode == fsAtMark) {
			pos = fcb->fcbCrPs - pb->ioPosOffset;		// ignore ioPosOffset - do nothing
		} else {
			return paramErr;
		}
		fcb->fcbCrPs = pos + pb->ioPosOffset;
		if (fcb->fcbCrPs < 0) {
			// HFS leaves the position at the negative value
			err = posErr;
		} else if (fcb->fcbCrPs > fcb->fcbEOF) {
			fcb->fcbCrPs = fcb->fcbEOF;
			err = eofErr;
		}
		pb->ioPosOffset = fcb->fcbCrPs;
	}
	return err;
}

OSErr DoGetFCBInfo (Boolean highLevel, void *paramBlock)
{
	#pragma unused (highLevel,paramBlock)
	// HFS has already filled everything in
	return noErr;
}

OSErr DoGetVolInfo (Boolean highLevel, void *paramBlock)
{
	#pragma unused (highLevel)

	HParmBlkPtr		hPBPtr;
	WDCBRecPtr		wdcb;
	OSErr			err;
	
	// The File Manager has already filled in the parameter block using the
	// data in our VCB.  All we have to do is give the foreign file system code
	// a chance to handle the case of a working directory number being passed

	hPBPtr = (HParmBlkPtr) paramBlock;
	err = UTResolveWDCB (0, 0, hPBPtr->volumeParam.ioVRefNum, &wdcb);
	if (!err) {
		err = UTCheckVolOffline (CurrVCB->vcbVRefNum);
		if (!err) {
			// If we were passed a working directory and the volume is online, then
			// ioVNmFls = the number of files in the working directory instead of
			// the number of file in the root directory (this is one of those weird
			// things we do to try to be compatible with old MFS file system calls).
			FSItemRef ref;
			err = FSItemLookup (wdcb->wdDirID, &ref);
			if (!err) {
				FSItemCountItems (ref, false, (UInt32*)&hPBPtr->volumeParam.ioVNmFls);
			}
		}
	} else {
		err = noErr;
	}
	return err;
}

static OSErr fillInFInfo (FileParam* f, FSItemRef &ref, FSItemProperties &prop, Str31 theName)
{
	#pragma unused (ref)
	
	OSErr err = noErr;
	
	f->ioFlRLgLen = prop.rsrcForkLogLen;
	f->ioFlRPyLen = RoundUp (prop.rsrcForkPhysLen, 512);
	f->ioFlLgLen = prop.dataForkLogLen;
	f->ioFlPyLen = RoundUp (prop.dataForkPhysLen, 512);
	f->ioFlRStBlk = 0;	// only used in MFS
	f->ioFlStBlk = 0;	// only used in MFS
	f->ioFlCrDat = prop.crDate;
	f->ioFlMdDat = prop.mdDate;
	f->ioFlAttrib = 0;
	f->ioFlVersNum = 0;	// (=ioACUser)
	
	// set FndrInfo
	memset (&f->ioFlFndrInfo, 0, sizeof (f->ioFlFndrInfo));
	f->ioFlFndrInfo.fdLocation.v = -1;
	f->ioFlFndrInfo.fdLocation.h = -1;

	f->ioFlFndrInfo.fdFlags = kHasBeenInited;

	f->ioFlFndrInfo.fdType = prop.fileType;
	f->ioFlFndrInfo.fdCreator = prop.creator;

	// this would optionally retrieve an appropriate type & creator from the file extension, using Internet Config:
	//GetDefaultTypeAndCreator (theName, &f->ioFlFndrInfo.fdType, &f->ioFlFndrInfo.fdCreator);

	if (prop.isReadOnly) {
		f->ioFlAttrib |= 1;	// 1: locked flag
	}
	
	return err;
}

static void setupFlAttribAndFRefNum (long recID, FileParam *f)
{
	// check if this file is open and set 'f->ioFRefNum' appropriately
	FCBRecPtr fcb;
	if (UTLocateFCB (CurrVCB, recID, nil, &f->ioFRefNum, &fcb) == noErr) {
		short fRefNum = f->ioFRefNum;
		do {
			if (fcb->fcbTypByt & fcbResourceMask) {
				f->ioFlAttrib |= 0x84;	// mark rsrc fork open
			} else {
				f->ioFlAttrib |= 0x88;	// mark data fork open
			}
		} while (UTLocateNextFCB (CurrVCB, recID, nil, &fRefNum, &fcb) == noErr);
	} else {
		f->ioFRefNum = 0;
	}
}

static void validateIconPosition (FileParam* f)
{
	if (f->ioFlFndrInfo.fdLocation.v == 0 && f->ioFlFndrInfo.fdLocation.h == 0) {
		// force the Finder to re-arrange the icons
		f->ioFlFndrInfo.fdLocation.v = -1;
		f->ioFlFndrInfo.fdLocation.h = -1;
	}
}

OSErr DoGetFInfo (Boolean highLevel, void *paramBlock)
// attention: index counts files only, excluding dirs!
{
	OSErr		err;
	short		origIdx;
	FileParam	*f = (FileParam*)paramBlock;
	DirEntryInfo		info;
	FSItemProperties	prop;

	#if TRACING
		HFileInfo *hf = (HFileInfo*)paramBlock;
		LogFStr (LKUser12, HERE, "GetFInfo - dirID %ld, dirIndex %d, \"%#s\"", highLevel?hf->ioDirID:0, hf->ioFDirIndex, hf->ioNamePtr?hf->ioNamePtr:"\p(NULL)");
	#endif

	err = UTCheckVolOffline (CurrVCB->vcbVRefNum);
	if (err) goto bail;
	
	origIdx = f->ioFDirIndex; if (f->ioFDirIndex < 0) f->ioFDirIndex = 0;
	err = GetDirRef (highLevel, paramBlock, 2, info);
	f->ioFDirIndex = origIdx;
	if (err) goto bail;
	
	FSItemGetItemProperties (info.objRef, &prop);

	if (prop.isDir) {
		#if TRACING
			LogFStr (LKWarning, HERE, "GetFInfo on a Dir! - dirID %ld, dirIndex %d, \"%#s\"", highLevel?hf->ioDirID:0, hf->ioFDirIndex, hf->ioNamePtr?hf->ioNamePtr:"\p(NULL)");
		#endif
		err = fnfErr;
		goto bail;
	}
	
	// fill in the fields
	Str31 theName;
	FSItemGetItemName31 (info.objRef, theName);
	if (f->ioFDirIndex > 0) {
		pstrcpy (f->ioNamePtr, theName);
	}
	err = fillInFInfo (f, info.objRef, prop, theName);
	if (err) goto bail;
	f->ioFlNum = info.objID;	// the File ID
	
	// set hidden bit
	if (prop.isHidden) {
		f->ioFlFndrInfo.fdFlags |= fInvisible;
	}
	
	setupFlAttribAndFRefNum (info.objID, f);

	validateIconPosition (f);

bail:
	#if TRACING
		AssertNum (!err || err == fnfErr, err);
	#endif
	return err;
}

static Boolean keyPressed (KeyMap km, short code)
{
	return (((Ptr)km)[code/8] & (1<<(code % 8))) != 0;
}

static 	DrvQElPtr	driveQElem;

/*
static OSErr uncachedRead (long start, long bytes, Ptr buffer, short cacheOptions)
{
	return DeviceRead (driveQElem->dQDrive, driveQElem->dQRefNum, start, bytes, buffer, cacheOptions);
}
*/

/*
static Boolean isSameVol (VolumeDescriptor *pvd, VCBPtr vcb)
{
	MyVolInfoRec		*myVolInfo;
	ISOVolumeGlobalsPtr isoGlobs;
	myVolInfo = (MyVolInfoRec*) ((Ptr)vcb + MyVolGlobalsOffset);
	isoGlobs = (ISOVolumeGlobalsPtr)((Ptr)vcb + FSVolGlobalsOffset);
	return
		(myVolInfo->ffsCreator == kFSID) &&
		(vcb->vcbFSID == kFSID) &&
		(ISODateStr2MacTime ((ISOTimeStr*)&pvd->creationDate) == vcb->vcbCrDate) &&
		(ISODateStr2MacTime ((ISOTimeStr*)&pvd->modificationDate) == vcb->vcbLsMod) &&
		IsSameVD (isoGlobs, vcb, pvd);
}

static void getVolNameFromVD (const char *isoName, VolDescTypes vdType, unsigned char *volName)
{
	CopyISOToPString (isoName, 32, vdType, volName, sizeof(Str27));
	if (volName[0] == 0) {
		if (vdType != vdJoliet) {
			pstrcpy (volName, "\pISO9660 Volume");	// emergency name
		} else {
			pstrcpy (volName, "\pJoliet Volume");	// emergency name
		}
	}
	for (int i = 1; i <= volName[0]; ++i) if (volName[i] == ':') volName[i] = '_';	// remove ":", just in case
}

static Boolean isRemount (VolumeDescriptor *pvd, VolDescTypes vdType, VCBPtr &newVCB) {
	OSErr		err;
	Str31		volName;	// mind. 1 Zeichen grer als Str27!
	short		foundVRefNum;
	VCBPtr 		foundVCB;
	short		moreMatches;
	
	if (FSVolGlobalsOffset == 0) return false;	// can't be a remount as long as this value hasn't been set

	getVolNameFromVD (pvd->volumeID, vdType, volName);
	pstrcatchar (volName, ':');
	err = UTLocateVCBByName (volName, &moreMatches, &foundVRefNum, &foundVCB);
	while (!err) {
		//DebugStr ("\pThis seems to be a re-mount. Check it:");
		if (isSameVol (pvd, foundVCB)) {
			assert (foundVRefNum == foundVCB->vcbVRefNum);
			newVCB = foundVCB;
			return true;
		}
		if (!moreMatches) break;
		err = UTLocateNextVCB (volName, &moreMatches, &foundVRefNum, &foundVCB);
	}
	return false;
}
*/

OSErr DoMountVol (Boolean highLevel, void *paramBlock)
{
	#pragma unused (highLevel)

	OSErr						err;
	VolumeParam					*pb = (VolumeParam *) paramBlock;
	unsigned short				sysVCBLen;
	VCB							*vcb;
	short						vRefNum, driveNumber;
	//char						buf[2048];	// temporary buffer for reading CD sectors
	
	// reject if mount has been disabled by user
	if (gMountEnableFlags == 0) {
		LogStr (LKUser11, HERE, "mounting is currently disabled");
		return extFSErr;
	}
	
	// Find the drive queue element
	driveNumber = pb->ioVRefNum;
	err = UTFindDrive (driveNumber, &driveQElem);
	if (err != noErr && err != extFSErr) {
		return err;
	}
	
	if (gDriveToMount && gDriveToMount != driveNumber) {
		LogPStr (LKUser11, HERE, "\pmounting not announced for drive ^0", driveNumber);
		return extFSErr;
	}
	gDriveToMount = 0;
	
	// Check if this volume is already mounted.
	/* +++ this is still pending to be revised
	Boolean remount = isRemount (vd, vdType, vcb);
	if (remount) {
		if (UTCheckVolOffline (vcb->vcbVRefNum) != noErr) {
			// Volume is offline, so bring it online
			vcb->vcbDRefNum = driveQElem->dQRefNum;
			vcb->vcbDrvNum = driveNumber;
		}
		driveQElem->dQFSID = kFSID;		// someone might have set this to zero, so we're setting it back again
		LogStr (LKUser11, HERE, "Remounted");
		return noErr;
	}
	*/

	// allocate the volume control block
	long fsAddSize = FSPrivateDataSize();
	err = UTAllocateVCB (&sysVCBLen, &vcb, fsAddSize + sizeof (MyVolInfoRec));

	// fill out the volume control block
	if (!err) {
		memset (vcb, 0, sysVCBLen + fsAddSize + sizeof (MyVolInfoRec));
		assert (sysVCBLen >= sizeof (VCB));
		
		MyVolGlobalsOffset = sysVCBLen;
		FSVolGlobalsOffset = MyVolGlobalsOffset + sizeof (MyVolInfoRec);
		// pass globals to FS module
		FSAtomicStart (vcb, (Ptr)vcb + FSVolGlobalsOffset);
		FSFeedPrivateDataPtr (gFSPrivateDataPtr);
		
		/*
		 * setup VCB values
		 */
		vcb->vcbFlags = 0;
		vcb->vcbSigWord = 0x4244;
		vcb->vcbCrDate = 1;	// must not be zero!
		vcb->vcbVolBkUp = vcb->vcbLsMod = 0;
		vcb->vcbAtrb = 0x0080;		// only locked by hardware, not by software
		vcb->vcbVBMSt = 0;			// must be zero
		vcb->vcbAllocPtr = 0;		// Unused
		vcb->vcbAlBlSt = 0;
		vcb->vcbNxtCNID = 16;
		vcb->vcbFreeBks = 0;
		vcb->vcbFSID = kFSID;
		vcb->vcbWrCnt = 0;

		// attention: setting the following two fields to zero means that the number of files and folders
		// on this volume is unknown - however, this will cause a zero-divide crash in Iomega "FindIt"!
		// So, if you want to support that tool despite that bug in it, you must not set these values to
		// zero!
		vcb->vcbFilCnt = 0;
		vcb->vcbDirCnt = 0;

		vcb->vcbDirBlk = 0;
		vcb->vcbDirIDM = 0;
		vcb->vcbOffsM = 0;
		
		/*
		 * setup private values for this volume
		 */
		MyVolInfo = (MyVolInfoRec*)((Ptr)vcb + MyVolGlobalsOffset);
		MyVolInfo->ffsCreator = kFSID;
		MyVolInfo->finderHandle = NewHandleSysClear (2);
		MyVolInfo->dbRefNum = 0;
		
		/*
		 * Install the new VCB
		 */
		err = UTAddNewVCB (driveQElem->dQDrive, &vRefNum, vcb);
		if (err) {
			UTDisposeVCB (vcb);
		} else {
			driveQElem->dQFSID = kFSID;	// but Apple's ISO 9660 File Access leaves a zero here.
			pb->ioVRefNum = vRefNum;
		}

		CurrVCB = vcb;
	}

	if (!err) {
		err = FSFinishMount (vcb);
		if (err) {
			UTDisposeVCB (vcb);
			driveQElem->dQFSID = 0;
		}
	}

	LogPStr (LKUser11, HERE, "\pMountVol finished with err: ^0", err);

	return err;
}

OSErr DoUnmountVol (Boolean highLevel, void *paramBlock)
// We only get this call when either all user files are closed or if highLevel is true.
// The latter case means an emergency unmount (when using RS/RB in MacsBug).
// In both cases we only flush our cached directory information.
{
	#pragma unused (highLevel, paramBlock)

	LogStr (LKUser11, HERE, "PBUnmount");
	#if TRACING
		//DebugStr("\punmount");
	#endif
	
	if (MyVolInfo->finderHandle) DisposeHandle (MyVolInfo->finderHandle);
	FSUnmountVol ();

	// close all files (including user files)
	{
		short fileRefNum = 0;
		FCBRecPtr fcb;
		while (UTIndexFCB (CurrVCB, &fileRefNum, &fcb) == noErr) {
			UTReleaseFCB (fileRefNum);
		}
	}
	UTTrashVolBlocks (CurrVCB);
	UTDisposeVCB (CurrVCB);

	return noErr;
}


OSErr DoEject (Boolean highLevel, void *paramBlock)
{
	#pragma unused (highLevel, paramBlock)
	LogStr (LKUser11, HERE, "PBEject");
	#if TRACING
		//DebugStr("\peject");
	#endif

	OSErr err;
	
	if ((signed long)CurrVCB > 0) {
		err = UTCheckVolOffline (CurrVCB->vcbVRefNum);
		if (!err && !IsVolumeEjected (CurrVCB)) {
			DrvQElPtr driveQElem;
			short drvNum = CurrVCB->vcbDrvNum;	// save this value for later
			err = UTFindDrive (-(CurrVCB->vcbDRefNum), &driveQElem);
			if (!err || err == extFSErr) {
				if (DriveIsEjectable (driveQElem)) {
					// clear the FSID (this is also done bei the Apple CD driver, so we do it just in case ...)
					// however, I have no indication that this is necessary - it's just for precaution
					driveQElem->dQFSID = 0;
					UTTrashVolBlocks (CurrVCB);
					if (UTEjectVol (CurrVCB) == noErr && drvNum > 0 && CurrVCB->vcbDRefNum < 0) {
						// put vol offline (is usually already done by FSM when Cmd-E is used in Finder,
						// but there are also cases when this does not happen automatically)
						CurrVCB->vcbDRefNum = drvNum;
						CurrVCB->vcbDrvNum = 0;
						LogStr (LKUser11, HERE, "Eject: vol was put offline");
					}
				}
			}
		}
	} else {
		Error ("Eject with vcb=nil");
		err = paramErr;
	}
	return noErr;
}

OSErr DoOffline (Boolean highLevel, void *paramBlock)
{
	#pragma unused (highLevel, paramBlock)
	LogStr (LKUser11, HERE, "PBOffline");
	if ((signed long)CurrVCB > 0) {
		if (!IsVolumeOffline (CurrVCB)) {

			// mark the volume as offline

			DrvQElPtr driveQElem;
			UTFindDrive (CurrVCB->vcbDrvNum, &driveQElem);
			driveQElem->dQFSID = 0;

			CurrVCB->vcbDRefNum = -(CurrVCB->vcbDrvNum);
			CurrVCB->vcbDrvNum = 0;
		}
	}
	return noErr;
}


OSErr DoOpenWD (Boolean highLevel, void *paramBlock)
{
	#pragma unused (highLevel)

	// Note: HFS returns the volRefNum if the dirID is fsRtDirID.
	//       UTAllocateWDCB() does not do this and creates always
	//       a unique WDCB.
	OSErr	err;
	ObjectInfoRec	objectInfo;
	WDPBPtr	pb = (WDPBPtr)paramBlock;
	ItemID	savedWDDirID, currentDirectory;
	short	status;
	DirEntryInfo	info;
	
	assert (highLevel);

	err = UTCheckVolOffline (CurrVCB->vcbVRefNum);
	if (err) return err;
	
	// save pb->ioWDDirID
	savedWDDirID = pb->ioWDDirID;

	err = GetCurrentDir ((ParmBlkPtr)pb, pb->ioWDDirID, &currentDirectory, &status);
	if (!err) {
		err = ParsePathName (currentDirectory, pb->ioNamePtr, &objectInfo);
		if (!err) err = GetDirRecRefByName (objectInfo.dirID, objectInfo.fileName, info);
		if (!err && objectInfo.isDir) {
			pb->ioWDDirID = objectInfo.dirID;
			pb->ioVRefNum = CurrVCB->vcbVRefNum;
			err = UTAllocateWDCB (pb);
		} else {
			err = dirNFErr;
		}
	} else {
		err = dirNFErr;
	}
	
	pb->ioWDDirID = savedWDDirID;
	
	return err;
}


OSErr DoCloseWD (Boolean highLevel, void *paramBlock)
{
#pragma unused (highLevel)
	return UTReleaseWDCB (((WDPBPtr)paramBlock)->ioVRefNum);
}


OSErr DoGetWDInfo (Boolean highLevel, void *paramBlock)
{
#pragma unused (highLevel, paramBlock)
	// Note: There's nothing to do because the FSM has already all set up
	return noErr;	// Must return noErr, not paramErr
}


OSErr DoGetCatInfo (Boolean highLevel, void *paramBlock)
{
	OSErr			err;
	HFileInfo		*f = (HFileInfo*)paramBlock;
	DirInfo			*d = (DirInfo*)paramBlock;
	DirEntryInfo	info;
	FSItemProperties	prop;
	
	#if TRACING
		LogFStr (LKUser12, HERE, "GetCatInfo - dirID %ld, dirIndex %d, \"%#s\"", f->ioDirID, f->ioFDirIndex, f->ioNamePtr?f->ioNamePtr:"\p(NULL)");
	#endif
	
	err = UTCheckVolOffline (CurrVCB->vcbVRefNum);
	if (err) goto bail;
	
	err = GetDirRef (highLevel, paramBlock, true, info);
	AssertNum (!err, err);
	if (err) goto bail;

	FSItemGetItemProperties (info.objRef, &prop);

	/*
	 * Now fill in the requested fields.
	 *
	 * Let's begin with the parent dir ID
	 */
	if (f->ioFDirIndex == -1) {
		UInt32 parID;
		err = FFSGetParent (info.objRef, &parID);
		AssertNum (!err, err);
		if (err) {
			ErrorNum ("FFSGetParent", info.objID);
		} else {
			d->ioDrParID = parID;
		}
		if (err) goto bail;
	} else {
		d->ioDrParID = info.parID;
	}

	/*
	 * Fill in the name of the file or directory
	 */
	Str31 theFName;
	FSItemGetItemName31 (info.objRef, theFName);
	if (f->ioNamePtr) {
		if (f->ioFDirIndex != 0) {
			pstrcpy (f->ioNamePtr, theFName);
		}
	} else {
		pstrcpy (f->ioNamePtr, theFName);
	}

	f->ioFVersNum = 0;
	f->filler1 = 0;
	
	/*
	 * Fill in the rest of the field, depending on whether it's a dir or a file
	 */
	//+++ einige der Werte sind noch nicht ganz korrekt (u.a. die Attrs und FndrInfos):
	if (prop.isDir) {
		// it's a directory
		d->ioDrDirID = info.objID;
		d->ioFlAttrib = ioDirMask;
		d->ioACUser = 0;
		d->ioFRefNum = 0;

		memset (&d->ioDrUsrWds, 0, sizeof (d->ioDrUsrWds));
		memset (&d->ioDrFndrInfo, 0, sizeof (d->ioDrFndrInfo));

		// consts from Toast:
		static const Rect	cLevel1FolderRect 	= { 46, 8, 258, 420 };	// +++ Wert abhngig von info.dirDepth bestimmen, siehe Toast
		static const Point	cScrollPosition		= { -4, -21 };

		d->ioDrUsrWds.frFlags = kHasBeenInited;
		d->ioDrUsrWds.frView = 0x0200;				// 2: view by name
		d->ioDrUsrWds.frRect = cLevel1FolderRect;	// may not be zero for "view by name"

		// why does Toast that? d->ioDrFndrInfo.frComment = 0x8000;
		
		d->ioDrCrDat = prop.crDate;
		d->ioDrMdDat = prop.mdDate;

		{
			UInt32 entries;
			err = FSItemCountItems (info.objRef, false, &entries);
			if (err) {
				WarnNum ("dir is inaccessible", info.objID);
				// we just ignore the error here in order to make the dir entry still visible in the listing
				entries = 0;
				err = noErr;
			}
			d->ioDrNmFls = (entries>0xffff)? 0xffff:entries;
		}
	} else {
		// it's a file

		f->ioDirID = info.objID;	// the File ID
		memset (&f->ioFlXFndrInfo, 0, sizeof (f->ioFlXFndrInfo));
		f->ioFlClpSiz = 0;

		err = fillInFInfo ((FileParam*)f, info.objRef, prop, theFName);
		AssertNum (!err, err);
		if (err) goto bail;

		setupFlAttribAndFRefNum (info.objID, (FileParam*)f);
	}
	f->ioFlBkDat = f->ioFlCrDat;	// means: has been backed up (0 would mean: never backed up)

	if (prop.isHidden) {
		f->ioFlFndrInfo.fdFlags |= fInvisible;
	}

	if (!err) validateIconPosition ((FileParam*)f);
	
bail:
	#if TRACING
		AssertNum (!err || err==fnfErr, err);
	#endif
	return err;
}

OSErr DoGetVolParms (Boolean highLevel, void *paramBlock)
{
	#pragma unused (highLevel)
	const long attrBits = (0
		// included:
			| 1L << bSupportsAsyncRequests
			| 1L << bNoMiniFndr
			| 1L << bNoSwitchTo			// (new 16 Sep 97) considered obsolete, so why not set it?
			| 1L << bNoBootBlks
			| 1L << bNoSysDir
			| 1L << bHasFileIDs
			| 1L << bHasDesktopMgr
			| 1L << bHasBTreeMgr		// must also be set if bHasDesktopMgr is set
			| 1L << bNoVNEdit
			| 1L << bNoDeskItems
		/* not included:
			| 1L << bLocalWList
			| 1L << bLimitFCBs
			| 1L << bNoLclSync
			| 1L << bTrshOffLine
			| 1L << bAccessCntl
			| 1L << bHasExtFSVol		// don't set because Finder would respond with "is sharing volume" when trying to erase
			| 1L << bHasOpenDeny
			| 1L << bHasCopyFile
			| 1L << bHasMoveRename
			| 1L << bHasShortName
			| 1L << bHasFolderLock
			| 1L << bHasPersonalAccessPrivileges
			| 1L << bHasUserGroupList
			| 1L << bHasCatSearch
			| 1L << bHasBlankAccessPrivileges
		*/
	);
	GetVolParmsInfoBuffer	vp = {2, attrBits, 0, 0};	// version=2 (use 1, if we only return 0xE bytes (see FSM 1.2 doc, D-8))

	/* include this bit if we support CatSearch:
		if (CanDirQuery ()) {
			vp.vMAttrib |= 1L << bHasCatSearch;
		}
	*/

	vp.vMLocalHand = MyVolInfo->finderHandle;

	IOParam	*pb = (IOParam*)paramBlock;
	long l = pb->ioReqCount;
	// 24 Nov 97 - taken out:	memset (pb->ioBuffer, 0, l);			// clear the dest buffer
	if (l > sizeof (vp)) l = sizeof (vp);	// copy only the known fields
	memcpy (pb->ioBuffer, &vp, l);
	pb->ioActCount = l;
	return noErr;
}

OSErr DoResolveFileIDRef (Boolean highLevel, void *paramBlock)
{
	#pragma unused (highLevel)

	FSItemRef	ref;
	ItemID		parID;
	FIDParam	*pb = (FIDParam*)paramBlock;
	OSErr		err;

	err = UTCheckVolOffline (CurrVCB->vcbVRefNum);
	if (!err) {
		err = FSItemLookup (pb->ioFileID, &ref);
		if (!err) {
			err = FFSGetParent (ref, &parID);
		}
	}
	if (!err) {
		pb->ioSrcDirID = parID;
		FSItemGetItemName31 (ref, pb->ioNamePtr);
	}
	
	if (!err) {
		if (FSItemIsDir (ref)) {
			err = notAFileErr;
		}
	} else if (err == fnfErr) {
		err = fidNotFound;	// otherwise System 7.5's Find File will not work!
	}
	
	return err;
}

OSErr DoMakeFSSpec (Boolean highLevel, void *paramBlock)
{
	#pragma unused (highLevel)

	OSErr		err;
	ItemID		dirID;
	FIDParam	*pb = (FIDParam*)paramBlock;
	short		status;
	ObjectInfoRec objectInfo;
	DirEntryInfo	info;

	if (!pb->ioDestNamePtr) return paramErr;
	memset (pb->ioDestNamePtr, 0, sizeof (FSSpec));

	err = UTCheckVolOffline (CurrVCB->vcbVRefNum);
	if (err) return err;

	assert (highLevel);
	err = GetCurrentDir ((ParmBlkPtr)paramBlock, pb->ioSrcDirID, &dirID, &status);
	if (!err) err = ParsePathName (dirID, pb->ioNamePtr, &objectInfo);
	if (!err) err = GetDirRecRefByName (objectInfo.dirID, objectInfo.fileName, info);
	if (err == 0 || err == fnfErr) {
		FSSpec *spec = (FSSpec*)pb->ioDestNamePtr;
		spec->vRefNum = CurrVCB->vcbVRefNum;
		
		if (objectInfo.fileName[0] == 0) {
			FSItemRef thisRef;
			err = FSItemLookup (objectInfo.dirID, &thisRef);
			if (!err) {
				UInt32 parID;
				err = FFSGetParent (info.objRef, &parID);
				if (!err) {
					spec->parID = parID;
				}
			}
			if (!err) {
				err = GetNameOfDir (objectInfo.dirID, spec->name);
			}
		} else {
			spec->parID = objectInfo.dirID;
			pstrcpy (spec->name, objectInfo.fileName);
		}
		
		#if TRACING
			LogFStr (LKUser13, HERE, "MakeFSSpec: dirID %ld, \"%#s\" -> dirID %ld, \"%#s\"", pb->ioSrcDirID, pb->ioNamePtr?pb->ioNamePtr:"\p(NULL)", spec->parID, spec->name);
		#endif

		if (!Assert(spec->name[0] != 0)) err = paramErr;	// just in case
	}
	return err;
}

OSErr DoSetVol (Boolean highLevel, void *paramBlock)
{
	OSErr			err;
	short			status;
	ItemID			dirID;
	ItemID			currentDirectory;
	ObjectInfoRec	objectInfo;
	
	err = UTCheckVolOffline (CurrVCB->vcbVRefNum);
	if (err) return err;
	
	if (highLevel) {
		dirID = ((WDPBPtr)paramBlock)->ioWDDirID;
	} else {
		dirID = 0;
	}
	
	// Call GetCurrentDir to find:
	//		currentDirectory for _HSetVol
	//		status for _SetVol
	err = GetCurrentDir((ParmBlkPtr)paramBlock, dirID, &currentDirectory, &status);

	if (!err) {
		if (highLevel) {
			// _HSetVol
			// Find the object from the partial pathname (if any):
			err = ParsePathName (currentDirectory, ((ParmBlkPtr)paramBlock)->volumeParam.ioNamePtr, &objectInfo);
			if (!err) {
				if (objectInfo.isDir) {	// make sure it's a directory
					err = UTSetDefaultVol (0, objectInfo.dirID, CurrVCB->vcbVRefNum);
				} else {
					err = dirNFErr;
				}
			}
		} else {
			// _SetVol
			if (status == dtmvWDRefNum) {
				// You are responding to _SetVol and ioVRefNum is a working directory
				// number, so nodeHint is ignored (nodeHint information is obtained from
				// the working directory control block), dirID must be 0 and refNum is
				// the working directory number.
				err = UTSetDefaultVol (0, 0, ((WDPBPtr)paramBlock)->ioVRefNum);
			} else {
				// You are responding to _SetVol and ioVRefNum is NOT a working directory
				// number, so nodeHint can contain optional file system specific information
				// about the directory, dirID MUST be set to fsRtDirID, and refNum is
				// the volume's vRefNum (obtained from the VCB).
				err = UTSetDefaultVol (0, fsRtDirID, CurrVCB->vcbVRefNum);
			}
		}
	}
	return err;
}

OSErr DoDTGetPath (Boolean highLevel, void *paramBlock)
{
	#pragma unused (highLevel)

	short fileRefNum;
	FCBRecPtr fcb;
	OSErr err;
	
	err = UTCheckVolOffline (CurrVCB->vcbVRefNum);
	if (err) return err;
	
	// is DT already opened?
	if (UTLocateFCB (CurrVCB, DT_objID, nil, &fileRefNum, &fcb) != noErr) {
		// create FCB entry for DT
		err = UTAllocateFCB (&fileRefNum, &fcb);
		if (err) return err;
		
		// set up the FCB fields
		// (we do it like Apple's driver: zero out almost all of the FCB)
		memset (fcb, 0, sizeof (FCBRec));
		fcb->fcbFlNm = DT_objID;
		fcb->fcbVPtr = CurrVCB;
	}
	
	DTPBPtr pb = (DTPBPtr)paramBlock;
	pb->ioDTRefNum = fileRefNum;

	return noErr;
}

OSErr DoDTCloseDown (Boolean highLevel, void *paramBlock)
{
	#pragma unused (highLevel)
	return UTReleaseFCB (((DTPBPtr)paramBlock)->ioDTRefNum);
}

OSErr DoDTGetInfo (Boolean highLevel, void *paramBlock)
{
	#pragma unused (highLevel, paramBlock)
	return paramErr;	// as suggested by FSM docs, pg 122 (Q&A)
}

OSErr DoDTGetIcon (Boolean highLevel, void *paramBlock)
{
	#pragma unused (highLevel)
	DTPBPtr pb = (DTPBPtr)paramBlock;
	if (pb->ioFileCreator == 'atco' && pb->ioFileType == 'atco') {
		// this case happens when "Mac OS Easy Open" is installed (new 14 Jan 99):
		if (Assert (pb->ioDTReqCount == 2)) {
			pb->ioTagInfo = 0;
			pb->ioDTActCount = pb->ioDTReqCount;
			*(short*)pb->ioDTBuffer = 2;	// this is what Easy Open 1.1.3 (of OS 7.6) expects here
			return noErr;
		}
	}
	//return fnfErr;
	return afpItemNotFound;
}

OSErr DoDTGetAPPL (Boolean highLevel, void *paramBlock)
{
	#pragma unused (highLevel)
	DTPBPtr pb = (DTPBPtr)paramBlock;
	if (pb->ioIndex == 0) {
		if (pb->ioNamePtr) pb->ioNamePtr[0] = 0;	// return a zero length file name
		pb->ioTagInfo = 0;				// just a dummy
		pb->ioAPPLParID = fsRtParID;	// use RtParID so that the file can never be found
		return noErr;
	}
	//return fnfErr;
	return afpItemNotFound;
}

OSErr DoDTGetIconInfo (Boolean highLevel, void *paramBlock)
{
	#pragma unused (highLevel, paramBlock)
	//return fnfErr;
	return afpItemNotFound;
}

OSErr DoDTGetComment (Boolean highLevel, void *paramBlock)
{
	OSErr			err;
	DTPBPtr			dt = (DTPBPtr)paramBlock;
	DirEntryInfo	info;

	err = UTCheckVolOffline (CurrVCB->vcbVRefNum);
	if (err) return err;

	short svRefNum = dt->ioVRefNum;
	dt->ioVRefNum = CurrVCB->vcbVRefNum;
	err = GetDirRef (highLevel, paramBlock, false, info);
	dt->ioVRefNum = svRefNum;
	if (err) return err;

	FSItemGetItemNameFull (info.objRef, (StringPtr)dt->ioDTBuffer);
	dt->ioDTActCount = dt->ioDTBuffer[0];
	p2cstr((StringPtr)dt->ioDTBuffer);

	return err;
}

OSErr DoCatSearch (Boolean highLevel, void *paramBlock)
{
	#pragma unused (highLevel, paramBlock)
	// not supported (yet)
	return paramErr;
}

// EOF
