/*
 * FS API 3.cp
 *
 * by Thomas Tempelmann, June 23, 00 at MacHack 15
 */

#pragma once

#include <string.h>
#include <FSM.h>
#include <Notification.h>
#include <DriverServices.h>
#include "PascalLib.h"
#include "LaunchLib.h"
#include "FS API.h"

static void InstallTMTask ();
Boolean HFSNamesMatch (const StringPtr name1, const StringPtr name2);

static long currSeed = 0, lastSeed = 0;
static long currModTime;
static Str63 lastAppName;
static VCBPtr gCurrVCB;
static short theVRefNum;


void FSAtomicStart (VCBPtr currVCB, Ptr privateDataPtr)
// attn: the pointers may be NULL!
{
	gCurrVCB = currVCB;
	theVRefNum = currVCB->vcbVRefNum;
}

void FSAtomicStop ()
{
	// NOP
}


/*
 * Item (file/dir) lookup
 */

OSErr FSItemLookup (ItemID itemID, FSItemRef *hdl)
{
	*hdl = (FSItemRef) itemID;
	return noErr;
}

OSErr FSItemLookup (ItemID parID, UInt32 index, Boolean filesOnly, FSItemRef *hdl)
{
	if (parID != fsRtDirID) return dirNFErr;
	if (index != 1) return fnfErr;
	*hdl = (FSItemRef) 16;
	return noErr;
}

OSErr FSItemLookup (ItemID parID, StringPtr name, FSItemRef *hdl)
{
	if (HFSNamesMatch (name, lastAppName)) {
		*hdl = (FSItemRef) 16;
		return noErr;
	}
	return fnfErr;
}


OSErr FSItemGetParent (FSItemRef hdl, FSItemRef *parHdl)
{
	long id = *(long*)&hdl;
	if (id == fsRtDirID) return fidNotFound;
	// ...
	return dirNFErr;
}


/*
 * Item access
 */

Boolean FSItemIsDir (FSItemRef hdl)
{
	long id = *(long*)&hdl;
	return id == fsRtDirID;
}

OSErr FSItemCountItems (FSItemRef hdl, Boolean filesOnly, UInt32 *items)
{
	*items = 1;
	return noErr;
}

void FSItemGetItemID (FSItemRef hdl, ItemID /*out*/ *itemID)
{
	long id = *(long*)&hdl;
	*itemID = id;
}

void FSItemGetItemProperties (FSItemRef hdl, FSItemProperties /*out*/ *prop)
{
	long id = *(long*)&hdl;
	prop->isDir = FSItemIsDir (hdl);
	if (prop->isDir) {
		// ...
	} else {
		prop->dataForkPhysLen = 512;
		prop->dataForkLogLen = 0;
		prop->rsrcForkPhysLen = 0;
		prop->rsrcForkLogLen = 0;
	}
	prop->crDate = currModTime;
	prop->mdDate = currModTime;
	prop->isReadOnly = false;
	prop->isHidden = false;
}


OSErr FSItemFileOpen (FSItemRef hdl, Boolean rsrcFork, short fRefNum, FCBRecPtr fcb, FSOpenFileRecord /*out*/ *ref)
{
	return noErr;
}

OSErr FSItemFileRead (FSOpenFileRecPtr ref, UInt32 fileOfs, UInt32 numBytes, Ptr destAddr, UInt32 *actCount)
{
	if (fileOfs == 0 && numBytes <= 10) {
		memcpy (destAddr, "abcdefghij", numBytes);
		*actCount = numBytes;
		return noErr;
	} else {
		return readErr;
	}
}

void FSItemFileClose (FSOpenFileRecPtr ref)
{
}

Boolean FSItemGetItemName31 (FSItemRef hdl, Str31 itemName)
// returns false if name is > 31 chars and if the FS has no good way to shorten it
{
	long dirID = *(long*)&hdl;
	if (dirID == fsRtDirID) {
		// root dir: fill in the volume name
		pstrcpy (itemName, gCurrVCB->vcbVN);
	} else {
		pstrcpy (itemName, lastAppName);
	}
	return true;
}

void FSItemGetItemNameFull (FSItemRef hdl, Str255 /*out*/ itemName)
{
	FSItemGetItemName31 (hdl, itemName);
}


/*
 * Volume mounting
 */

UInt32 FSPrivateDataSize ()
// must be a constant value!
{
	return 0;
}

OSErr FSFinishMount (VCBPtr vcb)
{
	long volSpaceSize = 32;
	
	vcb->vcbCrDate = 1;
	vcb->vcbVolBkUp = vcb->vcbLsMod = 0;

	long aBlks = 1L + (volSpaceSize/65536L);	// allocation blocks per sector
	vcb->vcbAlBlkSiz = aBlks * 2048L;
	vcb->vcbNmAlBlks = (volSpaceSize + aBlks-1) / aBlks;
	vcb->vcbClpSiz = vcb->vcbAlBlkSiz;

	pstrcpy (vcb->vcbVN, "\p/proc");

	vcb->vcbFilCnt = 1;
	vcb->vcbDirCnt = 0;
	
	InstallTMTask ();
	
	return noErr;
}

void FSUnmountVol ()
{
}


/*
 * Messaging
 */

void FSFeedPrivateDataPtr (Ptr dataPtr)
{
}



//----------------------------------------------------------------------
// TaskTime stuff

typedef void (*TaskTimeProc) (UInt32 param);

struct TaskTimeInfo {
	NMRec			nmRec;
	TaskTimeProc	proc;
	UInt32			param;
};
static TaskTimeInfo	cTaskTimeInfo;


// NMResponse is called by the Notification Manager at task time. All we have to do is
// remove the notification record, reset the busy flag, and call the user's proc.
// CAUTION: Doing it that way allows clients to post another request before the first one
// has completed.

static Boolean cIsBusy = false;

static pascal void NMResponse(NMRec *nmRec)
{
	TaskTimeProc	proc = cTaskTimeInfo.proc;
	UInt32			param = cTaskTimeInfo.param;

	::NMRemove(nmRec);
	cIsBusy = false;

	(*proc) (param);
}

static OSStatus CallBackAtTaskTime(TaskTimeProc proc, UInt32 param)
{
	if (cIsBusy)
		return paramErr; 

	cIsBusy = true;
	
	::memset(&cTaskTimeInfo, 0, sizeof(TaskTimeInfo));

	cTaskTimeInfo.proc = proc;
	cTaskTimeInfo.param = param;
	
	cTaskTimeInfo.nmRec.qType = nmType;
	cTaskTimeInfo.nmRec.nmResp = NewNMProc (NMResponse);

	OSStatus status = ::NMInstall(&cTaskTimeInfo.nmRec);

	if (status != noErr)
		cIsBusy = false;
		
	return status;
}


static void myTaskTime (UInt32 param)
{
	//if (*(Byte*)LMGetFSQHdr() == 0)
	{
		ProcessSerialNumber psn;
		GetFrontProcess (&psn);
		ProcessInfoRec pir;
		FSSpec spec;
		Str63 name;
		pir.processName = name;
		pir.processInfoLength = sizeof (pir);
		pir.processAppSpec = &spec;
		GetProcessInformation (&psn, &pir);
		pstrcpy (lastAppName, name);
		pstrcpy (lastAppName, name);
	}

	ProcessSerialNumber finderPSN;
	FSSpec finderSpec, spec;
	FSMakeFSSpec (theVRefNum, fsRtDirID, "\p", &spec);
	OSErr err = FindRunningAppBySignature ('FNDR', 'MACS', &finderPSN, &finderSpec);
	if (!err) {
		err = SendFSSEventToProcess (&finderPSN, 'fndr', 'fupd', &spec);
		FSMakeFSSpec (theVRefNum, fsRtDirID, lastAppName, &spec);
		err = SendFSSEventToProcess (&finderPSN, 'fndr', 'fupd', &spec);
	}
}

static TMTask	gTMTask;	// an extended task record

static pascal void MyTask ()
{
	CallBackAtTaskTime (myTaskTime, 0);
	
	long myDelay = 500;			// no. of milliseconds to delay
	PrimeTime((QElemPtr)&gTMTask, myDelay);
}

static void InstallTMTask ()
{
	long	myDelay;	// delay value

	myDelay	= 500;				// no. of milliseconds to delay
	gTMTask.tmAddr	= MyTask;	// get task address
	gTMTask.tmWakeUp	= 0;	// initialize tmWakeUp
	gTMTask.tmReserved	= 0;	// initialize tmReserved
	InsXTime((QElemPtr) &gTMTask);	// install the task record
	PrimeTime((QElemPtr) &gTMTask,	myDelay);

	pstrcpy (lastAppName, "\p abc");
}

// EOF
