/*
 * FS API.cp
 *
 * by Thomas Tempelmann, June 23, 00 at MacHack 15
 */

#include <string.h>
#include "PascalLib.h"
#include "FS API.h"

Boolean HFSNamesMatch (const StringPtr name1, const StringPtr name2);


static VCBPtr gCurrVCB;

void FSAtomicStart (VCBPtr currVCB, Ptr privateDataPtr)
// attn: currVCB may be NULL!
{
	gCurrVCB = currVCB;
}

void FSAtomicStop ()
{
}

/*
 * Item (file/dir) lookup
 */

OSErr FSItemLookup (ItemID itemID, FSItemRef *hdl)
{
	*hdl = (FSItemRef) itemID;
	return noErr;
}

OSErr FSItemLookup (ItemID parID, UInt32 index, Boolean filesOnly, FSItemRef *hdl)
{
	if (parID != fsRtDirID) return dirNFErr;
	if (index != 1) return fnfErr;
	*hdl = (FSItemRef) 16;
	return noErr;
}

OSErr FSItemLookup (ItemID parID, StringPtr name, FSItemRef *hdl)
{
	if (HFSNamesMatch (name, "\psome name")) {
		*hdl = (FSItemRef) 16;
		return noErr;
	}
	return fnfErr;
}



OSErr FSItemGetParent (FSItemRef hdl, FSItemRef *parHdl)
{
	long id = *(long*)&hdl;
	if (id == fsRtDirID) return fidNotFound;
	// ...
	return dirNFErr;
}


/*
 * Item access
 */

Boolean FSItemIsDir (FSItemRef hdl)
{
	long id = *(long*)&hdl;
	return id == fsRtDirID;
}

OSErr FSItemCountItems (FSItemRef hdl, Boolean filesOnly, UInt32 *items)
{
	*items = 1;
	return noErr;
}

void FSItemGetItemID (FSItemRef hdl, ItemID /*out*/ *itemID)
{
	long id = *(long*)&hdl;
	*itemID = id;
}

void FSItemGetItemProperties (FSItemRef hdl, FSItemProperties /*out*/ *prop)
{
	long id = *(long*)&hdl;
	prop->isDir = FSItemIsDir (hdl);
	if (prop->isDir) {
		// ...
	} else {
		prop->dataForkPhysLen = 512;
		prop->dataForkLogLen = 10;
		prop->rsrcForkPhysLen = 0;
		prop->rsrcForkLogLen = 0;
	}
	prop->crDate = 0;
	prop->mdDate = 0;
	prop->isReadOnly = false;
	prop->isHidden = false;
}


OSErr FSItemFileOpen (FSItemRef hdl, Boolean rsrcFork, short fRefNum, FCBRecPtr fcb, FSOpenFileRecord /*out*/ *ref)
{
	return noErr;
}

/*
static OSErr oneBlockRead (unsigned long filePos, Ptr *bufPtr, short fileRefNum)
{
	VCBPtr vcb = CurrVCB;
	Exiting ();
	// it might be the last block of the file - we cannot use UTCacheReadIP to read behind the logical EOF, so we use GetBlock instead:
	OSErr err = UTGetBlock (fileRefNum, FSLg2PhysProc, filePos / 512, gbReleaseMask, bufPtr);
	Entering (vcb);
	LogFStr (LKUser1, nil, "UTGetBlock err %d pos %lx", err, filePos);
	if (err > 0) err = ioErr;
	return err;
}
*/

OSErr FSItemFileRead (FSOpenFileRecPtr ref, UInt32 fileOfs, UInt32 numBytes, Ptr destAddr, UInt32 *actCount)
{
	if (fileOfs == 0 && numBytes <= 10) {
		memcpy (destAddr, "abcdefghij", numBytes);
		*actCount = numBytes;
		return noErr;
	} else {
		return readErr;
	}
/*
		Ptr  destPtr = pb->ioBuffer;
		long firstBlkOfs = startPos % 512;
		long filePos = startPos - firstBlkOfs;
		if (firstBlkOfs > 0 && reqCount > 0) {
			// Anfang lesen (Teilbereich bis hchstens zum Ende des Blocks)
			Ptr ioBuf;
			err = oneBlockRead (filePos, &ioBuf, pb->ioRefNum);
			if (!err) {
				assert (ioBuf);
				filePos += 512;
				long count = 512 - firstBlkOfs;
				if (count > reqCount) count = reqCount;
				BlockMoveData (&ioBuf[firstBlkOfs], destPtr, count);
				reqCount -= count;
				destPtr += count;
				pb->ioActCount += count;
			}
		}
		if (!err && reqCount >= 512) {
			// Ganze Blocks lesen
			long rem = RoundDown (reqCount, 512);
			do {
				unsigned long actCount;
				VCBPtr vcb = CurrVCB;
				Exiting ();
				err = UTCacheReadIP (FSLg2PhysProc, filePos, destPtr, pb->ioRefNum, rem, &actCount, pb->ioPosMode);
				Entering (vcb);
				LogFStr (LKUser1, nil, "UTCacheReadIP err %d pos %lx req %lx act %lx", err, filePos, rem, actCount);
				if (err) {
					if (err > 0) err = ioErr;
					break;
				}
				filePos += actCount;
				reqCount -= actCount;
				destPtr += actCount;
				pb->ioActCount += actCount;
				rem -= actCount;
			} while (rem > 0);

		}
		if (!err && reqCount > 0) {
			// Rest lesen (Teilbereich immer ab Beginn des Blocks)
			Ptr ioBuf;
			err = oneBlockRead (filePos, &ioBuf, pb->ioRefNum);
			if (!err) {
				BlockMoveData (ioBuf, destPtr, reqCount);
				pb->ioActCount += reqCount;
			}
		}
*/
}

void FSItemFileClose (FSOpenFileRecPtr ref)
{
}

Boolean FSItemGetItemName31 (FSItemRef hdl, Str31 itemName)
// returns false if name is > 31 chars and if the FS has no good way to shorten it
{
	long dirID = *(long*)&hdl;
	if (dirID == fsRtDirID) {
		// root dir: fill in the volume name
		pstrcpy (itemName, gCurrVCB->vcbVN);
	} else {
		pstrcpy (itemName, "\psome name");
	}
	return true;
}

void FSItemGetItemNameFull (FSItemRef hdl, Str255 /*out*/ itemName)
{
	FSItemGetItemName31 (hdl, itemName);
}


/*
 * Volume mounting
 */

UInt32 FSPrivateDataSize ()
// must be a constant value!
{
	return 0;
}

OSErr FSFinishMount (VCBPtr vcb)
{
	long volSpaceSize = 32;
	
	vcb->vcbCrDate = 1;
	vcb->vcbVolBkUp = vcb->vcbLsMod = 0;

	long aBlks = 1L + (volSpaceSize/65536L);	// allocation blocks per sector
	vcb->vcbAlBlkSiz = aBlks * 2048L;
	vcb->vcbNmAlBlks = (volSpaceSize + aBlks-1) / aBlks;
	vcb->vcbClpSiz = vcb->vcbAlBlkSiz;

	pstrcpy (vcb->vcbVN, "\pVolname");

	// attention: setting the following two fields to zero means that the number of files and folders
	// on this volume is unknown - however, this will cause a zero-divide crash in Iomega "FindIt"!
	// So, if you want to support that tool despite that bug in it, you must not set these values to
	// zero!
	vcb->vcbFilCnt = 0;
	vcb->vcbDirCnt = 0;

	/*+++
	// set number of files in root dir
	long n, fn;
	ISOGetNoOfDirRecs (fsRtDirID, n);
	ISOGetNoOfDirFiles (fsRtDirID, fn);
	vcb->vcbNmFls = fn;
	vcb->vcbNmRtDirs = n-fn;
	*/
	
	return noErr;
}

void FSUnmountVol ()
{
}

void FSFeedPrivateDataPtr (Ptr dataPtr)
{
}

// EOF
