/*
 * Log Mgr.h
 *
 * Provides a call interface to the Logging Application(s) through a Gestalt reference
 * and exports functions for retrieving the Log Information by the Log Viewer.
 */

#pragma once

#include <Types.h>

#pragma push

#pragma mark /*-- config consts --*/

#define	kMaxLogData			256

#define	gestaltLogMgr		'Log$'


#pragma mark /*-- public types --*/

typedef	void* LogEntryRef;

#pragma enumsalwaysint on

	typedef enum {
		LKGlobal = 0,
		LKUser1,
		LKUser2,
		LKUser3,
		LKUser4,
		LKUser5,
		LKUser6,
		LKUser7,
		LKUser8,
		LKUser9,
		LKUser10,
		LKUser11,
		LKUser12,
		LKUser13,
		LKUser14,
		LKUser15,
		LKUser16,
		LKUser17,
		LKUser18,
		LKUser19,
		LKUser20,
		LKUser21,
		LKUser22,
		LKUser23,
		LKUser24,
		LKBinary,
		LKAssert,
		LKMark,
		LKNote,
		LKWarning,
		LKError,
		LKFatal,
		LKTrigger		// this one is neither mapped to a switch nor used for logging
	} LogKinds;	// 32+1 total

	typedef enum {
		LDTText,
		LDTBin,
		LDTNum
	} LogDataTypes;

#pragma enumsalwaysint reset


typedef unsigned long LKSwitches;	// bit flags (LKGlobal..LKFatal); if LKGlobal is clr, log is suspended

typedef struct {
	const	LKSwitches	switches;		// read-only!
	char				privateData[];
} LogAppDesc;

typedef LogAppDesc *LogAppRef;

#pragma options align=mac68k

	typedef struct {		// one Log Buffer's entry
		long		index;
		long		time;
		LogKinds	kind;	// entries of type LKGlobal are never suppressed
		char		where[38];
		char*		appName;
		LogAppRef	appRef;
		long		used;				// [0..kMaxLogData]
		Byte		data[kMaxLogData];	// format: see 'lLSAddToEntry'
	} LogEntry;

#pragma options align=reset


#pragma mark /*-- defs private to "Log.cp" & "Log Mgr.cp" --*/

typedef LogAppRef (*LogInitProc) (const char* appIdentifier);
typedef void (*LogExitProc) (LogAppRef who);
typedef LogEntryRef (*LogNewEntryProc) (LogAppRef who, LogKinds type, const char* where);
typedef void (*LogAddToEntryProc) (LogEntryRef, LogDataTypes type, const void* addr, long length);
typedef void (*LogFinishEntryProc) (LogEntryRef);
typedef void (*LogChangeSwitchProc) (LogAppRef who, LogKinds type, Boolean on);

#pragma options align=mac68k

	typedef struct {
		long				version;		// = 2
		LogInitProc			doLogInit;
		LogExitProc			doLogExit;
		LogNewEntryProc		doLogNewEntry;
		LogAddToEntryProc	doLogAddToEntry;
		LogFinishEntryProc	doLogFinishEntry;
		LogChangeSwitchProc	doLogChangeSwitch;
		long				reserved[8];
	} LogCIDesc;

	typedef	LogCIDesc*	LogCIPtr;

#pragma options align=reset


#pragma mark /*-- functions exported to the Log Viewer --*/

extern	long	LogBufEntries;	// read only!

void LogAppSetRefCon (LogAppRef who, short idx, long refCon);
long LogAppGetRefCon (LogAppRef who, short idx);

void LogAppSetUserName (LogAppRef who, short idx, StringPtr name);
const StringPtr LogAppUserNamePtr (LogAppRef who, short idx);

Boolean LogAppEnabled (LogAppRef who, LogKinds lk);

void LogAppEnable (LogAppRef who, LogKinds lk, Boolean on);

const char* LogAppGetTitlePtr (LogAppRef who);

Boolean LogGetRecord (LogEntry *entry);
// Retrieves an entry from the log buffer. Returns false if no records available

Boolean LogMgrInit (long bufferEntries);
// Must be called at init time (installs the Gestalt value)
// bufferEntries: size of log buffer. Will be allocated from the heap.
// if bufferEntries=0 then all available heap memory will be used for the log buffer.

void LogMgrExit (void);
// Should be called at de-init time (disables the Gestalt value)


#pragma mark /*-- functions imported from the Log Viewer --*/

void LogSignalNewRecord ();

void LogAddApp (LogAppRef ref, Boolean existed);

void LogAppUpdateSwitch (LogAppRef who, LogKinds lk);

LogAppRef LogMgrPreloadApp (const char* appName);

#pragma pop

// EOF
