
#include <TextUtils.h>
#include <Gestalt.h>
#include <stdarg.h>
#include <stdio.h>
#include "PascalLib.h"
#include "Log Mgr.h"
#include "Logging.h"


#pragma mark /*-- globals (A4) setup --*/

#ifdef LOG_FAR_GLOBALS
	#pragma far_data on
	#pragma far_strings on
	#pragma far_vtables on
	#ifdef LOG_A4_GLOBALS
		-- Error: nur einer der beiden darf definiert sein.
	#endif
#else
	#ifdef LOG_A4_GLOBALS
		#include <A4Stuff.h>
		#define enter_log()		long oldA4=SetCurrentA4()
		#define exit_log()		SetA4(oldA4)
	#else
		#define enter_log()		(0)
		#define exit_log()		(0)
	#endif
#endif


#pragma mark /*-- private global vars --*/

#if	LOGGING

	static	LogCIPtr	*gLogCI;
	static	LogAppRef	gLogHdl = nil;
	static	Boolean		gLogEnabled = false;

#endif


#pragma mark /*-- low level functions --*/

#if	LOGGING && !defined(__powerc)

	static asm foo ()
	{
	l:	bra	l
		// falls hier ein Compiler-Fehler auftritt, liegt es daran, da das Modul
		// als PowerPC- und nicht als 68K-Code bersetzt wird. Dies wrde jedoch
		// nicht laufen, denn die folgenden Calls springen direkt in 68K-Code
		// hinein. Es mten erst CallUniversalProc-Aufrufe eingesetzt werden.
	}

	static inline LogEntryRef LogNewEntry (LogKinds kind, const char* where)
	{
		return (*gLogCI)->doLogNewEntry (gLogHdl, kind, where);
	}

	static inline void LogAddStrToEntry (LogEntryRef ref, const char* text)
	{
		(*gLogCI)->doLogAddToEntry (ref, LDTText, text, -1);
	}

	static inline void LogAddStrToEntry (LogEntryRef ref, const char* text, long len)
	{
		(*gLogCI)->doLogAddToEntry (ref, LDTText, text, len);
	}

	static inline void LogAddLongToEntry (LogEntryRef ref, long value)
	{
		(*gLogCI)->doLogAddToEntry (ref, LDTNum, &value, 4);
	}

	static inline void LogAddBytesToEntry (LogEntryRef ref, const void* data, long length)
	{
		(*gLogCI)->doLogAddToEntry (ref, LDTBin, data, length);
	}

	static inline void LogFinishEntry (LogEntryRef ref)
	{
		(*gLogCI)->doLogFinishEntry (ref);
	}

#endif	// #if LOGGING


#pragma mark /*-- high level functions --*/


#if LOGGING

	void LogBytes (LogKinds kind, const char *where, const void *addr, long length)
	{
		#if	!defined(__powerc)
			enter_log ();
			if (gLogEnabled && *gLogCI) {
				LogEntryRef ref = LogNewEntry (kind, where);
				if (ref) {
					LogAddBytesToEntry (ref, addr, length);
					LogFinishEntry (ref);
				}
			}
			exit_log ();
		#else
			#pragma unused (kind, where, addr, length)
			#ifdef __powerc
				if (kind == LKError || kind == LKFatal) {
					DebugStr("\pLogBytes(Error/Fatal)");
				}
			#endif
		#endif
	}

	void LogStr (LogKinds kind, const char *where, const char *text)
	{
		#if	!defined(__powerc)
			enter_log ();
			if (gLogEnabled && *gLogCI) {
				LogEntryRef ref = LogNewEntry (kind, where);
				if (ref) {
					LogAddStrToEntry (ref, text);
					LogFinishEntry (ref);
				}
			}
			exit_log ();
		#else
			#pragma unused (kind, where, text)
			#ifdef __powerc
				if (kind == LKError || kind == LKFatal) {
					DebugStr("\pLogStr(Error/Fatal)");
				}
			#endif
		#endif
	}

	void LogStr (LogKinds kind, const char *where, const char *text, long code)
	{
		#if	!defined(__powerc)
			enter_log ();
			if (gLogEnabled && *gLogCI) {
				LogEntryRef ref = LogNewEntry (kind, where);
				if (ref) {
					LogAddStrToEntry (ref, text);
					LogAddLongToEntry (ref, code);
					LogFinishEntry (ref);
				}
			}
			exit_log ();
		#else
			#pragma unused (kind, where, text, code)
			#ifdef __powerc
				if (kind == LKError || kind == LKFatal) {
					DebugStr("\pLogStr(Error/Fatal)");
				}
			#endif
		#endif
	}

	void LogNumber (LogKinds kind, const char *where, long code)
	{
		#if	!defined(__powerc)
			enter_log ();
			if (gLogEnabled && *gLogCI) {
				LogEntryRef ref = LogNewEntry (kind, where);
				if (ref) {
					LogAddLongToEntry (ref, code);
					LogFinishEntry (ref);
				}
			}
			exit_log ();
		#else
			#pragma unused (kind, where, code)
			#ifdef __powerc
				if (kind == LKError || kind == LKFatal) {
					DebugStr("\pLogNumber(Error/Fatal)");
				}
			#endif
		#endif
	}

	void LogFStr (LogKinds kind, const char *where, const char *text, ...)
	// Slower than "LogStr".
	// "text": uses sprintf format, max size is 256 chars.
	{
		#if	!defined(__powerc)
			enter_log ();
			if (gLogEnabled && *gLogCI) {
				LogEntryRef ref = LogNewEntry (kind, where);
				if (ref) {
					char s[256];
					va_list	ap;
					va_start (ap, text);
					vsprintf (s, text, ap);
					va_end (ap);
					LogAddStrToEntry (ref, s);
					LogFinishEntry (ref);
				}
			}
			exit_log ();
		#else
			#pragma unused (kind, where, text)
			#ifdef __powerc
				if (kind == LKError || kind == LKFatal) {
					DebugStr("\pLogFStr(Error/Fatal)");
				}
			#endif
	#endif
	}

	static void PPlugParams (StringPtr message, ConstStr255Param p0, ConstStr255Param p1, ConstStr255Param p2, ConstStr255Param p3)
	{
		unsigned char		*in, *out, *inEnd, *outEnd;
		const unsigned char	*param;
		long				len;
		unsigned char		buf[256];
		
		in = message + 1;
		out = buf + 1;
		inEnd = message + 1 + *message;
		outEnd = buf + 256;
		while (in < inEnd ) {
			if (*in == '^') {
				in++;
				if (in >= inEnd) break;
				switch (*in++) {
					case '0':
						param = p0;
						break;
					case '1':
						param = p1;
						break;
					case '2':
						param = p2;
						break;
					case '3':
						param = p3;
						break;
					default:
						continue;
				}
				if (!param) continue;
				len = *param;
				if (out + len > outEnd) {
					len = outEnd - out;
				}
				BlockMoveData (param + 1, out, len);
				out += len;
			} else {
				if (out >= outEnd) break;
				*out++ = *in++;
			}
		}
		*buf = (UInt8) (out - (buf + 1));
		BlockMoveData (buf, message, *buf + 1);
	}

	void LogPStr (LogKinds kind, const char *where, ConstStr255Param s, long l0, long l1, long l2, long l3)
	// Actually a "LogParamStr" - logs a Pascal string with up to 4 optional numbers fit into "^0", "^1" etc.
	{
		#if	!defined(__powerc)
			enter_log ();
			if (gLogEnabled && *gLogCI) {
				LogEntryRef ref = LogNewEntry (kind, where);
				if (ref) {
					Str255 m;
					{
						Str15 s0, s1, s2, s3;
						pstrcpy (m, (StringPtr)s);
						NumToString (l0, s0);
						NumToString (l1, s1);
						NumToString (l2, s2);
						NumToString (l3, s3);
						PPlugParams (m, s0, s1, s2, s3);	// from "PStrings.c"
					}
					LogAddStrToEntry (ref, (char *) &m[1], (unsigned char)m[0]);
					LogFinishEntry (ref);
				}
			}
			exit_log ();
		#else
			#pragma unused (kind, where, s, l0, l1, l2, l3)
			#ifdef __powerc
				if (kind == LKError || kind == LKFatal) {
					DebugStr("\pLogPStr(Error/Fatal)");
				}
			#endif
		#endif
	}

#endif

#if LOGGING

	static inline Boolean BitTst (unsigned long l, int b)
	{
		return (Boolean)((l >> b) & 1);
	}

	Boolean LKEnabled (LogKinds lk)
	{
		#if	LOGGING
			enter_log ();
			Boolean	b = (Boolean) (gLogCI && *gLogCI && BitTst (gLogHdl->switches, lk));
			exit_log ();
			return b;
		#else
			#pragma unused (lk)
			return false;
		#endif
	}

	Boolean LKEnabled (short lk)
	{
		return LKEnabled ((LogKinds)lk);
	}

	void LKEnable (LogKinds lk, Boolean on)
	{
		#if	LOGGING
			enter_log ();
			if (gLogEnabled && *gLogCI) {
				(*gLogCI)->doLogChangeSwitch (gLogHdl, lk, on);
			}
			exit_log ();
		#else
			#pragma unused (lk, on)
		#endif
	}

	void LKEnable (short lk, Boolean on)
	{
		LKEnable ((LogKinds)lk, on);
	}

	Boolean LogEnable (Boolean on)
	{
		#if	LOGGING
			enter_log ();
			Boolean	b = gLogEnabled;
			gLogEnabled = (gLogCI != nil) && on;
			exit_log ();
			return b;
		#else
			#pragma unused (on)
			return false;
		#endif
	}

	Boolean LogDisable (void)
	{
		return LogEnable (false);
	}

	Boolean LogEnabled ()
	{
		#if	LOGGING
			enter_log ();
			Boolean	b = gLogEnabled;
			exit_log ();
			return b;
		#else
			return false;
		#endif
	}

	Boolean LogInit (const char* appIdentifier)
	// must allow memory move, no call from intr!
	{
		#if	LOGGING
			enter_log ();
			OSErr err = Gestalt (gestaltLogMgr, (long*)&gLogCI);
			if (err || (*gLogCI == nil)) {
				// error: Log Server not installed
				gLogCI = nil;
			} else if ((*gLogCI)->version != 2) {
				// error: wrong version of Log Server
				gLogCI = nil;
			} else {
				gLogHdl = (*gLogCI)->doLogInit (appIdentifier);
				if (!gLogHdl) {
					gLogCI = nil;
				} else {
					// success
					// !!! hier Systemprofil ermitteln
					// datum/Zeit ermitteln und eintragen
					// ticks nullen
				}
			}
			LogEnable ();
			exit_log ();
			return gLogEnabled;
		#else
			#pragma unused (appIdentifier)
			return false;
		#endif
	}

	void LogExit (void)
	{
		#if	LOGGING
			enter_log ();
			// datum/Zeit ermitteln und eintragen
			if (gLogCI && *gLogCI) {
				(*gLogCI)->doLogExit (gLogHdl);
			}
			LogDisable ();
			gLogHdl = nil;
			exit_log ();
		#endif
	}

#endif	// #if LOGGING

// EOF
