
#pragma once

#include "Log Mgr.h"

#pragma mark /*-- defines --*/

#ifndef LOGGING
	// #pragma error -- you need to provide a definition for "LOGGING" as either "0" or "1"
	#define LOGGING 0
#endif

#pragma mark /*-- macros --*/

#define _STR(x)	_VAL(x)
#define _VAL(x)	#x
#define HERE (__FILE__ ":" _STR(__LINE__))

#if LOGGING
	#ifdef __powerc
		/* let's invoke the PPC Remote Debugger in serious cases */
		#define Assert(c)		((c) ? 1 : (DebugStr("\pAssert() failed"), 0))
		#define AssertNum(c,i)	((c) ? 1 : (DebugStr("\pAssertNum() failed"), 0))
		#ifndef assert
		#define assert(c)		((c) ? (void)0 : DebugStr("\passert() failed"))
		#endif
		#define Log(kind,msg)	((void)0)
		#define LogNum(k,m,i)	((void)0)
		#define Note(msg)		((void)0)
		#define NoteNum(m, i)	((void)0)
		#define Mark(msg)		((void)0)
		#define MarkNum(m, i)	((void)0)
		#define Warning(msg)	((void)0)
		#define WarnNum(m, i)	((void)0)
		#define Error(msg)		(Debugger())
		#define ErrorNum(m, i)	(Debugger())
		#define Fatal(msg)		(Debugger())
		#define FatalNum(m, i)	(Debugger())
		#define IOError(msg, i)	(Debugger())
		#define LogData(data)	((void)0)
		#define LKOn(kind)		(0)
	#else
		#define Assert(cond)	((cond) ? 1 : (LogStr (LKAssert, HERE, #cond), 0))
		#define AssertNum(c,i)	((c   ) ? 1 : (LogStr (LKAssert, HERE, #c, (long)(unsigned long)(i)), 0))
		#ifndef assert
		#define assert(cond)	((cond) ? (void)0 : LogStr (LKAssert, HERE, #cond))	// "#define assert(x) Assert(x)" doesn't work properly (#cond won't be expanded correctly)
		#endif
		#define Log(kind,msg)	(LogStr ((LogKinds)kind, HERE, msg))
		#define LogNum(k,m,i)	(LogStr ((LogKinds)k, HERE, m, (long)(unsigned long)(i)))
		#define Note(msg)		(LogStr (LKNote, HERE, msg))
		#define NoteNum(msg, i)	(LogStr (LKNote, HERE, msg, (long)(unsigned long)(i)))
		#define Mark(msg)		(LogStr (LKMark, HERE, msg))
		#define MarkNum(msg, i)	(LogStr (LKMark, HERE, msg, (long)(unsigned long)(i)))
		#define Warning(msg)	(LogStr (LKWarning, HERE, msg))
		#define WarnNum(msg, i)	(LogStr (LKWarning, HERE, msg, (long)(unsigned long)(i)))
		#define Error(msg)		(LogStr (LKError, HERE, msg))
		#define ErrorNum(m, i)	(LogStr (LKError, HERE, m, (long)(unsigned long)(i)))
		#define Fatal(msg)		(LogStr (LKFatal, HERE, msg))
		#define FatalNum(m, i)	(LogStr (LKFatal, HERE, m, (long)(unsigned long)(i)))
		#define IOError(msg, i)	(LogStr (LKError, HERE, msg, (long)(unsigned long)(i)))
		#define LogData(data)	(LogBytes (LKBinary, HERE, &data, sizeof(data)))
		#define LKOn(kind)		(LKEnabled ((LogKinds)kind))
	#endif
#else
	#define Assert(cond)	(cond)			/* always generates code */
	#define AssertNum(c,i)	(c)
	#ifndef assert
	#define assert(cond)	((void)0)		/* Attention: generates no code! */
	#endif
	#define Log(kind,msg)	((void)0)
	#define LogNum(k,m,i)	((void)0)
	#define Note(msg)		((void)0)
	#define NoteNum(m, i)	((void)0)
	#define Mark(msg)		((void)0)
	#define MarkNum(m, i)	((void)0)
	#define Warning(msg)	((void)0)
	#define WarnNum(m, i)	((void)0)
	#define Error(msg)		((void)0)
	#define ErrorNum(m, i)	((void)0)
	#define Fatal(msg)		((void)0)
	#define FatalNum(m, i)	((void)0)
	#define IOError(msg, i)	((void)0)
	#define LogData(data)	((void)0)
	#define LKOn(kind)		(0)
#endif


#pragma mark /*-- high level functions --*/

#if LOGGING
	void LogStr (LogKinds kind, const char *where, const char *msg);
	void LogStr (LogKinds kind, const char *where, const char *msg, long num);
	void LogNumber (LogKinds kind, const char *where, long num);
	void LogBytes (LogKinds kind, const char *where, const void *addr, long length);
	void LogFStr (LogKinds kind, const char *where, const char *msg, ...);
	void LogPStr (LogKinds kind, const char *where, ConstStr255Param s, long l0, long l1 = 0, long l2 = 0, long l3 = 0);
#else
	#define LogStr (void)
	#define LogFStr (void)
	#define LogPStr (void)
	#define LogNumber (void)
	#define LogBytes (void)
#endif

#if LOGGING
	Boolean LKEnabled (LogKinds lk);
	Boolean LKEnabled (short lk);
	void LKEnable (LogKinds lk, Boolean on);
	void LKEnable (short lk, Boolean on);

	Boolean LogEnabled ();
	Boolean LogEnable (Boolean on = 1);
	Boolean LogDisable (void);

	Boolean LogInit (const char* appIdentifier);	// must allow memory move, no call from intr!
	void LogExit (void);
#else
	#define LKEnabled(lk) (0)
	#define LKEnable(lk,on) ((void)0)
	#define LogEnabled() (0)
	#define LogEnable(void)
	#define LogDisable() ((void)0)
	#define LogInit(appIdentifier) ((void)0)
	#define LogExit() ((void)0)
#endif

// EOF
