//
// Lg2Phys routine.
// Last update by TT, 30 Oct 99: Now using Apple's latest sample from August '99
//

#include <Files.h>
#include <LowMem.h>
#include <FSM.h>

#include "TrapAvailable.h"
#include "Lg2Phys.h"


EXTERN_API( Ptr ) LMGetFCBSPtr(void) TWOWORDINLINE(0x2EB8, 0x034E);

FCBRec *RefNumToFCB (short refNum)
{
	if (TrapAvailable(_HFSUtilDispatch)) {
		FCBRecPtr f;
		if (UTResolveFCB (refNum, &f) == noErr) {
			return f;
		}
		return nil;
	} else {
		return (FCBRec *) (LMGetFCBSPtr() + refNum);
	}
}



// ******** this part by Jim Luther (Apple) *********

#include <Devices.h>
#include <Files.h>
#include <FSM.h>
#include <Traps.h>
#include <Errors.h>
#include <Gestalt.h>
#include <HFSVolumes.h>
#include <MacMemory.h>
#include <Math64.h>

#if (UNIVERSAL_INTERFACES_VERSION <= 0x0320)

	//	These are defined in Universal Interfaces v3.3

	enum {
		// FCBRec.fcbFlags
		fcbLargeFileBit				= 3,							/* File may grow beyond 2GB; cache uses file blocks, not bytes */
		fcbLargeFileMask			= 0x08,
		
		// gestaltFSAttr
		gestaltHasHFSPlusAPIs		= 12							/* must test fcbLargeFileBit if this is set */
	};


	// vMExtendedAttributes (GetVolParms) bit position constants
	enum {
		bIsEjectable				= 0,							/* volume is in an ejectable disk drive */
		bSupportsHFSPlusAPIs		= 1,							/* volume supports HFS Plus APIs directly (not through compatibility layer) */
		bSupportsFSCatalogSearch	= 2,							/* volume supports FSCatalogSearch */
		bSupportsFSExchangeObjects	= 3,							/* volume supports FSExchangeObjects */
		bSupports2TBFiles			= 4,							/* volume supports supports 2 terabyte files */
		bSupportsLongNames			= 5,							/* volume supports file/directory/volume names longer than 31 characters */
		bSupportsMultiScriptNames	= 6,							/* volume supports file/directory/volume names with characters from multiple script systems */
		bSupportsNamedForks			= 7,							/* volume supports forks beyond the data and resource forks */
		bSupportsSubtreeIterators	= 8,							/* volume supports recursive iterators not at the volume root */
		bL2PCanMapFileBlocks		= 9
	};

	/*
	  	GetForkSize
	  	Get the current logical size (end-of-file) of an open fork.
	  	->	ioCompletion	A pointer to a completion routine
	  	<-	ioResult		The result code of the function
	  	->	forkRefNum		The reference number of the fork
	  	<-	positionOffset	The logical size of the fork, in bytes
	*/
	EXTERN_API( OSErr )
	FSGetForkSize					(SInt16 				forkRefNum,
									 SInt64 *				forkSize)							THREEWORDINLINE(0x303C, 0x032D, 0xAA52);

#endif	// (UNIVERSAL_INTERFACES_VERSION <= 0x0320)

// version 3 GetVolParmsInfoBuffer
struct GetVolParmsInfoBufferV3 {
	short 							vMVersion;					/*version number*/
	long 							vMAttrib;					/*bit vector of attributes (see vMAttrib constants)*/
	Handle 							vMLocalHand;				/*handle to private data*/
	long 							vMServerAdr;				/*AppleTalk server address or zero*/
																/*		vMVersion 1 GetVolParmsInfoBuffer ends here */
	long 							vMVolumeGrade;				/*approx. speed rating or zero if unrated*/
	short 							vMForeignPrivID;			/*foreign privilege model supported or zero if none*/
																/*		vMVersion 2 GetVolParmsInfoBuffer ends here */
	long 							vMExtendedAttributes;		/*extended attribute bits (see vMExtendedAttributes constants)*/
																/*		vMVersion 3 GetVolParmsInfoBuffer ends here */
};
typedef struct GetVolParmsInfoBufferV3	GetVolParmsInfoBufferV3;

/*****************************************************************************/

//	Types and constants needed by code below
//

typedef HFSPlusExtentDescriptor	*ExtentsArray;
typedef ExtentsArray			*ExtentsArrayHandle;

enum
{
	kBitsInUInt32		= 32,
	
	kLog2BlockSize		= 9,										// each disk block is...
	kBlockSize			= (1 << kLog2BlockSize),					// 512 bytes
	
	// constants for mapping logical file blocks to physical disk blocks
	kMaximumBlocksIn2G	= ((0x80000000-1) >> kLog2BlockSize),		// max disk blocks in 2G
	kMaximumBytesIn2G	= (kMaximumBlocksIn2G << kLog2BlockSize)	// max bytes in 2G
};

/*****************************************************************************/

//
//	Function prototypes
//

asm OSErr Lg2Phys(SInt16 fileRefNum,		// -> file reference number
				  UInt32 fileBlockNumber,	// -> file block (512-byte blocks) to map to disk block
				  SInt32 requestBytes,		// -> size, in bytes, to map (multiple of 512-bytes only)
				  UInt32 *diskBlockNumber,	// <- disk block (512-byte blocks) where extent begins
				  SInt32 *contiguousBytes);	// <- size, in bytes, of contiguous disk space (always multiple of 512-bytes)

// unused	void SetWidePosOffset(unsigned long blockOffset, XIOParamPtr pb);

// unused	Boolean VolumeSupportsLg2Phys(short vRefNum);

OSErr GetExtents(short fileRefNum,						// file refNum of open file
				 SInt64 fileLEOF,						// file's logical EOF
				 UInt32 *extentsCount,					// number of HFSPlusExtentDescriptor in extentsHandle
				 ExtentsArrayHandle *extentsArrayHndl);	// locked handle containing array of HFSPlusExtentDescriptor records

/*****************************************************************************/

//	Lg2Phys
//
//	Maps a file's logical blocks to a disk's physical blocks.
//
//	Important Note #1:	This only works on volumes owned by the local file system (HFS/HFS Plus),
//						or on file systems that indicate they support Lg2Phys in the same manner
//						as the local file system, so you must verify that the volume's
//						file system ID is zero or the file system that owns the volume
//						supports Lg2Phys correctly before using this routine. See the routine
//						VolumeSupportsLg2Phys for the correct method of testing for Lg2Phys
//						support
//
//	Important Note #2:	The reqCount input parameter MUST be evenly divisible by 512.
//						The contiguousBytes output parameter will be evenly divisible by 512.
//						Both parameters are signed long values. That's because the API for
//						Read and Write uses a signed long for reqCount. Why is this important?
//						Because if you call Lg2Phys on a file larger than 2G that's contiguous
//						on disk, Lg2Phys will map it in multiple passes with the largest
//						contiguousBytes value returned being 0x7fffe00 (2G - 512-bytes).
//
//	Important Note 3:	If you used a previous version of this code, please note that the file
//						offset is now in blocks instead of bytes. That allows this routine to
//						map files up to 2T.
//
//	Important Note #4:	This System Programming Interface (SPI) is not guaranteed to work in the
//						future.
//

//	Structure for accessing parameters passed on the stack to Lg2Phys function.
struct Lg2PhysStack
{
	long	returnAddress;
	long	registers[6];
	SInt16	fileRefNum;
	UInt32	fileBlockNumber;
	SInt32	requestBytes;
	UInt32	diskBlockNumber;
	SInt32	contiguousBytes;
};

//	Some system private low memory locations.
//	(see Important Note #4 above - it applies to most of these low memory locations, too).
enum
{
	FCBSPtr		= 0x034E,
	FSBusy		= 0x0360,
	HFSStkTop	= 0x036A,
	FSCallAsync	= 0x0342,
	JLg2Phys	= 0x06FC
};

asm OSErr Lg2Phys(SInt16 fileRefNum, UInt32 fileBlockNumber, SInt32 requestBytes, UInt32 *diskBlockNumber, SInt32 *contiguousBytes)
{
			// The first time this routine is called, call
			// Gestalt to see what the File Manager has to offer.
			move.l	hasTestedFSAttr,d0							// have we checked the File Manager gestalt yet?
			bne.s	HaveGestalt									// -> yes, go to work
			
			move.l	#gestaltFSAttr,d0							// get the File Manager attributes
			_Gestalt
			bne		Exit										// -> exit with error
			
			move.l	a0,d0										// get the result
			btst	#gestaltHasHFSPlusAPIs,d0					// test for HFSPlus APIs
			lea		hasHFSPlusAPIs,a0
			sne		(a0)										// set the hasHFSPlusAPIs flag
			
			btst	#gestaltHasFileSystemManager,d0				// test for FSM
			lea		hasFSM,a0
			sne		(a0)										// set the FSM flag
			
			lea		hasTestedFSAttr,a0
			st		(a0)										// and set the flag so we don't call Gestalt again
			
HaveGestalt:
			movem.l	d3-d6/a2/a6,-(sp)							// save registers
			
			move.w	struct(Lg2PhysStack.fileRefNum)(sp),d1		// d1 = fileRefNum
			move.l	struct(Lg2PhysStack.requestBytes)(sp),d4	// d4 = reqCount
			move.l	struct(Lg2PhysStack.fileBlockNumber)(sp),d5	// d5 = fileBlockNumber
			
GrabFSLoop:
			bset	#0,FSBusy									// grab control of the File Manager
			bne.s	GrabFSLoop
			
			move.l	HFSStkTop,a6								// use the File Manager's stack (an a6 stack)
			
			lea		hasFSM,a0									// can we use FSM?
			tst.l	(a0)
			bne.s	UseFSM										// -> yes
			
			// No FSM, so we have to access the old FCB array directly
			// and we know that we don't have to test the fcbLargeFileBit because
			// pre-FSM systems never had large files.
			movea.l	FCBSPtr,a1									// a1 = FCBSPtr
			lea		(a1,d1.w),a1								// a1 = FCBRecPtr
			bra.s	IsSmallFile									// -> no FSM, so there's no HFSPlus APIs to worry about
			
UseFSM:
			// Use FSM to get the FCBRecPtr safely
			move.l	d1,d3										// save refNum in d3
			suba.l	#4,sp										// grab 4 bytes of stack for FCBRecPtr
			movea.l	sp,a2										// a2 = pointer to FCBRecPtr
			subq	#2,sp										// space for result
			move.w	d1,-(sp)									// pass fileRefNum
			move.l	a2,-(sp)									// pass ptr to FCBRecPtr
			moveq	#5,d0										// _UTResolveFCB is selector #5
			_HFSUtilDispatch
			move.w	(sp)+,d0									// d0 = result
			movea.l	(a2),a1										// a1 = FCBRecPtr
			adda.l	#4,sp										// give back space
			bne.s	ErrorExit									// -> exit now if error
			
			move.l	d3,d1										// restore refNum
			
			lea		hasHFSPlusAPIs,a0							// can we test the fcbLargeFileBit safely?
			tst.l	(a0)
			beq.s	IsSmallFile									// -> no, so the file cannot be a large file
			
			btst	#fcbLargeFileBit,struct(FCBRec.fcbFlags)(a1) // is this a large file?
			beq.s	IsSmallFile
//			_Debugger
			bne.s	IsLargeFile									// -> yes, so Lg2Phys is expecting a fileBlockNumber

IsSmallFile:
			moveq	#kLog2BlockSize,d0							// no, so convert fileBlockNumber to bytes
			lsl.l	d0,d5										// for small file
			
IsLargeFile:
			movea.l	struct(FCBRec.fcbVPtr)(a1),a2				// a2 = VCB pointer
			movea.l	FCBSPtr,a1									// a1 = FCBSPtr for compatibility with all versions of Lg2Phys
			
			clr.b	FSCallAsync									// tell the File Manager it's doing a synchronous operation
			
			movea.l	JLg2Phys,a0									// let the FS do the mapping
			jsr		(a0)
			
			bclr	#0,FSBusy									// all done with the File Manager
			
			movea.l	struct(Lg2PhysStack.diskBlockNumber)(sp),a0
			move.l	d3,(a0)										// return diskBlockNumber from d3
			movea.l	struct(Lg2PhysStack.contiguousBytes)(sp),a0
			move.l	d6,(a0)										// return contiguousBytes from d6

ErrorExit:
			movem.l	(sp)+,d3-d6/a2/a6							// restore registers
			
			ext.l	d0											// set up condition codes
			ble.s	Exit										// -> if zero or negative, we're OK
			
			moveq	#ioErr,d0									// positive errors are internal errors - map it to ioErr
Exit:
			rts													// and return

hasTestedFSAttr:
			ds.l	1	// true if File Manager's Gestalt has been called
hasHFSPlusAPIs:
			ds.l	1	// true if File Manager supports HFS Plus APIs (large file APIs)
hasFSM:
			ds.l	1	// true if File System Manager utility calls are available
}

/*****************************************************************************/

/* unused:
//	SetWidePosOffset
//
//	Set up ioPosMode and either ioPosOffset or ioWPosOffset for a
//	device Read or Write.
//
void SetWidePosOffset(unsigned long blockOffset, XIOParamPtr pb)
{
	// convert block number to wide byte offset
    pb->ioWPosOffset.lo = blockOffset << kLog2BlockSize;
    pb->ioWPosOffset.hi = blockOffset >> (kBitsInUInt32 - kLog2BlockSize);
	
    if ( pb->ioWPosOffset.hi ) {
        // Offset on disk is >= 4G, so use wide positioning mode
        pb->ioPosMode = fsFromStart | (1 << kWidePosOffsetBit);
    } else {
        // Offset on disk is < 4G, so use regular positioning mode,
        // and move the offset into ioPosOffset
        pb->ioPosMode = fsFromStart;
        ((IOParam *)pb)->ioPosOffset = pb->ioWPosOffset.lo;
    }
}
*/

/*****************************************************************************/

/* unused:
//	VolumeSupportsLg2Phys
//
//	Returns true if this volume's file system supports Lg2Phys correctly.
//	Note that some file systems may support Lg2Phys correctly but may not
//	support PBHGetVolParms version 3. There's no way to know what file systems
//	that might be true for, so this code takes the safe way out.
//
Boolean VolumeSupportsLg2Phys(short vRefNum)
{
	Boolean					result;
	OSErr					error;
	HParamBlockRec			pb;
	GetVolParmsInfoBufferV3	volParmsInfo;
	
	result = false;	// default response
	
	// try GetVolParms first
	pb.ioParam.ioNamePtr = NULL;
	pb.ioParam.ioVRefNum = vRefNum;
	pb.ioParam.ioBuffer = (Ptr)&volParmsInfo;
	pb.ioParam.ioReqCount = sizeof(GetVolParmsInfoBufferV3);
	error = PBHGetVolParmsSync(&pb);
	if ( (error == noErr) && (volParmsInfo.vMVersion >= 3) ) {
		// vMVersion 3 (and later) has the vMExtendedAttributes field, so use it
		result = ((volParmsInfo.vMExtendedAttributes & (1L << bL2PCanMapFileBlocks)) != 0);
	} else if ( (error == noErr) || (error == paramErr) )	// paramErr is OK, it just means this volume doesn't support GetVolParms {
		// ioNamePtr and ioVRefNum are still set up from GetVolParms call
		pb.volumeParam.ioVolIndex = 0;	// use ioVRefNum only
		// check for an FSID of 0 -- HFS and HFS Plus always support Lg2Phys correctly
		result = ((PBHGetVInfoSync(&pb) == noErr) && (pb.volumeParam.ioVFSID == 0));
	}
	
	return result;
}
*/

/*****************************************************************************/

//	GetExtents
//
//	Creates an array of HFSPlusExtentDescriptor records for an open file.
//
OSErr GetExtents(short fileRefNum,						// file refNum of open file
				 SInt64 fileLEOF,						// file's logical EOF
				 UInt32 *extentsCount,					// number of HFSPlusExtentDescriptor in extentsHandle
				 ExtentsArrayHandle *extentsArrayHndl)	// locked handle containing array of HFSPlusExtentDescriptor records
{
	OSErr					result;
	UInt32					fileBlockNumber;
	UInt32					blocksToMap;
	SInt32					requestBytes;
	UInt32					diskBlockNumber;
	SInt32					contiguousBytes;
	UInt32					contiguousBlocks;
	HFSPlusExtentDescriptor	currentExtent;
	ExtentsArrayHandle		extents;
	
	extents = (ExtentsArrayHandle)NewHandle(0);
	if ( extents != NULL ) {
		result = noErr;			// no errors yet
		*extentsCount = 0;		// initialize extents counter
		fileBlockNumber = 0;	// start at beginning of file

		// get number of disk blocks to map
		blocksToMap = (SInt64ToWide(fileLEOF).hi << (kBitsInUInt32 - kLog2BlockSize)) + (SInt64ToWide(fileLEOF).lo >> kLog2BlockSize);
		if ( (SInt64ToWide(fileLEOF).lo & (kBlockSize - 1)) != 0 ) {
			// fileLEOF wasn't multiple of kBlockSize, so add another
			++blocksToMap;
		}
		
		while ( (result == noErr) && (blocksToMap != 0) ) {
			// make sure requestBytes is a positive SInt32 since LG2Phys can only map 2G - 512-bytes at a time
			if ( blocksToMap <= kMaximumBlocksIn2G ) {
				requestBytes = (blocksToMap << kLog2BlockSize);
			} else {
				// requestBytes = largest multiple of 512-bytes less than max SInt32
				requestBytes = kMaximumBytesIn2G;
			}
			
			result = Lg2Phys(fileRefNum, fileBlockNumber, requestBytes, &diskBlockNumber, &contiguousBytes);
			if ( result == noErr ) {
				// get contiguous blocks
				contiguousBlocks = contiguousBytes >> kLog2BlockSize;
				
				// is this extent contiguous with last extent?
				if ( (extentsCount != 0) &&
					 ((currentExtent.startBlock + currentExtent.blockCount) == diskBlockNumber) )
				{
					// this extent was contiguous with the last extent, so add its blocks
					// to the last extent's blockCount
					(*extents)[*extentsCount - 1].blockCount += contiguousBlocks;
				} else {
					// this extent is not contiguous with the last extent,
					// so add a new extent to our extent list
					currentExtent.startBlock = diskBlockNumber;
					currentExtent.blockCount = contiguousBlocks;
					
					// one more extent in array
					++(*extentsCount);
					
					PtrAndHand(&currentExtent, (Handle)extents, sizeof(HFSPlusExtentDescriptor));
					result = MemError();
				}
				
				// update for next time through
				blocksToMap -= contiguousBlocks;
				fileBlockNumber += contiguousBlocks;
			}
		}
		
		if ( result == noErr ) {
			// lock array handle before returning
			HLock((Handle)extents);
		} else {
			// get rid of the handle and set it to NULL
			DisposeHandle((Handle)extents);
			extents = NULL;
		}
	} else {
		// could not allocate handle
		result = MemError();
	}
	
	// return the ExtentsArrayHandle (NULL if error)
	*extentsArrayHndl = extents;
	
	return result;
}



// ******** remaining part by Markus Fest & TT ***********


OSErr	GetFileExtents(short refNum, tFileExtent **h, long *numExtents)
{
	OSErr	err;
	SInt64	fileLen;
	long	response;

	FCBRec*	fcb = RefNumToFCB(refNum);
	if (!fcb) return rfNumErr;

	if ((Gestalt(gestaltFSAttr, &response) == noErr) && ((response & (1 << gestaltHasHFSPlusAPIs)) != 0) && (fcb->fcbFlags & fcbLargeFileMask) != 0) {
		err = FSGetForkSize (refNum, &fileLen);
	} else {
		fileLen = fcb->fcbEOF;
	}
	
	ExtentsArrayHandle extHdl;
	err = GetExtents (refNum, fileLen, (unsigned long *) numExtents, &extHdl);
	if (!err) {
		SetHandleSize((Handle) h, *numExtents * (long) sizeof(tFileExtent));
		err = MemError();
		if (!err) {
			tFileExtent *dst = *h;
			HFSPlusExtentDescriptor *src = *extHdl;
			for (int i = 0; i < *numExtents; ++i) {
				dst->extentVolBlock = src->startBlock;
				dst->extentBytes = ((long long)src->blockCount) << 9;
				dst++;
				src++;
			}
		} else {
			*numExtents = 0;
		}
		DisposeHandle ((Handle)extHdl);
	}
	
	return err;
}


// EOF
