/*
	
	FS API 2.cp
	
	by Florian Dejako
*/

#pragma once

#include <FSM.h>
#include <string.h>
#include <Notification.h>
#include <DriverServices.h>
#include "PascalLib.h"
#include "LaunchLib.h"
#include "ResForkInfo.h"
#include "FS API.h"

static tRFINodePtr sRFIRootInfo = NULL;

void FSAtomicStart (VCBPtr, Ptr /*privateDataPtr*/)
// attn: the pointers may be NULL!
{
}

void FSAtomicStop ()
{
	// NOP
}

/*
 * Item (file/dir) lookup
 */

static bool ItemIsValid(tRFINodePtr node)
{
	if (node == NULL)
		return false;

	switch (node->nodeType)
	{
		case kRFIDirectoryType:
		case kRFIResourceType:
			return true;
			
		default:
			return false;
	}
}

static tRFINodePtr RefToNode(FSItemRef ref)
{
	return ItemIsValid(tRFINodePtr(ref)) ? tRFINodePtr(ref) : NULL;
}

OSErr FSItemLookup (ItemID itemID, FSItemRef *hdl)
{
	if (itemID == fsRtDirID) {
		itemID = ItemID(sRFIRootInfo);
	}
	*hdl = (FSItemRef)(itemID);
	return ItemIsValid(tRFINodePtr(itemID)) ? noErr : fidNotFound;
}

OSErr FSItemLookup (ItemID parID, UInt32 index, Boolean filesOnly, FSItemRef *hdl)
{
	Boolean isRoot = false;
	
	if (parID == fsRtDirID) {
		isRoot = true;
		parID = ItemID(sRFIRootInfo);
	}
	
	if (not ItemIsValid(tRFINodePtr(parID)))
		return dirNFErr;

	tRFINodePtr dir = tRFINodePtr(RefToNode(FSItemRef(parID)));
		
	if(index==3 && isRoot) Debugger();
	
	int i = 0;
	int numItems = dir->numItems;

	dir++;

	while (i < index)
	{
		dir = tRFINodePtr(Ptr(dir) + dir->nodeSize);
		numItems--;
			
		if (not filesOnly or (dir->nodeType != kRFIDirectoryType))
		{
			i++;
		}
		
		if (numItems == 0)
		{
			return fnfErr;
		}
	}

	*hdl = (FSItemRef)dir;

	return noErr;
}

OSErr FSItemCountItems (FSItemRef hdl, Boolean filesOnly, UInt32 *items)
{
	tRFINodePtr dir = tRFINodePtr(RefToNode(hdl));
	
	if (dir == NULL)
		return dirNFErr;
	
	if (!filesOnly)
	{
		*items = dir->numItems;
	}
	else
	{
		*items = 0;

		dir++;
		
		for (int i = 0; i < dir->numItems; i++)
		{
			if (dir->nodeType != kRFIResourceType)
			{
				*items++;
			}

			dir = tRFINodePtr(Ptr(dir) + dir->nodeSize);
		}
	}

	return noErr;
}

OSErr FSItemLookup (ItemID parID, StringPtr name, FSItemRef *hdl)
{
	if (parID == fsRtDirID)
		parID = ItemID(sRFIRootInfo);

	if (not ItemIsValid(tRFINodePtr(parID)))
		return dirNFErr;

	tRFINodePtr dir = tRFINodePtr(parID);

	dir++;
	
	for (int i = 0; i < dir->numItems; i++)
	{
		if (!pstrcmp(name, dir->nodeName))
		{
			*hdl = (FSItemRef)dir;
			return noErr;
		}

		dir = tRFINodePtr(Ptr(dir) + dir->nodeSize);
	}
	
	return fnfErr;
}

OSErr FSItemGetParent (FSItemRef hdl, FSItemRef *parHdl)
{
	if (hdl == FSItemRef(sRFIRootInfo))
		return fidNotFound;

	if (not ItemIsValid(tRFINodePtr(hdl)))
		return fnfErr;

	tRFINodePtr node = tRFINodePtr(hdl);
	
	if (node->parent != NULL)
	{
		*parHdl = FSItemRef(node->parent);
		return noErr;
	}
	else
	{
		return dirNFErr;
	}
}

/*
 * Item access
 */

Boolean FSItemIsDir (FSItemRef hdl)
{
	if (not ItemIsValid(tRFINodePtr(hdl)))
		return fnfErr;

	tRFINodePtr node = tRFINodePtr(hdl);
	
	return (node->nodeType == kRFIDirectoryType);
}

void FSItemGetItemID (FSItemRef hdl, ItemID /*out*/ *itemID)
{
	if (hdl == FSItemRef(sRFIRootInfo))
		*itemID = fsRtDirID;
	else
		*itemID = ItemID(hdl);
}

Boolean FSItemGetItemName31 (FSItemRef hdl, Str31 /*out*/ itemName)
{
	// returns false if name is > 31 chars and if the FS has no good way to shorten it
	
	tRFINodePtr node = tRFINodePtr(hdl);
	
	if (*node->nodeName <= 31)
	{
		pstrcpy(itemName, node->nodeName);
		return true;
	}
	else
	{
		pstrncpy(itemName, node->nodeName, 32);
		return false;
	}
}

void FSItemGetItemNameFull (FSItemRef hdl, Str255 /*out*/ itemName)
{
	tRFINodePtr node = tRFINodePtr(hdl);
	pstrcpy(itemName, node->nodeName);
}

void FSItemGetItemProperties (FSItemRef hdl, FSItemProperties /*out*/ *prop)
{
	tRFINodePtr node = tRFINodePtr(RefToNode(hdl));

	prop->isDir = FSItemIsDir(hdl);

	if (not prop->isDir)
	{
		prop->dataForkLogLen = node->resSize;
		prop->dataForkPhysLen = (node->resSize + 511) & ~512;
		prop->rsrcForkLogLen = prop->rsrcForkPhysLen = 0;
	}

	prop->crDate = sRFIRootInfo->crDate;
	prop->mdDate = sRFIRootInfo->mdDate;
	prop->isReadOnly = true;
	prop->isHidden = false;
	prop->reserved = 0;
}

OSErr FSItemFileOpen (FSItemRef hdl, Boolean rsrcFork, FSOpenFileRecord /*out*/ *ref)
{
	if (not ItemIsValid(tRFINodePtr(hdl)))
		return fnfErr;

	if (rsrcFork)
		return errFSForkNotFound;

	ref->x[0] = (long)hdl;

	return noErr;
}

OSErr FSItemFileRead (FSOpenFileRecPtr ref, UInt32 fileOfs, UInt32 numBytes, Ptr destAddr, UInt32 *actBytes)
{
	//+++
	return -1;
}

void FSItemFileClose (FSOpenFileRecPtr /*ref*/)
{
	// NOP, we don't close anything
}

/*
 * Volume mounting
 */

UInt32 FSPrivateDataSize ()
{
	// must be a constant value!
	return sizeof(tRFINodePtr);
}

OSErr FSFinishMount (VCBPtr vcb)
{
	OSErr err = noErr;
	
	long volSpaceSize = 32;
	long aBlks = 1L + (volSpaceSize/65536L);	// allocation blocks per sector
	vcb->vcbAlBlkSiz = aBlks * 2048L;
	vcb->vcbNmAlBlks = (volSpaceSize + aBlks-1) / aBlks;
	vcb->vcbClpSiz = vcb->vcbAlBlkSiz;

	if (sRFIRootInfo) {
		// get the volume the file is on
		FCBRecPtr fcb;
		err = UTResolveFCB (sRFIRootInfo->rfRefNum, &fcb);
		if (!err) {
			sRFIRootInfo->reserved = fcb->fcbVPtr->vcbVRefNum;
		}
		pstrcpy (vcb->vcbVN, fcb->fcbCName);
	} else {
		pstrcpy (vcb->vcbVN, "\p?");
	}
	return err;
}

// TaskTime stuff:

typedef void (*TaskTimeProc) (UInt32 param);

struct TaskTimeInfo {
	NMRec			nmRec;
	TaskTimeProc	proc;
	UInt32			param;
};
static TaskTimeInfo	cTaskTimeInfo;


// NMResponse is called by the Notification Manager at task time. All we have to do is
// remove the notification record, reset the busy flag, and call the user's proc.
// CAUTION: Doing it that way allows clients to post another request before the first one
// has completed.

static bool cIsBusy = false;

static pascal void NMResponse(NMRec *nmRec)
{
	TaskTimeProc	proc = cTaskTimeInfo.proc;
	UInt32			param = cTaskTimeInfo.param;

	::NMRemove(nmRec);
	cIsBusy = false;

	(*proc) (param);
}


static OSStatus CallBackAtTaskTime(TaskTimeProc proc, UInt32 param)
{
	if (cIsBusy)
		return paramErr; 

	cIsBusy = true;
	
	::memset(&cTaskTimeInfo, 0, sizeof(TaskTimeInfo));

	cTaskTimeInfo.proc = proc;
	cTaskTimeInfo.param = param;
	
	cTaskTimeInfo.nmRec.qType = nmType;
	cTaskTimeInfo.nmRec.nmResp = NewNMProc (NMResponse);

	OSStatus status = ::NMInstall(&cTaskTimeInfo.nmRec);

	if (status != noErr)
		cIsBusy = false;
		
	return status;
}

static Boolean AnyVolsMountedByFS (short fsID)
// checks if any volumes are mounted by the given File System
{
	HVolumeParam pb;
	short err, idx = 1;
	Str63 name;
	do {
		pb.ioVolIndex = idx++;
		pb.ioNamePtr = name;
		err = PBHGetVInfoSync ((HParamBlockRec*)&pb);
		if (err) break;
		if (pb.ioVFSID == fsID) {
			return true;
		}
	} while (true);
	return false;
}

static OSErr SendQuitEvent (ProcessSerialNumber *targetPSN)
{
	OSErr retCode;
	AppleEvent theAppleEvent = {typeNull, nil}, theReplyEvent = {typeNull, nil};
	AEDesc targetAddrDesc = {typeNull, nil};
	
	// to simplify cleanup, ensure that handles are nil to start
	theAppleEvent.dataHandle = nil;
	//docDescList.dataHandle = nil;
	
	// create an address descriptor based on the serial number of
	// the target process

	retCode = AECreateDesc(typeProcessSerialNumber, (Ptr) targetPSN,
		sizeof(ProcessSerialNumber), &targetAddrDesc);
	if (retCode != noErr) goto Bail;
	
	// now make the AppleEvent descriptor and insert the 
	// document descriptor list as the direct object
	
	retCode = AECreateAppleEvent(kCoreEventClass, kAEQuitApplication,
		&targetAddrDesc, kAutoGenerateReturnID, kAnyTransactionID,
		&theAppleEvent);
	if (retCode != noErr) goto Bail;
	
	// finally, send the Apple event
	retCode = AESend (&theAppleEvent, &theReplyEvent, kAENoReply,
		kAEHighPriority, 2*60 /* timeout: 2 seconds */, nil, nil);
	
Bail:
	// dispose of everything that was allocated
	
	if (theAppleEvent.dataHandle != nil)  (void) AEDisposeDesc(&theAppleEvent);
	
	return retCode;
}


static void MyUnmountVolProc(UInt32 param)
{
	// This routine gets called back at task time by the notification manager.
	// We use it to close the file and to dispose of the ResForkInfo pointer.
	tRFINodePtr rfi = tRFINodePtr(param);
	
	::FSClose(rfi->rfRefNum);
	::DisposePtr(Ptr(rfi));
	
	if (!AnyVolsMountedByFS(kFSID))
	{
		ProcessSerialNumber psn;
		FSSpec lame;
		if (noErr == FindRunningAppBySignature('APPL', 'MtRf', &psn, &lame))
		{
			DebugStr("\pSendQuitEvent");
			SendQuitEvent(&psn);
		}
	}
}

void FSUnmountVol ()
{
	if (sRFIRootInfo) {
		CallBackAtTaskTime(MyUnmountVolProc, UInt32(sRFIRootInfo));
	}
}


/*
 * Messaging
 */

void FSFeedPrivateDataPtr (Ptr dataPtr)
{
	sRFIRootInfo = (tRFINodePtr)dataPtr;
}

// EOF

