/*
 * FS API.h
 */

#pragma once

typedef UInt32 ItemID;

typedef void* FSItemRef;	// some opaque handle

typedef struct {
	UInt32	dataForkPhysLen;
	UInt32	dataForkLogLen;
	UInt32	rsrcForkPhysLen;
	UInt32	rsrcForkLogLen;
	UInt32	crDate;
	UInt32	mdDate;
	UInt8	isDir : 1;
	UInt8	isReadOnly : 1;
	UInt8	isHidden : 1;
	UInt8	reserved : 5;
	// ...
} FSItemProperties;

typedef struct {
	long	x[6]; // can't be more than 6 longs!
} FSOpenFileRecord, *FSOpenFileRecPtr;


void FSAtomicStart (VCBPtr currVCB, Ptr privateDataPtr);	// attn: currVCB may be NULL!
void FSAtomicStop ();

/*
 * Item (file/dir) lookup
 */

OSErr FSItemLookup (ItemID itemID, FSItemRef *hdl);
OSErr FSItemLookup (ItemID parID, UInt32 index, Boolean filesOnly, FSItemRef *hdl);
OSErr FSItemLookup (ItemID parID, StringPtr name, FSItemRef *hdl);

OSErr FSItemGetParent (FSItemRef hdl, FSItemRef *parHdl);

/*
 * Item access
 */

Boolean FSItemIsDir (FSItemRef hdl);
OSErr FSItemCountItems (FSItemRef hdl, Boolean filesOnly, UInt32 *items);
void FSItemGetItemID (FSItemRef hdl, ItemID /*out*/ *itemID);
Boolean FSItemGetItemName31 (FSItemRef hdl, Str31 /*out*/ itemName);	// returns false if name is > 31 chars and if the FS has no good way to shorten it
void FSItemGetItemNameFull (FSItemRef hdl, Str255 /*out*/ itemName);
void FSItemGetItemProperties (FSItemRef hdl, FSItemProperties /*out*/ *prop);

OSErr FSItemFileOpen (FSItemRef hdl, Boolean rsrcFork, FSOpenFileRecord /*out*/ *ref);
OSErr FSItemFileRead (FSOpenFileRecPtr ref, UInt32 fileOfs, UInt32 numBytes, Ptr destAddr);
void FSItemFileClose (FSOpenFileRecPtr ref);

/*
 * Volume mounting
 */

UInt32 FSPrivateDataSize ();	// must be a constant value!
OSErr FSFinishMount (VCBPtr vcb);
void FSUnmountVol ();

// EOF
