/*

	CResForkMounter.cp
	
*/
#include "CResForkMounter.h"
#include "CResForkScanner.h"
#include "DCon.h"
#include "UFSInstall.h"

CResForkMounter::CResForkMounter(FSSpec &spec)
	: 	mFileSpec(spec)
{
}

const OSType kMountResForkFSSig = 'MtRf';	// you may put your signature here (to help the FS to identify you)


void CResForkMounter::Mount()
{
	CResForkScanner scanner(mFileSpec);
	
	long dataSize = scanner.GetDataSize();
	
	// allocate more memory than we need to avoid crashing! :-D 
	// -- It's MacHack, after all.
	
	tRFINodePtr dataPtr = (tRFINodePtr)::NewPtrSys(dataSize * 2);
	ThrowIfMemFail_(dataPtr);
	
	::BlockZero(dataPtr, dataSize * 2);

	long *sigMem = (long*)(Ptr(dataPtr) + dataSize);
	*sigMem = 'FDj?';

	scanner.GetData(dataPtr);

	ThrowIfOSErr_(::FSpOpenRF(&mFileSpec, fsRdPerm, &dataPtr->rfRefNum));
	
	// We now have all the data in the dataPtr
	
	DumpResForkInfo(dataPtr);
	
/*	FSMEntryProc entryProc = UFSInstall::LoadFS("\pFSM Code");
	
	if (entryProc)
	{
		UFSInstall::InstallFS(entryProc, kMountResForkFSSig);
	}
	else
	{
		dprintf("Couldn't load FSM Code resource...");
	}
We don't install here because TT does it manually beforehand... */
	
	OSErr err;
	
	dprintf("Announcing mount...");
	err = UFSInstall::FSAnnounceMount(64, kMountResForkFSSig);
	dprintf("%d\n", err);
	dprintf("Specifying data ptr...");
	err = UFSInstall::FSSpecifyResForkInfo(Ptr(dataPtr), kMountResForkFSSig);
	dprintf("%d\n", err);

	dprintf("Trying to mount...");
	VolumeParam pb;
	pb.ioVRefNum = 64; // hardcoded here for the sake of hackness.
	::PBMountVol ((ParmBlkPtr)&pb);
	dprintf("ioResult = %d\n", pb.ioResult);

	// At this point, we simply leave the file open and don't quit.
	// The file system will close the file and dispose of the memory we allocated.
}

	static void DumpBlanks(short numBlanks)
	{
		for (int blanks = 0; blanks < numBlanks; blanks++)
			dprintf(" ");
	}

	static void DumpResourceNode(tRFINodePtr info, short rfRefNum, short level)
	{
		DumpBlanks(level * 3);

	#pragma unused(rfRefNum)

		dprintf("Resource %#s @ %d\n", info->nodeName, info->rfOffset);
	}

	static void DumpDirectory(tRFINodePtr info, short rfRefNum, short level)
	{
		DumpBlanks(level * 3);
	
		dprintf("Directory %#s (nodeSize %d, %d subnodes)\n", 
					info->nodeName, 
					info->nodeSize, 
					info->numItems);

		tRFINodePtr node = info + 1;

		for (int i = 0; i < info->numItems; i++)
		{
			switch (node->nodeType)
			{
				case kRFIResourceType:
					DumpResourceNode(node, rfRefNum, level + 1);
					break;
					
				case kRFIDirectoryType:
					DumpDirectory(node, rfRefNum, level + 1); // recursive
					break;
					
				default:
					dprintf("Unknown Node Type %x!\n", node->nodeType);
			}
			
			node = tRFINodePtr(Ptr(node) + node->nodeSize);
		}
	}

void CResForkMounter::DumpResForkInfo(tRFINodePtr info)
{
	dprintf("Dumping ResForkInfo at %x for %d\n\n", info, info->rfRefNum);

	DumpDirectory(info, info->rfRefNum, 0);
}
