/*

	CResForkScanner.cp
	
*/
#include "CResForkScanner.h"
#include <UResourceMgr.h>
#include "PascalLib.h"

#if PRAGMA_STRUCT_ALIGN
	#pragma options align=mac68k
#elif PRAGMA_STRUCT_PACKPUSH
	#pragma pack(push, 2)
#elif PRAGMA_STRUCT_PACK
	#pragma pack(2)
#endif
	// See http://developer.apple.com/techpubs/mac/MoreToolbox/MoreToolbox-99.html
	// for a definition of this:
struct tResMapEntry {
	ResIDT		resID;
	short		nameOffset;
	long		attrsAndDataOffset;
	//	char	attr
	//	unsigned offset : 24
	Handle		resHandle;
};
typedef tResMapEntry * tResMapEntryPtr;
#if PRAGMA_STRUCT_ALIGN
	#pragma options align=reset
#elif PRAGMA_STRUCT_PACKPUSH
	#pragma pack(pop)
#elif PRAGMA_STRUCT_PACK
	#pragma pack()
#endif

CResForkScanner::CResForkScanner(FSSpec &spec)
	: 	mSpec(spec),
		mResDataStart(0)
{
}

UInt32 CResForkScanner::GetDataSize()
{
	return Scan(NULL);
}

void CResForkScanner::GetData(tRFINodePtr outData)
{
	Scan(outData);
}

static void GetFileDates(FSSpec &spec, long *outCrDate, long *outMdDate)
{
	CInfoPBRec pb;

	pb.hFileInfo.ioCompletion = NULL;
	pb.hFileInfo.ioNamePtr = spec.name;
	pb.hFileInfo.ioVRefNum = spec.vRefNum;
	pb.hFileInfo.ioFDirIndex = 0;
	pb.hFileInfo.ioDirID = spec.parID;

	ThrowIfOSErr_(::PBGetCatInfoSync(&pb));
	ThrowIfOSErr_(pb.hFileInfo.ioResult);

	*outCrDate = pb.hFileInfo.ioFlCrDat;
	*outMdDate = pb.hFileInfo.ioFlMdDat;
}

void CResForkScanner::GetRFIResourceData(tRFINodePtr curResPtr, Handle resH, tRFINodePtr parent)
{
	curResPtr->nodeType = kRFIResourceType;
	curResPtr->nodeSize = sizeof(tRFINode);
	curResPtr->parent = parent;

	long rsrcMapEntryOffset = ::RsrcMapEntry(resH);
	Handle mapHandle = ::LMGetTopMapHndl();
	tResMapEntryPtr mapEntry = tResMapEntryPtr((*mapHandle) + rsrcMapEntryOffset);

	// Offset from beginning of resource data:
	long resDataOffset = mapEntry->attrsAndDataOffset & 0x00ffffff;
	
	curResPtr->rfOffset = mResDataStart + resDataOffset + 4; 
									// 4 = length? anyway, it's in the resource fork

	Str255 resName;
	
	::GetResInfo(resH, &curResPtr->resID, &curResPtr->resType, resName);
	ThrowIfResError_();
	
	::NumToString(curResPtr->resID, curResPtr->nodeName);
	
	if (*resName)
	{
		pstrcat(curResPtr->nodeName, "\p, ");
		pstrcat(curResPtr->nodeName, resName);
	}
	
	curResPtr->resAttrs = ::GetResAttrs(resH);
	ThrowIfResError_();
	
	curResPtr->resSize = ::GetResourceSizeOnDisk(resH);
	ThrowIfResError_();
	
	switch (curResPtr->resType)
	{
		case 'PICT':
			curResPtr->fileType = 'PICT';
			curResPtr->fileCreator = 'ttxt';
			
			// for PICTs, add a PICT file header offset so that the file
			// can be opened as a PICT file:
			break;

		case 'STR#':
		case 'STR ':
		case 'TEXT':
			curResPtr->fileType = 'TEXT';
			curResPtr->fileCreator = 'ttxt';
			break;
			
		default:
			curResPtr->fileType = '????';
			curResPtr->fileCreator = '????';
	}
	
	if (curResPtr->resType == 'STR ')
	{
		curResPtr->resSize--;
		curResPtr->rfOffset++;
	}
}

UInt32 CResForkScanner::Scan(tRFINodePtr outData)
{
	UInt32 size = sizeof(tRFINode);

	if (outData)
	{
		outData->nodeType = kRFIDirectoryType;
		outData->parent = NULL; // root has no parent
		pstrcpy(outData->nodeName, mSpec.name);

		GetFileDates(mSpec, &outData->crDate, &outData->mdDate);

		// The resource fork starts with a resource fork header which starts
		// with a long containing the offset of the resource data in the file. We need this.

		short rfRefNum;
		ThrowIfOSErr_(::FSpOpenRF(&mSpec, fsRdPerm, &rfRefNum));

		long count = 4;
		ThrowIfOSErr_(::FSRead(rfRefNum, &count, &mResDataStart));
		Assert_(count == 4);
		
		ThrowIfOSErr_(::FSClose(rfRefNum));
	}

	tRFINodePtr curDirPtr = outData ? (tRFINodePtr)outData + 1 : NULL;

	LFile file(mSpec);
	StResourceContext context(file.OpenResourceFork(fsRdPerm));
	
	short numTypes = ::Count1Types();
	ThrowIfResError_();
	
	if (outData)
	{
		outData->numItems = numTypes;
	}
	
	for (int i = 1; i <= numTypes; i++)
	{
		OSType resType;
		::Get1IndType(&resType, i);
		ThrowIfResError_();

		UInt32 dirSize = sizeof(tRFINode);

		short numResources = ::Count1Resources(resType);
		ThrowIfResError_();

		if (curDirPtr)
		{
			curDirPtr->nodeType = kRFIDirectoryType;
			curDirPtr->parent = outData;
			curDirPtr->numItems = numResources;
			
			// copy name, lame:
			char *c = (char *)&resType;
			curDirPtr->nodeName[0] = 4;
			curDirPtr->nodeName[1] = c[0];
			curDirPtr->nodeName[2] = c[1];
			curDirPtr->nodeName[3] = c[2];
			curDirPtr->nodeName[4] = c[3];
		}

		if (resType == 'STR#')
		{
			tRFINodePtr strDir = curDirPtr ? curDirPtr + 1 : NULL;

			// special-case STR# resources:
			for (int j = 1; j <= numResources; j++)
			{
				StResource strH(::Get1IndResource(resType, j));
				if (strH.IsValid())
				{
					dirSize += sizeof(tRFINode);
					short numStrings = *((short *) *strH.Get());

					long strDataSize = sizeof(tRFINode) + numStrings * sizeof(tRFINode);
					
					if (outData)
					{
						strDir->nodeType = kRFIDirectoryType;
						strDir->numItems = numStrings;
						strDir->nodeSize = strDataSize;
						strDir->parent = curDirPtr;

						tRFINode strInfo;
						GetRFIResourceData(&strInfo, strH, strDir);

						::NumToString(strInfo.resID, strDir->nodeName);
						
						tRFINodePtr strOutPtr = strDir + 1;

						StHandleLocker lockStr(strH);
						StringPtr s = StringPtr(*strH.Get());
						
						long strRFOffset = strInfo.rfOffset;
						s += sizeof(short); // skip the numItems
						strRFOffset += sizeof(short);

						for (int k = 1; k <= strDir->numItems; k++)
						{
							*strOutPtr = strInfo; // copy info we already received before

							UInt8 strDataLen = s[0] + 1;
							// copy the name
							strOutPtr->resSize = strDataLen - 1;
							pstrncpy(strOutPtr->nodeName, s, strDataLen);
							strOutPtr->rfOffset = strRFOffset + 1;

							strOutPtr++;

							strRFOffset += strDataLen;
							s += strDataLen;
						}
					}
					
					dirSize += strDataSize;
					
					if (strDir)
					{
						strDir = tRFINodePtr(Ptr(strDir) + strDataSize);
					}
				}
			}
		}
		else
		{
			// all other resources

			dirSize += numResources * sizeof(tRFINode);
			
			if (curDirPtr)
			{
				// only scan if output data is requested:

				tRFINodePtr curResPtr = curDirPtr + 1;
			
				for (int j = 1; j <= numResources; j++)
				{
					StResLoad resLoadFalse;
					StResource resH(::Get1IndResource(resType, j));

					GetRFIResourceData(curResPtr, resH, curDirPtr);

					curResPtr++;
				}
			}
		}
		
		if (curDirPtr)
		{
			curDirPtr->nodeSize = dirSize;
		}
		
		size += dirSize;
		
		if (curDirPtr)
		{	
			curDirPtr->nodeSize = dirSize;

			curDirPtr = tRFINodePtr(Ptr(curDirPtr) + dirSize);
		}
	}
	
	if (outData)
	{
		outData->nodeSize = size;
	}

	// caller will provide refNum

	return size;
}
