/*

	UFSInstall.cp
	
*/
#include "UFSInstall.h"
#include <FSM.h>
#include "DCon.h"

OSErr UFSInstall::FSUninstall (OSType signature)
{
	tSampleFSMessage msg;
	msg.msgSize = 16;
	msg.version = 1;
	msg.senderID = signature;
	msg.senderVer = 1;
	msg.message = kSampleFSMsgUninstall;
	OSErr err = InformFFS (kSampleFSID, &msg);
	return err;
}

OSErr UFSInstall::FSAnnounceMount (short driveNum, OSType signature)
{
	tSampleFSMessage msg;
	msg.msgSize = 18;
	msg.version = 1;
	msg.senderID = signature;
	msg.senderVer = 1;
	msg.message = kSampleFSMsgSetDrive;
	msg.driveNum = driveNum;
	OSErr err = InformFFS (kSampleFSID, &msg);
	return err;
}

OSErr UFSInstall::FSSpecifyResForkInfo (Ptr rfiPtr, OSType signature)
{
	tSampleFSMessage msg;
	msg.msgSize = 20;
	msg.version = 1;
	msg.senderID = signature;
	msg.senderVer = 1;
	msg.message = kSampleFSMsgPassFSDataPtr;
	msg.ptr = rfiPtr;
	OSErr err = InformFFS (kSampleFSID, &msg);
	return err;
}

Boolean UFSInstall::VolMountedByFS (UInt16 fsID)
// checks if any volumes are mounted by the given File System
{
	HVolumeParam pb;
	short err, idx = 1;
	Str63 name;
	do {
		pb.ioVolIndex = idx++;
		pb.ioNamePtr = name;
		err = PBHGetVInfoSync ((HParamBlockRec*)&pb);
		if (err) break;
		if (pb.ioVFSID == fsID) {
			return true;
		}
	} while (true);
	return false;
}

Boolean UFSInstall::IsFSInstalled (UInt16 fsID)
{
	FSDRec	fsdRec;
	short	recSize = sizeof (FSDRec);
	OSErr err = GetFSInfo (fsmGetFSInfoByFSID, fsID, &recSize, &fsdRec);
	return (err == noErr);
}

void UFSInstall::FSMClientInstall (FSMEntryProc doFS, short *errCode, Str255 errMsg)
{
	*errCode = doFS (1, (Ptr)errMsg);
}

UInt16 UFSInstall::FSMClientGetFSID (FSMEntryProc doFS)
{
	UInt16 fsID;
	if (doFS (2, (Ptr)&fsID) == noErr) {
		return fsID;
	}
	return 0;	// = invalid
}

FSMEntryProc UFSInstall::LoadFS(ConstStr255Param resName)
{
	//!!! It's imperial that this CODE resource is set as "systemHeap" because Thomas is lame.
	Handle h = Get1NamedResource ('CODE', resName);
	if (!h) {
		dprintf ("FSM Client CODE rsrc not found.");
		return NULL;
	}
	DetachResource (h);
	HLock (h);
	return (FSMEntryProc) *h;
}

static void error (short errCode, StringPtr msg)
{
	dprintf (" \nFatal error (%d) when installing FSM Client: %#s", errCode, msg);
}


void UFSInstall::InstallFS(FSMEntryProc entryProc, OSType signature)
{
	if (kSampleFSID != FSMClientGetFSID(entryProc)) {
		dprintf ("Oops - FSM Client has unexpected FS ID!");
		return;
	}
	
	OSErr err = noErr;
	
	// check if there's already a client installed
	if (IsFSInstalled (kSampleFSID)) {
		dprintf ("Removing old FSM Client ... ");
		err = FSUninstall (signature);
		if (err) {
			dprintf ("Failed! \n");
			return;
		} else {
			dprintf ("OK. \n");
		}
	}
		
	Str255 msg;

	FSMClientInstall (entryProc, &err, msg);
	if (err || msg[0]) {
		error (err, msg);
	} else {
		dprintf ("\nFS is ready.");
	}
}
