/*	==========================
 *	AEAAccessorWildFromList.cc
 *	==========================
 */

#include "AEADebugging.h"

#include "AEADesc.hh"
#include "AEADescList.hh"
#include "AEAAccessorWildFromList.hh"

AEAAccessorWildFromList::AEAAccessorWildFromList()
: AEAAccessor(typeWildCard, typeAEList)
{
}

AEAAccessorWildFromList::~AEAAccessorWildFromList()
{
}

void
AEAAccessorWildFromList::AccessObject(DescType inDesiredClass, 
	const AEDesc *inContainerToken, DescType inContainerClass, 
	DescType inKeyForm, const AEDesc *inKeyData, AEDesc *outToken) const
{
	// Verify that inContainerToken is actually a list descriptor
	if (inContainerToken->descriptorType != typeAEList
		|| inContainerToken->dataHandle == NULL)
		ThrowOSErr_(errAEWrongDataType);
	
	const AEADescList containerList(*inContainerToken);
	AEADescList outList;  // will be disposed on exception
	
	outList.Create();
	
	// Get the number of tokens in the list.
	long count = containerList.CountItems();
	
	// For every token in the container list...
	for (int i = 1; i <= count; i++) {
		AEADesc oneItemToken, newItemToken;
		
		// Get the token.
		containerList.Get(i, oneItemToken.Ref());
		
		// Redispatch the access using the token instead of the list.
		OSErr err = ::AECallObjectAccessor(inDesiredClass, &oneItemToken.Ref(), 
			inContainerClass, inKeyForm, inKeyData, &newItemToken.Ref());
		ThrowIfOSErr_(err);
		
		// The access is complete. Dispose the container item token.
		::AEDisposeDesc(&oneItemToken.Ref());
		
		// Put the accessed token in the output list.
		outList.Put(0, newItemToken.Ref());
		
		// Dispose the accessed token.
		::AEDisposeDesc(&newItemToken.Ref());
	}
	// We're done, so we need the list structure to stick around.
	*outToken = outList.Ref();
	outList.Reset();  // Don't dispose the AEDesc
}
