/* C3DMatrix.cc */

#include <stddef.h>
#include <string.h>
#include "C3DMatrix.hh"

C3DMatrix::C3DMatrix(short inHeight, short inWidth)
: mHeight(inHeight), mWidth(inWidth)
{
	mData = new double [inHeight * inWidth];
	if (!mData) throw;
	
	for (short x = 0; x < inHeight; x++) {
		for (short y = 0; y < inHeight; y++) {
			mData[x * inHeight + y] = 0;
		}
		mData[x * inHeight + x] = 1;
	}
}

C3DMatrix::C3DMatrix(const C3DMatrix &inMatrix)
: mHeight(inMatrix.mHeight), mWidth(inMatrix.mWidth)
{
	mData = new double [mHeight * mWidth];
	if (!mData) throw;
	
	memcpy(mData, inMatrix.mData, sizeof (double) * mHeight * mWidth);
}

C3DMatrix::~C3DMatrix()
{
	delete mData;
}

C3DMatrix &
C3DMatrix::operator=(const C3DMatrix &inMatrix)
{
	if (mHeight * mWidth != inMatrix.mHeight * inMatrix.mWidth) {
		delete mData;
		mData = new double [inMatrix.mHeight * inMatrix.mWidth];
		if (!mData) throw;
	}
	mHeight = inMatrix.mHeight;
	mWidth = inMatrix.mWidth;
	
	memcpy(mData, inMatrix.mData, sizeof (double) * mHeight * mWidth);
	
	return *this;
}

double &
C3DMatrix::Cell(short inRow, short inCol)
{
	return mData[inRow * mHeight + inCol];
}

const double &
C3DMatrix::Cell(short inRow, short inCol) const
{
	return mData[inRow * mHeight + inCol];
}

void
C3DMatrix::Multiply(const C3DMatrix &inFactor, C3DMatrix &outProduct) const
{
	if (mWidth != inFactor.mHeight) throw;
	
	for (short x = 0; x < mHeight; x++) {
		for (short y = 0; y < inFactor.mWidth; y++) {
			outProduct.Cell(x, y) = 0;
			for (short z = 0; z < mWidth; z++) {
				outProduct.Cell(x, y) += Cell(x, z) * inFactor.Cell(z, y);
			}
		}
	}
}

C3DMatrix *
C3DMatrix::Times(const C3DMatrix &inFactor) const
{
	if (mWidth != inFactor.mHeight) throw;
	C3DMatrix *product = new C3DMatrix(mHeight, inFactor.mWidth);
	if (!product) throw;
	
	Multiply(inFactor, *product);
	return product;
}

C3DMatrix &
C3DMatrix::MultiplyBy(const C3DMatrix &inFactor)
{
	if (mWidth != inFactor.mHeight) throw;
	if (inFactor.mWidth != inFactor.mHeight) throw;
	C3DMatrix *product = Times(inFactor);
	delete mData;
	mData = product->mData;
	product->mData = NULL;
	
	return *this;
}

C3DMatrix &
C3DMatrix::ConverselyMultiplyBy(const C3DMatrix &inFactor)
{
	if (mWidth != inFactor.mHeight) throw;
	if (inFactor.mWidth != inFactor.mHeight) throw;
	C3DMatrix *product = inFactor.Times(*this);
	delete mData;
	mData = product->mData;
	product->mData = NULL;
	
	return *this;
}
