/* C3DObjects.cc */

#include <fp.h>

#include "C3DGeometry.hh"
#include "C3DObjects.hh"

void
C3DObject::Translate(double dx, double dy, double dz)
{
	loc.Cell(0, 0) += dx;
	loc.Cell(0, 1) += dy;
	loc.Cell(0, 2) += dz;
}

void
C3DThing::Translate(double dx, double dy, double dz)
{
	C3DTranslation trans(dx, dy, dz);
	xform.MultiplyBy(trans);
	C3DTranslation invtrans(-dx, -dy, -dz);
	inverse.MultiplyBy(invtrans);
}

void
C3DObject::Translate(const C3DVector &inVector)
{
	Translate(inVector.dx, inVector.dy, inVector.dz);
}

void
C3DThing::Yaw(double dtheta)
{
	// Thing's orientation is expressed as yaw-pitch-roll
	// from its context's point of view.
	// Delta theta is from the thing's point of view.
	// If you think of yaw-pitch-roll as a series of transforms
	// that convert from context coordinates to thing coordinates,
	// then we're adding another yaw to the end.
	// So we just multiply onto the existing orientation transform.
	
	C3DYaw yaw(dtheta);
	xform.MultiplyBy(yaw);
	C3DYaw invyaw(-dtheta);
	inverse.MultiplyBy(invyaw);
}

void
C3DThing::Pitch(double dphi)
{
	C3DPitch pitch(dphi);
	xform.MultiplyBy(pitch);
	C3DPitch invpitch(-dphi);
	inverse.MultiplyBy(invpitch);
}

void
C3DThing::Roll(double dpsi)
{
	C3DRoll roll(dpsi);
	xform.MultiplyBy(roll);
	C3DRoll invroll(-dpsi);
	inverse.MultiplyBy(invroll);
}

void
C3DThing::Scale(double factor)
{
	C3DScale scale(factor);
	xform.MultiplyBy(scale);
	C3DScale invscale(1.0/factor);
	inverse.MultiplyBy(invscale);
}

void
C3DCamera::WorldToEye(const C3DPoint &inPt, C3DPoint &outPt)
{
	inPt.Multiply(inverse, outPt);
}

bool
C3DCamera::EyeToPort(const C3DPoint &inPt, C3DPoint &outPt)
{
	double y = inPt.Cell(0, 1);
	if (y <= 0) return false;
	outPt = inPt;
	if (fabs(outPt.Cell(0, 0) *= viewAngle / y) > 1.0) return false;
	if (fabs(outPt.Cell(0, 2) *= viewAngle / y) > 1.0) return false;
	return true;
}
