/* OrganConsole.cc */

#include <ctype.h>

#include <console.h>

#include "OrganConsole.hh"
#include "GenieConsoleSystem.hh"

static GenieConsole *sDefaultConsole = NULL;
static GenieConsole *sCurrentConsole = NULL;

GenieConsole *
GetCurrentConsole()
{
	return sCurrentConsole;
}

void
SetCurrentConsole(GenieConsole *inConsole)
{
	sCurrentConsole = inConsole;
}

/*
 *	The following four functions provide the UI for the console package.
 *	Users wishing to replace SIOUX with their own console package need
 *	only provide the four functions below in a library.
 */

/*
 *	extern short InstallConsole(short fd);
 *
 *	Installs the Console package, this function will be called right
 *	before any read or write to one of the standard streams.
 *
 *	short fd:		The stream which we are reading/writing to/from.
 *	returns short:	0 no error occurred, anything else error.
 */

short InstallConsole(short fd)
{
#pragma unused (fd)
	// Because OrganConsole is used for several different output streams,
	// initialization must be handled per stream.
	// That leaves us with only general package initialization to do.
	try {
		
		sDefaultConsole = new GenieConsoleSystem();
		sDefaultConsole->InitWindow();
		//windDefaultConsole->retain(); // Is this superfluous?
		//GenieConsolePane *console = new GenieConsolePane(windDefaultConsole);
		//windDefaultConsole->SetPane(console);
		//windDefaultConsole->MakeNew(/*rect, "\pConsole"*/);
	} catch (...) {
		return -1;
	}
	
	return 0;
}

/*
 *	extern void RemoveConsole(void);
 *
 *	Removes the console package.  It is called after all other streams
 *	are closed and exit functions (installed by either atexit or _atexit)
 *	have been called.  Since there is no way to recover from an error,
 *	this function doesn't need to return any.
 */

void RemoveConsole(void)
{
	// Since this is called after the app module objects have been destroyed,
	// we need to be extremely careful.
	// In fact, we should probably just delete sDefaultConsole when the app goes.
	
	//sDefaultConsole.release();
}

/*
 *	extern long WriteCharsToConsole(char *buffer, long n);
 *
 *	Writes a stream of output to the Console window.  This function is
 *	called by write.
 *
 *	char *buffer:	Pointer to the buffer to be written.
 *	long n:			The length of the buffer to be written.
 *	returns short:	Actual number of characters written to the stream,
 *					-1 if an error occurred.
 */

long WriteCharsToConsole(char *buffer, long n)
{
	if (sCurrentConsole == NULL) {
		if (sDefaultConsole != NULL)
			sCurrentConsole = sDefaultConsole;
		else
			return 0;
	}
	
	return sCurrentConsole->WriteChars(buffer, n);
}

/*
 *	extern long ReadCharsFromConsole(char *buffer, long n);
 *
 *	Reads from the Console into a buffer.  This function is called by
 *	read.
 *
 *	char *buffer:	Pointer to the buffer which will recieve the input.
 *	long n:			The maximum amount of characters to be read (size of
 *					buffer).
 *	returns short:	Actual number of characters read from the stream,
 *					-1 if an error occurred.
 */

long ReadCharsFromConsole(char *buffer, long n)
{
#pragma unused (buffer, n)

	return 0;
}

/*
 *	extern char *__ttyname(long fildes);
 *
 *	Return the name of the current terminal (only valid terminals are
 *	the standard stream (ie stdin, stdout, stderr).
 *
 *	long fildes:	The stream to query.
 *
 *	returns char*:	A pointer to static global data which contains a C string
 *					or NULL if the stream is not valid.
 */

extern char *__ttyname(long fildes)
{
#pragma unused (fildes)
	/* all streams have the same name */
	static char *__devicename = "Pipe Organ";

	if (fildes >= 0 && fildes <= 2)
		return (__devicename);

	return (0L);
}

/*     Change record
*/
