/*	===================
 *	GenieConsoleTerm.cc
 *	===================
 */

// Standard library
#include <string.h>

// Mac OS Universal Headers
#include <Files.h>
#include <Menus.h>

#include "PedestalDebugging.h"
#include "DGHandleLocker.hh"
#include "SiesString.hh"

// Pedestal
#include "PedWindow.hh"
#include "PedPaneTE.hh"
#include "PedFSRef.hh"
#include "Ped1AppProcess.hh"

// Genie
#include "GenieConsoleTerm.hh"
#include "GenieShell.hh"
#include "version.h"


GenieConsoleTerm::GenieConsoleTerm()
{
	mShell = NULL;
	mPrompt = "wish$ ";
	mPromptLen = strlen(mPrompt);
	mCmdStart = 0;
}

GenieConsoleTerm::~GenieConsoleTerm()
{
	CloseWindow();
}

void
GenieConsoleTerm::InitWindow()
{
	GenieConsole::InitWindow();
	mWindow->SetTitle("\pgterm");
}

bool
GenieConsoleTerm::OpenWindow()
{
	if (GenieConsole::OpenWindow()) {
		if (!mPane) throw;
		if (!mShell) {
			mShell = new GenieShell(this, *this);
		}
		
		char *hello = "Genie " VERSION_STRING " at your service, Master!\r";
		mPane->AppendChars(hello, strlen(hello));
		
		FSSpec spec = Ped1AppProcess::Me().FSS();
		PedFSRef ref(spec);
		ref -= 1;
		spec = ref.FSS();
		char *dir1 = "Current directory is '";
		char *dir2 = "'.\r";
		mPane->AppendChars(dir1, strlen(dir1));
		mPane->AppendChars((char *)&spec.name[1], spec.name[0]);
		mPane->AppendChars(dir2, strlen(dir2));
		
		Prompt();
		return true;
	}
	return false;
}

bool
GenieConsoleTerm::CloseWindow()
{
	if (mShell) {
		mShell->release();
		mShell = NULL;
	}
	return GenieConsole::CloseWindow();
}

void
GenieConsoleTerm::ProcessKey(char inChar, short inCode)
{
	char c = inChar;
	Size start, end;
	// "Illegal jump past initializer." -- MWC68K
	// Thanks, guys.  Did you ever consider "Illegal constructor call in switch"?
	SiesString str;
	
	switch (c) {
		case 0x1c: // left arrow
		case 0x1d: // right arrow
			mPane->GetSelection(&start, &end);
			if (start < mCmdStart + mPromptLen) {
				start = mCmdStart + mPromptLen;
			}
			if (start > end) {
				end = start;
			}
			if (start == end) {
				start = end += (c == 0x1d) ? 1 : ((start > mCmdStart + mPromptLen) ? -1 : 0);
			} else {
				(c == 0x1d) ? start = end : end = start;
			}
			mPane->SetSelection(start, end);
			break;
		case 0x1e: // up arrow
		case 0x1f: // down arrow
			break;
		case '\r':
			mPane->SetSelection(32767, 32767);
			mPane->DoKey(c);
			short cmdLen = mPane->TextLength() - (mCmdStart + mPromptLen);
			//char *command = new char [cmdLen];
			//strncpy(command, *mPane->TextHandle() + mCmdStart + mPromptLen, cmdLen - 1);
			//command[cmdLen - 1] = '\0';
			if (1) {
				DGHandleLocker lock(mPane->TextHandle());
				str.Set(*mPane->TextHandle() + mCmdStart + mPromptLen, cmdLen - 1);
			}
			//GenieConsole *saved = GetCurrentConsole();
			//SetCurrentConsole(this);
			//try {
				mShell->ExecuteCmdLine(str);
			//} catch (...) {
			//	SetCurrentConsole(saved);
			//}
			//SetCurrentConsole(saved);
			//delete [] command;
			Prompt();
			break;
		default:
			mPane->GetSelection(&start, &end);
			if (start < mCmdStart + mPromptLen) {
				start = mCmdStart + mPromptLen;
			}
			if (start > end) {
				end = start;
			}
			mPane->SetSelection(start, end);
			if ((c != '\b' && c != 0x1c) || start != end || start > mCmdStart + mPromptLen) {
				mPane->DoKey(c);
			}
			break;
	}
}
