/*	=============
 *	GenieShell.cc
 *	=============
 */

#include <string.h>
#include <ctype.h>

#include "MoreFilesExtras.h"

#include "DGDebugging.h"

#include "GenieShell.hh"

#include "SiesString.hh"
#include "Ped1AppProcess.hh"
#include "GenieConsole.hh"
#include "OrganConsole.hh"
#include "execute.hh"
#include "pathnames.hh"


GenieShell::GenieShell(PedTask *inParent, GenieConsole &inConsole)
: PedTask(inParent), mConsole(inConsole), mCwd(Ped1AppProcess::Me().FSS())
{
	mCwd -= 1;
}

GenieShell::~GenieShell()
{
	
}


long
GenieShell::ExecuteCmdLine(const SiesString &inCmd)
{
	char *cmd, *p, *q, *r, *ch, *chunk, *arg, **argv;
	short n = 0, i;
	void **cons, **prev = NULL;
	long result;
	
	cmd = new char [inCmd.Length() + 1];
	inCmd.GetCStr(cmd, inCmd.Length());
	
	if (cmd == NULL)
		return 0;
	
	p = cmd;
	// skip leading space
	while (*p != '\0' && isspace(*p))
		p++;
	if (*p == '\0')
		return 0;
	
	chunk = new char [strlen(cmd)];
	for (r = q = p; *r != NULL; q = r, n++) {
		// scan over arg
		ch = chunk;
		while (*r != '\0' && !isspace(*r)) {
			if (*r == '\\') {
				*r++ = r[1];
				memcpy(ch, q, r - q);
				ch += r - q;
				q = *r ? ++r : r;
			} else
				r++;
		}
		memcpy(ch, q, r - q);
		ch += r - q;
		// copy arg
		arg = new char [ch - chunk + 1];
		memcpy(arg, chunk, ch - chunk);
		arg[ch - chunk] = '\0';
		// create new linked list node
		cons = new void * [2];
		cons[0] = arg;
		cons[1] = prev;
		prev = cons;
		// skip intervening space
		while (*r != '\0' && isspace(*r))
			r++;
	}
	delete [] chunk;
	
	// create the arg vector
	argv = new char * [n + 1];
	argv++;
	argv[-1] = NULL;
	
	// destructively traverse the linked list, copying args into the vector
	for (i = n - 1; i >= 0; i--) {
		argv[i] = (char *)cons[0];
		prev = (void **)cons[1];
		delete [] cons;
		cons = prev;
	}
	
	result = ExecuteArgs(n, argv);
	
	for (i = n - 1; i >= 0; i--) {
		delete [] argv[i];
	}
	
	argv--;
	delete [] argv;
	delete [] cmd;
	
	return result;
}

long
GenieShell::ExecuteArgs(long argC, char **argV)
{
	long result;
	// a pointer to a function taking argc and argv and returning int
	int (* mainFunc)(int, char **);
	
	if (argC < 1)
		return 0;
	
	GenieConsole *saved = GetCurrentConsole();
	SetCurrentConsole(&mConsole);
	try {
		mainFunc = NULL;
		if (0) {
		} else if (strcmp(argV[0], "echo") == 0) {
			short i;
			
			if (argC > 1) {
				fprintf(stdout, "%s", argV[1]);
				for (i = 2; i < argC; i++)
					fprintf(stdout, " %s", argV[i]);
			}
			fprintf(stdout, "\n");
			result = 0;
		} else if (strcmp(argV[0], "cd") == 0) {
			result = ChangeDirectory(argC > 1 ? argV[1] : NULL);
		} else if (strcmp(argV[0], "aevt") == 0) {
			mainFunc = aevt_main;
		} else if (strcmp(argV[0], "cat") == 0) {
			mainFunc = cat_main;
		} else if (strcmp(argV[0], "cp") == 0) {
			mainFunc = cp_main;
		} else if (strcmp(argV[0], "ls") == 0) {
			mainFunc = ls_main;
		} else if (strcmp(argV[0], "mv") == 0) {
			mainFunc = mv_main;
		} else if (strcmp(argV[0], "mkdir") == 0) {
			mainFunc = mkdir_main;
		} else if (strcmp(argV[0], "pwd") == 0) {
			mainFunc = pwd_main;
		} else if (strcmp(argV[0], "shutdown") == 0) {
			mainFunc = shutdown_main;
		} else if (strcmp(argV[0], "touch") == 0) {
			mainFunc = touch_main;
		} else {
			fprintf(stderr, "Genie: no such command '%s'.\n", argV[0]);
			result = -1;
		}
		
		if (mainFunc != NULL) {
			char **myArgV = new char *[(argC + 1) * sizeof (char **)];
			FSSpec cwd = mCwd.FSS();
			long dirID;
			Boolean isDir;
			OSErr err = ::FSpGetDirectoryID(&cwd, &dirID, &isDir);
			ThrowIfOSErr_(err);
			FSDirSpec dir = {0, cwd.vRefNum, dirID};
			myArgV[0] = (char *)&dir;
			myArgV++;
			memcpy(myArgV, argV, argC * sizeof (char **));
			// Call the imported main() function.  It's probably *not* reentrant.
			// This will cause us to block until it completes.
			result = mainFunc(argC, myArgV);
			delete myArgV;
		}
	} catch (...) {
		SetCurrentConsole(saved);
	}
	SetCurrentConsole(saved);
	
	return result;
}

long
GenieShell::ChangeDirectory(const char *inPathname)
{
	OSErr err;
	long dirID;
	Boolean isDir;
	
	if (inPathname != NULL) {
		FSSpec spec = mCwd.FSS();
		err = ::FSpGetDirectoryID(&spec, &dirID, &isDir);
		ThrowIfOSErr_(err);
		
		FSDirSpec cwd = {0, spec.vRefNum, dirID};
		FSDirSpec dir;
		SiesString nameStr;
		UnixPathname2DirAndName(cwd, inPathname, dir, nameStr);
		if (dir.dirID == 1) {
			return 0;
		}
		//mCwd += inPathname;
		Str255 name;
		nameStr.GetPStr(name, 255);
		mCwd.SetVolDirName(dir.vRefNum, dir.dirID, name);
		if (!mCwd.IsDirectory()) {
			mCwd.SetFSSpec(spec);
			fprintf(stderr, "Genie: cd: %s: No such file or directory\n", inPathname);
			return -1;
		}
	} else {
		// What constitutes home here?
	}
	return 0;
}
