/* aevt.c */

#include "AEADebugging.h"

// Standard C
#include <stdio.h>
#include <string.h>

// Mac Universal Headers
#include <AppleEvents.h>
#include <Processes.h>
#include <Script.h>
#include <ShutDown.h>

#include "pstring.h"

// Apple Event Assistant
#include "AEAAppleEventSender.hh"
#include "AEA1AEIdleFunction.hh"

#define NAM

#ifdef NAM
extern "C" int aevt_main(int argc, char *argv[]);
#else
#define aevt_main main
#endif

static OSErr LocateTarget(Str255 inName, Str255 inMachine, TargetID *outTarget);
static OSErr QuitProcesses(void);
static OSErr QuitPSN(const ProcessSerialNumber &inPSN);

// shell$ aevt -m Otter -a Genie |gan Exec '----':[shutdown,-h]
// shell$ aevt -s hhgg aevt quit
int aevt_main(int argc, char *argv[])
{
	char *argMachine, *argApp, *argSig, *argEventClass, *argEventID, *argBuild;
	argMachine = argApp = argSig = argEventClass = argEventID = argBuild = NULL;
	
	if (argc > 2 && strcmp(argv[1], "-m") == 0) {
		argMachine = argv[2];
		argv += 2, argc -= 2;
	};
	
	if (argc > 2 && strcmp(argv[1], "-a") == 0) {
		argApp = argv[2];
		argv += 2, argc -= 2;
	};
	
	if (argc > 2 && strcmp(argv[1], "-s") == 0) {
		argSig = argv[2];
		argv += 2, argc -= 2;
	};
	
	
	if (argc == 1) {
		fprintf(stderr, "%s", 
			"Usage:  aevt [-m machine] {-a app | -s sign} class id [params]\n"
			"Examples: aevt -s hhgg aevt quit\n"
			"          aevt -m Headless\\ Mac -a Genie |gan Exec '----':[shutdown -h]\n"
			"          aevt -s R*ch misc slct '----':obj{want:type(clin), form:indx, seld:42,\n"
			"               from:obj{want:type(cwin), form:indx, seld:1, from:null()}}\n");
		return 0;
	} else if (argc < 3) {
		fprintf(stderr, "aevt: %s\n", "missing arguments");
		return 1;
	} else if (argc < 4) {
		argBuild = "";
	} else
		argBuild = argv[3];
	argEventClass = argv[1];
	argEventID = argv[2];
	
	if (strlen(argEventClass) != 4 || strlen(argEventID) != 4) {
		fprintf(stderr, "aevt: %s\n", "invalid parameter");
		return 1;
	}
	
	Str255 machStr, appStr;
	if (argMachine != NULL) {
		Pstrcpy(machStr, argMachine);
		if (strlen(argMachine) > 31)
			machStr[0] = 31;
	} else
		machStr[0] = 0;
	if (argApp != NULL) {
		Pstrcpy(appStr, argApp);
		if (strlen(argApp) > 31)
			appStr[0] = 31;
	} else
		appStr[0] = 0;
	
	OSType sig = argSig ? *(OSType *)argSig : '\0\0\0\0';
	AEEventClass eventClass = *(AEEventClass *)argEventClass;
	AEEventID eventID = *(AEEventID *)argEventID;
	
	AEAAppleEventSender evt(eventClass, eventID);
	evt.SetGizmoBuildString(argBuild);
	if (sig)
		evt.SetAddress(sig);
	else {
		OSErr err;
		TargetID target;
		err = LocateTarget(appStr, machStr, &target);
		evt.SetAddress(target);
	}
	evt.Send();
	
	return 0;
}

OSErr LocateTarget(Str255 inName, Str255 inMachine, TargetID *outTarget)
{
	outTarget->name.nameScript = smRoman;
	PstrPcpy(outTarget->name.name, inName);
	outTarget->name.portKindSelector = ppcByString;
	PstrPcpy(outTarget->name.u.portTypeStr, "\p=");
	
	outTarget->location.locationKindSelector = inMachine[0] ? ppcNBPLocation : ppcNoLocation;
	PstrPcpy(outTarget->location.u.nbpEntity.objStr, inMachine);
	PstrPcpy(outTarget->location.u.nbpEntity.typeStr, "\pPPCToolbox");
	PstrPcpy(outTarget->location.u.nbpEntity.zoneStr, "\p*");
	
	PortInfoRec portInfo;
	IPCListPortsPBRec pb;
	pb.startIndex = 0;
	pb.requestCount = 1;
	pb.portName = &outTarget->name;
	pb.locationName = &outTarget->location;
	pb.bufferPtr = &portInfo;
	OSErr err;
	err = IPCListPorts(&pb, false);
	outTarget->name = portInfo.name;
	
	return err;
}

OSErr QuitPSN(const ProcessSerialNumber &inPSN)
{
	AEAAppleEventSender quit(kCoreEventClass, kAEQuitApplication);
	quit.SetGizmoBuildString("");
	quit.SetAddress(inPSN);
	quit.Send();
	
	return noErr;
}

OSErr QuitProcesses(void)
{
	OSErr err;
	const ProcessSerialNumber psnNone = {0, kNoProcess};
	ProcessSerialNumber psn, psnMe, psnFinder, psnFSE;
	ProcessInfoRec procInfo;
	Boolean same;
	Boolean runningFinder = false, runningFSE = false, runningApps = false;
	Boolean waiting = false;
	enum {apps, fse, finder, done} quitting;
	
	err = GetCurrentProcess(&psnMe);
	if (err != noErr) return err;
	
	procInfo.processInfoLength = sizeof procInfo;
	procInfo.processName = NULL;
	procInfo.processAppSpec = NULL;
	
	quitting = apps;
	
	while (quitting != done) {
		EventRecord evt;
		long tcQuit, tcNow;
		
		psn = psnNone;
		runningFinder = runningFSE = runningApps = false;
		// loop over each process
		while (1) {
			//OSErr err;
			
			err = GetNextProcess(&psn);
			if (err == procNotFound) break; // Okay, that's everyone
			if (err != noErr) return err; // This shouldn't fail
			
			err = SameProcess(&psn, &psnMe, &same);
			if (err != noErr) return err; // This shouldn't fail
			if (same) continue; // Don't quit self yet
			
			err = GetProcessInformation(&psn, &procInfo);
			if (err != noErr) return err; // This shouldn't fail
			
			switch (procInfo.processSignature) {
				case 'MWDB':
					// don't try to quit the debugger (if we're running in it)
					break;
				case 'MACS':
					runningFinder = true;
					//psnFinder = psn;
					if (!waiting && quitting == finder) {
						err = QuitPSN(psn);
					}
					break;
				case 'hhgg':
					runningFSE = true;
					//psnFSE = psn;
					if (!waiting && quitting == fse) {
						err = QuitPSN(psn);
					}
					break;
				default:
					runningApps = true;
					char sig[5] = "----";
					*(OSType *)sig = procInfo.processSignature;
					fprintf(stderr, waiting ? "%s still running\n" : "quit %s\n", sig);
					fflush(stderr);
					if (!waiting) {
						err = QuitPSN(psn);
						(void)WaitNextEvent(nullEvent, &evt, 30, NULL);
					}
					break;
			}
		}
		tcNow = TickCount();
		if (!waiting)
			tcQuit = tcNow;
		if (quitting == finder)
			if (!runningFinder)
				quitting = done, waiting = false;
			else if (tcNow - tcQuit > 300)
				quitting = done;
			else
				waiting = true;
		if (quitting == fse)
			if (!runningFSE)
				quitting = (runningFinder ? finder : done), waiting = false;
			else if (tcNow - tcQuit > 300)
				quitting = done;
			else
				waiting = true;
		if (quitting == apps)
			if (!runningApps)
				quitting = (runningFSE ? fse : runningFinder ? finder : done), waiting = false;
			else if (tcNow - tcQuit > 300)
				quitting = done;
			else
				waiting = true;
		FlushEvents(everyEvent, 0);
		
		if (waiting)
			(void)WaitNextEvent(nullEvent, &evt, 300 - (tcNow - tcQuit), NULL); // give apps a chance to quit
	}
	
	return (runningApps || runningFSE || runningFinder) ? userCanceledErr : noErr;
}
