/* cat.cc */

#include <stdio.h>

#include <Errors.h>

#include "MoreFilesExtras.h"

#include "PedestalDebugging.h"
#include "pstring.h"

#include "SiesString.hh"

#include "PedFSRef.hh"
#include "PedDataSourceFile.hh"
#include "PedStreamInputBufferedText.hh"

#include "pathnames.hh"

#define NAM

#ifdef NAM
extern "C" int cat_main(int argc, char *argv[]);
#else
#define cat_main main
#endif

static int DumpFile(
	short vol, long dir, const unsigned char *name);
	
int cat_main(int argc, char *argv[])
{
	OSErr err;
	FSDirSpec cwdir = {0, 0, 0}, *cwdp;
	FSDirSpec srcDir;
	SiesString str;
	Str31 name;
	int index, fail;
	
	// Current working directory: either passed 'under the table' or set to default.
	cwdp = (argv[-1] != NULL) ? (FSDirSpec *)argv[-1] : (&cwdir);
	
	// Check for sufficient number of args
	if (argc < 2) {
		fprintf(stderr, "cat: missing arguments\n");
		return 1;
	}
	
	// Print each file in turn.  Return whether any errors occurred.
	fail = 0;
	for (index = 1; index < argc; index++) {
		err = UnixRelativePathname2DirAndName(*cwdp, argv[index], srcDir, str); 
		str.GetPStr(name, 31);
		if (err == noErr)
			fail += DumpFile(srcDir.vRefNum, srcDir.dirID, name);
		else  {
			fprintf(stderr, "cat: %s: %s\n", argv[index], "No such file or directory");
			fail++;
		}
	}
	return (fail == 0) ? 0 : 1;
}

int DumpFile(
	short vol, long dir, const unsigned char *inName)
{
	OSErr err;
	Boolean isDir;
	
	try {
		PedFSRef ref(vol, dir, inName);
		PedDataSourceFile source(ref);
		PedStreamInputBufferedText stream(source);
		short c;
		while ((c = stream.GetByte()) != -1) {
			// Translate Mac to unix
			char byte = (c == 13) ? 10 : c;
			putc(byte, stdout);
		}
		fflush(stdout);
	} catch (ExceptionCode err) {
		char name[32];
		strPcpy(name, inName);
		fprintf(stderr, "cat: %s: %s %ld\n", name, "exception", err);
		return 1;
	}
	return 0;
}
