/* ls.c */

#include <stdio.h>

#include "IterateDirectory.h"
#include "MoreFilesExtras.h"
#include "pstring.h"
#include "SiesString.hh"
#include "pathnames.hh"

#define LAMP

#ifdef LAMP
extern "C" int ls_main(int argc, char *argv[]);
#else
#define ls_main main
#endif

pascal void ls_filter(const CInfoPBRec * const cpbPtr, Boolean *quitFlag, void *yourDataPtr);

int ls_main(int argc, char *argv[])
{
	OSErr err;
	FSDirSpec cwdir = {0, 0, 0}, *cwdp;
	FSDirSpec lsDir;
	IterateFilterProcPtr filter = &ls_filter;
	SiesString lsName;
	//char *lsName;
	Str31 pname;
	Boolean isDir;
	int i, fail = 0;
	long dirID;
	
	cwdp = (argv[-1] != NULL) ? (FSDirSpec *)argv[-1] : (&cwdir);
	
	if (argc > 1) {
		for (i = 1; i < argc; i++) {
			err = UnixRelativePathname2DirAndName(*cwdp, argv[i], lsDir, lsName);
			if (err != noErr) {
				fprintf(stderr, "ls: %s: %s %d\n", argv[i], "pathname conversion error", err);
				fail++;
				continue;
			}
			lsName.GetPStr(pname, 31);
			err = GetDirectoryID(lsDir.vRefNum, lsDir.dirID, pname, &dirID, &isDir);
			if (err != noErr) {
				fprintf(stderr, "ls: %s: %s %d\n", argv[i], "No such file or directory", err);
				fail++;
			} else if (!isDir) {
				printf("%s\n", argv[i]);
			} else {
				err = IterateDirectory(lsDir.vRefNum, lsDir.dirID, pname, 1, ls_filter, NULL);
				err && fail++;
			}
		}
	} else {
		lsDir = *cwdp;
		err = IterateDirectory(lsDir.vRefNum, lsDir.dirID, NULL, 1, ls_filter, NULL);
		err && fail++;
	}
	
	return fail;
}

pascal void ls_filter(const CInfoPBRec * const cpbPtr, Boolean *quitFlag, void *yourDataPtr)
{
	Str255 pname;
	char name[256];
	
	MangleMacFilename(cpbPtr->hFileInfo.ioNamePtr, pname);
	strPcpy(name, pname);
	printf("%s\n", name);
}
