/* touch.c */

#include <stdio.h>

#include <Errors.h>

#include "MoreFilesExtras.h"
#include "pstring.h"
#include "SiesString.hh"
#include "pathnames.hh"

#define NAM

#ifdef NAM
extern "C" int touch_main(int argc, char *argv[]);
#else
#define touch_main main
#endif

static int TryHFSSoftTouch(
	short vol, long dir, const unsigned char *name);
static int TryHFSHeavyTouch(
	short vol, long dir, const unsigned char *name);
	
int touch_main(int argc, char *argv[])
{
	OSErr err;
	FSDirSpec cwdir = {0, 0, 0}, *cwdp;
	FSDirSpec srcDir;
	SiesString str;
	Str31 name;
	int index, fail;
	
	// Current working directory: either passed 'under the table' or set to default.
	cwdp = (argv[-1] != NULL) ? (FSDirSpec *)argv[-1] : (&cwdir);
	
	// Check for sufficient number of args
	if (argc < 2) {
		fprintf(stderr, "touch: missing arguments\n");
		return 1;
	}
	
	// Try to touch each item.  Return whether any errors occurred.
	fail = 0;
	for (index = 1; index < argc; index++) {
		err = UnixRelativePathname2DirAndName(*cwdp, argv[index], srcDir, str); 
		str.GetPStr(name, 31);
		if (err == noErr)
			fail += TryHFSSoftTouch(srcDir.vRefNum, srcDir.dirID, name);
		else if (err == fnfErr)
			fail += TryHFSHeavyTouch(srcDir.vRefNum, srcDir.dirID, name);
		else  {
			fprintf(stderr, "touch: %s: %s\n", argv[index], "nonexistent directory in pathname");
			fail++;
		}
	}
	return (fail == 0) ? 0 : 1;
}

int TryHFSSoftTouch(
	short vol, long dir, const unsigned char *inName)
{
	OSErr err;
	//Str255 nameStr;
	Boolean isDir;
	
	//PstrPcpy(nameStr, name);
	err = BumpDate(vol, dir, inName);
	if (err != noErr) {
		char name[32];
		strPcpy(name, inName);
		fprintf(stderr, "touch: %s: %s %d\n", name, "BumpDate error", err);
		return 1;
	}
	return 0;
}

int TryHFSHeavyTouch(
	short vol, long dir, const unsigned char *inName)
{
	OSErr err;
	//Str255 nameStr;
	Boolean isDir;
	
	//Pstrcpy(nameStr, name);
	err = HCreate(vol, dir, inName, 'R*ch', 'TEXT');
	if (err != noErr) {
		char name[32];
		strPcpy(name, inName);
		fprintf(stderr, "touch: %s: %s %d\n", name, "HCreate error", err);
		return 1;
	}
	return 0;
}
