/*	===================
 *	PedViewMultipane.cc
 *	===================
 */

#include "PedestalDebugging.h"

#include "PedViewMultipane.hh"
#include "PedPaneSubView.hh"


static
void
OpenPane(PedPane *inPane, void *inPtr)
{
	inPane->Open();
}

static
void
ClosePane(PedPane *inPane, void *inPtr)
{
	inPane->Close();
}

static
void
ResizePane(PedPane *inPane, void *inPtr)
{
	Point *delta = (Point *)inPtr;
	inPane->Resize(delta->h, delta->v);
}

static
void
DrawPane(PedPane *inPane, void *inPtr)
{
	inPane->DrawContent();
}


PedViewMultipane::PedViewMultipane(PedPaneSubView &inSuperPane)
: PedViewSub(inSuperPane)
{
}

PedViewMultipane::~PedViewMultipane()
{
}

void
PedViewMultipane::AddPane(PedPane *inPane)
{
	if (!inPane) return;
	
	inPane->retain();
	mPanes.Append(inPane);
}

void
PedViewMultipane::RemovePane(PedPane *inPane)
{
	if (!inPane) return;
	
	mPanes.Remove(inPane);
	inPane->release();  // need to check if it was there to be removed or not
}

void
PedViewMultipane::Focus()
{
	//mPort.Set();
}

void
PedViewMultipane::Open()
{
	mPanes.ForEach(&OpenPane, NULL);
}

void
PedViewMultipane::Close()
{
	mPanes.ForEach(&ClosePane, NULL);
}

void
PedViewMultipane::Activate()
{
}

void
PedViewMultipane::Deactivate()
{
}

void
PedViewMultipane::Resize(short inWidth, short inHeight)
{
	PedView::Resize(inWidth, inHeight);
	Point delta;
	delta.h = inWidth;
	delta.v = inHeight;
	mPanes.ForEach(&ResizePane, &delta);
}


void
PedViewMultipane::DrawContent()
{
	while (0) {
		
	}
	// set up drawing environment
	// for each of mPanes, Draw()
	// restore drawing environment
	mPanes.ForEach(&DrawPane, NULL);
}

void
PedViewMultipane::DispatchNullEvent(EventRecord &inEvent)
{
	
}

void
PedViewMultipane::DispatchClickEvent(EventRecord &inEvent)
{
}

void
PedViewMultipane::DispatchKey(EventRecord &inEvent)
{
}
