#include "6502.h"
#include "global.h"
#include "stack.h"
#include "bcd.h"

#define SETSTATUSBIT(bit)	regP |= bit
#define CLEARSTATUSBIT(bit)	regP &= ~bit
#define GETSTATUS(bit)		((regP & bit) ? 1 : 0)

#define SETNSTATUS(reg) if (reg & NBit)	SETSTATUSBIT(NBit); else CLEARSTATUSBIT(NBit)
#define SETZSTATUS(reg)	if (reg == 0) 	SETSTATUSBIT(ZBit); else CLEARSTATUSBIT(ZBit)

#define UInt8ToSInt16(num)		((num & 0x7F) * ((num & 0x80) ? -1 : 1))

#define NMI 0xFFFA				// NON MASKABLE INTERUPT VECTOR
#define RES 0xFFFC				// RESET VECTOR
#define IRQ	0xFFFE				// INTERUPT REQUEST VECTOR

//////////////////////////////////////////////
//
//	ADC		Add with Carry to A
//
//////////////////////////////////////////////
void _ADC(UInt8 opByte)
{
	SInt16 l_a, h_a, l_opByte, h_opByte, l_result, h_result, result, tempA, tempopByte;

	if (GETSTATUS(DBit)) {
		l_a = (regA & 0x0F);
		h_a = ((regA & 0xF0) >> 4);
		l_opByte = (opByte & 0x0F);
		h_opByte = ((opByte & 0xF0) >> 4);

		l_result = (l_a + l_opByte + GETSTATUS(CBit));
		h_result = (l_result > 9) ? (h_a + h_opByte + 1) : (h_a + h_opByte);

		if (h_result > 9) {
			SETSTATUSBIT(CBit);
		}
		else {
			CLEARSTATUSBIT(CBit);
		}
		regA = (((h_result % 10) << 4) + (l_result % 10));
	}
	else {
		result = (regA + opByte + GETSTATUS(CBit));

		tempA = UInt8ToSInt16(regA);
		tempopByte = UInt8ToSInt16(opByte);

		if (((tempA<128) && (tempopByte<128) && ((result%256)>127)) ||
			((tempA>127) && (tempopByte>127) && ((result%256<128)))) {
			SETSTATUSBIT(VBit);
		}
		else {
			CLEARSTATUSBIT(VBit);
		}

		regA = (result % 256);
		if (result > 255) {
			SETSTATUSBIT(CBit);
		}
		else {
			CLEARSTATUSBIT(CBit);
		}
	}
	SETNSTATUS(regA);
	SETZSTATUS(regA);
}

//////////////////////////////////////////////
//
//	AND		And to A
//
//////////////////////////////////////////////
void _AND(UInt8 opByte)
{
	regA &= opByte;
	if (GETSTATUS(CBit)) {
		++regA;
	}
	SETNSTATUS(regA);
	SETZSTATUS(regA);
}

//////////////////////////////////////////////
//
//	_ASL		Arithmetic shift left
//
//////////////////////////////////////////////
void _ASL(UInt8* addr)
{
	UInt8 theBye = ((*addr) << 1);
	if (*addr & 0x80)
		SETSTATUSBIT(CBit);
	else
		CLEARSTATUSBIT(CBit);

	*addr = theBye;
	SETNSTATUS(theBye);
	SETZSTATUS(theBye);
}

//////////////////////////////////////////////
//
//	_BCC		Branch Carry Clear
//
//////////////////////////////////////////////
void _BCC(UInt8 opByte)
{
	SInt8 theOffset = 0;
	if (GETSTATUS(CBit) == 0) {
		theOffset |= opByte;
		if (theOffset < 0) {
			pc -= (-theOffset);
		}
		else {
			pc += theOffset;
		}
	}
}

//////////////////////////////////////////////
//
//	_BCS		Branch Carry Set
//
//////////////////////////////////////////////
void _BCS(UInt8 opByte)
{
	SInt8 theOffset = 0;
	if (GETSTATUS(CBit)) {
		theOffset |= opByte;
		if (theOffset < 0) {
			pc -= (-theOffset);
		}
		else {
			pc += theOffset;
		}
	}
}

//////////////////////////////////////////////
//
//	_BEQ		Branch Equal
//
//////////////////////////////////////////////
void _BEQ(UInt8 opByte)
{
	SInt8 theOffset = 0;
	if (GETSTATUS(ZBit)) {
		theOffset |= opByte;
		if (theOffset < 0) {
			pc -= (-theOffset);
		}
		else {
			pc += theOffset;
		}
	}
}

//////////////////////////////////////////////
//
//	_BNE		Branch Not Equal
//
//////////////////////////////////////////////
void _BNE(UInt8 opByte)
{
	SInt8 theOffset = 0;
	if (GETSTATUS(ZBit) == 0) {
		theOffset |= opByte;
		if (theOffset < 0) {
			pc -= (-theOffset);
		}
		else {
			pc += theOffset;
		}
	}
}

//////////////////////////////////////////////
//
//	_BMI		Branch Minus
//
//////////////////////////////////////////////
void _BMI(UInt8 opByte)
{
	SInt8 theOffset = 0;
	if (GETSTATUS(NBit)) {
		theOffset |= opByte;
		if (theOffset < 0) {
			pc -= (-theOffset);
		}
		else {
			pc += theOffset;
		}
	}
}

//////////////////////////////////////////////
//
//	_BPL		Branch Plus
//
//////////////////////////////////////////////
void _BPL(UInt8 opByte)
{
	SInt8 theOffset = 0;
	if (GETSTATUS(NBit) == 0) {
		theOffset |= opByte;
		if (theOffset < 0) {
			pc -= (-theOffset);
		}
		else {
			pc += theOffset;
		}
	}
}

//////////////////////////////////////////////
//
//	_BVC		Branch Overflow Clear
//
//////////////////////////////////////////////
void _BVC(UInt8 opByte)
{
	SInt8 theOffset = 0;
	if (GETSTATUS(VBit) == 0) {
		theOffset |= opByte;
		if (theOffset < 0) {
			pc -= (-theOffset);
		}
		else {
			pc += theOffset;
		}
	}
}

//////////////////////////////////////////////
//
//	_BVS		Branch Overflow Set
//
//////////////////////////////////////////////
void _BVS(UInt8 opByte)
{
	SInt8 theOffset = 0;
	if (GETSTATUS(VBit)) {
		theOffset |= opByte;
		if (theOffset < 0) {
			pc -= (-theOffset);
		}
		else {
			pc += theOffset;
		}
	}
}

//////////////////////////////////////////////
//
//	_BIT	And With Accumulator (A Unchanged)
//
//////////////////////////////////////////////
void _BIT(UInt8* addr)
{
	UInt8 theBye = *addr;

	theBye &= regA;
	SETNSTATUS(theBye);
	SETZSTATUS(theBye);
	if (theBye & 0x40)
		SETSTATUSBIT(VBit);
	else
		CLEARSTATUSBIT(VBit);
}

//////////////////////////////////////////////
//
//	_BRK		Break
//
//////////////////////////////////////////////
void _BRK(void)
{
	regP |= BBit;
	PushWord(pc);
	PushByte(regP);
	pc = IRQ;
	return;
}

//////////////////////////////////////////////
//
//	_CLC		Clear carry
//
//////////////////////////////////////////////
void _CLC(void)
{
	regP &= ~CBit;
}

//////////////////////////////////////////////
//
//	_CLD		Clear Decimal
//
//////////////////////////////////////////////
void _CLD(void)
{
	regP &= ~DBit;
}

//////////////////////////////////////////////
//
//	_CLI		Clear Interupt
//
//////////////////////////////////////////////
void _CLI(void)
{
	regP &= ~IBit;
}

//////////////////////////////////////////////
//
//	_CLV		Clear Overflow
//
//////////////////////////////////////////////
void _CLV(void)
{
	regP &= ~VBit;
}

//////////////////////////////////////////////
//
//	_CMP		Compare to A
//
//////////////////////////////////////////////
void _CMP(UInt8 opByte)
{
	SInt16 tempregA, tempopByte, result;

	tempregA = UInt8ToSInt16(regA);
	tempopByte = UInt8ToSInt16(opByte);
	
	result = tempregA - tempopByte;
	if (result < 0) {
		result += 256;
		CLEARSTATUSBIT(CBit);
	}
	else {
		SETSTATUSBIT(CBit);
	}
	SETNSTATUS(result);
	SETZSTATUS(result);
}

//////////////////////////////////////////////
//
//	_CPX		Compare to X
//
//////////////////////////////////////////////
void _CPX(UInt8 opByte)
{
	SInt16 tempregX, tempopByte, result;

	tempregX = UInt8ToSInt16(regX);
	tempopByte = UInt8ToSInt16(opByte);
	
	result = tempregX - tempopByte;
	if (result < 0) {
		result += 256;
		CLEARSTATUSBIT(CBit);
	}
	else {
		SETSTATUSBIT(CBit);
	}
	SETNSTATUS(result);
	SETZSTATUS(result);
}

//////////////////////////////////////////////
//
//	_CPY		Compare to Y
//
//////////////////////////////////////////////
void _CPY(UInt8 opByte)
{
	SInt16 tempregY, tempopByte, result;

	tempregY = UInt8ToSInt16(regY);
	tempopByte = UInt8ToSInt16(opByte);
	
	result = tempregY - tempopByte;
	if (result < 0) {
		result += 256;
		CLEARSTATUSBIT(CBit);
	}
	else {
		SETSTATUSBIT(CBit);
	}
	SETNSTATUS(result);
	SETZSTATUS(result);
}

//////////////////////////////////////////////
//
//	_DEC	Decrement by one
//
//////////////////////////////////////////////
void _DEC(UInt8* addr)
{
	UInt8 theBye = *addr;

	theBye--;
	SETNSTATUS(theBye);
	SETZSTATUS(theBye);
	 *addr = theBye;
}

//////////////////////////////////////////////
//
//	_DEX	Decrement X by one
//
//////////////////////////////////////////////
void _DEX(void)
{
	regX--;
	SETNSTATUS(regX);
	SETZSTATUS(regX);
}

//////////////////////////////////////////////
//
//	_DEY	Decrement Y by one
//
//////////////////////////////////////////////
void _DEY(void)
{
	regY--;
	SETNSTATUS(regY);
	SETZSTATUS(regY);
}

//////////////////////////////////////////////
//
//	_EOR		XOR to Y
//
//////////////////////////////////////////////
void _EOR(UInt8 opByte)
{
	regA ^= opByte;

	SETNSTATUS(regA);
	SETZSTATUS(regA);
}

//////////////////////////////////////////////
//
//	_INC	Increment by one
//
//////////////////////////////////////////////
void _INC(UInt8* addr)
{
	UInt8 theBye = *addr;

	theBye++;
	SETNSTATUS(theBye);
	SETZSTATUS(theBye);
	 *addr = theBye;
}

//////////////////////////////////////////////
//
//	_INX	Increment X by one
//
//////////////////////////////////////////////
void _INX(void)
{
	regX++;
	SETNSTATUS(regX);
	SETZSTATUS(regX);
}

//////////////////////////////////////////////
//
//	_INY	Increment Y by one
//
//////////////////////////////////////////////
void _INY(void)
{
	regY++;
	SETNSTATUS(regY);
	SETZSTATUS(regY);
}

//////////////////////////////////////////////
//
//	_JMP	Jump to new location
//
//////////////////////////////////////////////
void _JMP(UInt16 address)
{
	pc = address;
}

//////////////////////////////////////////////
//
//	_JSR	Jump to SubRoutine
//
//////////////////////////////////////////////
void _JSR(UInt16 address)
{
	PushWord(pc -1);
	pc = address;
}

//////////////////////////////////////////////
//
//	_LDA	Load accumulator
//
//////////////////////////////////////////////
void _LDA(UInt8 opByte)
{
	regA = opByte;

	SETNSTATUS(regA);
	SETZSTATUS(regA);
}

//////////////////////////////////////////////
//
//	_LDX	Load X
//
//////////////////////////////////////////////
void _LDX(UInt8 opByte)
{
	regX = opByte;

	SETNSTATUS(regX);
	SETZSTATUS(regX);
}

//////////////////////////////////////////////
//
//	_LDY	Load Y
//
//////////////////////////////////////////////
void _LDY(UInt8 opByte)
{
	regY = opByte;

	SETNSTATUS(regY);
	SETZSTATUS(regY);
}

//////////////////////////////////////////////
//
//	_LSR		Logical shift right
//
//////////////////////////////////////////////
void _LSR(UInt8* addr)
{
	UInt8 theBye = ((*addr) >> 1);
	if (*addr & 0x01)
		SETSTATUSBIT(CBit);
	else
		CLEARSTATUSBIT(CBit);

	*addr = theBye;
	SETNSTATUS(theBye);
	SETZSTATUS(theBye);
}

//////////////////////////////////////////////
//
//	_NOP		No operation
//
//////////////////////////////////////////////
void _NOP(void)
{
	return;
}

//////////////////////////////////////////////
//
//	_ORA	Or with Accumulator
//
//////////////////////////////////////////////
void _ORA(UInt8 opByte)
{
	regA |= opByte;
	SETNSTATUS(regA);
	SETZSTATUS(regA);
}

//////////////////////////////////////////////
//
//	_PHA	Push A on stack
//
//////////////////////////////////////////////
void _PHA(void)
{
	PushByte(regA);
}

//////////////////////////////////////////////
//
//	_PHP	Push P on stack
//
//////////////////////////////////////////////
void _PHP(void)
{
	PushByte(regP);
}

//////////////////////////////////////////////
//
//	_PLA	Pop A from stack
//
//////////////////////////////////////////////
void _PLA(void)
{
	regA = PopByte();
	SETNSTATUS(regA);
	SETZSTATUS(regA);
}

//////////////////////////////////////////////
//
//	_PLP	Pop P from stack
//
//////////////////////////////////////////////
void _PLP(void)
{
	regP = PopByte();
}

//////////////////////////////////////////////
//
//	_ROL		Rotate left through carry
//
//////////////////////////////////////////////
void _ROL(UInt8* addr)
{
	UInt8 theBye = ((*addr) << 1);
	
	if (GETSTATUS(CBit))
		theBye |= 0x01;

	if (*addr & 0x80)
		SETSTATUSBIT(CBit);
	else
		CLEARSTATUSBIT(CBit);

	*addr = theBye;
	SETNSTATUS(theBye);
	SETZSTATUS(theBye);
}

//////////////////////////////////////////////
//
//	_ROR		Rotate right through carry
//
//////////////////////////////////////////////
void _ROR(UInt8* addr)
{
	UInt8 theBye = ((*addr) >> 1);
	
	if (GETSTATUS(CBit))
		theBye |= 0x80;

	if (*addr & 0x01)
		SETSTATUSBIT(CBit);
	else
		CLEARSTATUSBIT(CBit);

	*addr = theBye;
	SETNSTATUS(theBye);
	SETZSTATUS(theBye);
}

//////////////////////////////////////////////
//
//	RTI		Return From Interupt
//
//////////////////////////////////////////////
void _RTI(void)
{
	regP = PopByte();
	pc = PopWord();
}

//////////////////////////////////////////////
//
//	RTS		Return Subroutine
//
//////////////////////////////////////////////
void _RTS(void)
{
	UInt8 oldsp = sp;
	UInt16 addr = PopWord();
	if (oldsp < sp) {
		gDone = true;
		return;
	}
	pc = addr;
	pc++;
}

//////////////////////////////////////////////
//
//	SBC		Subtract with Carry to A
//
//////////////////////////////////////////////
void _SBC(UInt8 opByte)
{
	SInt16 a_a,  a_opByte, result, tempA, tempopByte;

	if (GETSTATUS(DBit)) {
		a_a = (((regA & 0xF0) >> 4) * 10) + (regA & 0x0F);
		a_opByte = (((opByte & 0xF0) >> 4) * 10) + (opByte & 0x0F);
		result = a_a - a_opByte;

		if (result <0) {
			SETSTATUSBIT(CBit);
			result = 100 - result;
		}
		else {
			CLEARSTATUSBIT(CBit);
		}
		regA = ((result % 10)  + ((result / 10) << 4));
	}
	else {
		tempA = UInt8ToSInt16(regA);
		tempopByte = UInt8ToSInt16(opByte);

		result = (tempA + GETSTATUS(CBit) + (255 - tempopByte));

		tempA = UInt8ToSInt16(regA);
		tempopByte = UInt8ToSInt16(opByte);

		if (((tempA<128) && (tempopByte<128) && ((result & 0xFF)>127)) ||
			((tempA>127) && (tempopByte>127) && ((result & 0xFF)<128))) {
			SETSTATUSBIT(VBit);
		}
		else {
			CLEARSTATUSBIT(VBit);
		}

		if (result > 255) {
			SETSTATUSBIT(CBit);
		}
		else {
			CLEARSTATUSBIT(CBit);
		}
		regA = (result & 0xFF);
	}
	SETNSTATUS(regA);
	SETZSTATUS(regA);
}

//////////////////////////////////////////////
//
//	_SEC		Set Carry
//
//////////////////////////////////////////////
void _SEC(void)
{
	regP |= CBit;
}

//////////////////////////////////////////////
//
//	_SED		Set Decimal Mode
//
//////////////////////////////////////////////
void _SED(void)
{
	regP |= DBit;
}

//////////////////////////////////////////////
//
//	_SEI		Set IRQ Disable
//
//////////////////////////////////////////////
void _SEI(void)
{
	regP |= IBit;
}

//////////////////////////////////////////////
//
//	_STA	Store A
//
//////////////////////////////////////////////
void _STA(UInt8* addr)
{
	 *addr = regA;
}

//////////////////////////////////////////////
//
//	_STX	Store X
//
//////////////////////////////////////////////
void _STX(UInt8* addr)
{
	 *addr = regX;
}

//////////////////////////////////////////////
//
//	_STY	Store Y
//
//////////////////////////////////////////////
void _STY(UInt8* addr)
{
	 *addr = regY;
}

//////////////////////////////////////////////
//
//	_TAX	Transfer A to X
//
//////////////////////////////////////////////
void _TAX(void)
{
	 regX = regA;
	 SETNSTATUS(regX);
	 SETZSTATUS(regX);
}

//////////////////////////////////////////////
//
//	_TAY	Transfer A to Y
//
//////////////////////////////////////////////
void _TAY(void)
{
	 regY = regA;
	 SETNSTATUS(regA);
	 SETZSTATUS(regA);
}

//////////////////////////////////////////////
//
//	_TSX	Transfer Stack to X
//
//////////////////////////////////////////////
void _TSX(void)
{
	 regX = sp;
	 SETNSTATUS(regX);
	 SETZSTATUS(regX);
}

//////////////////////////////////////////////
//
//	_TXA	Transfer X to A
//
//////////////////////////////////////////////
void _TXA(void)
{
	 regA = regX;
	 SETNSTATUS(regA);
	 SETZSTATUS(regA);
}

//////////////////////////////////////////////
//
//	_TXS	Transfer X to Stack
//
//////////////////////////////////////////////
void _TXS(void)
{
	 sp = regX;
	 SETNSTATUS(sp);
	 SETZSTATUS(sp);
}

//////////////////////////////////////////////
//
//	_TYA	Transfer Y to A
//
//////////////////////////////////////////////
void _TYA(void)
{
	 regA = regY;
	 SETNSTATUS(regA);
	 SETZSTATUS(regA);
}

