
#ifndef __DLPIUTILS_H
#define __DLPIUTILS_H

#import <dlpi.h>
#import <tihdr.h>

#ifdef __cplusplus
extern "C" {
#endif

#define dumpM_PROTO
#define dumpM_DATA
#define dumpDetails
#define dumpHeaders

#if defined(dumpM_PROTO) || defined(dumpM_DATA) || defined(dumpDetails)
#define dumpingAnything
#endif

typedef struct IPHeader {
	// IP datagram header (without options)

	UInt8		versionAndHeaderLen;	// upper 4 bits = version
										// lower 4 bits = header length (# of longs)
	UInt8		typeOfService;
	UInt16		totalLength;
	UInt16		id;
	UInt16		offset;
	UInt8		timeToLive;
	UInt8		protocol;				// always kTCPProtocol
	UInt16		headerChecksum;
	UInt32		sourceAddr;
	UInt32		destAddr;

} IPHeader;

typedef struct TCPPacket {
	IPHeader	ipHeader;
	
	// TCP header (without options)
	UInt16		sourcePort;
	UInt16		destPort;
	UInt32		sequenceNum;
	UInt32		acknowledgementNum;
	UInt8		tcpOffset;
	UInt8		flags;
	UInt16		windowSize;
	UInt16		checksum;
	UInt16		urgentOffset;

} TCPPacket, *TCPPacketPtr;

typedef struct UDPPacket {

	IPHeader	ipHeader;

	// UDP header

	UInt16		sourcePort;
	UInt16		destPort;
	UInt16		udpLength;
	UInt16		udpChecksum;

} UDPPacket, *UDPPacketPtr;


const char *TypeToString( UInt32 type );
const char *ErrorAckCodeToString( UInt32 code );
const char *PrimitiveToString( UInt32 primitive );

void	DLPIDump_DL_INFO_ACK(dl_info_ack_t *ack);
void	DLPIDump_DL_BIND_REQ(dl_bind_req_t *req);
void	DLPIDump_DL_BIND_ACK(dl_bind_ack_t *ack);
void	DLPIDump_DL_SUBS_BIND_REQ(dl_subs_bind_req_t *req);
void	DLPIDump_DL_SUBS_BIND_ACK(dl_subs_bind_ack_t *req);
void	DLPIDump_DL_OK_ACK(dl_ok_ack_t *ack);
void	DLPIDump_DL_ENABMULTI_REQ(dl_enabmulti_req_t *ack);
void	DLPIDump_DL_UNITDATA_REQ(dl_unitdata_req_t *ack);
void	DLPIDump_DL_UNITDATA_IND(dl_unitdata_ind_t *ack);


void	TIDump_T_INFO_ACK(T_info_ack *ack);
void	TIDump_T_BIND_REQ(T_bind_req *ack);
void	TIDump_T_BIND_ACK(T_bind_ack *ack);
void	TIDump_T_CONN_CON(T_conn_con *ack);
void	TIDump_T_CONN_IND(T_conn_ind *ack);
void	TIDump_T_CONN_REQ(T_conn_req *ack);
void	TIDump_T_CONN_RES(T_conn_res *ack);
void	TIDump_T_OPTMGMT_ACK(T_optmgmt_ack *ack);
void	TIDump_T_OPTMGMT_REQ(T_optmgmt_req *ack);

void	MIDump_M_SETOPTS( struct stroptions *options );
void	MIDump_M_IOCTL( struct iocblk *block);

void	Dump_Packet_Type( char *type, mblk_t *mp );
void	Dump_Packet( mblk_t* );
long	GetPrimitive(mblk_t* mp);


typedef struct
{
	unsigned long dl_primitive; /* Set to kOTSetRecvMode */
	unsigned long dl_flags;
	unsigned long dl_truncation_length;
} dl_recv_control_t;

#define DL_NORMAL_STATUS	0x1
#define DL_ERROR_STATUS		0x2
#define DL_TRUNCATED_PACKET	0x4
#define DL_VERSION			0x0
#define DL_VERSION_BITS		0xF0000000

#ifdef __cplusplus
}
#endif
 
#endif
