// ActionLocus.cp

#ifndef ActionLocus_h
#include "ActionLocus.h"
#endif
#ifndef Action_h
#include "Action.h"
#endif

ActionLocus::ActionLocus( Focus& focus )
  : CommandHandler< Undoing >( focus ),
	 action( &Action::Irreversible() ),
	 done( true )
  {
  }

bool ActionLocus::CanUndo() const
  {
	return done && action->CanUndo();
  }

void ActionLocus::Undo()
  {
	Assert( done );
	action->Undo();
	done = false;
  }

bool ActionLocus::CanRedo() const
  {
	return !done && action->CanRedo();
  }

void ActionLocus::Redo()
  {
	Assert( !done );
	action->Redo();
	done = true;
  }

void ActionLocus::SetAction( bool isDone,
									  ConstStr255Param theName,
									  Action& theAction )
  {
	CommitAction();
	done = isDone;
	name = theName;
	action = &theAction;
  }

void ActionLocus::CommitAction()
  {
	Action *oldAction = action;
	action = &Action::Irreversible();
	oldAction->Commit( done );
  }
