// AEDescriptor.cp

#ifndef AEDescriptor_h
#include "AEDescriptor.h"
#endif
#ifndef Assert_h
#include "Assert.h"
#endif
#ifndef OSError_h
#include "OSError.h"
#endif
#ifndef ConstData_h
#include "ConstData.h"
#endif

AEDescriptor::AEDescriptor()
  {
	descriptorType = typeNull;
	dataHandle = 0;
  }

AEDescriptor::AEDescriptor( const AEDescriptor& source )
  {
	ThrowOSError( AEDuplicateDesc( &source, this ) );
  }

AEDescriptor::AEDescriptor( ConstData data, AEType type )
  {
	ThrowOSError( AECreateDesc( type.Type(),
										 data.Start(),
										 data.Length(),
										 this ) );
  }

AEDescriptor::AEDescriptor( const AEDescriptor& source, AEType type )
  {
	ThrowOSError( AECoerceDesc( &source,
										 type.Type(),
										 this ) );
  }

AEDescriptor::~AEDescriptor()
  {
	/*
		Ordinarily, I'd force the user to dispose
		the descriptor explicitly, to keep the
		destructor from throwing an error.  But
		there's no error I know of from this call
		that doesn't just mean "you're hosed."
	*/
	
	if ( Allocated() )
		OSError::Debug( AEDisposeDesc( this ) );
  }

void AEDescriptor::Get( AEType desiredType, Data result ) const
  {
	if ( Type() == desiredType )
	  {
		Assert( result.Length() == Value().Size() );
		result << Value().Data();
	  }
	 else
	  {
		AEDescriptor coerced( *this, desiredType );
		Assert( desiredType == coerced.Type() );
		Assert( result.Length() == coerced.Value().Size() );
		result << coerced.Value().Data();		
	  }
  }

void AEDescriptor::Dispose()
  {
	Assert( Allocated() );
	ThrowOSError( AEDisposeDesc( this ) );
	
	descriptorType = typeNull;
	dataHandle = 0;
  }

void AEDescriptor::Swap( AEDescriptor& target )
  {
	AEDesc temp = target;
	static_cast< AEDesc& >( target ) = *this;
	static_cast< AEDesc& >( *this ) = temp;
  }
