// AEEvent.h

#ifndef AEEvent_h
#define AEEvent_h

#ifndef AEDescriptor_h
#include "AEDescriptor.h"
#endif
#ifndef AEParameter_h
#include "AEParameter.h"
#endif
#ifndef AEAttribute_h
#include "AEAttribute.h"
#endif
#ifndef AEVerb_h
#include "AEVerb.h"
#endif
#ifndef AEList_h
#include "AEList.h"
#endif

class AEAddress;
class AEStruct;
class AEReplyHandler;
class AETransaction;

class AEEvent: public AEDescriptor
  {
	private:
		static pascal Boolean IgnoreEvents( EventRecord *, long *, RgnHandle * );

	public:
		AEEvent()	{}
		explicit AEEvent( const AEEvent& source )					: AEDescriptor( source )	{}
		AEEvent( AEVerb, const AEAddress& );
		AEEvent( AESuite, AEEventID, const AEAddress& );
		
		// not yet implemented:
			AEEvent( AEVerb, const AEAddress&, AEReplyHandler& );
			AEEvent( AEVerb, const AEAddress&, AETransaction& );
			AEEvent( AEVerb, const AEAddress&, AEReplyHandler&, AETransaction& );
		
		explicit AEEvent( const AEDescriptor& item )				: AEDescriptor( item, StaticType() )	{}
		explicit AEEvent( const ConstAEListItem& item )			: AEDescriptor( item, StaticType() )	{}
		explicit AEEvent( const ConstAEMember& item )			: AEDescriptor( item, StaticType() )	{}
		explicit AEEvent( const ConstAEParameter& item )		: AEDescriptor( item, StaticType() )	{}
		explicit AEEvent( const ConstAEAttribute& item )		: AEDescriptor( item, StaticType() )	{}
		
		static AEType StaticType()										{ return AEType( typeAppleEvent ); }
		bool TypeIsCorrect() const										{ return Type() == StaticType(); }
		
		uint32 ParameterCount() const;
		
		const ConstAEParameter Parameter( AEKey i ) const				{ return ConstAEParameter( *this, i ); }
		AEParameter Parameter( AEKey i )										{ return AEParameter( *this, i ); }
		
		const ConstAEListItem ParameterByNumber( uint32 i ) const	{ return ConstAEListItem( AsList(), i ); }
		AEListItem ParameterByNumber( uint32 i )							{ return AEListItem( AsList(), i ); }
		
		const ConstAEAttribute Attribute( AEKey i ) const				{ return ConstAEAttribute( *this, i ); }
		AEAttribute Attribute( AEKey i )										{ return AEAttribute( *this, i ); }
		
		AEVerb Verb() const;
		
		bool MissedParameter() const;
		void ThrowMissedParameter() const;
		
		static AEIdleUPP IgnoreIncomingEvents();
  };

#endif
