// ConstAEAttribute.cp

#ifndef ConstAEAttribute_h
#include "ConstAEAttribute.h"
#endif
#ifndef OSError_h
#include "OSError.h"
#endif
#ifndef Buffer_h
#include "Buffer.h"
#endif
#ifndef AEEvent_h
#include "AEEvent.h"
#endif

#include <Errors.h>

bool ConstAEAttribute::Exists() const
  {
	Assert( !Event().IsNull() );
	DescType type;
	int32 size;
	
	OSErr error = AESizeOfAttribute( &event, key.Key(), &type, &size );
	
	if ( error != noErr && error != errAEDescNotFound )
		throw OSError( error );
	
	return error == noErr;
  }

uint32 ConstAEAttribute::Length() const
  {
	Assert( !Event().IsNull() );
	DescType type;
	int32 size;
	ThrowOSError( AESizeOfAttribute( &event, key.Key(), &type, &size ) );
	Assert( size >= 0 );
	return size;
  }

AEType ConstAEAttribute::Type() const
  {
	Assert( !Event().IsNull() );
	DescType type;
	int32 size;
	ThrowOSError( AESizeOfAttribute( &event, key.Key(), &type, &size ) );
	return AEType( type );
  }

void ConstAEAttribute::operator>>( Data out ) const
  {
	Assert( !Event().IsNull() );
	DescType type;
	int32 size;
	ThrowOSError( AEGetAttributePtr( &event,
												key.Key(),
												typeWildCard,
												&type,
												out.Start(),
												out.Length(),
												&size ) );
  }

void ConstAEAttribute::operator>>( Buffer& out ) const
  {
	Assert( !Event().IsNull() );
	DescType type;
	int32 size;
	ThrowOSError( AEGetAttributePtr( &event,
												key.Key(),
												typeWildCard,
												&type,
												out.Unused().Start(),
												out.Unused().Length(),
												&size ) );
	Assert( size >= 0 );
	out.AdvanceMark( size );
  }

void ConstAEAttribute::Get( AEType desiredType, Data out ) const
  {
	Assert( !Event().IsNull() );
	DescType actualType;
	int32 actualSize;
	ThrowOSError( AEGetAttributePtr( &event,
												key.Key(),
												desiredType.Type(),
												&actualType,
												out.Start(),
												out.Length(),
												&actualSize ) );

	Assert( actualType == desiredType.Type() );
	Assert( actualSize == out.Length() );
  }

AEDescriptor::AEDescriptor( const ConstAEAttribute& attribute, AEType desired )
  {
	Assert( !attribute.Event().IsNull() );
	ThrowOSError( AEGetAttributeDesc( &attribute.Event(),
												 attribute.Key().Key(),
												 desired.Type(),
												 this ) );
  }
