// ConstAEParameter.cp

#ifndef ConstAEParameter_h
#include "ConstAEParameter.h"
#endif
#ifndef OSError_h
#include "OSError.h"
#endif
#ifndef Buffer_h
#include "Buffer.h"
#endif
#ifndef AEEvent_h
#include "AEEvent.h"
#endif

#include <Errors.h>

bool ConstAEParameter::Exists() const
  {
	Assert( !Event().IsNull() );
	DescType type;
	int32 size;
	
	OSErr error = AESizeOfParam( &event, key.Key(), &type, &size );
	
	if ( error != noErr && error != errAEDescNotFound )
		throw OSError( error );
	
	return error == noErr;
  }

uint32 ConstAEParameter::Length() const
  {
	Assert( !Event().IsNull() );
	DescType type;
	int32 size;
	ThrowOSError( AESizeOfParam( &event, key.Key(), &type, &size ) );
	Assert( size >= 0 );
	return size;
  }

AEType ConstAEParameter::Type() const
  {
	Assert( !Event().IsNull() );
	DescType type;
	int32 size;
	ThrowOSError( AESizeOfParam( &event, key.Key(), &type, &size ) );
	return AEType( type );
  }

void ConstAEParameter::operator>>( Data out ) const
  {
	Assert( !Event().IsNull() );
	DescType type;
	int32 size;
	ThrowOSError( AEGetParamPtr( &event,
										  key.Key(),
										  typeWildCard,
										  &type,
										  out.Start(),
										  out.Length(),
										  &size ) );
  }

void ConstAEParameter::operator>>( Buffer& out ) const
  {
	Assert( !Event().IsNull() );
	DescType type;
	int32 size;
	ThrowOSError( AEGetParamPtr( &event,
										  key.Key(),
										  typeWildCard,
										  &type,
										  out.Unused().Start(),
										  out.Unused().Length(),
										  &size ) );
	Assert( size >= 0 );
	out.AdvanceMark( size );
  }

void ConstAEParameter::Get( AEType desiredType, Data out ) const
  {
	Assert( !Event().IsNull() );
	DescType actualType;
	int32 actualSize;
	ThrowOSError( AEGetParamPtr( &event,
										  key.Key(),
										  desiredType.Type(),
										  &actualType,
										  out.Start(),
										  out.Length(),
										  &actualSize ) );

	Assert( actualType == desiredType.Type() );
	Assert( actualSize == out.Length() );
  }

AEDescriptor::AEDescriptor( const ConstAEParameter& parameter, AEType desired )
  {
	Assert( !parameter.Event().IsNull() );
	ThrowOSError( AEGetParamDesc( &parameter.Event(),
											parameter.Key().Key(),
											desired.Type(),
											this ) );
  }
