// AppleEventLink.cp

#ifndef AppleEventLink_h
#include "AppleEventLink.h"
#endif
#ifndef OSError_h
#include "OSError.h"
#endif
#ifndef Assert_h
#include "Assert.h"
#endif
#ifndef AppleEventMethodBase_h
#include "AppleEventMethodBase.h"
#endif

#include <Errors.h>

AppleEventLink::AppleEventLink( AEVerb theVerb, AppleEventMethodBase& method )
  : verb( theVerb )
  {
	#if GENERATINGCFM
		static RoutineDescriptor HandleAppleEvent =
			BUILD_ROUTINE_DESCRIPTOR( uppAEEventHandlerProcInfo,
											  &AppleEventLink::HandleAppleEvent );
	#endif

	ThrowOSError( AEInstallEventHandler( verb.Suite().Suite(),
													 verb.Event(),
													 &HandleAppleEvent,
													 reinterpret_cast<uint32>( &method ),
													 false ) );
  }

AppleEventLink::~AppleEventLink()
  {
	OSError::Debug( AERemoveEventHandler( verb.Suite().Suite(),
													  verb.Event(),
													  0,
													  false ) );
  }

pascal OSErr AppleEventLink::HandleAppleEvent( const AppleEvent *event,
															  AppleEvent *reply,
															  int32 theMethod )
  {
	Assert( event != 0 );
	Assert( reply != 0 );
	AppleEventMethodBase *method =
		reinterpret_cast<AppleEventMethodBase*>( theMethod );
	Assert( method != 0 );
	
	try
	  {
		(*method)( *reinterpret_cast< const AEEvent* >( event ),
					  *reinterpret_cast< AEEvent* >( reply ) );
	  }
	 catch ( OSError error )
	  {
		OSErr result = error;
		error.Fix();
		return result;
	  }
	 catch (...)
	  {
		Assert( 0 );
		return errAEEventFailed;
	  }
	
	return noErr;
  }

