// ConstAEListItem.cp

#ifndef ConstAEListItem_h
#include "ConstAEListItem.h"
#endif
#ifndef OSError_h
#include "OSError.h"
#endif
#ifndef AEList_h
#include "AEList.h"
#endif
#ifndef Buffer_h
#include "Buffer.h"
#endif

#include <Errors.h>

bool ConstAEListItem::Exists() const
  {
	Assert( !List().IsNull() );
	DescType type;
	int32 size;
	
	OSErr error = AESizeOfNthItem( &list, index, &type, &size );
	
	if ( error != noErr && error != errAEDescNotFound )
		throw OSError( error );
	
	return error == noErr;
  }

uint32 ConstAEListItem::Length() const
  {
	Assert( !List().IsNull() );
	DescType type;
	int32 size;
	ThrowOSError( AESizeOfNthItem( &list, index, &type, &size ) );
	Assert( size >= 0 );
	return size;
  }

AEType ConstAEListItem::Type() const
  {
	Assert( !List().IsNull() );
	DescType type;
	int32 size;
	ThrowOSError( AESizeOfNthItem( &list, index, &type, &size ) );
	return AEType( type );
  }

AEKey ConstAEListItem::Key() const
  {
	Assert( !List().IsNull() );
	AEKeyword key;
	DescType type;
	int32 size;
	ThrowOSError( AEGetNthPtr( &list,
										index,
										typeWildCard,
										&key,
										&type,
										0,
										0,
										&size ) );
	return AEKey( key );
  }

void ConstAEListItem::operator>>( Data out ) const
  {
	Assert( !List().IsNull() );
	AEKeyword key;
	DescType type;
	int32 size;
	ThrowOSError( AEGetNthPtr( &list,
										index,
										typeWildCard,
										&key,
										&type,
										out.Start(),
										out.Length(),
										&size ) );
  }

void ConstAEListItem::operator>>( Buffer& out ) const
  {
	Assert( !List().IsNull() );
	AEKeyword key;
	DescType type;
	int32 size;
	ThrowOSError( AEGetNthPtr( &list,
										index,
										typeWildCard,
										&key,
										&type,
										out.Unused().Start(),
										out.Unused().Length(),
										&size ) );
	Assert( size >= 0 );
	out.AdvanceMark( size );
  }

void ConstAEListItem::Get( AEType desiredType, Data out ) const
  {
	Assert( !List().IsNull() );
	AEKeyword key;
	DescType actualType;
	int32 actualSize;
	ThrowOSError( AEGetNthPtr( &list,
										index,
										desiredType.Type(),
										&key,
										&actualType,
										out.Start(),
										out.Length(),
										&actualSize ) );

	Assert( actualType == desiredType.Type() );
	Assert( actualSize == out.Length() );
  }

AEDescriptor::AEDescriptor( const ConstAEListItem& item, AEType desired )
  {
	Assert( !item.List().IsNull() );
	AEKeyword key;
	ThrowOSError( AEGetNthDesc( &item.List(),
										 item.Index(),
										 desired.Type(),
										 &key,
										 this ) );
  }
