// AppleEventRecorderList.cp

#ifndef AppleEventRecorderList_h
#include "AppleEventRecorderList.h"
#endif
#ifndef AppleEventRecorder_h
#include "AppleEventRecorder.h"
#endif
#ifndef AEEvent_h
#include "AEEvent.h"
#endif
#ifndef OSError_h
#include "OSError.h"
#endif

AppleEventRecorderList::AppleEventRecorderList()
  : notifyRecording( kCoreEventClass,
							kAENotifyRecording,
							*this,
							&AppleEventRecorderList::RecordEvent ),
	 thisProcess( Process::Application() )
  {
  }

AppleEventRecorderList::~AppleEventRecorderList()
  {
	if ( !recorders.IsEmpty() )
		StopRecording();
  }

void AppleEventRecorderList::StartRecording()
  {
	static const AEEvent startEvent( kCoreEventClass, kAEStartRecording, thisProcess );					

	AEDescriptor reply;
	ThrowOSError( AESend( &startEvent,
								 &reply,
								 kAEWaitReply,
								 kAENormalPriority,
								 kAEDefaultTimeout,
								 AEEvent::IgnoreIncomingEvents(),
								 0 ) );	// filterProc
  }

void AppleEventRecorderList::StopRecording()
  {
	static const AEEvent stopEvent( kCoreEventClass, kAEStopRecording, thisProcess );
	
	AEDescriptor reply;
	DebugOSError( AESend( &stopEvent,
								 &reply,
								 kAEWaitReply,
								 kAENormalPriority,
								 kAEDefaultTimeout,
								 AEEvent::IgnoreIncomingEvents(),
								 0 ) );	// filterProc
  }

void AppleEventRecorderList::RecordEvent( const AEEvent& event,
														AEEvent& /*reply*/ )
  {
	for ( ListLoop<AppleEventRecorder> recorder( recorders );
			recorder.Unfinished();
			recorder++ )
		recorder->Record( event );
  }

AppleEventRecorderList& AppleEventRecorderList::The()
  {
	static AppleEventRecorderList the;
	return the;
  }

void AppleEventRecorderList::Add( AppleEventRecorder& toAdd )
  {
	if ( !recorders.IsEmpty() )
		StopRecording();
	
	recorders.Add( toAdd.link, afterEnd );

	StartRecording();
  }

void AppleEventRecorderList::Remove( AppleEventRecorder& toRemove )
  {
	StopRecording();
	
	recorders.Remove( toRemove.link );

	if ( !recorders.IsEmpty() )
		StartRecording();
  }

void AppleEventRecorderList::RemoveWithoutFlushing( AppleEventRecorder& toRemove )
  {
	recorders.Remove( toRemove.link );

	if ( recorders.IsEmpty() )
		StopRecording();
  }

void AppleEventRecorderList::Flush()
  {
	if ( !recorders.IsEmpty() )
	  {
		StopRecording();
		StartRecording();
	  }
  }
