// EventHandler.cp

#ifndef EventHandler_h
#include "EventHandler.h"
#endif
#ifndef NullEvent_h
#include "NullEvent.h"
#endif
#ifndef MouseDownEvent_h
#include "MouseDownEvent.h"
#endif
#ifndef MouseUpEvent_h
#include "MouseUpEvent.h"
#endif
#ifndef MouseMovedEvent_h
#include "MouseMovedEvent.h"
#endif
#ifndef KeyDownEvent_h
#include "KeyDownEvent.h"
#endif
#ifndef KeyUpEvent_h
#include "KeyUpEvent.h"
#endif
#ifndef AutoKeyEvent_h
#include "AutoKeyEvent.h"
#endif
#ifndef ActivateEvent_h
#include "ActivateEvent.h"
#endif
#ifndef DeactivateEvent_h
#include "DeactivateEvent.h"
#endif
#ifndef UpdateEvent_h
#include "UpdateEvent.h"
#endif
#ifndef SuspendEvent_h
#include "SuspendEvent.h"
#endif
#ifndef ResumeEvent_h
#include "ResumeEvent.h"
#endif
#ifndef DiskInsertionEvent_h
#include "DiskInsertionEvent.h"
#endif
#ifndef HighLevelEvent_h
#include "HighLevelEvent.h"
#endif
#ifndef SplitIntegers_h
#include "SplitIntegers.h"
#endif
#ifndef Assert_h
#include "Assert.h"
#endif

void EventHandler::HandleEvent( const EventRecord& event )
  {
	switch( event.what )
	  {
		case nullEvent:			HandleNullEvent( event );				break;
		case mouseDown:			HandleMouseDownEvent( event );		break;
		case mouseUp:				HandleMouseUpEvent( event );			break;
		case keyDown:				HandleKeyDownEvent( event );			break;
		case keyUp:					HandleKeyUpEvent( event );				break;
		case autoKey:				HandleAutoKeyEvent( event );			break;
		case updateEvt:			HandleUpdateEvent( event );			break;
		case diskEvt:				HandleDiskInsertionEvent( event );	break;
		case kHighLevelEvent:	HandleHighLevelEvent( event );		break;
		case osEvt:					HandleOSEvent( event );					break;

		case activateEvt:
			if ( event.modifiers & activeFlag )
				HandleActivateEvent( event );
			 else
				HandleDeactivateEvent( event );
			break;
		
		default:
			Assert( 0 );
			break;
	  }
  }

void EventHandler::HandleOSEvent( const EventRecord& event )
  {
	Assert( event.what == osEvt );
	
	switch ( Byte3( event.message ) )
	  {
		case suspendResumeMessage:
			if ( (event.message & resumeFlag) != 0 )
				HandleResumeEvent( event );
			 else
				HandleSuspendEvent( event );
			break;
			
		case mouseMovedMessage:
			HandleMouseMovedEvent( event );
			break;
			
		default:
			Assert( 0 );
			break;
	  }
  }
