// MouseEventHandler.cp

#ifndef MouseEventHandler_h
#include "MouseEventHandler.h"
#endif
#ifndef MouseWatcher_h
#include "MouseWatcher.h"
#endif
#ifndef ClickCounter_h
#include "ClickCounter.h"
#endif
#ifndef UserCancelledError_h
#include "UserCancelledError.h"
#endif
#ifndef MouseDownEvent_h
#include "MouseDownEvent.h"
#endif
#ifndef MouseUpEvent_h
#include "MouseUpEvent.h"
#endif
#ifndef MouseMovedEvent_h
#include "MouseMovedEvent.h"
#endif
#ifndef ScreenPart_h
#include "ScreenPart.h"
#endif
#ifndef UserState_h
#include "UserState.h"
#endif

void MouseEventHandler::HandleMouseDownEvent( const MouseDownEvent& event )
  {
	MouseWatcher::Report( event );
	ClickCounter::Record( event );
	
	try
	  {
		switch ( event.ScreenPart() )
		  {
			case ScreenPart::desktop:				HandleClickInDesktop( event );			break;
			case ScreenPart::menuBar:				HandleClickInMenuBar( event );			break;
			case ScreenPart::systemWindow:		HandleClickInSystemWindow( event );		break;
			case ScreenPart::windowContent:		HandleClickInWindowContent( event );	break;
			case ScreenPart::windowDragRegion:	HandleClickInDragRegion( event );		break;
			case ScreenPart::windowGrowBox:		HandleClickInGrowBox( event );			break;
			case ScreenPart::windowCloseBox:		HandleClickInCloseBox( event );			break;
			case ScreenPart::windowZoomInBox:	HandleClickInZoomIn( event );				break;
			case ScreenPart::windowZoomOutBox:	HandleClickInZoomOut( event );			break;

			default:
				Assert( 0 );
				break;
		  }
	  }
	 catch ( const UserCancelledError& )
	  {
	  }
	
	UserState::The().Announce();
  }

void MouseEventHandler::HandleMouseUpEvent( const MouseUpEvent& event )
  {
	MouseWatcher::Report( event );
	ClickCounter::Record( event );
  }

void MouseEventHandler::HandleMouseMovedEvent( const MouseMovedEvent& event )
  {
	MouseWatcher::Report( event );
  }
