// ClickCounter.cp

#ifndef ClickCounter_h
#include "ClickCounter.h"
#endif
#ifndef MouseUpEvent_h
#include "MouseUpEvent.h"
#endif
#ifndef Assert_h
#include "Assert.h"
#endif
#ifndef MouseDownEvent_h
#include "MouseDownEvent.h"
#endif

bool ClickCounter::uncountedClickExists = false;
Tick ClickCounter::mouseUpTime = Tick( 0 );
ClickCounter *ClickCounter::lastClicked = 0;

ClickCounter::~ClickCounter()
  {
	if ( lastClicked == this )
		lastClicked = 0;
  }

void ClickCounter::Record( const MouseDownEvent& )
  {
	if ( uncountedClickExists )
		lastClicked = 0;
	
	uncountedClickExists = true;
  }

void ClickCounter::Record( const MouseUpEvent& event )
  {
	if ( uncountedClickExists )
		lastClicked = 0;
	
	mouseUpTime = event.When();
  }

void ClickCounter::Count( const MouseDownEvent& event,
								  bool closeEnough )
  {
	Assert( uncountedClickExists );
	uncountedClickExists = false;
	
	if ( lastClicked == this
		  && closeEnough
		  && event.When() - mouseUpTime < GetDblTime() )
	  {
		Assert( count < maxuint32 );
		count++;
	  }
	 else
		count = 1;
	
	lastClicked = this;
  }

void ClickCounter::TooFarAway()
  {
	Assert( uncountedClickExists );
	Assert( lastClicked == this );
	Assert( count > 0 );
	count = 1;
  }
