// ScrapWriter.cp

#ifndef ScrapWriter_h
#include "ScrapWriter.h"
#endif
#ifndef Assert_h
#include "Assert.h"
#endif
#ifndef MemoryFullError_h
#include "MemoryFullError.h"
#endif
#ifndef GlobalClipboard_h
#include "GlobalClipboard.h"
#endif

#include <Scrap.h>

ScrapWriter::ScrapWriter()
  : approved( false )
  {
	GlobalClipboard::Invalidate();
	ThrowError( ZeroScrap() );
  }

ScrapWriter::~ScrapWriter()
  {
	if ( approved )
		GlobalClipboard::MarkValid();
	 else
	  {
		GlobalClipboard::Invalidate();
	 	OSError::Debug( OSErr( ZeroScrap() ) );
	  }
  }

void ScrapWriter::Write( ScrapType type, ConstData data )
  {
	ThrowError( PutScrap( int32(data.Length()), type.Value(), data.Start() ) );
  }

void ScrapWriter::ThrowError( int32 error )
  {
	if ( error == noErr )
		return;
	
	switch ( error )
	  {
		case memFullErr:			throw MemoryFullError( OSErr(error) );
	  }
	
	throw OSError( OSErr(error) );
  }
