// ControlIdler.cp

#ifndef ControlIdler_h
#include "ControlIdler.h"
#endif
#ifndef ControlView_h
#include "ControlView.h"
#endif
#ifndef ValidCanvasLoop_h
#include "ValidCanvasLoop.h"
#endif
#ifndef Window_h
#include "Window.h"
#endif

ControlIdler::ControlIdler( ControlView& theView,
									 bool startEnabled )
  : Enableable( startEnabled ),
	 link( this ),
	 view( theView ),
	 started( false )
  {
	if ( startEnabled )
		ControlIdler::BeEnabled();
  }

ControlIdler::~ControlIdler()
  {
	if ( Enabled() )
		ControlIdler::BeDisabled();		
  }

void ControlIdler::BeEnabled()
  {
	view.OwningWindow().ControlIdlers().Add( link );
  }

void ControlIdler::BeDisabled()
  {
	view.OwningWindow().ControlIdlers().Remove( link );
  }

void ControlIdler::PrepareToIdle( RegionObject& displayArea )
  {
	Assert( !started );
	
	ValidCanvasLoop canvas( view );
	
	for ( ; canvas.Unfinished(); canvas++ )
	  {
		if ( !canvas->HasWindow() )
			continue;
		
		if ( &canvas->Window() != &view.OwningWindow().WindowObject() )
		  {
			canvas->Invalidate();
			continue;
		  }
		
		displayArea |= canvas->Clip();
		view.PrepareToDraw( *canvas );
		started = true;
		break;
	  }
	
	if ( canvas.Unfinished() )
		for ( canvas++; canvas.Unfinished(); canvas++ )
			if ( canvas->HasWindow() )
				canvas->Invalidate();
  }

void ControlIdler::StopIdling()
  {
	if ( started )
	  {
		view.StopDrawing();
		started = false;
	  }
  }
