// ControlObject.h

#ifndef ControlObject_h
#define ControlObject_h

#ifndef ControlDefinition_h
#include "ControlDefinition.h"
#endif
#ifndef ConstPString_h
#include "ConstPString.h"
#endif
#ifndef Str_h
#include "Str.h"
#endif
#ifndef Rectangle_h
#include "Rectangle.h"
#endif
#ifndef GrafPortObject_h
#include "GrafPortObject.h"
#endif

class WindowObject;

class ControlObject
  {
	private:
		ControlHandle control;
	
		// not implemented:
			ControlObject( const ControlObject& );
			void operator=( const ControlObject& );
			
	public:
		ControlObject( WindowObject&,
							ControlDefinition definition,
							const Rect& bounds,
							ConstPString title,
							bool visible,
							int16 minimum, int16 value, int16 maximum,
							uint32 refCon = 0 );

		~ControlObject();
		
		operator ControlHandle() const			{ return control; }

		int16 Value() const							{ return GetControlValue( control ); }
		void SetValue( int16 v );
		
		int16 Minimum() const						{ return GetControlMinimum( control ); }
		void SetMinimum( int16 m );
		
		int16 Maximum() const						{ return GetControlMaximum( control ); }
		void SetMaximum( int16 m );

		void SetRangeAndValue( int16 minimum,
									  int16 value,
									  int16 maximum );

		void GetTitle( String255& s )				{ GetControlTitle( control, s ); }
		void SetTitle( ConstPString s );
		
		void Draw() const								{ DrawControlInCurrentPort( control ); }

		bool Visible() const							{ return IsControlVisible( control ); }
		void Show()										{ SetControlVisibility( control, true, false ); }
		void Hide()										{ SetControlVisibility( control, false, false ); }

		bool Active() const							{ return IsControlActive( control ); }
		void Activate()								{ ActivateControl( control ); }
		void Deactivate()								{ DeactivateControl( control ); }
		
		void Highlight( int16 part )				{ HiliteControl( control, part ); }
		void Unhighlight()							{ HiliteControl( control, 0 ); }

		void Enable()									{ HiliteControl( control, 0 ); }
		void Disable()									{ HiliteControl( control, 255 ); }
		
		Rectangle Bounds() const					{ return (*control)->contrlRect; }
		PointObject RecommendedSize() const;
		
		void MoveTo( PointObject p )				{ MoveControl( control, p.h, p.v ); }
		void SetSize( PointObject size )			{ SizeControl( control, size.h, size.v ); }
		void SetBounds( const Rectangle& );
		
		int32 RefCon() const							{ return GetControlReference( control ); }
		void SetRefCon( int32 c )					{ SetControlReference( control, c ); }
		
		int16 Test( PointObject p ) const		{ return TestControl( control, p ); }
		int16 Track( PointObject start, ControlActionUPP action );
  };

#endif
