// ControlPart.cp

#ifndef ControlPart_h
#include "ControlPart.h"
#endif
#ifndef Functor_h
#include "Functor.h"
#endif
#ifndef __APPEARANCE__
#include <Appearance.h>
#endif
#ifndef MouseDownEvent_h
#include "MouseDownEvent.h"
#endif
#ifndef ControlDrawingContext_h
#include "ControlDrawingContext.h"
#endif

ControlPart *ControlPart::beingTracked = 0;

ControlPart::ControlPart( ControlView& theView,
								  int16 theCode,
								  Functor< void(*)() > *trackMethod,
								  Functor< void(*)() > *hitMethod )
  : view( theView ),
	 code( theCode ),
	 track( trackMethod ),
	 hit( hitMethod ),
	 link( this )
  {
	Assert( code > 0 );
	Assert( code < 254 );
	theView.parts.Add( link, afterEnd );
  }
	
void ControlPart::Click( const Canvas& canvas,
								 const MouseDownEvent& event )
  {
	beingTracked = this;
	
	view.PrepareToDraw( canvas );
	int16 hitPart = view.control.Track( event.LocalPoint(), Action() );
	view.StopDrawing();
	
	if ( hitPart == Code() && hit != 0 )
		(*hit)();
	
	beingTracked = 0;
  }

void ControlPart::Track()
  {
	if ( track != 0 )
		(*track)();
  }

void ControlPart::Highlight()
  {
	ControlDrawingContext context( view );
	view.control.Highlight( code );
  }

void ControlPart::Unhighlight()
  {
	ControlDrawingContext context( view );
	view.control.Unhighlight();
  }
