// ControlView.h

#ifndef ControlView_h
#define ControlView_h

#ifndef View_h
#include "View.h"
#endif
#ifndef ViewCell_h
#include "ViewCell.h"
#endif
#ifndef ConformsToPaneSize_h
#include "ConformsToPaneSize.h"
#endif
#ifndef DrawsSpontaneously_h
#include "DrawsSpontaneously.h"
#endif
#ifndef Activator_h
#include "Activator.h"
#endif
#ifndef Str_h
#include "Str.h"
#endif
#ifndef ControlDefinition_h
#include "ControlDefinition.h"
#endif
#ifndef ListOf_h
#include "ListOf.h"
#endif
#ifndef ControlObject_h
#include "ControlObject.h"
#endif

class Window;

class ControlView: virtual public View,
						 virtual public ViewCell,
						 virtual public ConformsToPaneSize,
						 virtual public DrawsSpontaneously,
						 public Activator
  {
	friend class ControlPart;
	friend class ControlThumb;
	friend class ControlNonThumb;
	friend class ControlIdler;
	
	private:
		ControlObject control;
		const Rectangle viewInset;
		Window& window;
		ListOf<ControlPart> parts;
		uint32 visibility;
		
		static const Rect defaultBounds;
		
	protected:
		virtual void PaneSizeChanged();

		ControlObject& Control()						{ return control; }
		const ControlObject& Control() const		{ return control; }
		
		ControlPart *FindPart( int16 number ) const;

		int16 Value() const								{ return control.Value(); }
		int16 Minimum() const							{ return control.Minimum(); }
		int16 Maximum() const							{ return control.Maximum(); }

		void SetValue( int16 v );
		void SetMinimum( int16 m );
		void SetMaximum( int16 m );

		void SetRangeAndValue( int16 minimum,
									  int16 value,
									  int16 maximum );

		void GetTitle( String255& s )					{ control.GetTitle( s ); }
		void SetTitle( ConstPString s );

		PointObject RecommendedSize() const;
		
	public:
		ControlView( Window&,
						 ControlDefinition definition,
						 ConstPString title,
						 int16 minimum, int16 value, int16 maximum,
						 Rectangle viewInset = Rectangle::zero );
		
		ControlView( Window&,
						 ControlDefinition definition,
						 ConstPString title,
						 int16 magicMinimum, int16 magicValue, int16 magicMaximum,
						 int16 minimum, int16 value, int16 maximum,
						 Rectangle viewInset = Rectangle::zero );
		
		~ControlView();
		
		Window& OwningWindow() const					{ return window; }
		void PrepareToDraw( const Canvas& );
		void StopDrawing();
		
		virtual void Draw( const Canvas& ) const;
		
		virtual void Click( const Canvas&,
								  const MouseDownEvent& );
		
		virtual void Activate();
		virtual void Deactivate();

		void Enable();
		void Disable();
  };

#endif
