// ProgressBar.cp

#ifndef ProgressBar_h
#include "ProgressBar.h"
#endif
#ifndef __APPEARANCE__
#include <Appearance.h>
#endif
#ifndef Window_h
#include "Window.h"
#endif
#ifndef Progress_h
#include "Progress.h"
#endif
#ifndef ControlDrawingContext_h
#include "ControlDrawingContext.h"
#endif

ProgressBar::ProgressBar( Window& theWindow, const Progress& theTarget )
  : ControlView( theWindow,
  					  ControlDefinition( kControlProgressBarProc ),
  					  "\p",
  					  0, 0, largeValue ),
  	 window( theWindow ),
  	 target( theTarget ),
  	 indeterminate( false ),
	 spin( *this, false ),
	 updateValue( theTarget, *this, &ProgressBar::UpdateValue )
  {
	UpdateValue();
  }

void ProgressBar::CheckIndeterminacy()
  {
	if ( indeterminate == !target.TotalKnown() )
		return;
	
	indeterminate = !target.TotalKnown();
	
	Boolean isIndeterminate = indeterminate;
	SetControlData( Control(),
						 0,
						 kControlProgressBarIndeterminateTag,
						 sizeof( isIndeterminate ),
						 reinterpret_cast<Ptr>( &isIndeterminate ) );
	Invalidate();
	spin.SetEnabled( indeterminate );
  }

void ProgressBar::UpdateValue()
  {
	CheckIndeterminacy();

	if ( !indeterminate )
		SetValue( target.Fraction() * double(largeValue) + 0.5 );
  }

int32 ProgressBar::BestWidth( Range32 bounds ) const
  {
	return bounds.WeaklyRestrict( 190 );
  }

int32 ProgressBar::MinimumWidth() const
  {
	return 20;
  }

int32 ProgressBar::MinimumHeight() const
  {
	return 14;
  }

int32 ProgressBar::MaximumWidth() const
  {
	return maxint32;
  }

int32 ProgressBar::MaximumHeight() const
  {
	return 14;
  }
