// PushButton.cp

#ifndef PushButton_h
#include "PushButton.h"
#endif
#ifndef __APPEARANCE__
#include <Appearance.h>
#endif
#ifndef Canvas_h
#include "Canvas.h"
#endif

PushButton::PushButton( Window& window,
								Functor< void(*)() >& toAnnounce,
								ConstPString title,
								Rectangle viewInset )
  : ControlView( window,
  					  ControlDefinition( kControlPushButtonProc ),
  					  title,
  					  0, 0, 0,
  					  viewInset ),
	 button( *this, kControlButtonPart, 0, &toAnnounce ),
	 announce( toAnnounce ),
	 iconWidth( 0 )
  {
  }

PushButton::PushButton( Window& window,
								Functor< void(*)() >& toAnnounce,
								ResourceID cicn,
								ConstPString title,
								Rectangle viewInset )
  : ControlView( window,
  					  ControlDefinition( kControlPushButLeftIconProc ),
  					  title,
  					  cicn.Value(), cicn.Value(), cicn.Value(),
  					  viewInset ),
	 button( *this, kControlCheckBoxPart /* ?! */, 0, &toAnnounce ),
	 announce( toAnnounce ),
	 iconWidth( 16 )
  {
  }

PushButton::PushButton( Window& window,
								Functor< void(*)() >& toAnnounce,
								ConstPString title,
								ResourceID cicn,
								Rectangle viewInset )
  : ControlView( window,
  					  ControlDefinition( kControlPushButRightIconProc ),
  					  title,
  					  cicn.Value(), cicn.Value(), cicn.Value(),
  					  viewInset ),
	 button( *this, kControlCheckBoxPart /* ?! */, 0, &toAnnounce ),
	 announce( toAnnounce ),
	 iconWidth( 16 )
  {
  }

void PushButton::Draw( const Canvas& canvas ) const
  {
	EraseRect( &canvas.OnScreen() );
	ControlView::Draw( canvas );
  }

void PushButton::FakeClick()
  {
	uint32 stopHighlighting = TickCount() + 8;
	
	button.Highlight();
	announce();
	
	int32 delay = stopHighlighting - TickCount();
	
	if ( delay > 0 )
	  {
		uint32 ignored;
		Delay( delay, &ignored );
	  }
	
	button.Unhighlight();
  }

int32 PushButton::BestWidth( Range32 bounds ) const
  {
	return bounds.WeaklyRestrict( RecommendedSize().h + iconWidth );
  }

int32 PushButton::MinimumHeight() const
  {
	return RecommendedSize().v;
  }

int32 PushButton::MinimumWidth() const
  {
	return RecommendedSize().h + iconWidth;
  }

int32 PushButton::MaximumHeight() const
  {
	return RecommendedSize().v;
  }
