// ScrollBar.cp

#ifndef ScrollBar_h
#include "ScrollBar.h"
#endif
#ifndef ScrollScaler_h
#include "ScrollScaler.h"
#endif
#ifndef BackgroundColorMaintainer_h
#include "BackgroundColorMaintainer.h"
#endif
#ifndef Canvas_h
#include "Canvas.h"
#endif
#ifndef NamedColors_h
#include "NamedColors.h"
#endif

ScrollBar::ScrollBar( Window& window,
							 Functor< void(*)() > *goBackOneLine,
							 Functor< void(*)() > *goBackOnePage,
							 Functor< void(*)() > *goForwardOnePage,
							 Functor< void(*)() > *goForwardOneLine )
  : ControlView( window,
  					  ControlDefinition( kControlScrollBarLiveProc ),
  					  "\p",
  					  0, 0, 0,
  					  Rectangle( -1, -1, 1, 1 ) ),
  	
	 backArrow(    *this, kControlUpButtonPart,   goBackOneLine,    0 ),
	 pageBack(     *this, kControlPageUpPart,     goBackOnePage,    0 ),
	 pageForward(  *this, kControlPageDownPart,   goForwardOnePage, 0 ),
	 forwardArrow( *this, kControlDownButtonPart, goForwardOneLine, 0 ),

	 thumbMoved( *this, &ScrollBar::ThumbMoved ),
	 thumb( *this, kControlIndicatorPart, &thumbMoved, 0 )
  {
  }

void ScrollBar::ThumbMoved()
  {
	int16 value = Value();
	ScrollScaler scaler( Scaler() );
	
	if ( scaler.BarValueFor( ScrollPosition() ) == value )
		return;
	
	ScrollTo( scaler.ScrollRangeFor( value ) );
  }

void ScrollBar::UpdateValue()
  {
	ScrollScaler scaler( Scaler() );
	SetRangeAndValue( 0,
							scaler.BarValueFor( ScrollPosition() ),
							scaler.MaximumBarValue() );
  }

void ScrollBar::Draw( const Canvas& canvas ) const
  {
	if ( Active() )
		ControlView::Draw( canvas );
	 else
	  {
		BackgroundColorMaintainer bg( NamedColors::appleGray1 );
		EraseRect( &canvas.OnScreen() );
	  }
  }

void ScrollBar::Activate()
  {
	Invalidate();
  }

void ScrollBar::Deactivate()
  {
	Invalidate();
  }

void ScrollBar::PaneSizeChanged()
  {
	UpdateValue();
  }
