// ScrollScaler.cp

#ifndef ScrollScaler_h
#include "ScrollScaler.h"
#endif
#ifndef MinMax_h
#include "MinMax.h"
#endif
#ifndef Overflow_h
#include "Overflow.h"
#endif

ScrollScaler::ScrollScaler( uint32 revealedSize,
									 uint32 panoramaSize,
									 uint32 theQuantum,
									 uint32 resolution,
									 uint32 currentScroll )
  : quantum( theQuantum )
  {
	Assert( resolution < maxint16 );
	Assert( theQuantum > 0 );
	
	maximumScroll = ( panoramaSize > revealedSize )
						 ? panoramaSize - revealedSize
						 : 0;
	
	if ( currentScroll > maximumScroll )
		maximumScroll = currentScroll;
	
	maximumScroll += quantum - 1;
	maximumScroll -= maximumScroll % quantum;

	Assert( currentScroll <= maximumScroll );

	maximumValue =  Min( maximumScroll / quantum, resolution );
  }

int16 ScrollScaler::BarValueFor( uint32 scroll ) const
  {
	Assert( scroll <= maximumScroll );
	
	if ( scroll == 0 )
		return 0;
	
	if ( scroll >= maximumScroll )
		return maximumValue;

 	return Asint16( uint64( scroll - 1 ) * ( maximumValue - 1 ) 
 					    / ( maximumScroll - 1 )
 					    + 1 );
  }

uint32 ScrollScaler::LeastScrollFor( int16 barValue ) const
  {
	Assert( barValue >= 0 );
	Assert( uint32(barValue) <= maximumValue );

	if ( barValue == 0 )
		return 0;
	
	if ( uint32(barValue) >= maximumValue )
		return maximumScroll;

 	return ( uint64( barValue - 1 ) * ( maximumScroll - 1 ) + maximumValue - 2 )
 		  / ( maximumValue - 1 )
 		  + 1;
  }

URange32 ScrollScaler::ScrollRangeFor( int16 barValue ) const
  {
	uint32 low = LeastScrollFor( barValue );
	
	uint32 high = ( barValue < maximumValue )
					  ? LeastScrollFor( barValue + 1 ) - 1
					  : maximumScroll;

	Assert( BarValueFor( low ) == barValue );
	Assert( BarValueFor( high ) == barValue );
	Assert( low + quantum - 1 <= high );

	low += quantum - 1;
	low -= low % quantum;
	high -= high % quantum;

	Assert( BarValueFor( low ) == barValue );
	Assert( BarValueFor( high ) == barValue );
	Assert( low <= high );
	
	return URange32( low, high );
  }
