// ScrollScaler.h

#ifndef ScrollScaler_h
#define ScrollScaler_h

#ifndef Range_h
#include "Range.h"
#endif

/*
	This class implements a rather complicated algorithm needed in scrolling.
	It's for scaling scroll values to scroll bar values, allowing for these
	constraints:
	
		There are enough scroll positions to accomodate scrolling to the end
			of the document, or to the current position, whichever is larger.
		
		A scroll bar will be given no more values than its resolution,
			and no more values than there are scroll positions.
		
		Only the first scroll position maps to the first bar poisition;
			only the last scroll position maps to the last bar position.
		
		Other scroll positions are divided evenly along the bar.
	
	These rules ensure that there is exactly one value for a given position;
	BarValueFor gives that value.
	
	On the other hand, one value can map to a range of positions.
	ScrollRangeFor goves that range, rounded inward to the nearest quantum.
*/

class ScrollScaler
  {
	private:
		uint32 maximumValue;
		uint32 maximumScroll;
		uint32 quantum;
	
		uint32 LeastScrollFor( int16 barValue ) const;
		
	public:
		ScrollScaler( uint32 revealedSize,
						  uint32 panoramaSize,
						  uint32 quantum,
						  uint32 resolution,
						  uint32 currentScroll );
		
		int16 BarValueFor( uint32 scrollAmount ) const;
		int16 MaximumBarValue() const								{ return maximumValue; }
		
		URange32 ScrollRangeFor( int16 barValue ) const;
  };

#endif
