// VerticalScrollBar.cp

#ifndef VerticalScrollBar_h
#include "VerticalScrollBar.h"
#endif
#ifndef Scrolling_h
#include "Scrolling.h"
#endif
#ifndef ScrollScaler_h
#include "ScrollScaler.h"
#endif

VerticalScrollBar::VerticalScrollBar( Window& window,
												  Scrolling& scrolling )
  : ScrollBar( window,
  					&backOneLine,
  					&backOnePage,
  					&forwardOnePage,
  					&forwardOneLine ),
	 target( scrolling ),
	 
	 backOneLine(    scrolling, &Scrolling::ScrollUpOneLine ),
	 backOnePage(    scrolling, &Scrolling::ScrollUpOnePage ),
	 forwardOnePage( scrolling, &Scrolling::ScrollDownOnePage ),
	 forwardOneLine( scrolling, &Scrolling::ScrollDownOneLine ),
	 
	 updateValue( scrolling.ScrollingAnnouncer(),
	 				  *this,
	 				  &ScrollBar::UpdateValue )
  {
  }

uint32 VerticalScrollBar::Resolution() const
  {
	if ( !PaneSizeKnown() )
		return 1;
	
	return PaneSize().v - 45;
	/* In theory, this is the number of positions the thumb
		on the control bar can take.  With theme switching,
		this may not be the exact value, but it's probably
		still close, and in particular won't undershoot by much.
	*/
  }

ScrollScaler VerticalScrollBar::Scaler() const
  {
	Rectangle32 revealed( target.RevealedArea() );
	Assert( revealed.top >= 0 );
	
	return ScrollScaler( revealed.Height(),
								target.PanoramaSize().v,
								target.ScrollingQuantum().v,
								Resolution(),
								revealed.top );
  }

uint32 VerticalScrollBar::ScrollPosition() const
  {
	return target.RevealedArea().top;
  }

void VerticalScrollBar::ScrollTo( URange32 range )
  {
	Assert( range.Start() <= maxint32 );
	Assert( range.End() <= maxint32 );
	
	target.ScrollTopTo( range.IsEmpty()
								 ? range.Start()
								 : target.BestVerticalPosition( Range32( range.Start(),
								 													  range.End() ) ) );
  }

int32 VerticalScrollBar::MinimumHeight() const
  {
	return 64;
  }

int32 VerticalScrollBar::MinimumWidth() const
  {
	return 16;
  }

int32 VerticalScrollBar::MaximumWidth() const
  {
	return 16;
  }
