// GraphicsDeviceObject.h

#ifndef GraphicsDeviceObject_h
#define GraphicsDeviceObject_h

#ifndef Rectangle_h
#include "Rectangle.h"
#endif
#ifndef __PALETTES__
#include <Palettes.h>
#endif

class GraphicsDeviceObject
  {
	private:
		GDHandle device;
		
	public:
		GraphicsDeviceObject( GDHandle theDevice )
		  : device( theDevice )
		  {
			Assert( device == 0 || *device != 0 );
		  }
		
		bool Null() const											{ return device == 0; }
		
		operator GDHandle() const								{ return device; }
		GDHandle Device() const									{ return device; }
		
		PixMapHandle PixMap() const							{ Assert( !Null() ); return (*device)->gdPMap; }
		
		int16 Driver() const										{ Assert( !Null() ); return (*device)->gdRefNum; }
		
		Rectangle Bounds() const								{ Assert( !Null() ); return (*device)->gdRect; }
		
		bool InColor() const										{ Assert( !Null() ); return TestDeviceAttribute( device, gdDevType ); }
		bool HasBlockTransfer() const							{ Assert( !Null() ); return TestDeviceAttribute( device, burstDevice ); }
		bool Needs32BitMode() const							{ Assert( !Null() ); return TestDeviceAttribute( device, ext32Device ); }
		bool InitializedFromRam() const						{ Assert( !Null() ); return TestDeviceAttribute( device, ramInit ); }
		bool IsMainScreen() const								{ Assert( !Null() ); return TestDeviceAttribute( device, mainScreen ); }
		bool AllFromResources() const							{ Assert( !Null() ); return TestDeviceAttribute( device, allInit ); }
		bool IsScreen() const									{ Assert( !Null() ); return TestDeviceAttribute( device, screenDevice ); }
		bool WithoutDriver() const								{ Assert( !Null() ); return TestDeviceAttribute( device, noDriver ); }
		bool ScreenActive() const								{ Assert( !Null() ); return TestDeviceAttribute( device, screenActive ); }
		
		int16 Depth() const										{ Assert( !Null() ); return (*PixMap())->pixelSize; }
		
		bool HasDepth( uint16 depth, bool inColor ) const
			{ Assert( !Null() ); return ::HasDepth( device, int16(depth), gdDevType, inColor ) != 0; }
		
		//void SetDepth( uint16 depth, bool inColor ) const
		//	{ Assert( !Null() ); ::SetDepth( device, depth, gdDevType, inColor ); }
			// it's not a depth; it's a mode returned by HasDepth!
		
		static GraphicsDeviceObject First()									{ return GetDeviceList(); }
		GraphicsDeviceObject Next() const									{ Assert( !Null() ); return GetNextDevice( device ); }

		static GraphicsDeviceObject Current()								{ return GetGDevice(); }
		static GraphicsDeviceObject Main()									{ return GetMainDevice(); }
		static GraphicsDeviceObject Deepest( const Rectangle& r )	{ return GetMaxDevice( &r ); }
		
		static GraphicsDeviceObject Nearest( const Rectangle& );
  };

#endif
