// Point32.cp

#ifndef Point32_h
#include "Point32.h"
#endif
#ifndef MinMax_h
#include "MinMax.h"
#endif
#ifndef Assert_h
#include "Assert.h"
#endif
#ifndef Sign_h
#include "Sign.h"
#endif
#ifndef Overflow_h
#include "Overflow.h"
#endif
#ifndef PointObject_h
#include "PointObject.h"
#endif
#ifndef UPoint32_h
#include "UPoint32.h"
#endif

const Point32 Point32::zero( 0, 0 );
const Point32 Point32::up( 0, minint32 );
const Point32 Point32::down( 0, maxint32 );
const Point32 Point32::left( minint32, 0 );
const Point32 Point32::right( maxint32, 0 );
const Point32 Point32::upLeft( minint32, minint32 );
const Point32 Point32::upRight( maxint32, minint32 );
const Point32 Point32::downLeft( minint32, maxint32 );
const Point32 Point32::downRight( maxint32, maxint32 );

Point32::Point32( UPoint32 p )
  : h( p.h ),
	 v( p.v )
  {
	Assert( p.h <= maxint32 );
	Assert( p.v <= maxint32 );
  }

void Point32::BeAboveLeftOf( Point32 r )
  {
	h = Min( h, r.h );
	v = Min( v, r.v );
  }

void Point32::BeBelowLeftOf( Point32 r )
  {
	h = Min( h, r.h );
	v = Max( v, r.v );
  }

void Point32::BeAboveRightOf( Point32 r )
  {
	h = Max( h, r.h );
	v = Min( v, r.v );
  }

void Point32::BeBelowRightOf( Point32 r )
  {
	h = Max( h, r.h );
	v = Max( v, r.v );
  }

void Point32::operator+=( Point32 r )
  {
	Assert( CanAdd( h, r.h ) );
	Assert( CanAdd( v, r.v ) );
	h += r.h;
	v += r.v;
  }

void Point32::operator-=( Point32 r )
  {
	Assert( CanSubtract( h, r.h ) );
	Assert( CanSubtract( v, r.v ) );
	h -= r.h;
	v -= r.v;
  }

void Point32::operator*=( int32 n )
  {
	Assert( CanMultiply( h, n ) );
	Assert( CanMultiply( v, n ) );
	h *= n;
	v *= n;
  }

void Point32::operator/=( int32 n )
  {
	Assert( n != 0 );
	h /= n;
	v /= n;
  }

const Point32 Point32::operator+( Point32 r ) const
  {
	Assert( CanAdd( h, r.h ) );
	Assert( CanAdd( v, r.v ) );
	return Point32( h + r.h, v + r.v );
  }

const Point32 Point32::operator-( Point32 r ) const
  {
	Assert( CanSubtract( h, r.h ) );
	Assert( CanSubtract( v, r.v ) );
	return Point32( h - r.h, v - r.v );
  }

const Point32 Point32::operator*( int32 n ) const
  {
	Assert( CanMultiply( h, n ) );
	Assert( CanMultiply( v, n ) );
	return Point32( h * n, v * n );
  }

const Point32 Point32::operator/( int32 n ) const
  {
	Assert( n != 0 );
	return Point32( h / n, v / n );
  }

const PointObject Point32::NearestPoint16() const
  {
	return PointObject( (h < minint16)
								? minint16
								: ( h > maxint16 ) ? maxint16 : h,
								(v < minint16)
								? minint16
								: ( v > maxint16 ) ? maxint16 : v );
  }

const PointObject Point32::AsPointObject() const
  {
	return PointObject( Asint16( h ), Asint16(v) );
  }
