// Rectangle32.cp

#ifndef Rectangle32_h
#include "Rectangle32.h"
#endif
#ifndef MinMax_h
#include "MinMax.h"
#endif
#ifndef Overflow_h
#include "Overflow.h"
#endif
#ifndef Rectangle_h
#include "Rectangle.h"
#endif

const Rectangle32 Rectangle32::big( minint32, minint32, maxint32, maxint32 );
const Rectangle32 Rectangle32::zero( 0, 0, 0, 0 );

Rectangle32::Rectangle32( Point32 p1, Point32 p2 )
  {
	if ( p1.h <= p2.h )
	  {
		left = p1.h;
		right = p2.h;
	  }
	 else
	  {
		left = p2.h;
		right = p1.h;
	  }
	
	if ( p1.v <= p2.v )
	  {
		top = p1.v;
		bottom = p2.v;
	  }
	 else
	  {
		top = p2.v;
		bottom = p1.v;
	  }
  }

bool Rectangle32::Contains( Point32 p ) const
  {
	return left <= p.h && p.h < right
		 && top <= p.v && p.v < bottom;
  }

bool Rectangle32::StrictlyContains( Point32 p ) const
  {
	return left < p.h && p.h < right
		 && top < p.v && p.v < bottom;
  }

bool Rectangle32::WeaklyContains( Point32 p ) const
  {
	return left <= p.h && p.h <= right
		 && top <= p.v && p.v <= bottom;
  }

bool Rectangle32::operator==( Rectangle32 r ) const
  {
	return left == r.left && right == r.right 
		 && top == r.top && bottom == r.bottom;
  }

bool Rectangle32::operator<=( Rectangle32 r ) const
  {
	return left >= r.left && right <= r.right 
		 && top >= r.top && bottom <= r.bottom;
  }

bool Rectangle32::operator>=( Rectangle32 r ) const
  {
	return left <= r.left && right >= r.right 
		 && top <= r.top && bottom >= r.bottom;
  }

void Rectangle32::operator&=( Rectangle32 r )
  {
	left = Max( left, r.left );
	top = Max( top, r.top );
	right = Min( right, r.right );
	bottom = Min( bottom, r.bottom );
	
	if ( IsEmpty() )
		*this = Rectangle32( left, top, left, top );
  }

void Rectangle32::operator|=( Rectangle32 r )
  {
	left = Min( left, r.left );
	top = Min( top, r.top );
	right = Max( right, r.right );
	bottom = Max( bottom, r.bottom );
  }

const Rectangle32 Rectangle32::operator&( Rectangle32 r ) const
  {
	Rectangle32 result( *this );
	result &= r;
	return result;
  }

const Rectangle32 Rectangle32::operator|( Rectangle32 r ) const
  {
	Rectangle32 result( *this );
	result |= r;
	return result;
  }

bool Rectangle32::Intersects( Rectangle32 r ) const
  {
	if ( left >= r.right || r.left >= right )
		return false;
	
	if ( top >= r.bottom || r.top >= bottom )
		return false;
	
	return true;
  }

bool Rectangle32::Touches( Rectangle32 r ) const
  {
	if ( left > r.right || r.left > right )
		return false;
	
	if ( top > r.bottom || r.top > bottom )
		return false;
	
	return true;
  }

uint32 Rectangle32::SharedSides( Rectangle32 r ) const
  {
	return ( ( left   == r.left   ) ? 1u : 0u )
		  + ( ( top    == r.top    ) ? 1u : 0u )
		  + ( ( right  == r.right  ) ? 1u : 0u )
		  + ( ( bottom == r.bottom ) ? 1u : 0u );
  }

void Rectangle32::operator+=( Rectangle32 r )
  {
	Assert( SharedSides( r ) == 3 );
	Assert( !Intersects( r ) );
	*this |= r;
  }

void Rectangle32::operator-=( Rectangle32 r )
  {
	Assert( SharedSides( r ) == 3 );
	Assert( *this > r );
	if ( left < r.left )
		right = r.left;
	if ( top < r.top )
		bottom = r.top;
	if ( right > r.right )
		left = r.right;
	if ( bottom > r.bottom )
		top = r.bottom;
  }

const Rectangle32 Rectangle32::operator+( Rectangle32 r ) const
  {
	Rectangle32 result( *this );
	result += r;
	return result;
  }

const Rectangle32 Rectangle32::operator-( Rectangle32 r ) const
  {
	Rectangle32 result( *this );
	result -= r;
	return result;
  }

void Rectangle32::Inset( int32 amount )
  {
	Assert( CanAdd( top, amount ) );
	Assert( CanAdd( left, amount ) );
	Assert( CanSubtract( bottom, amount ) );
	Assert( CanSubtract( right, amount ) );

	Assert( top + amount <= bottom - amount );
	Assert( left + amount <= right - amount );

	top += amount;
	left += amount;
	bottom -= amount;
	right -= amount;
  }

void Rectangle32::Inset( Point32 amount )
  {
	Assert( CanAdd( top, amount.v ) );
	Assert( CanAdd( left, amount.h ) );
	Assert( CanSubtract( bottom, amount.v ) );
	Assert( CanSubtract( right, amount.h ) );

	Assert( top + amount.v <= bottom - amount.v );
	Assert( left + amount.h <= right - amount.h );

	top += amount.v;
	left += amount.h;
	bottom -= amount.v;
	right -= amount.h;
  }

void Rectangle32::Inset( Rectangle32 amount )
  {
	Assert( CanSubtract( top, amount.top ) );
	Assert( CanSubtract( left, amount.left ) );
	Assert( CanSubtract( bottom, amount.bottom ) );
	Assert( CanSubtract( right, amount.right ) );

	Assert( top - amount.top <= bottom - amount.bottom );
	Assert( left - amount.left <= right - amount.right );

	top -= amount.top;
	left -= amount.left;
	bottom -= amount.bottom;
	right -= amount.right;
  }

void Rectangle32::Outset( int32 amount )
  {
	Assert( CanSubtract( top, amount ) );
	Assert( CanSubtract( left, amount ) );
	Assert( CanAdd( bottom, amount ) );
	Assert( CanAdd( right, amount ) );

	Assert( top - amount <= bottom + amount );
	Assert( left - amount <= right + amount );

	top -= amount;
	left -= amount;
	bottom += amount;
	right += amount;
  }

void Rectangle32::Outset( Point32 amount )
  {
	Assert( CanSubtract( top, amount.v ) );
	Assert( CanSubtract( left, amount.h ) );
	Assert( CanAdd( bottom, amount.v ) );
	Assert( CanAdd( right, amount.h ) );

	Assert( top - amount.v <= bottom + amount.v );
	Assert( left - amount.h <= right + amount.h );

	top -= amount.v;
	left -= amount.h;
	bottom += amount.v;
	right += amount.h;
  }

void Rectangle32::Outset( Rectangle32 amount )
  {
	Assert( CanAdd( top, amount.top ) );
	Assert( CanAdd( left, amount.left ) );
	Assert( CanAdd( bottom, amount.bottom ) );
	Assert( CanAdd( right, amount.right ) );

	Assert( top + amount.top <= bottom + amount.bottom );
	Assert( left + amount.left <= right + amount.right );

	top += amount.top;
	left += amount.left;
	bottom += amount.bottom;
	right += amount.right;
  }

void Rectangle32::operator+=( Point32 vector )
  {
	Assert( CanAdd( top, vector.v ) );
	Assert( CanAdd( left, vector.h ) );
	Assert( CanAdd( bottom, vector.v ) );
	Assert( CanAdd( right, vector.h ) );
	top += vector.v;
	left += vector.h;
	bottom += vector.v;
	right += vector.h;
  }

void Rectangle32::operator-=( Point32 vector )
  {
	Assert( CanSubtract( top, vector.v ) );
	Assert( CanSubtract( left, vector.h ) );
	Assert( CanSubtract( bottom, vector.v ) );
	Assert( CanSubtract( right, vector.h ) );
	top -= vector.v;
	left -= vector.h;
	bottom -= vector.v;
	right -= vector.h;
  }

const Rectangle32 Rectangle32::operator+( Point32 vector ) const
  {
	Rectangle32 result( *this );
	result += vector;
	return result;
  }

const Rectangle32 Rectangle32::operator-( Point32 vector ) const
  {
	Rectangle32 result( *this );
	result -= vector;
	return result;
  }

const UPoint32 Rectangle32::Size() const
  {
	return UPoint32( Width(), Height() );
  }

const Rectangle Rectangle32::AsRectangle() const
  {
	return Rectangle( Asint16( left ),
							Asint16( top ),
							Asint16( right ),
							Asint16( bottom ) );
  }
