// Rectangle32.h

#ifndef Rectangle32_h
#define Rectangle32_h

#ifndef Point32_h
#include "Point32.h"
#endif
#ifndef UPoint32_h
#include "UPoint32.h"
#endif
#ifndef Range_h
#include "Range.h"
#endif

class Rectangle;

class Rectangle32
  {
	public:
		int32 left;
		int32 top;
		int32 right;
		int32 bottom;
		
		Rectangle32()														{}
		Rectangle32( int32 l, int32 t, int32 r, int32 b )		: left(l), top(t), right(r), bottom(b)			{}
		Rectangle32( Point32 p1, Point32 p2 );
		Rectangle32( const Rect& r )			: left(r.left), top(r.top), right(r.right), bottom(r.bottom)	{}
		
		static const Rectangle32 big;
		static const Rectangle32 zero;
		
		const Point32 TopLeft() const							{ return Point32( left, top ); }
		const Point32 TopRight() const						{ return Point32( right, top ); }
		const Point32 BottomLeft() const						{ return Point32( left, bottom ); }
		const Point32 BottomRight() const					{ return Point32( right, bottom ); }
		
		void SetTopLeft( Point32 p )							{ left = p.h; top = p.v; }
		void SetTopRight( Point32 p )							{ right = p.h; top = p.v; }
		void SetBottomLeft( Point32 p )						{ left = p.h; bottom = p.v; }
		void SetBottomRight( Point32 p )						{ right = p.h; bottom = p.v; }

		bool IsEmpty() const										{ return left >= right || top >= bottom; }
		bool IsNormal() const									{ return left <= right && top <= bottom; }
		bool Contains( Point32 p ) const;
		bool StrictlyContains( Point32 p ) const;
		bool WeaklyContains( Point32 p ) const;
		
		bool operator==( Rectangle32 r ) const;
		bool operator!=( Rectangle32 r ) const				{ return !(*this == r); }
		bool operator<=( Rectangle32 r ) const;
		bool operator>=( Rectangle32 r ) const;
		bool operator<( Rectangle32 r ) const				{ return *this <= r && *this != r; }
		bool operator>( Rectangle32 r ) const				{ return *this >= r && *this != r; }

		void operator&=( Rectangle32 r );
		void operator|=( Rectangle32 r );
		const Rectangle32 operator&( Rectangle32 r ) const;
		const Rectangle32 operator|( Rectangle32 r ) const;

		bool Intersects( Rectangle32 r ) const;
		bool Touches( Rectangle32 r ) const;
		
		uint32 SharedSides( Rectangle32 r ) const;
		
		void operator+=( Rectangle32 r );
		void operator-=( Rectangle32 r );
		const Rectangle32 operator+( Rectangle32 r ) const;
		const Rectangle32 operator-( Rectangle32 r ) const;
		
		void Inset( int32 amount );
		void Inset( Point32 amount );
		void Inset( Rectangle32 amount );
		void Outset( int32 amount );
		void Outset( Point32 amount );
		void Outset( Rectangle32 amount );
		
		void operator+=( Point32 vector );
		void operator-=( Point32 vector );
		const Rectangle32 operator+( Point32 vector ) const;
		const Rectangle32 operator-( Point32 vector ) const;
		
		uint32 Height() const								{ return uint32( bottom - top ); }
		uint32 Width() const									{ return uint32( right - left ); }
		const UPoint32 Size() const;
		
		const Range32 Vertical() const					{ return Range32( top, bottom ); }
		const Range32 Horizontal() const					{ return Range32( left, right ); }
		
		const Rectangle AsRectangle() const;
  };

#endif
