// ThemeBackgroundMaintainer.cp

#ifndef ThemeBackgroundMaintainer_h
#include "ThemeBackgroundMaintainer.h"
#endif
#ifndef GrafPortObject_h
#include "GrafPortObject.h"
#endif
#ifndef GraphicsDeviceObject_h
#include "GraphicsDeviceObject.h"
#endif
#ifndef OSError_h
#include "OSError.h"
#endif
#ifndef NamedColors_h
#include "NamedColors.h"
#endif

const ThemeBackgroundMaintainer *ThemeBackgroundMaintainer::current = 0;

ThemeBackgroundMaintainer::ThemeBackgroundMaintainer( ThemeBrush theBrush,
																		const GraphicsDeviceObject& device )
  : port( GrafPortObject::Current() ),
	 oldColor( GrafPortObject::Current().BackgroundColor() ),
	 inColor( device.InColor() ),
	 depth( device.Depth() ),
	 brush( theBrush ),
	 hasBrush( true ),
	 previous( current )
  {
	SetThemeBackground();
	current = this;
  }

ThemeBackgroundMaintainer::ThemeBackgroundMaintainer( RGBColor toUse )
  : port( GrafPortObject::Current() ),
	 oldColor( GrafPortObject::Current().BackgroundColor() ),
	 hasBrush( false ),
	 previous( current )
  {
	port.SetBackgroundPattern( qd.white );
	port.SetBackgroundColor( toUse );
	current = this;
  }

ThemeBackgroundMaintainer::~ThemeBackgroundMaintainer()
  {
	current = previous;
	
	const ThemeBackgroundMaintainer *elder = previous;
	while ( elder != 0 && &elder->port != &port )
		elder = elder->previous;
	
	if ( elder != 0 && elder->hasBrush )
		elder->SetThemeBackground();
	 else
		port.SetBackgroundPattern( qd.white );

	port.SetBackgroundColor( oldColor );
  }

void ThemeBackgroundMaintainer::SetBrush( ThemeBrush theBrush,
														const GraphicsDeviceObject& device )
  {
	inColor = device.InColor();
	depth = device.Depth();
	brush = theBrush;
	hasBrush = true;
	SetThemeBackground();
  }

void ThemeBackgroundMaintainer::SetColor( RGBColor toUse )
  {
	hasBrush = false;
	port.SetBackgroundPattern( qd.white );
	port.SetBackgroundColor( toUse );
  }

void ThemeBackgroundMaintainer::SetThemeBackground() const
  {
	Assert( hasBrush );

	OSStatus error = ::SetThemeBackground( brush, depth, inColor );
	OSError::Debug( error );
	
	if ( error != noErr )
	  {
		port.SetBackgroundPattern( qd.white );
		port.SetBackgroundColor( NamedColors::white );
	  }
  }
