// ThemeBackgroundMaintainer.h

#ifndef ThemeBackgroundMaintainer_h
#define ThemeBackgroundMaintainer_h

#ifndef __APPEARANCE__
#include <Appearance.h>
#endif
#ifndef Integers_h
#include "Integers.h"
#endif

class GrafPortObject;
class GraphicsDeviceObject;

/*
	This class works differently from the other Maintainers.
	Because it's truly a pain to save & restore color patterns,
	this object looks back to the previous ThemeBackgroundMaintainer
	on the same port, and restores that theme.  If there was none,
	it sets the pattern to white.
	
	This means that ThemeBackgroundMaintainers don't interact well
	with other code that changes the background pattern.  To avoid
	similar problems with background color, it separately saves
	and restores the color.
*/

class ThemeBackgroundMaintainer
  {
	private:
		GrafPortObject& port;
		RGBColor oldColor;
		bool hasBrush;
			ThemeBrush brush;
			bool inColor;
			int16 depth;
		
		const ThemeBackgroundMaintainer *previous;
		
		static const ThemeBackgroundMaintainer *current;
		
		void SetThemeBackground() const;
		
		// not implemented:
			ThemeBackgroundMaintainer( const ThemeBackgroundMaintainer& );
			void operator=( const ThemeBackgroundMaintainer& );
		
	public:
		ThemeBackgroundMaintainer( ThemeBrush, const GraphicsDeviceObject& );
		ThemeBackgroundMaintainer( RGBColor );
		~ThemeBackgroundMaintainer();
		
		void SetBrush( ThemeBrush, const GraphicsDeviceObject& );
		void SetColor( RGBColor );
  };

#endif
