// ThemeForegroundMaintainer.cp

#ifndef ThemeForegroundMaintainer_h
#include "ThemeForegroundMaintainer.h"
#endif
#ifndef GrafPortObject_h
#include "GrafPortObject.h"
#endif
#ifndef GraphicsDeviceObject_h
#include "GraphicsDeviceObject.h"
#endif
#ifndef OSError_h
#include "OSError.h"
#endif
#ifndef NamedColors_h
#include "NamedColors.h"
#endif

const ThemeForegroundMaintainer *ThemeForegroundMaintainer::current = 0;

ThemeForegroundMaintainer::ThemeForegroundMaintainer( ThemeBrush theBrush,
																		const GraphicsDeviceObject& device )
  : port( GrafPortObject::Current() ),
	 oldColor( GrafPortObject::Current().ForegroundColor() ),
	 inColor( device.InColor() ),
	 depth( device.Depth() ),
	 brush( theBrush ),
	 hasBrush( true ),
	 previous( current )
  {
	SetThemePen();
	current = this;
  }

ThemeForegroundMaintainer::ThemeForegroundMaintainer( RGBColor toUse )
  : port( GrafPortObject::Current() ),
	 oldColor( GrafPortObject::Current().ForegroundColor() ),
	 hasBrush( false ),
	 previous( current )
  {
	port.SetPenPattern( qd.black );
	port.SetForegroundColor( toUse );
	current = this;
  }

ThemeForegroundMaintainer::~ThemeForegroundMaintainer()
  {
	current = previous;
	
	const ThemeForegroundMaintainer *elder = previous;
	while ( elder != 0 && &elder->port != &port )
		elder = elder->previous;
	
	if ( elder != 0 && elder->hasBrush )
		elder->SetThemePen();
	 else
		port.SetPenPattern( qd.black );

	port.SetForegroundColor( oldColor );
  }

void ThemeForegroundMaintainer::SetBrush( ThemeBrush theBrush,
														const GraphicsDeviceObject& device )
  {
	inColor = device.InColor();
	depth = device.Depth();
	brush = theBrush;
	hasBrush = true;
	SetThemePen();
  }

void ThemeForegroundMaintainer::SetColor( RGBColor toUse )
  {
	hasBrush = false;
	port.SetPenPattern( qd.black );
	port.SetForegroundColor( toUse );
  }

void ThemeForegroundMaintainer::SetThemePen() const
  {
	Assert( hasBrush );

	OSStatus error = ::SetThemePen( brush, depth, inColor );
	OSError::Debug( error );
	
	if ( error != noErr )
	  {
		port.SetPenPattern( qd.black );
		port.SetForegroundColor( NamedColors::black );
	  }
  }
