// ThemeForegroundMaintainer.h

#ifndef ThemeForegroundMaintainer_h
#define ThemeForegroundMaintainer_h

#ifndef __APPEARANCE__
#include <Appearance.h>
#endif
#ifndef Integers_h
#include "Integers.h"
#endif

class GrafPortObject;
class GraphicsDeviceObject;

/*
	This class works differently from the other Maintainers.
	Because it's truly a pain to save & restore color patterns,
	this object looks back to the previous ThemeForegroundMaintainer
	on the same port, and restores that theme.  If there was none,
	it sets the pattern to white.
	
	This means that ThemeForegroundMaintainers don't interact well
	with other code that changes the pen pattern.  To avoid
	similar problems with foreground color, it separately saves
	and restores the color.
*/

class ThemeForegroundMaintainer
  {
	private:
		GrafPortObject& port;
		RGBColor oldColor;
		bool hasBrush;
			ThemeBrush brush;
			bool inColor;
			int16 depth;

		const ThemeForegroundMaintainer *previous;
		
		static const ThemeForegroundMaintainer *current;
		
		void SetThemePen() const;
		
		// not implemented:
			ThemeForegroundMaintainer( const ThemeForegroundMaintainer& );
			void operator=( const ThemeForegroundMaintainer& );
		
	public:
		ThemeForegroundMaintainer( ThemeBrush, const GraphicsDeviceObject& );
		ThemeForegroundMaintainer( RGBColor );
		~ThemeForegroundMaintainer();
		
		void SetBrush( ThemeBrush, const GraphicsDeviceObject& );
		void SetColor( RGBColor );
  };

#endif
