// UPoint32.cp

#ifndef UPoint32_h
#include "UPoint32.h"
#endif
#ifndef MinMax_h
#include "MinMax.h"
#endif
#ifndef Assert_h
#include "Assert.h"
#endif
#ifndef Sign_h
#include "Sign.h"
#endif
#ifndef Overflow_h
#include "Overflow.h"
#endif
#ifndef Point32_h
#include "Point32.h"
#endif
#ifndef PointObject_h
#include "PointObject.h"
#endif

const UPoint32 UPoint32::zero( 0, 0 );
const UPoint32 UPoint32::down( 0, maxuint32 );
const UPoint32 UPoint32::right( maxuint32, 0 );
const UPoint32 UPoint32::downRight( maxuint32, maxuint32 );

UPoint32::UPoint32( Point32 p )
  : h( p.h ),
	 v( p.v )
  {
	Assert( p.h >= 0 );
	Assert( p.v >= 0 );
  }

void UPoint32::BeAboveLeftOf( UPoint32 r )
  {
	h = Min( h, r.h );
	v = Min( v, r.v );
  }

void UPoint32::BeBelowLeftOf( UPoint32 r )
  {
	h = Min( h, r.h );
	v = Max( v, r.v );
  }

void UPoint32::BeAboveRightOf( UPoint32 r )
  {
	h = Max( h, r.h );
	v = Min( v, r.v );
  }

void UPoint32::BeBelowRightOf( UPoint32 r )
  {
	h = Max( h, r.h );
	v = Max( v, r.v );
  }

void UPoint32::operator+=( UPoint32 r )
  {
	Assert( CanAdd( h, r.h ) );
	Assert( CanAdd( v, r.v ) );
	h += r.h;
	v += r.v;
  }

void UPoint32::operator-=( UPoint32 r )
  {
	Assert( CanSubtract( h, r.h ) );
	Assert( CanSubtract( v, r.v ) );
	h -= r.h;
	v -= r.v;
  }

void UPoint32::operator*=( uint32 n )
  {
	Assert( CanMultiply( h, n ) );
	Assert( CanMultiply( v, n ) );
	h *= n;
	v *= n;
  }

void UPoint32::operator/=( uint32 n )
  {
	Assert( n != 0 );
	h /= n;
	v /= n;
  }

const UPoint32 UPoint32::operator+( UPoint32 r ) const
  {
	Assert( CanAdd( h, r.h ) );
	Assert( CanAdd( v, r.v ) );
	return UPoint32( h + r.h, v + r.v );
  }

const UPoint32 UPoint32::operator-( UPoint32 r ) const
  {
	Assert( CanSubtract( h, r.h ) );
	Assert( CanSubtract( v, r.v ) );
	return UPoint32( h - r.h, v - r.v );
  }

const UPoint32 UPoint32::operator*( uint32 n ) const
  {
	Assert( CanMultiply( h, n ) );
	Assert( CanMultiply( v, n ) );
	return UPoint32( h * n, v * n );
  }

const UPoint32 UPoint32::operator/( uint32 n ) const
  {
	Assert( n != 0 );
	return UPoint32( h / n, v / n );
  }

const PointObject UPoint32::NearestPoint16() const
  {
	return PointObject( ( h > maxint16 ) ? maxint16 : h,
							  ( v > maxint16 ) ? maxint16 : v );
  }

const PointObject UPoint32::AsPointObject() const
  {
	return PointObject( Asint16( h ), Asint16(v) );
  }
