// URectangle32.h

#ifndef URectangle32_h
#define URectangle32_h

#ifndef Range_h
#include "Range.h"
#endif
#ifndef UPoint32_h
#include "UPoint32.h"
#endif

class URectangle32
  {
	public:
		uint32 left;
		uint32 top;
		uint32 right;
		uint32 bottom;
		
		URectangle32()															{}
		URectangle32( uint32 l, uint32 t, uint32 r, uint32 b )	: left(l), top(t), right(r), bottom(b)		{}
		URectangle32( UPoint32 p1, UPoint32 p2 );
		
		static const URectangle32 big;
		static const URectangle32 zero;
		
		const UPoint32 TopLeft() const						{ return UPoint32( left, top ); }
		const UPoint32 TopRight() const						{ return UPoint32( right, top ); }
		const UPoint32 BottomLeft() const					{ return UPoint32( left, bottom ); }
		const UPoint32 BottomRight() const					{ return UPoint32( right, bottom ); }
		
		void SetTopLeft( UPoint32 p )							{ left = p.h; top = p.v; }
		void SetTopRight( UPoint32 p )						{ right = p.h; top = p.v; }
		void SetBottomLeft( UPoint32 p )						{ left = p.h; bottom = p.v; }
		void SetBottomRight( UPoint32 p )					{ right = p.h; bottom = p.v; }

		bool IsEmpty() const										{ return left >= right || top >= bottom; }
		bool IsNormal() const									{ return left <= right && top <= bottom; }
		bool Contains( UPoint32 p ) const;
		bool StrictlyContains( UPoint32 p ) const;
		bool WeaklyContains( UPoint32 p ) const;
		
		bool operator==( URectangle32 r ) const;
		bool operator!=( URectangle32 r ) const			{ return !(*this == r); }
		bool operator<=( URectangle32 r ) const;
		bool operator>=( URectangle32 r ) const;
		bool operator<( URectangle32 r ) const				{ return *this <= r && *this != r; }
		bool operator>( URectangle32 r ) const				{ return *this >= r && *this != r; }

		void operator&=( URectangle32 r );
		void operator|=( URectangle32 r );
		const URectangle32 operator&( URectangle32 r ) const;
		const URectangle32 operator|( URectangle32 r ) const;

		bool Intersects( URectangle32 r ) const;
		bool Touches( URectangle32 r ) const;
		
		uint32 SharedSides( URectangle32 r ) const;
		
		void operator+=( URectangle32 r );
		void operator-=( URectangle32 r );
		const URectangle32 operator+( URectangle32 r ) const;
		const URectangle32 operator-( URectangle32 r ) const;
		
		void Inset( uint32 amount );
		void Inset( UPoint32 amount );
		void Inset( URectangle32 amount );
		void Outset( uint32 amount );
		void Outset( UPoint32 amount );
		void Outset( URectangle32 amount );
		
		void operator+=( UPoint32 vector );
		void operator-=( UPoint32 vector );
		const URectangle32 operator+( UPoint32 vector ) const;
		const URectangle32 operator-( UPoint32 vector ) const;
		
		uint32 Height() const								{ return bottom - top; }
		uint32 Width() const									{ return right - left; }
		const UPoint32 Size() const;
		
		const URange32 Vertical() const					{ return URange32( top, bottom ); }
		const URange32 Horizontal() const				{ return URange32( left, right ); }
  };

#endif
