// OSError.cp

#ifndef OSError_h
#include "OSError.h"
#endif
#ifndef Assert_h
#include "Assert.h"
#endif
#ifndef DebugMessage_h
#include "DebugMessage.h"
#endif

#include "ConfigureOSErrorDebugging.h"

#include <Errors.h>

OSError::OSError( OSError& e )
  : error( e.error )
  {
	e.Fix();
  }

OSError::~OSError()
  {
	if ( displayDestroyedErrors && !OK() && error != userCanceledErr )
		AnnounceDestruction( error );
  }

void OSError::operator=( OSStatus e )
  {
	if ( OK() )
		error = e;
	 else
	  {
		if ( displayCancelledErrors && e != noErr )
			AnnounceCancellation( e, error );
		
		if ( displayDuplicateErrors && e == noErr )
			AnnounceDuplication( e );
	  }
  }

void OSError::operator=( OSError& e )
  {
	if ( this == &e )
		return;
	
	*this = e.error;
	e.Fix();
  }

void OSError::Fix( OSStatus e )
  {
	Assert( e != noErr );
	if ( error == e )
		error = noErr;
  }

void OSError::Debug() const
  {
	if ( displayExplicitErrors && !OK() )
		AnnounceError( error );
  }

void OSError::Debug( OSStatus error )
  {
	if ( displayExplicitErrors && error != noErr )
		AnnounceError( error );
  }

void OSError::AnnounceCancellation( OSStatus lost, OSStatus kept )
  {
	DebugMessage message;
	message << "Error " << lost;
	message << " lost in favor of " << kept;
	message << ". ;error #" << lost;
	message << ";error #" << kept;
	message.Show();
  }

void OSError::AnnounceDuplication( OSStatus error )
  {
	DebugMessage message;
	message << "Error " << error;
	message << " duplicated. ;error #" << error;
	message.Show();
  }

void OSError::AnnounceDestruction( OSStatus error )
  {
	DebugMessage message;
	message << "Error " << error;
	message << " destroyed. ;error #" << error;
	message.Show();
  }

void OSError::AnnounceError( OSStatus error )
  {
	DebugMessage message;
	message << ";error #" << error;
	message.Show();
  }
