// OSError.h

#ifndef OSError_h
#define OSError_h

#ifndef Integers_h
#include "Integers.h"
#endif
#ifndef __MACTYPES__
#include <MacTypes.h>
#endif

class OSError
  {
	private:
		OSStatus error;
	
		static void AnnounceDuplication( OSStatus );
		static void AnnounceCancellation( OSStatus lost, OSStatus kept );
		static void AnnounceDestruction( OSStatus destroyed );
		static void AnnounceError( OSStatus );
		
	public:
		OSError()					: error( 0 )	{}
		OSError( OSStatus e )	: error( e )	{}
		OSError( OSError& e );
		
		~OSError();
		
		operator OSStatus() const				{ return error; }
		
		bool OK() const							{ return error == noErr; }
		bool operator==( OSStatus e ) const	{ return error == e; }
		
		void operator=( OSStatus );
		void operator=( OSError& );
		
		void Fix()									{ error = noErr; }
		void Fix( OSStatus );
		
		void Debug() const;
		static void Debug( OSStatus error );
  };

inline void ThrowOSError( OSStatus e )
  {
	if ( e != noErr )
		throw OSError(e);
  }

inline void DebugOSError( OSStatus e )
  {
	OSError::Debug( e );
  }

#endif
