// AsyncFileReader.cp

#ifndef AsyncFileReader_h
#include "AsyncFileReader.h"
#endif
#ifndef FileReadingPath_h
#include "FileReadingPath.h"
#endif
#ifndef Data_h
#include "Data.h"
#endif
#ifndef FilePermissionError_h
#include "FilePermissionError.h"
#endif
#ifndef FileLockError_h
#include "FileLockError.h"
#endif

#include <Errors.h>

AsyncFileReader::AsyncFileReader()
  : finished( true ),
  	 isComplete( true )
  {
	ioParam.ioRefNum = 0;
	ioParam.ioResult = noErr;
	ioParam.ioCompletion = 0;
	ioParam.ioPosMode = fsFromStart;
  }

AsyncFileReader::AsyncFileReader( const FileReadingPath& theFile,
											 uint32 position )
  : finished( true ),
  	 isComplete( true )
  {
	ioParam.ioResult = noErr;
	ioParam.ioCompletion = 0;
	ioParam.ioPosMode = fsFromStart;
	Start( theFile, position );
  }

AsyncFileReader::~AsyncFileReader()
  {
	while ( !ReadyToComplete() )
		SystemTask();
  }

void AsyncFileReader::Start( const FileReadingPath& theFile,
									  uint32 position )
  {
	Assert( theFile.IsOpen() );
	Assert( isComplete );
	
	ioParam.ioRefNum = theFile.RefNum();
	ioParam.ioPosOffset = position;
	finished = false;
  }

void AsyncFileReader::LaunchRead( Data toFill )
  {
	Assert( !Finished() );
	Assert( isComplete );
	Assert( ioParam.ioRefNum != 0 );
	
	ioParam.ioBuffer = reinterpret_cast<char *>( toFill.Start() );
	ioParam.ioReqCount = toFill.Length();
	
	ThrowError( PBReadAsync( this ) );
	
	isComplete = false;
  }

ConstData AsyncFileReader::CompleteRead()
  {
	Assert( !isComplete );
	Assert( !Finished() );
	Assert( ioParam.ioRefNum != 0 );

	while ( !ReadyToComplete() )
		SystemTask();
	
	isComplete = true;
	finished = (ioParam.ioResult == eofErr);
	ThrowError( ioParam.ioResult );
	
	return ConstData( ioParam.ioBuffer, ioParam.ioActCount );
  }

void AsyncFileReader::ThrowError( OSErr error )
  {
	if ( error == noErr || error == eofErr )
		return;
	
	switch ( error )
	  {
		case afpAccessDenied:	throw FilePermissionError( error );
		
			// This is a documented possibility:  (?)
		case fLckdErr:				throw FileLockError( error );
	  }
	
	throw FileError( error );
  }
