// AsyncFileReader.h

#ifndef AsyncFileReader_h
#define AsyncFileReader_h

#ifndef Integers_h
#include "Integers.h"
#endif
#ifndef Assert_h
#include "Assert.h"
#endif

#include <QuickDraw.h>
#include <Devices.h>

class FileReadingPath;
class Data;
class ConstData;

class AsyncFileReader: private ParamBlockRec
  {
	private:
		bool isComplete;
		bool finished;
		
		void ThrowError( OSErr );
		
	public:
		AsyncFileReader();
		AsyncFileReader( const FileReadingPath&, uint32 position = 0 );

		~AsyncFileReader();
		
		void Start( const FileReadingPath&, uint32 position = 0 );
		
		uint32 Position() const				{ Assert( isComplete ); return uint32( ioParam.ioPosOffset ); }
		void SetPosition( uint32 p )		{ Assert( isComplete ); ioParam.ioPosOffset = int32(p); finished = false; }
		
		bool Finished() const				{ return finished; }
		bool Unfinished() const				{ return !finished; }
			// Has the end of file been reached?  Finished may return
			// false if no read has exceeded the end.

		void LaunchRead( Data );
		ConstData CompleteRead();
		
		bool IsComplete() const				{ return isComplete; }
			// Has the latest data been picked up?
			
		bool ReadyToComplete() const		{ return const_cast<const volatile AsyncFileReader*>(this)
																->ioParam.ioResult <= 0; }
			// Can the data be picked up immediately?
  };

#endif
