// AsyncFileWriter.cp

#ifndef AsyncFileWriter_h
#include "AsyncFileWriter.h"
#endif
#ifndef DiskFullError_h
#include "DiskFullError.h"
#endif
#ifndef HardwareVolumeLockError_h
#include "HardwareVolumeLockError.h"
#endif
#ifndef SoftwareVolumeLockError_h
#include "SoftwareVolumeLockError.h"
#endif
#ifndef FileLockError_h
#include "FileLockError.h"
#endif
#ifndef FileWritingPath_h
#include "FileWritingPath.h"
#endif
#ifndef ConstData_h
#include "ConstData.h"
#endif

#include <Errors.h>

AsyncFileWriter::AsyncFileWriter()
  : isComplete( true )
  {
	ioParam.ioRefNum = 0;
	ioParam.ioResult = noErr;
	ioParam.ioCompletion = 0;
	ioParam.ioPosMode = fsFromStart;
  }

AsyncFileWriter::AsyncFileWriter( const FileWritingPath& theFile,
											 uint32 position )
  : isComplete( true )
  {
	ioParam.ioResult = noErr;
	ioParam.ioCompletion = 0;
	ioParam.ioPosMode = fsFromStart;
	Start( theFile, position );
  }

AsyncFileWriter::~AsyncFileWriter()
  {
	Assert( isComplete );
	while ( !ReadyToComplete() )
		SystemTask();
  }

void AsyncFileWriter::Start( const FileWritingPath& theFile,
									  uint32 position )
  {
	Assert( theFile.IsOpen() );
	Assert( isComplete );
	
	ioParam.ioRefNum = theFile.RefNum();
	ioParam.ioPosOffset = position;
  }

void AsyncFileWriter::LaunchWrite( ConstData toWrite )
  {
	Assert( isComplete );
	Assert( ioParam.ioRefNum != 0 );
	
	ioParam.ioBuffer = const_cast< char * >(
								reinterpret_cast<const char *>(
									toWrite.Start() ) );
	ioParam.ioReqCount = toWrite.Length();
	
	ThrowError( PBWriteAsync( this ) );
	
	isComplete = false;
  }

void AsyncFileWriter::CompleteWrite()
  {
	Assert( !isComplete );
	Assert( ioParam.ioRefNum != 0 );

	while ( !ReadyToComplete() )
		SystemTask();
	
	isComplete = true;
	ThrowError( ioParam.ioResult );
  }

void AsyncFileWriter::ThrowError( OSErr error )
  {
	if ( error == noErr )
		return;
	
	switch ( error )
	  {
		case dskFulErr:			throw DiskFullError( error );
		case wPrErr:				throw HardwareVolumeLockError( error );
		case vLckdErr:				throw SoftwareVolumeLockError( error );
		case fLckdErr:				throw FileLockError( error );
	  }
	
	throw FileError( error );
  }
