// AsyncFileWriter.h

#ifndef AsyncFileWriter_h
#define AsyncFileWriter_h

#ifndef Integers_h
#include "Integers.h"
#endif
#ifndef Assert_h
#include "Assert.h"
#endif

#include <Devices.h>

class FileWritingPath;
class ConstData;

class AsyncFileWriter: private ParamBlockRec
  {
	private:
		bool isComplete;
		
		void ThrowError( OSErr );
		
	public:
		AsyncFileWriter();
		AsyncFileWriter( const FileWritingPath&, uint32 position = 0 );

		~AsyncFileWriter();
		
		void Start( const FileWritingPath&, uint32 position = 0 );
		
		uint32 Position() const				{ Assert( isComplete ); return uint32( ioParam.ioPosOffset ); }
		void SetPosition( uint32 p )		{ Assert( isComplete ); ioParam.ioPosOffset = int32(p); }

		void LaunchWrite( ConstData );
		void CompleteWrite();
		
		bool IsComplete() const				{ return isComplete; }
			// Has CompleteWrite been called?
			
		bool ReadyToComplete() const		{ return const_cast<const volatile AsyncFileWriter*>(this)
																->ioParam.ioResult <= 0; }
			// Will CompleteWrite return without delay?
  };

#endif
