// FileReader.cp

#ifndef FileReader_h
#include "FileReader.h"
#endif
#ifndef Buffer_h
#include "Buffer.h"
#endif
#ifndef FileReadingPath_h
#include "FileReadingPath.h"
#endif
#ifndef FilePermissionError_h
#include "FilePermissionError.h"
#endif
#ifndef FileLockError_h
#include "FileLockError.h"
#endif
#ifndef EndOfFileError_h
#include "EndOfFileError.h"
#endif

#include <Errors.h>

FileReader::FileReader()
  : finished( true )
  {
	ioParam.ioRefNum = 0;
	ioParam.ioCompletion = 0;
	ioParam.ioPosMode = fsFromStart;
  }

FileReader::FileReader( const FileReadingPath& theFile,
								uint32 position )
  : finished( true )
  {
	ioParam.ioCompletion = 0;
	ioParam.ioPosMode = fsFromStart;
	Start( theFile, position );
  }

void FileReader::Start( const FileReadingPath& theFile,
								uint32 position )
  {
	Assert( theFile.IsOpen() );
	
	ioParam.ioRefNum = theFile.RefNum();
	ioParam.ioPosOffset = position;
	finished = false;
  }

FileReader& FileReader::operator>>( Data toFill )
  {
	Assert( !Finished() );
	Assert( ioParam.ioRefNum != 0 );
	
	ioParam.ioBuffer = reinterpret_cast<char *>( toFill.Start() );
	ioParam.ioReqCount = toFill.Length();
	
	OSErr error = PBReadSync( this );
	
	ThrowError( error );
	
	if ( ioParam.ioActCount != toFill.Length() )
	  {
		Assert( 0 );
		throw EndOfFileError( eofErr );
	  }
	
	return *this;
  }

FileReader& FileReader::operator>>( Buffer& buffer )
  {
	Assert( !Finished() );
	Assert( ioParam.ioRefNum != 0 );
	
	if ( Finished() )
		throw EndOfFileError( eofErr );
	
	ioParam.ioBuffer = reinterpret_cast<char *>( buffer.UnusedStart() );
	ioParam.ioReqCount = buffer.UnusedLength();
	
	OSErr error = PBReadSync( this );
	buffer.AdvanceMark( ioParam.ioActCount );
	
	if ( error == eofErr )
		finished = true;
	 else
		ThrowError( error );
	
	return *this;
  }

void FileReader::ThrowError( OSErr error )
  {
	if ( error == noErr )
		return;
	
	switch ( error )
	  {
		case eofErr:				throw EndOfFileError( error );
		case afpAccessDenied:	throw FilePermissionError( error );
		
			// This is a documented possibility:  (?)
		case fLckdErr:				throw FileLockError( error );
	  }
	
	throw FileError( error );
  }
