// FileReader.h

#ifndef FileReader_h
#define FileReader_h

#ifndef Integers_h
#include "Integers.h"
#endif

#include <Devices.h>

class FileReadingPath;
class Buffer;
class Data;

class FileReader: private ParamBlockRec
  {
	private:
		bool finished;
		
		void ThrowError( OSErr );
		
	public:
		FileReader();
		FileReader( const FileReadingPath&, uint32 position = 0 );
		
		void Start( const FileReadingPath&, uint32 position = 0 );
		
		uint32 Position() const				{ return uint32( ioParam.ioPosOffset ); }
		void SetPosition( uint32 p )		{ ioParam.ioPosOffset = int32(p); finished = false; }
		
		bool Finished() const				{ return finished; }
		bool Unfinished() const				{ return !finished; }
			// Has the end of file been reached?  Finished may return
			// false on some systems if no read has exceeded the end.
		
		FileReader& operator>>( Data );
			// The ConstData operator will throw an error
			// if the space cannot be completely filled with data.
		
		FileReader& operator>>( Buffer& );
			// The ConstBuffer operator may fill only part of the
			// space and not throw, if it comes to the end of file. 
			// In that case, Finished must return true.
  };

#endif
