// FileReadingAheadLoop.cp

#ifndef FileReadingAheadLoop_h
#include "FileReadingAheadLoop.h"
#endif

void FileReadingAheadLoop::DivideBuffer( Data bufferSpace )
  {
	Assert( bufferSpace.Length() >= 2 );
	bufferSections[0] = Data( bufferSpace.Start(), bufferSpace.Length() / 2 );
	bufferSections[1] = Data( bufferSections[0].End(), bufferSpace.End() );
  }

FileReadingAheadLoop::FileReadingAheadLoop( Data bufferSpace )
  : nextSectionToRead( 0 ),
	 remaining( 0 )
  {
	DivideBuffer( bufferSpace );
  }

FileReadingAheadLoop::FileReadingAheadLoop( Data bufferSpace,
														  const FileReadingPath& file,
														  uint32 start,
				 										  uint32 stop )
  {
	DivideBuffer( bufferSpace );
	Start( file, start, stop );
  }

void FileReadingAheadLoop::LaunchNextRead()
  {
	Assert( Unfinished() );
	Data toRead = bufferSections[ nextSectionToRead++ % 2 ];
	toRead.LimitLength( remaining );
	reader.LaunchRead( toRead );
  }

void FileReadingAheadLoop::Start( const FileReadingPath& file,
											 uint32 start,
											 uint32 stop )
  {
	Assert( start <= stop );
	
	nextSectionToRead = 0;
	remaining = stop - start;
	availableData = ConstData();
	
	if ( Unfinished() )
	  {
		reader.Start( file, start );
		LaunchNextRead();
		operator++();
	  }
  }

void FileReadingAheadLoop::operator++()
  {
	Assert( Unfinished() );
	Assert( remaining >= availableData.Length() )
	
	remaining -= availableData.Length();
	
	if ( Finished() )
		return;
	
	availableData = reader.CompleteRead();
	
	if ( reader.Finished() )
		remaining = availableData.Length();

	if ( remaining > availableData.Length() )
		LaunchNextRead();
  }

bool FileReadingAheadLoop::ReadyToAdvance() const
  {
	Assert( Unfinished() );
	Assert( remaining >= availableData.Length() )

	return remaining <= availableData.Length()
			 || reader.ReadyToComplete();
  }
