// FileReadingPath.h

#ifndef FileReadingPath_h
#define FileReadingPath_h

#ifndef Integers_h
#include "Integers.h"
#endif
#ifndef FilePermission_h
#include "FilePermission.h"
#endif
#ifndef Assert_h
#include "Assert.h"
#endif

class FileLocation;

class FileReadingPath
  {
	friend class FileWritingPath;
	
	private:
		int16 refNum;
		bool isOpen;
		
		// not implemented:
			FileReadingPath( const FileReadingPath& );
			void operator=( const FileReadingPath& );

		static void ThrowError( OSErr );
		
		void OpenDataFork( const FileLocation&, FilePermission );
		void OpenResourceFork( const FileLocation&, FilePermission );
		
	public:
		// Implementable cross-platform
			FileReadingPath();
			FileReadingPath( const FileLocation& location );
			
			~FileReadingPath();
		
			bool IsOpen() const									{ return isOpen; }
			void Close();

			void OpenExisting( const FileLocation& location )
				{ OpenDataFork( location, FilePermission::ReadOnly() ); }

			uint32 Length() const;
		
		// Implemented for Macintosh only
			enum DataFork		{ dataFork };
			enum ResourceFork { resourceFork };
			
			FileReadingPath( const FileLocation& location, DataFork );
			FileReadingPath( const FileLocation& location, ResourceFork );

			void Open( const FileLocation& location, DataFork )
				{ OpenDataFork( location, FilePermission::ReadOnly() ); }

			void Open( const FileLocation& location, ResourceFork )
				{ OpenResourceFork( location, FilePermission::ReadOnly() ); }
			
			int16 RefNum() const						{ Assert( isOpen ); return refNum; }
  };

#endif
