// FileWriter.cp

#ifndef FileWriter_h
#include "FileWriter.h"
#endif
#ifndef ConstBuffer_h
#include "ConstBuffer.h"
#endif
#ifndef FileWritingPath_h
#include "FileWritingPath.h"
#endif
#ifndef DiskFullError_h
#include "DiskFullError.h"
#endif
#ifndef HardwareVolumeLockError_h
#include "HardwareVolumeLockError.h"
#endif
#ifndef SoftwareVolumeLockError_h
#include "SoftwareVolumeLockError.h"
#endif
#ifndef FileLockError_h
#include "FileLockError.h"
#endif

#include <Errors.h>

FileWriter::FileWriter()
  {
	ioParam.ioCompletion = 0;
	ioParam.ioRefNum = 0;
	ioParam.ioPosMode = fsFromStart;
  }

FileWriter::FileWriter( const FileWritingPath& theFile,
								uint32 position )
  {
	Assert( theFile.IsOpen() );
	
	ioParam.ioCompletion = 0;
	ioParam.ioRefNum = theFile.RefNum();
	ioParam.ioPosMode = fsFromStart;
	ioParam.ioPosOffset = position;
  }

void FileWriter::Start( const FileWritingPath& theFile,
						 uint32 position )
  {
	Assert( theFile.IsOpen() );
	
	ioParam.ioRefNum = theFile.RefNum();
	ioParam.ioPosOffset = position;
  }

FileWriter& FileWriter::operator<<( ConstBuffer& buffer )
  {
	Assert( ioParam.ioRefNum != 0 );
	
	ioParam.ioBuffer = reinterpret_cast<char *>(
								const_cast<uint8 *>(
									buffer.UnusedStart() ) );
	ioParam.ioReqCount = buffer.UnusedLength();
	
	OSErr error = PBWriteSync( this );
	buffer.AdvanceMark( ioParam.ioActCount );
	
	ThrowError( error );
	
	return *this;
  }

FileWriter& FileWriter::operator<<( ConstData data )
  {
	Assert( ioParam.ioRefNum != 0 );

	ioParam.ioBuffer = reinterpret_cast<char *>(
								const_cast<uint8 *>(
									data.Start() ) );
	ioParam.ioReqCount = data.Length();
	
	ThrowError( PBWriteSync( this ) );
	
	Assert( ioParam.ioActCount == data.Length() );
	
	return *this;
  }

void FileWriter::ThrowError( OSErr error )
  {
	if ( error == noErr )
		return;
	
	switch ( error )
	  {
		case dskFulErr:			throw DiskFullError( error );
		case wPrErr:				throw HardwareVolumeLockError( error );
		case vLckdErr:				throw SoftwareVolumeLockError( error );
		case fLckdErr:				throw FileLockError( error );
	  }
	
	throw FileError( error );
  }
