// FileWritingPath.cp

#ifndef FileWritingPath_h
#include "FileWritingPath.h"
#endif
#ifndef Overflow_h
#include "Overflow.h"
#endif
#ifndef FileNotFoundError_h
#include "FileNotFoundError.h"
#endif
#ifndef FileLocation_h
#include "FileLocation.h"
#endif
#ifndef DuplicateFileError_h
#include "DuplicateFileError.h"
#endif

void FileWritingPath::SetLength( uint32 length )
  {
	Assert( IsOpen() );
	ThrowError( SetEOF( RefNum(), length ) );
  }

uint32 FileWritingPath::Allocate( uint32 amount )
  {
	int32 truncatedAmount = amount;
	Assert( IsOpen() );
	ThrowError( ::Allocate( RefNum(), &truncatedAmount ) );
	return truncatedAmount;
  }

uint32 FileWritingPath::AllocateContiguous( uint32 amount )
  {
	int32 truncatedAmount = amount;
	Assert( IsOpen() );
	ThrowError( AllocContig( RefNum(), &truncatedAmount ) );
	return amount;
  }

void FileWritingPath::Flush() const
  {
	ParamBlockRec block;
	block.ioParam.ioCompletion = 0;
	block.ioParam.ioRefNum = RefNum();
	ThrowError( PBFlushFileSync( &block ) );
  }

FileWritingPath::FileWritingPath( const FileLocation& location,
											 OpenExistingFile )
  {
	OpenExisting( location );
  }

FileWritingPath::FileWritingPath( const FileLocation& location,
											 OpenNewFile,
											 FileType type )
  {
	OpenNew( location, type );
  }

FileWritingPath::FileWritingPath( const FileLocation& location,
											 OpenExistingFileOrNewFile,
											 FileType type )
  {
	OpenExistingOrNew( location, type );
  }

FileWritingPath::FileWritingPath( const FileLocation& location,
											 OpenNewFileOrAsNewFile,
											 FileType type )
  {
	OpenNewOrAsNew( location, type );
  }

void FileWritingPath::OpenExisting( const FileLocation& location )
  {
	OpenDataFork( location, FilePermission::ReadWrite() );
  }

void FileWritingPath::OpenNew( const FileLocation& location,
										 FileType type )
  {
	CreateFile( location, type );
	OpenExisting( location );
  }

void FileWritingPath::OpenExistingOrNew( const FileLocation& location,
													  FileType type )
  {
	try
	  {
		OpenExisting( location );
	  }
	 catch ( FileNotFoundError )
	  {
		CreateFile( location, type );
		OpenExisting( location );
	  }
  }

void FileWritingPath::OpenNewOrAsNew( const FileLocation& location,
												  FileType type )
  {
	try
	  {
		CreateFile( location, type );
	  }
	 catch ( DuplicateFileError )
	  {
		DeleteFile( location );
		CreateFile( location, type );
	  }

	OpenExisting( location );
  }
