// FileWritingPath.h

#ifndef FileWritingPath_h
#define FileWritingPath_h

#ifndef FileReadingPath_h
#include "FileReadingPath.h"
#endif
#ifndef FileType_h
#include "FileType.h"
#endif

class FileWritingPath: public FileReadingPath
  {
	public:
		// These are part of the portable specification:
			FileWritingPath()									{}
			~FileWritingPath()								{ Assert( !IsOpen() ); }
				// Writable paths must be explicitly closed, so that errors are reported.
		
			enum OpenExistingFile          { openExisting };
			enum OpenNewFile               { openNew };
			enum OpenExistingFileOrNewFile { openNewOrAsIs };
			enum OpenNewFileOrAsNewFile    { openNewOrAsNew };
			
			FileWritingPath( const FileLocation&, OpenExistingFile );
			FileWritingPath( const FileLocation&, OpenNewFile,               FileType );
			FileWritingPath( const FileLocation&, OpenExistingFileOrNewFile, FileType );
			FileWritingPath( const FileLocation&, OpenNewFileOrAsNewFile,    FileType );
			
			void OpenExisting(      const FileLocation& );
			void OpenNew(           const FileLocation&, FileType );
			void OpenExistingOrNew( const FileLocation&, FileType );
			void OpenNewOrAsNew(    const FileLocation&, FileType );
			
			void SetLength( uint32 );
		
			void Flush() const;
		
		// These are not considered portable:
		
			FileWritingPath( const FileLocation& location, FilePermission permission )
			  { OpenDataFork( location, permission ); }
			
			FileWritingPath( const FileLocation& location, FilePermission permission, DataFork )
			  { OpenDataFork( location, permission ); }

			FileWritingPath( const FileLocation& location, FilePermission permission, ResourceFork )
			  { OpenResourceFork( location, permission ); }
			
			void Open( const FileLocation& location, FilePermission permission )
			  { OpenDataFork( location, permission ); }
			
			void Open( const FileLocation& location, FilePermission permission, DataFork )
			  { OpenDataFork( location, permission ); }
			
			void Open( const FileLocation& location, FilePermission permission, ResourceFork )
			  { OpenResourceFork( location, permission ); }

			uint32 Allocate( uint32 );					// returns amount actually allocated;
			uint32 AllocateContiguous( uint32 );	// throws disk full on failure.
  };

#endif
