// Face.h

#ifndef Face_h
#define Face_h

#ifndef FontNumber_h
#include "FontNumber.h"
#endif
#ifndef FontSize_h
#include "FontSize.h"
#endif
#ifndef ScriptID_h
#include "ScriptID.h"
#endif
#ifndef __APPEARANCE__
#include <Appearance.h>
#endif
#ifndef __SCRIPT__
#include <Script.h>
#endif

class Face
  {
	private:
		FontNumber font;
		FontSize size;
		::Style style;
	
		static Face ScriptManagerFace( ScriptID, int16 scriptVariable );
		static Face AppearanceManagerFace( ScriptID, ThemeFontID );
		
	public:
		Face( FontNumber theFont, FontSize theSize, ::Style theStyle = 0 )
		  : font( theFont ),
			 size( theSize ),
			 style( theStyle )
		  {}
		
		FontNumber Font() const							{ return font; }
		void SetFont( FontNumber f )					{ font = f; }
		
		FontSize Size() const							{ return size; }
		void SetSize( FontSize s )						{ size = s; }

		::Style Style() const							{ return style; }
		void SetStyle( ::Style s )						{ style = s; }
		
		bool operator==( const Face& ) const;
		bool operator!=( const Face& f ) const		{ return !(*this == f); }
		
		// These are appearance-savvy, with fallbacks for pre-8.5
			static Face System( ScriptID = ScriptID(smSystemScript) );
			static Face SmallSystem( ScriptID = ScriptID(smSystemScript) );
			static Face SmallEmphasizedSystem( ScriptID = ScriptID(smSystemScript) );
			static Face Views( ScriptID = ScriptID(smSystemScript) );

		// These are pre-appearance, using the script manager
			inline static Face Application( ScriptID = ScriptID(smSystemScript) );
			inline static Face Small( ScriptID = ScriptID(smSystemScript) );
			inline static Face Help( ScriptID = ScriptID(smSystemScript) );
  };

inline Face Face::Application( ScriptID script )
  {
	return ScriptManagerFace( script, smScriptAppFondSize );
  }

inline Face Face::Small( ScriptID script )
  {
	return ScriptManagerFace( script, smScriptSmallFondSize );
  }

inline Face Face::Help( ScriptID script )
  {
	return ScriptManagerFace( script, smScriptHelpFondSize );
  }

#endif
