// IdleTimer.cp

#ifndef IdleTimer_h
#include "IdleTimer.h"
#endif
#ifndef Assert_h
#include "Assert.h"
#endif
#ifndef Tick_h
#include "Tick.h"
#endif
#ifndef EventQueue_h
#include "EventQueue.h"
#endif
#ifndef Application_h
#include "Application.h"
#endif

Tick IdleTimer::expiration( 0 );
uint32 IdleTimer::idleTime = 4;
bool IdleTimer::set = false;

IdleTimer::IdleTimer( uint32 timeToIdle )
  {
	Assert( !set );
	idleTime = timeToIdle;
	expiration = Tick::Now() + timeToIdle;
	set = true;
  }

IdleTimer::~IdleTimer()
  {
	Assert( set );
	idleTime = 4;
	set = false;
  }

bool IdleTimer::Expired()
  {
	Assert( set );
	return Tick::Now() >= expiration;
  }

void IdleTimer::ThrowExpiration()
  {
	if ( Expired() )
		throw IdleTimerExpired();
  }

void IdleTimer::ThrowUserEvent()
  {
	if ( !Expired() )
		return;
	
	EventQueue& queue( EventQueue::The() );
	
	for ( EventKind what = queue.Next().what;
			what != nullEvent;
			what = queue.Next().what )
	  {
		switch ( what )
		  {
			case mouseDown:
			case keyDown:
			case autoKey:
			case diskEvt:
			case kHighLevelEvent:
				throw IdleTimerExpired();
		  }

		queue++;
		Application::The().HandleEvent( *queue );
	  }
	
	expiration = Tick::Now() + idleTime;
  }
