// Postponer.cp

#ifndef Postponer_h
#include "Postponer.h"
#endif
#ifndef Functor_h
#include "Functor.h"
#endif
#ifndef IdleTimer_h
#include "IdleTimer.h"
#endif

Postponer::Postponer( Functor< void(*)() >& action )
  : link( this ),
	 perform( action )
  {
	TheList();
  }

Postponer::~Postponer()
  {
  }

ListOf< Postponer >& Postponer::TheList()
  {
	static ListOf< Postponer > list;
	return list;
  }

void Postponer::Schedule()
  {
	Assert( !Pending() );
	TheList().Add( link, afterEnd );
  }

void Postponer::Cancel()
  {
	Assert( Pending() );
	TheList().Remove( link );
  }

void Postponer::Flush()
  {
	if ( Pending() )
	  {
		Cancel();
		perform();
	  }
  }

void Postponer::Execute()
  {
	ListOf< Postponer >& list( TheList() );
	
	while ( !list.IsEmpty() )
	  {
		Postponer *p = list.First()->Target();
		list.Remove( p->link );
		p->perform();
		
		if ( IdleTimer::Expired() )
			break;
	  }
  }

bool Postponer::CanSleep()
  {
	return TheList().IsEmpty();
  }
