// SwitchedMenuItem.h

#ifndef SwitchedMenuItem_h
#define SwitchedMenuItem_h

#ifndef ResourceID_h
#include "ResourceID.h"
#endif
#ifndef Commander_h
#include "Commander.h"
#endif
#ifndef MessageString_h
#include "MessageString.h"
#endif
#ifndef SingleMenuItem_h
#include "SingleMenuItem.h"
#endif

template < class Protocol >
struct SwitchedMenuItemEntry
  {
	int16 nameIndex;
	void (Protocol::*doIt)();
	bool (Protocol::*canDo)();
  };

template < class Protocol, class SwitchIndex >
class SwitchedMenuItem: public SingleMenuItem
  {
	public:
		typedef SwitchIndex (Protocol::*Chooser)() const;
		typedef SwitchedMenuItemEntry<Protocol> Entry;
		
	private:
		const Chooser choose;
		
		const Entry *const items;
		const uint32 itemCount;
		uint32 currentItem;
		
		const ResourceID names;
		uint16 currentName;
		
	public:
		SwitchedMenuItem( Menu&,
								Chooser choose,
								const Entry items[],
								ResourceID names );
		
		virtual void Prepare();
		virtual void Choose();
  };


template < class Protocol >
uint32 CountItems( const SwitchedMenuItemEntry<Protocol> items[] )
  {
	uint32 count = 0;
	
	while ( items[ count ].doIt != 0 )
		count++;
		
	return count;
  }

template < class Protocol, class SwitchIndex >
SwitchedMenuItem<Protocol,SwitchIndex>::SwitchedMenuItem( Menu& menu,
																			 Chooser theChooser,
																			 const Entry theItems[],
																			 ResourceID theNames )
  : SingleMenuItem( menu ),
	 choose( theChooser ),
	 items( theItems ),
	 itemCount( CountItems( items ) ),
	 currentItem( maxuint32 ),
	 names( theNames ),
	 currentName( maxuint16 )
  {
  }
		
template < class Protocol, class SwitchIndex >
void SwitchedMenuItem<Protocol,SwitchIndex>::Prepare()
  {
	if ( Commander<Protocol>::Null() )
	  {
		currentItem = itemCount;
		Disable();
	  }
	 else
	  {
		Protocol& handler( *Commander<Protocol>() );

		currentItem = (handler.*choose)();
		Assert( currentItem < itemCount );

		SetEnabled( ( handler.*( items[currentItem].canDo ) ) () );
     }

	if ( currentName != items[ currentItem ].nameIndex )
	  {
		currentName = items[ currentItem ].nameIndex;
		SetText( MessageString( names, currentName ) );
	  }
  }

template < class Protocol, class SwitchIndex >
void SwitchedMenuItem<Protocol,SwitchIndex>::Choose()
  {
	Assert( currentItem < itemCount );
	Protocol& handler( *Commander<Protocol>() );
	
	if ( ( handler.*( items[currentItem].canDo ) ) () )
		( handler.*( items[currentItem].doIt ) ) ();
  }

#endif
