// OpenerList.cp

#ifndef OpenerList_h
#include "OpenerList.h"
#endif
#ifndef CatInfo_h
#include "CatInfo.h"
#endif
#ifndef Opener_h
#include "Opener.h"
#endif
#ifndef UnopenableFileType_h
#include "UnopenableFileType.h"
#endif

OpenerList::OpenerList()
  {
  }

OpenerList::~OpenerList()
  {
  }

OpenerList& OpenerList::The()
  {
	static OpenerList the;
	return the;
  }

Opener *OpenerList::OpenerFor( FileType type ) const
  {
	const RedBlackLink< uint32, Opener > *link = tree.Find( type.Type() );
	
	if ( link == 0 )
		return 0;

	return *link;
  }

bool OpenerList::CanOpen( const CatInfo& info ) const
  {
	Opener *opener = OpenerFor( info.File().Type() );
	return opener != 0 && opener->CanOpen( info );
  }

bool OpenerList::CanPrint( const CatInfo& info ) const
  {
	Opener *opener = OpenerFor( info.File().Type() );
	return opener != 0 && opener->CanPrint( info );
  }

void OpenerList::Open( const CatInfo& info ) const
  {
	Opener *opener = OpenerFor( info.File().Type() );
	
	if ( opener == 0 )
		throw UnopenableFileType( info.Location(), info.File().Type() );
	
	opener->Open( info );
  }

void OpenerList::Print( const CatInfo& info ) const
  {
	Opener *opener = OpenerFor( info.File().Type() );
	
	if ( opener == 0 )
		throw UnopenableFileType( info.Location(), info.File().Type() );
	
	opener->Print( info );
  }
