// PrintJob.h

#ifndef PrintJob_h
#define PrintJob_h

#ifndef Assert_h
#include "Assert.h"
#endif
#ifndef HandleTo_h
#include "HandleTo.h"
#endif
#ifndef Rectangle_h
#include "Rectangle.h"
#endif

#include <Printing.h>

class PrintManager;
class PageSetupInfo;
class PrintJobInfo;

class PrintJob
  {
	private:
		HandleTo<TPrint> printHandle;
		uint32 firstPage;
		uint32 lastPage;
		
		// not implemented:
			PrintJob( const PrintJob& );
			void operator=( const PrintJob& );
	
	public:
		PrintJob( const PrintManager&,
					 const PageSetupInfo&,
					 const PrintJobInfo& );

		const HandleTo<TPrint>& PrintHandle() const	{ return printHandle; }
		
		uint32 CopyCount() const							{ return printHandle->prJob.iCopies; }

		uint32 FirstPage() const							{ return firstPage; }
		uint32 LastPage() const								{ return lastPage; }
		
		Rectangle PageArea() const							{ return printHandle->prInfo.rPage; }
		Rectangle PaperArea() const						{ return printHandle->rPaper; }
		
		uint16 VerticalResolution() const				{ return printHandle->prInfo.iVRes; }
		uint16 HorizontalResolution() const				{ return printHandle->prInfo.iHRes; }
			// both in dots per inch
		
		bool Spooled() const			{ return printHandle->prJob.bJDocLoop == bSpoolLoop; }
  };

#endif
