// Scrolling.h

#ifndef Scrolling_h
#define Scrolling_h

#ifndef Rectangle32_h
#include "Rectangle32.h"
#endif
#ifndef Announcer_h
#include "Announcer.h"
#endif

class Scrolling
  {
	private:
		Announcer announcer;
	
	protected:
		~Scrolling()											{}
		
		void AnnounceScrollingChange()					{ announcer.Announce(); }
		
	public:
		const Announcer& ScrollingAnnouncer()			{ return announcer; }
		
		virtual Point32 PanoramaSize() const = 0;
		virtual Rectangle32 RevealedArea() const = 0;
		
		virtual void ScrollTo( Point32 ) = 0;
		void ScrollTo( int32 h, int32 v )				{ ScrollTo( Point32( h, v ) ); }
		
		virtual UPoint32 ScrollingQuantum() const;

		virtual Range32 LineContaining( int32 pixel ) const;
		virtual Range32 ColumnContaining( int32 pixel ) const;
				
		virtual int32 BestHorizontalPosition( Range32 ) const;
		virtual int32 BestVerticalPosition( Range32 ) const;
			// These ranges are interpreted weakly: start <= result <= end
		
		virtual Range32 ReasonableLineSize() const;
		virtual Range32 ReasonableColumnSize() const;
		
		void ScrollTopTo( int32 );
		void ScrollLeftSideTo( int32 );
		
		void ScrollUp( int32 );
		void ScrollDown( int32 );
		void ScrollLeft( int32 );
		void ScrollRight( int32 );
		
		void ScrollToHome();
		void ScrollToEnd();
		
		void ScrollUpOneLine();
		void ScrollUpOnePage();
		void ScrollDownOnePage();
		void ScrollDownOneLine();

		void ScrollLeftOneLine();
		void ScrollLeftOnePage();
		void ScrollRightOnePage();
		void ScrollRightOneLine();
  };

#endif
