// TextEditView.h

#ifndef TextEditView_h
#define TextEditView_h

#ifndef Activator_h
#include "Activator.h"
#endif
#ifndef CommandHandler_h
#include "CommandHandler.h"
#endif
#ifndef Typing_h
#include "Typing.h"
#endif
#ifndef ReturnKeys_h
#include "ReturnKeys.h"
#endif
#ifndef HorizontalArrows_h
#include "HorizontalArrows.h"
#endif
#ifndef VerticalArrows_h
#include "VerticalArrows.h"
#endif
#ifndef DeleteKeys_h
#include "DeleteKeys.h"
#endif
#ifndef Editing_h
#include "Editing.h"
#endif
#ifndef TextEditObject_h
#include "TextEditObject.h"
#endif
#ifndef PeriodicMethod_h
#include "PeriodicMethod.h"
#endif
#ifndef Selecting_h
#include "Selecting.h"
#endif
#ifndef ViewCell_h
#include "ViewCell.h"
#endif
#ifndef DrawsSpontaneously_h
#include "DrawsSpontaneously.h"
#endif
#ifndef SuggestsPaneSize_h
#include "SuggestsPaneSize.h"
#endif

class TextEditView: virtual public ViewCell,
						  virtual public DrawsSpontaneously,
						  virtual public SuggestsPaneSize,
						  public Activator,
						  public CommandHandler<Typing>,
						  public CommandHandler<ReturnKeys>,
						  public CommandHandler<HorizontalArrows>,
						  public CommandHandler<VerticalArrows>,
						  public CommandHandler<DeleteKeys>,
						  public CommandHandler<Editing>,
						  public CommandHandler<Selecting>
  {
	private:
		TextEditObject editor;
		
		PeriodicMethod<TextEditView> idle;
	
		void Idle();

		void AdjustToCanvas( const Canvas& ) const;
		
	public:
		TextEditView( GrafPortObject&, Focus& );

		TextEditObject& Editor()						{ return editor; }
		const TextEditObject& Editor() const		{ return editor; }
		
		virtual void Draw( const Canvas& ) const;
		virtual void Click( const Canvas&,
								  const MouseDownEvent& );

		virtual const CursorObject& Cursor( const Canvas&,
														const MouseEvent&,
														RegionObject& ) const;
	
		virtual void Activate();
		virtual void Deactivate();
	
		virtual void Type( ConstData );
		
		virtual void ReturnKey( const KeystrokeEvent& );
		
		virtual void LeftArrow( const KeystrokeEvent& );
		virtual void RightArrow( const KeystrokeEvent& );
		virtual void UpArrow( const KeystrokeEvent& );
		virtual void DownArrow( const KeystrokeEvent& );
		
		virtual void DeleteBackward( const KeystrokeEvent& );
		virtual void DeleteForward( const KeystrokeEvent& );

		virtual bool CanCut();
		virtual bool CanCopy();
		virtual bool CanPaste();
		virtual bool CanClear();

		virtual void Cut();
		virtual void Copy();
		virtual void Paste();
		virtual void Clear();
		
		virtual bool CanSelectAll() const;
		virtual void SelectAll();
		
		virtual int32 BestWidth( Range32 bounds ) const;
		virtual int32 BestHeight( Range32 bounds ) const;
		virtual int32 MinimumHeight() const;
  };

#endif
