// RedBlackLinkTree.h

#ifndef RedBlackLinkTree_h
#define RedBlackLinkTree_h

#ifndef RedBlackKeyTree_h
#include "RedBlackKeyTree.h"
#endif
#ifndef LinkLoop_h
#include "LinkLoop.h"
#endif

template < class Key, class Content > class RedBlackLink;

template < class Key, class Content >
class RedBlackLinkTree: private RedBlackKeyTree<Key>
  {
	friend class RedBlackLink< Key, Content >;
	
	private:
		typedef RedBlackKeyTree<Key> TreeBase;
		typedef typename TreeBase::Node NodeBase;

	public:
		typedef RedBlackLink< Key, Content > Node;
		typedef typename LinkLoop< typename TreeBase::Loop,
											RedBlackLinkTree<Key,Content>,
											TreeBase,
											Node,
											typename TreeBase::Node,
											Content > Loop;
	
	private:
		static Node *DownCast( NodeBase *n );
		static const Node *DownCast( const NodeBase *n );
		
	public:
		void Add( Node& n )									{ TreeBase::Add( n ); }
		void Remove( Node& n )								{ TreeBase::Remove( n ); }
		
		// Access declarations are broken in templates in CW Pro 5
		//TreeBase::RemoveAll;
		//TreeBase::IsEmpty;
		bool IsEmpty() const				{ return TreeBase::IsEmpty(); }
		void RemoveAll()					{ TreeBase::RemoveAll(); }

		Node *Root()											{ return DownCast( TreeBase::Root() ); }
		const Node *Root() const							{ return DownCast( TreeBase::Root() ); }
		
		Node *First()											{ return DownCast( TreeBase::First() ); }
		const Node *First() const							{ return DownCast( TreeBase::First() ); }
		
		Node *Last()											{ return DownCast( TreeBase::Last() ); }
		const Node *Last() const							{ return DownCast( TreeBase::Last() ); }
		
		Node *First( const Key& k )						{ return DownCast( TreeBase::First( k ) ); }
		const Node *First( const Key& k ) const		{ return DownCast( TreeBase::First( k ) ); }
		
		Node *Last( const Key& k )							{ return DownCast( TreeBase::Last( k ) ); }
		const Node *Last( const Key& k ) const			{ return DownCast( TreeBase::Last( k ) ); }

		Node *Find( const Key& k )							{ return DownCast( TreeBase::Find( k ) ); }
		const Node *Find( const Key& k ) const			{ return DownCast( TreeBase::Find( k ) ); }
		
		Node& operator[]( const Key& k )					{ return *DownCast( &TreeBase::operator[]( k ) ); }
		const Node& operator[]( const Key& k ) const	{ return *DownCast( &TreeBase::operator[]( k ) ); }
		
		TreeBase::Valid;
  };

#ifndef RedBlackLinkTree_cp
#include "RedBlackLinkTree.cp"
#endif

#endif
