// AcceptsConformanceToSize.cp

#ifndef AcceptsConformanceToSize_h
#include "AcceptsConformanceToSize.h"
#endif
#ifndef Assert_h
#include "Assert.h"
#endif

AcceptsConformanceToSize::AcceptsConformanceToSize()
  : paneSize( 0, 0 ),
	 paneSizeKnown( false ),
	 conformingView( 0 )
  {
  }

void AcceptsConformanceToSize::SetConformingView( ConformsToPaneSize *newContent )
  {
	if ( conformingView == newContent )
		return;
	
	Clear();

	conformingView = newContent;
	
	if ( conformingView != 0 )
	  {
		Assert( conformingView->conformTo == 0 );
		Assert( conformingView->paneSizeKnown == false );
		Assert( conformingView->paneSize == UPoint32::zero );
		
		conformingView->conformTo = this;
		conformingView->paneSizeKnown = paneSizeKnown;
		conformingView->paneSize = paneSize;
		
		if ( paneSizeKnown )
			conformingView->PaneSizeChanged();
	  }
  }

void AcceptsConformanceToSize::Clear()
  {
	if ( conformingView != 0 )
	  {
		Assert( conformingView->paneSizeKnown == paneSizeKnown );
		Assert( conformingView->paneSize == paneSize );
		Assert( conformingView->conformTo == this );
		
		conformingView->paneSize = UPoint32::zero;
		conformingView->paneSizeKnown = false;
		conformingView->conformTo = 0;
		
		if ( paneSizeKnown )
			conformingView->PaneSizeChanged();
		
		conformingView = 0;
	  }
  }

void AcceptsConformanceToSize::AnnouncePaneSize( UPoint32 newSize )
  {
	if ( paneSizeKnown && paneSize == newSize )
		return;
	
	if ( conformingView != 0 )
	  {
		Assert( conformingView->paneSizeKnown == paneSizeKnown );
		Assert( conformingView->paneSize == paneSize );
		Assert( conformingView->conformTo == this );

		conformingView->paneSizeKnown = true;
		conformingView->paneSize = newSize;
		conformingView->PaneSizeChanged();
	  }
	
	paneSizeKnown = true;
	paneSize = newSize;
  }
