// PaneStack.cp

#ifndef PaneStack_h
#include "PaneStack.h"
#endif
#ifndef Rectangle32_h
#include "Rectangle32.h"
#endif
#ifndef Canvas_h
#include "Canvas.h"
#endif
#ifndef CellVisitor_h
#include "CellVisitor.h"
#endif
#ifndef StackedPane_h
#include "StackedPane.h"
#endif

PaneStack::PaneStack( const Announcing<UPoint32>& theSize )
  : size( theSize ),
	 listenToSize( theSize, *this, &PaneStack::SizeChanged )
  {
  }

void PaneStack::SizeChanged()
  {
	Arrange( size );
  }

void PaneStack::Deliver( CellVisitor& visitor, const Canvas& canvas )
  {
	Rectangle32 destination( canvas.FromScreen( visitor.Destination() ) );
	Canvas remainderOfParent( canvas );
	Canvas child;
	
	for ( PaneLoop pane( Panes() );
			pane.Unfinished()
				&& remainderOfParent.Clip().Intersects( visitor.Destination() );
			pane++ )
	  {
		if ( destination.Intersects( pane->Bounds() ) )
		  {
			child = remainderOfParent;
			child.Submap( pane->Bounds() );
			if ( child.Visible() )
				pane->Content().Deliver( visitor, child );
		  }
		
		remainderOfParent.ObstructOffScreen( pane->Bounds() );
	  }
  }
