// WindowRootPane.cp

#ifndef WindowRootPane_h
#include "WindowRootPane.h"
#endif
#ifndef Canvas_h
#include "Canvas.h"
#endif
#ifndef WindowObject_h
#include "WindowObject.h"
#endif
#ifndef View_h
#include "View.h"
#endif
#ifndef SuggestsPaneSize_h
#include "SuggestsPaneSize.h"
#endif

WindowRootPane::WindowRootPane( WindowObject& theWindow )
  : Pane( 0 ),
	 window( theWindow )
  {
  }

void WindowRootPane::Clear()
  {
	AcceptsSizeSuggestions::Clear();
	Pane::Clear();
	AcceptsConformanceToSize::Clear();
  }

void WindowRootPane::Adjust( Canvas& canvas ) const
  {
	Assert( !canvas.HasPort() );
	canvas = window;
  }

void WindowRootPane::AnnounceSizeChange()
  {
	AnnouncePaneSize( UPoint32( window.Port().LocalBounds().Size() ) );
  }

void WindowRootPane::Deliver( CellVisitor& visitor ) const
  {
	Canvas canvas( window );
	if ( canvas.Visible() )
		Content().Deliver( visitor, canvas );
  }

const PointObject WindowRootPane::MinimumSize() const
  {
	return Sizer().MinimumSize().NearestPoint16();
  }

const PointObject WindowRootPane::MaximumSize() const
  {
	return Sizer().MaximumSize().NearestPoint16();
  }

const PointObject WindowRootPane::ReasonableSize() const
  {
	return Sizer().ReasonableSize().NearestPoint16();
  }

const PointObject WindowRootPane::BestSize( PointObject maximum ) const
  {
	return Sizer().BestSize( Range32( 0, maximum.h ),
									 Range32( 0, maximum.v ) ).NearestPoint16();
  }
