// PaneRow.cp

#ifndef PaneRow_h
#include "PaneRow.h"
#endif
#ifndef MinMax_h
#include "MinMax.h"
#endif
#ifndef Overflow_h
#include "Overflow.h"
#endif
#ifndef TiledPane_h
#include "TiledPane.h"
#endif

void PaneRow::PaneSizeChanged()
  {
	if ( !PaneSizeKnown() )
		return;
	
	Assert( !Panes().IsEmpty() );
	
	int32 oldWidth = (*Panes().Last())->Bounds().right;
	
	for ( PaneLoop pane( Panes() ); pane.Unfinished(); pane++ )
		pane->proposedSize = 0;
	
	int32 remainder = Asint32( PaneSize().h );
	
	for ( PaneLoop pane( Panes() ); remainder > 0 && pane.Unfinished(); pane++ )
	  {
		uint32 minimum = Min( remainder, pane->Sizer().MinimumWidth() );
		
		pane->proposedSize = minimum;
		remainder -= minimum;
	  }

	Assert( TotalProposedSize() + remainder == Asint32( PaneSize().h ) );
	
	for ( PaneLoop pane( Panes() ); remainder > 0 && pane.Unfinished(); pane++ )
	  {
		int32 best = pane->Sizer().BestWidth( Range32( pane->proposedSize,
																	  pane->proposedSize + remainder ) );
		
		int32 increase = best - pane->proposedSize;
		remainder -= increase;
		pane->proposedSize += increase;
	  }

	Assert( TotalProposedSize() + remainder == Asint32( PaneSize().h ) );
	
	for ( PaneLoop pane( *Panes().Last() ); remainder > 0 && pane.Unfinished(); pane-- )
	  {
		int32 maximum = Min( pane->proposedSize + remainder,
									pane->Sizer().MaximumWidth() );
		
		int32 increase = maximum - pane->proposedSize;
		remainder -= increase;
		pane->proposedSize += increase;
	  }

	Assert( TotalProposedSize() + remainder == Asint32( PaneSize().h ) );
	
	if ( remainder > 0 )
		(*Panes().Last())->proposedSize += remainder;

	Assert( TotalProposedSize() == Asint32( PaneSize().h ) );
	
	uint32 position = 0;
	
	for ( PaneLoop pane( Panes() ); pane.Unfinished(); pane++ )
	  {
		pane->proposedBounds = Rectangle32( position,								0,
														position + pane->proposedSize,	PaneSize().v );
		position += pane->proposedSize;
	  }
	
	if ( oldWidth <= PaneSize().h )
		ShufflePanesForward();
	 else
		ShufflePanesBackward();
  }

int32 PaneRow::MinimumWidth() const
  {
	return TotalOf( &SuggestsPaneSize::MinimumWidth );
  }

int32 PaneRow::MinimumHeight() const
  {
	return MaximumOf( &SuggestsPaneSize::MinimumHeight );
  }

int32 PaneRow::MaximumWidth() const
  {
	return TotalOf( &SuggestsPaneSize::MaximumWidth );
  }

int32 PaneRow::MaximumHeight() const
  {
	return MinimumOf( &SuggestsPaneSize::MaximumHeight );
  }

int32 PaneRow::ReasonableWidth() const
  {
	return TotalOf( &SuggestsPaneSize::ReasonableWidth );
  }

int32 PaneRow::ReasonableHeight() const
  {
	return MaximumOf( &SuggestsPaneSize::ReasonableHeight );
  }

int32 PaneRow::BestWidth( Range32 bounds ) const
  {
	int32 remainder = bounds.End();
	
	for ( PaneLoop pane( Panes() ); remainder > 0 && pane.Unfinished(); pane++ )
	  {
		uint32 minimum = Min( remainder, pane->Sizer().MinimumWidth() );
		
		pane->proposedSize = minimum;
		remainder -= minimum;
	  }
	
	for ( PaneLoop pane( Panes() ); remainder > 0 && pane.Unfinished(); pane++ )
	  {
		int32 best = pane->Sizer().BestWidth( Range32( pane->proposedSize,
																	  pane->proposedSize + remainder ) );
		
		remainder -= best - pane->proposedSize;
		pane->proposedSize = best;
	  }
	
	return bounds.WeaklyRestrict( bounds.End() - remainder );
  }

int32 PaneRow::BestHeight( Range32 bounds ) const
  {
	bounds.SetStart( bounds.WeaklyRestrict( MinimumHeight() ) );
	bounds.SetEnd( bounds.WeaklyRestrict( MaximumHeight() ) );
	
	int32 best = 0;
	
	for ( PaneLoop pane( Panes() ); pane.Unfinished(); pane++ )
		best = Max( best, pane->Sizer().BestHeight( bounds ) );

	return best;
  }
