// TiledView.h

#ifndef TiledView_h
#define TiledView_h

#ifndef ListOf_h
#include "ListOf.h"
#endif
#ifndef LightListLoop_h
#include "LightListLoop.h"
#endif
#ifndef DrawsSpontaneously_h
#include "DrawsSpontaneously.h"
#endif

class SuggestsPaneSize;

class TiledView: virtual public DrawsSpontaneously
  {
	friend class TiledPane;
	
	private:
		ListOf<TiledPane> panes;
	
	protected:
		typedef LightListLoop<TiledPane> PaneLoop;
		const ListOf<TiledPane>& Panes() const		{ return panes; }
		
		void ShufflePanes( TiledPane *shuffleFirst );
		
		void ShufflePanesForward();
		void ShufflePanesBackward();

		int32 TotalProposedSize() const;
				
	public:
		virtual void Deliver( CellVisitor&, const Canvas& );
		
		typedef int32 (SuggestsPaneSize::*Measurement)() const;
		
		int32 MinimumOf( Measurement ) const;
		int32 MaximumOf( Measurement ) const;
		int32 TotalOf( Measurement ) const;
		
		void Add( TiledPane&, BeforeStart );
		void Add( TiledPane&, AfterEnd );
		
		void Add( TiledPane&, Before, const TiledPane& position );
		void Add( TiledPane&, After, const TiledPane& position );
		
		void Remove( TiledPane& n );
		void RemoveAll();
  };

#endif
