// WindowDefinition.h

#ifndef WindowDefinition_h
#define WindowDefinition_h

#ifndef WindowObject_h
#include "WindowObject.h"
#endif

class WindowDefinition
  {
	private:
		WindowObject::Definition procedure;
		Rectangle frameSize;
		
		bool closeable;
		bool resizable;
		bool zoomable;
		
		bool titled;
		bool modal;
		bool floating;
		
	public:
		WindowDefinition( WindowObject::Definition theProcedure,
								Rectangle theFrameSize,
								bool isCloseable,
								bool isResizable,
								bool isZoomable,
								bool isTitled,
								bool isModal,
								bool isFloating )
		  : procedure( theProcedure ),
			 frameSize( theFrameSize ),
			 closeable( isCloseable ),
			 resizable( isResizable ),
			 zoomable( isZoomable ),
			 titled( isTitled ),
			 modal( isModal ),
			 floating( isFloating )
		  {}
		
		WindowObject::Definition Procedure() const
														{ return procedure; }
		
		const Rectangle& FrameSize() const	{ return frameSize; }

		bool Closeable() const					{ return closeable; }
		bool Resizable() const					{ return resizable; }
		bool Zoomable() const					{ return zoomable; }
		
		bool Titled() const						{ return titled; }
		bool Modal() const						{ return modal; }
		bool Floating() const					{ return floating; }
		
		enum NoCloseBox	{ noCloseBox };
		enum NoGrowBox		{ noGrowBox };
		enum NoZoomBox		{ noZoomBox };
		
		static const WindowDefinition& Document();
		static const WindowDefinition& Document( NoCloseBox );
		static const WindowDefinition& Document( NoGrowBox );
		static const WindowDefinition& Document( NoCloseBox, NoGrowBox );
		static const WindowDefinition& Document( NoZoomBox );
		static const WindowDefinition& Document( NoCloseBox, NoZoomBox );
		static const WindowDefinition& Document( NoGrowBox, NoZoomBox );
		static const WindowDefinition& Document( NoCloseBox, NoGrowBox, NoZoomBox );
		
		static const WindowDefinition& Dialog();
		static const WindowDefinition& MovableModal();
		static const WindowDefinition& MovableModal( NoCloseBox );
  };

#endif
