// BufferMaster.h

#ifndef BufferMaster_h
#define BufferMaster_h

#ifndef Integers_h
#include "Integers.h"
#endif
#ifndef DebugMessage_h
#include "DebugMessage.h"
#endif
#ifndef Assert_h
#include "Assert.h"
#endif
#ifndef Range_h
#include "Range.h"
#endif
#ifndef ConstArrayOf_h
#include "ConstArrayOf.h"
#endif
#ifndef ConstData_h
#include "ConstData.h"
#endif
#ifndef ArrayOf_h
#include "ArrayOf.h"
#endif
#ifndef Data_h
#include "Data.h"
#endif
#ifndef Buffer_h
#include "Buffer.h"
#endif

// The includes above have been flattened so that weak compilers
// can cope better.

/*
	A BufferMaster is the owner's interface to a Buffer.
*/

class BufferMaster: public Buffer
  {
	private:
		uint8 *start;

		// not implemented:
			BufferMaster( const BufferMaster& );
			void operator=( const BufferMaster& );
	
	public:
		BufferMaster()
		  : Buffer( Data() ),
			 start( 0 )
		  {}
		
		explicit BufferMaster( Data space )
		  : Buffer( space ),
			 start( space.Start() )
		  {}
		
		void Reset()							{ unused = Data( start, unused.End() ); }
		void Reset( Data space )			{ unused = space; start = space.Start(); }
		
		ConstData Used() const				{ return ConstData( start, unused.Start() ); }
		const uint8 *UsedStart() const	{ return start; }

		uint32 TotalLength() const			{ return uint32( unused.End() - start ); }
		uint32 UsedLength() const			{ return uint32( unused.Start() - start ); }

		bool Fresh() const					{ return unused.Start() == start; }

		void ShiftOut( uint32 amount );
  };

#endif
