// BufferOf.h

#ifndef BufferOf_h
#define BufferOf_h

#ifndef Integers_h
#include "Integers.h"
#endif
#ifndef DebugMessage_h
#include "DebugMessage.h"
#endif
#ifndef Assert_h
#include "Assert.h"
#endif
#ifndef Range_h
#include "Range.h"
#endif
#ifndef ConstArrayOf_h
#include "ConstArrayOf.h"
#endif
#ifndef ArrayOf_h
#include "ArrayOf.h"
#endif

// The includes above have been flattened so that weak compilers
// can cope better.

template < class Element > class ConstBufferOf;

template < class Element >
class BufferOf
  {
	typedef ArrayOf<Element> ArrayType;
	typedef ConstArrayOf<Element> ConstArrayType;

	typedef BufferOf<Element> BufferType;
	typedef ConstBufferOf<Element> ConstBufferType;
	
	private:
		ArrayType space;
		uint32 mark;
	
		// not implemented:
			BufferOf( const BufferType& );
			void operator=( const BufferType& );
		
	public:
		BufferOf( ArrayType );
		
		void Reset()							{ mark = 0; }
		void Reset( ArrayType );

		inline void AdvanceMark( uint32 amount );
		
		const ConstArrayType Used() const	{ return space.Head( mark ); }
		const ArrayType Unused() const		{ return space.Tail( mark ); }
		
		uint32 TotalLength() const				{ return space.Length(); }
		uint32 UsedLength() const				{ return mark; }
		uint32 UnusedLength() const			{ return space.Length() - mark; }
		
		bool IsEmpty() const						{ return mark == 0; }
		bool IsFull() const						{ return mark >= space.Length(); }
		
		void operator<<( ConstArrayType );
		void operator<<( ConstBufferType& );
  };

template < class Element >
inline void BufferOf<Element>::AdvanceMark( uint32 amount )
  {
	Assert( amount <= UnusedLength() );
	mark += amount;
  }

#endif
