// ConstBuffer.h

#ifndef ConstBuffer_h
#define ConstBuffer_h

#ifndef Integers_h
#include "Integers.h"
#endif
#ifndef DebugMessage_h
#include "DebugMessage.h"
#endif
#ifndef Assert_h
#include "Assert.h"
#endif
#ifndef Range_h
#include "Range.h"
#endif
#ifndef ConstArrayOf_h
#include "ConstArrayOf.h"
#endif
#ifndef ConstData_h
#include "ConstData.h"
#endif
#ifndef ArrayOf_h
#include "ArrayOf.h"
#endif
#ifndef Data_h
#include "Data.h"
#endif

// The includes above have been flattened so that weak compilers
// can cope better.

class Buffer;

/*
	A ConstBuffer is like a ConstData which shrinks (from the front) as you
	use it.  You don't create ConstBuffers, but rather ConstBufferMasters.
	That gives you special powers, like filling it up again.  Then you pass
	the ConstBuffer& to functions, which can use up the data.
*/

class ConstBuffer
  {
	friend class ConstBufferMaster;
	friend ConstBuffer& operator>>( ConstBuffer&, Data );
	
	private:
		ConstData unused;
	
		explicit ConstBuffer( ConstData space )
		  : unused( space )
		  {}
		
		// not implemented:
			ConstBuffer( const ConstBuffer& );
			void operator=( const ConstBuffer& );
		
	public:
		const uint8& operator[]( uint32 i ) const		{ return unused[i]; }
		
		ConstData Unused() const							{ return unused; }
		
		const uint8 *UnusedStart() const					{ return unused.Start(); }
		const uint8 *UnusedEnd() const					{ return unused.End(); }
		
		uint32 UnusedLength() const						{ return unused.Length(); }

		void AdvanceMark( uint32 amount )				{ unused = unused.Tail( amount ); }

		bool Exhausted() const								{ return unused.IsEmpty(); }

		inline uint8 Get();
  };

inline uint8 ConstBuffer::Get()
  {
	uint8 c = unused[0];
	unused = unused.Tail(1);
	return c;
  }

inline ConstBuffer& operator>>( ConstBuffer& from, Data to )
  {
	from.unused = from.unused >> to;
	return from;
  }

ConstBuffer& operator>>( ConstBuffer& from, Buffer& to );
	// returns from, for chaining

#endif
