// ConstBufferMaster.h

#ifndef ConstBufferMaster_h
#define ConstBufferMaster_h

#ifndef Integers_h
#include "Integers.h"
#endif
#ifndef DebugMessage_h
#include "DebugMessage.h"
#endif
#ifndef Assert_h
#include "Assert.h"
#endif
#ifndef Range_h
#include "Range.h"
#endif
#ifndef ConstArrayOf_h
#include "ConstArrayOf.h"
#endif
#ifndef ConstData_h
#include "ConstData.h"
#endif
#ifndef ArrayOf_h
#include "ArrayOf.h"
#endif
#ifndef Data_h
#include "Data.h"
#endif
#ifndef ConstBuffer_h
#include "ConstBuffer.h"
#endif

// The includes above have been flattened so that weak compilers
// can cope better.

/*
	A ConstBufferMaster is the owner's interface to a ConstBuffer.
*/

class ConstBufferMaster: public ConstBuffer
  {
	private:
		const uint8 *start;

		// not implemented:
			ConstBufferMaster( const ConstBufferMaster& );
			void operator=( const ConstBufferMaster& );
	
	public:
		ConstBufferMaster()
		  : ConstBuffer( ConstData() ),
			 start( 0 )
		  {}
		
		explicit ConstBufferMaster( ConstData space )
		  : ConstBuffer( space ),
			 start( space.Start() )
		  {}
		
		void Reset()							{ unused = ConstData( start, unused.End() ); }
		void Reset( ConstData space )		{ unused = space; start = space.Start(); }
		
		ConstData Used() const				{ return ConstData( start, unused.Start() ); }
		const uint8 *UsedStart() const	{ return start; }

		uint32 TotalLength() const			{ return uint32( unused.End() - start ); }
		uint32 UsedLength() const			{ return uint32( unused.Start() - start ); }

		bool Fresh() const					{ return unused.Start() == start; }
  };

#endif
