// ConstData.cp

#ifndef ConstData_h
#include "ConstData.h"
#endif
#ifndef MinMax_h
#include "MinMax.h"
#endif
#ifndef Data_h
#include "Data.h"
#endif

const ConstData ConstData::operator>>( Data r ) const
  {
	Assert( !Null() );
	Assert( !r.Null() );
	
	const uint8 *from = Start();
	uint8 *to = r.Start();
	
	const uint8 *stop = from + BoundedLength( r.Length() );
	
	while ( from < stop )
		*to++ = *from++;

	return ConstData( from, End() );
  }

bool operator==( const ConstData& a, const ConstData& b )
  {
	Assert( !a.Null() );
	Assert( !b.Null() );

	if ( a.Length() != b.Length() )
		return false;
	
	for ( uint32 i = 0; i < a.Length(); i++ )
		if ( a[i] != b[i] )
			return false;
	
	return true;
  }

bool operator<( const ConstData& a, const ConstData& b )
  {
	Assert( !a.Null() );
	Assert( !b.Null() );

	uint32 matched = Min( a.Length(), b.Length() );
	
	for ( uint32 i = 0; i < matched; i++ )
		if ( a[i] < b[i] )
			return true;
	
	return a.Length() < b.Length();
  }

bool operator<=( const ConstData& a, const ConstData& b )
  {
	Assert( !a.Null() );
	Assert( !b.Null() );

	uint32 matched = Min( a.Length(), b.Length() );
	
	for ( uint32 i = 0; i < matched; i++ )
		if ( a[i] > b[i] )
			return false;
	
	return a.Length() <= b.Length();
  }

bool ConstData::StartsWith( ConstData b ) const
  {
	Assert( !Null() );
	Assert( !b.Null() );

	if ( this->Length() < b.Length() )
		return false;
	
	//return Head( b.Length() ) == b;
	return operator==( Head( b.Length() ), b );
  }

bool ConstData::EndsWith( ConstData b ) const
  {
	Assert( !Null() );
	Assert( !b.Null() );

	if ( this->Length() < b.Length() )
		return false;

	//return Tail( Length() - b.Length() ) == b;
	return operator==( Tail( Length() - b.Length() ), b );
  }

bool ConstData::Contains( ConstData b ) const
  {
	Assert( !Null() );
	Assert( !b.Null() );

	if ( this->Length() < b.Length() )
		return false;

	uint32 stop = this->Length() - b.Length() + 1;
	
	for ( uint32 i = 0; i < stop; i++ )
		if ( b == Middle( i, i+b.Length() ) )
			return true;
	
	return false;
  }

int32 Compare( ConstData a, ConstData b )
  {
	Assert( !a.Null() );
	Assert( !b.Null() );

	uint32 matched = Min( a.Length(), b.Length() );
	
	for ( uint32 i = 0; i < matched; i++ )
	  {
		int32 comparison = int32(a[i]) - int32(b[i]);
		if ( comparison != 0 )
			return comparison;
	  }
	
	if ( a.Length() < b.Length() )
		return -1;
	
	if ( a.Length() > b.Length() )
		return 1;
	
	return 0;
  }

int32 CompareReversed( ConstData a, ConstData b )
  {
	Assert( !a.Null() );
	Assert( !b.Null() );

	const uint8 *aChar = a.End();
	const uint8 *bChar = b.End();
	
	while ( aChar > a.Start() && bChar > b.Start() )
	  {
		int32 comparison = int32( *--aChar ) - int32( *--bChar );
		if ( comparison != 0 )
			return comparison;
	  }
	
	if ( a.Length() < b.Length() )
		return -1;
	
	if ( a.Length() > b.Length() )
		return 1;
	
	return 0;
  }
