// ListLoop.h

#ifndef ListLoop_h
#define ListLoop_h

#ifndef ListLink_h
#include "ListLink.h"
#endif
#ifndef SequenceLoop_h
#include "SequenceLoop.h"
#endif
#ifndef ListOf_h
#include "ListOf.h"
#endif

template < class TargetType >
class ListLoop: private SequenceLoop<List,ListNode>
  {
	typedef ListLink<TargetType> LinkType;
	typedef ListOf<TargetType> HeadType;
	typedef ListLoop<TargetType> LoopType;
	typedef SequenceLoop<List,ListNode> Inherited;
	
	friend class ListOf<TargetType>;
	
	private:
		static const ListLink<TargetType> *DownCast( const ListNode *n )
		  {
			return static_cast< const ListLink<TargetType>* >( n );
		  }

		static const ListOf<TargetType>& DownCast( const List& n )
		  {
			return static_cast< const ListOf<TargetType>& >( n );
		  }
		
	public:
		ListLoop( const HeadType& h )										: Inherited( h )					{}
		ListLoop( const HeadType& h, AtStart )							: Inherited( h, atStart )		{}
		ListLoop( const HeadType& h, AtEnd )							: Inherited( h, atEnd )			{}
		ListLoop( const HeadType& h, Nowhere )							: Inherited( h, nowhere )		{}
		ListLoop( const HeadType& h, const LinkType& p )			: Inherited( h, p )				{}
		ListLoop( const HeadType& h, Before, const LinkType& p )	: Inherited( h, before, p )	{}
		ListLoop( const HeadType& h, After, const LinkType& p )	: Inherited( h, after, p )		{}
		ListLoop( const HeadType& h, BeforeStart )					: Inherited( h, beforeStart )	{}
		ListLoop( const HeadType& h, AfterEnd )						: Inherited( h, afterEnd )		{}

		const HeadType& Owner() const						{ return DownCast( Inherited::Owner() ); }
		
		// Access declarations are broken in templates in CW Pro 5
		//Inherited::Finished;
		//Inherited::Unfinished;
		bool Finished() const								{ return Inherited::Finished(); }
		bool Unfinished() const								{ return Inherited::Unfinished(); }

		Inherited::MoveToFinish;
		Inherited::MoveToFirst;
		Inherited::MoveToLast;
		Inherited::MoveBeforeFirst;
		Inherited::MoveAfterLast;
		
		void MoveTo( const LinkType& p )					{ Inherited::MoveTo( p ); }
		void MoveBefore( const LinkType& p )			{ Inherited::MoveBefore( p ); }
		void MoveAfter( const LinkType& p )				{ Inherited::MoveAfter( p ); }
					
		bool operator==( const LoopType& r ) const	{ return Inherited::operator==( r ); }
		bool operator!=( const LoopType& r ) const	{ return Inherited::operator!=( r ); }

		bool operator==( const LinkType& r ) const	{ return Inherited::operator==( r ); }
		bool operator!=( const LinkType& r ) const	{ return Inherited::operator!=( r ); }
			
		Inherited::Null;
		const LinkType *Position() const		{ return DownCast( Inherited::Position() ); }
		
		const LinkType *Next() const			{ return DownCast( Inherited::Next() ); }
		const LinkType *Previous() const		{ return DownCast( Inherited::Previous() ); }
		
		TargetType& operator*() const			{ return **DownCast( Inherited::operator->() ); }
		TargetType *operator->() const		{ return &**DownCast( Inherited::operator->() ); }
		
		void operator++()							{ Inherited::operator++(); }
		void operator++(int)						{ Inherited::operator++(0); }
		void operator--()							{ Inherited::operator--(); }
		void operator--(int)						{ Inherited::operator--(0); }
  };

#endif
