// PointerToArray.h

#ifndef PointerToArray_h
#define PointerToArray_h

#ifndef Integers_h
#include "Integers.h"
#endif
#ifndef DebugMessage_h
#include "DebugMessage.h"
#endif
#ifndef Assert_h
#include "Assert.h"
#endif
#ifndef Range_h
#include "Range.h"
#endif
#ifndef ConstArrayOf_h
#include "ConstArrayOf.h"
#endif
#ifndef ArrayOf_h
#include "ArrayOf.h"
#endif

// The includes above have been flattened so that weak compilers
// can cope better.

template < class Element >
class PointerToArray
  {
	public:
		typedef ArrayOf<Element> ArrayType;
	
	private:
		ArrayType array;
	
		// These methods intertionally left unimplemented.
			PointerToArray( const PointerToArray<Element>& r );
			void operator=( const PointerToArray<Element>& r );
		
	public:
		PointerToArray()									{}

		PointerToArray( uint32 count )
			: array( new Element[count], count )
			{}

		PointerToArray( ArrayType theArray )
			: array( theArray )
			{}
		
		~PointerToArray()
		  {
			if ( !Null() )
				Dispose();
		  }
		
		void Allocate( uint32 count )
		  {
			Assert( Null() );
			array = ArrayType( new Element[count], count );
		  }
		
		void Dispose()
		  {
			Assert( !Null() );
			delete[] array.Start();
			array = ArrayType( 0, 0 );
		  }
		
		bool Null() const								{ return array.Null(); }
		
		ArrayType operator*() const				{ Assert( !Null() ); return array; }
		const ArrayType *operator->() const		{ Assert( !Null() ); return &array; }
		
		Element& operator[]( uint32 i ) const	{ Assert( !Null() ); return array[i]; }
		
		void Swap( PointerToArray<Element>& r )
		  {
			ArrayType temp( r.array );
			r.array = array;
			array = temp;
		  }
		
		void operator<<( PointerToArray<Element>& r )
		  {
			Assert( Null() );
			array = r.array;
			r.array = ArrayType( 0, 0 );
		  }
  };

#endif
