// FiniteQueue.h

#ifndef FiniteQueue_h
#define FiniteQueue_h

#ifndef FiniteQueueBase_h
#include "FiniteQueueBase.h"
#endif
#ifndef AlignedBlock_h
#include "AlignedBlock.h"
#endif
#ifndef Assert_h
#include "Assert.h"
#endif

template < class Element, uint32 size >
class FiniteQueue: public FiniteQueueBase
  {
	private:
		AlignedBlock< size * sizeof( Element ) > space;
	
		static Element *Bless( void *p )						{ return static_cast<Element*>(p); }
		static const Element *Bless( const void *p )		{ return static_cast<const Element*>(p); }
	
	public:
		FiniteQueue()
		  : FiniteQueueBase( space.Start(), size, sizeof( Element ) )
		  {}
		
		void Append( const Element& e )
		  {
			new (*this) Element( e );
		  }
		
		void Delete( Element *p )
		  {
			Assert( p != 0 );
			p->~Element();
			Release( p );
		  }	
		
		void DropFirst()
		  {
			Bless( FiniteQueueBase::First() )->~Element();
			FiniteQueueBase::DropFirst();
		  }
		
		void Drop( uint32 count )
		  {
			for ( uint32 i = 0; i < count; i++ )
				Bless( FiniteQueueBase::operator[](i) )->~Element();
			FiniteQueueBase::Drop( count );
		  }
		
		void DropAll()
		  {
			for ( uint32 i = 0; i < Length(); i++ )
				Bless( FiniteQueueBase::operator[](i) )->~Element();
			FiniteQueueBase::DropAll();
		  }
		
		Element& First()										{ return *Bless( FiniteQueueBase::First() ); }
		Element& operator[]( uint32 i )					{ return *Bless( FiniteQueueBase::operator[](i) ); }
		
		const Element& First() const						{ return *Bless( FiniteQueueBase::First() ); }
		const Element& operator[]( uint32 i ) const	{ return *Bless( FiniteQueueBase::operator[](i) ); }
  };

#endif
