// FiniteQueueBase.cp

#ifndef FiniteQueueBase_h
#include "FiniteQueueBase.h"
#endif
#ifndef Assert_h
#include "Assert.h"
#endif
#ifndef Overflow_h
#include "Overflow.h"
#endif

void FiniteQueueBase::AssertCoherent()
  {
	Assert( allocateNext >= start );
	Assert( allocateNext < end );
	Assert( disposeNext >= start );
	Assert( disposeNext < end );
	
	Assert( ( allocateNext - start ) % elementSize == 0 );
	Assert( ( end - allocateNext ) % elementSize == 0 );
	
	Assert( ( disposeNext - start ) % elementSize == 0 );
	Assert( ( end - disposeNext ) % elementSize == 0 );
	
	Assert( maximumLength == (end-start) / elementSize );
	Assert( length <= maximumLength );
	
	Assert( ( disposeNext - allocateNext + length*elementSize ) % (end - start) == 0 );
  }

FiniteQueueBase::FiniteQueueBase( void *theStart,
											 uint32 theMaximumLength,
											 uint32 theElementSize )
  : start( static_cast<uint8 *>(theStart) ),
	 end( static_cast<uint8 *>(theStart) + theMaximumLength * theElementSize ),
	 elementSize( theElementSize ),
	 maximumLength( theMaximumLength ),
	 length( 0 ),
	 allocateNext( static_cast<uint8 *>(theStart) ),
	 disposeNext( static_cast<uint8 *>(theStart) )
  {
	Assert( CanMultiply( theMaximumLength, theElementSize ) );
	AssertCoherent();
  }

void *FiniteQueueBase::Allocate( uint32 size )
  {
	Assert( size == elementSize );
	Assert( !Full() );
	AssertCoherent();

	if ( Full() )
		throw QueueFull();
	
	void *result = allocateNext;
	
	allocateNext += elementSize;
	
	Assert( allocateNext <= end );
	if ( allocateNext >= end )
		allocateNext = start;
	
	length++;
	
	return result;
  }

void FiniteQueueBase::Release( void *p )
  {
	Assert( !IsEmpty() );
	Assert( p == disposeNext );
	AssertCoherent();

	disposeNext += elementSize;
	Assert( disposeNext <= end );
	if ( disposeNext >= end )
		disposeNext = start;

	length--;
  }

void FiniteQueueBase::DropFirst()
  {
	Assert( !IsEmpty() );
	AssertCoherent();

	disposeNext += elementSize;
	Assert( disposeNext <= end );
	if ( disposeNext >= end )
		disposeNext = start;

	length--;
  }

void FiniteQueueBase::Drop( uint32 count )
  {
	Assert( length >= count );
	AssertCoherent();
	
	disposeNext += count * elementSize;
	if ( disposeNext >= end )
		disposeNext -= end - start;
	
	length -= count;
  }

void FiniteQueueBase::DropAll()
  {
	allocateNext = start;
	disposeNext = start;
	length = 0;
	AssertCoherent();
  }

void *FiniteQueueBase::operator[]( uint32 index )
  {
	uint8 *result = disposeNext + index * elementSize;
	if ( result >= end )
		result -= end - start;
	return result;
  }

const void *FiniteQueueBase::operator[]( uint32 index ) const
  {
	const uint8 *result = disposeNext + index * elementSize;
	if ( result >= end )
		result -= end - start;
	return result;
  }
