// FiniteQueueBase.h

#ifndef FiniteQueueBase_h
#define FiniteQueueBase_h

#ifndef Integers_h
#include "Integers.h"
#endif
#ifndef Assert_h
#include "Assert.h"
#endif

class QueueFull	{};

class FiniteQueueBase
  {
	private:
		uint8 *const start;
		uint8 *const end;
		const uint32 elementSize;
		const uint32 maximumLength;
		
		uint8 *allocateNext;
		uint8 *disposeNext;
		uint32 length;
		
		// not implemented:
			FiniteQueueBase( const FiniteQueueBase& );
			void operator=( const FiniteQueueBase& );
		
		inline void AssertCoherent();
		
	public:
		FiniteQueueBase( void *theSpace, uint32 maximumLength, uint32 elementSize );
		
		bool Full() const			{ return length >= maximumLength; }
		bool IsEmpty() const		{ return length == 0; }
		uint32 Length() const	{ return length; }
		
		void *Allocate( uint32 size );
		void Release( void * );
		
		void DropFirst();
		void Drop( uint32 count );
		void DropAll();
		
		void *First()										{ Assert( length > 0 ); return disposeNext; }
		void *operator[]( uint32 i );
		
		const void *First() const						{ Assert( length > 0 ); return disposeNext; }
		const void *operator[]( uint32 i ) const;
  };

inline void *operator new( size_t size, FiniteQueueBase& pool )
	{ return pool.Allocate( size ); }

#if !defined(__MWERKS__) && !defined(_MSC_VER)
	// placement delete is not implemented by CW Pro 4
	inline void operator delete( void *p,  FiniteQueueBase& pool )
		{ pool.Release( p ); }
#endif

#endif
