// FiniteStack.h

#ifndef FiniteStack_h
#define FiniteStack_h

#ifndef FiniteStackBase_h
#include "FiniteStackBase.h"
#endif
#ifndef AlignedBlock_h
#include "AlignedBlock.h"
#endif
#ifndef Assert_h
#include "Assert.h"
#endif

template < class Element, uint32 size >
class FiniteStack: public FiniteStackBase
  {
	private:
		AlignedBlock< size * sizeof( Element ) > space;
	
		static Element *Bless( void *p )						{ return static_cast<Element*>(p); }
		static const Element *Bless( const void *p )		{ return static_cast<const Element*>(p); }
		
	public:
		FiniteStack()
		  : FiniteStackBase( space.Start(), size, sizeof( Element ) )
		  {}
		
		Element& Top()											{ return *Bless( FiniteStackBase::Top() ); }
		Element& operator[]( uint32 i )					{ return *Bless( FiniteStackBase::operator[](i) ); }
		
		const Element& Top() const							{ return *Bless( FiniteStackBase::Top() ); }
		const Element& operator[]( uint32 i ) const	{ return *Bless( FiniteStackBase::operator[](i) ); }
		
		void Delete( Element *p )
		  {
			Assert( p != 0 );
			p->~Element();
			Release( p );
		  }

		inline void Push( const Element& e )
		  {
			new (*this) Element( e );
		  }		
		
		void Pop()
		  {
			Bless( FiniteStackBase::Top() )->~Element();
			FiniteStackBase::Pop();
		  }
		
		void Pop( uint32 count )
		  {
			for ( uint32 i = 0; i < count; i++ )
				Bless( FiniteStackBase::operator[](i) )->~Element();
			FiniteStackBase::Pop( count );
		  }
		
		void PopAll()
		  {
			for ( uint32 i = 0; i < Depth(); i++ )
				Bless( FiniteStackBase::operator[](i) )->~Element();
			FiniteStackBase::PopAll();
		  }
  };

#endif
