// FiniteStackBase.cp

#ifndef FiniteStackBase_h
#include "FiniteStackBase.h"
#endif
#ifndef Overflow_h
#include "Overflow.h"
#endif

inline void FiniteStackBase::AssertCoherence() const
  {
	Assert( top >= full );
	Assert( top <= empty );
	Assert( ( empty - top ) % elementSize == 0 );
	Assert( ( top - full ) % elementSize == 0 );
	Assert( depth == ( empty - top ) / elementSize );
  }

FiniteStackBase::FiniteStackBase( void *theStart,
											 uint32 theLength,
											 uint32 theElementSize )
  : full( static_cast<uint8 *>(theStart) ),
	 empty( static_cast<uint8 *>(theStart) + theLength * theElementSize ),
	 elementSize( theElementSize ),
	 depth( 0 )
  {
	top = empty;
	Assert( CanMultiply( theLength, theElementSize ) );
	Assert( elementSize > 0 );
	AssertCoherence();
  }

void *FiniteStackBase::Allocate( size_t size )
  {
	Assert( size == elementSize );
	Assert( depth < maxuint32 );
	Assert( !Full() );
	AssertCoherence();
	
	if ( Full() )
		throw StackFull();
	
	top -= elementSize;
	depth++;
	
	AssertCoherence();
	
	return top;
  }

void FiniteStackBase::Release( void *p )
  {
	Assert( !IsEmpty() );
	Assert( p == top );
	Assert( depth > 0 );
	AssertCoherence();
	
	top += elementSize;
	depth--;

	AssertCoherence();
  }

void FiniteStackBase::Pop( uint32 count )
  {
	Assert( depth >= count );
	AssertCoherence();
	
	top += count * elementSize;
	depth -= count;

	AssertCoherence();
  }
