// FiniteStackBase.h

#ifndef FiniteStackBase_h
#define FiniteStackBase_h

#ifndef Integers_h
#include "Integers.h"
#endif
#ifndef Assert_h
#include "Assert.h"
#endif

class StackFull	{};

class FiniteStackBase
  {
	private:
		uint8 *const full;
		uint8 *const empty;
		const uint32 elementSize;
		uint32 depth;
		
		uint8 *top;
		
		// not implemented:
			FiniteStackBase( const FiniteStackBase& );
			void operator=( const FiniteStackBase& );
		
		inline void AssertCoherence() const;
		
	public:
		FiniteStackBase( void *theSpace, uint32 length, uint32 elementSize );
		
		bool Full() const									{ return top <= full; }
		bool IsEmpty() const								{ return top >= empty; }

		uint32 Depth() const								{ return depth; }
		
		void *Top()											{ Assert( depth > 0 ); return top; }
		void *operator[]( uint32 i )					{ Assert( i < depth ); return top + i * elementSize; }
		
		const void *Top() const							{ Assert( depth > 0 ); return top; }
		const void *operator[]( uint32 i ) const	{ Assert( i < depth ); return top + i * elementSize; }
		
		void *Allocate( size_t size );
		void Release( void * );
		
		void Pop()											{ Assert( !IsEmpty() ); top += elementSize; depth--; }
		void Pop( uint32 count );
		void PopAll()										{ depth = 0; top = empty; }
  };

inline void *operator new( size_t size, FiniteStackBase& pool )
	{ return pool.Allocate( size ); }

#if !defined(__MWERKS__) && !defined(_MSC_VER)
	// placement delete is not implemented by CW Pro 4
	inline void operator delete( void *p,  FiniteStackBase& pool )
		{ pool.Release( p ); }
#endif

#endif
