// StackAllocatorBase.h

#ifndef StackAllocatorBase_h
#define StackAllocatorBase_h

#ifndef Integers_h
#include "Integers.h"
#endif
#ifndef DebugMessage_h
#include "DebugMessage.h"
#endif
#ifndef Assert_h
#include "Assert.h"
#endif
#ifndef Range_h
#include "Range.h"
#endif
#ifndef ConstArrayOf_h
#include "ConstArrayOf.h"
#endif
#ifndef ConstData_h
#include "ConstData.h"
#endif
#ifndef ArrayOf_h
#include "ArrayOf.h"
#endif
#ifndef Data_h
#include "Data.h"
#endif

// The includes above have been flattened so that weak compilers
// can cope better.

class StackAllocatorBase
  {
	private:
		struct Piece
		  {
			const Data storage;
			Piece *const deeper;
			
			Piece( Data s, Piece *d )
			  : storage( s ),
				 deeper( d )
			  {}
		  };
		
		Piece deepest;

		Piece *topPiece;
		Piece *freePiece;

		uint8 *top;
		
		void AllocatePiece( uint32 minimumSize );
		void ReleasePiece();
	
		void AddPiece( uint32 minimumSize );
		void RemovePiece();

		// not implemented:
			StackAllocatorBase( const StackAllocatorBase& );
			void operator=( const StackAllocatorBase& );
		
	public:
		StackAllocatorBase( Data baseSpace );
		~StackAllocatorBase();
		
		void *Allocate( size_t size );
		void Release( const void *p );
		
		void ReleaseAll();
		
		const void *Level()						{ return top; }
		void ReleaseTo( const void *level );
  };

inline void *operator new( size_t size, StackAllocatorBase& pool )
	{ return pool.Allocate( size ); }

#if !defined(__MWERKS__) && !defined(_MSC_VER)
	// placement delete is not implemented by CW Pro 4
	inline void operator delete( void *p,  StackAllocatorBase& pool )
		{ pool.Release( p ); }
#endif

#endif
