// StackBase.h

#ifndef StackBase_h
#define StackBase_h

#ifndef Integers_h
#include "Integers.h"
#endif
#ifndef DebugMessage_h
#include "DebugMessage.h"
#endif
#ifndef Assert_h
#include "Assert.h"
#endif
#ifndef Range_h
#include "Range.h"
#endif
#ifndef ConstArrayOf_h
#include "ConstArrayOf.h"
#endif
#ifndef ConstData_h
#include "ConstData.h"
#endif
#ifndef ArrayOf_h
#include "ArrayOf.h"
#endif
#ifndef Data_h
#include "Data.h"
#endif

// The includes above have been flattened so that weak compilers
// can cope better.

class StackBase
  {
	friend class StackLoopBase;
	friend class ConstStackLoopBase;
	
	private:
		struct Piece
		  {
			const Data storage;
			Piece *const deeper;
			
			Piece( Data s, Piece *d )
			  : storage( s ),
				 deeper( d )
			  {}
		  };
		
		Piece deepest;
		const uint32 unitSize;
		
		Piece *topPiece;
		Piece *freePiece;
		
		uint32 depth;
		uint8 *top;

		void AllocatePiece();
		void ReleasePiece();
	
		void AddPiece();
		void RemovePiece();

		uint8 *AtDepth( uint32 d ) const;
		
		// not implemented:
			StackBase( const StackBase& );
			void operator=( const StackBase& );
		
	public:
		StackBase( Data baseSpace, uint32 unit );
		~StackBase();
		
		void *Allocate();
		
		void Pop();
		void Pop( uint32 n );
		void PopAll();

		void *Allocate( size_t s )						{ Assert( s == unitSize ); return Allocate(); }
		void Delete( void *p )							{ Assert( p == Top() ); Pop(); }

		bool IsEmpty() const								{ return depth == 0; }
		uint32 Depth() const								{ return depth; }
		
		void *Top()											{ Assert( !IsEmpty() ); return top; }
		const void *Top() const							{ Assert( !IsEmpty() ); return top; }
		
		void *operator[]( uint32 d )					{ return AtDepth( d ); }
		const void *operator[]( uint32 d ) const	{ return AtDepth( d ); }
  };

inline void *operator new( size_t size, StackBase& stack )
  {
	return stack.Allocate( size );
  }

#if !defined(__MWERKS__) && !defined(_MSC_VER)
	// placement delete is not implemented by CW Pro 4
	inline void operator delete( void *p, StackBase& pool )
		{ pool.Delete( p ); }
#endif

#endif
