// StackOf.h

#ifndef StackOf_h
#define StackOf_h

#ifndef Integers_h
#include "Integers.h"
#endif
#ifndef DebugMessage_h
#include "DebugMessage.h"
#endif
#ifndef Assert_h
#include "Assert.h"
#endif
#ifndef Range_h
#include "Range.h"
#endif
#ifndef ConstArrayOf_h
#include "ConstArrayOf.h"
#endif
#ifndef ConstData_h
#include "ConstData.h"
#endif
#ifndef ArrayOf_h
#include "ArrayOf.h"
#endif
#ifndef Data_h
#include "Data.h"
#endif
#ifndef StackBase_h
#include "StackBase.h"
#endif

// The includes above have been flattened so that weak compilers
// can cope better.

template < class Element >
class StackOf: public StackBase
  {
	private:
		static Element *Bless( void *p )						{ return static_cast<Element*>(p); }
		static const Element *Bless( const void *p )		{ return static_cast<const Element*>(p); }
	
	public:
		StackOf( Data baseSpace )
		  : StackBase( baseSpace, sizeof(Element) )
		  {}
		
		~StackOf()												{ PopAll(); }
		
		Element& Top()											{ return *Bless( StackBase::Top() ); }
		Element& operator[]( uint32 i )					{ return *Bless( StackBase::operator[](i) ); }
		
		const Element& Top() const							{ return *Bless( StackBase::Top() ); }
		const Element& operator[]( uint32 i ) const	{ return *Bless( StackBase::operator[](i) ); }
		
		void Delete( Element *p )
		  {
			p->~Element();
			StackBase::Delete( p );
		  }

		inline void Push( const Element& e )
		  {
			new (*this) Element( e );
		  }		
		
		void Pop()
		  {
			// VC++ 5.0 sp3 has a bug and can't handle  "Top().~Element();"
			(&Top())->~Element();
			StackBase::Pop();
		  }
		
		void Pop( uint32 count )
		  {
			// VC++ 5.0 sp3 has a bug and can't handle  "(*this)[i].~Element();"
			for ( uint32 i = 0; i < count; i++ )
				(&(*this)[i])->~Element();
			StackBase::Pop( count );
		  }
		
		void PopAll()
		  {
			// VC++ 5.0 sp3 has a bug and can't handle  "(*this)[i].~Element();"
			for ( uint32 i = 0; i < Depth(); i++ )
				(&(*this)[i])->~Element();
			StackBase::PopAll();
		  }
  };

#endif
