// StorageFor.h

#ifndef StorageFor_h
#define StorageFor_h

#ifndef AlignedBlock_h
#include "AlignedBlock.h"
#endif

template < class T >
class StorageFor
  {
	private:
		AlignedBlock< sizeof( T ) > storage;
	
	public:
		void *Allocate( size_t size )
		  {
			Assert( size == sizeof( T ) );
			return storage.Start();
		  }
		
		void Destroy( T *object )
		  {
			Assert( static_cast<const void *>( storage.Start() ) == object );
			object->~T();
		  }
		
		T& Downcast()						{ return *reinterpret_cast<T*>( this ); }
		const T& Downcast() const		{ return *reinterpret_cast<const T*>( this ); }
  };

template < class T >
void *operator new( size_t size, StorageFor<T>& storage )
  {
	return storage.Allocate();
  }

#if !defined(__MWERKS__) && !defined(_MSC_VER)
	// placement delete is not implemented by CW Pro 4
	template < class T >
	inline void operator delete( void *p,  StorageFor<T>& storage )
		{}
#endif

#endif
