// Alignment.h

#ifndef Alignment_h
#define Alignment_h

#include <stddef.h>

template < class T >
class AlignmentHelper
  {
	private:
		char c;
		T t;

	// not implemented:
		AlignmentHelper();
		AlignmentHelper( const AlignmentHelper& );
		void operator=( const AlignmentHelper& );
  };

template < class T >
class Alignment
  {
	public:
		enum { increment = sizeof( AlignmentHelper<T> ) - sizeof( T ) };
		
		static size_t Padding( size_t size )
			{ return increment - 1 - ( size + increment - 1 ) % increment; }
	
		static size_t Pad( size_t size )
			{ return size + Padding( size ); }
	
		static bool IsAligned( void *p )
			{ return reinterpret_cast<size_t>( p ) % increment == 0; }
  };

union HardestTypeToAlign
  {
	private:
		char c;
		unsigned char uc;
		signed char sc;
		short s;
		unsigned short us;
		long l;
		unsigned long ul;
		int i;
		unsigned int ui;
		float f;
		double d;
		void *p;
		void (*fp)();
  };

typedef Alignment<HardestTypeToAlign> BestAlignment;

#endif
