// AutoString.cp

#ifndef AutoString_h
#include "AutoString.h"
#endif
#ifndef MinMax_h
#include "MinMax.h"
#endif

AutoString::AutoString( uint32 allocationSize )
  {
	if ( allocationSize == 0 )
		return;
	
	allocated = Data( ::operator new( allocationSize ), allocationSize );
	
	uint8 *usedStart = allocated.Start() + allocationSize / 2;
	
	used = Data( usedStart, usedStart );
  }

AutoString::AutoString( ConstData text, uint32 allocationSize )
  {
	if ( allocationSize < text.Length() )
		allocationSize = text.Length();
	
	if ( allocationSize == 0 )
		return;
	
	allocated = Data( ::operator new( allocationSize ), allocationSize );
	
	uint8 *usedStart = allocated.Start()
							 + ( allocationSize - text.Length() ) / 2;
	
	used = Data( usedStart, usedStart + text.Length() );
	used << text;
  }

void AutoString::Insert( uint8 *where, ConstData source )
  {
	InsertGap( where, source.Length() ) << source;
  }

void AutoString::Replace( Data destination, ConstData source )
  {
	MakeGap( destination, source.Length() ) << source;
  }

void AutoString::operator=( ConstData source )
  {
	Replace( Text(), source );
  }

#if 0
Data AutoString::MakeGap( Data destination, uint32 size )
  {
	Assert( allocated.Start() <= used.Start() );
	Assert( used.Start() <= destination.Start() );
	Assert( destination.Start() <= destination.End() );
	Assert( destination.End() <= used.End() );
	Assert( used.End() <= allocated.End() );
	
	if ( size == destination.Length() )
		return destination;
	
	uint32 newLength = used.Length() - destination.Length() + size;
	
	if ( size < destination.Length() )
	  {
		if ( newLength < allocated.Length() / 4 && allocated.Length() )
			;
	  }
	 else // size > destination.Length()
	  {
	  }
	
  }
#endif

Data AutoString::ReallocateWithGap( Data keep1, uint32 gapSize, Data keep2 )
  {
	uint32 dataSize = keep1.Length() + gapSize + keep2.Length();
	uint32 allocationSize = Max( 64ul, 2 * dataSize );
	
	AutoString newString( allocationSize );
	
	uint8 *dataStart = newString.allocated.Start() + allocationSize / 4;
	newString.used = Data( dataStart, dataStart + dataSize );
	
	Data kept1 = newString.used.Head( keep1.Length() );
	Data kept2 = newString.used.Tail( keep2.Length() );
	
	kept1 << keep1;
	kept2 << keep2;
	
	*this = newString;
	
	return Data( kept1.End(), kept2.Start() );
  }
