// AutoString.h

#ifndef AutoString_h
#define AutoString_h

#ifndef Data_h
#include "Data.h"
#endif

class AutoStringRef
  {
	friend class AutoString;
	private:
		Data allocated;
		Data used;
	
		AutoStringRef( Data a, Data u )
		  : allocated( a ),
			 used( u )
		  {}
  };

class AutoString
  {
	private:
		Data allocated;
		Data used;
	
		void Forget()												{ allocated = Data(); used = Data(); }
		
		Data ReallocateWithGap( Data keep1, uint32 gapSize, Data keep2 );
		
	public:
		AutoString()												{}
		explicit AutoString( uint32 allocationSize );
		explicit AutoString( ConstData, uint32 minAllocationSize = 0 );
		explicit inline AutoString( AutoString& );
		explicit inline AutoString( const AutoStringRef& );
		
		inline ~AutoString();
		
		void operator=( AutoString& );
		void operator=( ConstData );
		
		operator AutoStringRef();
		
		Data Text()													{ return used; }
		ConstData Text() const									{ return used; }
		
		Data MakeGap( Data destination, uint32 size );
		
		Data InsertGap( uint8 *where, uint32 size )		{ return MakeGap( Data( where, where ), size ); }
		void Remove( Data destination )						{ MakeGap( destination, 0 ); }
		
		void Insert( uint8 *where, ConstData source );
		void Replace( Data destination, ConstData source );
		
		AutoString Duplicate() const							{ return AutoString( Text() ); }
  };

inline AutoString::AutoString( AutoString& source )
  : allocated( source.allocated ),
	 used( source.used )
  {
	source.Forget();
  }

inline AutoString::AutoString( const AutoStringRef& source )
  : allocated( source.allocated ),
	 used( source.used )
  {
  }

inline AutoString::~AutoString()
  {
	if ( allocated.Start() != 0 )
		::operator delete( allocated.Start() );
  }

inline void AutoString::operator=( AutoString& source )
  {
	if ( this == &source )
		return;
	AutoString lost( *this );
	allocated = source.allocated;
	used = source.used;
	source.Forget();
  }

inline AutoString::operator AutoStringRef()
  {
	AutoStringRef result( allocated, used );
	Forget();
	return result;
  }

#endif
