// ConstCString.cp

#ifndef ConstCString_h
#include "ConstCString.h"
#endif

uint32 ConstCString::Length() const
  {
	uint32 length = 0;
	
	for ( const uint8 *string = text; *string != 0; string++ )
		length++;
	
	return length;
  }

int32 Compare( ConstCString a, ConstCString b )
  {
	const uint8 *left = a.Start();
	const uint8 *right = b.Start();
	
	if ( left == right )
		return 0;
	
	while ( *left != 0 && *left == *right )
	  {
		left++;
		right++;
	  }
	
	return int32( *left ) - int32( *right );
  }

int32 Compare( ConstCString a, ConstData b )
  {
	const uint8 *left = a.Start();
	const uint8 *right = b.Start();
	
	if ( left == right )
		return 0;

	const uint8 *rightEnd = b.End();
	
	// We'll pretend there's a null at the end of the ConstData
	uint8 rightCharacter = (right == rightEnd) ? 0 : *right;
	
	while ( *left != 0 && *left == rightCharacter )
	  {
		left++;
		right++;
		rightCharacter = (right == rightEnd) ? 0 : *right;
	  }
	
	// If the ConstData really contains a null, then the strings
	// are not equal.  The ConstData is longer, and thus larger.
	if ( right < rightEnd && *left == 0 )
		return -1;
	
	return int32( *left ) - int32( rightCharacter );
  }

bool ConstCString::operator==( ConstCString s ) const
  {
	return Compare( *this, s ) == 0;
  }

bool ConstCString::operator>=( ConstCString s ) const
  {
	return Compare( *this, s ) >= 0;
  }

bool ConstCString::operator==( ConstData s ) const
  {
	return Compare( *this, s ) == 0;
  }

bool ConstCString::operator>=( ConstData s ) const
  {
	return Compare( *this, s ) >= 0;
  }
