// ConstCString.h

#ifndef ConstCString_h
#define ConstCString_h

#ifndef Integers_h
#include "Integers.h"
#endif
#ifndef DebugMessage_h
#include "DebugMessage.h"
#endif
#ifndef Assert_h
#include "Assert.h"
#endif
#ifndef Range_h
#include "Range.h"
#endif
#ifndef ConstArrayOf_h
#include "ConstArrayOf.h"
#endif
#ifndef ConstData_h
#include "ConstData.h"
#endif

// The includes above have been flattened so that weak compilers
// can cope better.

class ConstCString
  {
	private:
		const uint8 *text;
		
	public:
		ConstCString( const char *theText )
		  : text( reinterpret_cast<const uint8 *>( theText ) )
		  {
			Assert( theText != 0 );
		  }

		const int8 *SignedStart() const					{ return reinterpret_cast<const int8 *>( text ); }
		const char *Characters() const					{ return reinterpret_cast<const char *>( text ); }
		
		const uint8 *Start() const							{ return text; }
		const uint8 *End() const							{ return text + Length(); }

		uint32 Length() const;
		const URange32 Range() const						{ return URange32( 0, Length() ); }
				
		bool IsEmpty() const									{ return *text == 0; }
		
		const uint8& operator[]( uint32 i ) const		{ return text[i]; }
		
		const ConstData Text() const								{ return ConstData( text, Length() ); }
		const ConstData Head( uint32 position ) const		{ return Text().Head( position ); }
		const ConstData Tail( uint32 position ) const		{ return Text().Tail( position ); }
		const ConstData Middle( URange32 range ) const		{ return Text().Middle( range ); }

		operator ConstData() const							{ return Text(); }
		
		bool operator==( ConstCString s ) const;
		bool operator!=( ConstCString s ) const		{ return !( *this == s ); }
		bool operator>=( ConstCString s ) const;
		bool operator<=( ConstCString s ) const		{ return s >= *this; }
		bool operator>( ConstCString s ) const			{ return !( s >= *this ); }
		bool operator<( ConstCString s ) const			{ return !( *this >= s ); }
		
		bool operator==( ConstData s ) const;
		bool operator!=( ConstData s ) const			{ return !( *this == s ); }
		bool operator>=( ConstData s ) const;
		bool operator<=( ConstData s ) const			{ return s >= *this; }
		bool operator>( ConstData s ) const				{ return !( s >= *this ); }
		bool operator<( ConstData s ) const				{ return !( *this >= s ); }
  };

int32 Compare( ConstCString a, ConstCString b );
int32 Compare( ConstCString a, ConstData b );
inline int32 Compare( ConstData a, ConstCString b )	{ return -Compare(b,a); }

#endif
