//
//  HQApplication.m
//  HackMac
//

#import "HQApplication.h"
#import <objc/objc-class.h>  // We need this in order to dereference a Class
#import <unistd.h>

@implementation HQApplication

+ (void)doNSAppInstancePose {
    if (NSApp) {
        // Swizzle our class object (and its meta-class) to be a subclass of the class that NSApp is a member of.  This could be a real stupid thing to do if any instances were going to be created and our new superclass had ivars.  But this situation is fairly controlled and no more instances of NSApplication subclasses should ever be created.
        Class realNSAppClass = ((HQApplication *)NSApp)->isa;
        Class hqAppClass = [HQApplication class];
        Class realNSAppMetaClass = ((HQApplication *)realNSAppClass)->isa;
        Class hqAppMetaClass = ((HQApplication *)hqAppClass)->isa;
        
        hqAppClass->super_class = realNSAppClass;
        hqAppMetaClass->super_class = realNSAppMetaClass;
        
        // Swizzle NSApp into an instance of our subclass.
        // We cast to HQApplication as a simple way to get around the fact that isa is @protected.  It does not matter what we cast to, really, since everything has an isa.
        ((HQApplication *)NSApp)->isa = self;
    } else {
        NSLog(@"NSApp does not exist yet!");
    }
}

static BOOL _lastEventWasKeyDown = NO;
static NSTimeInterval _lastKeyDownTimeStamp = 0.0;
static unsigned _numberOfConsecutiveKeyDowns = 0;
static NSTimeInterval _avgTimeBetweenKeyDowns = 0.0;
static BOOL _lastEventWasLetter = NO;

static const NSTimeInterval _frequencyThreshold = 0.25;
static const unsigned _mutationConsecutiveCountThreshold = 5;
static const float _mutationProbability = 0.20;

#define MUTATION_TYPE_COUNT 4
static const float mutationTypeProbabilities[MUTATION_TYPE_COUNT] = {0.6, 0.65, 0.8, 1.0};
static SEL _mutationTypeSelectors[MUTATION_TYPE_COUNT] = {0, 0, 0, 0};

static SEL *mutationTypeSelectors() {
    if (_mutationTypeSelectors[0] == 0) {
        _mutationTypeSelectors[0] = @selector(transposeEventArrayWithKeyEvent:);
        _mutationTypeSelectors[1] = @selector(doubleEventArrayWithKeyEvent:);
        _mutationTypeSelectors[2] = @selector(skipEventArrayWithKeyEvent:);
        _mutationTypeSelectors[3] = @selector(nearKeyEventArrayWithKeyEvent:);
    }
    return _mutationTypeSelectors;
}

static float _randomProbability() {
    static BOOL rngInited = NO;
    long randomInt;
    float randomProb;
    
    if (!rngInited) {
        srandom((time(NULL) & 0xFFFF) | ((getpid() & 0xFFFF) << 16));
        rngInited = YES;
    }
    randomInt = random();
    if (randomInt < 0) {
        randomInt = -randomInt;
    }
    randomProb = (float)randomInt / (float)INT_MAX;
    
    return randomProb;
}

- (NSArray *)transposeEventArrayWithKeyEvent:(NSEvent *)event {
    NSMutableArray *eventArray = nil;
    unichar buff[1];
    NSString *leftArrowStr;
    NSString *rightArrowStr;

    NSLog(@"Transpose key event: %@", [event characters]);
    
    buff[0] = NSLeftArrowFunctionKey;
    leftArrowStr = [NSString stringWithCharacters:buff length:1];
    buff[0] = NSRightArrowFunctionKey;
    rightArrowStr = [NSString stringWithCharacters:buff length:1];
    
    eventArray = [NSMutableArray array];
    [eventArray addObject:[NSEvent keyEventWithType:[event type] location:[event locationInWindow] modifierFlags:[event modifierFlags] timestamp:[event timestamp] windowNumber:[event windowNumber] context:[event context] characters:leftArrowStr charactersIgnoringModifiers:leftArrowStr isARepeat:[event isARepeat] keyCode:[event keyCode]]];
    [eventArray addObject:event];
    [eventArray addObject:[NSEvent keyEventWithType:[event type] location:[event locationInWindow] modifierFlags:[event modifierFlags] timestamp:[event timestamp] windowNumber:[event windowNumber] context:[event context] characters:rightArrowStr charactersIgnoringModifiers:rightArrowStr isARepeat:[event isARepeat] keyCode:[event keyCode]]];
    return eventArray;
}

- (NSArray *)doubleEventArrayWithKeyEvent:(NSEvent *)event {
    NSMutableArray *eventArray = nil;

    NSLog(@"Double key event: %@", [event characters]);
    
    eventArray = [NSMutableArray array];
    [eventArray addObject:event];
    [eventArray addObject:event];
    return eventArray;
}

- (NSArray *)skipEventArrayWithKeyEvent:(NSEvent *)event {
    NSMutableArray *eventArray = nil;

    NSLog(@"Skip key event: %@", [event characters]);
    
    eventArray = [NSMutableArray array];
    return eventArray;
}

- (NSArray *)nearKeyEventArrayWithKeyEvent:(NSEvent *)event {
    static NSDictionary *nearKeyDict = nil;
    NSMutableArray *eventArray = nil;
    NSString *characters = [event characters];
    NSString *lowercaseCharacters;
    BOOL isUppercase;
    NSArray *nearKeys;
    
    if (nearKeyDict == nil) {
        NSString *path = [[NSBundle bundleForClass:[HQApplication class]] pathForResource:@"KeysNearKeys" ofType:@"plist"];
        if (path) {
            nearKeyDict = [[NSDictionary allocWithZone:[self zone]] initWithContentsOfFile:path];
        } else {
            nearKeyDict = [[NSDictionary allocWithZone:[self zone]] init];
        }
    }
    
    lowercaseCharacters = [characters lowercaseString];
    isUppercase = ([characters isEqualToString:lowercaseCharacters] ? NO : YES);
    nearKeys = [nearKeyDict objectForKey:lowercaseCharacters];
    if (nearKeys) {
        unsigned numNearKeys = [nearKeys count];
        if (numNearKeys > 0) {
            float rNum = _randomProbability();
            float fInd = rNum * ((float)numNearKeys - 0.0000001);
            unsigned i = (unsigned)floor(fInd);
            NSString *keyStr;
            
            if (i >= numNearKeys) {
                NSLog(@"Index out of range.");
                i = numNearKeys - 1;
            }
            
            keyStr = [nearKeys objectAtIndex:i];
            if (isUppercase) {
                keyStr = [keyStr uppercaseString];
            }
            eventArray = [NSMutableArray array];
            [eventArray addObject:[NSEvent keyEventWithType:[event type] location:[event locationInWindow] modifierFlags:[event modifierFlags] timestamp:[event timestamp] windowNumber:[event windowNumber] context:[event context] characters:keyStr charactersIgnoringModifiers:keyStr isARepeat:[event isARepeat] keyCode:[event keyCode]]];
            
            NSLog(@"Near key event: %@ (using %@)", [event characters], keyStr);
        }
    }
    
    return eventArray;
}

- (NSArray *)fudgeKeyEvent:(NSEvent *)event {
    NSArray *eventArray = nil;
    
    if ((_numberOfConsecutiveKeyDowns >= _mutationConsecutiveCountThreshold) && (_avgTimeBetweenKeyDowns <= _frequencyThreshold)) {
        float randomProb = _randomProbability();
        
        if (randomProb <= _mutationProbability) {
            // Do it!
            unsigned i;
            
            randomProb = _randomProbability();
            
            for (i=0; i<MUTATION_TYPE_COUNT; i++) {
                if (randomProb <= mutationTypeProbabilities[i]) {
                    eventArray = [self performSelector:mutationTypeSelectors()[i] withObject:event];
                    break;
                }
            }
        }
    }
    return eventArray;
}

- (void)sendEvent:(NSEvent *)event {
    NSEventType eventType = [event type];
    NSArray *eventArray = nil;
    
    if (eventType == NSKeyDown) {
        NSTimeInterval timestamp = [event timestamp];
        BOOL thisEventIsLetter = NO;
        NSString *characters = [event characters];
        
        if ([characters length] >= 1) {
            if ([[NSCharacterSet letterCharacterSet] characterIsMember:[characters characterAtIndex:0]]) {
                thisEventIsLetter = YES;
            }
        }
        
        if (!_lastEventWasKeyDown || ((timestamp - _lastKeyDownTimeStamp) > 3.0)) {
            _numberOfConsecutiveKeyDowns = 1;
            _avgTimeBetweenKeyDowns = 0.0;
        } else {
            if (_numberOfConsecutiveKeyDowns > 1) {
                _avgTimeBetweenKeyDowns = ((_numberOfConsecutiveKeyDowns * _avgTimeBetweenKeyDowns) + (timestamp - _lastKeyDownTimeStamp)) / (_numberOfConsecutiveKeyDowns + 1);
            } else {
                _avgTimeBetweenKeyDowns = (timestamp - _lastKeyDownTimeStamp);
            }
            _numberOfConsecutiveKeyDowns++;
        }
        _lastKeyDownTimeStamp = timestamp;
        _lastEventWasKeyDown = YES;
        
#ifdef ENABLE_LOGS
        if (_numberOfConsecutiveKeyDowns % 10 == 0) {
            NSLog(@"%u consecutive key events with average interval %.2f", _numberOfConsecutiveKeyDowns, _avgTimeBetweenKeyDowns);
        }
#endif
        if (_lastEventWasLetter && thisEventIsLetter) {
            eventArray = [self fudgeKeyEvent:event];
        }
        
        _lastEventWasLetter = thisEventIsLetter;
    } else if (eventType != NSKeyUp) {
#ifdef ENABLE_LOGS
        if (_lastKeyDownTimeStamp) {
            NSLog(@"%u consecutive key events with average interval %.2f", _numberOfConsecutiveKeyDowns, _avgTimeBetweenKeyDowns);
        }
#endif
        _lastEventWasKeyDown = NO;
        _lastEventWasLetter = NO;
    }
    
    if (eventArray) {
        unsigned i, c = [eventArray count];
        for (i=0; i<c; i++) {
            [super sendEvent:[eventArray objectAtIndex:i]];
        }
    } else {
        [super sendEvent:event];
    }
}

@end
